/*******************************************************************************
 * Copyright (c) 2006 GXS (http://www.gxs.com).
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Ashish Billore - initial API and implementation
 *    Bugzilla Bug ID - 147439
 *******************************************************************************/

package org.eclipse.ftp.internal;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.IDirectoryEntry;

/**
 * Factory for converting the following MS-DOS style 
 * LS line to an FTPFDirectoryEntry:
 * 
 * 04-26-06  07:30PM       <DIR>          abc
 * 03-27-06  10:51AM       <DIR>          acctest_sample
 * 03-18-06  08:01PM              5989263 mycode.jar
 * 
 */

public class DOSStyleDirEntryParser extends DirectoryEntryParser {
 
	public IDirectoryEntry parseEntry(String line) throws FtpException {
		boolean hasDirectorySemantics = hasDirectorySementics(line);
		boolean hasFileSemantics = !hasDirectorySemantics;
		if (!hasFileSemantics && !hasDirectorySemantics) {
			return null;
		}
		StringTokenizer tok = new StringTokenizer(line, " \t");  //$NON-NLS-1$
		try {
			Calendar calendar = GregorianCalendar.getInstance(Constants.DEFAULT_TIMEZONE);
			// read date
			if(!parseDate(calendar, tok.nextToken()))
				return null;
			if(!parseTime(calendar, tok.nextToken()))
				return null;
			long size = 0;
			try{
				size = Long.parseLong(tok.nextToken(), 10);
			}
			catch(NumberFormatException nfe)
			{
				// ignore this exception as this token is '<DIR>'
			}
			String filename = parseName(tok);
			if (filename == null) {
				return null;
			}
			
			// done!
			return new FTPDirectoryEntry(filename, hasDirectorySemantics, hasFileSemantics,
				size, calendar.getTime());
		} catch (NoSuchElementException e) {
			// expected more tokens
			return null;
		} catch (NumberFormatException e) {
			// don't understand the format so ignore the line
			return null;
		}
	}


	private String parseName(StringTokenizer tokenizer) 
	{
		String token = tokenizer.nextToken();
		// read filename
		StringBuffer filename = new StringBuffer(token);
		while (tokenizer.hasMoreTokens()) {
			token = tokenizer.nextToken();
			filename.append(' ');
			filename.append(token);
		}
		return filename.toString();
	}


	public boolean isReliable() {
		// TODO Auto-generated method stub
		return false;
	}
	
	protected boolean hasDirectorySementics(String line) {
		if(line.indexOf("<DIR>") >= 0 || line.indexOf("<dir>") >= 0) //$NON-NLS-1$ //$NON-NLS-2$
			return true;
		else
			return false;
	}

	protected boolean parseDate(Calendar cal, String date)
	{
		StringTokenizer tok1 = new StringTokenizer(date, "-"); //$NON-NLS-1$
		if(tok1.countTokens() == 3)
		{
			int month = new Integer(tok1.nextToken()).intValue();
			int dayOfMonth = new Integer(tok1.nextToken()).intValue(); 
			int year = new Integer(tok1.nextToken()).intValue(); 
		    if (year < 50) year += 2000;
		    if (year < 1000) year += 1900;
			cal.set(year, month-1, dayOfMonth);
			return true;
		}
		return false;
	}


	protected boolean parseTime(Calendar calendar, String time) {
		try {
			int colonPos = time.indexOf(':');
			if (colonPos != -1) {
				int hour = Integer.parseInt(time.substring(0, colonPos));
			    if (hour == 12) hour = 0;
			    String minAMPM = time.substring(colonPos + 1);
			    String min = minAMPM.substring(0,2);
	
				if (hour == 24) hour = 0; // ideally this should never happen, just to be on safe side.
				char amOrpm = minAMPM.charAt(2);
				if(amOrpm == 'P' || amOrpm == 'p' 
					|| amOrpm == 'A' || amOrpm == 'a')
				{
					if(amOrpm == 'P' || amOrpm == 'p' )
						 hour += 12;
					if (hour >= 0 && hour <= 23) {
						int minute = Integer.parseInt(min);
						if (minute >= 0 && minute <= 59) {
							calendar.set(Calendar.HOUR_OF_DAY, hour);
							calendar.set(Calendar.MINUTE, minute);
							calendar.set(Calendar.SECOND, 0);
							calendar.set(Calendar.MILLISECOND, 0);
							return true;
						}
					}
				}
			}
		} catch (NumberFormatException e) {
		}
		return false;
	}
}