/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal;

import java.io.*;
import java.util.*;

import org.eclipse.core.runtime.*;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.target.*;
import org.eclipse.target.internal.registry.SiteDescriptor;
import org.eclipse.target.internal.registry.SiteRegistry;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.XMLMemento;

public class SiteManager extends PlatformObject implements ISiteManager {

	// Name of the file containing the sites defined by the user
	// This file is stored in the TeamPlugin state location for backwards compatibility
	private static final String STATE_FILE = "sites.xml"; //$NON-NLS-1$

	private List sites = new ArrayList();
	private List listeners = new ArrayList();

	// The id of the targets extension point
	public static final String TARGETS_EXTENSION = "targets"; //$NON-NLS-1$

	// Constants used when persisting and loading sites
	private static final String CTX_SITES = "sites"; //$NON-NLS-1$
	private static final String CTX_SITE = "site"; //$NON-NLS-1$
	private static final String CTX_TYPE = "type"; //$NON-NLS-1$
	private static final String CTX_URL = "url"; //$NON-NLS-1$
	private static final String CTX_SITE_DATA = "data"; //$NON-NLS-1$

	private SiteRegistry registry;
	
	public SiteManager() {
		initialize();
	}
	
	private void initialize() {
		loadFormDescriptors();
		try {
			restoreState();
		} catch (CoreException e) {
			TargetPlugin.log(e);
		}
	}

	private void loadFormDescriptors() {
		registry = new SiteRegistry();
		registry.readRegistry(Platform.getExtensionRegistry(), TargetPlugin.ID, SiteRegistry.PT_SITES);
	}
	
	public Site[] getSites() {
		return (Site[]) sites.toArray(
			new Site[sites.size()]);
	}

	public void addSite(Site site) {
		sites.add(site);
		saveState();
		for (Iterator it = listeners.iterator(); it.hasNext();) {
			ITargetListener element = (ITargetListener) it.next();
			element.siteAdded(site);
		}
	}
	
	public void removeSite(Site site) {
		sites.remove(site);
		saveState();
		for (Iterator it = listeners.iterator(); it.hasNext();) {
			ITargetListener element = (ITargetListener) it.next();
			element.siteRemoved(site);
		}
	}

	public Site getSite(String locationId) {
		for (Iterator it = sites.iterator(); it.hasNext();) {
			Site element = (Site) it.next();
			if (element.getLocation().equals(locationId)) {
				return element;
			}
		}
		return null;
	}

	public void addSiteListener(ITargetListener listener) {
		listeners.add(listener);
	} 
	
	public void removeSiteListener(ITargetListener listener) {
		listeners.remove(listener);
	}

	private File getStateFile() {
		IPath pluginStateLocation = TargetPlugin.getDefault().getStateLocation();
		return pluginStateLocation.append(STATE_FILE).toFile();
	}
	
	private void saveState() {
		XMLMemento xmlMemento = XMLMemento.createWriteRoot(CTX_SITES);
		for (Iterator it = sites.iterator(); it.hasNext(); ) {
			Site site = (Site) it.next();				
			// Create the state placeholder for a site 
			IMemento siteNode = xmlMemento.createChild(CTX_SITE);
			siteNode.putString(CTX_TYPE, site.getTypeId());	
			siteNode.putString(CTX_URL, site.getLocation());	
			IMemento siteData = siteNode.createChild(CTX_SITE_DATA);
			site.save(siteData);
		}
		try {
			Writer writer = new BufferedWriter(new FileWriter(getStateFile()));
			try {
				xmlMemento.save(writer);
			} finally {
				writer.close();
			}
		} catch (IOException e) {
			TargetPlugin.log(new Status(IStatus.ERROR, TargetPlugin.ID, 1, "An error occurred saving the sites", e)); //$NON-NLS-1$
		}
	}
	
	private void restoreState() throws CoreException {
		File file = getStateFile();
		Reader reader;
		try {
			reader = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			return;
		}
		IMemento memento = XMLMemento.createReadRoot(reader);
		IMemento[] siteNodes = memento.getChildren(CTX_SITE);
		for (int i = 0; i < siteNodes.length; i++) {
			IMemento memento2 = siteNodes[i];
			String type = memento2.getString(CTX_TYPE);
			String urlString = memento2.getString(CTX_URL);
			try {
				if (type != null && urlString != null) {
					Site site = createSite(type);
					if (site == null) {
						TargetPlugin.log(new Status(IStatus.ERROR, TargetPlugin.ID, 1, "The site for {0} could not be created because type {1} is not known." + urlString + type, null)); //$NON-NLS-1$
					} else {
						site.init(urlString, memento2.getChild(CTX_SITE_DATA));
						addSite(site);
					}
				}
			} catch (CoreException e) {
				TargetPlugin.log(new Status(IStatus.ERROR, TargetPlugin.ID, 1, "An error occurred creating the site for {0}." + urlString, e)); //$NON-NLS-1$
			}
		}
	}

	public Site createSite(String type) throws CoreException {
		SiteDescriptor desc = registry.getSiteDescriptor(type);
		if (desc == null) return null;
		return desc.createSite();
	}

	public boolean okToRemoveSites(Site[] sites, Shell shell) {
		for (Iterator it = listeners.iterator(); it.hasNext();) {
			ITargetListener element = (ITargetListener) it.next();
			if (!element.aboutToRemove(sites, shell)) {
				return false;
			}
		}
		return true;
	}
}
