/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.ftp;

import java.util.*;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ftp.*;
import org.eclipse.jface.util.Assert;
import org.eclipse.target.AuthenticatedSite;
import org.eclipse.target.ITargetResource;
import org.eclipse.ui.IMemento;

public class FtpSite extends AuthenticatedSite {

	private static final String PASSIVE_CTX = "passive"; //$NON-NLS-1$
	private static final String TIMEOUT_CTX = "connectionTimeout"; //$NON-NLS-1$
	public static final int DEFAULT_TIMEOUT = 60;

	private int timeout = DEFAULT_TIMEOUT;
	private boolean passive = false;
	
	private Map openClients = new HashMap();
	
	public FtpSite() {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#save(org.eclipse.ui.IMemento)
	 */
	public void save(IMemento memento) {
		memento.putInteger(TIMEOUT_CTX, timeout);
		memento.putString(PASSIVE_CTX, Boolean.toString(passive));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#init(org.eclipse.ui.IMemento)
	 */
	protected void init(IMemento memento) {
		Assert.isNotNull(memento); 

		restoreAuthenticationInformation();
		
		// Timeouts.
		Integer to = memento.getInteger(TIMEOUT_CTX);
		if (to != null) {
			timeout = to.intValue();
		}
		
		// usePassive.
		String usePassiveString = memento.getString(PASSIVE_CTX);
		if (usePassiveString == null) {
			passive = true;
		} else {
			passive = usePassiveString.equals(Boolean.toString(true));
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#getRootResource()
	 */
	public ITargetResource getRootResource() {
		return new FtpTargetResource(this, "", true); //$NON-NLS-1$
	}

	public boolean isPassive() {
		return passive;
	}
	public void setPassive(boolean passive) {
		this.passive = passive;
	}
	public int getTimeout() {
		return timeout;
	}
	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}
	
	public IClient createClient() throws FtpException {
		IClient client = Ftp.createClient(toUrl());
		client.setAuthentication(getUsername(), getPassword());
		client.setDataTransferMode(isPassive() ? IClient.PASSIVE_DATA_TRANSFER : IClient.PORT_DATA_TRANSFER, true);
		client.setTimeout(getTimeout());
		return client;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#canBeReached(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean canBeReached(IProgressMonitor monitor) throws CoreException {
		IClient client = createClient();
		try {
			client.open(monitor);
		} catch (FtpException e) {
			throw e;
		} finally {
			client.close(monitor);
		}
		return true;
	}

	/**
	 * Use this method to ensure that the same connection is used for multiple commands
	 * issued from the same thread.
	 * @throws FtpException
	 */
	public void run(IFtpRunnable runnable, IProgressMonitor monitor) throws FtpException {
		Thread thread = Thread.currentThread();
		IClient openClient = getOpenClient(thread);
		boolean outerRun = false;
		try {
			monitor.beginTask(null, 100);
			if (openClient == null) {
				openClient = createClient();
				openClient.open(Policy.subMonitorFor(monitor, 5));
				addOpenClient(thread, openClient);
				outerRun = true;
			}
			openClient.run(runnable, Policy.subMonitorFor(monitor, 90));
		} finally {
			if (outerRun && openClient != null) {
				openClient.close(Policy.subMonitorFor(monitor, 5));
				removeOpenClient(thread);
			}
		}
	}
    
    public boolean isConnected() {
        return getOpenClient(Thread.currentThread()) != null;
    }

	IClient getOpenClient(Thread thread) {
		return (IClient)openClients.get(thread);
	}

	private void addOpenClient(Thread thread, IClient openClient) {
		openClients.put(thread, openClient);
		
	}

	private void removeOpenClient(Thread thread) {
		openClients.remove(thread);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.target.Site#dispose()
	 */
	public void dispose() throws CoreException {
		super.dispose();
		for (Iterator iter = openClients.values().iterator(); iter.hasNext();) {
			IClient element = (IClient) iter.next();
			element.close(new NullProgressMonitor());
		}
	}
}
