/*******************************************************************************
 * Copyright (c) 2000, 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.target.internal.webdav.ui;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.target.Site;
import org.eclipse.target.Target;
import org.eclipse.target.internal.webdav.Policy;
import org.eclipse.target.internal.webdav.WebDavSite;
import org.eclipse.target.internal.webdav.WebdavPlugin;
import org.eclipse.target.ui.SiteCreationWizardPage;

public class WebDavSiteCreationPage extends SiteCreationWizardPage {

	boolean isCheckout;
	
	// Widgets
	private Combo usernameCombo;
	private Combo hostnameUrlCombo;
	private Text password;
	private Combo proxyCombo;
	private Combo timeoutCombo;
	
	// Dialog store id constants
	private static final String STORE_USERNAME_ID =
		"DavWizardMainPage.STORE_USERNAME_ID";//$NON-NLS-1$
	private static final String STORE_URL_ID =
		"DavWizardMainPage.STORE_HOSTNAME_ID";//$NON-NLS-1$
	private static final String STORE_PROXY_ID =
		"DavWizardMainPage.STORE_PATH_ID";//$NON-NLS-1$
	private static final String STORE_TIMEOUT_ID =
		"DavWizardMainPage.STORE_MODULE_ID";//$NON-NLS-1$

		
	/*
	 * ConfigurationWizardMainPage constructor.
	 */
	public WebDavSiteCreationPage(String pageName, String title, String description, ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		setDescription(description);
		setTitle(title);
	}	
	
	/*
	 * Creates the UI part of the page.
	 * 
	 * @param parent  the parent of the created widgets
	 */
	public void createControl(Composite parent) {
		Composite composite = createComposite(parent, 2);

		createLabel(composite, Policy.bind("WebDavWizardMainPage.url")); //$NON-NLS-1$
		hostnameUrlCombo = createEditableCombo(composite);
		hostnameUrlCombo.addListener(SWT.Modify, new Listener() {
			public void handleEvent(Event e) {
				WebDavSiteCreationPage.this.validateFields();		
			}
		});

		createLabel(composite, Policy.bind("WebDavWizardMainPage.userName")); //$NON-NLS-1$
		usernameCombo = createEditableCombo(composite);
		
		createLabel(composite, Policy.bind("WebDavWizardMainPage.password")); //$NON-NLS-1$
		password = createTextField(composite);
		password.setEchoChar('*');

		createLabel(composite, Policy.bind("WebDavWizardMainPage.proxy")); //$NON-NLS-1$
		proxyCombo = createEditableCombo(composite);
		proxyCombo.addListener(SWT.Modify, new Listener() {
			public void handleEvent(Event e) {
				WebDavSiteCreationPage.this.validateFields();		
			}
		});

		createLabel(composite, Policy.bind("WebDavWizardMainPage.timeout")); //$NON-NLS-1$
		timeoutCombo = createEditableCombo(composite);
		timeoutCombo.addListener(SWT.Modify, new Listener() {
			public void handleEvent(Event e) {
				WebDavSiteCreationPage.this.validateFields();		
			}
		});

		initializeValues();
		validateFields();
		hostnameUrlCombo.setFocus();
		setControl(composite);
	}
	
	public Site getSite() {
		if(!isPageComplete()) {
			return null;
		}

		WebDavSite site;
		try {
			site = (WebDavSite)Site.createSite(WebdavPlugin.SITE_TYPE_ID, hostnameUrlCombo.getText());
			site.setUsername(usernameCombo.getText());
			site.setPassword(password.getText());
			try {
				String text = timeoutCombo.getText();
				int timeout = Integer.parseInt(text);
				site.setTimeout(timeout);
			} catch (NumberFormatException e) {
				// Just skip the timeout
			}
			site.setProxyUrl(proxyCombo.getText());
			return site;
		} catch (CoreException e) {
			WebdavPlugin.log(e);
			return null;
		}
	}
			
	/*
	 * @see WizardPage#finish
	 */
	public boolean finish(IProgressMonitor monitor) {
		// Set the result to be the current values
		saveWidgetValues();
		return true;
	}
	
	/*
	 * Initializes states of the controls.
	 */
	private void initializeValues() {
		IDialogSettings settings = getDialogSettings();
		if (settings != null) {
			String[] hostNames = settings.getArray(STORE_URL_ID);
			if (hostNames != null) {
				for (int i = 0; i < hostNames.length; i++) {
					hostnameUrlCombo.add(hostNames[i]);
				}
			}
			String[] userNames = settings.getArray(STORE_USERNAME_ID);
			if (userNames != null) {
				for (int i = 0; i < userNames.length; i++) {
					usernameCombo.add(userNames[i]);
				}
			}
			String[] proxies = settings.getArray(STORE_PROXY_ID);
			if (proxies != null) {
				for (int i = 0; i < proxies.length; i++) {
					proxyCombo.add(proxies[i]);
				}
			}			
			String[] timeouts = settings.getArray(STORE_TIMEOUT_ID);
			if (timeouts != null) {
				for (int i = 0; i < timeouts.length; i++) {
					timeoutCombo.add(timeouts[i]);
				}
			}
			timeoutCombo.setText(String.valueOf(WebDavSite.DEFAULT_TIMEOUT));
		}
	}
	
	/*
	 * Saves the widget values
	 */
	private void saveWidgetValues() {
		// Update history
		IDialogSettings settings = getDialogSettings();
		if (settings != null) {
			String[] userNames = settings.getArray(STORE_USERNAME_ID);
			if (userNames == null) userNames = new String[0];
			userNames = addToHistory(userNames, usernameCombo.getText());
			settings.put(STORE_USERNAME_ID, userNames);
	
			String[] hostNames = settings.getArray(STORE_URL_ID);
			if (hostNames == null) hostNames = new String[0];
			hostNames = addToHistory(hostNames, hostnameUrlCombo.getText());
			settings.put(STORE_URL_ID, hostNames);
	
			String[] paths = settings.getArray(STORE_PROXY_ID);
			if (paths == null) paths = new String[0];
			paths = addToHistory(paths, proxyCombo.getText());
			settings.put(STORE_PROXY_ID, paths);

			String[] modules = settings.getArray(STORE_TIMEOUT_ID);
			if (modules == null) modules = new String[0];
			modules = addToHistory(modules, timeoutCombo.getText());
			settings.put(STORE_TIMEOUT_ID, modules);	
		}
	}

	/*
	 * Validates the contents of the editable fields and set page completion 
	 * and error messages appropriately.
	 */
	private void validateFields() {
		// Validate the server URL
		String host = hostnameUrlCombo.getText();
		if (host.length() == 0) {
			setPageComplete(false);
			return;
		}
		
		Site site = Target.getSiteManager().getSite(host);
		if(site != null) {
			setMessage(Policy.bind("WebDAVWizardMainPage.locationAlreadyExists"), WARNING); //$NON-NLS-1$
			setPageComplete(false);
			return;
		}

		// Validate the proxy URL.
		String proxy = proxyCombo.getText();
		if (proxy.length() != 0) {
			try {
				new URL(proxy);
			} catch(MalformedURLException e) {
				setMessage(Policy.bind("WebDAVWizardMainPage.invalidProxyURL"), WARNING); //$NON-NLS-1$
				setPageComplete(false);
				return;
			}
		}
		
		// validate the timeout
		String timeout = timeoutCombo.getText();
		try {
			int timeoutInt = Integer.parseInt(timeout);
			if(timeoutInt <= 0) {
				setMessage(Policy.bind("WebDAVWizardMainPage.invalidTimeout"), WARNING); //$NON-NLS-1$
				setPageComplete(false);
				return;		
			}
		} catch(NumberFormatException e) {
			setMessage(Policy.bind("WebDAVWizardMainPage.invalidTimeout"), WARNING); //$NON-NLS-1$
			setPageComplete(false);
			return;		
		}
		
		setMessage(null);
		setPageComplete(true);
	}
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if (visible) {
			hostnameUrlCombo.setFocus();
		}
	}

}
