/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.bus.core;

import java.lang.reflect.InvocationTargetException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop. Manage Log in a "error log" console
 */
public class CorePlugin extends Plugin
{

	// The shared instance.
	private static CorePlugin plugin;

	// Resource bundle.
	private ResourceBundle resourceBundle;

	// Preference : Name of the prerence "path template"
	public static String PREFERENCEPATHTEMPLATE = "template_path";

	/**
	 * The constructor.
	 */
	public CorePlugin()
	{
		super();
		plugin = this;
		try
		{
			resourceBundle = ResourceBundle.getBundle("org.topcased.bus.core.CorePluginResources");
		}
		catch (MissingResourceException x)
		{
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 * 
	 * @param context : the BundleContext
	 * @exception Exception
	 */
	public void start(BundleContext context) throws Exception
	{
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 * 
	 * @param context : the BundleContext
	 * @exception Exception
	 */
	public void stop(BundleContext context) throws Exception
	{
		super.stop(context);
	}

	/**
	 * Getter for the shared instance.
	 * 
	 * @return CorePlugin
	 */
	public static CorePlugin getDefault()
	{
		return plugin;
	}

	/**
	 * @param key
	 * @return String : the string from the plugin's resource bundle, or 'key' if not found.
	 */
	public static String getResourceString(String key)
	{
		ResourceBundle bundle = CorePlugin.getDefault().getResourceBundle();
		try
		{
			return (bundle != null) ? bundle.getString(key) : key;
		}
		catch (MissingResourceException e)
		{
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle
	 * 
	 * @return ResourceBundle : the plugin resource bundle
	 */
	public ResourceBundle getResourceBundle()
	{
		return resourceBundle;
	}

	/**
	 * Getter for the plugin identifier
	 * 
	 * @return the Plugin Id
	 */
	public static String getId()
	{
		return getDefault().getBundle().getSymbolicName();
	}

	/**
	 * Log an IStatus
	 * 
	 * @param status
	 */
	public static void log(IStatus status)
	{
		ResourcesPlugin.getPlugin().getLog().log(status);
	}

	/**
	 * Log a message with a given level into the Eclipse log file
	 * 
	 * @param message : the message to log
	 * @param level : the message priority
	 */
	public static void log(String message, int level)
	{
		IStatus status = new Status(level, getId(), level, message, null);

		log(status);
	}

	/**
	 * Log a message with given level into the Eclipse log file and with given name class which call it
	 * 
	 * @param serviceClass calling => you must write : this.getClass()
	 * @param message : the message to log
	 * @param level : the message priority (IStatus.INFO or .ERROR or .WARNING ...)
	 * 
	 */
	public static void log(Class<?> serviceClass, String message, int level)
	{
		IStatus status = new Status(level, serviceClass.getName(), level, message, null);
		log(status);
	}

	/**
	 * Log a message with given level into the Eclipse log file and with given name class which call it
	 * 
	 * @param serviceClass calling => you must write : this.getClass()
	 * @param message : the message to log
	 * @param level : the message priority (IStatus.INFO or IStatus.ERROR or IStatus.WARNING ...)
	 * @param e : exception stack
	 */
	public static void log(Class<?> serviceClass, String message, int level, Exception e)
	{
		IStatus status = new Status(level, serviceClass.getName(), level, message, e);
		log(status);
	}

	/**
	 * Log an exception into the Eclipse log file
	 * 
	 * @param e : the exception to log
	 */
	public static void log(Throwable e)
	{
		if (e instanceof InvocationTargetException)
			e = ((InvocationTargetException) e).getTargetException();

		IStatus status = null;
		if (e instanceof CoreException)
			status = ((CoreException) e).getStatus();
		else
			status = new Status(IStatus.ERROR, getId(), IStatus.OK, "Error", e);

		log(status);
	}

}
