/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.bus.core;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.topcased.bus.core.parameter.Parameter;
import org.topcased.bus.core.parameter.Type;

/**
 * Manage services of bus <br>
 */
public class ServicesManager
{

	private static ServicesManager manager = new ServicesManager();

	private Map<String, Service> services; // key = IdService, value = Service

	private Map<String, Type> types; // key = IdService, value = Type

	private Map<String, Parameter> parameters; // key = IdService, value = Parameter

	private Map<String, Parameter> parametersIn; // key = IdService, value = Parameter

	private Map<String, Parameter> parametersOut; // key = IdService, value = Parameter

	/** Extension point name */
	public static final String SERVICES_EXTENSION_POINT = "Services";

	// Service arguments
	/** TAG : service */
	public static final String TAG_SERVICE = "service";

	/** Attribut of service : id */
	public static final String ATT_ID_SERVICE = "id";

	/** Attribut of service : name */
	public static final String ATT_NAME_SERVICE = "name";

	/** Attribut of service : class */
	public static final String ATT_CLASS_SERVICE = "class";

	// Type arguments
	/** TAG : ontologyType */
	public static final String TAG_TYPE = "ontologyType";

	/** Attribut of ontologyType : id */
	public static final String ATT_ID_TYPE = "id";

	/** Attribut of ontologyType : name */
	public static final String ATT_NAME_TYPE = "name";

	/** Attribut of ontologyType : parentId */
	public static final String ATT_PARENTID_TYPE = "parentId";

	/** Attribut of ontologyType : category */
	public static final String ATT_CATEGORY_TYPE = "category";

	// Parameter arguments
	/** TAG : parameter */
	public static final String TAG_PARAM = "parameter";

	/** Attribute of parameter : id */
	public static final String ATT_ID_PARAM = "id";

	/** Attribute of parameter : name */
	public static final String ATT_NAME_PARAM = "name";

	/** Attribute of parameter : description */
	public static final String ATT_DESCRIPTION_PARAM = "description";

	/** Attribute of parameter : kind */
	public static final String ATT_KIND_PARAM = "kind";

	/** List of input parameters */
	public static final String LIST_INPUT_PARAM = "listInputParam";

	/** Output parameter */
	public static final String OUTPUT_PARAM = "outputParam";

	/**
	 * Constructor ServicesManager : Instances of "services" and "types" are done
	 */
	private ServicesManager()
	{
		services = new HashMap<String, Service>();
		types = new HashMap<String, Type>();
		parameters = new HashMap<String, Parameter>();
		parametersIn = new HashMap<String, Parameter>();
		parametersOut = new HashMap<String, Parameter>();
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the singleton
	 */
	public static ServicesManager getInstance()
	{
		return manager;
	}

	/**
	 * Initialization of private Map "services" and "types" if it is not done yet RETRIEVE CHAIN service |_type |_outputParam |_Param |_inputParam |_Param
	 */
	private void initialize()
	{
		if (services.isEmpty())
		{
			retrieveServices();
		}
	}

	/**
	 * Clears the private Map "services"
	 */
	public void setEmptyServices()
	{
		services.clear();
	}

	/**
	 * Clears the private Map "types"
	 */
	public void setEmptyTypes()
	{
		types.clear();
	}

	/**
	 * Clears the private Map "parameters"
	 */
	public void setEmptyParameters()
	{
		parameters.clear();
	}

	/**
	 * Searchs all plugins which use SERVICES extention point with tag SERVICE and TYPE. Fill private Map "services" and "types"
	 */
	private void retrieveServices()
	{
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(CorePlugin.getDefault().getBundle().getSymbolicName(),
		        SERVICES_EXTENSION_POINT);

		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length; i++)
		{
			IConfigurationElement[] elements = extensions[i].getConfigurationElements();
			for (int j = 0; j < elements.length; j++)
			{
				Service thisService = null;
				if (elements[j].getName().compareTo(TAG_SERVICE) == 0 && elements[j].getAttribute(ATT_ID_SERVICE) != null
				        && elements[j].getAttribute(ATT_NAME_SERVICE) != null && elements[j].getAttribute(ATT_CLASS_SERVICE) != null)
				{
					// Infos Service
					try
					{
						IService serviceClass = (IService) elements[j].createExecutableExtension(ATT_CLASS_SERVICE);
						thisService = new Service(elements[j].getAttribute(ATT_ID_SERVICE), elements[j].getAttribute(ATT_NAME_SERVICE),
						        serviceClass);
						services.put(elements[j].getAttribute(ATT_ID_SERVICE), thisService);
					}
					catch (CoreException e)
					{
						CorePlugin.log(e);
						CorePlugin.log(this.getClass(), "Attribute class of the extension point Services is not a IService ",
						        IStatus.WARNING);
					}

				}
				retrieveTypes(elements[j], thisService);
				retrieveOutputParam(elements[j], thisService);
				retrieveInputParam(elements[j], thisService);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag TYPE. Fill private Map "types" and the parameter "this Service"
	 * 
	 * @param serviceElement : an element of the private MAP "services"
	 * @param thisService : Service
	 */
	private void retrieveTypes(IConfigurationElement serviceElement, Service thisService)
	{
		IConfigurationElement[] elementsType = serviceElement.getChildren(TAG_TYPE);

		for (int j = 0; j < elementsType.length; j++)
		{
			Type thisType;
			if (elementsType[j].getName().compareTo(TAG_TYPE) == 0 && elementsType[j].getAttribute(ATT_ID_TYPE) != null
			        && elementsType[j].getAttribute(ATT_NAME_TYPE) != null
			        && elementsType[j].getAttribute(ATT_CATEGORY_TYPE) != null)
			{
				thisType = new Type(elementsType[j].getAttribute(ATT_ID_TYPE), elementsType[j].getAttribute(ATT_NAME_TYPE), elementsType[j]
				        .getAttribute(ATT_PARENTID_TYPE), elementsType[j].getAttribute(ATT_CATEGORY_TYPE));
				types.put(elementsType[j].getAttribute(ATT_ID_TYPE), thisType);
				thisService.setTypeId(elementsType[j].getAttribute(ATT_ID_TYPE));
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag OUTPUTPARAM. Fill the parameter "this Service"
	 * 
	 * @param serviceElement : an element of the private MAP "services"
	 * @param thisService : Service
	 */
	private void retrieveOutputParam(IConfigurationElement serviceElement, Service thisService)
	{
		IConfigurationElement[] elementsParam = serviceElement.getChildren(OUTPUT_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			if (elementsParam[j].getName().compareTo(OUTPUT_PARAM) == 0)
			{
				retrieveParameters(elementsParam[j], thisService, false);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag INPUTPARAM. Fill the parameter "this Service"
	 * 
	 * @param serviceElement : an element of the private MAP "services"
	 * @param thisService : Service
	 */
	private void retrieveInputParam(IConfigurationElement serviceElement, Service thisService)
	{
		IConfigurationElement[] elementsParam = serviceElement.getChildren(LIST_INPUT_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			if (elementsParam[j].getName().equals(LIST_INPUT_PARAM))
			{
				retrieveParameters(elementsParam[j], thisService, true);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag PARAMETER. Fill private Map "parameters"
	 * 
	 * @param paramElement : an element of the private MAP "services"
	 * @param thisService : Service
	 * @param in : "true" if the prameter is a input and "false" if the parameter is an output
	 */
	private void retrieveParameters(IConfigurationElement paramElement, Service thisService, boolean in)
	{
		IConfigurationElement[] elementsParam = paramElement.getChildren(TAG_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			Parameter thisParameter;
			if (elementsParam[j].getName().compareTo(TAG_PARAM) == 0 && elementsParam[j].getAttribute(ATT_ID_PARAM) != null
			        && elementsParam[j].getAttribute(ATT_NAME_PARAM) != null
			        && elementsParam[j].getAttribute(ATT_DESCRIPTION_PARAM) != null
			        && elementsParam[j].getAttribute(ATT_KIND_PARAM) != null)
			{
				thisParameter = new Parameter(elementsParam[j].getAttribute(ATT_ID_PARAM), elementsParam[j].getAttribute(ATT_NAME_PARAM),
				        elementsParam[j].getAttribute(ATT_DESCRIPTION_PARAM), elementsParam[j].getAttribute(ATT_KIND_PARAM));
				parameters.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
				if (in == true)
				{
					parametersIn.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
					thisService.setParamInId(elementsParam[j].getAttribute(ATT_ID_PARAM));
				}
				if (in == false)
				{
					parametersOut.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
					thisService.setParamOutId(elementsParam[j].getAttribute(ATT_ID_PARAM));
				}

			}
		}
	}

	/**
	 * Gets all services ids
	 * 
	 * @return Set : set of services Id
	 */
	public Set<String> getAllServicesIds()
	{
		initialize();
		return services.keySet();
	}

	/**
	 * Gets all types ids
	 * 
	 * @return Set : set of types Id
	 */
	public Set<String> getAllTypesIds()
	{
		initialize();
		return types.keySet();
	}

	/**
	 * Gets all parameters ids
	 * 
	 * @return Set : set of parameters Id
	 */
	public Set<String> getAllParametersIds()
	{
		initialize();
		return parameters.keySet();
	}

	/**
	 * Get all service ids with their description
	 * 
	 * @return Map : key=services Id; value=Service
	 */
	public Map<String, Service> getAllServices()
	{
		initialize();
		return services;
	}

	/**
	 * Gets all parameters IDs with their description
	 * 
	 * @return Map : key=parameters in Id; value=Parameter
	 */
	public Map<String, Parameter> getAllParametersIn()
	{
		initialize();
		return parametersIn;
	}

	/**
	 * Gets all output parameters IDs with their description
	 * 
	 * @return Map : key=parameters out Id; value=Parameter
	 */
	public Map<String, Parameter> getAllParametersOut()
	{
		initialize();
		return parametersOut;
	}

	/**
	 * Gets all types IDs with their description
	 * 
	 * @return Map : key=types in Id; value=Type
	 */
	public Map<String, Type> getAllType()
	{
		initialize();
		return types;
	}

	/**
	 * Gives the Service corrresponding to an accurate service id from the private Map services
	 * 
	 * @param serviceId : service Id
	 * @return Service corresponding to service id
	 */
	public Service serviceId2Service(String serviceId)
	{
		initialize();
		for (String id : services.keySet())
		{
			if (id.equals(serviceId))
			{
				return services.get(id);
			}
		}
		return null;
	}

	/**
	 * Gives the Type corrresponding to a precise type id from the private Map types
	 * 
	 * @param typeId : type id
	 * @return Type corresponding to type id
	 */
	public Type typeId2Type(String typeId)
	{
		initialize();
		for (String id : types.keySet())
		{
			if (id.equals(typeId))
			{
				return types.get(id);
			}
		}
		return null;
	}

	/**
	 * Gives the Parameter corrresponding to a precise parameter id from the private Map parameters
	 * 
	 * @param paramId : parameter id
	 * @return Parameter corresponding to parameter id
	 */
	public Parameter parameterId2Parameter(String paramId)
	{
		initialize();
		for (String id : parameters.keySet())
		{
			if (id.equals(paramId))
			{
				return parameters.get(id);
			}
		}
		return null;
	}

	/**
	 * Gives a Map (serviceId,Service) corrresponding to a precise service id from the private Map "services"
	 * 
	 * @param serviceId
	 * @return key=serviceId, value=Service : only one line
	 */
	public Map<String, Service> serviceId2ServiceMap(String serviceId)
	{
		initialize();
		Map<String, Service> serviceMap = new HashMap<String, Service>();
		serviceMap.put(serviceId, serviceId2Service(serviceId));
		return serviceMap;
	}

	/**
	 * Gives a Map (typeId,Type) corrresponding to a precise service id from the private Map "types"
	 * 
	 * @param typeId
	 * @return key=typeId, value=Type : only one line
	 */
	public Map<String, Type> typeId2TypeMap(String typeId)
	{
		initialize();
		Map<String, Type> typeMap = new HashMap<String, Type>();
		typeMap.put(typeId, typeId2Type(typeId));
		return typeMap;
	}

	/**
	 * Gives a Map (parameterId,Paramter) corrresponding to a precise service id from the private Map "paramters"
	 * 
	 * @param parameterId
	 * @return key=parameterId, value=Paramter : only one line
	 */
	public Map<String, Parameter> parameterId2ParameterMap(String parameterId)
	{
		initialize();
		Map<String, Parameter> paramMap = new HashMap<String, Parameter>();
		paramMap.put(parameterId, parameterId2Parameter(parameterId));
		return paramMap;
	}

	/**
	 * Gives Map (key=serviceId, value=Service) corresponding to the Set of serviceId
	 * 
	 * @param serviceIds : Set (value = serviceId)
	 * @return Map (key=serviceId, value=Service)
	 */
	public Map<String, Service> serviceIds2ServiceMap(Set<String> serviceIds)
	{
		initialize();
		Map<String, Service> serviceMap = new HashMap<String, Service>();
		for (String idService : serviceIds)
		{
			serviceMap.putAll(serviceId2ServiceMap(idService));
		}
		return serviceMap;
	}

	/**
	 * Gives Map (key=serviceId, value=Service) from service Map without basic services
	 * 
	 * @param serviceMap (key=serviceId, value=Service)
	 * @return Map (key=serviceId, value=Service) without basics Services
	 */
	public Map<String, Service> serviceMapNotBasic(Map<String, Service> serviceMap)
	{
		initialize();
		Map<String, Service> serviceMapNoBasic = new HashMap<String, Service>();
		for (String serviceId : serviceMap.keySet())
		{
			if (!isBasicService(serviceId))
			{
				serviceMapNoBasic.put(serviceId, serviceMap.get(serviceId));
			}
		}
		return serviceMapNoBasic;
	}

	/**
	 * Gives Map (key=typeId, value=Type) corresponding to the Set of typeId
	 * 
	 * @param typeIds : Set (value = typeId)
	 * @return Map (key=typeId, value=Type)
	 */
	public Map<String, Type> typeIds2TypeMap(Set<String> typeIds)
	{
		initialize();
		Map<String, Type> typeMap = new HashMap<String, Type>();
		for (String id : typeIds)
		{
			typeMap.putAll(typeId2TypeMap(id));
		}
		return typeMap;
	}

	/**
	 * Gives Map (key=paramId, value=Parameter) corresponding to the Set of parameterIds
	 * 
	 * @param paramIds : Set (value = paramId)
	 * @return Map (key=paramId, value=Parameter)
	 */
	public Map<String, Parameter> parameterIds2ParameterMap(Set<String> paramIds)
	{
		initialize();
		Map<String, Parameter> paramMap = new HashMap<String, Parameter>();
		for (String param : paramIds)
		{
			paramMap.putAll(parameterId2ParameterMap(param));
		}
		return paramMap;
	}

	/**
	 * Returns the Map (key=type id; value=Type) corresponding to the service id
	 * 
	 * @param serviceId : service id
	 * @return Map : key=type id; value=Type
	 */
	public Map<String, Type> serviceId2TypeMap(String serviceId)
	{
		initialize();
		return typeId2TypeMap(serviceId2Service(serviceId).getIdType());
	}

	/**
	 * Returns the Map (key=parameter id; value=Parameter) of input parameter corresponding to the service id
	 * 
	 * @param serviceId : service id
	 * @return Map : key=parameter id; value=Parameter
	 */
	public Map<String, Parameter> serviceId2ParamOutMap(String serviceId)
	{
		initialize();
		return parameterId2ParameterMap(serviceId2Service(serviceId).getIdParameterOut());
	}

	/**
	 * Returns the Map (key=parameter id; value=Parameter) of input parameter corresponding to the service id
	 * 
	 * @param serviceId : service id
	 * @return Map : key=parameter id; value=Parameter
	 */
	public Map<String, Parameter> serviceId2ParamInMap(String serviceId)
	{
		initialize();
		Set<String> paramInId = serviceId2Service(serviceId).getIdParameterIn();
		return parameterIds2ParameterMap(paramInId);
	}

	/**
	 * Returns the Map (key=type id; value=Type) corresponding to the typeName
	 * 
	 * @param typeName : type name
	 * @param listType : key=type id; value=Type
	 * @return Map (key=type id; value=Type)
	 */
	public Map<String, Type> typeName2TypeMap(String typeName, Map<String, Type> listType)
	{
		initialize();
		Map<String, Type> typeMap = new HashMap<String, Type>();
		for (String type : listType.keySet())
		{
			String aTypeName = listType.get(type).getNameType();
			if (typeName.equals(aTypeName))
			{
				typeMap.put(type, listType.get(type));
			}
		}
		return typeMap;
	}

	/**
	 * Returns the Map (key=parameter id; value=Parameter) of (In or Out) parameter corresponding to the parameterKind
	 * 
	 * @param parameterKind : parameter kind
	 * @param listParam : key=parameter id; value=Parameter
	 * @return Map (key=parameter id; value=Parameter)
	 */
	public Map<String, Parameter> parameterKind2ParameterMap(String parameterKind, Map<String, Parameter> listParam)
	{
		initialize();
		Map<String, Parameter> paramInMap = new HashMap<String, Parameter>();
		for (String param : listParam.keySet())
		{
			String aParamKind = listParam.get(param).getKindParam();
			if (parameterKind.equals(aParamKind))
			{
				paramInMap.put(param, listParam.get(param));
			}
		}
		return paramInMap;
	}

	/**
	 * Returns IService from Service Id.
	 * 
	 * @param serviceId the id of a service
	 * @return the associated service or <code>null</code> if the service is not found.
	 */
	public IService getService(String serviceId)
	{
		initialize();
		if (serviceId != null && !serviceId.equals(""))
		{
			return serviceId2Service(serviceId).getClass_();
		}
		return null;
	}

	/**
	 * Returns Map corresponding to typeService, listInputKind, outputKind in all services on the bus
	 * 
	 * @param typeService : service type
	 * @param listInputKind : kind input list (value = String)
	 * @param outputKind : kind output
	 * @return Map (key = IdService, value = Service) of all services corresponding to criteria : typeService, listInputKind, outputKind
	 */
	public Map<String, Service> getService(String typeService, Set<String> listInputKind, String outputKind)
	{
		initialize();
		Map<String, Service> allServices = getAllServices();
		for (String kind : listInputKind)
		{
			allServices = getServicesWithCriteria(allServices, typeService, kind, outputKind);
		}
		return allServices;
	}

	/**
	 * Gets all services which have this "typeName", this "paramInputKind" and this "paramOutputKind" in parameters It is a filter on "type", on "input" and on
	 * "output"
	 * 
	 * @param listService : Map where the filter is realized
	 * @param typeName : type name
	 * @param paramInputKind : input kind
	 * @param paramOutputKind : output kind
	 * @return a Map of all services which have this "typeName", this "paramInputKind" and this "paramOutputKind" : key = service id, value = Service
	 */
	public Map<String, Service> getServicesWithCriteria(Map<String, Service> listService, String typeName, String paramInputKind,
	        String paramOutputKind)
	{
		initialize();
		return getServicesPerOutput(paramOutputKind, getServicesPerInput(paramInputKind, getServicesPerType(typeName, listService)));
	}

	/**
	 * Gets the list of parameters ID from a list of services.<br>
	 * This list corresponds to all parameter used like input parameters in this "listService"
	 * 
	 * @param listService : key=service id, value=Service
	 * @return Set of parameterId which are used like parameters input : value = parameter id
	 */
	public Set<String> getListTypeId(Map<String, Service> listService)
	{
		initialize();
		Set<String> typeId = new HashSet<String>();
		for (String service : listService.keySet())
		{
			typeId.add(listService.get(service).getIdType());
		}
		return typeId;
	}

	/**
	 * Gets list of parameters ID from a list of services.<br>
	 * This list corresponds to all parameters used like input parameters in this "listService"
	 * 
	 * @param listService : key=service id, value=Service
	 * @return Set of parameterId which are used like parameters input : value = parameter id
	 */
	public Set<String> getListInputParamId(Map<String, Service> listService)
	{
		initialize();
		Set<String> inputParamId = new HashSet<String>();
		for (String service : listService.keySet())
		{
			inputParamId.addAll(listService.get(service).getIdParameterIn());
		}
		return inputParamId;
	}

	/**
	 * Gets list of parameters ID from a list of services.<br>
	 * This list corresponds to all parameters used like input parameters in this "listService"
	 * 
	 * @param listService : key=service id, value=Service
	 * @return Set of parameterId which are used like parameters input : value = parameter id
	 */
	public Set<String> getListOutputParamId(Map<String, Service> listService)
	{
		initialize();
		Set<String> outputParamId = new HashSet<String>();
		for (String service : listService.keySet())
		{
			outputParamId.add(listService.get(service).getIdParameterOut());
		}
		return outputParamId;
	}

	/**
	 * Gets a Map of all Type which exist in the Services list
	 * 
	 * @param listService : Services list
	 * @return a Map of all type . (key=idType, value=Type)
	 */
	public Map<String, Type> getListType(Map<String, Service> listService)
	{
		initialize();
		return typeIds2TypeMap(getListTypeId(listService));
	}

	/**
	 * Get a Map of all Input parameters which exist in the Services list
	 * 
	 * @param listService : Services list
	 * @return a Map of all input parameter. (key=idParameter, value=Parameter)
	 */
	public Map<String, Parameter> getListInput(Map<String, Service> listService)
	{
		initialize();
		return parameterIds2ParameterMap(getListInputParamId(listService));
	}

	/**
	 * Gets a Map of all Output parameters which exists in the Service list
	 * 
	 * @param listService : Services list
	 * @return a Map of all Output. (key=idParameter, value=Parameter)
	 */
	public Map<String, Parameter> getListOutput(Map<String, Service> listService)
	{
		initialize();
		return parameterIds2ParameterMap(getListOutputParamId(listService));
	}

	// /**
	// * Gets all services which exist for the meta model id "metaModelIdOrigin" in the
	// * listService
	// *
	// * @param metaModelIdOrigin : MetaModel Origin id
	// * @param listService : services list (key=service id, value=Service)
	// * @return a Map of services. (key=service id, value=Service)
	// */
	// public Map<String, Service> getServicePerMetaModelOrigin(String metaModelIdOrigin, Map<String, Service> listService)
	// {
	// initialize();
	// if (metaModelIdOrigin.equals(""))
	// {
	// return listService;
	// }
	// else
	// {
	// Map<String, Service> serviceMap = new HashMap<String, Service>();
	// for (String serviceId : listService.keySet())
	// {
	// Service service = serviceId2Service(serviceId);
	// if (service != null)
	// {
	// if (metaModelIdOrigin.equals(service.getIdMetaModelOrigin()))
	// {
	// serviceMap.put(serviceId, listService.get(serviceId));
	// }
	// }
	// }
	// return serviceMap;
	// }
	// }

	/**
	 * Gets all services which exist for the type "typeName" in the listService
	 * 
	 * @param typeName : type name
	 * @param listService : services list (key=service id, value=Service)
	 * @return a Map of services. (key=service id, value=Service)
	 */
	public Map<String, Service> getServicesPerType(String typeName, Map<String, Service> listService)
	{
		initialize();
		if (typeName.equals(""))
		{
			return listService;
		}
		else
		{
			Map<String, Service> serviceMap = new HashMap<String, Service>();
			for (String serviceId : listService.keySet())
			{
				Service service = serviceId2Service(serviceId);
				if (service != null)
				{
					String aTypeName = typeId2Type(service.getIdType()).getNameType();
					if (typeName.equals(aTypeName))
					{
						serviceMap.put(serviceId, listService.get(serviceId));
					}
				}
			}
			return serviceMap;
		}
	}

	/**
	 * Gets all services which exist with this "input" in the listService
	 * 
	 * @param inputKind : kind parameter input
	 * @param listService : services list (key=service id, value=Service)
	 * @return a Map of services. (key=service id, value=Service)
	 */
	public Map<String, Service> getServicesPerInput(String inputKind, Map<String, Service> listService)
	{
		initialize();
		if (inputKind.equals(""))
		{
			return listService;
		}
		else
		{
			Map<String, Service> serviceMap = new HashMap<String, Service>();
			for (String serviceId : listService.keySet())
			{
				for (String service : serviceId2Service(serviceId).getIdParameterIn())
				{
					String aInputKind = parameterId2Parameter(service).getKindParam();
					if (inputKind.equals(aInputKind))
					{
						serviceMap.put(serviceId, listService.get(serviceId));
					}
				}
			}
			return serviceMap;
		}
	}

	/**
	 * Get all services which exist with this "output" in the listService
	 * 
	 * @param outputKind : kind parameter output
	 * @param listService : Services list (key=service id, value=Service)
	 * @return a Map of services. (key=service id, value=Service)
	 */
	public Map<String, Service> getServicesPerOutput(String outputKind, Map<String, Service> listService)
	{
		initialize();
		if (outputKind.equals(""))
		{
			return listService;
		}
		else
		{
			Map<String, Service> serviceMap = new HashMap<String, Service>();
			for (String service : listService.keySet())
			{
				String paramOutputId = serviceId2Service(service).getIdParameterOut();
				String aOutputKind = parameterId2Parameter(paramOutputId).getKindParam();
				if (outputKind.equals(aOutputKind))
				{
					serviceMap.put(service, listService.get(service));
				}
			}
			return serviceMap;
		}
	}

	/**
	 * @return a Map of all Services using Services extension Point Full Map is the list for Administrator of project
	 */
	public Map<String, Service> getRegisteredServicesOnBus()
	{
		initialize();
		return services;
	}

	/**
	 * Return a list of all services which are activated ???? c'est le meme que getUsersAvailableServicesOnProject???
	 * 
	 * @param listService
	 * @return a Map of all services which are activated
	 */
	public Map<String, Service> getActiveServices(Map<String, Service> listService)
	{
		return listService;
	}

	/**
	 * This function is used to determine whether a service is a basic one or not
	 * 
	 * @param serviceId the id of the service
	 * @return true is the service is a basic one, false otherwise
	 */
	public static boolean isBasicService(String serviceId)
	{
		return BasicServices.isBasicService(serviceId);
	}
}
