/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.bus.core.metamodel;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;

/**
 * Manage MetaModels of bus <br>
 * creation : 7 juin 2005
 * 
 * @author <a href="mailto:marion.feau@c-s.fr">mfeau</a>
 */
public class MetamodelsManager
{
	// Instance of the MetaModelsManager
	static private MetamodelsManager manager = new MetamodelsManager();

	/** Extension point name */
	public static final String METAMODEL_EXTENSION_POINT = "Metamodels";

	// MetaModels arguments
	/** Service tag */
	public static final String TAG_METAMODEL = "metamodel";

	/** Attribute of service : url */
	public static final String ATT_METAMODEL_URI = "uri";

	/** Attribute of service : name */
	public static final String ATT_METAMODEL_IDENTIFIER = "id";

	/** Attribute of service : version */
	public static final String ATT_METAMODEL_VERSION = "version";

	/** Attribute of service : completName */
	public static final String ATT_METAMODEL_NAME = "name";

	/** Attribute of service : description */
	public static final String ATT_METAMODEL_DESCRIPTION = "description";

	/** Attribute of service : directory to find path to MM */
	public static final String ATT_METAMODEL_PATH = "MetamodelPath";

	/** Map containing the whole of MetaModels */
	private Map<String, Metamodel> metaModelMap = null;

	/**
	 * Constructor
	 */
	private MetamodelsManager()
	{
		metaModelMap = new HashMap<String, Metamodel>();
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the singleton
	 */
	public static MetamodelsManager getInstance()
	{
		return manager;
	}

	/**
	 * Retrieve MetaModels Retrieve all MetaModels extension point Fill the map metaModelsMap with all inforamtions from MetaModels extension point
	 */
	private void retrieveMetaModels()
	{
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(CorePlugin.getDefault().getBundle().getSymbolicName(),
				METAMODEL_EXTENSION_POINT);

		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length; i++)
		{
			String nameSpacePlugin = extensions[i].getNamespaceIdentifier();
			extensions[i].getExtensionPointUniqueIdentifier();
			IConfigurationElement[] elements = extensions[i].getConfigurationElements();
			for (int j = 0; j < elements.length; j++)
			{
				if (elements[j].getName().compareTo(TAG_METAMODEL) == 0 && elements[j].getAttribute(ATT_METAMODEL_URI) != null
						&& elements[j].getAttribute(ATT_METAMODEL_IDENTIFIER) != null && elements[j].getAttribute(ATT_METAMODEL_VERSION) != null
						&& elements[j].getAttribute(ATT_METAMODEL_NAME) != null
						&& elements[j].getAttribute(ATT_METAMODEL_DESCRIPTION) != null
						&& elements[j].getAttribute(ATT_METAMODEL_PATH) != null)
				{
					// Retrieve value
					String uri = elements[j].getAttribute(ATT_METAMODEL_URI);
					String id = elements[j].getAttribute(ATT_METAMODEL_IDENTIFIER);
					String version = elements[j].getAttribute(ATT_METAMODEL_VERSION);
					String name = elements[j].getAttribute(ATT_METAMODEL_NAME);
					String description = elements[j].getAttribute(ATT_METAMODEL_DESCRIPTION);
					String recipient = elements[j].getAttribute(ATT_METAMODEL_PATH);
					// String pluginName = elements[j].getDeclaringExtension().getDeclaringPluginDescriptor().getInstallURL().getPath();
					String pluginName = elements[j].getDeclaringExtension().getNamespaceIdentifier();
					Metamodel thisMetaModel = new Metamodel(uri, id, version, name, description, recipient, nameSpacePlugin,
							pluginName);
					metaModelMap.put(uri, thisMetaModel);
				}
			}
		}
	}

	/**
	 * Initializes private map : metaModelMap
	 */
	private void initialize()
	{
		if (metaModelMap.isEmpty())
		{
			retrieveMetaModels();
		}
	}

	/**
	 * @return all MetaModels url
	 */
	public Set<String> getMetaModelsIds()
	{
		initialize();
		return metaModelMap.keySet();
	}

	// ************************* MM (Uri) --> MM (EObject) **********************
	/**
	 * Get EObject Meta Model from Meta Model uri (xmlns:X ="thisUri")
	 * 
	 * @param uriMM : String of the metat model uri
	 * @return Meta Model EObject
	 */
	public EObject uri2EObject(String uriMM)
	{
		String xmiFileMM = uriMM2XmiFileMM(uriMM);
		EObject eObjectMM = xmiFileMM2EObjectMM(xmiFileMM);
		return eObjectMM;
	}

	/**
	 * Get XmiFile from the uri meta model
	 * 
	 * @param uriMM
	 * @return xmiFile
	 */
	public String uriMM2XmiFileMM(String uriMM)
	{
		String xmiFile = "";
		xmiFile = getPluginName(uriMM) + File.separator + getMetaModelDirectory(uriMM) + File.separator + getName(uriMM);
		xmiFile = Platform.getInstallLocation().getURL().getPath() + File.separator + "plugins" + File.separator + xmiFile;
		CorePlugin.log(this.getClass(), xmiFile, IStatus.INFO);
		return xmiFile;
	}

	/**
	 * Get EObject (meta model) with the xmiFile
	 * 
	 * @param xmiFile : xmiFile of the MM
	 * @return EObject : return EObject of the xmiFile
	 */
	public EObject xmiFileMM2EObjectMM(String xmiFile)
	{
		// Message Read
		Map<String, Service> services = ServicesManager.getInstance().getAllServices();
		Map<String, Service> services2 = ServicesManager.getInstance().getServicesPerType("Read", services);
		Map<String, Object> param = new HashMap<String, Object>();

		Service s = null;
		for (Iterator<String> it = services2.keySet().iterator(); it.hasNext();)
		{
			String key = it.next();
			s = services2.get(key);
		}
		IService is = ServicesManager.getInstance().getService(s.getId());
		param.put("XMIFile", xmiFile);
		EObject eObjectMM = (EObject) is.serviceRun(param);

		return eObjectMM;
	}

	/**
	 * Get URI of the Meta model corresponding to pluginNameSpace && metamodelVersion
	 * 
	 * @param pluginNameSpace
	 * @param metamodelVersion
	 * @return meta model URI
	 * @throws URISyntaxException
	 */
	public URI getUriMM(String pluginNameSpace, String metamodelVersion) throws URISyntaxException
	{
		URI uri = null;
		String uriString = getUri(pluginNameSpace, metamodelVersion);
		uri = new URI(uriString);
		return uri;
	}

	/**
	 * Get the URI of the meta model corresponding to pluginNameSpace && metaModelVersion
	 * 
	 * @param pluginNameSpace : name space of the plugin
	 * @param metaModelVersion : version of the meta model
	 * @return String URI (metaModelMap key) corresponding to pluginNameSpace and metaModelVersion Return "" if no one URI exist for pluginNameSpace and
	 *         metaModelVersion.
	 */
	private String getUri(String pluginNameSpace, String metaModelVersion)
	{
		String uri = "";
		boolean first = true;
		Map<String, Metamodel> filterMapMM = metaModelMap;
		filterMapMM = filterVersionMM(filterMapMM, metaModelVersion);
		filterMapMM = filterPluginNameSpaceMM(filterMapMM, pluginNameSpace);
		Iterator<String> it = filterMapMM.keySet().iterator();
		while (it.hasNext() && first)
		{
			uri = it.next();
			first = false;
		}
		return uri;
	}

	/**
	 * Filter aMetaModelMap with the plugin nameSpace
	 * 
	 * @param aMetaModelMap : key=uri, value=MetaModel
	 * @param nameSpaceMM : plugin meta model name space
	 * @return Map : key=uri, value=MetaModel
	 */
	private Map<String, Metamodel> filterPluginNameSpaceMM(Map<String, Metamodel> aMetaModelMap, String nameSpaceMM)
	{
		Map<String, Metamodel> aMetaModelMapReturn = new HashMap<String, Metamodel>();
		if (aMetaModelMap != null && nameSpaceMM != null)
		{
			for (String keyUrl : aMetaModelMap.keySet())
			{
				Metamodel mm = aMetaModelMap.get(keyUrl);
				String mmPluginNameSpace = mm.getPluginNameSpace();
				if (nameSpaceMM.equals(mmPluginNameSpace))
				{
					aMetaModelMapReturn.put(keyUrl, mm);
				}
			}
		}
		return aMetaModelMapReturn;
	}

	/**
	 * Filter aMetaModelMap with the version
	 * 
	 * @param aMetaModelMap : key=uri, value=MetaModel
	 * @param versionMM : version of the meta model
	 * @return Map : key=uri, value=MetaModel
	 */
	private Map<String, Metamodel> filterVersionMM(Map<String, Metamodel> aMetaModelMap, String versionMM)
	{
		Map<String, Metamodel> aMetaModelMapReturn = new HashMap<String, Metamodel>();
		if (aMetaModelMap != null && versionMM != null)
		{
			for (String keyUrl : aMetaModelMap.keySet())
			{
				Metamodel mm = aMetaModelMap.get(keyUrl);
				String mmVersion = mm.getVersion();
				if (versionMM.equals(mmVersion))
				{
					aMetaModelMapReturn.put(keyUrl, mm);
				}
			}
		}
		return aMetaModelMapReturn;
	}

	// //////////////////////////// end "GET" URI[] ///////////////////////////////////////////

	// ////////////////////////////// "GET" FROM METAMODEL WITH "URL" /////////////////////////
	/**
	 * @param url : url of MetaModel
	 * @return plugin path for the meta model indicate by url
	 */
	public String getMetaModelDirectory(String url)
	{
		initialize();
		return metaModelMap.get(url).getMetamodelPath();
	}

	/**
	 * @param url : url of MetaModel
	 * @return name space of the MetaModel url
	 */
	public String getPluginNameSpace(String url)
	{
		initialize();
		return metaModelMap.get(url).getPluginNameSpace();
	}

	/**
	 * Gets plugin name which contains MetaModel
	 * 
	 * @param url : url of MetaModel
	 * @return plugin name of the MetaModel url
	 */
	public String getPluginName(String url)
	{
		initialize();
		return metaModelMap.get(url).getPluginName();
	}

	/**
	 * Gets MetaModel name
	 * 
	 * @param url : url of MetaModel
	 * @return name of the MetaModel url
	 */
	public String getName(String url)
	{
		initialize();
		return metaModelMap.get(url).getName();
	}

	/**
	 * Gets MetaModel description
	 * 
	 * @param url : url of MetaModel
	 * @return description of the MetaModel url
	 */
	public String getDescription(String url)
	{
		initialize();
		return metaModelMap.get(url).getDescription();
	}

	/**
	 * Gets MetaModel complete name
	 * 
	 * @param url : url of MetaModel
	 * @return complete name of the MetaModel url
	 */
	public String getCompleteName(String url)
	{
		initialize();
		return metaModelMap.get(url).getCompleteName();
	}

	/**
	 * Gets MetaModel version
	 * 
	 * @param url : url of MetaModel
	 * @return version of the MetaModel url
	 */
	public String getVersion(String url)
	{
		initialize();
		return metaModelMap.get(url).getVersion();
	}
}
