/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.checkresultmodel.dialog;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.osgi.framework.Bundle;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;

public class CheckresultmodelTreeViewerFactory
{

	private static final Image REJECTED_IMAGE;

	private static final Image SELECTED_IMAGE;

	static
	{
		Image tempImage;
		Bundle bundle = Platform.getBundle("org.topcased.checkresultmodel");

		tempImage = ImageDescriptor.createFromURL(FileLocator.find(bundle, new Path("icons/Rejected.gif"), null)).createImage();
		REJECTED_IMAGE = tempImage;

		tempImage = ImageDescriptor.createFromURL(FileLocator.find(bundle, new Path("icons/Selected.gif"), null)).createImage();
		SELECTED_IMAGE = tempImage;
	}

	public static TreeViewer createTreeViewer(Composite parent, boolean showSelected)
	{
		TreeViewer tree = new TreeViewer(parent);
		configureTreeViewer(tree, showSelected);
		return tree;
	}

	private static void configureTreeViewer(TreeViewer tree, final boolean showSelected)
	{
		tree.setContentProvider(getContentProvider(showSelected));

		tree.setLabelProvider(new LabelProvider()
		{
			public String getText(Object element)
			{
				if (element instanceof Model)
				{
					return "Model : " + getText(((Model) element).getInformation().getModelContent());
				}
				else if (element instanceof Package)
				{
					return "Package : " + getText(((Package) element).getPackageReference());
				}
				else if (element instanceof Context)
				{
					return "Context : " + getText(((Context) element).getContextClass());
				}
				else if (element instanceof Rule)
				{
					String ruleText = "Rule : " + ((Rule) element).getRuleText();
					ruleText = ruleText + " : " + (((Rule) element).getRejectedItems().isEmpty());
					return ruleText;
				}
				else if (element instanceof Item)
				{
					if (((Rule) ((Item) element).eContainer()).getRejectedItems().contains(element))
					{
						return "Rejected : " + getText(((Item) element).getReference());
					}
					else
					{
						return "Selected : " + getText(((Item) element).getReference());
					}
				}
				else if (element instanceof ENamedElement)
				{
					return ((ENamedElement) element).getName();
				}
				else if (element instanceof EObject)
				{
					String resultTemp = null;
					boolean nameFound = false;
					Iterator<EAttribute> itAtt = ((EObject) element).eClass().getEAllAttributes().iterator();
					while (itAtt.hasNext() && !nameFound)
					{
						EAttribute currentAttribute = itAtt.next();
						if (currentAttribute.getEAttributeType().getName().equals("EString"))
						{
							Object attribValue = ((EObject) element).eGet(currentAttribute);
							if (attribValue != null)
							{
								resultTemp = attribValue.toString();
								if (currentAttribute.getName().equalsIgnoreCase("Name"))
								{
									nameFound = true;
								}
							}

						}
					}
					if (resultTemp != null)
					{
						return ((EObject) element).eClass().getName() + " " + resultTemp;
					}
					else
					{
						return ((EObject) element).eClass().getName();
					}
				}
				else if (element != null)
				{
					return element.toString();
				}
				else
				{
					return "";
				}

			}

			public Image getImage(Object element)
			{
				if (element instanceof Item)
				{
					if (((Rule) ((Item) element).eContainer()).getRejectedItems().contains(element))
					{
						return REJECTED_IMAGE;
					}
					else
					{
						return SELECTED_IMAGE;
					}
				}
				return super.getImage(element);
			}
		});

		tree.setSorter(new ViewerSorter());
	}

	public static ITreeContentProvider getContentProvider(final boolean showSelected)
	{
		return new ITreeContentProvider()
		{

			private boolean opening = true;

			public Object[] getChildren(Object parentElement)
			{
				if (parentElement instanceof List<?>)
				{
					return ((List<?>) parentElement).toArray();
				}
				if (parentElement instanceof Model)
				{
					return ((Model) parentElement).getContextPackages().toArray();
				}
				if (parentElement instanceof Package)
				{
					return ((Package) parentElement).getContext().toArray();
				}
				if (parentElement instanceof Context)
				{
					return ((Context) parentElement).getRules().toArray();
				}
				if (parentElement instanceof Rule)
				{
					List<Item> children = new ArrayList<Item>();
					children.addAll(((Rule) parentElement).getRejectedItems());

					if (showSelected)
					{
						children.addAll(((Rule) parentElement).getSelectedItems());
					}
					return children.toArray();
				}
				return null;
			}

			public Object getParent(Object element)
			{
				if (element instanceof EObject)
				{
					return ((EObject) element).eContainer();
				}
				return null;
			}

			public boolean hasChildren(Object element)
			{
				if (element instanceof List)
				{
					return !((List<?>) element).isEmpty();
				}
				if (element instanceof Model)
				{
					return !((Model) element).getContextPackages().isEmpty();
				}
				if (element instanceof Package)
				{
					return !((Package) element).getContext().isEmpty();
				}
				if (element instanceof Context)
				{
					return !((Context) element).getRules().isEmpty();
				}
				if (element instanceof Rule)
				{
					boolean result = !((Rule) element).getRejectedItems().isEmpty();
					result = result || (showSelected && !((Rule) element).getSelectedItems().isEmpty());
					return result;
				}
				return false;
			}

			public Object[] getElements(Object inputElement)
			{
				if (inputElement instanceof List)
				{
					return ((List<?>) inputElement).toArray();
				}
				if (inputElement instanceof Model)
				{
					if (opening)
					{
						opening = false;
						return new Object[] { inputElement };
					}
					return ((Model) inputElement).getContextPackages().toArray();
				}
				if (inputElement instanceof Package)
				{
					return ((Package) inputElement).getContext().toArray();
				}
				if (inputElement instanceof Context)
				{
					return ((Context) inputElement).getRules().toArray();
				}
				if (inputElement instanceof Rule)
				{
					List<Item> children = new ArrayList<Item>();
					children.addAll(((Rule) inputElement).getRejectedItems());

					if (showSelected)
					{
						children.addAll(((Rule) inputElement).getSelectedItems());
					}
					return children.toArray();
				}
				return null;
			}

			public void dispose()
			{
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
			{

			}
		};
	}
}
