/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.checkresultmodel.helper;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.topcased.checkresultmodel.CheckresultmodelFactory;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.ModelInformation;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.checkresultmodel.State;

/**
 * Helper class proposing a set of convenient static methods to easily handle CheckResultModel.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class CheckResultModelHelper
{
	/**
	 * Creates a default Topcased CheckResultModel
	 * 
	 * @param model The EMF model
	 * @param mmURI The metamodel of the model
	 * @param context The context where the rule must be take into account
	 * @param rule The OCL rule
	 * @return a default CheckResultModel
	 */
	public static Model createCheckResulModel(EObject model, EPackage package_, EClass context, String rule)
	{
		ModelInformation information = CheckresultmodelFactory.eINSTANCE.createModelInformation();
		information.setModelContent(model);
		
		Model modelResult = CheckresultmodelFactory.eINSTANCE.createModel();
		modelResult.setInformation(information);

		Package contextPackageModel = createPackage(modelResult, package_);
		
		Context contextModel = createContext(contextPackageModel, context);
		
		createRule(contextModel, rule);

		return modelResult;
	}
	
	/**
	 * Creates a Package and adds it to the Check Result Model
	 *  
	 * @param model The check result model 
	 * @param ePackage A EPackage to represent into the model
	 * @return the created package object
	 */
	public static Package createPackage(Model model, EPackage ePackage)
	{
		Package contextPackageModel = CheckresultmodelFactory.eINSTANCE.createPackage();
		contextPackageModel.setPackageReference(ePackage);
		contextPackageModel.setName(ePackage.getName());
		model.getContextPackages().add(contextPackageModel);
		return contextPackageModel;
	}
	
	/**
	 * Creates a Context and adds it to the current Package 
	 * 
	 * @param package_ The current Package of the check result model
	 * @param context A EClass representing the context of an OCL rule.
	 * @return the created context object
	 */
	public static Context createContext(Package package_, EClass context)
	{
		Context contextModel = CheckresultmodelFactory.eINSTANCE.createContext();
		contextModel.setContextClass(context);
		contextModel.setName(context.getName());
		package_.getContext().add(contextModel);
		return contextModel;
	}
	
	/**
	 * Creates a Rule and adds it to the current context
	 * 
	 * @param context the rule context
	 * @param rule The rule applied
	 * @return the created rule object
	 */
	public static Rule createRule(Context context, String rule)
	{
		Rule ruleModel = CheckresultmodelFactory.eINSTANCE.createRule();
		ruleModel.setName(rule);
		ruleModel.setRuleText(rule);
		context.getRules().add(ruleModel);
		return ruleModel;
	}
	
	/**
	 * Creates a list of the accepted items concerning a rule. 
	 * 
	 * @param rule The rule on which the items must be added
	 * @param acceptedItems List of accepted items
	 */
	public static void createAcceptedItems(Rule rule, List<?> acceptedItems)
	{
		for (int i = 0; i < acceptedItems.size(); i++)
		{
			Item accepted = CheckresultmodelFactory.eINSTANCE.createItem();
			accepted.setName(State.SELECTED);
			accepted.setReference((EObject) acceptedItems.get(i));
			accepted.setValue(Boolean.TRUE);
			rule.getSelectedItems().add(accepted);
		}
	}
	
	/**
	 * Creates a list of the rejected items concerning a rule. 
	 * 
	 * @param rule The rule on which the items must be added
	 * @param rejectedItems List of rejected items
	 */
	public static void createRejectedItems(Rule rule, List<?> acceptedItems)
	{
		for (int i = 0; i < acceptedItems.size(); i++)
		{
			Item accepted = CheckresultmodelFactory.eINSTANCE.createItem();
			accepted.setName(State.REJECTED);
			accepted.setReference((EObject) acceptedItems.get(i));
			accepted.setValue(Boolean.FALSE);
			rule.getSelectedItems().add(accepted);
		}
	}
	
	/**
	 * Gets the first Rule found into a given CheckResultModel.<br>
	 * <b>NOTE</b> : only used by ModelBus services.
	 * 
	 * @param checkResultModel A CheckResultModel previously created
	 * @return The unique object Rule contained in the given CheckResultModel
	 */
	public static Rule getFirstRule(Model checkResultModel)
	{
		return checkResultModel.getContextPackages().get(0).getContext().get(0).getRules().get(0);
	}
}
