/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.editors.ocleditor.assistant;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.ETypedElement;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.ocl.EnvironmentFactory;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.helper.Choice;
import org.eclipse.ocl.helper.ChoiceKind;
import org.eclipse.ocl.helper.ConstraintKind;
import org.eclipse.ocl.helper.OCLHelper;
import org.eclipse.swt.graphics.Image;
import org.topcased.bus.core.CorePlugin;
import org.topcased.editors.ocleditor.OCLEditorPlugin;
import org.topcased.editors.ocleditor.editor.OCLEditorDocumentPartitioner;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;

/**
 * 
 * @author <a href="mailto:christophe.le-camus@c-s.fr">Christophe LE CAMUS</a>
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class OCLEditorContentProcessor implements IContentAssistProcessor
{

	private EPackage metamodelPackage = null;

	private OCLContentScanner oclScanner;

	private static final Image PACKAGE_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/package.GIF")).createImage();

	private static final Image CLASS_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/class.GIF")).createImage();

	private static final Image ATTRIB_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/attrib.GIF")).createImage();

	private static final Image OPERAT_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/operat.GIF")).createImage();

	private static final Image VAR_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/var.GIF")).createImage();

	private static final Image LET_IMAGE = ImageDescriptor.createFromURL(
	        OCLEditorPlugin.getDefault().getBundle().getEntry("/icons/let.GIF")).createImage();


	/**
	 * Constructor
	 */
	public OCLEditorContentProcessor()
	{
		oclScanner = new OCLContentScanner();
	}

	/**
	 * @param ePackage
	 */
	public void setBasePackage(EPackage ePackage)
	{
		metamodelPackage = ePackage;
	}

	/**
	 * Gets rule text 
	 * 
	 * @param oclDocument The OCL document to analyse
	 * @param region
	 * @return
	 */
	private String getRuleText(Document oclDocument, ITypedRegion region)
	{
		String ruleText = new String("");
		try
		{
			ruleText = oclDocument.get(region.getOffset(), region.getLength());
		}
		catch (Exception e)
		{
			CorePlugin.log(getClass(), "BadLocationException :" + e.getMessage(), IStatus.ERROR, e);
		}
		return ruleText.trim();
	}

	/**
	 * @see org.eclipse.jface.text.contentassist.IContentAssistProcessor#computeCompletionProposals(org.eclipse.jface.text.ITextViewer, int)
	 */
	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int offset)
	{
		// Defines the document to be analyzed
		Document oclDocument = new Document(viewer.getDocument().get());
		oclScanner.setDocument(oclDocument);
		
		// Defines the root URI
		String firstLine = new String("");
		try
		{
			firstLine = oclDocument.get(0, oclDocument.getLineLength(0));
		}
		catch (BadLocationException e)
		{
			CorePlugin.log(getClass(), "BadLocationException :" + e.getMessage(), IStatus.ERROR, e);
		}
		
		String rootModelURI = firstLine.substring("MainModel :".length()).trim();
		URI rootURI = URI.createURI(rootModelURI);
		EPackage modelPackage = OCLRuleUtil.findMetamodelPackage(rootURI);
		EPackage rootPackage = modelPackage;

		// create the Environment and the Helper
		EnvironmentFactory<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> env = EcoreEnvironmentFactory.INSTANCE
		        .createInstanceContext(EcoreEnvironmentFactory.INSTANCE.createEnvironment(), metamodelPackage).getFactory();

		OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
		        .newInstance(env);
		OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper = ocl.createOCLHelper();

		// analyzed the document to get the different regions
		OCLEditorDocumentPartitioner partitionner = new OCLEditorDocumentPartitioner();
		partitionner.connect(viewer.getDocument());
		ITypedRegion[] regions = partitionner.computePartitioning(0, offset);
		List<Object> myList = new ArrayList<Object>(0);
		EClassifier contextClassifier = null;
		ITypedRegion packageRegion = regions[0];
		String startString = new String("");

		for (ITypedRegion typedRegion : Arrays.asList(regions))
		{
			if (typedRegion.getType().equals(IDocument.DEFAULT_CONTENT_TYPE))
			{
				String ruleText = getRuleText(oclDocument, typedRegion);

				if (!ruleText.matches("\\s*"))
				{
					if (ruleText.startsWith("context"))
					{
						contextClassifier = OCLRuleUtil.computeContextClassifier(oclDocument, metamodelPackage, typedRegion, packageRegion);

						if (contextClassifier != null && contextClassifier instanceof EClass)
						{
							oclHelper.setContext(contextClassifier);
						}
						myList.clear();
						if (ruleText.indexOf("::") == -1)
						{
							myList.addAll(modelPackage.getEClassifiers());
						}
						else
						{
							ETypedElement typedContext = OCLRuleUtil.computeTypedContext(oclDocument, modelPackage, typedRegion,
							        packageRegion);
							if (typedContext == null)
							{
								myList.addAll(((EClass) contextClassifier).getEAllStructuralFeatures());
								myList.addAll(((EClass) contextClassifier).getEAllOperations());
							}
							else
							{
								if (typedContext instanceof EOperation)
								{
									EOperation operation = (EOperation) typedContext;
									oclHelper.setOperationContext(contextClassifier, operation);
								}
								
								if (typedContext instanceof EStructuralFeature)
								{
									EStructuralFeature feature = (EStructuralFeature) typedContext;
									oclHelper.setInstanceAttributeContext(contextClassifier, feature);
								}
							}
						}
						startString = computeStartOfContextString(ruleText);
						myList = filterList(myList, startString);
					}
					if (ruleText.startsWith("package"))
					{
						String modelURIString = ruleText.substring("Package".length()).trim();
						modelURIString = modelURIString.replaceAll("::", "/");
						URI modelURI = null;
						modelURI = URI.createURI(rootModelURI);
						if (modelURIString != "")
						{
							if (modelURI.isFile())
							{
								modelURI = modelURI.appendFragment(modelURIString);
							}
							else
							{
								modelURI = modelURI.appendSegment(modelURIString);
							}
						}
						modelPackage = OCLRuleUtil.findMetamodelPackage(modelURI);
						packageRegion = typedRegion;
						oclHelper.setInstanceContext(modelPackage);
						myList.clear();
						myList.addAll(modelPackage.getESubpackages());
					}
					if (ruleText.startsWith("endpackage"))
					{
						packageRegion = typedRegion;
						modelPackage = rootPackage;
						oclHelper.setInstanceContext(rootPackage);
						myList.clear();
						myList.addAll(rootPackage.getESubpackages());
					}
				}
			}
			else
			{
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.RULE_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.INVARIANT, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.PRE_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.PRECONDITION, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.POST_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.POSTCONDITION, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.BODY_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.BODYCONDITION, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.DEFINITION_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.INVARIANT, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.INIT_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.INVARIANT, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
				
				if (typedRegion.getType().equals(OCLEditorDocumentPartitioner.DERIVE_CONTENT))
				{
					String ruleText = getRuleText(oclDocument, typedRegion);
					myList.clear();
					myList.addAll(oclHelper.getSyntaxHelp(ConstraintKind.INVARIANT, ruleText));
					startString = computeStartString(ruleText);
					myList = filterList(myList, startString);
				}
			}
		}

		return fillProposals(oclHelper, myList, offset, startString);
	}

	/**
	 * @param ruleText
	 * @return
	 */
	private String computeStartString(String ruleText)
	{
		String returnString = new String("");
		int posRuleSeparator = ruleText.lastIndexOf(":") + 1;
		int posPoint = ruleText.lastIndexOf(".") + 1;
		int posArrow = ruleText.lastIndexOf("->") + 2;
		int maxSeparator = Math.max(posPoint, posArrow);
		maxSeparator = Math.max(maxSeparator, posRuleSeparator);
		returnString = returnString.concat(ruleText.substring(maxSeparator));
		return returnString.trim();
	}

	/**
	 * @param ruleText
	 * @return
	 */
	private String computeStartOfContextString(String ruleText)
	{
		String returnString = new String("");
		int posDoubleColon = ruleText.lastIndexOf("::") + 2;
		if (posDoubleColon - 2 == -1)
		{
			int posContext = ruleText.lastIndexOf("context ") + "context ".length();
			returnString = returnString.concat(ruleText.substring(posContext));
		}
		else
		{
			returnString = returnString.concat(ruleText.substring(posDoubleColon));
		}
		return returnString.trim();
	}

	/**
	 * @param myList
	 * @param startString
	 * @return
	 */
	private List<Object> filterList(List<Object> myList, String startString)
	{
		if (startString.length() == 0)
		{
			return myList;
		}
		else
		{
			if (startString.indexOf("(") != -1)
			{
				startString = startString.substring(0, startString.indexOf("("));
			}
		}
		
		List<Object> filteredList = new ArrayList<Object>();
		for (Object anObject : myList)
		{

			if (anObject instanceof Choice)
			{
				Choice completionItem = (Choice) anObject;
				String completionName = completionItem.getName();
				if (completionName.startsWith(startString))
				{
					filteredList.add(anObject);
				}
			}
			
			if (anObject instanceof ENamedElement)
			{
				ENamedElement completionItem = (ENamedElement) anObject;
				String completionName = completionItem.getName();
				if (completionName.startsWith(startString))
				{
					filteredList.add(anObject);
				}
			}
		}
		return filteredList;
	}

	/**
	 * @param oclHelper
	 * @param item
	 * @return
	 */
	private String computeStructuralFeatureDescription(OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper,
	        Choice item)
	{
		EClass contextClass = (EClass) oclHelper.getContextClassifier();
		EList<EStructuralFeature> allFeatures = contextClass.getEAllStructuralFeatures();
		boolean found = false;
		int i = 0;
		EStructuralFeature myFeature = null;
		while (i < allFeatures.size() && !found)
		{
			if (((EStructuralFeature) allFeatures.get(i)).getName().equals(item.getName()))
			{
				myFeature = (EStructuralFeature) allFeatures.get(i);
				found = !found;
			}
			i++;
		}

		String typeString = new String("");
		if (myFeature != null)
		{
			typeString = myFeature.getEType().getName();
			if (myFeature.isMany())
			{
				if (myFeature.isOrdered())
				{
					if (myFeature.isUnique())
					{
						typeString = "OrderedSet(".concat(typeString).concat(")");
					}
					else
					{
						typeString = "Sequence(".concat(typeString).concat(")");
					}
				}
				else
				{
					if (myFeature.isUnique())
					{
						typeString = "Set(".concat(typeString).concat(")");
					}
					else
					{
						typeString = "Bag(".concat(typeString).concat(")");
					}
				}
			}
			else
			{
				typeString = myFeature.getEType().getName();
			}
		}
		else
		{
			typeString = item.getDescription().trim();
		}
		return typeString;
	}

	/**
	 * @param item
	 * @return
	 */
	private String splitOperationDescription(Choice item)
	{
		String name = new String("");
		int posRihgtPar = item.getDescription().indexOf(")");
		name = name.concat(item.getDescription().substring(0, posRihgtPar + 1));
		return name;
	}

	/**
	 * @param oclHelper
	 * @param myList
	 * @param offset
	 * @param startString
	 * @return
	 */
	CompletionProposal[] fillProposals(OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper, List<Object> myList,
	        int offset, String startString)
	{
		CompletionProposal[] proposals = null;
		
		// Compute the proposals
		List<String> limitedList = new ArrayList<String>();
		List<Image> listImage = new ArrayList<Image>();
		List<String> listDescription = new ArrayList<String>();

		if (myList.isEmpty())
		{
			limitedList.add("No possible completion here ... ");
			listDescription.add("No possible completion here ... ".concat(new String("Try another way.")));
			listImage.add(PACKAGE_IMAGE);
		}

		for (int i = 0; i < myList.size(); i++)
		{
			if (myList.get(i) instanceof Choice)
			{
				Choice completionItem = (Choice) myList.get(i);
				ChoiceKind kind = completionItem.getKind();
				switch (kind)
				{
					case PROPERTY:
						if (completionItem.getElement() instanceof EAttribute)
						{
							EAttribute attr = (EAttribute) completionItem.getElement();

							if (attr.getEType() instanceof EEnum)
							{
								// maintain the old mapping of enumeration-type attributes
								// ENUMERATION_LITERAL
								limitedList.add(completionItem.getName());
								listDescription.add(completionItem.getName().concat(
								        new String(" # ".concat(completionItem.getDescription()))));
								listImage.add(LET_IMAGE);
								break;
							}
						}
						// // STRUCTURAL_FEATURE
					case TYPE:
					case STATE:
					case ASSOCIATION_CLASS:
					case PACKAGE:
					case ENUMERATION_LITERAL:
						// STRUCTURAL_FEATURE
						limitedList.add(completionItem.getName());
						String typeString = computeStructuralFeatureDescription(oclHelper, completionItem);
						listDescription.add(completionItem.getName().concat(new String(" : ".concat(typeString))));
						listImage.add(ATTRIB_IMAGE);
						break;
					case SIGNAL:
					case OPERATION:
						// BEHAVIORAL_FEATURE
						// limitedList.add(completionItem.getName());
						limitedList.add(splitOperationDescription(completionItem));
						listDescription.add(new String(completionItem.getDescription()));
						listImage.add(OPERAT_IMAGE);
						break;
					case VARIABLE:
						// VARIABLE
						limitedList.add(completionItem.getName());
						listDescription.add(completionItem.getName().concat(new String(" : ".concat(completionItem.getDescription()))));
						listImage.add(VAR_IMAGE);
						break;
					default:
						// UNCATEGORIZED
						limitedList.add(completionItem.getName());
						listDescription.add(completionItem.getName().concat(new String(" : UNCATEGORIZED")));
						listImage.add(CLASS_IMAGE);
						break;
				}
			}
			else
			{
				if (myList.get(i) instanceof EPackage)
				{
					EPackage aPackage = (EPackage) myList.get(i);
					limitedList.add(aPackage.getName());
					listDescription.add(aPackage.getName().concat(new String(" : EPackage")));
					listImage.add(PACKAGE_IMAGE);
				}
				if (myList.get(i) instanceof EClass)
				{
					EClass aClass = (EClass) myList.get(i);
					limitedList.add(aClass.getName());
					listDescription.add(aClass.getName().concat(new String(" : EClass")));
					listImage.add(CLASS_IMAGE);
				}
				if (myList.get(i) instanceof EAttribute)
				{
					EAttribute anAttribute = (EAttribute) myList.get(i);
					limitedList.add(anAttribute.getName());
					listDescription.add(anAttribute.getName().concat(new String(" : ").concat(anAttribute.getEType().getName())));
					listImage.add(ATTRIB_IMAGE);
				}
				if (myList.get(i) instanceof EReference)
				{
					EReference aReference = (EReference) myList.get(i);
					limitedList.add(aReference.getName());
					listDescription.add(aReference.getName().concat(new String(" : ").concat(aReference.getEType().getName())));
					listImage.add(ATTRIB_IMAGE);
				}
				if (myList.get(i) instanceof EOperation)
				{
					EOperation anOperation = (EOperation) myList.get(i);
					limitedList.add(buildOperationNameWithReturnType(anOperation));
					listDescription.add(buildOperationNameWithReturnType(anOperation));
					listImage.add(OPERAT_IMAGE);
				}
			}
		}

		proposals = new CompletionProposal[limitedList.size()];
		// TODO : compute the length of the word which has to be replaced
		// case1 : a word between two points case of completion on an existing word
		// case2 : if we ask from a point
		for (int i = 0; i < limitedList.size(); i++)
		{
			String name = (String) limitedList.get(i);
			// Object tooltipValue = tooltips.get(name);
			String tooltipString = "";
			// if(tooltipValue != null)
			// {
			// tooltipString = (String) tooltipValue;
			// }
			//			
			// * @param replacementString the actual string to be inserted into the document
			// * @param replacementOffset the offset of the text to be replaced
			// * @param replacementLength the length of the text to be replaced
			// * @param cursorPosition the position of the cursor following the insert relative to replacementOffset
			// * @param image the image to display for this proposal
			// * @param displayString the string to be displayed for the proposal
			// * @param contextInformation the context information associated with this proposal
			// * @param additionalProposalInfo the additional information associated with this proposal
			//			 
			proposals[i] = new CompletionProposal(name, offset - startString.length(), startString.length(), name.length(),
			        (Image) listImage.get(i), (String) listDescription.get(i),
			        // (String) showItem.get(name),
			        null, tooltipString);
		}
		return proposals;
	}

	public IContextInformation[] computeContextInformation(ITextViewer viewer, int offset)
	{
		return null;
	}

	public char[] getCompletionProposalAutoActivationCharacters()
	{
		return new char[] { '.', '>' };
	}

	public char[] getContextInformationAutoActivationCharacters()
	{
		return null;
	}

	public String getErrorMessage()
	{
		return null;
	}

	public IContextInformationValidator getContextInformationValidator()
	{
		return null;
	}

	/**
	 * Compute the first context of the given document
	 * 
	 * @param document the document where the context must be found
	 * @return An EClassifier corresponding to the first context found in the document
	 */
	public EClassifier computeContextClassifier(IDocument document)
	{
		oclScanner.setDocument(document);
		List<String> packagePrefixes = new ArrayList<String>();
		IToken tokenTemp = oclScanner.lookFor("package", 0, document.getLength());
		if (tokenTemp != null)
		{
			if (!oclScanner.endOfScanner())
			{
				tokenTemp = oclScanner.nextToken();
				String tokenValue = oclScanner.getTokenValue();
				while (!oclScanner.endOfScanner() && tokenTemp.getData() != null && tokenTemp.getData().equals(OCLContentScanner.WORD))
				{
					packagePrefixes.add(tokenValue);
					tokenTemp = oclScanner.nextToken();
					tokenValue = oclScanner.getTokenValue();
					if (!oclScanner.endOfScanner() && tokenTemp.getData() != null
					        && tokenTemp.getData().equals(OCLContentScanner.DOUBLE_COLON))
					{
						tokenTemp = oclScanner.nextToken();
						tokenValue = oclScanner.getTokenValue();
					}
				}
			}
		}

		tokenTemp = oclScanner.lookFor("context", 0, document.getLength());
		if (tokenTemp != null)
		{
			tokenTemp = oclScanner.nextToken();
			String contextClass = "";
			if (!tokenTemp.isEOF())
			{
				contextClass = oclScanner.getTokenValue();
			}
			if (packagePrefixes.size() == 0)
			{
				return metamodelPackage.getEClassifier(contextClass);
			}
			else
			{
				if (metamodelPackage.getNsPrefix().equals(packagePrefixes.get(0)))
				{
					EPackage tempPackage = metamodelPackage;
					boolean packageError = false;
					for (int i = 1; i < packagePrefixes.size() && !packageError; i++)
					{
						List<EPackage> subPackages = tempPackage.getESubpackages();
						boolean packageFound = false;
						for (int j = 0; j < subPackages.size() && !packageFound; j++)
						{
							EPackage subi = subPackages.get(j);
							if (subi.getNsPrefix().equals(packagePrefixes.get(i)))
							{
								packageFound = true;
								tempPackage = subi;
							}
						}
						if (!packageFound)
						{
							packageError = true;
						}
					}
					if (!packageError)
					{
						return (EClass) tempPackage.getEClassifier(contextClass);
						// currentPackage = tempPackage;
					}
				}
			}
		}
		return null;
	}

	/**
	 * Compute the context of a specific partition
	 * 
	 * @param document the whole document this function should work on
	 * @param region the region where the context should be found
	 * @return An EClassifier corresponding to the ocntext of the region
	 */
	public EClassifier computeContextClassifierForPartition(IDocument document, ITypedRegion region)
	{
		oclScanner.setDocument(document);
		List<String> packagePrefixes = new Vector<String>();
		IToken tokenTemp = oclScanner.lookFor("package", 0, region.getOffset());
		if (tokenTemp != null)
		{
			if (!oclScanner.endOfScanner())
			{
				tokenTemp = oclScanner.nextToken();
				String tokenValue = oclScanner.getTokenValue();
				while (!oclScanner.endOfScanner() && tokenTemp.getData() != null && tokenTemp.getData().equals(OCLContentScanner.WORD))
				{
					packagePrefixes.add(tokenValue);
					tokenTemp = oclScanner.nextToken();
					tokenValue = oclScanner.getTokenValue();
					if (!oclScanner.endOfScanner() && tokenTemp.getData() != null
					        && tokenTemp.getData().equals(OCLContentScanner.DOUBLE_COLON))
					{
						tokenTemp = oclScanner.nextToken();
						tokenValue = oclScanner.getTokenValue();
					}
				}
			}
		}

		tokenTemp = null;
		int lastPosition = 0;
		tokenTemp = oclScanner.lookFor("context", 0, region.getOffset());
		while (tokenTemp != null)
		{
			lastPosition = oclScanner.getTokenOffset() + "context".length();
			tokenTemp = oclScanner.lookFor("context", lastPosition, region.getOffset() - lastPosition);

		}

		if (lastPosition != 0)
		{
			oclScanner.setRange(lastPosition, region.getOffset() - lastPosition);
			tokenTemp = oclScanner.nextToken();
			String contextClass = "";
			if (!tokenTemp.isEOF())
			{
				contextClass = oclScanner.getTokenValue();
			}
			if (packagePrefixes.size() == 0)
			{
				return metamodelPackage.getEClassifier(contextClass);
			}
			else
			{
				if (metamodelPackage.getNsPrefix().equals(packagePrefixes.get(0)))
				{
					EPackage tempPackage = metamodelPackage;
					boolean packageError = false;
					for (int i = 1; i < packagePrefixes.size() && !packageError; i++)
					{
						List<EPackage> subPackages = tempPackage.getESubpackages();
						boolean packageFound = false;
						for (int j = 0; j < subPackages.size() && !packageFound; j++)
						{
							EPackage subi = subPackages.get(j);
							if (subi.getNsPrefix().equals(packagePrefixes.get(i)))
							{
								packageFound = true;
								tempPackage = subi;
							}
						}
						if (!packageFound)
						{
							packageError = true;
						}
					}
					if (!packageError)
					{
						return (EClass) tempPackage.getEClassifier(contextClass);
						// currentPackage = tempPackage;
					}
				}
			}
		}
		return null;
	}

	/**
	 * @param operation
	 * @return
	 */
	private String buildOperationNameWithReturnType(EOperation operation)
	{
		if (operation.getName() == "")
		{
			return "unNamedOperation()";
		}
		String result = new String(operation.getName().concat("("));
		for (Iterator<EParameter> it = operation.getEParameters().iterator(); it.hasNext();)
		{
			EParameter param = it.next();
			result = result.concat(param.getName()).concat(": ".concat(param.getEType().getName())).concat(", ");
		}
		if (operation.getEParameters().size() != 0)
		{
			result = result.substring(0, result.length() - 2);
		}
		result = result.concat(") : ");
		if (operation.getEType() != null)
		{
			result = result.concat(operation.getEType().getName());
		}
		else
		{
			result = result.concat("OclIsUndefined");
		}
		return result;
	}
}
