package org.topcased.editors.ocleditor.builders;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.ETypedElement;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.ocl.EnvironmentFactory;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.helper.OCLHelper;
import org.topcased.bus.core.CorePlugin;
import org.topcased.editors.ocleditor.editor.OCLEditorDocumentPartitioner;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;

public class OCLEditorBuilder extends IncrementalProjectBuilder
{

	/**
	 * Default constructor
	 */
	public OCLEditorBuilder()
	{
		super();
	}

	/**
	 * @see org.eclipse.core.resources.IncrementalProjectBuilder#build(int, java.util.Map, org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor) throws CoreException
	{
		IResource[] resources = getProject().members();
		if ((kind & INCREMENTAL_BUILD) != 0)
		{
			IResourceDelta projectDelta = getDelta(getProject());
			IResourceDelta[] delta;
			if (projectDelta != null)
			{
				delta = projectDelta.getAffectedChildren(IResourceDelta.ADDED | IResourceDelta.CHANGED);
			}
			else
			{
				delta = new IResourceDelta[0];
			}
			resources = new IResource[delta.length];
			for (int i = 0; i < delta.length; i++)
			{
				resources[i] = delta[i].getResource();
			}
		}
		buildResources(resources);
		return null;
	}

	/**
	 * Builds the resources recursively. It is recursive when a folder is found.
	 * 
	 * @param resources an array of the resources to build
	 * @throws CoreException If there is a problem with a marker or the content of a file resource
	 */
	private void buildResources(IResource[] resources) throws CoreException
	{
		URI modelURI = null;
		ITypedRegion packageRegion = null;
		ITypedRegion modelRegion = null;
		for (int i = 0; i < resources.length; i++)
		{
			if (resources[i] instanceof IFile)
			{
				IFile resourceFile = (IFile) resources[i];
				if (resourceFile.getFileExtension().equals("ocl"))
				{
					InputStream content = resourceFile.getContents();
					resourceFile.deleteMarkers(null, true, IResource.DEPTH_INFINITE);
					BufferedReader contentReader = new BufferedReader(new InputStreamReader(content));
					String text = "";
					try
					{
						String buffer = contentReader.readLine();
						while (buffer != null)
						{
							text = text.concat(buffer + "\n");
							buffer = contentReader.readLine();
						}

						contentReader.close();
						content.close();

						Document oclDocument = new Document(text);
						String firstLine = oclDocument.get(0, oclDocument.getLineLength(0));
						String rootModelURI = firstLine.substring("MainModel :".length()).trim();
						URI rootURI = URI.createURI(rootModelURI);
						EPackage modelPackage = OCLRuleUtil.findMetamodelPackage(rootURI);

						EnvironmentFactory<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> env = EcoreEnvironmentFactory.INSTANCE
						        .createInstanceContext(EcoreEnvironmentFactory.INSTANCE.createEnvironment(), modelPackage).getFactory();

						OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
						        .newInstance(env);
						OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper = ocl.createOCLHelper();

						if (modelPackage != null)
						{
							EClassifier contextClassifier = null;
							{
								EClass context = null;

								OCLEditorDocumentPartitioner partitionner = new OCLEditorDocumentPartitioner();
								partitionner.connect(oclDocument);
								ITypedRegion[] regions = partitionner.computePartitioning(0, oclDocument.getLength());
								modelRegion = regions[0]; // ou 1
								packageRegion = modelRegion;

								for (int j = 0; j < regions.length; j++)
								{
									// Let's examine the typed regions defined
									// inv, pre, post, body, init, derive, definition
									if (!regions[j].getType().equals(IDocument.DEFAULT_CONTENT_TYPE))
									{
										if (context != null)
										{
											try
											{
												Constraint newConstraint = OCLRuleUtil.createConstraint(regions[j], oclDocument, oclHelper);
												if (newConstraint != null)
												{
													newConstraint.getConstrainedElements().add(context);
												}
											}
											catch (ParserException e)
											{
												IMarker marker = resourceFile.createMarker(IMarker.PROBLEM);
												if (e.getCause() instanceof NullPointerException)
												{
													marker.setAttribute(IMarker.MESSAGE, "Context can not be found.");
													marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
													marker.setAttribute(IMarker.LINE_NUMBER, oclDocument.getLineOfOffset(regions[j]
													        .getOffset()) + 1);
												}
												else
												{

													marker.setAttribute(IMarker.MESSAGE, e.getMessage());
													marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
													marker.setAttribute(IMarker.LINE_NUMBER, oclDocument.getLineOfOffset(regions[j]
													        .getOffset()) + 1);
												}
											}
											catch (IllegalArgumentException e)
											{
												IMarker marker = resourceFile.createMarker(IMarker.PROBLEM);
												marker.setAttribute(IMarker.MESSAGE, e.getMessage());
												marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
												marker.setAttribute(IMarker.LINE_NUMBER, oclDocument
												        .getLineOfOffset(regions[j].getOffset()) + 1);
											}
										}
									}
									// Regions : Package, endpackage, Context
									else
									{
										String textTemp = oclDocument.get(regions[j].getOffset(), regions[j].getLength());
										if (!textTemp.matches("\\s*"))
										{
											// if the text is not empty
											// try to find a context
											if (textTemp.startsWith("package"))
											{
												String modelURIString = textTemp.substring("Package".length()).trim();
												modelURIString = modelURIString.replaceAll("::", "/");
												modelURI = URI.createURI(rootModelURI);
												if (modelURIString != "")
												{
													if (modelURI.isFile())
													{
														modelURI = modelURI.appendFragment(modelURIString);
													}
													else
													{
														modelURI = modelURI.appendSegment(modelURIString);
													}
												}
												modelPackage = OCLRuleUtil.findMetamodelPackage(modelURI);
												packageRegion = regions[j];
												oclHelper.setInstanceContext(modelPackage);
											}
											if (textTemp.startsWith("endpackage"))
											{
												modelPackage = OCLRuleUtil.findMetamodelPackage(rootURI);
												packageRegion = modelRegion;
												oclHelper.setInstanceContext(modelPackage);
											}
											if (textTemp.startsWith("context"))
											{
												contextClassifier = OCLRuleUtil.computeContextClassifier(oclDocument, modelPackage,
												        regions[j], packageRegion);
												if (contextClassifier != null && contextClassifier instanceof EClass)
												{
													context = (EClass) contextClassifier;
													oclHelper.setContext(context);
													ETypedElement typedContext = OCLRuleUtil.computeTypedContext(oclDocument, modelPackage,
													        regions[j], packageRegion);
													if (typedContext != null)
													{
														if (typedContext instanceof EOperation)
														{
															oclHelper.setOperationContext(context, (EOperation) typedContext);
														}
														else
														{
															oclHelper.setInstanceAttributeContext(contextClassifier,
															        (EStructuralFeature) typedContext);
														}
													}
												}
												else
												{
													context = null;
													IMarker marker = resourceFile.createMarker(IMarker.PROBLEM);
													// context not found
													String packageName = new String("' Null package '");
													if ((EPackage) modelPackage != null)
													{
														packageName = ((EPackage) modelPackage).getName();
													}
													marker.setAttribute(IMarker.MESSAGE, "Incorrect value : ".concat(textTemp).concat(
													        ". Context not found in package ".concat(packageName)));
													marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
													marker.setAttribute(IMarker.LINE_NUMBER, oclDocument.getLineOfOffset(regions[j]
													        .getOffset()) + 1);
												}
											}
										}
									}
								}
							}
						}
						else
						{
							IMarker marker = resourceFile.createMarker(IMarker.PROBLEM);
							marker.setAttribute(IMarker.MESSAGE, "Meta Model not available : " + rootModelURI);
							marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
							marker.setAttribute(IMarker.LINE_NUMBER, 1);
						}
					}
					catch (Exception e)
					{
						CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
					}
				}
			}
			else if (resources[i] instanceof IContainer)
			{
				IContainer container = (IContainer) resources[i];
				buildResources(container.members());
			}
		}
	}
}
