/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), S�bastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.editors.ocleditor.editor;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.ContentAssistAction;
import org.eclipse.ui.texteditor.DefaultMarkerAnnotationAccess;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.SourceViewerDecorationSupport;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.editors.ocleditor.MetaModelOCL;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;

public class OCLEditor extends TextEditor
{

	private EPackage metamodelPackage;

	private EPackage oclTypesPackage;

	private static final String CONTEXT_MENU_ID = "org.topcased.editors.ocleditor.contextmenu.id";

	public OCLEditor()
	{
		super();
		OCLEditorSourceViewerConfiguration svcConfig = new OCLEditorSourceViewerConfiguration();
		setSourceViewerConfiguration(svcConfig);
		initOCLMetaModel();
		setEditorContextMenuId(CONTEXT_MENU_ID);
		configureSourceViewerDecorationSupport(new SourceViewerDecorationSupport(getSourceViewer(), getOverviewRuler(),
		        new DefaultMarkerAnnotationAccess(), getSharedColors()));
	}

	protected void createActions()
	{
		super.createActions();
		ContentAssistAction ca = new ContentAssistAction(ResourceBundle.getBundle("org.topcased.editors.ocleditor.OCLEditorMessages"),
		        "ContentAssistProposal", this);
		ca.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
		setAction("ContentAssistProposal", ca);
		getSourceViewer().getTextWidget().addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent e)
			{
				try
				{
					if (getSourceViewer().getDocument().getLineOfOffset(e.start) == 0)
					{
						e.doit = false;
					}
					if (getSourceViewer().getDocument().getLineOfOffset(e.start) == 1)
					{
						e.doit = false;
					}
				}
				catch (BadLocationException ex)
				{
					CorePlugin.log(getClass(), ex.getMessage(), IStatus.ERROR, ex);
				}
			}
		});
	}

	public void init(IEditorSite site, IEditorInput input) throws PartInitException
	{
		super.init(site, input);

		setEditorContextMenuId(CONTEXT_MENU_ID);

		OCLEditorDocumentPartitioner partitionner = new OCLEditorDocumentPartitioner();
		IDocument document = getDocumentProvider().getDocument(input);
		document.setDocumentPartitioner(partitionner);
		partitionner.connect(document);

		IPersistableElement element = input.getPersistable();
		if (element != null && element instanceof FileEditorInput)
		{
			FileEditorInput file = (FileEditorInput) element;
			try
			{
				BufferedReader reader = new BufferedReader(new FileReader(file.getFile().getLocation().toFile()));
				String firstLine = reader.readLine();

				if (firstLine != null && firstLine.startsWith("MainModel : "))
				{
					// looks for the OCL metamodel
					String mmUri = firstLine.substring("MainModel :".length()).trim();
					boolean findMetaModel = findInEMF(mmUri);

					// First try to find the meta model in the EMF registry
					if (!findMetaModel)
					{
						CorePlugin.log("Model not found in Registry : ".concat(mmUri), IStatus.WARNING);
						// The input must be a resource coming from workspace
						try
						{
							findMetaModel = findAsEcoreFile(URI.createURI(mmUri).path());
						}
						catch (Exception e)
						{
							CorePlugin.log("Model not found at given location : ".concat(mmUri), IStatus.WARNING);
						}
					}

					// Third case : the meta model is not registered but a subModel is registered
					// The subModel clause is on the second line
					// Register the parent model of the submodel in the registry
					if (!findMetaModel)
					{
						// Read the SubModel clause
						String secondLine = reader.readLine();
						if (secondLine.length() > 0)
						{
							String subUri = secondLine.substring("SubModel :".length()).trim();
							// Get the root package of the submodel
							String hightestURI = OCLRuleUtil.getESuperPackage(subUri);
							// Try to find the root model in the registry
							findMetaModel = findInEMF(hightestURI);
							if (!findMetaModel)
							{
								CorePlugin.log("Parent Model not found in Registry : ".concat(hightestURI), IStatus.WARNING);
								// try to find the root model in the ecore files
								try
								{
									findMetaModel = findAsEcoreFile(URI.createURI(hightestURI).path());
									if (findMetaModel)
									{
										CorePlugin.log("Model found at given location from : ".concat(subUri), IStatus.OK);
									}
									else
									{
										CorePlugin.log("Model not found from given location : ".concat(subUri), IStatus.WARNING);
									}
								}
								catch (Exception e)
								{
									CorePlugin.log("Parent Model not found at given location : ".concat(hightestURI), IStatus.WARNING);
								}
							}
							else
							{
								CorePlugin.log("Model found in the registry from : ".concat(subUri), IStatus.OK);
							}
						}
					}

					if (metamodelPackage != null)
					{
						((OCLEditorSourceViewerConfiguration) getSourceViewerConfiguration()).setContentProcessorPackage(metamodelPackage);
						CorePlugin.log("The user metamodel was found", IStatus.OK);
					}
					else
					{
						CorePlugin.log("The user metamodel could not be found", IStatus.ERROR);
					}
				}
				else
				{
					CorePlugin.log("The file is empty. No model can be loaded.", IStatus.WARNING);
				}
			}
			catch (Exception e)
			{
				CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
			}
		}
	}

	/**
	 * Intents to load the OCL meta-model
	 */
	private void initOCLMetaModel()
	{
		EPackage packageOCL = MetaModelOCL.getOclPackage();
		if (packageOCL != null)
		{
			oclTypesPackage = packageOCL;
			OCLEditorSourceViewerConfiguration svc = ((OCLEditorSourceViewerConfiguration) getSourceViewerConfiguration());
			svc.setContentProcessorOclTypesPackage(oclTypesPackage);
			CorePlugin.log("The ocl metamodel was found", IStatus.OK);
		}
		else
		{
			CorePlugin.log("The ocl metamodel could not be found", IStatus.ERROR);
		}
	}

	public EClassifier getContextClassifier()
	{
		OCLEditorSourceViewerConfiguration svc = (OCLEditorSourceViewerConfiguration) getSourceViewerConfiguration();
		if (svc.getContentProcessor() != null)
		{
			return svc.getContentProcessor().computeContextClassifier(getSourceViewer().getDocument());
		}
		return null;
	}

	public EClassifier getContextClassifierForPartition(ITypedRegion partition)
	{
		OCLEditorSourceViewerConfiguration svc = (OCLEditorSourceViewerConfiguration) getSourceViewerConfiguration();
		if (svc.getContentProcessor() != null)
		{
			return svc.getContentProcessor().computeContextClassifierForPartition(getSourceViewer().getDocument(), partition);
		}
		return null;
	}

	/**
	 * Intents to locate the metamodel in the EMF register.
	 * 
	 * @param mmURI the URI to check
	 * @return a boolean indicating if the package was found in the register
	 */
	private boolean isRegisteredInEMF(String mmURI)
	{
		// Checks if the meta model is registered with EMF
		return (EPackage.Registry.INSTANCE.containsKey(mmURI));
	}

	/**
	 * Tries to find in the EMF register the high level package corresponding to the giver URI.
	 * 
	 * @param mmURI the URI to search and solve
	 * @return <code>true</code> if the URI has been solved, <code>false</code> otherwise.
	 */
	public boolean findInEMF(String mmURI)
	{
		if (isRegisteredInEMF(mmURI))
		{
			metamodelPackage = EPackage.Registry.INSTANCE.getEPackage(mmURI);
			return true;
		}
		return false;
	}

	/**
	 * 
	 * @param mmURI
	 * @return
	 */
	public boolean findAsEcoreFile(String mmURI)
	{
		IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
		HashMap<String, Object> readParameters = new HashMap<String, Object>();
		readParameters.put("XMIFile", mmURI);
		EObject model = (EObject) readService.serviceRun(readParameters);
		if (model != null)
		{
			mmURI = ((EPackage) model).getNsURI();
			OCLRuleUtil.registerMetaModel(model);
		}
		return findInEMF(mmURI);
	}
}
