package org.topcased.editors.ocleditor.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.ETypedElement;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.ocl.EnvironmentFactory;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.CallOperationAction;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.SendSignalAction;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.ocl.helper.OCLHelper;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.editors.ocleditor.assistant.OCLContentScanner;
import org.topcased.editors.ocleditor.editor.OCLEditorDocumentPartitioner;

/**
 * A class with useful methods
 * 
 * @author <a href="mailto:christophe.le-camus@c-s.fr">Christophe LE CAMUS</a>
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class OCLRuleUtil
{

	/**
	 * Create the typed constraint for the identified region
	 * 
	 * @param region : the identified region
	 * @param oclDocument : the document partitionned in region
	 * @param oclHelper : the helper which has the context set
	 * @return : the constraint Rule, Pre, Post ... Def
	 */
	public static Constraint createConstraint(ITypedRegion region, Document oclDocument,
	        OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper) throws ParserException
	{
		Constraint constraint = null;

		String ruleText = new String("");
		try
		{
			ruleText = oclDocument.get(region.getOffset(), region.getLength());
		}
		catch (BadLocationException e1)
		{
			CorePlugin.log("Empty Rule.", IStatus.ERROR);
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.RULE_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(true);
				OCLExpression<EClassifier> oclExpression = oclHelper.createQuery(ruleText);

				// the expression can be an non-boolean expression, let's test it before going on
				if (!"Integer".equals(oclExpression.getType().getName()))
				{
					constraint = oclHelper.createInvariant(ruleText);
				}
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in Invariant : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.PRE_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(true);
				constraint = oclHelper.createPrecondition(ruleText);
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in Precondition : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.POST_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(false);
				constraint = oclHelper.createPostcondition(ruleText);
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in Postcondition : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.BODY_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(true);
				constraint = oclHelper.createBodyCondition(ruleText);
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in BodyExpression : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.INIT_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(true);
				constraint = oclHelper.createInitialValueExpression(ruleText);
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in InitialExpression : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		if (region.getType().equals(OCLEditorDocumentPartitioner.DERIVE_CONTENT))
		{
			ruleText = ruleText.substring(ruleText.indexOf(':') + 1);
			try
			{
				oclHelper.setValidating(true);
				constraint = oclHelper.createDerivedValueExpression(ruleText);
			}
			catch (ParserException e)
			{
				CorePlugin.log("Parser Exception in DerivationExpression : ".concat(ruleText), IStatus.ERROR);
				throw e;
			}
		}
		return constraint;
	}

	/**
	 * Reads, extracts and returns the list of rules found in the text file.
	 * 
	 * @param text
	 * @param model
	 * @return
	 */
	public static List<Query<EClassifier, EClass, EObject>> loadRulesFromText(String text, EObject model, EClassifier ruleType)
	{
		List<Query<EClassifier, EClass, EObject>> ruleList = new ArrayList<Query<EClassifier, EClass, EObject>>();
		URI modelURI = null;
		try
		{
			Document oclDocument = new Document(text);
			// If the document has less than 2 lines, then it cannot contain any rule
			// Only documents with rules are interesting here.
			if (oclDocument.getNumberOfLines() > 1)
			{
				String firstLine = oclDocument.get(0, oclDocument.getLineLength(0));
				// If the first line does not begin with "MainModel :", then it cannot be an ocl rule file
				if (firstLine.startsWith("MainModel :"))
				{
					String rootModelURI = firstLine.substring("MainModel :".length()).trim();
					URI rootURI = URI.createURI(rootModelURI);
					EPackage modelPackage = OCLRuleUtil.findMetamodelPackage(rootURI);

					EnvironmentFactory<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> env = EcoreEnvironmentFactory.INSTANCE
					        .createInstanceContext(EcoreEnvironmentFactory.INSTANCE.createEnvironment(), modelPackage).getFactory();
					OCL<EPackage, EClassifier, EOperation, EStructuralFeature, EEnumLiteral, EParameter, EObject, CallOperationAction, SendSignalAction, Constraint, EClass, EObject> ocl = OCL
					        .newInstance(env);
					OCLHelper<EClassifier, EOperation, EStructuralFeature, Constraint> oclHelper = ocl.createOCLHelper();

					modelPackage = (EPackage) EPackage.Registry.INSTANCE.get(modelPackage.getNsURI());
					EClass context = null;
					EClassifier contextClassifier = null;

					if (modelPackage != null)
					{
						OCLEditorDocumentPartitioner partitionner = new OCLEditorDocumentPartitioner();
						partitionner.connect(oclDocument);
						ITypedRegion[] regions = partitionner.computePartitioning(0, oclDocument.getLength());
						ITypedRegion packageRegion = regions[0];
						ITypedRegion modelRegion = regions[0];

						for (int j = 0; j < regions.length; j++)
						{
							if (!regions[j].getType().equals(IDocument.DEFAULT_CONTENT_TYPE))
							{
								if (context != null
								        && (contextClassifier.getEPackage().getNsURI().equals(model.eClass().getEPackage().getNsURI())))
								{
									try
									{
										Constraint newConstraint = OCLRuleUtil.createConstraint(regions[j], oclDocument, oclHelper);
										if (newConstraint != null)
										{
											newConstraint.getConstrainedElements().add(context);
											Query<EClassifier, EClass, EObject> query = ocl.createQuery(newConstraint);
											if (query.resultType() == ruleType)
											{
												ruleList.add(query);
											}
											else
											{
												String expected = ruleType.getName();
												if (expected.lastIndexOf('.') > 0 && expected.lastIndexOf('.') < expected.length() - 1)
												{
													expected = expected.substring(expected.lastIndexOf('.') + 1);
												}
												CorePlugin.log(OCLRuleUtil.class, "Result type expected : " + expected
												        + ". Result type found : " + query.resultType().getName(), IStatus.WARNING);
												CorePlugin.log("This rule was not loaded : " + query.queryText(), IStatus.WARNING);
											}
										}
									}

									catch (ParserException e)
									{
										CorePlugin.log("ParserException : " + e.getMessage(), IStatus.ERROR);
									}
									catch (IllegalArgumentException e)
									{
										CorePlugin.log("Illegal Argument Exception : " + e.getMessage(), IStatus.ERROR);
									}
								}
							}
							else
							{
								String textTemp = oclDocument.get(regions[j].getOffset(), regions[j].getLength());
								if (!textTemp.matches("\\s*"))
								{
									// if the text is not empty
									// try to find a context
									if (textTemp.startsWith("package"))
									{
										String modelURIString = textTemp.substring("Package".length()).trim();
										modelURIString = modelURIString.replaceAll("::", "/");
										modelURI = URI.createURI(rootModelURI);
										if (modelURIString != "")
										{
											if (modelURI.isFile())
											{
												modelURI = modelURI.appendFragment(modelURIString);
											}
											else
											{
												modelURI = modelURI.appendSegment(modelURIString);
											}
										}
										// modelURI = modelURI.appendFragment(modelURIString);
										modelPackage = OCLRuleUtil.findMetamodelPackage(modelURI);
										packageRegion = regions[j];
										oclHelper.setInstanceContext(modelPackage);
									}
									if (textTemp.startsWith("endpackage"))
									{
										modelPackage = OCLRuleUtil.findMetamodelPackage(rootURI);
										packageRegion = modelRegion;
										oclHelper.setInstanceContext(modelPackage);
									}
									if (textTemp.startsWith("context"))
									{
										contextClassifier = OCLRuleUtil.computeContextClassifier(oclDocument, modelPackage, regions[j],
										        packageRegion);
										if (contextClassifier != null && contextClassifier instanceof EClass)
										{
											context = (EClass) contextClassifier;
											oclHelper.setContext(context);
											ETypedElement typedContext = OCLRuleUtil.computeTypedContext(oclDocument, modelPackage,
											        regions[j], packageRegion);
											if (typedContext != null)
											{
												if (typedContext instanceof EOperation)
												{
													oclHelper.setOperationContext(context, (EOperation) typedContext);
												}
												else
												{
													oclHelper.setInstanceAttributeContext(context, (EStructuralFeature) typedContext);
												}
											}
										}
										else
										{
											context = null;
											oclHelper.setContext(context);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		catch (BadLocationException e)
		{
			CorePlugin.log(OCLRuleUtil.class, "BadLocationException : " + e.getMessage(), IStatus.ERROR, e);
		}
		return ruleList;
	}

	/**
	 * Uses the read service to read a model.
	 * 
	 * @param file
	 * @return
	 */
	public static EObject loadModel(IFile file)
	{
		IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
		HashMap<String, Object> parameters = new HashMap<String, Object>();
		parameters.put("XMIFile", ((IFile) file).getLocation().toString());

		try
		{
			return (EObject) readService.serviceRun(parameters);
		}
		catch (Exception ex)
		{
			return null;
		}
	}

	/**
	 * Computes the context of the given region in the document
	 * 
	 * @param document
	 * @param modelPackage
	 * @param region
	 * @return
	 */
	public static EClassifier computeContextClassifier(IDocument document, EPackage modelPackage, ITypedRegion region,
	        ITypedRegion packageRegion)
	{
		OCLContentScanner oclScanner = new OCLContentScanner();
		oclScanner.setDocument(document);

		IToken tokenTemp = oclScanner.lookFor("package", packageRegion.getOffset(), packageRegion.getLength());
		tokenTemp = oclScanner.lookFor("context", region.getOffset(), region.getLength());
		if (tokenTemp != null)
		{
			tokenTemp = oclScanner.nextToken();
			String contextClass = "";
			if (tokenTemp != null)
			{
				contextClass = oclScanner.getTokenValue();
				return modelPackage.getEClassifier(contextClass);
			}
		}
		return null;
	}

	public static ETypedElement computeTypedContext(IDocument document, EPackage modelPackage, ITypedRegion region,
	        ITypedRegion packageRegion)
	{
		OCLContentScanner oclScanner = new OCLContentScanner();
		oclScanner.setDocument(document);

		IToken tokenTemp = oclScanner.lookFor("context", region.getOffset(), region.getLength());
		if (tokenTemp != null)
		{
			tokenTemp = oclScanner.nextToken();
			String contextClass = "";
			if (tokenTemp != null)
			{
				contextClass = oclScanner.getTokenValue();
				EClass context = (EClass) modelPackage.getEClassifier(contextClass);
				tokenTemp = oclScanner.nextToken();// oclScanner.getTokenValue()
				if (tokenTemp.getData() != null && tokenTemp.getData().equals(OCLContentScanner.DOUBLE_COLON))
				{
					tokenTemp = oclScanner.nextToken();
				}
				else
				{
					// The separator must be '::' else we cannot compute an operation context so return null
					return null;
				}
				// Search inside the operations List
				EList<EOperation> operationsList = context.getEAllOperations();
				boolean notFound = true;
				int i = 0;
				EOperation myOp = null;
				while (i < operationsList.size() && notFound)
				{
					if ((operationsList.get(i)).getName().equals(oclScanner.getTokenValue()))
					{
						myOp = operationsList.get(i);
					}
					i++;
				}
				if (myOp != null)
				{
					return myOp;
				}
				// Search inside the operations List
				EList<EStructuralFeature> structuralFeaturesList = context.getEAllStructuralFeatures();
				notFound = true;
				i = 0;
				EStructuralFeature myFeature = null;
				while (i < structuralFeaturesList.size() && notFound)
				{
					if ((structuralFeaturesList.get(i)).getName().equals(oclScanner.getTokenValue()))
					{
						myFeature = structuralFeaturesList.get(i);
					}
					i++;
				}
				return myFeature;
			}
		}
		return null;
	}

	public static EPackage findMetamodelPackage(URI mmUri)
	{
		try
		{
			EPackage returnPackage = null;

			URI fileUri = URI.createURI(mmUri.path());
			EPackage tempPackage = null;
			if (mmUri.toString().indexOf("#") != -1)
			{
				tempPackage = EPackage.Registry.INSTANCE.getEPackage(mmUri.toString().substring(0, mmUri.toString().indexOf("#")));
			}
			else
			{
				tempPackage = EPackage.Registry.INSTANCE.getEPackage(mmUri.toString());
			}

			if (tempPackage != null)
			{
				returnPackage = tempPackage;
				return returnPackage;
			}

			if (returnPackage == null)
			{
				IPath pathFile = new Path(fileUri.toString());
				IFile modelFile = ResourcesPlugin.getWorkspace().getRoot().getFile(pathFile);
				if (modelFile.exists())
				{
					IService readService = ServicesManager.getInstance().getService("org.topcased.service.readxmiemf.read");
					HashMap<String, Object> readParameters = new HashMap<String, Object>();
					readParameters.put("XMIFile", fileUri.toString());
					try
					{
						EObject model = (EObject) readService.serviceRun(readParameters);
						tempPackage = (EPackage) model;
						registerMetaModel(model);

						if (mmUri.fragment() != null)
						{
							List<String> packagePrefixes = new ArrayList<String>();
							String uriFragment = mmUri.fragment();
							String[] fragments = uriFragment.split("/", uriFragment.length());
							// the package list is composed of subpackages of package above
							// the first is the high level package
							// the last is the sub x level package
							for (int i = 0; i < fragments.length; i++)
							{
								packagePrefixes.clear();
								packagePrefixes.add(fragments[i]);
								tempPackage = OCLRuleUtil.findSubPackage(tempPackage, packagePrefixes);
							}
						}
						else
						{
							tempPackage = (EPackage) EPackage.Registry.INSTANCE.get(tempPackage.getNsURI());
						}
						if (tempPackage != null)
						{
							returnPackage = tempPackage;
						}

					}
					catch (WrappedException e)
					{
						// If the read service cannot read the file.
						CorePlugin.log("Can't read file : " + modelFile + ". This is not a model.", IStatus.ERROR);
					}
				}
			}

			if (returnPackage == null && EPackage.Registry.INSTANCE.getEPackage(mmUri.toString()) != null)
			{
				returnPackage = EPackage.Registry.INSTANCE.getEPackage(mmUri.toString());
				return returnPackage;
			}

			if (returnPackage == null)
			{
				IService readService = ServicesManager.getInstance().getService("org.topcased.service.readxmiemf.read");
				HashMap<String, Object> readParameters = new HashMap<String, Object>();
				readParameters.put("XMIFile", fileUri.toString());
				try
				{
					EObject model = (EObject) readService.serviceRun(readParameters);
					if (model != null)
					{
						returnPackage = (EPackage) model;

						tempPackage = EPackage.Registry.INSTANCE.getEPackage(fileUri.toString());
						if (tempPackage != null)
						{
							returnPackage = tempPackage;
						}
					}
				}
				catch (WrappedException e)
				{
					CorePlugin.log("The base model cannot be found with path : ".concat(fileUri.toString()), IStatus.ERROR);
					// If the read service cannot read the file. (File not found or not a model)
				}
			}

			return returnPackage;
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/**
	 * Register the meta model if it has not been registered before. This is necessary to take into account local meta meta models that have not been prepared
	 * as new plugin.
	 * 
	 * @param model
	 */
	public static void registerMetaModel(EObject model)
	{

		if (model instanceof EPackage)
		{

			EPackage modelPackage = (EPackage) model;
			List<EPackage> packages = new ArrayList<EPackage>();
			packages.addAll(modelPackage.getESubpackages());
			packages.add(0, modelPackage);
			for (Iterator<EPackage> it = packages.iterator(); it.hasNext();)
			{
				EPackage p = it.next();
				String nsURI = p.getNsURI();
				if (nsURI == null)
				{
					nsURI = p.getName();
					p.setNsURI(nsURI);
				}
				if (!EPackage.Registry.INSTANCE.containsKey(nsURI))
				{
					EPackage.Registry.INSTANCE.put(nsURI, p);
					CorePlugin.log("Model loaded : ".concat(nsURI), IStatus.INFO);
				}
			}
		}
	}

	/**
	 * Gets the highest package from a meta model
	 * 
	 * @param mmUri
	 * @return URI from the container package at the highest level
	 */
	public static String getESuperPackage(String mmUri)
	{

		EPackage packageTemp = EPackage.Registry.INSTANCE.getEPackage(mmUri);
		if (packageTemp == null)
		{
			IService readService = ServicesManager.getInstance().getService("org.topcased.service.readxmiemf.read");
			HashMap<String, Object> readParameters = new HashMap<String, Object>();
			readParameters.put("XMIFile", mmUri);

			EObject model = (EObject) readService.serviceRun(readParameters);
			if (model != null)
				packageTemp = ((EPackage) model);
		}
		// Calling register on superPackage
		if (packageTemp != null)
		{
			EPackage eSuperPackage = packageTemp.getESuperPackage();
			if (eSuperPackage != null)
			{
				while (eSuperPackage != null && eSuperPackage.getESuperPackage() != null)
				{
					eSuperPackage = eSuperPackage.getESuperPackage();
					// if the current package is not known, we register it
					registerMetaModel(eSuperPackage);
				}
				return eSuperPackage.getNsURI();
			}
			else
			{
				return packageTemp.getNsURI();
			}
		}
		return "";
	}

	/**
	 * 
	 * @param currentPackage
	 * @param packagePrefixes
	 * @return
	 */
	public static EPackage findSubPackage(EPackage currentPackage, List<String> packagePrefixes)
	{

		EPackage tempPackage = currentPackage;

		if (currentPackage.getName() != null)
		{
			boolean packageError = false;
			for (int i = 0; i < packagePrefixes.size() && !packageError; i++)
			{
				List<EPackage> subPackages = currentPackage.getESubpackages();
				boolean packageFound = false;
				for (int j = 0; j < subPackages.size() && !packageFound; j++)
				{
					EPackage subi = subPackages.get(j);
					// change getPrefix by getName()
					if (subi.getName() != null)
					{
						if (subi.getName().equals(packagePrefixes.get(i)))
						{
							packageFound = true;
							tempPackage = subi;
						}
					}
					else
					{
						CorePlugin.log("Sub package name null. Check the meta model.", IStatus.ERROR);
					}
				}
				if (!packageFound)
				{
					packageError = true;
				}
			}
		}
		else
		{
			CorePlugin.log("Metamodel package name null.: Check the meta model.", IStatus.ERROR);
		}
		return tempPackage;
	}

	/**
	 * Tries to resolve all resources
	 * 
	 * @param model
	 * @return
	 */
	public static EList<Resource> registerResolvedResources(EObject model)
	{
		EcoreUtil.resolveAll(model);
		EList<Resource> allResources = null;
		if (model.eResource().getResourceSet() != null)
		{
			allResources = model.eResource().getResourceSet().getResources();
			for (Iterator<Resource> it = allResources.iterator(); it.hasNext();)
			{
				Resource aResrc = (Resource) it.next();
				EList<EObject> allPackages = aResrc.getContents();
				for (Iterator<EObject> itP = allPackages.iterator(); itP.hasNext();)
				{
					EPackage aPackage = (EPackage) itP.next();
					OCLRuleUtil.registerMetaModel(aPackage);
				}
			}
		}
		return allResources;
	}

	/**
	 * Gives the 'subModels' clause.
	 * 
	 * @return The string representation
	 */
	public static String subModelsClauses()
	{
		return new String("SubModel");
	}
}
