/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS)
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *********************************************************************************************/
package org.topcased.editors.ocleditor.wizard;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.topcased.bus.core.CorePlugin;
import org.topcased.editors.ocleditor.OCLEditorPlugin;
import org.topcased.editors.ocleditor.utils.OCLRuleUtil;
import org.topcased.facilities.dialogs.ITopcasedWizardConstants;

/**
 * Wizard enabling to create a new OCL rule file.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class NewRuleFileWizard extends Wizard implements INewWizard, ITopcasedWizardConstants
{
	/** Constant of the editor project nature */
	private static final String PROJECT_NATURE = "org.topcased.editors.ocleditor.builders.oclEditorProjectNature";
	
	/** The only page contributing to the wizard */
	private NewRuleFileWizardPage page;

	/**
	 * Constructor
	 */
	public NewRuleFileWizard()
	{
		super();
		
		setWindowTitle("TOPCASED : New Ocl Rule File Wizard");
		
		setDefaultPageImageDescriptor(OCLEditorPlugin.getImageDescriptor(TOPCASED_LOGO_PATH));
	}

	/**
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish()
	{
		String containerName = page.getContainerName();
		String fileName = page.getFileName();
		String selectedModel = page.getMetamodelUri();

		String metamodelUri = OCLRuleUtil.getESuperPackage(page.getMetamodelUri());
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (!resource.exists() || !(resource instanceof IContainer))
		{
			return false;
		}

		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));

		try
		{
			IProjectDescription description = resource.getProject().getDescription();
			List<String> newNatures = new ArrayList<String>(Arrays.asList(description.getNatureIds()));
			
			if (!newNatures.contains(PROJECT_NATURE))
			{
				newNatures.add(PROJECT_NATURE);
				description.setNatureIds((String[]) newNatures.toArray(new String[newNatures.size()]));
				resource.getProject().setDescription(description, null);
			}
			
			String line = new String("MainModel : ");
			// TODO find another solution for the end of line.
			// Try to check if the ResolveAll Resources cannot be used here.
			if (page.getMetaModelFromFile() != null)
			{
				line = line.concat(page.getMetaModelFromFile()).concat("\n");
				IPath pathFile = new Path(selectedModel);
				IFile modelFile = ResourcesPlugin.getWorkspace().getRoot().getFile(pathFile);
				EPackage packageTemp = (EPackage) OCLRuleUtil.loadModel(modelFile);
				OCLRuleUtil.registerResolvedResources(packageTemp);

			}
			else
			{
				// selected from the URI
				line = line.concat(metamodelUri);
				EPackage packageTemp = EPackage.Registry.INSTANCE.getEPackage(metamodelUri);
				OCLRuleUtil.registerResolvedResources(packageTemp);
				if (!selectedModel.equals(metamodelUri))
				{
					line = line.concat("\n").concat("SubModel : ").concat(selectedModel);
					packageTemp = EPackage.Registry.INSTANCE.getEPackage(selectedModel);
					OCLRuleUtil.registerResolvedResources(packageTemp);

				}
			}
			line = line + "\n\ncontext ";

			InputStream stream = new ByteArrayInputStream(line.getBytes());
			if (file.exists())
			{
				file.setContents(stream, true, true, null);
			}
			else
			{
				file.create(stream, true, null);
			}
			stream.close();
		}
		catch (Exception e)
		{
			CorePlugin.log(e);
			return false;
		}
		
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		try
		{
			IDE.openEditor(page, file, true);
		}
		catch (PartInitException e)
		{
			CorePlugin.log(e);
			return false;
		}

		return true;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection)
	{
		page = new NewRuleFileWizardPage(selection);
		addPage(page);
	}
}
