/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.export.bnftosablecc.services;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.service.transformation.Transformation;

/**
 * Main service class for importing a BNF grammar to a SableCC format.
 * 
 * @author <a href="mailto:christophe.le-camus@c-s.fr">Christophe Le Camus</a>
 */
public class BNFModelToSableCCService implements IService
{
	private static final String BNF_MM_URI = "http://www.topcased.org/BNF/1.0";
	
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{
		if (!parameters.containsKey("IN"))
		{
			CorePlugin.log("The parameter InFile must be set.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("OUT"))
		{
			CorePlugin.log("The parameter OutFile must be set.", IStatus.ERROR);
			return null;
		}

		if (!parameters.containsKey("Path"))
		{
			CorePlugin.log("The parameter WorkingDir must be set.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("IN") instanceof String))
		{
			CorePlugin.log("The parameter InFile must be a String.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("OUT") instanceof String))
		{
			CorePlugin.log("The parameter OutFile must be a String.", IStatus.ERROR);
			return null;
		}

		if (!(parameters.get("Path") instanceof IPath))
		{
			CorePlugin.log("The parameter WorkingDir must be a String.", IStatus.ERROR);
			return null;
		}

		String beginTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Entering BNF Model to SableCC service at ".concat(beginTime), IStatus.INFO);

		String inputFile = (String) parameters.get("IN");
		IPath workingDir = (IPath) parameters.get("Path");

		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();

		IService changeUriService = ServicesManager.getInstance().getService(IServiceCst.CHANGE_URI);
		
		Map<String, Object> serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("InFilePath", inputFile);
		serviceParameters.put("OutFilePath", workingDir.append("bnfmodified.tmp").toString());
		
		Map<String, String> nsuri = new HashMap<String, String>();
		nsuri.put("BNF", BNF_MM_URI);
		serviceParameters.put("NSURI", nsuri);
		changeUriService.serviceRun(serviceParameters);

		Map<String, String> models = new HashMap<String, String>();
		models.put("IN", root.getLocation().toOSString() + workingDir.append("bnfmodified.tmp").toString());
		models.put("OUT", "null");

		new Transformation("org.topcased.export.bnftosablecc.transformation", models, workingDir, false, null);

		File temp = new File(System.getProperty("user.dir") + File.separator + "grammar.sab");
		File target = root.getLocation().append(workingDir.append((String) parameters.get("OUT"))).toFile();
		File nullFile = root.getLocation().append(workingDir.append("null")).toFile();

		if (target.exists())
		{
			target.delete();
		}
		temp.renameTo(target);

		new File(root.getLocation().toOSString() + workingDir.append("bnfmodified.tmp")).delete();
		new File(System.getProperty("user.dir") + File.separator + "grammar.sab").delete();

		if (nullFile.exists())
		{
			nullFile.delete();
		}

		try
		{
			root.findMember(workingDir).refreshLocal(IResource.DEPTH_ONE, null);
		}
		catch (CoreException e)
		{
			CorePlugin.log(e);
		}

		String endTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Leaving BNF Model to SableCC service at ".concat(endTime), IStatus.INFO);

		return null;
	}
}
