/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.externalbus.core;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.custom.BusyIndicator;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.<br>
 * "org.topcased.externalbus.core" allows to access Topcased services from external server.<br>
 * Define extension point ExternalServices and the interface IExternalService.
 */
public class CorePlugin extends Plugin
{
	// The shared instance.
	private static CorePlugin plugin;

	// Resource bundle.
	private ResourceBundle resourceBundle;

	/**
	 * The constructor.
	 */
	public CorePlugin()
	{
		super();
		plugin = this;
		try
		{
			resourceBundle = ResourceBundle.getBundle("org.topcased.externalbus.core.CorePluginResources");
		}
		catch (MissingResourceException x)
		{
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void start(BundleContext context) throws Exception
	{
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void stop(BundleContext context) throws Exception
	{
		super.stop(context);
		// plugin = null;
		// resourceBundle = null;
	}

	/**
	 * @return the shared instance
	 */
	public static CorePlugin getDefault()
	{
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not found.
	 * 
	 * @param key
	 * @return plugin's resource bundle or key
	 */
	public static String getResourceString(String key)
	{
		ResourceBundle bundle = CorePlugin.getDefault().getResourceBundle();
		try
		{
			return (bundle != null) ? bundle.getString(key) : key;
		}
		catch (MissingResourceException e)
		{
			return key;
		}
	}

	/**
	 * @return the plugin's resource bundle
	 */
	public ResourceBundle getResourceBundle()
	{
		return resourceBundle;
	}

	/**
	 * Creates an extension. If the extension plugin has not been loaded a busy cursor will be activated during the duration of the load.
	 * 
	 * @param element the config element defining the extension
	 * @param classAttribute the name of the attribute carrying the class
	 * @return the extension object
	 * @exception CoreException
	 */
	public static Object createExtension(final IConfigurationElement element, final String classAttribute) throws CoreException
	{
		try
		{
			Bundle extensionBundle = Platform.getBundle(element.getDeclaringExtension().getNamespaceIdentifier());
			if (extensionBundle.getState() == Bundle.ACTIVE)
			{
				String clazzName = element.getAttribute(classAttribute);
				try
				{
					Class<?> clazz = extensionBundle.loadClass(clazzName);
					Class<?> paramsClass[] = {};
					Constructor<?> constructor = clazz.getConstructor(paramsClass);

					Object paramsObj[] = {};
					return constructor.newInstance(paramsObj);

				}
				catch (NoSuchMethodException e)
				{
					log("Unable to load class " + clazzName, IStatus.ERROR);
					log(e);
				}
				catch (Exception e)
				{
					log("Unable to load class " + clazzName, IStatus.ERROR);
					log(e);
				}
			}
			else
			{
				final Object[] ret = new Object[1];
				final Exception[] exc = new Exception[1];
				BusyIndicator.showWhile(null, new Runnable()
				{
					public void run()
					{
						Bundle bundle = Platform.getBundle(element.getDeclaringExtension().getNamespaceIdentifier());
						String clazzName = element.getAttribute(classAttribute);
						try
						{
							Class<?> clazz = bundle.loadClass(clazzName);
							Class<?> paramsClass[] = {};
							Constructor<?> constructor = clazz.getConstructor(paramsClass);

							Object paramsObj[] = {};
							ret[0] = constructor.newInstance(paramsObj);

						}
						catch (NoSuchMethodException e)
						{
							exc[0] = e;
						}
						catch (Exception e)
						{
							exc[0] = e;
						}
					}
				});
				if (exc[0] != null)
					throw exc[0];
				else
					return ret[0];
			}

			return null;
		}
		catch (CoreException core)
		{
			throw core;
		}
		catch (Exception e)
		{
			throw new CoreException(new Status(IStatus.ERROR, getId(), IStatus.ERROR, "Unable to create extension", e));
		}
	}

	/**
	 * @return the Plugin Id
	 */
	public static String getId()
	{
		return getDefault().getBundle().getSymbolicName();
	}

	/**
	 * Logs an IStatus
	 * 
	 * @param status
	 */
	public static void log(IStatus status)
	{
		ResourcesPlugin.getPlugin().getLog().log(status);
	}

	/**
	 * Logs a message with given level into the Eclipse log file
	 * 
	 * @param message : the message to log
	 * @param level : the message priority
	 */
	public static void log(String message, int level)
	{
		IStatus status = new Status(level, getId(), level, message, null);

		log(status);
	}

	/**
	 * Logs a message with given level into the Eclipse log file and with given name class which call it
	 * 
	 * @param serviceClass calling => you must write : this.getClass()
	 * @param message : the message to log
	 * @param level : the message priority (IStatus.INFO or .ERROR or .WARNING ...)
	 * 
	 */
	public static void log(Class<?> serviceClass, String message, int level)
	{
		IStatus status = new Status(level, serviceClass.getName(), level, message, null);

		log(status);
	}
	
	/**
	 * Logs a message with given level into the Eclipse log file and with given name class which call it
	 * 
	 * @param serviceClass calling => you must write : this.getClass()
	 * @param message : the message to log
	 * @param level : the message priority (IStatus.INFO or IStatus.ERROR or IStatus.WARNING ...)
	 * @param e : exception stack
	 */
	public static void log(Class<?> serviceClass, String message, int level, Exception e)
	{
		IStatus status = new Status(level, serviceClass.getName(), level, message, e);
		log(status);
	}

	/**
	 * Log an exception into the Eclipse log file
	 * 
	 * @param e : the exception to log
	 */
	public static void log(Throwable e)
	{
		if (e instanceof InvocationTargetException)
			e = ((InvocationTargetException) e).getTargetException();

		IStatus status = null;
		if (e instanceof CoreException)
			status = ((CoreException) e).getStatus();
		else
			status = new Status(IStatus.ERROR, getId(), IStatus.OK, "Error", e);

		log(status);
	}

}
