/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.externalbus.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.w3c.dom.Document;

/**
 * Manage ExternalServices extension point <br>
 */
public final class ExternalServicesManager
{
	/** Extension point name */
	public static final String EXTERNALSERVICES_EXTENSION_POINT = "ExternalServices";

	// Service arguments
	/** TAG : externalService */
	public static final String TAG_EXTERNALSERVICE = "externalService";

	/** Attribut of externalService : id */
	public static final String ATT_ID_EXTERNALSERVICE = "id";

	/** Attribut of externalService : name */
	public static final String ATT_NAME_EXTERNALSERVICE = "name";

	/** Attribut of externalService : class */
	public static final String ATT_CLASS_EXTERNALSERVICE = "class";

	// Type arguments
	/** TAG : ontologyType */
	public static final String TAG_TYPE = "ontologyType";

	/** Attribut of ontologyType : id */
	public static final String ATT_ID_TYPE = "id";

	/** Attribut of ontologyType : name */
	public static final String ATT_NAME_TYPE = "name";

	/** Attribut of ontologyType : parentId */
	public static final String ATT_PARENTID_TYPE = "parentId";

	/** Attribut of ontologyType : category */
	public static final String ATT_CATEGORY_TYPE = "category";

	// ExternalParameter arguments
	/** TAG : parameter */
	public static final String TAG_PARAM = "parameter";

	/** Attribut of parameter : id */
	public static final String ATT_ID_PARAM = "id";

	/** Attribut of parameter : name */
	public static final String ATT_NAME_PARAM = "name";

	/** Attribut of parameter : description */
	public static final String ATT_DESCRIPTION_PARAM = "description";

	/** Attribut of parameter : kind */
	public static final String ATT_KIND_PARAM = "kind";

	/** Attribut of parameter : version */
	public static final String ATT_VERSION_PARAM = "version";

	/** List of input externalParameters */
	public static final String LIST_INPUT_PARAM = "listInputParam";

	/** Output parameter */
	public static final String OUTPUT_PARAM = "outputParam";
	
	private static ExternalServicesManager manager = new ExternalServicesManager();

	private Map<String, ExternalService> services; // key = IdService, value = ExternalService

	private Map<String, ExternalParameter> externalParameters; // key = IdService, value = ExternalParameter

	private Map<String, ExternalParameter> parametersIn; // key = IdService, value = ExternalParameter

	private Map<String, ExternalParameter> parametersOut; // key = IdService, value = ExternalParameter



	/**
	 * Constructor ExternalServicesManager : Instances of "services" and "types" are done
	 */
	private ExternalServicesManager()
	{
		services = new HashMap<String, ExternalService>();
		externalParameters = new HashMap<String, ExternalParameter>();
		parametersIn = new HashMap<String, ExternalParameter>();
		parametersOut = new HashMap<String, ExternalParameter>();
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the singleton
	 */
	public static ExternalServicesManager getInstance()
	{
		return manager;
	}

	/**
	 * Initialization of private Map "services" and "types" if it is not done yet RETRIEVE CHAIN service |_type |_outputParam |_Param |_inputParam |_Param
	 */
	private void initialize()
	{
		if (services.isEmpty())
		{
			retrieveServices();
		}
	}

	/**
	 * Clears the private Map "services"
	 */
	public void setEmptyServices()
	{
		services.clear();
	}

	/**
	 * Clear the private Map "externalParameters"
	 */
	public void setEmptyParameters()
	{
		externalParameters.clear();
	}

	/**
	 * Search all plugins which use SERVICES extention point with tag SERVICE and TYPE. Fill private Map "services" and "types"
	 */
	private void retrieveServices()
	{
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(CorePlugin.getDefault().getBundle().getSymbolicName(),
		        EXTERNALSERVICES_EXTENSION_POINT);

		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length; i++)
		{
			IConfigurationElement[] elements = extensions[i].getConfigurationElements();
			for (int j = 0; j < elements.length; j++)
			{
				ExternalService thisService = null;
				if (elements[j].getName().compareTo(TAG_EXTERNALSERVICE) == 0 && elements[j].getAttribute(ATT_ID_EXTERNALSERVICE) != null
				        && elements[j].getAttribute(ATT_NAME_EXTERNALSERVICE) != null
				        && elements[j].getAttribute(ATT_CLASS_EXTERNALSERVICE) != null)
				{
					try
					{
						IExternalService serviceClass = (IExternalService) elements[j].createExecutableExtension(ATT_CLASS_EXTERNALSERVICE);
						thisService = new ExternalService(elements[j].getAttribute(ATT_ID_EXTERNALSERVICE), elements[j]
						        .getAttribute(ATT_NAME_EXTERNALSERVICE), elements[j].getAttribute(ATT_CLASS_EXTERNALSERVICE), serviceClass);
						services.put(elements[j].getAttribute(ATT_ID_EXTERNALSERVICE), thisService);
					}
					catch (CoreException e)
					{
						CorePlugin.log(e);
						CorePlugin.log(this.getClass(), "Attribut class of the extension point Services is not a IService.",
						        IStatus.WARNING);
					}

				}
				retrieveOutputParam(elements[j], thisService);
				retrieveInputParam(elements[j], thisService);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag OUTPUTPARAM. Fill the parameter "this Service"
	 * 
	 * @param serviceElement : an element of the private MAP "services"
	 * @param thisService : Service
	 */
	private void retrieveOutputParam(IConfigurationElement serviceElement, ExternalService thisService)
	{
		IConfigurationElement[] elementsParam = serviceElement.getChildren(OUTPUT_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			if (elementsParam[j].getName().compareTo(OUTPUT_PARAM) == 0)
			{
				retrieveParameters(elementsParam[j], thisService, false);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag INPUTPARAM. Fill the parameter "this Service"
	 * 
	 * @param serviceElement : an element of the private MAP "services"
	 * @param thisService : Service
	 */
	private void retrieveInputParam(IConfigurationElement serviceElement, ExternalService thisService)
	{
		IConfigurationElement[] elementsParam = serviceElement.getChildren(LIST_INPUT_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			if (elementsParam[j].getName().compareTo(LIST_INPUT_PARAM) == 0)
			{
				retrieveParameters(elementsParam[j], thisService, true);
			}
		}
	}

	/**
	 * Search all plugins which use SERVICES extention point whith tag PARAMETER. Fill private Map "externalParameters"
	 * 
	 * @param paramElement : an element of the private MAP "services"
	 * @param thisService : Service
	 * @param in : "true" if the prameter is a input and "false" if the parameter is an output
	 */
	private void retrieveParameters(IConfigurationElement paramElement, ExternalService thisService, boolean in)
	{
		IConfigurationElement[] elementsParam = paramElement.getChildren(TAG_PARAM);

		for (int j = 0; j < elementsParam.length; j++)
		{
			ExternalParameter thisParameter;
			if (elementsParam[j].getName().equals(TAG_PARAM) && elementsParam[j].getAttribute(ATT_ID_PARAM) != null
			        && elementsParam[j].getAttribute(ATT_NAME_PARAM) != null && elementsParam[j].getAttribute(ATT_KIND_PARAM) != null)
			{
				thisParameter = new ExternalParameter(elementsParam[j].getAttribute(ATT_ID_PARAM), elementsParam[j].getAttribute(ATT_NAME_PARAM),
				        elementsParam[j].getAttribute(ATT_DESCRIPTION_PARAM), elementsParam[j].getAttribute(ATT_KIND_PARAM));
				externalParameters.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
				if (in)
				{
					parametersIn.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
					thisService.setParamInId(elementsParam[j].getAttribute(ATT_ID_PARAM));
				}
				if (!in)
				{
					parametersOut.put(elementsParam[j].getAttribute(ATT_ID_PARAM), thisParameter);
					thisService.setParamOutId(elementsParam[j].getAttribute(ATT_ID_PARAM));
				}
			}
		}
	}

	// **************************************** GET Complex from Id ***************************************

	/**
	 * Get ExternalService from externalServiceId
	 * 
	 * @param externalServiceId
	 * @return ExternalService
	 */
	public ExternalService getExternalService(String externalServiceId)
	{
		initialize();
		ExternalService extService = null;
		if (services != null)
		{
			if (services.containsKey(externalServiceId))
			{
				if (services.get(externalServiceId) instanceof ExternalService)
				{
					extService = (ExternalService) services.get(externalServiceId);
				}
			}
		}
		return extService;
	}

	/**
	 * Get ExternalParameter from parameterId
	 * 
	 * @param parameterId
	 * @return ExternalParameter
	 */
	public ExternalParameter getParameter(String parameterId)
	{
		initialize();
		ExternalParameter param = null;
		if (externalParameters != null)
		{
			if (externalParameters.containsKey(parameterId))
			{
				if (externalParameters.get(parameterId) instanceof ExternalParameter)
				{
					param = (ExternalParameter) externalParameters.get(parameterId);
				}
			}
		}
		return param;
	}

	// ************************************ GET XML********************************************************

	/**
	 * Get document Dom (ExternalService Serialisable) from externalServiceId
	 * 
	 * @param externalServiceId
	 * @return Document DOM
	 */
	public Document getSerialisableExternalService(String externalServiceId)
	{
		initialize();
		SerializableExternalService serial = new SerializableExternalService(externalServiceId);
		Document docExternalService = serial.externalServiceSerializated();
		return docExternalService;

	}

	/**
	 * Get a document DOM with all informations on all externalServices
	 * 
	 * @return document
	 */
	public Document getExternalService()
	{
		initialize();
		SerializableExternalService serial = new SerializableExternalService(getExternalServiceIds());
		Document docExternalService = serial.externalServiceSerializated();
		return docExternalService;
	}

	// ************************************** GET ExternalServiceId ***************************************
	/**
	 * Get all External services Id
	 * 
	 * @return Set of external service Id
	 */
	public Set<String> getExternalServiceIds()
	{
		initialize();
		return services.keySet();
	}

	// With criteria TODO

	// ************************************** GET ExternalService class ***************************************
	/**
	 * Get externalService class which implements IExternalService This class allows to launch the external service.
	 * 
	 * @param serviceId
	 * @return class which implements IExternalService
	 */
	public IExternalService getExternalServiceClass(String serviceId)
	{
		initialize();
		IExternalService iExtService = null;
		if (services != null)
		{
			if (services.containsKey(serviceId))
			{
				if (services.get(serviceId) instanceof ExternalService)
				{
					ExternalService externService = (ExternalService) services.get(serviceId);
					iExtService = externService.getClazz();
				}
			}
		}
		return iExtService;
	}
}
