/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.generator.services.generation;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.codegen.jet.JETEmitter;
import org.eclipse.emf.codegen.jet.JETException;
import org.topcased.generator.services.TopcasedBusServicesGeneratorPlugin;

/**
 * This class generates plug-in files using the JET Templates.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * 
 */
public class TopcasedBusServicesGenerator
{
	/**
	 * Launches the Topcased plugin generation process
	 * 
	 * @param monitor A Reference to the task monitor
	 * @throws CoreException
	 * @throws Exception
	 */
	public static void generateTopcasedPlugin(Map<String, String> parameters, IProject project) throws CoreException, Exception
	{
		// Prepare the path of the resource to generate
		String projectName = project.getName().toLowerCase();
		IPath srcPath = project.getFullPath().append(TopcasedBusServicesConstants.SOURCE_DIRECTORY);
		srcPath = srcPath.append(projectName.replace('.', IPath.SEPARATOR));

		// Creation of the empty structure of a standard Topcased Plugin.
		generateDirectory(srcPath.append(TopcasedBusServicesConstants.ACTION_DIRECTORY));
		generateDirectory(srcPath.append(TopcasedBusServicesConstants.DIALOG_DIRECTORY));
		generateDirectory(srcPath.append(TopcasedBusServicesConstants.SERVICE_DIRECTORY));
		generateDirectory(srcPath.append(TopcasedBusServicesConstants.PREFERENCE_DIRECTORY));

		// JETEmitter calls to generate classes
		// Creates Topcased Activator class
		IPath outputPluginClass = srcPath.append(TopcasedBusServicesConstants.PLUGIN_FILE);
		generateFile(outputPluginClass, TopcasedBusServicesConstants.PLUGIN_TEMPLATE_PATH, new Object[] { projectName });
		
		// Creates Topcased Dialog class
		IPath outputDialogClass = srcPath.append(TopcasedBusServicesConstants.DIALOG_DIRECTORY).append(
				TopcasedBusServicesConstants.DIALOG_FILE);
		generateFile(outputDialogClass, TopcasedBusServicesConstants.DIALOG_TEMPLATE_PATH, new Object[] { projectName });
		
		// Creates Topcased Dialog Constant class
		IPath outputDialogCstClass = srcPath.append(TopcasedBusServicesConstants.DIALOG_DIRECTORY).append(
				TopcasedBusServicesConstants.DIALOG_CST_FILE);
		generateFile(outputDialogCstClass, TopcasedBusServicesConstants.DIALOG_CST_TEMPLATE_PATH, new Object[] { projectName });
		
		// Creates Topcased Service class
		IPath outputServiceClass = srcPath.append(TopcasedBusServicesConstants.SERVICE_DIRECTORY).append(
				TopcasedBusServicesConstants.SERVICE_FILE);
		generateFile(outputServiceClass, TopcasedBusServicesConstants.SERVICE_TEMPLATE_PATH, new Object[] { projectName });

		// Creates Topcased Action class
		IPath outputActionClass = srcPath.append(TopcasedBusServicesConstants.ACTION_DIRECTORY).append(
				TopcasedBusServicesConstants.ACTION_FILE);
		generateFile(outputActionClass, TopcasedBusServicesConstants.ACTION_TEMPLATE_PATH, new Object[] { parameters });

		// Creates Topcased Preferences class
		IPath outputPreferenceClass = srcPath.append(TopcasedBusServicesConstants.PREFERENCE_DIRECTORY).append(
				TopcasedBusServicesConstants.PREFERENCE_FILE);
		generateFile(outputPreferenceClass, TopcasedBusServicesConstants.PREFERENCE_TEMPLATE_PATH, new Object[] { parameters });
	}

	/**
	 * Gets a folder from the current workspace
	 * 
	 * @param path Represents the path of the resource to find.
	 * @return the IFolder representing the current directory.
	 */
	private static IFolder getFolder(IPath path)
	{
		return ResourcesPlugin.getWorkspace().getRoot().getFolder(path);
	}

	/**
	 * Generates a directory into the new created project.
	 * 
	 * @param projectPath the path of the folder to create
	 * @throws JETException
	 * @throws CoreException
	 */
	private static void generateDirectory(IPath projectPath) throws JETException, CoreException
	{
		IFolder folder = getFolder(projectPath);
		if (!folder.exists())
		{
			folder.create(true, true, null);
		}
	}

	/**
	 * This method applies JET templates. It generates a given resource of the new plug-in.
	 * 
	 * @param templateURI a String representation of the template URI to apply
	 * @param outputFile the Path where the output file must be created.
	 * @throws CoreException
	 * @throws JETException
	 */
	private static void generateFile(IPath outputFile, String templateURI, Object[] parameters) throws JETException, CoreException
	{
		String templatePath = getTemplateURI(templateURI);
		JETEmitter emitter = new JETEmitter(templatePath);
		IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(outputFile);
		if (!file.exists())
		{
			file.create(new ByteArrayInputStream("empty".getBytes()), false, null);
		}
		String generated = emitter.generate(new NullProgressMonitor(), parameters);
		saveGenerated(generated, file);
	}

	/**
	 * Returns a template URI based on the plug-in.
	 * 
	 * @param relativePath the relative path of the resource to resolve
	 * @return a String representing the URI of the resource given in parameter
	 */
	private static String getTemplateURI(String relativePath)
	{
		return TopcasedBusServicesGeneratorPlugin.getDefault().getBundle().getEntry(relativePath).toString();
	}

	/**
	 * Saves generated file.
	 * 
	 * @param generated
	 * @param file
	 * @param monitor
	 * @throws CoreException
	 */
	private static void saveGenerated(String generated, IFile file) throws CoreException
	{
		InputStream contents = new ByteArrayInputStream(generated.getBytes());
		IContainer parent = file.getParent();
		IFile target = parent.getFile(new Path(file.getName()));

		if (target.exists())
		{
			target.setContents(contents, true, false, null);
		}
		else
		{
			File systemFile = target.getLocation().toFile();
			if (systemFile.exists())
			{
				// Checks if out of sync
				parent.refreshLocal(1, null);
				target.setContents(contents, true, false, null);
			}
			else
			{
				target.create(contents, false, null);
			}
		}
	}
}