/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.generator.services.generation;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.pde.core.plugin.IExtensionsModelFactory;
import org.eclipse.pde.core.plugin.IPluginElement;
import org.eclipse.pde.core.plugin.IPluginExtension;
import org.eclipse.pde.core.plugin.IPluginModelBase;
import org.topcased.bus.core.CorePlugin;

/**
 * This class is responsible for generating the plugin.xml file of the generated Topcased
 * plugin.<br>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public final class TopcasedPluginFileGenerator
{
	/** Represents the <i>id</i> attribute */
	private static final String ID = "id";

	/** Represents the <i>name</i> attribute */
	private static final String NAME = "name";

	/** Represents the <i>description</i> attribute */
	private static final String DESCRIPTION = "description";

	/** Represents the <i>class</i> attribute */
	private static final String CLASS = "class";

	/** Represents the <i>category</i> attribute */
	private static final String CATEGORY = "category";

	/** Represents the <i>kind</i> attribute */
	private static final String KIND = "kind";

	/**
	 * Generates the plugin.xml file with the good extension points, dependencies and so on...
	 * 
	 * @param project The current prosject
	 * @param model The current model representing the structure of the plugin.xml file
	 * @param serviceName The name affected to the new project
	 */
	public static void generatePluginXMLFile(IProject project, IPluginModelBase model, String serviceName)
	{
		if (model.isEditable())
		{
			IExtensionsModelFactory factory = model.getFactory();

			// Elements for the preference Extension point
			IPluginExtension prefExtension = factory.createExtension();
			IPluginElement prefChildObject = factory.createElement(prefExtension);

			// Elements for the service Extension point
			IPluginExtension serviceExtension = factory.createExtension();
			IPluginElement serviceChildObject = factory.createElement(serviceExtension);
			IPluginElement ontologyExtension1 = factory.createElement(serviceChildObject);
			IPluginElement outputParamExtension1 = factory.createElement(serviceChildObject);
			IPluginElement outputParam1 = factory.createElement(outputParamExtension1);
			IPluginElement listInputParamExtension1 = factory.createElement(serviceChildObject);
			IPluginElement inputParam1 = factory.createElement(listInputParamExtension1);
			IPluginElement inputParam2 = factory.createElement(listInputParamExtension1);

			// Elements for the popup menu item prefExtension point
			IPluginExtension popupExtension = factory.createExtension();
			IPluginElement popupChildObject = factory.createElement(popupExtension);
			IPluginElement action = factory.createElement(popupChildObject);

			try
			{
				String projectName = project.getName().toLowerCase();
				
				// Defines the preference extension
				prefExtension.setPoint("org.eclipse.ui.preferencePages");
				prefChildObject.setName("page");
				prefChildObject.setAttribute(ID, projectName.concat(".preferences.id"));
				prefChildObject.setAttribute(NAME, serviceName);
				prefChildObject.setAttribute(CLASS, projectName.concat(".preferences.NewPreferencePage"));
				prefChildObject.setAttribute(CATEGORY, "org.topcased.facilities.preferences.TopcasedPreferencePage");
				prefExtension.add(prefChildObject);

				// Defines the service extension
				serviceExtension.setPoint("org.topcased.bus.core.Services");
				serviceChildObject.setName("service");
				serviceChildObject.setAttribute(ID, projectName.concat(".sampleservice"));
				serviceChildObject.setAttribute(NAME, serviceName);
				serviceChildObject.setAttribute(CLASS, projectName.concat(".services.SampleService"));
				serviceExtension.add(serviceChildObject);

				// Defines the ontology for the service
				ontologyExtension1.setName("ontologyType");
				ontologyExtension1.setAttribute(ID, projectName.concat(".ontology"));
				ontologyExtension1.setAttribute(NAME, "Sample Service Ontology");
				ontologyExtension1.setAttribute(CATEGORY, "Others");
				serviceChildObject.add(ontologyExtension1);

				// The first input parameter
				listInputParamExtension1.setName("listInputParam");
				inputParam1.setName("parameter");
				inputParam1.setAttribute(ID, projectName.concat(".").concat(TopcasedBusServicesConstants.FIRST_NAME_KEY));
				inputParam1.setAttribute(NAME, TopcasedBusServicesConstants.FIRST_NAME_KEY);
				inputParam1.setAttribute(DESCRIPTION, "A first name for test purpose");
				inputParam1.setAttribute(KIND, "String");
				listInputParamExtension1.add(inputParam1);

				// The second input parameter
				inputParam2.setName("parameter");
				inputParam2.setAttribute(ID, projectName.concat(".").concat(TopcasedBusServicesConstants.LAST_NAME_KEY));
				inputParam2.setAttribute(NAME, TopcasedBusServicesConstants.LAST_NAME_KEY);
				inputParam2.setAttribute(DESCRIPTION, "A last name for test purpose");
				inputParam2.setAttribute(KIND, "String");
				listInputParamExtension1.add(inputParam2);
				serviceChildObject.add(listInputParamExtension1);

				// Defines the output parameter
				outputParamExtension1.setName("outputParam");
				outputParam1.setName("parameter");
				outputParam1.setAttribute(ID, projectName.concat(".outParam"));
				outputParam1.setAttribute(NAME, "Void");
				outputParam1.setAttribute(DESCRIPTION, "Void return");
				outputParam1.setAttribute(KIND, "Void");
				outputParamExtension1.add(outputParam1);
				serviceChildObject.add(outputParamExtension1);

				// Defines the pop-up action
				popupExtension.setPoint("org.eclipse.ui.popupMenus");
				popupChildObject.setName("objectContribution");
				popupChildObject.setAttribute(ID, projectName.concat(".objectContribution"));
				popupChildObject.setAttribute("objectClass", "org.eclipse.core.resources.IResource");
				popupChildObject.setAttribute("adaptable", "true");
				popupExtension.add(popupChildObject);

				// Defines the actions attached to the pop-up
				action.setName("action");
				action.setAttribute(ID, projectName.concat(".action"));
				action.setAttribute("label", serviceName);
				action.setAttribute(CLASS, projectName.concat(".actions.LaunchAction"));
				action
				        .setAttribute("menubarPath",
				                "org.topcased.facilities.menu.topcased/org.topcased.facilities.menu.other/topcasedGroup");
				popupChildObject.add(action);

				model.getExtensions().add(prefExtension);
				model.getExtensions().add(popupExtension);
				model.getExtensions().add(serviceExtension);
			}
			catch (CoreException e)
			{
				CorePlugin.log(TopcasedPluginFileGenerator.class, "An error occured during the creation of plugin.xml file", IStatus.ERROR,
				        e);
			}
		}
	}
}