/*******************************************************************************
 * Copyright (c) 2006 David Sciamma. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma - initial API and implementation 
 ******************************************************************************/

package org.topcased.java.reverse;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * This class converts objects from the JDT model to a UML2 model.
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class Java2UMLConverter
{

    /**
     * Constructor
     */
    public Java2UMLConverter()
    {
        // Do nothing
    }

    /**
     * Convert an Package or a Java project into a UML2 model
     * 
     * @param javaElement the JavaElement
     * @return The UML2 model or <code>null</code> if the IJavaElement is not
     *         a Package or a Java Project.
     * @throws JavaModelException A error occured during Java Model analysis
     */
    public Model convert(IJavaElement javaElement) throws JavaModelException
    {
        switch (javaElement.getElementType())
        {
            case IJavaElement.JAVA_PROJECT:
                return doConvertion((IJavaProject) javaElement);
            case IJavaElement.PACKAGE_FRAGMENT:
                return doConvertion((IPackageFragment) javaElement);
            default:
                return null;
        }
    }

    /**
     * Convert a Java project into a UML2 model
     * 
     * @param project the Java Project
     * @return The UML2 model.
     * @throws JavaModelException A error occured during Java Model analysis
     */
    private Model doConvertion(IJavaProject project) throws JavaModelException
    {
        Model model = UMLFactory.eINSTANCE.createModel();

        initializeModel(model);

        IPackageFragment[] children = project.getPackageFragments();
        for (int i = 0; i < children.length; i++)
        {
            IPackageFragment fragment = children[i];
            if (fragment.getCompilationUnits().length > 0)
            {
                Package packageObject = findOrCreatePackage(model, fragment);

                createTypes(packageObject, fragment);
            }
        }

        for (int i = 0; i < children.length; i++)
        {
            IPackageFragment fragment = children[i];
            if (fragment.getCompilationUnits().length > 0)
            {
                Package packageObject = findOrCreatePackage(model, fragment);

                initializeTypes(packageObject, fragment);
            }
        }

        for (int i = 0; i < children.length; i++)
        {
            IPackageFragment fragment = children[i];
            if (fragment.getCompilationUnits().length > 0)
            {
                Package packageObject = findOrCreatePackage(model, fragment);

                createAssociations(packageObject);
            }
        }

        return model;

    }

    /**
     * Convert a Java project into a UML2 model
     * 
     * @param packageModel the Java Package
     * @return The UML2 model.
     * @throws JavaModelException A error occured during Java Model analysis
     */
    private Model doConvertion(IPackageFragment packageModel) throws JavaModelException
    {
        Model model = UMLFactory.eINSTANCE.createModel();

        return convert(model, packageModel);
    }


    /**
     * Convert a Java Package Fragment into a UML2 PAckage and update the
     * given UML2 Model
     * 
     * @param model The UML2 model where the Package must be inserted
     * @param fragment the Package Fragment
     * @return The updated UML2 model.
     * @throws JavaModelException A error occured during Java Model analysis
     */
    public Model convert(Model model, IPackageFragment fragment) throws JavaModelException
    {
        initializeModel(model);

        Package packageObject = findOrCreatePackage(model, fragment);

        createTypes(packageObject, fragment);
        initializeTypes(packageObject, fragment);
        createAssociations(packageObject);

        return model;
    }

    /**
     * Creates the Java Primitive Types.<br>
     * We could also import these Primitive Types from the UML2 Java Library but
     * this solution is lighter.
     * 
     * @param model the UML2 model to update
     */
    protected void initializeModel(Model model)
    {
        if (model.getOwnedType("boolean") == null)
        {
            model.createOwnedPrimitiveType("boolean");
        }
        if (model.getOwnedType("byte") == null)
        {
            model.createOwnedPrimitiveType("byte");
        }
        if (model.getOwnedType("char") == null)
        {
            model.createOwnedPrimitiveType("char");
        }
        if (model.getOwnedType("double") == null)
        {
            model.createOwnedPrimitiveType("double");
        }
        if (model.getOwnedType("float") == null)
        {
            model.createOwnedPrimitiveType("float");
        }
        if (model.getOwnedType("int") == null)
        {
            model.createOwnedPrimitiveType("int");
        }
        if (model.getOwnedType("long") == null)
        {
            model.createOwnedPrimitiveType("long");
        }
        if (model.getOwnedType("short") == null)
        {
            model.createOwnedPrimitiveType("short");
        }
    }

    protected Package findOrCreatePackage(Model model, IPackageFragment fragment)
    {
        String qualifiedName = fragment.getElementName();

        // Creates recursively the hierarchy of packages
        StringTokenizer tokenizer = new StringTokenizer(qualifiedName, ".");
        Package parent = model;
        Package packageObject = model;
        while (tokenizer.hasMoreTokens())
        {
            String packageName = tokenizer.nextToken();
            packageObject = parent.getNestedPackage(packageName);
            if (packageObject == null)
            {
                packageObject = parent.createNestedPackage(packageName);
            }
            parent = packageObject;
        }

        return packageObject;
    }

    protected Type findType(Package packageObject, String typeName)
    {
        Type type = null;
        if (typeName != null)
        {
            NamedElement element = packageObject.getMember(typeName);
            if (element instanceof Type)
            {
                type = (Type) element;
            }

            if (type == null)
            {
                type = findType(packageObject.getModel(), typeName, true);
            }
        }
        return type;
    }

    protected Type findType(Package packageObject, String typeName, boolean deep)
    {
        Type type = null;
        if (typeName != null)
        {
            NamedElement element = packageObject.getMember(typeName);
            if (element instanceof Type)
            {
                type = (Type) element;
            }

            if (type == null && deep)
            {
                Iterator it = packageObject.getPackagedElements().iterator();
                while (it.hasNext() && type == null)
                {
                    PackageableElement child = (PackageableElement) it.next();
                    if (child instanceof Package)
                    {
                        type = findType((Package) child, typeName, true);
                    }
                }
            }
        }
        return type;
    }

    protected Type findOrCreateType(Package packageObject, String typeName)
    {
        Type type = findType(packageObject, typeName);

        if (typeName != null && !"void".equals(typeName))
        {
            if (type == null)
            {
                DataType dataType = UMLFactory.eINSTANCE.createDataType();
                dataType.setName(typeName);

                packageObject.getPackagedElements().add(dataType);

                type = dataType;
            }
        }
        return type;
    }

    protected void initializeTypes(Package packageObject, IPackageFragment fragment) throws JavaModelException
    {
        // For all the package member
        // // Find the IJavaElement
        // // If interface
        // //// update super types
        // //// for all fields
        // ////// update type
        // //// for all methods
        // ////// update return type
        // ////// for all parameters
        // ////// update type
        // // If class
        // //// update extend class
        // //// update super types
        // //// for all fields
        // ////// update type
        // ////// if type is in the model
        // //////// create an association
        // ////// if type is a primitive type or is a datatype
        // //////// do nothing
        // ////// else
        // //////// create a datatype
        // //// for all methods
        // ////// update return type
        // ////// for all parameters
        // //////// update type

        IJavaElement[] children = fragment.getChildren();
        for (int i = 0; i < children.length; i++)
        {
            if (children[i] instanceof ICompilationUnit)
            {
                ICompilationUnit unit = (ICompilationUnit) children[i];
                IType[] types = unit.getAllTypes();
                for (int j = 0; j < types.length; j++)
                {
                    IType type = types[j];
                    String typeName = type.getElementName();
                    NamedElement element = packageObject.getOwnedMember(typeName);

                    if (element instanceof Interface)
                    {
                        Interface interfaceObject = (Interface) element;

                        // Super interfaces
                        String[] superInterfaces = type.getSuperInterfaceNames();
                        for (int k = 0; k < superInterfaces.length; k++)
                        {
                            String interfaceName = superInterfaces[k];
                            Type interfaceType = findType(packageObject, interfaceName);
                            if (interfaceType != null && interfaceType instanceof Classifier)
                            {
                                interfaceObject.createGeneralization((Classifier) interfaceType);
                            }
                        }

                        // Inner objects
                        createProperties(type, interfaceObject);
                        createOperations(type, interfaceObject);
                    }
                    if (element instanceof Class)
                    {
                        Class classObject = (Class) element;

                        // Super Interfaces
                        String[] superInterfaces = type.getSuperInterfaceNames();
                        for (int k = 0; k < superInterfaces.length; k++)
                        {
                            String interfaceName = superInterfaces[k];
                            Type interfaceType = findType(packageObject, interfaceName);
                            if (interfaceType != null && interfaceType instanceof Interface)
                            {
                                classObject.createInterfaceRealization(interfaceName, (Interface) interfaceType);
                            }
                        }

                        // Generalization
                        String superClassName = type.getSuperclassName();
                        Type classType = findOrCreateType(packageObject, superClassName);
                        if (classType != null && classType instanceof Classifier)
                        {
                            classObject.createGeneralization((Classifier) classType);
                        }

                        // Inner objects
                        createProperties(type, classObject);
                        createOperations(type, classObject);
                    }
                }
            }
        }
    }

    protected void createAssociations(Package packageObject) throws JavaModelException
    {
        List workingCopy = new ArrayList(packageObject.getPackagedElements());
        Iterator itType = workingCopy.iterator();
        while (itType.hasNext())
        {
            PackageableElement element = (PackageableElement) itType.next();
            if (element instanceof Interface)
            {
                Interface interfaceObject = (Interface) element;
                Iterator itProp = interfaceObject.getOwnedAttributes().iterator();
                while (itProp.hasNext())
                {
                    Property property = (Property) itProp.next();
                    Type targetType = property.getType();
                    if (targetType instanceof StructuredClassifier || targetType instanceof Interface)
                    {
                        createAssociation(property);
                    }
                }
            }
            if (element instanceof StructuredClassifier)
            {
                StructuredClassifier classifierObject = (StructuredClassifier) element;
                Iterator itProp = classifierObject.getOwnedAttributes().iterator();
                while (itProp.hasNext())
                {
                    Property property = (Property) itProp.next();
                    Type targetType = property.getType();
                    if (targetType instanceof StructuredClassifier || targetType instanceof Interface)
                    {
                        createAssociation(property);
                    }
                }
            }
        }
    }

    protected void createAssociation(Property property)
    {
        if (property.getAssociation() == null)
        {
            Property opposite = foundOpposite(property);
            Element propOwner = property.getOwner();
            if (opposite != null)
            {
                Association association = UMLFactory.eINSTANCE.createAssociation();
                association.getMemberEnds().add(property);
                association.getMemberEnds().add(opposite);

                property.getNearestPackage().getPackagedElements().add(association);
            }
            else
            {
                Association association = UMLFactory.eINSTANCE.createAssociation();
                association.getMemberEnds().add(property);
                Property createdOpposite = association.createOwnedEnd("target", (Type) propOwner);

                property.getNearestPackage().getPackagedElements().add(association);
            }
        }
    }

    protected Property foundOpposite(Property source)
    {
        Property target = null;

        Type propType = source.getType();
        Element propOwner = source.getOwner();
        if (propType instanceof StructuredClassifier || propType instanceof Interface)
        {
            // Search if the property type has a property of the type of
            // propOwner
            List properties = null;
            if (propType instanceof StructuredClassifier)
            {
                properties = ((StructuredClassifier) propType).getOwnedAttributes();
            }
            if (propType instanceof Interface)
            {
                properties = ((Interface) propType).getOwnedAttributes();
            }

            Iterator it = properties.iterator();
            while (it.hasNext() && target == null)
            {
                Property prop = (Property) it.next();
                // Check that is a different property
                if (prop != source && propOwner.equals(prop.getType()))
                {
                    target = prop;
                }
            }
        }

        return target;
    }

    protected void createTypes(Package packageObject, IPackageFragment fragment) throws JavaModelException
    {
        IJavaElement[] children = fragment.getChildren();
        for (int i = 0; i < children.length; i++)
        {
            if (children[i] instanceof ICompilationUnit)
            {
                ICompilationUnit unit = (ICompilationUnit) children[i];
                IType[] types = unit.getAllTypes();
                for (int j = 0; j < types.length; j++)
                {
                    IType type = types[j];

                    if (type.isInterface())
                    {
                        Interface interfaceObject = UMLFactory.eINSTANCE.createInterface();
                        interfaceObject.setName(type.getElementName());

                        update(interfaceObject, type.getFlags());

                        packageObject.getPackagedElements().add(interfaceObject);
                    }
                    else
                    {
                        Class classObject = UMLFactory.eINSTANCE.createClass();
                        classObject.setName(type.getElementName());

                        update(classObject, type.getFlags());

                        packageObject.getPackagedElements().add(classObject);
                    }
                }
            }
        }
    }

    protected void createOperations(IType type, Class classifier) throws JavaModelException
    {
        IMethod[] methods = type.getMethods();
        for (int k = 0; k < methods.length; k++)
        {
            IMethod method = methods[k];
            if (!method.isConstructor())
            {
                Operation operationObject = createOperation(classifier, method);
                classifier.getOwnedOperations().add(operationObject);
            }
        }
    }

    protected Operation createOperation(Element element, IMethod method) throws JavaModelException
    {
        String methodName = method.getElementName();

        Operation operationObject = UMLFactory.eINSTANCE.createOperation();
        operationObject.setName(methodName);

        update(operationObject, method.getFlags());

        int flags = method.getFlags();
        operationObject.setIsAbstract(Flags.isAbstract(flags));
        operationObject.setIsStatic(Flags.isStatic(flags));

        createParameters(element, operationObject, method);

        if (method.exists() && !method.isConstructor())
        {
            String returnTypeSig = method.getReturnType();

            String returnTypeWithoutArray = Signature.getElementType(returnTypeSig);
            String returnTypeName = Signature.toString(Signature.getTypeErasure(returnTypeWithoutArray));
            Type returnType = findOrCreateType(element.getNearestPackage(), returnTypeName);
            if (returnType != null)
            {
                operationObject.createReturnResult("return", returnType);
            }
        }

        return operationObject;
    }

    protected void createParameters(Element element, Operation operation, IMethod method) throws JavaModelException
    {
        String[] paramNames = method.getParameterNames();
        String[] paramTypeSigs = method.getParameterTypes();
        for (int i = 0; i < paramNames.length; i++)
        {
            String name = paramNames[i];
            String typeSig = paramTypeSigs[i];

            Parameter parameter = UMLFactory.eINSTANCE.createParameter();
            parameter.setName(name);

            String typeWithoutArray = Signature.getElementType(typeSig);
            String typeName = Signature.toString(Signature.getTypeErasure(typeWithoutArray));
            Type paramType = findOrCreateType(element.getNearestPackage(), typeName);
            if (paramType != null)
            {
                parameter.setType(paramType);
            }

            operation.getOwnedParameters().add(parameter);
        }
    }

    protected void createOperations(IType type, Interface interfaceObject) throws JavaModelException
    {
        IMethod[] methods = type.getMethods();
        for (int k = 0; k < methods.length; k++)
        {
            IMethod method = methods[k];
            if (!method.isConstructor())
            {
                Operation operationObject = createOperation(interfaceObject, method);
                interfaceObject.getOwnedOperations().add(operationObject);
            }
        }
    }

    protected void createProperties(IType type, StructuredClassifier classifier) throws JavaModelException
    {
        IField[] fields = type.getFields();
        for (int k = 0; k < fields.length; k++)
        {
            IField field = fields[k];

            Property propertyObject = createProperty(classifier, field);

            classifier.getOwnedAttributes().add(propertyObject);

        }
    }

    protected Property createProperty(Element element, IField field) throws JavaModelException
    {
        String fieldName = field.getElementName();
        String fieldTypeSig = field.getTypeSignature();

        String fieldTypeWithoutArray = Signature.getElementType(fieldTypeSig);
        String fieldTypeName = Signature.toString(Signature.getTypeErasure(fieldTypeWithoutArray));
        Type fieldType = findOrCreateType(element.getNearestPackage(), fieldTypeName);

        Property propertyObject = UMLFactory.eINSTANCE.createProperty();
        propertyObject.setName(fieldName);
        if (fieldType != null)
        {
            propertyObject.setType(fieldType);
        }

        update(propertyObject, field.getFlags());

        return propertyObject;
    }

    protected void createProperties(IType type, Interface interfaceObject) throws JavaModelException
    {
        IField[] fields = type.getFields();
        for (int k = 0; k < fields.length; k++)
        {
            IField field = fields[k];

            Property propertyObject = createProperty(interfaceObject, field);

            interfaceObject.getOwnedAttributes().add(propertyObject);
        }
    }

    protected void update(NamedElement element, int flags) throws JavaModelException
    {
        if (Flags.isPrivate(flags))
        {
            element.setVisibility(VisibilityKind.PRIVATE_LITERAL);
        }
        else if (Flags.isProtected(flags))
        {
            element.setVisibility(VisibilityKind.PROTECTED_LITERAL);
        }
        else if (Flags.isPublic(flags))
        {
            element.setVisibility(VisibilityKind.PUBLIC_LITERAL);
        }
    }

    protected String getName(IPackageFragment fragment)
    {
        String name = "";
        String fullName = fragment.getElementName();
        int dotIndex = fullName.lastIndexOf('.');
        if (dotIndex > 0)
        {
            name = fullName.substring(dotIndex + 1);
        }

        return name;
    }
}
