/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.model.management.internal.actions;

import java.util.Iterator;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.PageBookView;
import org.topcased.model.management.IAnnotationConstants;

/**
 * An action used to remove UUID Annotations that may have been added after a save with the TOPCASED graphical editor.
 * 
 * Creation 18 déc. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class RemoveUUIDAction implements IObjectActionDelegate
{
    private EObject eObject;

    private EditingDomain editingDomain;

    /**
     * Constructor
     */
    public RemoveUUIDAction()
    {
        // Do nothing
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action)
    {
        // TODO Use the CommandStack to remove Annotation UUID
        if (eObject != null && editingDomain != null)
        {
            // The CompoundCommand where are added all the RemoveCommand commands
            CompoundCommand compoundCommand = new CompoundCommand();

            // Remove UUID on this object
            removeUUIDForEObject(eObject, compoundCommand);

            // Remove UUID on children
            Iterator<EObject> itChildren = eObject.eAllContents();
            while (itChildren.hasNext())
            {
                removeUUIDForEObject(itChildren.next(), compoundCommand);
            }

            // Execute the global command
            editingDomain.getCommandStack().execute(compoundCommand);
        }

    }

    /**
     * Remove UUID on elements : elements can only be modified if the class inherits from EModelElement
     * 
     * @param modelObject the model to modify
     * @param compoundCommand The CompoundCommand where the currentCommand should be appended
     */
    private void removeUUIDForEObject(EObject modelObject, CompoundCommand compoundCommand)
    {
        if (modelObject instanceof EModelElement)
        {
            if (((EModelElement) modelObject).getEAnnotation(IAnnotationConstants.UUID_SOURCE) != null)
            {
                compoundCommand.append(new RemoveCommand(editingDomain, modelObject, EcorePackage.eINSTANCE.getEModelElement_EAnnotations(),
                        ((EModelElement) modelObject).getEAnnotation(IAnnotationConstants.UUID_SOURCE)));
            }
        }
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
        eObject = null;

        if (selection instanceof IStructuredSelection)
        {
            IStructuredSelection ssel = (IStructuredSelection) selection;
            if (!ssel.isEmpty() && ssel.size() == 1)
            {
                Object selectedObj = ssel.getFirstElement();
                if (selectedObj instanceof EObject)
                {
                    eObject = (EObject) selectedObj;
                }

                // Try to adapt
                if (selectedObj instanceof IAdaptable)
                {
                    eObject = (EObject) ((IAdaptable) selectedObj).getAdapter(EObject.class);
                }
            }
        }
        action.setEnabled(eObject != null);
    }

    /**
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        this.editingDomain = getEditingDomain(targetPart);
    }

    /**
     * This returns the editing domain associated with an IWorkbenchPart.
     * 
     * @param targetPart The IWorkbenchPart that is linked with the current selected EObject
     * @return The required editing domain or null when the part cannot be adapted in any EditingDomain.
     */
    private EditingDomain getEditingDomain(IWorkbenchPart targetPart)
    {
        if (targetPart.getAdapter(EditingDomain.class) != null)
        {
            return (EditingDomain) targetPart.getAdapter(EditingDomain.class);
        }

        if (targetPart instanceof IEditingDomainProvider)
        {
            return ((IEditingDomainProvider) targetPart).getEditingDomain();
        }

        if (targetPart.getAdapter(IEditingDomainProvider.class) != null)
        {
            return ((IEditingDomainProvider) targetPart.getAdapter(IEditingDomainProvider.class)).getEditingDomain();
        }

        if (targetPart instanceof PageBookView)
        {
            IPage page = ((PageBookView) targetPart).getCurrentPage();
            if (page instanceof IEditingDomainProvider)
            {
                return ((IEditingDomainProvider) page).getEditingDomain();
            }
        }

        return null;
    }
}
