/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.resource.StringConverter;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * This registry manages all the allocated colors and dispose its when the
 * current display is disposed. <br>
 * <br>
 * creation : 8 oct. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public final class ColorRegistry
{
    private static ColorRegistry instance;

    /**
     * This registries <code>Display</code>. All colors will be allocated
     * using it.
     */
    private Display display;

    /**
     * Table of known colors, keyed by symbolic color name (key type:
     * <code>String</code>, value type:
     * <code>org.eclipse.swt.graphics.Color</code>.
     */
    private Map stringToColor = new HashMap();

    /**
     * Runnable that cleans up the manager on disposal of the display.
     */
    private Runnable displayRunnable = new Runnable()
    {
        public void run()
        {
            clearCaches();
        }
    };

    /**
     * Create a new instance of the receiver that is hooked to the current
     * display.
     * 
     * @see org.eclipse.swt.widgets.Display#getCurrent()
     */
    private ColorRegistry()
    {
        this(Display.getCurrent());
    }

    /**
     * Create a new instance of the receiver.
     * 
     * @param d the <code>Display</code> to hook into.
     */
    private ColorRegistry(Display d)
    {
        Assert.isNotNull(d);
        this.display = d;
        hookDisplayDispose();
    }

    /**
     * Returns the singleton registry instance
     * 
     * @return the registry
     */
    public static ColorRegistry getInstance()
    {
        if (instance == null)
        {
            instance = new ColorRegistry();
        }

        return instance;
    }

    /**
     * Hook a dispose listener on the SWT display.
     */
    private void hookDisplayDispose()
    {
        display.disposeExec(displayRunnable);
    }

    /**
     * Releases all the allocated colors
     */
    private void clearCaches()
    {
        Iterator it = stringToColor.values().iterator();
        while (it.hasNext())
        {
            Object next = it.next();
            ((Color) next).dispose();
        }
        stringToColor.clear();
    }

    /**
     * Returns the <code>color</code> associated with the given symbolic color
     * name, or <code>null</code> if no such definition exists.
     * 
     * @param symbolicName symbolic color name
     * @return the <code>Color</code> or <code>null</code>
     */
    public Color get(String symbolicName)
    {
        Assert.isNotNull(symbolicName);
        if ("".equals(symbolicName)) //$NON-NLS-1$
        {
            return new Color(display, new RGB(0, 0, 0));
        }

        return get(StringConverter.asRGB(symbolicName));
    }

    /**
     * Returns the <code>color</code> associated with the given RGB object, or
     * <code>null</code> if no such definition exists.
     * 
     * @param rgb the rgb color
     * @return the <code>Color</code> or <code>null</code>
     */
    public Color get(RGB rgb)
    {
        if (rgb == null)
        {
            return null;
        }

        String symbolicName = StringConverter.asString(rgb);
        Object result = stringToColor.get(symbolicName);
        if (result != null)
        {
            return (Color) result;
        }

        Color color = new Color(display, rgb);

        stringToColor.put(symbolicName, color);

        return color;
    }
}