/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.StringConverter;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.utils.Utils;

/**
 * <b>Node creation </b> <br>
 * This command just add a graphNode into another. It also add the contained
 * element to the element of the parentNode graph node if needed. <br>
 * creation : 30 nov. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 * @deprecated Use the CreateGraphNodeCommand instead
 */
public class AddGraphNodeCommand extends Command
{
    /** The child GraphNode that is dropped */
    private GraphNode childNode;

    /** The old parent GraphNode that contained the childNode */
    private GraphNode oldParentNode;

    /** The new parent GraphNode that should contain the childNode */
    private GraphNode newParentNode;

    private Object newConstraint;

    private Object oldConstraint;

    /**
     * Constructor
     * 
     * @param newParent the parentNode node
     * @param child the child Node that is moved
     * @param constraint the new constraint
     */
    public AddGraphNodeCommand(GraphNode newParent, GraphNode child, Object constraint)
    {
        super("Drag'n'Drop inside the Editor");

        childNode = child;
        newParentNode = newParent;

        newConstraint = constraint;
    }

    /**
     * Check if the object that we try to add to the model is not already set
     * (because the cardinality is 1)
     * 
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return getEStructuralFeature(getNewParentEObject(), getChildEObject()) != null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldParentNode = (GraphNode) childNode.getContainer();

        if (newConstraint instanceof Rectangle)
        {
            oldConstraint = new Rectangle(childNode.getPosition(), childNode.getSize());
        }
        else if (newConstraint instanceof PortConstraint)
        {
            oldConstraint = new PortConstraint(new Rectangle(childNode.getPosition(), childNode.getSize()),
                    StringConverter.asInt(DIUtils.getPropertyValue(childNode, ModelerPropertyConstants.PORT_POSITION)));
        }

        redo();
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        oldParentNode.getContained().remove(childNode);
        newParentNode.getContained().add(childNode);

        if (newConstraint instanceof Rectangle)
        {
            childNode.setPosition(((Rectangle) newConstraint).getLocation());
            childNode.setSize(((Rectangle) newConstraint).getSize());
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION, null);
        }
        else if (newConstraint instanceof PortConstraint)
        {
            childNode.setPosition(((PortConstraint) newConstraint).getRectConstraints().getLocation());
            childNode.setSize(((PortConstraint) newConstraint).getRectConstraints().getSize());
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION,
                    StringConverter.asString(((PortConstraint) newConstraint).getAttachment()));
        }

        redoModel();
    }

    /**
     * Update the model after at the end of the redo() method. This method
     * should be overriden by subclasses.
     */
    protected void redoModel()
    {
        EObject childEObject = getChildEObject();
        EObject oldParentEObject = getOldParentEObject();
        EObject newParentEObject = getNewParentEObject();

        EReference oldSelectedFeature = getEStructuralFeature(oldParentEObject, childEObject);
        EReference newSelectedFeature = getEStructuralFeature(newParentEObject, childEObject);

        if (oldSelectedFeature != null && oldParentEObject != null && childEObject != null)
        {
            EList ownerList = Utils.getOwnerList(oldParentEObject, oldSelectedFeature);
            if (ownerList.contains(childEObject))
            {
                ownerList.remove(childEObject);
            }
        }

        if (newSelectedFeature != null && newParentEObject != null && childEObject != null)
        {
            EList ownerList = Utils.getOwnerList(newParentEObject, newSelectedFeature);
            if (!ownerList.contains(childEObject))
            {
                ownerList.add(childEObject);
            }
        }
    }

    /**
     * Get the EStructuralFeature (type Containment) of the parent that may
     * contain the child
     * 
     * @param parent the parent EObject
     * @param child the child EObject
     * @return the EStructuralFeature
     */
    private EReference getEStructuralFeature(EObject parent, EObject child)
    {
        if (parent != null && child != null)
        {
            // we check all the EStructuralFeatures of type Containment
            for (Iterator iter = parent.eClass().getEAllContainments().iterator(); iter.hasNext();)
            {
                EReference eRef = (EReference) iter.next();
                // Check if the Reference is of the expected type
                if (eRef.getEReferenceType().isInstance(child))
                {
                    return eRef;
                }
            }
        }
        return null;
    }

    /**
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        newParentNode.getContained().remove(childNode);
        oldParentNode.getContained().add(childNode);

        if (oldConstraint instanceof Rectangle)
        {
            childNode.setPosition(((Rectangle) oldConstraint).getLocation());
            childNode.setSize(((Rectangle) oldConstraint).getSize());
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION, null);
        }
        else if (oldConstraint instanceof PortConstraint)
        {
            childNode.setPosition(((PortConstraint) oldConstraint).getRectConstraints().getLocation());
            childNode.setSize(((PortConstraint) oldConstraint).getRectConstraints().getSize());
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION,
                    StringConverter.asString(((PortConstraint) oldConstraint).getAttachment()));
        }

        undoModel();
    }

    /**
     * Update the model at the end of the undo() method. This method should be
     * overriden by subclasses.
     */
    protected void undoModel()
    {
        EObject childEObject = getChildEObject();
        EObject oldParentEObject = getOldParentEObject();
        EObject newParentEObject = getNewParentEObject();

        EReference oldSelectedFeature = getEStructuralFeature(oldParentEObject, childEObject);
        EReference newSelectedFeature = getEStructuralFeature(newParentEObject, childEObject);

        if (newSelectedFeature != null && newParentEObject != null && childEObject != null)
        {
            EList ownerList = Utils.getOwnerList(newParentEObject, newSelectedFeature);
            if (ownerList.contains(childEObject))
            {
                ownerList.add(childEObject);
            }
        }

        if (oldSelectedFeature != null && oldParentEObject != null && childEObject != null)
        {
            EList ownerList = Utils.getOwnerList(oldParentEObject, oldSelectedFeature);
            if (!ownerList.contains(childEObject))
            {
                ownerList.remove(childEObject);
            }
        }
    }

    /**
     * Return the childNode to create
     * 
     * @return the childNode
     */
    protected GraphNode getChildNode()
    {
        return childNode;
    }

    /**
     * Return the childNode EObject to create
     * 
     * @return the childNode EObject
     */
    protected EObject getChildEObject()
    {
        return Utils.getElement(childNode);
    }

    /**
     * Return the parentNode EObject that originally contained the childNode
     * 
     * @return the old parent EObject
     */
    protected EObject getOldParentEObject()
    {
        return Utils.getElement(oldParentNode);
    }

    /**
     * Return the parentNode EObject that will contain the childNode soon
     * 
     * @return the new parent EObject
     */
    protected EObject getNewParentEObject()
    {
        return Utils.getElement(newParentNode);
    }

}
