/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies), 
 *    Jose Alfredo Serrano (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.command.DeleteCommand;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.providers.IDeleteObjectProvider;
import org.topcased.modeler.providers.IDeletePartnerProvider;
import org.topcased.modeler.utils.Utils;

/**
 * A command that erase an object from the model and all the associated
 * graphical objects. <br>
 * creation : 1 juin 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class DeleteModelCommand extends CompoundCommand
{

    /**
     * The Modeler editDomain
     */
    private MixedEditDomain editDomain;

    private Diagrams diagrams;

    private boolean deleteAllReferences;

    /**
     * This boolean stores if a "Go to root diagram" command have been added
     */
    private boolean goToRoot = false;

    /**
     * Constructor
     * 
     * @param domain
     * @param diagrams
     * @param model
     * @param deleteAllRef
     */
    public DeleteModelCommand(MixedEditDomain domain, Diagrams diagrams, EObject model, boolean deleteAllRef)
    {
        super("Delete Object from Model");
        this.editDomain = domain;
        this.diagrams = diagrams;
        this.deleteAllReferences = deleteAllRef;

        initialize(model);
    }

    /**
     * Constructor
     * 
     * @param domain
     * @param diagrams
     * @param model
     */
    public DeleteModelCommand(MixedEditDomain domain, Diagrams diagrams, EObject model)
    {
        this(domain, diagrams, model, true);
    }

    private void initialize(EObject model)
    {
        if (model != null)
        {
            if (model instanceof Diagram)
            {
                Command removeDiag = createRemoveDiagramCommand((Diagram) model);
                if (removeDiag != null)
                {
                    add(removeDiag);
                }
            }
            // check if it is not the main object
            else if (model != model.eResource().getContents().get(0))
            {
                // Création de la commande de suppression de l'objet dans le
                // modèle

                // Suppression dans le modele
                // 1- Recuperation du IDeletePartnerProvider : Le
                // IDeletePartnerProvider
                // fournit les objets qui representent une connection vers ou
                // depuis
                // un objet
                // 2- Recuperation des objets connections
                // 3- Suppression des objets connections du modele (appel
                // recursif a
                // cette meme fonction)
                // 4- Suppression de l'objet
                List deletedObjects = getCompleteList(model);

                // Nettoyage :
                // 1- On nettoie d'abord en supprimant les diagrammes pour cet
                // objet et ses fils
                // 2- Le nettoyage parcourt les GraphElements et renvoie ceux
                // qui
                // sont lies à des objets EMF de la liste precedente.
                // Les GraphElements a nettoyer sont ensuite supprimes du
                // modèle.
                //
                // List deletedDiagrams = getDeleteDiagrams(deletedObjects);
                // Command removeDiags =
                // createRemoveDiagramsCommand(deletedDiagrams);
                // if (removeDiags != null)
                // {
                // add(removeDiags);
                // }

                // Suppression d'un GraphElement :
                // 1- Recuperation des connections liees a cet element
                // 3- Suppression des connections (appel recursif a cette meme
                // methode)
                // 2- Suppression du GraphElement

                List deletedGraphElements = getDeleteGraphElements(deletedObjects);
                Command removeGraphCmd = createRemoveGraphCommand(deletedGraphElements);
                if (removeGraphCmd != null)
                {
                    add(removeGraphCmd);
                }

                Command removeCmd = createRemoveCommand(deletedObjects);
                add(removeCmd);
            }
        }
    }

    /**
     * Create the remove command for a diagram
     * 
     * @param diagram the diagram to remove
     * @return the command or <code>null</code> if action cannot be performed
     */
    protected Command createRemoveDiagramCommand(Diagram diagram)
    {
        Command removeCmd = null;
        // check if it is not the main diagram
        // if (diagram != DiagramsUtils.getRootDiagram(diagrams))
        // {
        List deletedObjects = new ArrayList();
        deletedObjects.add(diagram);
        removeCmd = createRemoveCommand(deletedObjects);
        // }

        return removeCmd;
    }

    /**
     * Create the CompoundCommand that remove model objects
     * 
     * @param deletedObjects the list that contain the model objects to remove
     * @return the CompoundCommand
     */
    protected Command createRemoveCommand(List deletedObjects)
    {
        CompoundCommand removeCmd = null;
        if (deletedObjects != null && !deletedObjects.isEmpty())
        {
            removeCmd = new CompoundCommand();
            Iterator it = deletedObjects.iterator();
            while (it.hasNext())
            {
                Object obj = it.next();

                // The list may contains wrapped objects
                if (obj instanceof IWrapperItemProvider || obj instanceof FeatureMap.Entry || obj instanceof EObject)
                {
                    goToRootDiagram(obj, removeCmd);

                    List objList = new ArrayList();

                    // Get the AdapterFactoryEditingDomain
                    AdapterFactoryEditingDomain domain = editDomain.getEMFEditingDomain();
                    // The object must be wrapped if necessary
                    Object wrappedObj = domain.getWrapper(obj);
                    objList.add(wrappedObj);
                    if (deleteAllReferences)
                    {
                        org.eclipse.emf.common.command.Command cmd = DeleteCommand.create(
                                editDomain.getEMFEditingDomain(), objList);
                        Command gefCommand = new EMFtoGEFCommandWrapper(cmd);

                        // FIX the bug Topcased #277
                        // the Model object must be deleted first, and only
                        // after the references are deleted
                        removeCmd.add(gefCommand);
                    }
                    else
                    {
                        org.eclipse.emf.common.command.Command cmd = RemoveCommand.create(
                                editDomain.getEMFEditingDomain(), objList);
                        Command gefCommand = new EMFtoGEFCommandWrapper(cmd);

                        removeCmd.add(gefCommand);
                    }
                }
            }
        }

        return removeCmd;
    }

    /**
     * Add a "Go to Root diagram" command if needed
     * 
     * @param deletedObj the deleted objecct
     * @param removeCmd the list of commands
     */
    private void goToRootDiagram(Object deletedObj, CompoundCommand removeCmd)
    {
        if (deletedObj != null && !goToRoot)
        {
            // Return to root diagram if needed
            EObject unwrapObject = (EObject) AdapterFactoryEditingDomain.unwrap(deletedObj);
            // Check if the active diagram is not a child of the deleted
            // sub-tree
            final Modeler modeler = (Modeler) editDomain.getEditorPart();
            EObject diagObject = Utils.getElement(modeler.getActiveDiagram());
            if (diagObject.equals(unwrapObject) || EcoreUtil.isAncestor(unwrapObject, diagObject)
                    || deletedObj.equals(modeler.getActiveDiagram()))
            {
                // If it is, return to the root diagram
                removeCmd.add(new Command()
                {
                    public void execute()
                    {
                        Diagram rootDiagram = DiagramsUtils.getRootDiagram(modeler.getDiagrams());
                        if (rootDiagram != modeler.getActiveDiagram())
                        {
                            modeler.setActiveDiagram(rootDiagram);
                        }
                        else
                        {
                            // The diagram that is being deleted was the last
                            // diagram of the model
                            modeler.setActiveDiagram(null);

                            // TODO : There is a bug when we try to delete the
                            // current Root diagram while another diagram exists
                            // and may be the new Root Diagram, the
                            // NoDiagramConfiguration is then activated
                        }
                    }
                });
            }

            goToRoot = true;
        }
    }

    /**
     * Create the CompoundCommand that remove the GraphElement objects
     * 
     * @param deletedGraphElts the list that contain the graphElements to remove
     * @return the CompoundCommand
     */
    protected Command createRemoveGraphCommand(List deletedGraphElts)
    {
        CompoundCommand removeCmd = null;
        if (deletedGraphElts != null && !deletedGraphElts.isEmpty())
        {
            removeCmd = new CompoundCommand();

            Iterator itElts = deletedGraphElts.iterator();
            while (itElts.hasNext())
            {
                GraphElement elt = (GraphElement) itElts.next();

                if (elt instanceof Diagram)
                {
                    Command removeDiag = createRemoveDiagramCommand((Diagram) elt);
                    if (removeDiag != null)
                    {
                        removeCmd.add(removeDiag);
                    }
                }
                else
                {
                    DeleteGraphElementCommand deleteCmd = new DeleteGraphElementCommand(elt, true);
                    if (deleteCmd != null)
                    {
                        removeCmd.add(deleteCmd);
                    }
                }
            }
        }

        return removeCmd;
    }

    private List getDeleteGraphElements(List deletedObjects)
    {
        List elementList = new ArrayList();

        // Parcours de chaque diagramme
        Iterator it = DiagramsUtils.findAllDiagrams(diagrams).iterator();
        while (it.hasNext())
        {
            Diagram diag = (Diagram) it.next();

            checkElement(diag, deletedObjects, elementList);
        }

        return elementList;
    }

    private void checkElement(GraphElement elt, List deletedObjects, List deletedGraphElt)
    {
        EObject model = Utils.getElement(elt);
        // On verifie que l'objet du modele fait partie de la liste des objets
        // effaces
        if (model != null && isDelete(model, deletedObjects))
        {
            // Le GraphElement doit etre supprime
            if (!deletedGraphElt.contains(elt))
            {
                deletedGraphElt.add(elt);
            }
        }
        else
        {
            // On regarde les fils qui doivent etre supprimes
            Iterator itChildren = elt.getContained().iterator();
            while (itChildren.hasNext())
            {
                DiagramElement child = (DiagramElement) itChildren.next();
                if (child instanceof GraphElement)
                {
                    checkElement((GraphElement) child, deletedObjects, deletedGraphElt);
                }
            }
        }
    }

    private boolean isDelete(EObject model, List deletedObjects)
    {
        if (deletedObjects.contains(model) || EcoreUtil.isAncestor(deletedObjects, model))
        {
            return true;
        }

        return false;
    }

    /**
     * This method iterates on the selection and complete with EObjects that
     * must be deleted from each object of the selection.
     * 
     * @param selection the initial selection
     * @return the new list of objects to delete
     */
    private List getCompleteList(EObject selection)
    {
        List completeList = new ArrayList();
        addObject(selection, completeList);

        return completeList;
    }

    private void addObject(Object selection, List listToAppend)
    {
        // Verify if the selected object is not linked with the main diagram
        Diagram d = DiagramsUtils.getRootDiagram(diagrams);
        EObject model = Utils.getElement(d);
        if (selection.equals(model))
        {
            return;
        }

        // Verify that the selected object is not a direct child of the root
        // object
        AdapterFactory adapterFactory = editDomain.getAdapterFactory();
        IDeleteObjectProvider deleteObjectProvider = null;
        if (adapterFactory != null)
        {
            deleteObjectProvider = (IDeleteObjectProvider) adapterFactory.adapt(selection, IDeleteObjectProvider.class);
            if (deleteObjectProvider != null && !deleteObjectProvider.canBeDeleted(selection))
            {
                return;
            }
        }

        // Handle the rest of the model objects which can be deleted
        if (selection instanceof EObject && !listToAppend.contains(selection))
        {
            for (Iterator iter = ((EObject) selection).eContents().iterator(); iter.hasNext();)
            {
                addObject(iter.next(), listToAppend);
            }

            listToAppend.add(selection);

            // Deals with dependent objects
            if (adapterFactory != null)
            {
                // Get the provider from the adapterFactory
                IDeletePartnerProvider deleteProvider = (IDeletePartnerProvider) adapterFactory.adapt(selection,
                        IDeletePartnerProvider.class);

                // Add dependencies
                List dependencies = null;
                if (deleteProvider != null)
                {
                    dependencies = deleteProvider.getDeletePartners(selection);
                    if (dependencies != null)
                    {
                        Iterator it = dependencies.iterator();
                        while (it.hasNext())
                        {
                            Object obj = it.next();
                            addObject(obj, listToAppend);
                        }
                    }
                }
            }
        }
    }
}
