/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.documentation;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.topcased.model.management.IAnnotationConstants;

/**
 * A class defining a composite to edit the comments of a EModelElement as an
 * EAnnotation. <br/> Creation : 10 oct. 2005
 * 
 * @author <a href="mailto:thomas.friol@anyware-tech.com">Thomas FRIOL</a>
 */
public class EAnnotationCommentsComposite extends DocPageComposite
{
    private Text commentsText;

    private FocusListener focusListener;

    private String annotationKey;

    private String annotationSource;

    /**
     * Constructor.
     * 
     * @param parent the parent composite
     * @param style the composite style
     * @param commandStack the command stack to use to execute commands
     */
    public EAnnotationCommentsComposite(Composite parent, int style, CommandStack commandStack)
    {
        this(parent, style, commandStack, IAnnotationConstants.DOCUMENTATION_SOURCE, IAnnotationConstants.DOCUMENTATION_KEY);
    }

    /**
     * Constructor.
     * 
     * @param parent the parent composite
     * @param style the composite style
     * @param commandStack the command stack to use to execute commands
     * @param annotationSource The source of the annotation where the documentation annotation is stored
     * @param annotationKey The key of the documentation annotation
     */
    public EAnnotationCommentsComposite(Composite parent, int style, CommandStack commandStack, String annotationSource, String annotationKey)
    {
        super(parent, style, commandStack);
        
        this.annotationSource = annotationSource;
        this.annotationKey = annotationKey;
    }

    protected void createContents(Composite parent)
    {
        commentsText = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
        commentsText.setLayoutData(new GridData(GridData.FILL_BOTH));

        focusListener = new FocusAdapter()
        {
            private String previousComment;

            public void focusGained(FocusEvent e)
            {
                previousComment = commentsText.getText();
            }

            public void focusLost(FocusEvent e)
            {
                if (!commentsText.getText().equals(previousComment))
                {
                    handleDocChanged();
                }
            }
        };

        commentsText.addFocusListener(focusListener);
    }

    protected void refresh()
    {
        commentsText.setText("");
        commentsText.setEnabled(getModelElement() != null);
        if (getModelElement() != null)
        {
            EAnnotation annotation = getModelElement().getEAnnotation(annotationSource);
            if (annotation != null)
            {
                String doc = (String) annotation.getDetails().get(annotationKey);
                if (doc != null)
                {
                    commentsText.setText(doc);
                }
            }
        }
    }

    private void handleDocChanged()
    {
        String newDoc = commentsText.getText();
        if (getModelElement() != null)
        {
            Command cmd = new ChangeComments(getModelElement(), newDoc);
            getCommandStack().execute(cmd);
        }
    }

    private class ChangeComments extends Command
    {
        private EModelElement element;

        private String newComments;

        private String oldComments;

        public ChangeComments(EModelElement element, String comments)
        {
            super("Change comments of EModelElement");

            this.element = element;
            this.newComments = comments;
        }

        /**
         * @see org.eclipse.gef.commands.Command#execute()
         */
        public void execute()
        {
            // stores the previous doc
            oldComments = null;
            EAnnotation annotation = element.getEAnnotation(annotationSource);
            if (annotation != null)
            {
                oldComments = (String) annotation.getDetails().get(annotationKey);
            }

            redo();
        }

        /**
         * Set the documentation for the given Model Element
         * 
         * @param elt the element to document
         * @param newDoc the documentation text
         */
        protected void changeDocumentation(EModelElement elt, String newDoc)
        {
            EAnnotation annotation = elt.getEAnnotation(annotationSource);
            if (newDoc != null && !"".equals(newDoc))
            {
                // creates EAnnotation if needed
                if (annotation == null)
                {
                    annotation = EcoreFactory.eINSTANCE.createEAnnotation();
                    annotation.setSource(annotationSource);
                    elt.getEAnnotations().add(annotation);
                }

                annotation.getDetails().put(annotationKey, newDoc);
            }
            else
            {
                // remove the documentation
                if (annotation != null)
                {
                    annotation.getDetails().remove(annotationKey);

                    // remove the EAnnotation if empty
                    if (annotation.getDetails().size() == 0)
                    {
                        elt.getEAnnotations().remove(annotation);
                    }
                }
            }
        }

        /**
         * @see org.eclipse.gef.commands.Command#redo()
         */
        public void redo()
        {
            changeDocumentation(element, newComments);
        }

        /**
         * @see org.eclipse.gef.commands.Command#undo()
         */
        public void undo()
        {
            changeDocumentation(element, oldComments);
        }
    }
}
