/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), 
 *    Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies),
 *    Thomas Friol (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.documentation;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.Page;

/**
 * This class creates the page to edit documentation EAnnotation of the edited
 * Ecore model.<br>
 * <br>
 * creation : 3 juin 2005
 * 
 * @author <a href="mailto:david@anyware-tech.com">David Sciamma</a>
 */
public class EAnnotationDocPage extends Page implements IDocPage
{
    private EModelElement documentedElement;

    private CommandStack stack;

    // SWT Widgets
    private Composite mainComp;

    private EAnnotationCommentsComposite commentsComposite;

    private EAnnotationResourcesComposite resourcesComposite;

    /**
     * Build the page with the editor command stack : used to execute commands.
     * 
     * @param stack the editor command stack
     */
    public EAnnotationDocPage(CommandStack stack)
    {
        this.stack = stack;
    }

    /**
     * Returns the editor command stack
     * 
     * @return the command stack
     */
    protected CommandStack getCommandStack()
    {
        return stack;
    }

    /**
     * @see org.eclipse.ui.part.Page#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        mainComp = new Composite(parent, SWT.NONE);
        mainComp.setLayout(new GridLayout());
        mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));

        TabFolder tabFolder = new TabFolder(mainComp, SWT.TOP);
        tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));

        createTabItems(tabFolder);
    }

    /**
     * This method creates the different documentation tab items
     * 
     * @param tabFolder the parent tabFolder
     */
    protected void createTabItems(TabFolder tabFolder)
    {
        createCommentsTabItem(tabFolder);

        createResourcesTabItem(tabFolder);
    }

    /**
     * Create the content of the comments tab item. It contains a multi lines
     * text field in order to edit the comments of the documentation.
     * 
     * @param parent the parent tab folder
     */
    private void createCommentsTabItem(TabFolder parent)
    {
        TabItem tabItem = new TabItem(parent, SWT.NONE);
        tabItem.setText("Comments");

        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());
        container.setLayoutData(new GridData(GridData.FILL_BOTH));

        tabItem.setControl(container);

        commentsComposite = new EAnnotationCommentsComposite(container, SWT.NONE, stack);
        commentsComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
    }

    /**
     * Create the content of the resources tab item. It contains a list and
     * three buttons in order to view, add, remove and edit linked resources.
     * 
     * @param parent the parent tab folder
     */
    private void createResourcesTabItem(TabFolder parent)
    {
        TabItem tabItem = new TabItem(parent, SWT.NONE);
        tabItem.setText("Resources");

        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());
        container.setLayoutData(new GridData(GridData.FILL_BOTH));

        tabItem.setControl(container);

        resourcesComposite = new EAnnotationResourcesComposite(container, SWT.NONE, stack);
        resourcesComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
    }

    public void setModelElement(EModelElement elt)
    {
        if (elt  == null || elt != documentedElement)
        {
            documentedElement = elt;
            commentsComposite.setModelElement(documentedElement);
            resourcesComposite.setModelElement(documentedElement);
        }
    }

    /**
     * @see org.eclipse.ui.part.Page#getControl()
     */
    public Control getControl()
    {
        return mainComp;
    }

    /**
     * @see org.eclipse.ui.part.Page#setFocus()
     */
    public void setFocus()
    {
        // Do nothing
    }

    private EModelElement getSelectedModelElement(ISelection selection)
    {

        if (selection instanceof IStructuredSelection && ((IStructuredSelection) selection).size() == 1)
        {
            Object selectedObject = ((IStructuredSelection) selection).getFirstElement();

            if (selectedObject instanceof EModelElement)
            {
                return (EModelElement) selectedObject;
            }
            if (selectedObject instanceof IAdaptable)
            {
                Object adaptedObject = ((IAdaptable) selectedObject).getAdapter(EObject.class);
                if (adaptedObject != null && adaptedObject instanceof EModelElement)
                {
                    return (EModelElement) adaptedObject;
                }
            }

            Object adaptedObject = Platform.getAdapterManager().getAdapter(selectedObject, EObject.class);
            if (adaptedObject != null && adaptedObject instanceof EModelElement)
            {
                return (EModelElement) adaptedObject;
            }
        }

        return null;
    }

    /**
     * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection)
    {
        EModelElement elt = getSelectedModelElement(selection);
        setModelElement(elt);
    }
}
