/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.edit.policies;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.DirectEditPolicy;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.jface.viewers.CellEditor;
import org.topcased.modeler.commands.ChangeLabelTextCommand;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.providers.ILabelFeatureProvider;

/**
 * Manage the DirectEditPolicy for a Label in the Editing Window
 * 
 * @author jako
 */
public class LabelDirectEditPolicy extends DirectEditPolicy
{
    private String oldValue;

    private EObject modelObject = null;

    /**
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#getDirectEditCommand(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected Command getDirectEditCommand(DirectEditRequest request)
    {
        EObject eObject = modelObject;

        if (eObject == null)
        {
            if (getHost().getModel() instanceof GraphNode)
            {
                GraphNode graphNode = (GraphNode) getHost().getModel();
                EMFSemanticModelBridge e = (EMFSemanticModelBridge) graphNode.getSemanticModel();
                eObject = e.getElement();
            }
            else if (getHost().getModel() instanceof GraphEdge)
            {
                GraphEdge graphEdge = (GraphEdge) getHost().getModel();
                EMFSemanticModelBridge e = (EMFSemanticModelBridge) graphEdge.getSemanticModel();
                eObject = e.getElement();
            }
            else
            {
                ModelerPlugin.log("The model object is neither a GraphNode nor a GraphEdge. Unable to create command.",
                        IStatus.ERROR);
                return UnexecutableCommand.INSTANCE;
            }
        }

        if (eObject == null)
        {
            ModelerPlugin.log("Current model object is null. Unable to create command.", IStatus.ERROR);
            return UnexecutableCommand.INSTANCE;
        }

        ILabelFeatureProvider featureProvider = (ILabelFeatureProvider) ((MixedEditDomain) getHost().getViewer().getEditDomain()).getAdapterFactory().adapt(
                eObject, ILabelFeatureProvider.class);
        EAttribute eAttribute = null;
        if (featureProvider != null)
        {
            eAttribute = featureProvider.getLabelFeature(eObject);

            if (eAttribute != null)
            {
                CellEditor cellEditor = request.getCellEditor();
                ChangeLabelTextCommand command = new ChangeLabelTextCommand(eObject, eAttribute);
                command.setOldName(oldValue);
                command.setName((String) cellEditor.getValue());
                return command;
            }
        }

        return UnexecutableCommand.INSTANCE;

    }

    /**
     * @see org.eclipse.gef.editpolicies.DirectEditPolicy#showCurrentEditValue(org.eclipse.gef.requests.DirectEditRequest)
     */
    protected void showCurrentEditValue(DirectEditRequest request)
    {
        String value = (String) request.getCellEditor().getValue();
        if (getHost() instanceof GraphNodeEditPart)
        {
            GraphNodeEditPart graphNodeEditPart = (GraphNodeEditPart) getHost();
            graphNodeEditPart.handleNameChange(value);
        }
    }

    /**
     * Saves the initial text value so that if the user's changes are not
     * committed then
     * 
     * @param request
     */
    protected void storeOldEditValue(DirectEditRequest request)
    {
        oldValue = (String) request.getCellEditor().getValue();
    }

    /**
     * Set the model Object that handle the LabelFeature
     * 
     * @param obj the model object
     */
    public void setModelObject(EObject obj)
    {
        this.modelObject = obj;
    }

}
