/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.editor;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.topcased.modeler.commands.INonDirtyingCommand;
import org.topcased.modeler.di.model.Diagram;

/**
 * This class is responsible of the navigation throught the diagrams. Then it
 * stores the history of this navigation to allow next and previous actions.
 * <br>
 * creation : 16 déc. 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class NavigationManager
{
    private Modeler modeler;

    private int previousLimit = 30;

    private Stack previousDiagrams = new Stack();

    private Stack nextDiagrams = new Stack();

    private List eventListeners = new ArrayList();

    /**
     * Constructor
     * 
     * @param editor the modeler
     */
    NavigationManager(Modeler editor)
    {
        super();
        this.modeler = editor;
    }

    /**
     * Appends the listener to the list of navigation listeners. Multiple adds
     * result in multiple notifications.
     * 
     * @param listener the event listener
     */
    public void addNavigationListener(INavigationListener listener)
    {
        eventListeners.add(listener);
    }

    /**
     * Removes the first occurrence of the specified listener.
     * 
     * @param listener the listener
     */
    public void removeNavigationListener(INavigationListener listener)
    {
        eventListeners.remove(listener);
    }

    private void notifyListeners(Diagram newDiagram)
    {
        for (int i = 0; i < eventListeners.size(); i++)
        {
            ((INavigationListener) eventListeners.get(i)).diagramChanged(newDiagram);
        }
    }

    /**
     * Returns <code>true</code> if a previous action can be performed
     * 
     * @return a boolean
     */
    public boolean canDoPrevious()
    {
        return !previousDiagrams.isEmpty();
    }

    /**
     * Returns <code>true</code> if a next action can be performed
     * 
     * @return a boolean
     */
    public boolean canDoNext()
    {
        return !nextDiagrams.isEmpty();
    }

    /**
     * Go backward in the browse history
     */
    public void previous()
    {
        Diagram oldDiagram = modeler.getActiveDiagram();

        // Add a new browse command to the editor command stack
        // Change the diagram
        Diagram prevDiagram = (Diagram) previousDiagrams.pop();
        getEditorCommandStack().execute(createNavigateCommand(prevDiagram));

        // Change the position in the browse stack
        nextDiagrams.add(oldDiagram);

        // Send event
        notifyListeners(prevDiagram);
    }

    /**
     * Go forward in the browse history
     */
    public void next()
    {
        Diagram oldDiagram = modeler.getActiveDiagram();
        // Add a new browse command to the editor command stack
        // Change the diagram
        Diagram nextDiagram = (Diagram) nextDiagrams.pop();
        getEditorCommandStack().execute(createNavigateCommand(nextDiagram));

        // Change the position in the browse stack
        previousDiagrams.add(oldDiagram);

        // Send event
        notifyListeners(nextDiagram);
    }

    /**
     * Creates the naviagtion command for the given diagram
     * 
     * @param newDiagram the destination diagram
     * @return the navigation command
     */
    private Command createNavigateCommand(Diagram newDiagram)
    {
        return new ChangeActiveDiagramCommand(modeler, newDiagram);
    }

    /**
     * Change the current diagram
     * 
     * @param newDiagram the new active diagram
     */
    public void set(Diagram newDiagram)
    {
        Diagram oldDiagram = modeler.getActiveDiagram();
        if (oldDiagram != newDiagram || (oldDiagram == null && newDiagram == null))
        {
            // Add a new browse command to the editor command stack
            // Change the diagram
            getEditorCommandStack().execute(createNavigateCommand(newDiagram));

            // Clear the next stack
            nextDiagrams.clear();

            // Add a new previous diagram
            addPreviousDiagram(oldDiagram);

            // Send event
            notifyListeners(newDiagram);
        }
    }

    private void addPreviousDiagram(Diagram diag)
    {
        if (diag != null)
        {
            if (getPreviousLimit() > 0)
            {
                while (previousDiagrams.size() >= getPreviousLimit())
                {
                    previousDiagrams.remove(0);
                }
            }
            previousDiagrams.push(diag);
        }
    }

    /**
     * Returns the depth of the browse stack
     * 
     * @return the limit
     */
    public int getPreviousLimit()
    {
        return previousLimit;
    }

    /**
     * Returns the command stack of the modeler
     * 
     * @return the command stack
     */
    private CommandStack getEditorCommandStack()
    {
        return ((CommandStack) modeler.getAdapter(CommandStack.class));
    }

    /**
     * The GEF command used to store the navigation actions in the editor's
     * command stack <br>
     * creation : 16 déc. 2005
     * 
     * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
     */
    private class ChangeActiveDiagramCommand extends Command implements INonDirtyingCommand
    {
        private Modeler editor;

        private Diagram oldDiagram;

        private Diagram newDiagram;

        /**
         * Constructor
         * 
         * @param ed the Modeler
         * @param diag the new active diagram
         */
        public ChangeActiveDiagramCommand(Modeler ed, Diagram diag)
        {
            super("Create Diagram");
            this.editor = ed;
            this.newDiagram = diag;
        }

        /**
         * @see org.eclipse.gef.commands.Command#canExecute()
         */
        public boolean canExecute()
        {
            return editor != null;
        }

        /**
         * @see org.eclipse.gef.commands.Command#canUndo()
         */
        public boolean canUndo()
        {
            return editor != null;
        }

        /**
         * @see org.eclipse.gef.commands.Command#execute()
         */
        public void execute()
        {
            oldDiagram = editor.getActiveDiagram();
            redo();
        }

        /**
         * @see org.eclipse.gef.commands.Command#redo()
         */
        public void redo()
        {
            editor.changeActiveDiagram(newDiagram);
        }

        /**
         * @see org.eclipse.gef.commands.Command#undo()
         */
        public void undo()
        {
            editor.changeActiveDiagram(oldDiagram);
        }
    }
}
