/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.editor;

import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.topcased.modeler.editor.outline.OutlineToDiagramTransfer;
import org.topcased.modeler.exceptions.BoundsFormatException;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.tools.Importer;

/**
 * Listener that manage the outline to diagram dragging. <br>
 * creation : 7 dec. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public class OutlineDropListener extends AbstractTransferDropTargetListener
{
    private Modeler modeler;

    private DropFactory factory;

    private Importer importer;

    /**
     * Constructor
     * 
     * @param viewer the GraphicalViewer of the editing window
     * @param mod the Topcased editor
     */
    public OutlineDropListener(GraphicalViewer viewer, Modeler mod)
    {
        super(viewer, OutlineToDiagramTransfer.getInstance());
        this.modeler = mod;
    }

    /**
     * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#getTargetRequest()
     */
    protected Request getTargetRequest()
    {
        List transfer = (List) ((OutlineToDiagramTransfer) getTransfer()).getObject();
        importer = new Importer(modeler, transfer);
        return importer.getCreateRequest();
    }

    /**
     * Called whenever the User drags over the target. By default, the target
     * Request and target EditPart are updated, feedback is shown, and
     * auto-expose occurs.
     * 
     * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#handleDragOver()
     */
    protected void handleDragOver()
    {
        getCurrentEvent().detail = DND.DROP_COPY;
        super.handleDragOver();
    }

    /**
     * Overridden to select the created object.
     * 
     * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#handleDrop()
     */
    protected void handleDrop()
    {
        if (getTargetEditPart() instanceof GraphicalEditPart)
        {
            try
            {
                importer.setLocation(getDropLocation());
                importer.setTargetEditPart((GraphicalEditPart) getTargetEditPart());
                importer.run(new NullProgressMonitor());
            }
            catch (BoundsFormatException bfe)
            {
                ModelerPlugin.displayDialog("Drag'n Drop Problem", "A problem occured during drag'n drop :\n"
                        + bfe.getMessage(), IStatus.ERROR);
            }
            catch (InterruptedException e)
            {
                ModelerPlugin.displayDialog("Operation Canceled", "The import was stopped.", IStatus.INFO);
            }
        }
    }

    /**
     * Over-ridden so that the cursor can be updated to "disallow" when the
     * command is not executable
     * 
     * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#isEnabled(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public boolean isEnabled(DropTargetEvent event)
    {
        boolean result = super.isEnabled(event);
        if (result)
        {
            updateTargetRequest();
            updateTargetEditPart();
            Command cmd = getCommand();
            result = cmd != null && cmd.canExecute();
        }
        return result;
    }

    /**
     * Update the Request
     * 
     * @see org.eclipse.gef.dnd.AbstractTransferDropTargetListener#updateTargetRequest()
     */
    protected void updateTargetRequest()
    {
        CreateRequest req = (CreateRequest) getTargetRequest();
        req.setLocation(getDropLocation());
    }
}
