/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.editor.properties.sections;

import java.util.Iterator;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.widgets.CSingleObjectChooser;

/**
 * An abstract implementation of a section with a field using a
 * CSingleObjectChooser composite (CCombo with a Button).
 * 
 * @author Jacques LESCOT
 */
public abstract class AbstractComboPropertySection extends AbstractModelerPropertySection
{
    /**
     * A boolean that store if refreshing is happening and no model
     * modifications should be performed
     */
    protected boolean isRefreshing = false;

    /**
     * The combo box control for the section.
     */
    protected CSingleObjectChooser cSingleObjectChooser;

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#createControls(org.eclipse.swt.widgets.Composite,
     *      org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
     */
    public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage)
    {
        super.createControls(parent, aTabbedPropertySheetPage);
        Composite composite = getWidgetFactory().createFlatFormComposite(parent);
        FormData data;

        CLabel labelCombo = getWidgetFactory().createCLabel(composite, getLabelText());

        cSingleObjectChooser = new CSingleObjectChooser(composite, getWidgetFactory(), SWT.NONE);
        cSingleObjectChooser.setLabelProvider(getLabelProvider());

        data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(cSingleObjectChooser, -ITabbedPropertyConstants.HSPACE);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        labelCombo.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0, getStandardLabelWidth(composite, new String[] {getLabelText()}));
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(labelCombo, 0, SWT.CENTER);
        cSingleObjectChooser.setLayoutData(data);

        hookListeners();
    }

    /**
     * Adds the listeners on the widgets
     */
    protected void hookListeners()
    {
        cSingleObjectChooser.addSelectionListener(new SelectionAdapter()
        {
            /**
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e)
            {
                handleComboModified();
            }
        });
    }

    /**
     * Handle the combo modified event.
     */
    protected void handleComboModified()
    {
        if (!isRefreshing && getFeatureValue() != cSingleObjectChooser.getSelection())
        {
            EditingDomain editingDomain = ((Modeler) getPart().getAdapter(Modeler.class)).getEditingDomain();
            if (!getFeature().isMany())
            {
                if (eObjectList.size() == 1)
                {
                    /* apply the property change to single selected object */
                    editingDomain.getCommandStack().execute(
                            SetCommand.create(editingDomain, eObject, getFeature(), cSingleObjectChooser.getSelection()));
                }
                else
                {
                    CompoundCommand compoundCommand = new CompoundCommand();
                    /* apply the property change to all selected elements */
                    for (Iterator i = eObjectList.iterator(); i.hasNext();)
                    {
                        EObject nextObject = (EObject) i.next();
                        compoundCommand.append(SetCommand.create(editingDomain, nextObject, getFeature(),
                                cSingleObjectChooser.getSelection()));
                    }
                    editingDomain.getCommandStack().execute(compoundCommand);
                }
            }
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#refresh()
     */
    public void refresh()
    {
        isRefreshing = true;

        Object[] results = getComboFeatureValues();
        cSingleObjectChooser.setChoices(results);
        cSingleObjectChooser.setSelection(getFeatureValue());
        
        isRefreshing = false;
    }

    /**
     * Get the LabelProvider to use to display the Object contained in the
     * ComboViewer
     * 
     * @return IBaseLabelProvider
     */
    protected abstract ILabelProvider getLabelProvider();

    /**
     * Get the feature for the combo field for the section.
     * 
     * @return the feature for the text.
     */
    protected abstract EStructuralFeature getFeature();

    /**
     * Get the current feature value of the selected model object.
     * 
     * @return the feature value to select in the ccombo.
     */
    protected abstract Object getFeatureValue();

    /**
     * Get the enumeration values of the feature for the combo field for the
     * section.
     * 
     * @return the list of values of the feature as text.
     */
    protected abstract Object[] getComboFeatureValues();

    /**
     * Get the label for the combo field for the section.
     * 
     * @return the label for the text field.
     */
    protected abstract String getLabelText();
}