/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.editor.properties.sections.graphics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ItemProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.topcased.modeler.commands.RecordingChangeCommand;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.EdgeObject;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.editor.Modeler;

/**
 * The EdgeObject visibility section on the graphics tab. Here you can choose to show/hide an EdgeObject
 * 
 * Creation 13 févr. 07
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class EdgeObjectVisibilityPropertySection extends AbstractGraphicPropertySection
{
    private ILabelProvider labelProvider;

    private TableViewer visibleElementsTableViewer;

    private TableViewer hiddenElementsTableViewer;

    private Button hideButton;

    private Button showButton;

    private IDoubleClickListener visibleElementsTableDoubleClickListener = new IDoubleClickListener()
    {
        public void doubleClick(DoubleClickEvent event)
        {
            if (hideButton.isEnabled())
            {
                hideButton.notifyListeners(SWT.Selection, null);
            }
        }
    };

    private IDoubleClickListener hiddenElementsTableDoubleClickListener = new IDoubleClickListener()
    {
        public void doubleClick(DoubleClickEvent event)
        {
            if (showButton.isEnabled())
            {
                showButton.notifyListeners(SWT.Selection, null);
            }
        }
    };

    private SelectionAdapter hideButtonSelectionAdapter = new SelectionAdapter()
    {
        // event is null when availableElementsTableViewer is double clicked
        public void widgetSelected(SelectionEvent event)
        {
            if (visibleElementsTableViewer != null)
            {
                final List newElements = new ArrayList();
                IStructuredSelection selection = (IStructuredSelection) visibleElementsTableViewer.getSelection();
                for (Iterator i = selection.iterator(); i.hasNext();)
                {
                    Object value = i.next();

                    ((ItemProvider) hiddenElementsTableViewer.getInput()).getChildren().add(value);
                    ((ItemProvider) visibleElementsTableViewer.getInput()).getChildren().remove(value);
                    newElements.add(value);
                }

                Command cmd = new RecordingChangeCommand(getDiagElt().eResource())
                {
                    /**
                     * @see org.topcased.modeler.commands.RecordingChangeCommand#doExecute()
                     */
                    protected void doExecute()
                    {
                        for (Iterator itNewElements = newElements.iterator(); itNewElements.hasNext();)
                        {
                            ((EdgeObject) itNewElements.next()).setVisible(false);
                        }
                    }

                    /**
                     * This command is not Undoable. The Unapply Stereotype action should be used instead.
                     * 
                     * @see org.topcased.modeler.commands.RecordingChangeCommand#canUndo()
                     */
                    public boolean canUndo()
                    {
                        return true;
                    }
                };

                if (cmd != null && cmd.canExecute())
                {
                    ((CommandStack) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(CommandStack.class)).execute(cmd);
                }
            }
        }
    };

    private SelectionAdapter showButtonSelectionAdapter = new SelectionAdapter()
    {
        // event is null when selectedElementsTableViewer is double clicked
        public void widgetSelected(SelectionEvent event)
        {
            IStructuredSelection selection = (IStructuredSelection) hiddenElementsTableViewer.getSelection();
            Object firstValue = null;

            final List oldElements = new ArrayList();
            for (Iterator i = selection.iterator(); i.hasNext();)
            {
                Object value = i.next();
                if (firstValue == null)
                {
                    firstValue = value;
                }

                ((ItemProvider) visibleElementsTableViewer.getInput()).getChildren().add(value);
                ((ItemProvider) hiddenElementsTableViewer.getInput()).getChildren().remove(value);

                oldElements.add(value);
            }

            Command cmd = new RecordingChangeCommand(getDiagElt().eResource())
            {
                /**
                 * @see org.topcased.modeler.commands.RecordingChangeCommand#doExecute()
                 */
                protected void doExecute()
                {
                    for (Iterator itOldElements = oldElements.iterator(); itOldElements.hasNext();)
                    {
                        ((EdgeObject) itOldElements.next()).setVisible(true);
                    }
                }

                /**
                 * This command is not Undoable. The Unapply Stereotype action should be used instead.
                 * 
                 * @see org.topcased.modeler.commands.RecordingChangeCommand#canUndo()
                 */
                public boolean canUndo()
                {
                    return false;
                }
            };

            if (cmd != null && cmd.canExecute())
            {
                ((CommandStack) ((Modeler) getPart().getAdapter(Modeler.class)).getAdapter(CommandStack.class)).execute(cmd);
            }
        }
    };

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#createControls(org.eclipse.swt.widgets.Composite,
     *      org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
     */
    public void createControls(Composite parent, TabbedPropertySheetPage aTabbedPropertySheetPage)
    {
        super.createControls(parent, aTabbedPropertySheetPage);
        Composite composite = getWidgetFactory().createFlatFormComposite(parent);
        composite.setLayout(new GridLayout(3, false));

        Composite choiceComposite = getWidgetFactory().createComposite(composite, SWT.NONE);
        choiceComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        choiceComposite.setLayout(new GridLayout());

        Label choiceLabel = getWidgetFactory().createLabel(choiceComposite, "Visible Elements", SWT.NONE);
        choiceLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        final Table choiceTable = getWidgetFactory().createTable(choiceComposite, SWT.MULTI | SWT.BORDER);
        choiceTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        visibleElementsTableViewer = new TableViewer(choiceTable);

        Composite controlButtons = getWidgetFactory().createComposite(composite, SWT.NONE);
        controlButtons.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));

        GridLayout controlsButtonGridLayout = new GridLayout();
        controlButtons.setLayout(controlsButtonGridLayout);

        new Label(controlButtons, SWT.NONE);

        hideButton = getWidgetFactory().createButton(controlButtons, "Hide >", SWT.PUSH);
        hideButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));

        showButton = getWidgetFactory().createButton(controlButtons, "< Show", SWT.PUSH);
        showButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));

        Label spaceLabel = new Label(controlButtons, SWT.NONE);
        GridData spaceLabelGridData = new GridData();
        spaceLabelGridData.verticalSpan = 2;
        spaceLabel.setLayoutData(spaceLabelGridData);

        Composite featureComposite = getWidgetFactory().createComposite(composite, SWT.NONE);
        featureComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        featureComposite.setLayout(new GridLayout());

        Label featureLabel = getWidgetFactory().createLabel(featureComposite, "Hidden Elements", SWT.NONE);
        featureLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));

        final Table featureTable = getWidgetFactory().createTable(featureComposite, SWT.MULTI | SWT.BORDER);
        featureTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        hiddenElementsTableViewer = new TableViewer(featureTable);
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#setInput(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void setInput(IWorkbenchPart part, ISelection selection)
    {
        super.setInput(part, selection);

        AdapterFactory adapterFactory = new ComposedAdapterFactory(Collections.EMPTY_LIST);
        visibleElementsTableViewer.setLabelProvider(getLabelProvider());
        visibleElementsTableViewer.setContentProvider(new AdapterFactoryContentProvider(adapterFactory));
        visibleElementsTableViewer.setInput(new ItemProvider(adapterFactory, getVisibleElements()));

        AdapterFactory adapterFactory2 = new ComposedAdapterFactory(Collections.EMPTY_LIST);
        hiddenElementsTableViewer.setLabelProvider(getLabelProvider());
        hiddenElementsTableViewer.setContentProvider(new AdapterFactoryContentProvider(adapterFactory2));
        hiddenElementsTableViewer.setInput(new ItemProvider(adapterFactory2, getHiddenElements()));
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.ISection#aboutToBeShown()
     */
    public void aboutToBeShown()
    {
        super.aboutToBeShown();

        visibleElementsTableViewer.addDoubleClickListener(visibleElementsTableDoubleClickListener);
        hiddenElementsTableViewer.addDoubleClickListener(hiddenElementsTableDoubleClickListener);

        hideButton.addSelectionListener(hideButtonSelectionAdapter);
        showButton.addSelectionListener(showButtonSelectionAdapter);
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#aboutToBeHidden()
     */
    public void aboutToBeHidden()
    {
        super.aboutToBeHidden();

        visibleElementsTableViewer.removeDoubleClickListener(visibleElementsTableDoubleClickListener);
        hiddenElementsTableViewer.removeDoubleClickListener(hiddenElementsTableDoubleClickListener);

        if (!hideButton.isDisposed())
        {
            hideButton.removeSelectionListener(hideButtonSelectionAdapter);
            showButton.removeSelectionListener(showButtonSelectionAdapter);
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#shouldUseExtraSpace()
     */
    public boolean shouldUseExtraSpace()
    {
        return true;
    }

    /**
     * Get the LabelProvider to use to display the Object
     * 
     * @return ILabelProvider
     */
    protected ILabelProvider getLabelProvider()
    {
        if (labelProvider == null)
        {
            labelProvider = new AdapterFactoryLabelProvider(((Modeler) getPart().getAdapter(Modeler.class)).getAdapterFactory())
            {
                public String getColumnText(Object object, int columnIndex)
                {
                    return super.getColumnText(object, columnIndex);
                    // IItemQualifiedTextProvider itemQualifiedTextProvider = (IItemQualifiedTextProvider)
                    // adapterFactory.adapt(object, IItemQualifiedTextProvider.class);
                    //
                    // return itemQualifiedTextProvider != null ? itemQualifiedTextProvider.getQualifiedText(object) :
                    // super.getColumnText(object, columnIndex);
                }

                public String getText(Object object)
                {
                    return super.getText(object);
                    // IItemQualifiedTextProvider itemQualifiedTextProvider = (IItemQualifiedTextProvider)
                    // adapterFactory.adapt(object, IItemQualifiedTextProvider.class);
                    //
                    // return itemQualifiedTextProvider != null ? itemQualifiedTextProvider.getQualifiedText(object) :
                    // super.getText(object);
                }
            };
        }

        return labelProvider;
    }

    /**
     * Get the list of visible EdgeObjects for the current selected GraphElement.
     * 
     * @return List the list of EdgeObjects that are visible
     */
    protected List getVisibleElements()
    {
        List choiceOfValues = new ArrayList();

        for (Iterator itContainedElements = ((GraphElement) getDiagElt()).getContained().iterator(); itContainedElements.hasNext();)
        {
            DiagramElement diagElt = (DiagramElement) itContainedElements.next();
            if (diagElt instanceof EdgeObject && ((EdgeObject) diagElt).isVisible())
            {
                choiceOfValues.add(diagElt);
            }
        }

        return choiceOfValues;
    }

    /**
     * Get the list of EdgeObjects contained by the current selected GraphElement which are hidden.
     * 
     * @return List the list of EdgeObjects that are not visible
     */
    protected List getHiddenElements()
    {
        List choiceOfValues = new ArrayList();

        for (Iterator itContainedElements = ((GraphElement) getDiagElt()).getContained().iterator(); itContainedElements.hasNext();)
        {
            DiagramElement diagElt = (DiagramElement) itContainedElements.next();
            if (diagElt instanceof EdgeObject && !((EdgeObject) diagElt).isVisible())
            {
                choiceOfValues.add(diagElt);
            }
        }

        return choiceOfValues;
    }

    /**
     * @see org.topcased.modeler.editor.properties.sections.graphics.AbstractGraphicPropertySection#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    protected void handleModelChanged(Notification msg)
    {
        // Do nothing

    }

}
