/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.export.internal;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.OutputStream;

import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.ImageTranscoder;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gmf.runtime.draw2d.ui.render.awt.internal.graphics.GraphicsToGraphics2DAdaptor;
import org.eclipse.swt.graphics.Rectangle;
import org.topcased.modeler.export.ExportException;
import org.topcased.modeler.export.IImageExporter;

/**
 * This abstract class defines basic actions for exporters that uses AWT classes to make the conversion.
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public abstract class AbstractAWTImageExporter implements IImageExporter
{

    /**
     * Constructor
     */
    public AbstractAWTImageExporter()
    {
        super();
    }

    /**
     * @see org.topcased.modeler.export.IImageExporter#export(org.eclipse.draw2d.IFigure, java.io.OutputStream)
     */
    public void export(IFigure figure, OutputStream os) throws ExportException
    {

        java.awt.Image image = createAWTImage(figure);
        save(os, (BufferedImage) image);
    }

    /**
     * Render a GEF figure into an AWT image
     * 
     * @param figure the gef figure
     * @return the image
     */
    protected java.awt.Image createAWTImage(IFigure figure)
    {
        if (figure instanceof Viewport)
        {
            // Reinit the figure
            ((Viewport) figure).setViewLocation(0, 0);
        }

        Dimension size = figure.getPreferredSize();

        BufferedImage awtImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_4BYTE_ABGR_PRE);

        Graphics2D g2d = awtImage.createGraphics();
        g2d.setColor(Color.white);
        g2d.fillRect(0, 0, awtImage.getWidth(), awtImage.getHeight());

        // TODO Check anti-aliasing preference
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        g2d.clip(new java.awt.Rectangle(0, 0, awtImage.getWidth(), awtImage.getHeight()));

        Graphics graphics = new GraphicsToGraphics2DAdaptor(g2d, new Rectangle(0, 0, size.width, size.height));

        figure.paint(graphics);

        graphics.dispose();
        g2d.dispose();

        return awtImage;

    }

    /**
     * Saves the given image into the output stream using the image transcoder defined by the subclass
     * 
     * @param os the output stream where the image is serialized
     * @param image the AWT ilage
     * @throws ExportException thrown if an error occured during transcoding
     */
    protected void save(OutputStream os, BufferedImage image) throws ExportException
    {

        ImageTranscoder imageTranscoder = createImageTranscoder();
        BufferedImage preparedImage = preprocessImage(image);

        TranscoderOutput to = new TranscoderOutput(os);

        try
        {
            imageTranscoder.writeImage(preparedImage, to);
        }
        catch (TranscoderException te)
        {
            throw new ExportException(te);
        }
    }

    /**
     * <b>This method can be overriden by subclasses to preprocess the image.</b> : initialize some values...
     * 
     * @param image the basic image
     * @return the corrected image
     */
    protected BufferedImage preprocessImage(BufferedImage image)
    {
        return image;
    }

    /**
     * Creates the image transcoder used to serialized the image
     * 
     * @return the transcoder
     */
    protected abstract ImageTranscoder createImageTranscoder();
}