/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.export.internal.ui;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.export.ExportException;
import org.topcased.modeler.export.ExporterDescriptor;
import org.topcased.modeler.export.ExporterManager;
import org.topcased.modeler.export.internal.Activator;

/**
 * This action displays the export dialog and save the diagram into a file. <br>
 * creation : 17 mai 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class Export2FileAction implements IEditorActionDelegate
{
    private static File lastDirectory;

    private static String lastExtension;

    private Modeler modeler;

    /**
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action)
    {
        FileOutputStream fos = null;
        try
        {
            GraphicalViewer viewer = (GraphicalViewer) modeler.getAdapter(GraphicalViewer.class);
            IFigure figure = ((AbstractGraphicalEditPart) viewer.getRootEditPart()).getFigure();

            File file = getFile(modeler.getSite().getShell());
            if (checkFile(file))
            {
                fos = new FileOutputStream(file);

                ExporterDescriptor descriptor = getExporter(file);
                if (descriptor != null)
                {
                    descriptor.getExporter().export(figure, fos);

                    // Remember values
                    lastDirectory = file.getParentFile();
                    lastExtension = descriptor.getExtension();

                    Activator.displayDialog(null, "Export done to file '" + file.getName() + "'", IStatus.INFO);
                }
                else
                {
                    Activator.displayDialog(null, "Unable to find exporter for file '" + file.getName() + "'", IStatus.WARNING);
                }
            }
        }
        catch (IOException ioe)
        {
            Activator.displayDialog(null, "An error occured during export. See the error log for more details.", IStatus.ERROR);
            Activator.log(ioe);
        }
        catch (ExportException ee)
        {
            Activator.displayDialog(null, "An error occured during export. See the error log for more details.", IStatus.ERROR);
            Activator.log(ee);
        }
        catch (CoreException ce)
        {
            Activator.displayDialog(null, "An error occured during export. See the error log for more details.", IStatus.ERROR);
            Activator.log(ce);
        }
        finally
        {
            try
            {
                if (fos != null)
                {
                    fos.close();
                }
            }
            catch (IOException ioe)
            {
                // Ignore, Do nothing
                Activator.log(ioe);
            }
        }
    }

    private boolean checkFile(File file) throws IOException
    {
        if (file != null)
        {
            if (file.exists())
            {
                if (!MessageDialog.openQuestion(modeler.getSite().getShell(), "Overwrite", "The file already exists. Do you really want to overwrite it ?"))
                {
                    return false;
                }
            }
            else
            {
                file.createNewFile();
            }

            return true;
        }

        return false;
    }

    private File getFile(Shell shell)
    {
        FileDialog dialog = new FileDialog(shell, SWT.SAVE);

        dialog.setText("Select The exported file : ");
        dialog.setFilterExtensions(getExtensions());
        if (lastDirectory != null)
        {
            dialog.setFilterPath(lastDirectory.getAbsolutePath());
        }

        String filepath = dialog.open();
        if (filepath != null && filepath.length() > 0)
        {
            return new File(filepath);
        }

        return null;
    }

    /**
     * Returns the exporter associated with the given file
     * 
     * @param f the file
     * @return the exporter descriptor
     */
    protected ExporterDescriptor getExporter(File f)
    {
        ExporterDescriptor foundExporter = null;
        ExporterDescriptor[] descriptors = ExporterManager.getInstance().getExporters();

        for (int i = 0; i < descriptors.length && foundExporter == null; i++)
        {
            if (f.getName().endsWith(descriptors[i].getExtension()))
            {
                foundExporter = descriptors[i];
            }
        }
        return foundExporter;
    }

    /**
     * Returns the extensions handled by the registered exporters
     * 
     * @return the list of extensions
     */
    protected String[] getExtensions()
    {
        ExporterDescriptor[] descriptors = ExporterManager.getInstance().getExporters();
        String[] extensions = new String[descriptors.length];

        for (int i = 0; i < descriptors.length; i++)
        {
            ExporterDescriptor descriptor = descriptors[i];

            extensions[i] = "*." + descriptor.getExtension();
        }

        // Set the last extension first
        if (lastExtension != null)
        {
            String last = "*." + lastExtension;
            boolean done = false;
            for (int j = 0; j < extensions.length && !done; j++)
            {
                if (last.equals(extensions[j]))
                {
                    extensions[j] = extensions[0];
                    extensions[0] = last;
                    done = true;
                }
            }
        }

        return extensions;
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection)
    {
        // Do nothing
    }

    /**
     * @see org.eclipse.ui.IEditorActionDelegate#setActiveEditor(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IEditorPart)
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor)
    {
        if (targetEditor instanceof Modeler)
        {
            this.modeler = (Modeler) targetEditor;
            action.setEnabled(true);
        }
        else
        {
            action.setEnabled(false);
        }
    }
}
