/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.export.internal.ui;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.ide.IDE;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.export.ExportException;
import org.topcased.modeler.export.internal.Activator;

/**
 * Action that export all the diagrams as images at the same location of the selected file. Exported images are named
 * with the diagram name and JPG format is used by default. These images are then used in the generation of the
 * documentation when needed.
 * 
 * Additional work would consist in :
 * - give the user the choice of the output format
 * - create a hierarchy in the generation of the images that follows the hierarchy of the model contents
 * - do not overwrite already existing files or at least alert the user 
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ExportAllDiagramsAction implements IActionDelegate
{
    private ISelection selection;

    /**
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection sel)
    {
        selection = sel;
        action.setEnabled(true);
    }

    /**
     * Try to retrieve the selected file from the given selection
     * 
     * @param sel the selection
     * @return the selected file
     */
    private IFile convertSelection2File(ISelection sel)
    {
        // get the selected diagrams file
        if (sel instanceof IStructuredSelection)
        {
            IStructuredSelection ssel = (IStructuredSelection) sel;
            // Only one file should be selected
            if (!ssel.isEmpty() && ssel.size() == 1)
            {
                Object selectedObj = ssel.getFirstElement();
                if (selectedObj instanceof IFile)
                {
                    return (IFile) selectedObj;
                }
                // Try to adapt
                if (selectedObj instanceof IAdaptable)
                {
                    return (IFile) ((IAdaptable) selectedObj).getAdapter(IFile.class);
                }
            }
        }
        return null;
    }

    /**
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action)
    {
        final ExportAllDiagramsDialog exportDialog = new ExportAllDiagramsDialog(Activator.getActiveWorkbenchShell(), convertSelection2File(selection));
        if (exportDialog.open() == Window.OK) {
            
            WorkspaceModifyOperation op = new WorkspaceModifyOperation()
            {
                public void execute(IProgressMonitor monitor) throws CoreException, InvocationTargetException, InterruptedException
                {
                    monitor.beginTask("Export diagrams images process ...", 1);
    
                    // Run the process in the UI Thread
                    Activator.getActiveWorkbenchShell().getDisplay().asyncExec(new Runnable()
                    {
                        /**
                         * @see java.lang.Runnable#run()
                         */
                        public void run()
                        {
                            // get the selected diagrams file
                            final IFile file = convertSelection2File(selection);
                            if (file == null)
                            {
                                Activator.displayDialog(null, "Invalid selection : you should select a file.", IStatus.ERROR);
                                return;
                            }
                                IEditorPart editorPart = null;
                                try
                                {
                                    editorPart = IDE.openEditor(Activator.getActiveWorkbenchWindow().getActivePage(), file, true);
                                }
                                catch (PartInitException e)
                                {
                                    Activator.log(e);
                                    Activator.displayDialog(null, "An error occurred during the opening of the graphical editor", IStatus.ERROR);
                                }
                                if (editorPart != null && editorPart instanceof Modeler)
                                {
                                    Modeler editor = (Modeler) editorPart;
                                    GraphicalViewer viewer = (GraphicalViewer) editor.getAdapter(GraphicalViewer.class);
                                    
                                    // List of all the diagram names contained by the Diagrams file
                                    List<String> diagramNames = new ArrayList<String>();
                                    // Indicate that at least two diagrams with the same name were present in the Diagrams file
                                    boolean duplicates = false;
                                      
                                    // Then iterates on all the diagrams and export them one by one
                                    for (Diagram currentDiagram : DiagramsUtils.findAllDiagrams(editor.getDiagrams()))
                                    {
                                        editor.setActiveDiagram(currentDiagram);
    
                                        IFigure figure = ((AbstractGraphicalEditPart) viewer.getRootEditPart()).getFigure();
                                        // Remove selection handles
                                        ((EditPart) ((AbstractGraphicalEditPart) viewer.getRootEditPart()).getChildren().get(0)).setSelected(EditPart.SELECTED_NONE);
                                        // Store initial zoom value
                                        double initialZoomValue = currentDiagram.getZoom();
                                        // Enforce zoom level as 100%
                                        ((ScalableFreeformRootEditPart) viewer.getRootEditPart()).getZoomManager().setZoom(1.0);

                                        FileOutputStream fos;
                                        try
                                        {
                                            String uniqueFileName = currentDiagram.getName();
                                            if (diagramNames.contains(uniqueFileName)) {
                                                duplicates = true;
                                                uniqueFileName = getFirstAvailableName(uniqueFileName, diagramNames, 1);
                                            }
                                            diagramNames.add(uniqueFileName);
                                            
                                            String str = exportDialog.getOutputDirectory().getLocation().toString();
                                            String ext = "." + exportDialog.getExporter().getExtension();
                                            File outputFile = new File(str.concat(File.separator).concat(uniqueFileName).concat(ext));
                                            outputFile.createNewFile();
                                            fos = new FileOutputStream(outputFile);
                                            exportDialog.getExporter().getExporter().export(figure, fos);
                                            fos.close();
                                        }
                                        catch (ExportException e)
                                        {
                                            Activator.log(e);
                                            Activator.displayDialog(null, "An error occurred during the export process using ImageExporter", IStatus.ERROR);
                                        }
                                        catch (IOException e)
                                        {
                                            Activator.log(e);
                                            Activator.displayDialog(null, "An error occurred while trying to create the output file", IStatus.ERROR);
                                        }
                                        catch (CoreException e)
                                        {
                                            Activator.log(e);
                                            Activator.displayDialog(null, "An error occurred while retrieving the ImageExporter", IStatus.ERROR);
                                        }
                                        finally {
                                            // Restore initial zool level
                                            currentDiagram.setZoom(initialZoomValue);
                                        }
                                    }

                                    // Alert the user that file names have been changed to avoid duplicates
                                    if (duplicates) {
                                        MessageDialog.openInformation(Activator.getActiveWorkbenchShell(), "Renaming Issue", "During export, at least two diagrams had the same name, created files have been renamed to avoid confusion.");
                                    }

                                    try
                                    {
                                        // Refresh the parent Container to reveal the new created file(s)
                                        exportDialog.getOutputDirectory().refreshLocal(1, new NullProgressMonitor());
                                    }
                                    catch (CoreException e)
                                    {
                                        Activator.log(e);
                                        Activator.displayDialog(null, "An error occurred while refreshing the navigator view", IStatus.ERROR);
                                    }
                                }
                        }                    
                    });
                    
                    monitor.done();
                }
            };
        
            try
            {
                // The ProgressMonitor is not visible (hidden because the graphical editor is open and activated ?)
                ProgressMonitorDialog dialog = new ProgressMonitorDialog(Activator.getActiveWorkbenchShell());
                dialog.run(false, false, op);
            }
            catch (InvocationTargetException e)
            {
                Activator.log(e);
                Activator.displayDialog(null, "An error occurred during the diagrams images generation", IStatus.ERROR);
            }
            catch (InterruptedException e)
            {
                Activator.log(e);
                Activator.displayDialog(null, "An error occurred during the diagrams images generation", IStatus.ERROR);
            }
        }
    }
    
    private String getFirstAvailableName(String commonBasis, List<String> existingNames, int cpt) {
        if (existingNames.contains(commonBasis+cpt)) {
            return getFirstAvailableName(commonBasis, existingNames, cpt+1);
        }
        
        return commonBasis+cpt;
    }

}
