/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. 
 * All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.internal.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.dialogs.ConfirmationDialog;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * This action is used to delete selected element(s) in the outline.<br>
 * 
 * Creation : 30 mai 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class DeleteAction extends Action
{
    private Diagrams diagrams;

    /**
     * The selected objects
     */
    private IStructuredSelection selection;

    /**
     * The Modeler editDomain
     */
    private MixedEditDomain editDomain;

    /**
     * Constructor
     * 
     * @param domain The editor EditDomain
     * @param diagrams the root Diagrams element
     * @param sel the selected element(s)
     */
    public DeleteAction(MixedEditDomain domain, Diagrams diagrams, ISelection sel)
    {
        super("Delete From Model", ModelerPlugin.getImageDescriptor("icons/deleteFromModel.gif"));

        this.diagrams = diagrams;
        this.editDomain = domain;
        if (sel instanceof IStructuredSelection)
        {
            this.selection = (IStructuredSelection) sel;
        }
        setEnabled(isParametersOK());
    }

    /**
     * Check the parameters validity
     * 
     * @return <code>true</code> if the parameters are valid
     */
    protected boolean isParametersOK()
    {
        return diagrams != null && editDomain != null && selection != null && !selection.isEmpty();
    }

    /**
     * Creates the GEF command that deletes the objects
     * 
     * @return the creation command
     */
    protected Command createActionCommand(IStructuredSelection sel)
    {
        CompoundCommand globalCommand = new CompoundCommand();

        Iterator it = selection.iterator();
        while (it.hasNext())
        {
            Object obj = it.next();

            if (obj instanceof IWrapperItemProvider || obj instanceof FeatureMap.Entry || obj instanceof EObject)
            {
                EObject model = (EObject) AdapterFactoryEditingDomain.unwrap(obj);

                globalCommand.add(new DeleteModelCommand(editDomain, diagrams, model));
            }
        }

        return globalCommand;
    }

    /**
     * This executes the command.
     */
    public void run()
    {
        if (isParametersOK())
        {
            Modeler modeler = (Modeler) editDomain.getEditorPart();
            ConfirmationDialog dialog = new ConfirmationDialog(ModelerPlugin.getActiveWorkbenchShell(), "Delete From Model",
                    "Are you sure you want to delete these model elements ?", modeler.getPreferenceStore(),
                    ModelerPreferenceConstants.DELETE_MODEL_ACTION_CONFIRM);
            int result = dialog.open();
            if (result == Window.OK)
            {
                ProgressMonitorDialog progressDialog = new ProgressMonitorDialog(Display.getDefault().getActiveShell());
                try
                {
                    progressDialog.run(true, false, new IRunnableWithProgress()
                    {
                        public void run(IProgressMonitor monitor) throws InvocationTargetException,
                                InterruptedException
                        {
                            DeleteAction.this.run(monitor);
                        }
                    });
                }
                catch (InvocationTargetException ite)
                {
                    ModelerPlugin.log(ite);
                    ModelerPlugin.displayDialog("Unable to delete",
                            "An error occurred during the delete action. See error logs for more details.",
                            IStatus.ERROR);
                }
                catch (InterruptedException ie)
                {
                    ModelerPlugin.log(ie);
                    ModelerPlugin.displayDialog("Unable to delete",
                            "An error occurred during the delete action. See error logs for more details.",
                            IStatus.ERROR);
                }
            }
        }
    }

    /**
     * Compute and execute the delete command
     * 
     * @param monitor the progress monitor
     */
    protected void run(final IProgressMonitor monitor)
    {
        monitor.beginTask("Delete the objects", 2);

        monitor.setTaskName("Compute command");
        Command command = createActionCommand(selection);
        monitor.worked(1);

        if (command != null && command.canExecute())
        {
            monitor.setTaskName("Execute command");
            editDomain.getCommandStack().execute(command);
            monitor.worked(1);
        }
        else
        {
            Display.getDefault().syncExec(new Runnable()
            {
                public void run()
                {
                    ModelerPlugin.displayDialog("Unable to delete", "One of the selected object cannot be deleted",
                            IStatus.WARNING);
                }
            });
        }

        monitor.done();
    }
}
