/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.internal.actions;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.ui.dialogs.ResourceDialog;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.ui.EMFEditUIPlugin;
import org.eclipse.emf.edit.ui.action.CommandActionHandler;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.topcased.modeler.commands.CreateDiagramsCommand;
import org.topcased.modeler.commands.EMFtoGEFCommandWrapper;
import org.topcased.modeler.commands.GEFtoEMFCommandWrapper;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;

/**
 * A custom Control action that is run from the outline view when a Controllable
 * model object is selected.<br>
 * 
 * Creation 5 oct. 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ModelerControlAction extends CommandActionHandler
{
    private IStructuredSelection selection = null;

    private Modeler modeler;

    private EObject eObject;

    /**
     * The constructor
     * 
     * @param theModeler The Modeler
     */
    public ModelerControlAction(Modeler theModeler)
    {
        super(theModeler.getEditingDomain(), EMFEditUIPlugin.INSTANCE.getString("_UI_Control_menu_item"));
        setDescription("_UI_Control_menu_item_description");
        setToolTipText("Split the model into an external model");
        this.modeler = theModeler;
    }

    /**
     * @see org.eclipse.jface.action.Action#isEnabled()
     */
    public boolean isEnabled()
    {
        return getEditingDomain().isControllable(eObject) && !AdapterFactoryEditingDomain.isControlled(eObject);
    }

    /**
     * The creation of the AddCommand is done in the run() method, when the user
     * specifies a target resource.
     * 
     * @see org.eclipse.emf.edit.ui.action.CommandActionHandler#updateSelection(org.eclipse.jface.viewers.IStructuredSelection)
     */
    public boolean updateSelection(IStructuredSelection theSelection)
    {
        boolean result = false;
        this.selection = theSelection;

        if (selection.size() == 1)
        {
            Object object = AdapterFactoryEditingDomain.unwrap(selection.getFirstElement());
            // Check whether the selected object is controllable
            result = getEditingDomain().isControllable(object);
            eObject = result ? (EObject) object : null;
        }

        return result;
    }

    /**
     * We have to execute a Control command on the selected model object and on
     * the diagrams that are referenced by objects that are controlled.
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run()
    {
        // Get Resources that will be created (One for the diagrams, another one
        // for the Model)
        ControlResourceDialog dialog = new ControlResourceDialog(
                PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), getEditingDomain(),
                eObject.eResource());
        dialog.open();
        Resource resourceModel = dialog.getControlResource();
        if (resourceModel == null)
        {
            return;
        }

        // Create the URI from the model that will be created
        URI newDiagramsURI = URI.createURI(resourceModel.getURI().toString() + "di");
        // TODO Sometimes the Resource already exist, we do not need to
        // create it but only get it
        Resource resourceDiagrams = modeler.getResourceSet().createResource(newDiagramsURI);

        // Create the CompoundCommand
        CompoundCommand compoundCommand = new CompoundCommand();

        Diagrams containerDiagrams = DiagramsUtils.findNearestContainerDiagrams(modeler.getDiagrams(), eObject);
        if (!eObject.equals(containerDiagrams.getModel()))
        {
            // Create the Command that will create a Diagrams model object to
            // associate with the eObject when none is defined and execute it so
            // that the Diagrams exists before the Control action onto the
            // Diagrams is created.
            modeler.getEditingDomain().getCommandStack().execute(
                    new GEFtoEMFCommandWrapper(new CreateDiagramsCommand(modeler, eObject)));
        }

        // Create the Command to Control model objects
        compoundCommand.append(new AddCommand(getEditingDomain(), resourceModel.getContents(), eObject));

        // Create the Command to Control the Diagrams associated
        compoundCommand.append(new AddCommand(getEditingDomain(), resourceDiagrams.getContents(),
                DiagramsUtils.findNearestContainerDiagrams(modeler.getDiagrams(), eObject)));

        // Execute the CompoundCommand
        ((MixedEditDomain) modeler.getAdapter(MixedEditDomain.class)).getCommandStack().execute(
                new EMFtoGEFCommandWrapper(compoundCommand));
    }

    /**
     * A save-type {@link ResourceDialog resource dialog} that attempts to
     * create the specified resource and load it, if it already exists.
     */
    protected static class ControlResourceDialog extends ResourceDialog
    {
        private EditingDomain domain;

        private Resource controlResource;

        private Resource currentResource;

        /**
         * The constructor
         * 
         * @param parent
         * @param theDomain
         * @param theCurrentResource
         */
        public ControlResourceDialog(Shell parent, EditingDomain theDomain, Resource theCurrentResource)
        {
            super(parent, EMFEditUIPlugin.INSTANCE.getString("_UI_ControlDialog_title"), SWT.SAVE);
            this.domain = theDomain;
            this.currentResource = theCurrentResource;
        }

        /**
         * Creates and, if it already exists, loads the specified resource. This
         * implementation verifies that a resource can be opened for that URI,
         * that the resource is not the object's current container, and that it
         * is not read-only in the editing domain. If there is an existing
         * resource with that URI, it prompts before overriding or adding to it.
         * 
         * @see org.eclipse.emf.common.ui.dialogs.ResourceDialog#processResources()
         */
        protected boolean processResources()
        {
            List uris = getURIs();
            if (uris.isEmpty())
            {
                return false;
            }

            URI uri = (URI) uris.get(0);
            ResourceSet resourceSet = domain.getResourceSet();
            Resource resource = resourceSet.getResource(uri, false);
            boolean resourceInSet = resource != null;

            if (resource == currentResource)
            {
                MessageDialog.openError(getShell(), EMFEditUIPlugin.INSTANCE.getString("_UI_InvalidURI_label"),
                        EMFEditUIPlugin.INSTANCE.getString("_WARN_AlreadyInResource"));
                return false;
            }
            if (domain.isReadOnly(resource))
            {
                MessageDialog.openError(getShell(), EMFEditUIPlugin.INSTANCE.getString("_UI_InvalidURI_label"),
                        EMFEditUIPlugin.INSTANCE.getString("_WARN_ReadOnlyResource"));
                return false;
            }

            boolean resourceExists = false;
            try
            {
                InputStream stream = resourceSet.getURIConverter().createInputStream(uri);
                if (stream != null)
                {
                    resourceExists = true;
                    stream.close();
                }
            }
            catch (IOException exception)
            {
                // Do nothing
            }

            boolean resourceBad = false;
            if (!resourceInSet)
            {
                resource = resourceSet.createResource(uri);
                if (resource == null)
                {
                    MessageDialog.openError(getShell(), EMFEditUIPlugin.INSTANCE.getString("_UI_InvalidURI_label"),
                            EMFEditUIPlugin.INSTANCE.getString("_WARN_CannotCreateResource"));
                    return false;
                }

                if (resourceExists)
                {
                    try
                    {
                        resource = resourceSet.getResource(uri, true);
                    }
                    catch (RuntimeException exception)
                    {
                        EMFEditUIPlugin.INSTANCE.log(exception);
                        resourceBad = resource.getContents().isEmpty();
                    }
                }
            }

            boolean result = true;
            if (resourceBad)
            {
                result = MessageDialog.openQuestion(getShell(),
                        EMFEditUIPlugin.INSTANCE.getString("_UI_ExistingResource_label"),
                        EMFEditUIPlugin.INSTANCE.getString("_WARN_ReplaceResource"));
            }
            else if (resourceExists)
            {
                result = MessageDialog.openQuestion(getShell(),
                        EMFEditUIPlugin.INSTANCE.getString("_UI_ExistingResource_label"),
                        EMFEditUIPlugin.INSTANCE.getString("_WARN_AddToResource"));
            }

            if (!result && !resourceInSet)
            {
                resource.unload();
                resourceSet.getResources().remove(resource);
            }
            else
            {
                this.controlResource = resource;
            }
            return result;
        }

        /**
         * Return the created Resource
         * 
         * @return the Resource
         */
        public Resource getControlResource()
        {
            return controlResource;
        }
    }

}
