/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.internal.preferences;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * This class represents a preference page that is contributed to the
 * Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>,
 * we can use the field support built into JFace that allows us to create a page
 * that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 */

public class ModelerPreferencePage extends PreferencePage implements IWorkbenchPreferencePage
{
    /** The parent composite */
    private Composite parentComposite;

    private Group defaultGp;

    private Combo unit;
    
    private Button useAntialiasBt;
    
    private Button useUUIDBt;
    
    private Button checkIntegrityBt;

    /**
     * The Constructor
     */
    public ModelerPreferencePage()
    {
        super("Modeler Preferences");
        setPreferenceStore(ModelerPlugin.getDefault().getPreferenceStore());
        setDescription("Topcased Editors preference page");
    }

    /**
     * Create the contents of the preference page
     * 
     * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents(Composite parent)
    {
        parentComposite = new Composite(parent, SWT.NULL);
        GridLayout layout = new GridLayout();
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        parentComposite.setLayout(layout);
        parentComposite.setFont(parent.getFont());

        createDefaultGroup(parentComposite);

        doLoad();

        return parentComposite;
    }

    /**
     * Initialize the preferencePage
     * 
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init(IWorkbench workbench)
    {
        // does nothing
    }

    /**
     * Creates a group where the user choose the default unit to use
     * 
     * @param parent the parent Composite
     */
    protected void createDefaultGroup(Composite parent)
    {
        defaultGp = new Group(parent, SWT.NONE);
        defaultGp.setLayout(new GridLayout(2, false));
        defaultGp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        
        Label unitText = new Label(defaultGp, SWT.NONE);
        unitText.setText("Default unit :");

        unit = new Combo(defaultGp, SWT.NONE | SWT.READ_ONLY);
        unit.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        useAntialiasBt = new Button(defaultGp, SWT.CHECK);
        useAntialiasBt.setText("Use antialias");
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        useAntialiasBt.setLayoutData(gd);

        useUUIDBt = new Button(defaultGp, SWT.CHECK);
        useUUIDBt.setText("Use UUID in the model serialization");
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        useUUIDBt.setLayoutData(gd);

        checkIntegrityBt = new Button(defaultGp, SWT.CHECK);
        checkIntegrityBt.setText("Check file integrity before saving");
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        checkIntegrityBt.setLayoutData(gd);
    }

    /**
     * Update the ModelerPreferencePage with values contained in the
     * PreferenceStore
     */
    protected void doLoad()
    {
        String units = getPreferenceStore().getString(ModelerPreferenceConstants.P_DEFAULT_UNIT);
        unit.setItems(new String[] {units});
        unit.setText(units);
        
        boolean useAntialias = getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_USE_ANTIALIAS);
        useAntialiasBt.setSelection(useAntialias);
        
        boolean useUUID = getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_USE_UUID);
        useUUIDBt.setSelection(useUUID);
        
        boolean checkIntegrity = getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_CHECK_INTEGRITY);
        checkIntegrityBt.setSelection(checkIntegrity);
    }

    /**
     * Update the ModelerPreferencePage with default values
     */
    protected void doLoadDefault()
    {
        unit.setText(getPreferenceStore().getDefaultString(ModelerPreferenceConstants.P_DEFAULT_UNIT));

        boolean useAntialias = getPreferenceStore().getDefaultBoolean(ModelerPreferenceConstants.P_USE_ANTIALIAS);
        useAntialiasBt.setSelection(useAntialias);

        boolean useUUID = getPreferenceStore().getDefaultBoolean(ModelerPreferenceConstants.P_USE_UUID);
        useUUIDBt.setSelection(useUUID);
        
        boolean checkIntegrity = getPreferenceStore().getDefaultBoolean(ModelerPreferenceConstants.P_CHECK_INTEGRITY);
        checkIntegrityBt.setSelection(checkIntegrity);
    }

    /**
     * Save values and changes made in the ModelerPreferencePage
     */
    protected void doStore()
    {
        getPreferenceStore().setValue(ModelerPreferenceConstants.P_DEFAULT_UNIT, unit.getText());
        getPreferenceStore().setValue(ModelerPreferenceConstants.P_USE_ANTIALIAS, useAntialiasBt.getSelection());
        getPreferenceStore().setValue(ModelerPreferenceConstants.P_USE_UUID, useUUIDBt.getSelection());
        getPreferenceStore().setValue(ModelerPreferenceConstants.P_CHECK_INTEGRITY, checkIntegrityBt.getSelection());
    }

    /**
     * Restore the default values
     * 
     * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults()
    {
        doLoadDefault();
        super.performDefaults();
    }

    /**
     * Store the values in the PreferenceStore
     * 
     * @see org.eclipse.jface.preference.IPreferencePage#performOk()
     */
    public boolean performOk()
    {
        doStore();
        return super.performOk();
    }

}
