/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.commands;

import java.util.Iterator;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.resource.StringConverter;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;

/**
 * The Command used to move a Port contained by MultiPort to the container of
 * the MultiPort or move a Port to a MultiPort in the same hierarchy of the Port
 * 
 * Creation : 21 dec 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class MovePortCommand extends Command
{
    /** The child GraphNode that is dropped */
    private GraphNode childNode;
    
    /** The old parent GraphNode that contained the childNode */
    private GraphNode oldParentNode;
    
    /** The new parent GraphNode that should contain the childNode */
    private GraphNode newParentNode;
    
    /** The new Constraint of the moved Port node */
    private Point newPosition;
    
    /** The old Constraint of the moved Port node */
    private PortConstraint oldConstraint;
    
    /** The new Attachment of the Port */
    private int newAttachment;
    
    /**
     * Constructor
     * 
     * @param parent the parentEditPart that will contain the PortEditPart
     * @param child the PortEditPart that is moved
     * @param position the new position
     * @param attach the new attachment
     */
    public MovePortCommand(GraphNode parent, GraphNode child, Point position, int attach)
    {
        super("Move a Port to or from a MultiPort");
        this.newParentNode = parent;
        this.childNode = child;
        this.newPosition = position;
        this.oldParentNode = (GraphNode) childNode.getContainer();
        this.newAttachment = attach;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        Object o = Utils.getElement(childNode);
        if (!(o instanceof Port))
        {
            return false;
        }
        if (newParentNode != null && childNode != null)
        {
            // check if the Move is from or to a MultiPort and if the new
            // container is valid
            if (Utils.getElement(newParentNode) instanceof MultiPort)
            {
                return newParentNode.getContainer() == childNode.getContainer();
            }
            else if (Utils.getElement(oldParentNode) instanceof MultiPort)
            {
                return oldParentNode.getContainer() == newParentNode;
            }
        }
        return false;
    }
    
    /**
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        // store the old constraint
        Point oldPos = childNode.getPosition() == null ? new Point(0, 0) : childNode.getPosition();
        Dimension oldDim = childNode.getSize() == null ? new Dimension(-1, -1) : childNode.getSize();
        int oldAttach = StringConverter.asInt(DIUtils.getPropertyValue(childNode,
                ModelerPropertyConstants.PORT_POSITION));
        
        this.oldConstraint = new PortConstraint(new Rectangle(oldPos, oldDim), oldAttach);
        
        redo();
    }
    
    /**
     * Add a new DataPort object to the SynchronisationGate model object
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        if (newPosition != null)
        {
            // update the new position of the port and its new attachment
            childNode.setPosition(newPosition);
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION,
                    StringConverter.asString(newAttachment));
        }
        
        // remove the node from the old container and add it to the new
        // container
        oldParentNode.getContained().remove(childNode);
        newParentNode.getContained().add(childNode);
        
        redoModel();
    }
    
    /**
     * Update the model
     */
    protected void redoModel()
    {
        EObject childEObject = Utils.getElement(childNode);
        EObject parentEObject = Utils.getElement(newParentNode);
        
        EReference selectedFeature = null;
        
        // we check all the EStructuralFeatures of type Containment
        for (Iterator i = parentEObject.eClass().getEAllContainments().iterator(); i.hasNext();)
        {
            EReference eRef = (EReference) i.next();
            // Check if the EReference is of the expected type
            if (eRef.getEReferenceType().isInstance(childEObject))
            {
                selectedFeature = eRef;
            }
            
        }
        
        if (selectedFeature != null)
        {
            if (parentEObject != null && childEObject != null)
            {
                EList ownerList = Utils.getOwnerList(parentEObject, selectedFeature);
                if (!ownerList.contains(childEObject))
                {
                    ownerList.add(childEObject);
                }
            }
        }
    }
    
    /**
     * Remove the last added DataPort
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if (oldConstraint != null)
        {
            // set the old position of the port and its old attachment
            childNode.setPosition(oldConstraint.getRectConstraints().getLocation());
            DIUtils.setProperty(childNode, ModelerPropertyConstants.PORT_POSITION,
                    StringConverter.asString(oldConstraint.getAttachment()));
        }
        
        newParentNode.getContained().remove(childNode);
        oldParentNode.getContained().add(childNode);
        
        undoModel();
    }
    
    /**
     * Undo the model modifications
     */
    protected void undoModel()
    {
        EObject childEObject = Utils.getElement(childNode);
        EObject parentEObject = Utils.getElement(oldParentNode);
        
        EReference selectedFeature = null;
        
        // we check all the EStructuralFeatures of type Containment
        for (Iterator i = parentEObject.eClass().getEAllContainments().iterator(); i.hasNext();)
        {
            EReference eRef = (EReference) i.next();
            // Check if the Reference is of the expected type
            if (eRef.getEReferenceType().isInstance(childEObject))
            {
                selectedFeature = eRef;
            }
            
        }
        
        if (selectedFeature != null)
        {
            if (parentEObject != null && childEObject != null)
            {
                EList ownerList = Utils.getOwnerList(parentEObject, selectedFeature);
                if (!ownerList.contains(childEObject))
                {
                    ownerList.add(childEObject);
                }
            }
        }
    }
    
    /**
     * Calculate the nearest border where the graphNode should be attached <b>at
     * the creation</b> depending on its location (<code>Point</code>) and
     * its container (<code>Rectangle</code>)
     * 
     * @param loc the location where the user has clicked in the container
     *        coordinates.
     * @param container the container Figure
     * @return a <code>PositionConstant</code>
     */
    protected int calculateChildAttachment(Point loc, Rectangle container)
    {
        int position = PositionConstants.NONE;
        
        // define the borders on which it is possible to attach the moved Port
        int border = PositionConstants.NORTH | PositionConstants.SOUTH | PositionConstants.EAST
                | PositionConstants.WEST;
        
        boolean bottom = ((border & PositionConstants.SOUTH) != 0);
        boolean top = ((border & PositionConstants.NORTH) != 0);
        boolean right = ((border & PositionConstants.WEST) != 0);
        boolean left = ((border & PositionConstants.EAST) != 0);
        
        int toLeft = loc.x;
        int toTop = loc.y;
        int toRight = container.width - loc.x;
        int toBottom = container.height - loc.y;
        
        if (left && (!top || toLeft < toTop) && (!right || toLeft < toRight) && (!bottom || toLeft < toBottom))
            position = PositionConstants.LEFT;
        else if (top && (!right || toTop < toRight) && (!bottom || toTop < toBottom))
            position = PositionConstants.TOP;
        else if (right && (!bottom || toRight < toBottom))
            position = PositionConstants.RIGHT;
        else
            position = PositionConstants.BOTTOM;
        
        return position;
    }
    
}
