/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.documentation;

import java.util.EventObject;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.Page;
import org.topcased.modeler.documentation.EAnnotationResourcesComposite;
import org.topcased.modeler.documentation.IDocPage;
import org.topcased.sam.IdentifiedItem;

/**
 * This class creates the page to edit documentation of the edited SAM model.<br>
 * <br>
 * 
 * Created : 12 July 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class IdentifiedItemDocPage extends Page implements IDocPage
{
    
    private IdentifiedItem documentedElement;
    
    private CommandStackListener commandStackListener;
    
    private CommandStack stack;
    
    // SWT Widgets
    private Composite mainComp;
    
    private Text textField;
    
    private EAnnotationResourcesComposite resourcesComposite;
    
    private FocusListener focusListener = new FocusAdapter()
    {
        private String previousComment;
        
        public void focusGained(FocusEvent e)
        {
            previousComment = textField.getText();
        }
        
        public void focusLost(FocusEvent e)
        {
            if (!textField.getText().equals(previousComment))
            {
                handleDocChanged();
            }
        }
    };
    
    private class ChangeDocumentationCommand extends Command
    {
        private IdentifiedItem element;
        
        private String newDocumentation;
        
        private String oldDocumentation;
        
        public ChangeDocumentationCommand(IdentifiedItem element, String documentation)
        {
            super("Change documentation of an IdentifiedItem");
            
            this.element = element;
            this.newDocumentation = documentation;
        }
        
        /**
         * @see org.eclipse.gef.commands.Command#execute()
         */
        public void execute()
        {
            // stores the previous doc
            oldDocumentation = element.getComment();
            
            redo();
        }
        
        /**
         * Set the documentation for the given Identified Item
         * 
         * @param elt the element to document
         * @param newDoc the documentation text
         */
        protected void changeDocumentation(IdentifiedItem elt, String newDoc)
        {
            elt.setComment(newDoc);
        }
        
        /**
         * @see org.eclipse.gef.commands.Command#redo()
         */
        public void redo()
        {
            changeDocumentation(element, newDocumentation);
        }
        
        /**
         * @see org.eclipse.gef.commands.Command#undo()
         */
        public void undo()
        {
            changeDocumentation(element, oldDocumentation);
        }
    }
    
    /**
     * Build the page with the editor command stack : used to execute commands.
     * 
     * @param stack the editor command stack
     */
    public IdentifiedItemDocPage(CommandStack stack)
    {
        setCommandStack(stack);
    }
    
    /**
     * @see org.eclipse.ui.views.properties.IPropertySheetEntry#dispose()
     */
    public void dispose()
    {
        if (stack != null)
        {
            stack.removeCommandStackListener(commandStackListener);
        }
        super.dispose();
    }
    
    private void setCommandStack(CommandStack stack)
    {
        this.stack = stack;
        commandStackListener = new CommandStackListener()
        {
            public void commandStackChanged(EventObject e)
            {
                refresh();
            }
        };
        stack.addCommandStackListener(commandStackListener);
    }
    
    /**
     * Returns the editor command stack
     * 
     * @return the command stack
     */
    protected CommandStack getCommandStack()
    {
        return stack;
    }
    
    protected void refresh()
    {
        textField.setText("");
        if (documentedElement != null)
        {
            String doc = documentedElement.getComment();
            if (doc != null)
            {
                textField.setText(doc);
            }
        }
    }
    
    /**
     * @see org.eclipse.ui.part.Page#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        mainComp = new Composite(parent, SWT.NONE);
        mainComp.setLayout(new GridLayout());
        mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        TabFolder tabFolder = new TabFolder(mainComp, SWT.TOP);
        tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        createCommentsTabItem(tabFolder);
        createResourcesTabItem(tabFolder);
    }
    
    /**
     * Create the content of the comments tab item. It contains a multi lines
     * text field in order to edit the comments of the documentation.
     * 
     * @param parent the parent tab folder
     */
    private void createCommentsTabItem(TabFolder parent)
    {
        TabItem tabItem = new TabItem(parent, SWT.NONE);
        tabItem.setText("Comments");
        
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());
        container.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        tabItem.setControl(container);
        
        textField = new Text(container, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
        textField.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        textField.addFocusListener(focusListener);
    }
    
    /**
     * Create the content of the resources tab item. It contains a list and
     * three buttons in order to view, add, remove and edit linked resources.
     * 
     * @param parent the parent tab folder
     */
    private void createResourcesTabItem(TabFolder parent)
    {
        TabItem tabItem = new TabItem(parent, SWT.NONE);
        tabItem.setText("Resources");
        
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout());
        container.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        tabItem.setControl(container);
        
        resourcesComposite = new EAnnotationResourcesComposite(container, SWT.NONE, stack);
        resourcesComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
    }
    
    private void handleDocChanged()
    {
        String newDoc = textField.getText();
        if (documentedElement != null)
        {
            Command cmd = new ChangeDocumentationCommand(documentedElement, newDoc);
            getCommandStack().execute(cmd);
        }
    }
    
    public void setIdentifiedItem(IdentifiedItem elt)
    {
        if (elt == null || elt != documentedElement)
        {
            documentedElement = elt;
            resourcesComposite.setModelElement(elt);
            refresh();
        }
    }
    
    /**
     * @see org.eclipse.ui.part.Page#getControl()
     */
    public Control getControl()
    {
        return mainComp;
    }
    
    /**
     * @see org.eclipse.ui.part.Page#setFocus()
     */
    public void setFocus()
    {
        // Do nothing
    }
    
    private IdentifiedItem getSelectedIdentifiedItem(ISelection selection)
    {
        
        if (selection instanceof IStructuredSelection && ((IStructuredSelection) selection).size() == 1)
        {
            Object selectedObject = ((IStructuredSelection) selection).getFirstElement();
            
            if (selectedObject instanceof IdentifiedItem)
            {
                return (IdentifiedItem) selectedObject;
            }
            if (selectedObject instanceof IAdaptable)
            {
                Object adaptedObject = ((IAdaptable) selectedObject).getAdapter(EObject.class);
                if (adaptedObject != null && adaptedObject instanceof IdentifiedItem)
                {
                    return (IdentifiedItem) adaptedObject;
                }
            }
            
            Object adaptedObject = Platform.getAdapterManager().getAdapter(selectedObject, EObject.class);
            if (adaptedObject != null && adaptedObject instanceof IdentifiedItem)
            {
                return (IdentifiedItem) adaptedObject;
            }
        }
        
        return null;
    }
    
    /**
     * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IWorkbenchPart part, ISelection selection)
    {
        if (textField == null || textField.isDisposed())
        {
            return;
        }
        
        textField.setText("");
        textField.setEnabled(false);
        IdentifiedItem identifiedItem = getSelectedIdentifiedItem(selection);
        setIdentifiedItem(identifiedItem);
        if (identifiedItem != null)
        {
            textField.setEnabled(true);
            String comment = identifiedItem.getComment();
            textField.setText(comment != null ? comment : "");
        }
    }
    
}
