/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sam.edit;

import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.commands.PortRestoreConnectionCommand;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.modeler.sam.edit.policies.SAMMultiPortLayoutEditPolicy;
import org.topcased.modeler.sam.edit.policies.SamLabelDirectEditPolicy;
import org.topcased.modeler.sam.systemdiagram.commands.RestorePortsCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.MultiPort;
import org.topcased.sam.SAMPackage;

/**
 * An EditPart used to handle MultiPort behavior in SAM. A MultiPort contains
 * children Port and has a Property that record the visual state of the
 * MultiPort (CLOSED or OPENED)
 * 
 * Creation : 19 oct. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class MultiPortEditPart extends org.topcased.modeler.edit.MultiPortEditPart
{
    /**
     * @param obj
     */
    public MultiPortEditPart(GraphNode obj)
    {
        super(obj);
    }

    /**
     * Creates edit policies and associates these with roles
     */
    @Override
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.LAYOUT_ROLE, new SAMMultiPortLayoutEditPolicy());

        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            /**
             * @see org.topcased.modeler.edit.policies.RestoreEditPolicy#getRestoreConnectionsCommand(org.topcased.modeler.requests.RestoreConnectionsRequest)
             */
            @Override
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                CompoundCommand compCmd = new CompoundCommand();
                for (Iterator iter = getHost().getChildren().iterator(); iter.hasNext();)
                {
                    EditPart editPart = (EditPart) iter.next();
                    Command cmd = new PortRestoreConnectionCommand(editPart);
                    if ((cmd != null) && cmd.canExecute())
                        compCmd.add(cmd);
                }
                return compCmd;
            }

        });

        // edit policy forbidding deleting multiports that are instances of
        // other multiports.
        installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY, new DeleteModelObjectEditPolicy()
        {
            @Override
            public boolean understandsRequest(Request req)
            {
                if (!super.understandsRequest(req))
                {
                    return false;
                }
                if (getHost() instanceof SAMPortEditPart)
                {
                    MultiPort port = (MultiPort) Utils.getElement((GraphElement) ((MultiPortEditPart) getHost()).getModel());
                    // do not allow deletion of ports that are instances of
                    // others.
                    if (port.getIsInstanceOf() != null)
                    {
                        return false;
                    }
                }
                return true;
            }
        });

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

    }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
     */
    @Override
    protected IFigure createFigure()
    {
        // REV weird code ....use a command ?
        RestorePortsCommand restorePortsCommand = new RestorePortsCommand(getViewer().getEditDomain(),
                (GraphNode) getModel(),
                ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
        if ((restorePortsCommand != null) && restorePortsCommand.canExecute())
        {
            restorePortsCommand.execute();
        }

        return super.createFigure();
    }

    /**
     * @see org.topcased.modeler.edit.BaseEditPart#handleModelChanged(org.eclipse.emf.common.notify.Notification)
     */
    @Override
    protected void handleModelChanged(Notification msg)
    {
        if (msg.getNotifier() instanceof MultiPort)
        {
            switch (msg.getFeatureID(MultiPort.class))
            {
                case SAMPackage.MULTI_PORT__LIST_PORT:
                    // update graphically all the ports that may have been added
                    // in the model so that they are synchronized
                    // REV weird code ....use a command ?
                    RestorePortsCommand restorePortsCommand = new RestorePortsCommand(
                            getViewer().getEditDomain(),
                            (GraphNode) getModel(),
                            ((ModelerGraphicalViewer) getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils());
                    if ((restorePortsCommand != null) && restorePortsCommand.canExecute())
                    {
                        restorePortsCommand.execute();
                    }
                    break;
            }
        }

        super.handleModelChanged(msg);
    }
}
