/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.ModelerEditPolicyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EMFGraphNodeEditPart;
import org.topcased.modeler.edit.PortEditPart;
import org.topcased.modeler.edit.policies.LabelDirectEditPolicy;
import org.topcased.modeler.edit.policies.NonRemovableComponentEditPolicy;
import org.topcased.modeler.edit.policies.RestoreEditPolicy;
import org.topcased.modeler.figures.ModelerLabelledPortFigure;
import org.topcased.modeler.figures.PortAnchor;
import org.topcased.modeler.requests.RestoreConnectionsRequest;
import org.topcased.modeler.sam.SAMEditPolicyConstants;
import org.topcased.modeler.sam.commands.PortRestoreConnectionCommand;
import org.topcased.modeler.sam.edit.policies.CustomEditPolicy;
import org.topcased.modeler.sam.edit.policies.DeleteSAMPortEditPolicy;
import org.topcased.modeler.sam.edit.policies.SamLabelDirectEditPolicy;
import org.topcased.modeler.sam.systemdiagram.policies.FlowCreationEditPolicy;
import org.topcased.sam.Flow;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;
import org.topcased.sam.SAMPackage;

/**
 * The Port object controller for SAM Editor
 * 
 * Creation : 29 sept. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class SAMPortEditPart extends PortEditPart
{
    /**
     * Constructor
     * 
     * @param obj
     */
    public SAMPortEditPart(GraphNode obj)
    {
        super(obj);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.edit.PortEditPart#createEditPolicies()
     */
    protected void createEditPolicies()
    {
        super.createEditPolicies();

        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new FlowCreationEditPolicy());

        installEditPolicy(ModelerEditPolicyConstants.RESTORE_EDITPOLICY, new RestoreEditPolicy()
        {
            protected Command getRestoreConnectionsCommand(RestoreConnectionsRequest request)
            {
                return new PortRestoreConnectionCommand(getHost());
            }
        });

        installEditPolicy(ModelerEditPolicyConstants.CHANGE_BACKGROUND_COLOR_EDITPOLICY, null);

        installEditPolicy(SAMEditPolicyConstants.CUSTOM_EDITPOLICY, new CustomEditPolicy());

        installEditPolicy(EditPolicy.COMPONENT_ROLE, new NonRemovableComponentEditPolicy());

        // Add a custom policy that prevent from deleting ports in
        // SynchonisationGate when a minimum number of input (or output) ports
        // is reached.
        installEditPolicy(ModelerEditPolicyConstants.DELETE_MODEL_OBJECT_EDITPOLICY, new DeleteSAMPortEditPolicy());

        installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LabelDirectEditPolicy());

    }

    /**
     * Set the name of the port if it is not null
     * 
     * @see org.topcased.modeler.edit.EMFGraphNodeEditPart#refreshHeaderLabel()
     */
    protected void refreshHeaderLabel()
    {
        if (getFigure() instanceof ModelerLabelledPortFigure)
        {
            String inFlow = "";
            String outFlow = "";
            String textPort = "";

            if (((Port) getEObject()).getInlink() != null && ((Port) getEObject()).getInlink().getName() != null)
            {
                inFlow = ((Port) getEObject()).getInlink().getName();
            }
            if (((Port) getEObject()).getOutlink() != null && ((Port) getEObject()).getOutlink().getName() != null)
            {
                outFlow = ((Port) getEObject()).getOutlink().getName();
            }

            // When the flows connected to each side of a port have the same
            // name,
            // only display this name instead of flow1>flow2.
            if ("".equals(inFlow) && "".equals(outFlow))
            {
                textPort = ((Port) getEObject()).getName();
            }
            else if (inFlow.equals(outFlow))
            {
                textPort = inFlow;
            }
            else
            {
                textPort = inFlow + ">" + outFlow;
            }

            ((ModelerLabelledPortFigure) getFigure()).setPortName(textPort);
        }
    }

    /**
     * Activate the adapters
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
     */
    public void activate()
    {
        super.activate();

        if (((Port) getEObject()).getInlink() != null)
            ((Port) getEObject()).getInlink().eAdapters().add(getModelListener());

        if (((Port) getEObject()).getOutlink() != null)
            ((Port) getEObject()).getOutlink().eAdapters().add(getModelListener());
    }

    /**
     * Deactivate the adapters
     * 
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
     */
    public void deactivate()
    {
        if (((Port) getEObject()).getOutlink() != null)
            ((Port) getEObject()).getOutlink().eAdapters().remove(getModelListener());

        if (((Port) getEObject()).getInlink() != null)
            ((Port) getEObject()).getInlink().eAdapters().remove(getModelListener());

        super.deactivate();
    }

    /**
     * Update model and graphics
     * 
     * @param msg the Notification
     */
    protected void handleModelChanged(Notification msg)
    {
        super.handleModelChanged(msg);

        Object newObject = msg.getNewValue();
        Object oldObject = msg.getOldValue();

        if (msg.getNotifier() instanceof Port)
        {
            switch (msg.getFeatureID(Port.class))
            {
                case SAMPackage.PORT__INLINK:
                case SAMPackage.PORT__OUTLINK:
                    if (newObject != null && oldObject == null && newObject instanceof Flow)
                    {
                        Flow flow = (Flow) newObject;
                        if (!flow.eAdapters().contains(getModelListener()))
                        {
                            flow.eAdapters().add(getModelListener());
                        }
                    }
                    if (newObject == null && oldObject != null && oldObject instanceof Flow)
                    {
                        Flow flow = (Flow) oldObject;
                        if (flow.eAdapters().contains(getModelListener()))
                        {
                            flow.eAdapters().remove(getModelListener());
                        }
                    }
                    refreshVisuals();
                    break;
            }
        }
        if (msg.getNotifier() instanceof Flow)
        {
            switch (msg.getFeatureID(Flow.class))
            {
                case SAMPackage.FLOW__NAME:
                    refreshVisuals();
                    break;
                default:
                    break;
            }
        }
    }

    /**
     * @see org.topcased.modeler.edit.PortEditPart#getPortConnectionAnchor()
     */
    protected ConnectionAnchor getPortConnectionAnchor()
    {
        // Determine the Anchor with the container of the Port
        if (getParent().getModel() instanceof Diagram
                || ((EMFGraphNodeEditPart) getParent()).getEObject() instanceof MultiPort
                && getParent().getParent().getModel() instanceof Diagram)
        {
            return new PortAnchor(getPortFigure(), false);
        }
        else
        {
            return new PortAnchor(getPortFigure(), true);
        }
    }

    /**
     * Only display the manager used to perform the directEdit action if no
     * connection is assosiated with the port
     * 
     * @see org.topcased.modeler.edit.GraphNodeEditPart#performDirectEdit()
     */
    protected void performDirectEdit()
    {
        if (((Port) getEObject()).getInlink() == null && ((Port) getEObject()).getOutlink() == null)
        {
            super.performDirectEdit();
        }
    }
}
