/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit.policies;

import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.requests.AutoLayoutRequest;
import org.topcased.modeler.sam.commands.CreateInstanceCommand;
import org.topcased.modeler.sam.commands.MovePortCommand;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.AbstractState;
import org.topcased.sam.Automaton;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;

/**
 * This is a custom LayoutEditPolicy that handles Ports layout. Ports are
 * attached to the borders of the editing area.
 * 
 * Added support for drag and drops of systems/automatas from the oultline.
 * 
 * Created 7 juin 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * 
 */
public class SAMDiagramLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor. None border are available for anchoring a port.
     */
    public SAMDiagramLayoutEditPolicy()
    {
        super();
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     */
    @Override
    protected boolean isAttachedToBorder(GraphNode node)
    {
        if ((Utils.getElement(node) instanceof Port) || (Utils.getElement(node) instanceof MultiPort))
        {
            return true;
        }
        return super.isAttachedToBorder(node);
    }

    /**
     * Copy the code of the LayoutEditPolicy and pass to the createAddCommand
     * the mouseLocation given by the request
     * 
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getAddCommand(org.eclipse.gef.Request)
     */
    @Override
    protected Command getAddCommand(Request generic)
    {
        ChangeBoundsRequest request = (ChangeBoundsRequest) generic;
        List editParts = request.getEditParts();
        CompoundCommand command = new CompoundCommand();
        command.setDebugLabel("Add in ConstrainedLayoutEditPolicy");//$NON-NLS-1$
        GraphicalEditPart childPart;
        Rectangle r;
        Object constraint;

        for (int i = 0; i < editParts.size(); i++)
        {
            childPart = (GraphicalEditPart) editParts.get(i);
            r = childPart.getFigure().getBounds().getCopy();
            // convert r to absolute from childpart figure
            childPart.getFigure().translateToAbsolute(r);
            r = request.getTransformedRectangle(r);
            // convert this figure to relative
            getLayoutContainer().translateToRelative(r);
            getLayoutContainer().translateFromParent(r);
            r.translate(getLayoutOrigin().getNegated());
            constraint = getConstraintFor(r);
            /*
             * @TODO:Pratik Should create a new createAddCommand(...) which is
             * given the request so that attaching to/detaching from guides can
             * be taken care of.
             */
            GraphElement graphElement = (GraphElement) childPart.getModel();
            Command addCmd;
            Object o = Utils.getElement(graphElement);
            if (o instanceof AbstractState)
            {
                addCmd = createAddCommand(childPart, translateToModelConstraint(constraint));
            }
            else
            {
                addCmd = createAddCommand(childPart, translateToModelConstraint(constraint),
                        getAbsoluteMouseLocation(request.getLocation()));
            }
            command.add(addCmd);
        }
        return command.unwrap();
    }

    /**
     * Create and return the Command used to perform the DND of a Port
     * 
     * @param child
     * @param constraint
     * @param mouseLocation
     * @return the Command to perform the DND Command
     */
    protected Command createAddCommand(EditPart child, Object constraint, Point mouseLocation)
    {
        if ((getHost().getModel() instanceof GraphNode) && (child.getModel() instanceof GraphNode))
        {
            if (constraint instanceof Rectangle)
            {
                int attachment = calculateChildAttachment((GraphNode) child.getModel(), mouseLocation,
                        ((AbstractGraphicalEditPart) getHost()).getContentPane().getClientArea());

                return new MovePortCommand((GraphNode) getHost().getModel(), (GraphNode) child.getModel(),
                        ((Rectangle) constraint).getLocation(), attachment);
            }
        }
        return UnexecutableCommand.INSTANCE;
    }

    /**
     * This is called when performing a drag and drop from the outline into a
     * diagram, thus it has been customized to deal with instance creation.
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphNode,
     *      org.topcased.modeler.di.model.GraphNode,
     *      org.eclipse.emf.ecore.EObject, org.eclipse.draw2d.geometry.Point,
     *      org.eclipse.draw2d.geometry.Dimension, int, java.util.List, boolean)
     */
    @Override
    protected Command getCreateCommand(EditDomain domain, GraphNode newObject, GraphNode newParent,
            EObject newContainerParent, Point location, Dimension dimension, int attach, List featuresList,
            boolean needModelUpdate)
    {
        EObject referenced = InstancesUtil.getEmfElement(newObject);

        // When the referenced element is not associated with an EResource, the creation is made from the Palette (not
        // an instance one)
        if (referenced.eResource() != null && InstancesUtil.isAnImport(referenced))
        {
            return new CreateInstanceCommand(newContainerParent, newParent, newObject, location);
        }

        return new CreateGraphNodeCommand(domain, newObject, newParent, newContainerParent, location, dimension,
                attach, featuresList, needModelUpdate);
    }

    /**
     * Overriden to allow drag and drops of systems/automatas from the outline.
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isExternalObjectAllowed(org.topcased.modeler.di.model.GraphNode,
     *      org.topcased.modeler.di.model.GraphNode)
     */
    @Override
    protected boolean isExternalObjectAllowed(GraphNode parent, GraphNode child)
    {
        EObject element = Utils.getElement(child);
        if (element != null)
        {
            if ((element instanceof org.topcased.sam.System) || (element instanceof Automaton))
            {
                if (parent instanceof Diagram)
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Overriden so as to disallow the autolayout requested during external drag
     * and drops (it doesn't work well with our references to other systems)
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getAutoLayoutCommand(org.topcased.modeler.requests.AutoLayoutRequest)
     */
    @Override
    protected Command getAutoLayoutCommand(AutoLayoutRequest request)
    {
        return super.getAutoLayoutCommand(request);
    }

}
