/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.sam.edit.policies;

import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;

import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.sam.commands.MovePortCommand;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;

/**
 * An EditPolicy that manage creation and moves of a Port inside its container
 * (System or Automaton). The Port is attached to the nearest border (Left,
 * Right or Bottom)
 * 
 * Created 6 June 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class SAMPortLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor. None border are available for anchoring a port.
     */
    public SAMPortLayoutEditPolicy()
    {
        super();
    }

    /**
     * Added a condition to disallow adding new ports to instances of a
     * system/automaton. Also disallow working with an instanciated port.
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     */
    @Override
    protected boolean isValid(EObject child, EObject parent)
    {
        if ((child instanceof Port) || (child instanceof MultiPort))
        {
            if (!InstancesUtil.isAReference(parent) && !InstancesUtil.isAReference(child))
            {

                return super.isValid(child, parent);
            }

        }

        return false;
    }

    /**
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     */
    @Override
    protected boolean isAttachedToBorder(GraphNode node)
    {
        if ((Utils.getElement(node) instanceof Port) || (Utils.getElement(node) instanceof MultiPort))
        {
            return true;
        }
        return super.isAttachedToBorder(node);
    }

    /**
     * Copy the code of the LayoutEditPolicy and pass to the createAddCommand
     * the mouseLocation given by the request
     * 
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getAddCommand(org.eclipse.gef.Request)
     */
    @Override
    protected Command getAddCommand(Request generic)
    {
        ChangeBoundsRequest request = (ChangeBoundsRequest) generic;
        List editParts = request.getEditParts();
        CompoundCommand command = new CompoundCommand();
        command.setDebugLabel("Add in ConstrainedLayoutEditPolicy");//$NON-NLS-1$
        GraphicalEditPart childPart;
        Rectangle r;
        Object constraint;

        for (int i = 0; i < editParts.size(); i++)
        {
            childPart = (GraphicalEditPart) editParts.get(i);
            r = childPart.getFigure().getBounds().getCopy();
            // convert r to absolute from childpart figure
            childPart.getFigure().translateToAbsolute(r);
            r = request.getTransformedRectangle(r);
            // convert this figure to relative
            getLayoutContainer().translateToRelative(r);
            getLayoutContainer().translateFromParent(r);
            r.translate(getLayoutOrigin().getNegated());
            constraint = getConstraintFor(r);
            /*
             * @TODO:Pratik Should create a new createAddCommand(...) which is
             * given the request so that attaching to/detaching from guides can
             * be taken care of.
             */
            command.add(createAddCommand(childPart, translateToModelConstraint(constraint),
                    getAbsoluteMouseLocation(request.getLocation())));
        }
        return command.unwrap();
    }

    /**
     * Create and return the Command used to perform the DND of a Port
     * 
     * @param child
     * @param constraint
     * @param mouseLocation
     * @return the Command to perform the DND Command
     */
    protected Command createAddCommand(EditPart child, Object constraint, Point mouseLocation)
    {
        if ((getHost().getModel() instanceof GraphNode) && (child.getModel() instanceof GraphNode))
        {
            if (constraint instanceof Rectangle)
            {
                int attachment = calculateChildAttachment((GraphNode) child.getModel(), mouseLocation,
                        ((AbstractGraphicalEditPart) getHost()).getContentPane().getClientArea());

                return new MovePortCommand((GraphNode) getHost().getModel(), (GraphNode) child.getModel(),
                        ((Rectangle) constraint).getLocation(), attachment);
            }
        }
        return UnexecutableCommand.INSTANCE;
    }

}
