/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jrme Wrd (Anyware Technologies) - initial API and implementation
 *               Jacques Lescot (Anyware Technologies) - Manage controlled models
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.viewers.ITreeContentProvider;

import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.ModelContentProvider;
import org.topcased.sam.Model;
import org.topcased.sam.ModelContent;

/**
 * 
 * Modification of the default {@link ModelContentProvider} for the outline navigator that adds a section containing the
 * resources that it is possible to import from a SAM diagram.
 * 
 * @author jrme
 */
public class SamOutlineContentProvider extends ModelContentProvider
{
    /**
     * Constructor. Nothing special done.
     * 
     * @see ModelContentProvider#ModelContentProvider(Modeler, ITreeContentProvider)
     * @param modeler
     * @param delegatedProvider
     */
    public SamOutlineContentProvider(Modeler modeler, ITreeContentProvider delegatedProvider)
    {
        super(modeler, delegatedProvider);
    }

    /**
     * Returns the possible imports from the resources when the <code>parentElement</code> is a
     * {@link FilteredAdditionalResources}.
     * 
     * @see org.topcased.modeler.editor.outline.ModelContentProvider#getChildren(java.lang.Object)
     */
    @Override
    public Object[] getChildren(Object parentElement)
    {
        if (parentElement instanceof Diagrams)
        {
            return new Object[] {EcoreUtil.getRootContainer(((Diagrams) parentElement).getModel())};
        }

        if (parentElement instanceof FilteredAdditionalResources)
        {
            // delegate
            InstanciableModel[] instanciableModels = ((FilteredAdditionalResources) parentElement).getInstanciablesFromResources();
            return instanciableModels;
        }

        if (parentElement instanceof InstanciableModel)
        {
            // delegate this to InstanciableModel in case we want to add support for nested imports later.
            return ((InstanciableModel) parentElement).getChildren();
        }

        return super.getChildren(parentElement);
    }

    /**
     * @see org.topcased.modeler.editor.outline.ModelContentProvider#hasChildren(java.lang.Object)
     */
    @Override
    public boolean hasChildren(Object element)
    {
        if (element instanceof InstanciableModel)
        {
            // delegate to InstanciableModel
            boolean hasChildren = ((InstanciableModel) element).hasChildren();
            return hasChildren;
        }
        if (element instanceof FilteredAdditionalResources)
        {
            // delegate
            ((FilteredAdditionalResources) element).hasChildren();
            return (getChildren(element).length != 0);
        }

        return super.hasChildren(element);

    }

    @Override
    public Object[] getElements(Object inputElement)
    {
        if (inputElement instanceof Diagrams)
        {
            if (SamOutlinePage.DISPLAY_ADDITIONAL_RESOURCES_SECTION)
            {
                return new Object[] {EcoreUtil.getRootContainer(((Diagrams) inputElement).getModel()), getAdditionalResources(), new FilteredAdditionalResources()};
            }
            return new Object[] {EcoreUtil.getRootContainer(((Diagrams) inputElement).getModel()), new FilteredAdditionalResources()};

        }
        return super.getElements(inputElement);
    }

    /**
     * Convenience class needed for the 'instanceof' in the label/content providers for the outline navigator.
     * 
     * Nested to get access to {@link SamOutlineContentProvider#getAdditionalResources()}.
     * 
     * @author jrme
     */
    class FilteredAdditionalResources
    {
        InstanciableModel[] getInstanciablesFromResources()
        {
            ArrayList<InstanciableModel> instanciableModelsList = new ArrayList<InstanciableModel>();

            List<Resource> resources = getAdditionalResources().getResources();

            for (Resource res : resources)
            {
                if (!res.getContents().isEmpty())
                {
                    EObject resourceRoot = res.getContents().get(0);
                    if (resourceRoot instanceof Model)
                    {
                        InstanciableModel wrappedModel = new InstanciableModel(((Model) resourceRoot).getModelContent());
                        instanciableModelsList.add(wrappedModel);
                    }
                    // Root element could be a ModelContent (case of a controlled model)
                    else if (resourceRoot instanceof ModelContent)
                    {
                        InstanciableModel wrappedModel = new InstanciableModel((ModelContent) resourceRoot);
                        instanciableModelsList.add(wrappedModel);
                    }
                }
            }
            return instanciableModelsList.toArray(new InstanciableModel[0]);
        }

        public boolean hasChildren()
        {
            return (getInstanciablesFromResources().length != 0);
        }
    }

}
