/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IWrapperItemProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.IPageSite;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.editor.outline.IOutlineMenuConstants;
import org.topcased.modeler.editor.outline.ModelNavigator;
import org.topcased.modeler.sam.SAMPlugin;
import org.topcased.modeler.sam.editor.outline.SamOutlineContentProvider.FilteredAdditionalResources;
import org.topcased.modeler.sam.editor.outline.actions.LoadSamResource;
import org.topcased.modeler.sam.editor.outline.actions.RemoveInstanceAction;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.Automaton;
import org.topcased.sam.ModelContent;
import org.topcased.sam.System;

/**
 * Added this class to initialize the tree viewer with our customized
 * content/label providers (models library section) and customize the context
 * menu to forbid unsupported actions on external models.
 * 
 * @author jrme
 */
public class SamOutlineNavigator extends ModelNavigator
{
    /**
     * Constructor. Nothing special added.
     * 
     * @see ModelNavigator
     * 
     * @param parent
     * @param modeler
     * @param pageSite
     */
    public SamOutlineNavigator(Composite parent, Modeler modeler, IPageSite pageSite)
    {
        super(parent, modeler, pageSite);
    }

    /**
     * Overriden so as to provide custom content/label provider for the outline :
     * adds items of type {@link ModelLibrary} and {@link InstanciableModel} for
     * the 'Possible imports' section.
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#initProviders()
     */
    @Override
    protected void initProviders()
    {
        AdapterFactoryContentProvider adapterContentProvider = new NavigatorAdapterFactoryContentProvider(
                getModeler().getAdapterFactory());

        adapterContentProvider.inputChanged(getTreeViewer(), null, null);

        // custom content provider
        getTreeViewer().setContentProvider(new SamOutlineContentProvider(getModeler(), adapterContentProvider));

        // custom label provider
        ILabelProvider fullLabelProvider = new DecoratingLabelProvider(new SamOutlineLabelProvider(
                new AdapterFactoryLabelProvider(getModeler().getAdapterFactory())),
                SAMPlugin.getDefault().getWorkbench().getDecoratorManager().getLabelDecorator());

        getTreeViewer().setLabelProvider(fullLabelProvider);
    }

    /**
     * By default, this adds a delete action and a load additionnal resources
     * action. We don't want the additionnal resources action, and we need to
     * control the delete (eg {@link FilteredAdditionalResources} selection
     * shouldn't be deleted).
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createMultiSelectionMenu(org.eclipse.jface.action.IMenuManager,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    @Override
    protected void createMultiSelectionMenu(IMenuManager manager, IStructuredSelection selection)
    {
        Iterator<Object> iterator = selection.iterator();
        // this list contains the selection without InstanciableModel instances,
        // FilteredAdditionnalResources instances and instanciated
        // ports/multiports. It will be used to hook the delete action.

        ArrayList<Object> newSelection = new ArrayList<Object>();
        while (iterator.hasNext())
        {
            Object object = iterator.next();
            if (!(object instanceof InstanciableModel))
            {
                if (!(object instanceof FilteredAdditionalResources))
                {
                    // instances of systems and automatons can be deleted, not
                    // the ports/multiports though.
                    if (object instanceof EObject && InstancesUtil.isAReference((EObject) object))
                    {
                        if (object instanceof System || object instanceof Automaton)
                        {
                            newSelection.add(object);
                        }
                    }
                    else
                    {
                        newSelection.add(object);
                    }
                }
            }
        }
        StructuredSelection newStructuredSelection = new StructuredSelection(newSelection);

        // this is needed since createDeleteAction takes the current selection
        // directly from the tree.
        getTreeViewer().setSelection(newStructuredSelection);

        MixedEditDomain domain = (MixedEditDomain) getModeler().getAdapter(MixedEditDomain.class);
        if (domain != null)
        {
            // Add only the delete from model action, not the Add Additionnal
            // Resources one.
            IAction deleteAction = createDeleteAction();
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, deleteAction);
        }
    }

    /**
     * Convenience method returning the current selection as an EObject.
     * 
     * @return the unwrapped single selection.
     */
    private EObject getEObjectSelection()
    {
        EObject selectedObject = null;
        Object fullSelection = getTreeViewer().getSelection();
        Object selection = fullSelection;

        if (selection instanceof IStructuredSelection)
        {
            selection = ((IStructuredSelection) fullSelection).getFirstElement();
        }

        if (selection instanceof EObject)
        {
            selectedObject = (EObject) selection;
        }
        else if ((selection instanceof IWrapperItemProvider) || (selection instanceof FeatureMap.Entry))
        {
            selectedObject = (EObject) AdapterFactoryEditingDomain.unwrap(selection);
        }
        return selectedObject;
    }

    /*
     * Adds the action allowing to unwire an instanciated model.
     * 
     * Also filters out InstanciableModels, otherwise there will be some actions
     * that we don't want (eg 'add child') since for the drag and drop to work,
     * InstanciableModel had to implement IWrapperItemProvider, and
     * super.createSingleSelectionMenu works on its implementations.
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createSingleSelectionMenu(org.eclipse.jface.action.IMenuManager,
     *      java.lang.Object)
     */
    @Override
    protected void createSingleSelectionMenu(IMenuManager manager, Object selection)
    {
        // filters InstanciableModel instances.
        if (selection instanceof InstanciableModel)
        {
            return;
        }
        else
        {
            super.createSingleSelectionMenu(manager, selection);
        }

        EObject selectedObject = getEObjectSelection();

        // adds the Unwire Action for instanciated systems/automatas.
        if (InstancesUtil.isAReference(selectedObject) && selectedObject instanceof ModelContent)
        {
            MixedEditDomain domain = (MixedEditDomain) getModeler().getAdapter(MixedEditDomain.class);

            RemoveInstanceAction removeInstanceAction = new RemoveInstanceAction(domain.getEMFEditingDomain(),
                    (ModelContent) selection);
            manager.appendToGroup(IOutlineMenuConstants.EDIT_GROUP, removeInstanceAction);
            return;
        }

    }

    /*
     * Overriden so as to not show this menu for instanciated elements.
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createControlActions(org.eclipse.jface.action.IMenuManager)
     */
    @Override
    protected boolean isControlActionEnabledFor(EObject selectedObject)
    {
        if (InstancesUtil.isAReference(selectedObject))
        {
            return false;
        }
        return super.isControlActionEnabledFor(selectedObject);
    }

    /*
     * Overriden so as to not show this menu for instanciated elements.
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createDiagramsMenu(org.eclipse.jface.action.IMenuManager,
     *      org.eclipse.emf.ecore.EObject)
     */
    @Override
    protected boolean isDiagramsMenuEnabledFor(EObject selectedObject)
    {
        if (InstancesUtil.isAReference(selectedObject))
        {
            return false;
        }
        return super.isDiagramsMenuEnabledFor(selectedObject);
    }

    /*
     * Overriden so as to not show this menu for instanciated elements.
     * 
     * (non-Javadoc)
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#createEMFMenu(org.eclipse.jface.action.IMenuManager,
     *      org.eclipse.emf.ecore.EObject)
     */
    @Override
    protected boolean isEMFMenuEnabledFor(EObject selectedObject)
    {
        if (InstancesUtil.isAReference(selectedObject))
        {
            return false;
        }
        return super.isEMFMenuEnabledFor(selectedObject);
    }

    /**
     * Overriden to forbid deleting of a port or multiport in an instance of a
     * system/automaton in the Outline.
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#hookKeyListeners()
     */
    @Override
    protected void hookKeyListeners()
    {
        KeyListener keyListener = new KeyListener()
        {

            public void keyPressed(KeyEvent e)
            {
                // Do nothing
            }

            public void keyReleased(KeyEvent e)
            {
                if (e.keyCode == SWT.DEL)
                {
                    ISelection selection = getTreeViewer().getSelection();
                    if (selection instanceof IStructuredSelection)
                    {
                        // the selections without the instances
                        ArrayList<Object> newSelection = new ArrayList<Object>();

                        Iterator<Object> it = ((IStructuredSelection) selection).iterator();
                        while (it.hasNext())
                        {
                            Object o = it.next();
                            if (o instanceof EObject && InstancesUtil.isAReference((EObject) o))
                            {
                                if (o instanceof System || o instanceof Automaton)
                                {
                                    // allow the delete action for instances of
                                    // systems or automatons.
                                    newSelection.add(o);
                                }
                            }
                            else
                            {
                                newSelection.add(o);
                            }
                        }
                        getTreeViewer().setSelection(new StructuredSelection(newSelection), false);
                    }

                    IAction deleteAction = createDeleteAction();
                    deleteAction.run();
                }
            }

        };
        getTreeViewer().getControl().addKeyListener(keyListener);
    }

    /*
     * Overriding this method so as to add our 'Load Sam Instanciables Models'
     * action when the selection is a FilteredAdditionnalResources.
     * 
     * Since (non-Javadoc)
     * 
     * @see org.topcased.modeler.editor.outline.ModelNavigator#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
     */
    @Override
    public void menuAboutToShow(IMenuManager menuManager)
    {
        super.menuAboutToShow(menuManager);
        if (getTreeViewer().getSelection() instanceof IStructuredSelection)
        {
            Object selection = ((IStructuredSelection) getTreeViewer().getSelection()).getFirstElement();
            if (selection instanceof FilteredAdditionalResources)
            {
                MixedEditDomain domain = (MixedEditDomain) getModeler().getAdapter(MixedEditDomain.class);
                if (domain != null)
                {
                    // Adds our customized load resources action (only allows
                    // sam files)
                    LoadSamResource resourceAction = new LoadSamResource(domain.getEMFEditingDomain());
                    menuManager.appendToGroup(IOutlineMenuConstants.ADDITIONS_GROUP, resourceAction);
                }
            }
        }
    }
}
