/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.editor.outline.actions;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.topcased.modeler.sam.SAMPlugin;
import org.topcased.modeler.sam.dialogs.FilteredFilesResourceSelectionDialog;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.Model;
import org.topcased.sam.ModelContent;

public class LoadSamResource extends Action
{
    private static final String LOAD_SAM_RESOURCE_LABEL = "Load Instanciable Models";

    private static final String LOAD_SAM_RESOURCE_DESC = "Allows you to add SAM models to the 'Instanciable Models' section of the Outline so that they can be dragged and dropped in the current diagram.";

    private static final String LOAD_SAM_RESOURCE_DIALOG = "Choose existing SAM model files :";

    /**
     * Used for testing if we want to load the diagram associated with a sam
     * model.
     */
    private static final boolean LOAD_ASSOCIATED_DIAGRAM = false;

    protected EditingDomain domain;

    public LoadSamResource(EditingDomain domain)
    {
        this();
        if (domain == null)
        {
            throw new NullPointerException();
        }
        this.domain = domain;
        setEnabled(true);
    }

    public LoadSamResource()
    {
        super(LOAD_SAM_RESOURCE_LABEL);
        setDescription(LOAD_SAM_RESOURCE_DESC);
    }

    /*
     * Opens a custom ResourceSelectionDialog (only shows sam files) and process
     * the results : loads the associated resource into this domain resource set
     * if the resource represents a SAM model.
     * 
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    @Override
    public void run()
    {
        Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
        IAdaptable rootElement = ResourcesPlugin.getWorkspace().getRoot();

        FilteredFilesResourceSelectionDialog dialog = new FilteredFilesResourceSelectionDialog(shell, rootElement,
                LOAD_SAM_RESOURCE_DIALOG);
        dialog.setFileExtensionFilter("sam");
        if (dialog.open() == Window.OK)
        {
            Object[] results = dialog.getResult();

            // a list of the workspace relative paths of the selected sam file
            // resources.
            ArrayList<String> modelsPaths = new ArrayList<String>();
            for (Object result : results)
            {
                if (result instanceof IFile && ((IFile) result).getFileExtension().equals("sam"))
                {
                    String relativePath = ((IFile) result).getFullPath().toString();
                    modelsPaths.add(relativePath);
                }
            }
            int nbrModelsAdded = processPaths(modelsPaths.toArray(new String[0]));

            // used to correct a bug when only one resource is added : the tree
            // viewer is not refreshed properly.
            if (nbrModelsAdded == 1)
            {
                domain.getResourceSet().eNotify(new NotificationImpl(Notification.ADD, null, null));
            }
        }
    }

    /**
     * Process the paths returned by the dialog. That is, tries to load the
     * associated resource by calling
     * {@link #processSingleResource(URI, boolean)}.
     * 
     * @param pathStrings
     * @return the number of models effectively added to the resource set.
     */
    protected int processPaths(String[] pathStrings)
    {
        int nbrOfModelsAdded = 0;
        if (domain != null)
        {
            for (String path : pathStrings)
            {
                URI uri = URI.createPlatformResourceURI(path, true);
                if (InstancesUtil.isSamUri(uri))
                {
                    try
                    {
                        if (processSingleResource(uri, LOAD_ASSOCIATED_DIAGRAM))
                        {
                            ++nbrOfModelsAdded;
                        }
                    }
                    catch (RuntimeException exception)
                    {
                        SAMPlugin.log(exception);
                    }
                }
            }
        }
        else
        {
            throw new AssertionError("The EditingDomain should have been set before this.");
        }
        return nbrOfModelsAdded;
    }

    /**
     * Method used only by {@link LoadLibraryDialog#processResources()} that
     * tries to load the resource associated with the uri into the
     * {@link ResourceSet} associated with the current {@link EditingDomain}.
     * 
     * @param uri
     * @param loadAssociatedDiagram
     * @return true if the resource was added and it was that of a SAM model.
     */
    private boolean processSingleResource(URI uri, boolean loadAssociatedDiagram)
    {
        Resource newResource = domain.getResourceSet().getResource(uri, true);

        if (newResource == null)
        {
            SAMPlugin.log("The resource is null for uri : " + uri + " in LoadSamResource:processSingleResource",
                    IStatus.ERROR);
            return false;
        }
        if (newResource.getContents().isEmpty())
        {
            SAMPlugin.log("The resource is empty for uri : " + uri + " in LoadSamResource:processSingleResource",
                    IStatus.ERROR);
            domain.getResourceSet().getResources().remove(newResource);
            return false;
        }

        EObject rootElement = newResource.getContents().get(0);

        // The root element might be a Model (current case) or directly the System or Automaton element (case of a controlled model)
        if (!(rootElement instanceof Model || rootElement instanceof ModelContent))
        {
            SAMPlugin.log("The resource is not that of a sam model for uri : " + uri
                    + " in LoadSamResource:processSingleResource", IStatus.ERROR);
            domain.getResourceSet().getResources().remove(newResource);
            return false;
        }

        if (loadAssociatedDiagram)
        {
            URI diagramUri = uri.trimFileExtension().appendFileExtension("samdi");
            domain.getResourceSet().getResource(diagramUri, true);
        }
        return true;
    }
}
