/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.commands;

import java.util.Iterator;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.PortEditPart;
import org.topcased.modeler.figures.ModelerLabelledPortFigure;
import org.topcased.modeler.figures.ModelerPortFigure;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.InputPort;
import org.topcased.sam.OutputPort;

/**
 * This command add a move command for each Port contained by the EditPart
 * container. The new position is calculated depending on the current position
 * of the Port (on which border it is attached) and the zoom value (the
 * moveDelta is not the same as the resizeDelta which does not care about the
 * current zoom).
 * 
 * Created 8 juin 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 * @deprecated Since the use of the BorderAttachedLayout, this command is not
 *             useful
 */
public class ResizePortContainerCommand extends CompoundCommand
{
    /**
     * The Constructor
     * 
     * Determine the new deltas depending on the current zoom and apply it to
     * all the Port
     * 
     * @param container the EditPart that contain the Ports
     * @param sizeDelta the Dimension Delta of the resize command
     */
    public ResizePortContainerCommand(EditPart container, Dimension sizeDelta)
    {
        super("Resize a container that has children of type Port");
        
        // get the current zoom value
        double zoom = ((ScalableFreeformRootEditPart) container.getViewer().getRootEditPart()).getZoomManager().getZoom();
        double widthDeltaZoomed = sizeDelta.width / zoom;
        double heightDeltaZoomed = sizeDelta.height / zoom;
        
        // iterates on all Ports to move them so that they keep their attachment
        // to the borders
        Iterator it = container.getChildren().iterator();
        while (it.hasNext())
        {
            PortEditPart portEditPart = (PortEditPart) it.next();
            GraphNode portNode = (GraphNode) portEditPart.getModel();
            
            // the initial Port constraint
            Rectangle portConstraint = new Rectangle(portNode.getPosition(), portNode.getSize());
            // calculate the new location
            portConstraint.setLocation(calculatePortPosition(widthDeltaZoomed, heightDeltaZoomed, portEditPart,
                    portConstraint));
            add(new ChangeBoundsCommand(portNode, portConstraint));
        }
    }
    
    /**
     * Determine the Point where to move the Port
     * 
     * @param widthDeltaZoomed the width delta of the resizing command with the
     *        current zoom value
     * @param heightDeltaZoomed the width delta of the resizing command with the
     *        current zoom value
     * @param portEditPart the PortEditPart
     * @param portConstraint the initial constraint at the beginning of the
     *        resize command
     * 
     * @return the new Point where to move the Port
     */
    private Point calculatePortPosition(double widthDeltaZoomed, double heightDeltaZoomed, PortEditPart portEditPart,
            Rectangle portConstraint)
    {
        EObject portEObject = Utils.getElement((GraphNode) portEditPart.getModel());
        GraphicalEditPart container = (GraphicalEditPart) portEditPart.getParent();
        
        // determine the multiplicator factor to keep the port position relative
        // to the container size
        float factorWidth = 1.0f + ((float) widthDeltaZoomed / container.getContentPane().getBounds().width);
        float factorHeight = 1.0f + ((float) heightDeltaZoomed / container.getContentPane().getBounds().height);
        
        ModelerPortFigure portFigure = null;
        if (portEditPart.getFigure() instanceof ModelerLabelledPortFigure)
        {
            portFigure = ((ModelerLabelledPortFigure) portEditPart.getFigure()).getPortFigure();
        }
        else
        {
            portFigure = (ModelerPortFigure) portEditPart.getFigure();
        }
        
        switch (portFigure.getPosition())
        {
            case PositionConstants.LEFT:
                return new Point(portConstraint.x, portConstraint.y * factorHeight);
            case PositionConstants.RIGHT:
                return new Point(portConstraint.x + widthDeltaZoomed, portConstraint.y * factorHeight);
            case PositionConstants.TOP:
                return new Point(portConstraint.x * factorWidth, portConstraint.y);
            case PositionConstants.BOTTOM:
                return new Point(portConstraint.x * factorWidth, portConstraint.y + heightDeltaZoomed);
        }
        
        // the port is attached to the left border
        if (portEObject instanceof InputPort && portFigure.getPosition() == PositionConstants.LEFT
                || portEObject instanceof OutputPort && portFigure.getPosition() == PositionConstants.RIGHT)
            return new Point(portConstraint.x, portConstraint.y * factorHeight);
        
        // the port is attached to the right border
        if (portEObject instanceof InputPort && portFigure.getPosition() == PositionConstants.RIGHT
                || portEObject instanceof OutputPort && portFigure.getPosition() == PositionConstants.LEFT)
            return new Point(portConstraint.x + widthDeltaZoomed, portConstraint.y * factorHeight);
        
        // the port is attached to the bottom border
        if (portEObject instanceof InputPort && portFigure.getPosition() == PositionConstants.BOTTOM
                || portEObject instanceof OutputPort && portFigure.getPosition() == PositionConstants.TOP)
            return new Point(portConstraint.x * factorWidth, portConstraint.y + heightDeltaZoomed);
        
        return portConstraint.getLocation();
    }
}
