/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.DeleteGraphElementCommand;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Automaton;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;
import org.topcased.sam.System;

/**
 * This command is used to restore Ports contained in a System or Automaton
 * model object
 * 
 * Creation : 17 nov. 2005
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class RestorePortsCommand extends CompoundCommand
{
    /** The GraphNode that should be updated */
    private GraphNode graphNode;
    
    /** The EditDomain associated */
    private EditDomain editDomain;
    
    private ICreationUtils creationUtils;
    
    /**
     * The Constructor
     * 
     * @param domain the EditDomain
     * @param gNode the GraphNode
     * @param creationUtils the creationUtils factory
     */
    public RestorePortsCommand(EditDomain domain, GraphNode gNode, ICreationUtils creationUtils)
    {
        super("Refresh Ports Command");
        
        this.graphNode = gNode;
        this.editDomain = domain;
        this.creationUtils = creationUtils;
        
        initialize();
    }
    
    private void initialize()
    {
        EObject eObject = Utils.getElement(graphNode);
        
        // a counter that is used to place the features that are not already
        // present
        int yPos = 10;
        
        List listPorts = new ArrayList();
        List listMultiPorts = new ArrayList();
        
        if (eObject instanceof System)
        {
            listPorts = ((System) eObject).getListPorts();
            listMultiPorts = ((System) eObject).getListMultiPort();
        }
        else if (eObject instanceof Automaton)
        {
            listPorts = ((Automaton) eObject).getListPorts();
            listMultiPorts = ((Automaton) eObject).getListMultiPort();
        }
        else if (eObject instanceof MultiPort)
        {
            listPorts = ((MultiPort) eObject).getListPort();
        }
        
        restorePortsAndMultiPorts(yPos, listPorts, listMultiPorts);
    }
    
    private void restorePortsAndMultiPorts(int yPos, List listPorts, List listMultiPorts)
    {
        if (listPorts != null)
        {
            for (Iterator iter = listPorts.iterator(); iter.hasNext();)
            {
                // retrieve the ports to display
                Port port = (Port) iter.next();
                
                // check if the element is already present
                GraphNode node = getGraphicalRepresentation(port);
                
                if (node == null)
                {
                    // create the new graphNode and add it to the parentNode
                    node = (GraphNode) creationUtils.createGraphElement(port);
                    add(new CreateGraphNodeCommand(editDomain, node, graphNode, new Point(1, yPos), node.getSize(),
                            PositionConstants.LEFT));
                    yPos = yPos + 40;
                }
            }
            
            // Delete nodes that are not associated with a Port any more
            for (Iterator iter = graphNode.getContained().iterator(); iter.hasNext();)
            {
                Object obj = iter.next();
                if (obj instanceof GraphNode)
                {
                    GraphNode node = (GraphNode) obj;
                    EObject eObject = Utils.getElement(node);
                    // only threat the nodes associated with a Port
                    if (eObject instanceof Port && !listPorts.contains(eObject))
                    {
                        add(new DeleteGraphElementCommand(node, true));
                    }
                }
            }
        }
        
        if (listMultiPorts != null)
        {
            for (Iterator iter = listMultiPorts.iterator(); iter.hasNext();)
            {
                // retrieve the multiPorts to display
                MultiPort multiPort = (MultiPort) iter.next();
                
                // check if the element is already present
                GraphNode node = getGraphicalRepresentation(multiPort);
                
                if (node == null)
                {
                    // create the new graphNode and add it to the parentNode
                    node = (GraphNode) creationUtils.createGraphElement(multiPort);
                    add(new CreateGraphNodeCommand(editDomain, node, graphNode, new Point(1, yPos), node.getSize(),
                            PositionConstants.LEFT));
                    yPos = yPos + 30 + multiPort.getListPort().size() * 22;
                }
            }
            
            // Delete nodes that are not associated with a MultiPort any more
            for (Iterator iter = graphNode.getContained().iterator(); iter.hasNext();)
            {
                Object obj = iter.next();
                if (obj instanceof GraphNode)
                {
                    GraphNode node = (GraphNode) obj;
                    EObject eObject = Utils.getElement(node);
                    // only threat the nodes associated with a MultiPort
                    if (eObject instanceof MultiPort && !listMultiPorts.contains(eObject))
                    {
                        add(new DeleteGraphElementCommand(node, true));
                    }
                }
            }
        }
    }
    
    /**
     * Get the graphNode associated with the model object in the current
     * selected editPart
     * 
     * @param modelObject the model object to display
     * @return the GraphNode or null if it does not exist
     */
    private GraphNode getGraphicalRepresentation(EObject modelObject)
    {
        for (Iterator iter = graphNode.getContained().iterator(); iter.hasNext();)
        {
            GraphElement elt = (GraphElement) iter.next();
            if (Utils.getElement(elt) == modelObject && elt instanceof GraphNode)
            {
                return (GraphNode) elt;
            }
        }
        return null;
    }
    
}
