/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 ******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.edit;

import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.workspace.util.WorkspaceSynchronizer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.sam.SAMPlugin;
import org.topcased.modeler.sam.util.InstancesUtil;
import org.topcased.sam.ModelContent;

public class OpenReferencedModel extends Action
{
    private static final String ERROR_TITLE = "Error opening diagram";

    private static final String ERROR_NO_DIAGRAMS_FILE = "The imported model doesn't have an associated '.samdi' diagrams file";

    private static final String ERROR_INTERNAL = "Could not open diagram!";

    private final ModelContent referencedObject;

    private final ModelContent referencerObject;

    private final Modeler modeler;

    public OpenReferencedModel(Modeler modeler, ModelContent referencerObject, ModelContent referencedObject)
    {
        this.referencedObject = referencedObject;
        this.referencerObject = referencerObject;
        this.modeler = modeler;
    }

    @Override
    public void run()
    {
        final Resource referencerResource = referencerObject.eResource();
        final Resource referencedResource = referencedObject.eResource();

        // find the '.samdi' associated with the referencedSystem Model.
        URI diagramUri = InstancesUtil.getSamdiUriFromSamUri(referencedResource.getURI());

        // try to load it in the resources of the currently edited model
        // (the one containing the reference to the existing system)
        final ResourceSet referencerResourceSet = referencerResource.getResourceSet();

        Resource referencedSystemDiagramResource = null;

        try
        {
            referencedSystemDiagramResource = referencerResourceSet.getResource(diagramUri, true);
        }
        catch (RuntimeException e)
        {
            // do that since the first time, an exception is always raised
            // (don't know why)
            // but it works the second time.

            try
            {
                referencedSystemDiagramResource = referencerResourceSet.getResource(diagramUri, true);
            }
            catch (RuntimeException f)
            {
            }
        }

        if (referencedSystemDiagramResource != null)
        {
            IFile diagramsFile = WorkspaceSynchronizer.getFile(referencedSystemDiagramResource);
            IWorkbenchPage page = modeler.getSite().getPage();
            if (diagramsFile != null)
            {
                try
                {
                    IEditorPart newEditor = IDE.openEditor(page, diagramsFile, true);

                    if (newEditor instanceof Modeler)
                    {
                        Modeler newModeler = (Modeler) newEditor;

                        Diagrams rootDiagrams = (Diagrams) referencedSystemDiagramResource.getContents().get(0);
                        Diagrams[] subDiagrams = rootDiagrams.getSubdiagrams().toArray(new Diagrams[0]);
                        for (Diagrams diagrams : subDiagrams)
                        {
                            for (Diagram diagram : diagrams.getDiagrams())
                            {
                                EObject correspondingElement = ((EMFSemanticModelBridge) diagram.getSemanticModel()).getElement();
                                if ((correspondingElement instanceof ModelContent)
                                        && (correspondingElement == referencedObject))
                                {
                                    final List navigatorDiagrams = DiagramsUtils.findAllDiagrams(DiagramsUtils.findRootDiagrams((Diagrams) diagram.eContainer()));
                                    final List editorDiagrams = DiagramsUtils.findAllDiagrams(newModeler.getDiagrams());
                                    newModeler.setActiveDiagram((Diagram) editorDiagrams.get(navigatorDiagrams.indexOf(diagram)));
                                    return;
                                }
                            }
                        }
                    }
                }
                catch (PartInitException pie)
                {
                    SAMPlugin.log(pie);
                    SAMPlugin.log(ERROR_INTERNAL, IStatus.ERROR);
                    MessageDialog.openError(Display.getDefault().getActiveShell(), ERROR_TITLE, ERROR_INTERNAL);
                    return;
                }
            }

        } else 
        {
        // TODO should open the diagram creation dialog instead
        Shell shell = Display.getDefault().getActiveShell();
        MessageDialog.openError(shell, ERROR_TITLE, ERROR_NO_DIAGRAMS_FILE);

        // remove the diagram resource that was created (not a valid one, leads
        // to problems when trying to open it)
        referencerResourceSet.getResources().remove(referencerResourceSet.getResources().size() - 1);
        }
        return;

    }
}