/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.sam.systemdiagram.policies;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.sam.systemdiagram.commands.CreateFlowCommand;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.ControlFlow;
import org.topcased.sam.ControlPort;
import org.topcased.sam.DataFlow;
import org.topcased.sam.DataPort;
import org.topcased.sam.InputPort;
import org.topcased.sam.MultiPort;
import org.topcased.sam.OutputPort;
import org.topcased.sam.Port;

/**
 * Provides the EditPolicies which make the command when the user make a
 * connection between two Ports
 * 
 * @author jako
 */
public class FlowCreationEditPolicy extends GraphicalNodeEditPolicy
{
    /**
     * Update the request with the source element of the command
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCreateCommand(CreateConnectionRequest request)
    {
        Object newObject = request.getNewObject();
        if (newObject instanceof GraphEdge)
        {
            GraphEdge edge = (GraphEdge) newObject;
            GraphNode sourceNode = (GraphNode) getHost().getModel();
            
            // check if the port is compatible with the type of Flow
            if (Utils.getElement(edge) instanceof DataFlow && Utils.getElement(sourceNode) instanceof DataPort
                    || Utils.getElement(edge) instanceof ControlFlow
                    && Utils.getElement(sourceNode) instanceof ControlPort)
            {
                // check if the source node is compatible with its type (Input
                // or Output)
                if (isDiagramObject(sourceNode) && Utils.getElement(sourceNode) instanceof InputPort
                        || !isDiagramObject(sourceNode) && Utils.getElement(sourceNode) instanceof OutputPort)
                {
                    // create the command associated with these parameters
                    CreateFlowCommand command = new CreateFlowCommand(getHost().getViewer().getEditDomain(), edge,
                            sourceNode, true);
                    request.setStartCommand(command);
                    return command;
                }
            }
        }
        return null;
    }
    
    /**
     * Determine if a GraphNode should be considered as a "direct" child of the
     * Diagram
     * 
     * @param node the GraphNode
     * @return true if the GraphNode must be considered as a direct child of the
     *         Diagram
     */
    private boolean isDiagramObject(GraphNode node)
    {
        return node.getContainer() instanceof Diagram
                || (Utils.getElement(node.getContainer()) instanceof MultiPort && node.getContainer().getContainer() instanceof Diagram);
    }
    
    /**
     * Return the command associated with the target element
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
     */
    protected Command getConnectionCompleteCommand(CreateConnectionRequest request)
    {
        // check if the command created from the source node is an command to
        // create an EReference
        if (request.getStartCommand() instanceof CreateFlowCommand)
        {
            // get the two nodes that will be connected
            GraphNode targetNode = (GraphNode) getHost().getModel();
            GraphNode sourceNode = (GraphNode) request.getSourceEditPart().getModel();
            
            // can not make a flow from and to the same Port
            if (targetNode != sourceNode)
            {
                // check if the two ports are compatible with the type of flow
                if (Utils.getElement(sourceNode) instanceof DataPort
                        && Utils.getElement(targetNode) instanceof DataPort
                        || Utils.getElement(sourceNode) instanceof ControlPort
                        && Utils.getElement(targetNode) instanceof ControlPort)
                {
                    // check if the target node is compatible with its type
                    // (Input or Output)
                    if (Utils.getElement(targetNode) instanceof InputPort && !isDiagramObject(targetNode)
                            || Utils.getElement(targetNode) instanceof OutputPort && isDiagramObject(targetNode))
                    {
                        // check if the target node is not already connected to
                        // an existing flow
                        if (((Port) Utils.getElement(targetNode)).getInlink() == null)
                        {
                            CreateFlowCommand command = (CreateFlowCommand) request.getStartCommand();
                            if (command != null)
                                // set the target node
                                command.setTarget(targetNode);
                            
                            return command;
                        }
                    }
                }
            }
        }
        return null;
    }
    
    /**
     * not implemented
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectSourceCommand(ReconnectRequest request)
    {
        return UnexecutableCommand.INSTANCE;
    }
    
    /**
     * not implemented
     * 
     * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
     */
    protected Command getReconnectTargetCommand(ReconnectRequest request)
    {
        return UnexecutableCommand.INSTANCE;
    }
    
}
