/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Alfredo Serrano
 * (Anyware Technologies)- initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.sam.util;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.common.ui.dialogs.ResourceDialog;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.SemanticModelBridge;
import org.topcased.modeler.editor.AbstractCreationUtils;
import org.topcased.modeler.editor.IConfiguration;
import org.topcased.modeler.extensions.DiagramsManager;
import org.topcased.modeler.utils.Utils;
import org.topcased.sam.Automaton;
import org.topcased.sam.ControlPort;
import org.topcased.sam.DataPort;
import org.topcased.sam.InputPort;
import org.topcased.sam.ModelContent;
import org.topcased.sam.MultiPort;
import org.topcased.sam.OutputPort;
import org.topcased.sam.Port;
import org.topcased.sam.SAMFactory;
import org.topcased.sam.System;

/**
 * Library of helper functions to deal with instances creation/checking in a
 * "generic" way.
 * 
 * @author jrme
 */
public class InstancesUtil
{

    /**
     * Checks wether the argument has its IsInstanceOf reference set.
     * 
     * @param object
     * @return
     */
    public static boolean isAReference(EObject object)
    {
        if ((object instanceof System) && (((System) object).getIsInstanceOf() != null))
        {
            return true;
        }

        if ((object instanceof Automaton) && (((Automaton) object).getIsInstanceOf() != null))
        {
            return true;
        }

        if ((object instanceof Port) && (((Port) object).getIsInstanceOf() != null))
        {
            return true;
        }
        if ((object instanceof MultiPort) && (((MultiPort) object).getIsInstanceOf() != null))
        {
            return true;
        }
        return false;

    }

    /**
     * Checks that the element to import is a ModelContent and that it is at the
     * top level of the hierarchy.
     * 
     * @param elementToImport
     * @return
     */
    public static boolean isAnImport(EObject elementToImport)
    {
        if (elementToImport instanceof ModelContent)
        {
            EObject objectRoot = EcoreUtil.getRootContainer(elementToImport);
            List<EObject> contents = objectRoot.eContents();
            // When the model is Controlled, the System or Automaton could be the root element
            if (contents.contains(elementToImport) || objectRoot.equals(elementToImport))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Creates a new empty Port and set its name to that of the existing port.
     * 
     * @param existingPort
     * @return
     */
    public static Port createEmptyCopyPort(Port existingPort)
    {
        Port newPort = null;
        if (existingPort instanceof ControlPort)
        {
            if (existingPort instanceof InputPort)
            {
                newPort = SAMFactory.eINSTANCE.createInControlPort();
            }
            else if (existingPort instanceof OutputPort)
            {
                newPort = SAMFactory.eINSTANCE.createOutControlPort();
            }
            else
            {
                throw new AssertionError();
            }
        }
        else if (existingPort instanceof DataPort)
        {
            if (existingPort instanceof InputPort)
            {
                newPort = SAMFactory.eINSTANCE.createInDataPort();
            }
            else if (existingPort instanceof OutputPort)
            {
                newPort = SAMFactory.eINSTANCE.createOutDataPort();
            }
            else
            {
                throw new AssertionError();
            }
        }
        newPort.setName(existingPort.getName());
        return newPort;
    }

    /**
     * Creates a new multiport and sets its name to that of the existing port.
     * 
     * @param existingPort
     * @return
     */
    public static MultiPort createEmptyCopyMultiPort(MultiPort existingPort)
    {
        MultiPort newPort = SAMFactory.eINSTANCE.createMultiPort();
        newPort.setName(existingPort.getName());
        return newPort;
    }

    /**
     * Convenience method returning the corresponding emf element.
     * 
     * @param graphElement
     * @return
     */
    public static EObject getEmfElement(GraphElement graphElement)
    {
        SemanticModelBridge bridge = graphElement.getSemanticModel();
        if (bridge instanceof EMFSemanticModelBridge)
        {
            return ((EMFSemanticModelBridge) bridge).getElement();
        }
        return null;
    }

    public static List<Port> getListPorts(ModelContent content)
    {
        if (content instanceof System)
        {
            return ((System) content).getListPorts();
        }
        return ((Automaton) content).getListPorts();
    }

    public static List<MultiPort> getListMultiPorts(ModelContent content)
    {
        if (content instanceof System)
        {
            return ((System) content).getListMultiPort();
        }
        return ((Automaton) content).getListMultiPort();

    }

    /**
     * Used to create the ports in the referencer according to those found in
     * the referenced. Works not only for creation, but also for keeping the
     * referencer and the referenced ports in synch after a modification of the
     * external model.
     * 
     * 
     * @param referencer
     * @param referenced
     */
    private static void updatePorts(ModelContent referencer, ModelContent referenced)
    {
        if (((referencer instanceof System) && (referenced instanceof Automaton))
                || ((referencer instanceof Automaton) && (referenced instanceof System)))
        {
            throw new AssertionError();
        }
        if (!InstancesUtil.isAReference(referencer))
        {
            throw new IllegalArgumentException("Not an instance.");
        }

        // Ports update.
        List<Port> listPortReferencer = InstancesUtil.getListPorts(referencer);
        List<Port> listPortReferenced = InstancesUtil.getListPorts(referenced);

        // check that all the ports in the referenced system are represented
        // in the instance
        for (Port port : listPortReferenced)
        {
            // if the port is not represented in the referencer
            if (!InstancesUtil.containsAReferenceToPort(listPortReferencer, port))
            {
                // create a new one
                Port newPort = InstancesUtil.createEmptyCopyPort(port);
                newPort.setIsInstanceOf(port);
                if (referencer instanceof System)
                {
                    newPort.setParentSystem((System) referencer);
                }
                else
                {
                    newPort.setParentAutomaton((Automaton) referencer);
                }
            }
        }

        // using a different list for removing ports since we're using the
        // list now.
        List<Port> listReferencerPortsToBeRemoved = new ArrayList<Port>();

        // check that no port has been deleted in the external model
        for (Port port : listPortReferencer)
        {
            // if the port is not an actual reference to an existing port in
            // the external model
            if (!listPortReferenced.contains(port.getIsInstanceOf()))
            {
                // add this port to the list of ports to be removed
                listReferencerPortsToBeRemoved.add(port);
            }
        }

        // actually remove the ports
        for (Port port : listReferencerPortsToBeRemoved)
        {
            InstancesUtil.getListPorts(referencer).remove(port);
        }
    }

    /**
     * Used to create the multiports in the referencer according to those found
     * in the referenced. Works not only for creation, but also for keeping the
     * referencer and the referenced multiports in synch after a modification of
     * the external model.
     * 
     * A copy/paste of {@link #updatePorts(ModelContent, ModelContent)}.
     * 
     * @param referencer
     * @param referenced
     */
    private static void updateMultiPorts(ModelContent referencer, ModelContent referenced)
    {
        if (((referencer instanceof System) && (referenced instanceof Automaton))
                || ((referencer instanceof Automaton) && (referenced instanceof System)))
        {
            throw new AssertionError();
        }
        if (!InstancesUtil.isAReference(referencer))
        {
            throw new IllegalArgumentException("Not an instance.");
        }

        // update the multiports
        List<MultiPort> listMultiportReferencer = InstancesUtil.getListMultiPorts(referencer);
        List<MultiPort> listMultiportReferenced = InstancesUtil.getListMultiPorts(referenced);

        // check that all the ports in the referenced system are represented
        // in the instance
        for (MultiPort port : listMultiportReferenced)
        {
            // if the port is not represented in the referencer
            if (!InstancesUtil.containsAReferenceToMultiPort(listMultiportReferencer, port))
            {
                // create a new one
                MultiPort newPort = InstancesUtil.createEmptyCopyMultiPort(port);
                newPort.setIsInstanceOf(port);
                newPort.setParent(referencer);
            }
        }

        // using a different list for removing ports since we're using the
        // list now.
        List<MultiPort> listReferencerMultiPortsToBeRemoved = new ArrayList<MultiPort>();

        // check that no port has been deleted in the external model
        for (MultiPort port : listMultiportReferencer)
        {
            // if the port is not an actual reference to an existing port in
            // the external model
            if (!listMultiportReferenced.contains(port.getIsInstanceOf()))
            {
                // add this port to the list of ports to be removed
                listReferencerMultiPortsToBeRemoved.add(port);
            }
        }

        // actually remove the multiports
        for (MultiPort port : listReferencerMultiPortsToBeRemoved)
        {
            referencer.getListMultiPort().remove(port);
        }
    }

    /**
     * Used to update the multiports/ports relationships in the referencer
     * system in function of those in the referenced.
     * 
     * Assumes that the ports and multiports were updated before calling this
     * method.
     * 
     * @param referencer
     * @param referenced
     */
    private static void updateCrossReferencesPortsMultiports(ModelContent referencer, ModelContent referenced)
    {
        if (((referencer instanceof System) && (referenced instanceof Automaton))
                || ((referencer instanceof Automaton) && (referenced instanceof System)))
        {
            throw new AssertionError();
        }
        if (!InstancesUtil.isAReference(referencer))
        {
            throw new IllegalArgumentException("Not an instance.");
        }

        List<MultiPort> listMultiportReferencer = InstancesUtil.getListMultiPorts(referencer);

        List<Port> listPortReferencer = InstancesUtil.getListPorts(referencer);

        for (MultiPort multiPort : listMultiportReferencer)
        {
            MultiPort referencedMultiport = multiPort.getIsInstanceOf();
            for (Port childOfReferencedMultiport : referencedMultiport.getListPort())
            {

                for (Port port : listPortReferencer)
                {
                    if (port.getIsInstanceOf() == childOfReferencedMultiport)
                    {
                        port.setParentMultiPort(multiPort);
                    }
                }

            }
        }
    }

    /**
     * Updates all the ports and multiports of the referencer according to
     * what's found in the referenced.
     * 
     * @param referencer
     * @param referenced
     */
    public static void updateAllContents(ModelContent referencer, ModelContent referenced)
    {
        if (((referencer instanceof System) && (referenced instanceof Automaton))
                || ((referencer instanceof Automaton) && (referenced instanceof System)))
        {
            throw new AssertionError();
        }
        if (!InstancesUtil.isAReference(referencer))
        {
            throw new IllegalArgumentException("Not an instance.");
        }

        updatePorts(referencer, referenced);
        updateMultiPorts(referencer, referenced);
        updateCrossReferencesPortsMultiports(referencer, referenced);

    }

    /**
     * Returns wether the referenced port can be found in the IsInstanceOf
     * references of the port list.
     * 
     * @param listPort the list of ports that can contain a reference to the
     *            referenced port.
     * 
     * @param referenced
     * @return
     */
    private static boolean containsAReferenceToPort(List<Port> listPort, Port referenced)
    {
        if ((listPort == null) || (referenced == null))
        {
            throw new NullPointerException();
        }

        for (Port port : listPort)
        {
            if (port.getIsInstanceOf() == referenced)
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns wether the referenced multiport can be found in the IsInstanceOf
     * references of the given multiport list.
     * 
     * @param listPort the list of multiports that can contain a reference to
     *            the referenced multiport.
     * 
     * @param referenced
     * @return
     */
    private static boolean containsAReferenceToMultiPort(List<MultiPort> listPort, MultiPort referenced)
    {
        if ((listPort == null) || (referenced == null))
        {
            throw new NullPointerException();
        }

        for (MultiPort port : listPort)
        {
            if (port.getIsInstanceOf() == referenced)
            {
                return true;
            }
        }
        return false;
    }

    /**
     * An empty copy of the ModelContent given in parameter.
     * 
     * @param model
     * @return
     */
    public static ModelContent createModelContentCopy(ModelContent model)
    {
        if (model instanceof System)
        {
            return SAMFactory.eINSTANCE.createSystem();
        }
        else if (model instanceof Automaton)
        {
            return SAMFactory.eINSTANCE.createAutomaton();
        }
        throw new IllegalArgumentException("Only accepts a ModelContent : System or Automaton");
    }

    /**
     * @param node
     * @return the configuration for the given node
     */
    public static IConfiguration getconfiguration(GraphNode node)
    {
        IConfiguration configuration = null;
        try
        {
            AbstractCreationUtils utils;
            configuration = DiagramsManager.getInstance().getConfiguration(
                    Utils.getDiagram(node).getSemanticModel().getPresentation());

        }
        catch (CoreException e)
        {

        }
        return configuration;
    }

    /**
     * Used to get the uri to a '.samdi' file corresponding to the '.sam' file
     * given in parameter.
     * 
     * @param samUri
     * @return
     */
    public static URI getSamdiUriFromSamUri(URI samUri)
    {
        final URI diagramUri = samUri.trimFileExtension().appendFileExtension("samdi");
        return diagramUri;
    }

    /**
     * Sets the name and instanceOf properties of the referencer aproprietly.
     * 
     * @param referencer
     * @param referenced
     */
    public static void setIsInstanceOf(EObject referencer, EObject referenced)
    {
        if (!((referencer instanceof ModelContent) && (referenced instanceof ModelContent)))
        {
            throw new IllegalArgumentException("The arguments have to be ModelContent.");
        }

        if ((referenced instanceof System) && (referencer instanceof System))
        {
            ((System) referencer).setIsInstanceOf((System) referenced);
        }
        else if ((referenced instanceof Automaton) && (referencer instanceof Automaton))
        {
            ((Automaton) referencer).setIsInstanceOf((Automaton) referenced);
        }
        else
        {
            throw new AssertionError("ModelContent subclasses are Automaton and System.");
        }

    }

    /**
     * Convenience method testing wether a given uri (as returned by
     * {@link ResourceDialog#getURIs()}, ie either a 'platform' or a 'file'
     * uri) points to a local sam file.
     * 
     * @param uri
     * @return true if the uri represents a '.sam' file, false otherwise.
     */
    public static boolean isSamUri(URI uri)
    {
        String path = null;

        if (uri.isFile())
        {
            path = uri.toFileString();
        }
        else if (uri.isPlatform())
        {
            path = uri.toPlatformString(true);
        }

        if (path != null)
        {
            if (path.substring(path.length() - 4).equals(".sam"))
            {
                return true;
            }
        }
        return false;
    }

}
