/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.commands.update;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.modeler.sysml.blockdefinitiondiagram.dialogs.OperationEditDialog;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.OperationDataStructure;

/**
 * Class that create a command in order to update operation parameters <br>
 * creation : 8 avr. 2005
 */
public class OperationUpdateCommand extends Command
{
    /** Current operation */
    private Operation operation;

    /** Old values */
    private String oldName;

    private VisibilityKind oldVisibility;

    private Type oldReturnType;

    private OperationDataStructure oldInputTypes;

    /** New values */
    private String name;

    private VisibilityKind visibility;

    private Type returnType;

    private OperationDataStructure inputTypes;

    /**
     * Create a command to update parameters, name, etc... on a given operation
     * 
     * @param op the Operation to update
     * @param data the map containing the new values
     */
    public OperationUpdateCommand(Operation op, Map< ? , ? > data)
    {
        operation = op;

        // Store new data
        name = (String) data.get(OperationEditDialog.OPERATION_NAME);
        visibility = (VisibilityKind) data.get(OperationEditDialog.OPERATION_VISIBILITY);
        returnType = (Type) data.get(OperationEditDialog.OPERATION_RETURN_TYPE);
        inputTypes = (OperationDataStructure) data.get(OperationEditDialog.OPERATION_INPUTS);
    }

    /**
     * Get the old values
     */
    protected void getOldValues()
    {
        // Store old data
        oldName = operation.getName();
        oldVisibility = operation.getVisibility();

        oldReturnType = operation.getType();
        oldInputTypes = new OperationDataStructure(operation);
    }

    /**
     * Set the values on the Operation
     */
    protected void setValues()
    {
        // Perform update for main data
        operation.setName(name);
        operation.setVisibility(visibility);

        // Perform update for input parameters
        List newParameters = new ArrayList();
        Iterator< ? > iterator = inputTypes.getData().iterator();
        while (iterator.hasNext())
        {
            Object object = iterator.next();
            String displayName = inputTypes.getDisplayName(object);
            Type type = inputTypes.getType(object);
            ParameterDirectionKind direction = inputTypes.getDirection(object);
            Parameter parameter = UMLFactory.eINSTANCE.createParameter();
            parameter.setName(displayName);
            parameter.setType(type);
            parameter.setDirection(direction);
            newParameters.add(parameter);
        }
        operation.getOwnedParameters().clear();
        operation.getOwnedParameters().addAll(newParameters);

        // Perform update for return type data
        if (returnType != null)
        {
            operation.createReturnResult("return", returnType);
        }
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        // Reverse old and new value for name
        String tempName = name;
        name = oldName;
        oldName = tempName;

        // Reverse old and new value for visibility
        VisibilityKind tempVisibility = visibility;
        visibility = oldVisibility;
        oldVisibility = tempVisibility;

        // Reverse old and new values for type
        Type tempType = returnType;
        returnType = oldReturnType;
        oldReturnType = tempType;

        // Reverse old and new values for inputType
        OperationDataStructure tempInputTypes = inputTypes;
        inputTypes = oldInputTypes;
        oldInputTypes = tempInputTypes;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}
