/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.dialogs;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.VisibilityKind;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.facilities.widgets.SingleObjectChooser;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.BlockTools;
import org.topcased.modeler.sysml.blockdefinitiondiagram.util.OperationDataStructure;
import org.topcased.sysml.provider.SysMLItemProviderAdapterFactory;

/**
 * Updating operation parameters
 */
public class OperationEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** The ID of the operation name */
    public static final String OPERATION_NAME = "operation name";

    /** The ID of the operation visibility */
    public static final String OPERATION_VISIBILITY = "operation visibility";

    /** The ID of the operation return type */
    public static final String OPERATION_RETURN_TYPE = "operation return type";

    /** The ID of the operation inputs */
    public static final String OPERATION_INPUTS = "operation inputs";

    private OperationParameterViewer inputParameters;

    /** Current edited operation */
    private Operation operation;

    // SWT Objects
    private SingleObjectChooser returnTypeChooser;

    private Text operationNameTxt;

    private ComboViewer visibilityCb;

    private Button checkReturnBt;

    // UML2 types data
    private String[] typeNames;

    private Type[] types;

    private Map data;

    /**
     * Create the dialog for a given operation
     * 
     * @param op the operation to edit
     * @param parentShell the owning shell
     */
    public OperationEditDialog(Operation op, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        operation = op;
        initializeTypes();
    }

    /**
     * Set the title of the shell
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Operation " + operation.getName());
        // newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        Composite dialogComposite = (Composite) super.createDialogArea(parent);
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;

        createOperationGroup(dialogComposite);
        hookListeners();
        loadData();

        return dialogComposite;
    }

    /**
     * Initialize attributes about types that model currently contains
     * 
     */
    private void initializeTypes()
    {
        int index = 0;
        Collection< ? > typesCollection = BlockTools.removeRelationship(BlockTools.getAllAvailableTypes(operation));
        types = new Type[typesCollection.size()];

        Iterator< ? > itTypes = typesCollection.iterator();
        while (itTypes.hasNext())
        {
            Type type = (Type) itTypes.next();
            types[index] = type;
            index++;
        }
        typeNames = BlockTools.getAllAvailableTypeNames(typesCollection);
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    private void createOperationGroup(Composite parent)
    {
        TabFolder tabFolder = new TabFolder(parent, SWT.TOP);
        tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));

        createOperationTabItem(tabFolder);
        createParametersTabItem(tabFolder);
    }

    /**
     * Create the first tab
     * 
     * @param parent the owning tab folder
     */
    private void createOperationTabItem(TabFolder parent)
    {

        // Create tab item and add it composite that fills it
        TabItem firstItem = new TabItem(parent, SWT.NONE);
        firstItem.setText("General Informations");
        Composite composite = new Composite(parent, SWT.NONE);
        firstItem.setControl(composite);

        // Add layout on composite
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        Label operationNameLbl = new Label(composite, SWT.NONE);
        operationNameLbl.setText("Name : ");
        operationNameTxt = new Text(composite, SWT.BORDER);
        operationNameTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        checkReturnBt = new Button(composite, SWT.CHECK);
        checkReturnBt.setText("Return Type : ");

        ComposedAdapterFactory factory = new ComposedAdapterFactory(new AdapterFactory[] {
                new UMLItemProviderAdapterFactory(), new SysMLItemProviderAdapterFactory()});
        ILabelProvider labelProvider = new AdapterFactoryLabelProvider(factory);

        returnTypeChooser = new SingleObjectChooser(composite, SWT.NONE);
        returnTypeChooser.setLabelProvider(labelProvider);
        returnTypeChooser.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label visibilityLbl = new Label(composite, SWT.NONE);
        visibilityLbl.setText("Visibility : ");

        visibilityCb = new ComboViewer(composite, SWT.READ_ONLY);
        visibilityCb.setContentProvider(new ArrayContentProvider());
        visibilityCb.setLabelProvider(labelProvider);
        visibilityCb.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Create the secong tab
     * 
     * @param parent the owning tab folder
     */
    private void createParametersTabItem(TabFolder parent)
    {
        // Create tab item and add it composite that fills it
        TabItem secondItem = new TabItem(parent, SWT.NONE);
        secondItem.setText("Parameters");
        Composite composite = new Composite(parent, SWT.NONE);
        secondItem.setControl(composite);

        // Add layout on composite
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        new Label(composite, SWT.NONE).setText("Choose parameters :");

        inputParameters = new OperationParameterViewer(composite, new OperationDataStructure(operation), types,
                typeNames);

        Button add = new Button(composite, SWT.PUSH | SWT.CENTER);
        add.setText("Add");

        GridData gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END);
        gd.widthHint = 80;
        add.setLayoutData(gd);
        add.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                inputParameters.addParameter();
            }
        });

        Button delete = new Button(composite, SWT.PUSH | SWT.CENTER);
        delete.setText("Delete");
        gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_BEGINNING);
        gd.widthHint = 80;
        delete.setLayoutData(gd);

        delete.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                inputParameters.removeParameter();
            }
        });
    }

    /**
     * Add listeners on the SWT widgets
     */
    protected void hookListeners()
    {
        checkReturnBt.addSelectionListener(new SelectionAdapter()
        {
            /**
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e)
            {
                // Enable the type selection
                setReturnTypeChooserEnabled(checkReturnBt.getSelection());
            }
        });
    }

    /**
     * Fill fields with current operation values
     */
    private void loadData()
    {
        // Name
        operationNameTxt.setText(operation.getName());

        // Types
        checkReturnBt.setSelection(operation.getType() != null);
        setReturnTypeChooserEnabled(operation.getType() != null);

        Collection< ? > reachableTypes = ItemPropertyDescriptor.getReachableObjectsOfType(operation,
                UMLPackage.eINSTANCE.getType());
        returnTypeChooser.setChoices(reachableTypes.toArray());
        if (operation.getType() != null)
        {
            returnTypeChooser.setSelection(operation.getType());
        }

        // Visibility
        visibilityCb.setInput(VisibilityKind.VALUES);
        visibilityCb.setSelection(new StructuredSelection(operation.getVisibility()));
    }

    /**
     * Save the datas before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        data = new HashMap();
        data.put(OPERATION_NAME, operationNameTxt.getText());
        Object selectedVisibility = ((IStructuredSelection) visibilityCb.getSelection()).getFirstElement();
        data.put(OPERATION_VISIBILITY, selectedVisibility);
        if (checkReturnBt.getSelection())
        {
            data.put(OPERATION_RETURN_TYPE, returnTypeChooser.getSelection());
        }
        data.put(OPERATION_INPUTS, inputParameters.getData());
        super.okPressed();
    }

    /**
     * Enables or disable the return type chooser
     */
    private void setReturnTypeChooserEnabled(boolean enabled)
    {
        for (int i = 0; i < returnTypeChooser.getChildren().length; i++)
            returnTypeChooser.getChildren()[i].setEnabled(enabled);
    }

    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        return data;
    }

}
