/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.dialogs;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.VisibilityKind;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.facilities.widgets.SingleObjectChooser;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.sysml.provider.SysMLItemProviderAdapterFactory;

/**
 * Updating property parameters
 */
public class PropertyEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** The default height */
    public static final int DEFAULT_DIALOG_HEIGHT = 100;

    /** The minimum height */
    public static final int MIN_DIALOG_HEIGHT = 180;

    /** The ID of the property name */
    public static final String PROPERTY_NAME = "property name";

    /** The ID of the property type */
    public static final String PROPERTY_TYPE = "property type";

    /** The ID of the property visibility */
    public static final String PROPERTY_VISIBILITY = "property visibility";

    /** Current edited property */
    private Property property;

    // SWT Objects
    private Text propertyNameTxt;

    private SingleObjectChooser typeChooser;

    private ComboViewer visibilityCb;

    private Map data;

    /**
     * Create the dialog for a given property
     * 
     * @param prop the property to edit
     * @param parentShell the parent shell
     */
    public PropertyEditDialog(Property prop, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        property = prop;
    }

    /**
     * Set the title of the new shell
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Property " + property.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        Composite dialogComposite = (Composite) super.createDialogArea(parent);
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayoutData(dialogLayoutData);

        createPropertyGroup(dialogComposite);

        loadData();

        return dialogComposite;
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    private void createPropertyGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        Label propertyNameLbl = new Label(composite, SWT.NONE);
        propertyNameLbl.setText("Name : ");
        propertyNameTxt = new Text(composite, SWT.BORDER);
        propertyNameTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label typeLbl = new Label(composite, SWT.NONE);
        typeLbl.setText("Type : ");

        ComposedAdapterFactory factory = new ComposedAdapterFactory(new AdapterFactory[] {
                new UMLItemProviderAdapterFactory(), new SysMLItemProviderAdapterFactory()});
        ILabelProvider labelProvider = new AdapterFactoryLabelProvider(factory);

        typeChooser = new SingleObjectChooser(composite, SWT.NONE);
        typeChooser.setLabelProvider(labelProvider);
        typeChooser.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label visibilityLbl = new Label(composite, SWT.NONE);
        visibilityLbl.setText("Visibility : ");

        visibilityCb = new ComboViewer(composite, SWT.READ_ONLY);
        visibilityCb.setContentProvider(new ArrayContentProvider());
        visibilityCb.setLabelProvider(labelProvider);
        visibilityCb.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Initialize the content of the widgets
     */
    private void loadData()
    {
        // Name
        propertyNameTxt.setText(property.getName());

        // Types
        Collection reachableTypes = ItemPropertyDescriptor.getReachableObjectsOfType(property,
                UMLPackage.eINSTANCE.getType());
        typeChooser.setChoices(reachableTypes.toArray());
        typeChooser.setSelection(property.getType());

        // Visibility
        visibilityCb.setInput(VisibilityKind.VALUES);
        visibilityCb.setSelection(new StructuredSelection(property.getVisibility()));
    }

    /**
     * Save the values before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        data = new HashMap();
        try
        {
            data.put(PROPERTY_NAME, propertyNameTxt.getText());
            data.put(PROPERTY_TYPE, typeChooser.getSelection());
            Object selectedVisibility = ((IStructuredSelection) visibilityCb.getSelection()).getFirstElement();
            data.put(PROPERTY_VISIBILITY, selectedVisibility);
            super.okPressed();
        }
        catch (Exception e)
        {
            // TODO change this with a validation listener that disable the ok
            // button until the widgets are valid
            SysMLPlugin.log("Required fields", IStatus.WARNING);
            MessageDialog.openWarning(getShell(), "Required parameters",
                    "Some parameters are not set.\nPlease, fill those fields before validating.");
        }
    }

    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        return data;
    }

}
