/*******************************************************************************
 * 
 ******************************************************************************/
package org.topcased.modeler.sysml.blockdefinitiondiagram.policies;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Interface;
import org.eclipse.uml2.uml.Port;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.ModelerRequestConstants;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.EListEditPart;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.requests.AutoLayoutRequest;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Block;

/**
 * Layout Edit Policy for a block It manages creation of elements in the block
 * and creation of elements inside block childs which have a EListEditPart edit
 * part. Block childs which have a EListEditPart edit part don't have their own
 * layout edit policy in order to enable to move port in the block
 */
public class BlockLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor.
     * 
     * @generated
     */
    public BlockLayoutEditPolicy()
    {
        super();
    }

    /**
     * @see org.topcased.modeleler.edit.policies.ModelerLayoutEditPolicy#isAttachedToBorder(org.topcased.modeler.di.model.GraphNode)
     */
    protected boolean isAttachedToBorder(GraphNode node)
    {
        return Utils.getElement(node) instanceof Port;
    }

    /**
     * @see org.topcased.modeleler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        if (child instanceof Interface || child instanceof Block || child instanceof DataType || child instanceof Actor)
            return false;
        return super.isValid(child, parent);
    }

    /**
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
     */
    protected Command getCreateCommand(CreateRequest request)
    {
        Point where = getAbsoluteMouseLocation(request.getLocation() == null ? null : request.getLocation().getCopy());
        Dimension dim = request.getSize();

        Command result = null;
        Object newObject = request.getNewObject();

        if (newObject instanceof EObject)
        {
            result = getCreateSingleObjectCommand(newObject, where, dim, true);
        }
        else
        {
            result = new CompoundCommand("Drag from Outline");

            List< ? > objects = (List< ? >) newObject;
            for (int i = 0; i < objects.size(); i++)
            {
                Command cmd = getCreateSingleObjectCommand(objects.get(i), where, dim, false);
                if (cmd != null)
                    ((CompoundCommand) result).add(cmd);
            }
        }
        return result;
    }

    /**
     * Get the absolute coordinates from relative coordinates given by the
     * pointer Location
     * 
     * @param relativeLocation
     * @return Point with absolute coordinates
     */
    protected Point getAbsoluteMouseLocation(Point relativeLocation)
    {
        if (relativeLocation != null)
        {
            Point location = relativeLocation.getCopy();
            if (location != null)
            {
                IFigure figure = getLayoutContainer();

                figure.translateToRelative(location);
                figure.translateFromParent(location);
                location.translate(getLayoutOrigin().getNegated());

                return location;
            }
        }
        return new Point(0, 0);
    }

    /**
     * Get the command to create a single object
     * 
     * @param objectToCreate object to create
     * @param requestLocation location of the creation request
     * @param requestSize size of the creation request
     * @param needModelUpdate <code>true</code> if the model must be updated
     *            with this children, <code>false</code> if we only went to
     *            add its graphical representation.
     * @return the command
     */
    private Command getCreateSingleObjectCommand(Object objectToCreate, Point requestLocation, Dimension requestSize,
            boolean needModelUpdate)
    {
        Rectangle container = getLayoutContainer().getClientArea();
        Rectangle constraint = new Rectangle(requestLocation, requestSize != null ? requestSize : new Dimension(-1, -1));

        Command result = null;
        // We iterate on all the children until we found a valid Command.
        Iterator< ? > it = getHost().getChildren().iterator();
        while (it.hasNext())
        {
            EditPart ep = (EditPart) it.next();
            if (ep instanceof EListEditPart)
            {
                Command childCmd = getEListEditPartCreateCommand((EListEditPart) ep, objectToCreate, needModelUpdate);
                if (childCmd != null && childCmd.canExecute())
                {
                    result = childCmd;
                }
            }
        }
        // Case where no child can process the request
        if (result == null)
        {
            // get the GraphNodes associated in the diagram model
            GraphNode parentGraphNode = (GraphNode) getHost().getModel();
            GraphNode childGraphNode = (GraphNode) objectToCreate;

            Point loc = calculateChildPosition(constraint, (GraphicalEditPart) getHost());
            int attachment = calculateChildAttachment(childGraphNode, loc, container);
            result = getCreateCommand(parentGraphNode, childGraphNode, loc, requestSize, attachment, needModelUpdate);
        }
        return result;
    }

    /**
     * Get the command to create an object inside a block child which have a
     * EListEditPart edit part
     * 
     * @param editPart the EListEditPart
     * @param objectToCreate the object to create
     * @return the command
     */
    private Command getEListEditPartCreateCommand(EListEditPart editPart, Object objectToCreate, boolean needModelUpdate)
    {
        List<EStructuralFeature> referencesList = new ArrayList<EStructuralFeature>();
        referencesList.add(editPart.getEStructuralFeature());

        if (objectToCreate instanceof GraphNode)
        {
            EditDomain domain = editPart.getViewer().getEditDomain();

            // get the GraphNodes associated in the diagram model
            GraphNode parentGraphNode = (GraphNode) editPart.getModel();
            GraphNode childGraphNode = (GraphNode) objectToCreate;

            // get the EObject of the model
            EObject childEObject = Utils.getElement(childGraphNode);

            if (!isAttachedToBorder(childGraphNode)
                    && editPart.getEStructuralFeature().getEType().isInstance(childEObject))
            {
                // create the command associated with these parameters
                return new CreateGraphNodeCommand(domain, childGraphNode, parentGraphNode, new Point(),
                        new Dimension(), referencesList, needModelUpdate);
            }
        }

        return null;
    }

    /**
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest,
     *      org.eclipse.gef.EditPart, java.lang.Object)
     */
    protected Command createChangeConstraintCommand(ChangeBoundsRequest request, EditPart child, Object constraint)
    {
        IFigure childFigure = ((GraphicalEditPart) child).getFigure();
        if (childFigure.getParent().getLayoutManager().getConstraint(childFigure) instanceof PortConstraint)
        {
            Rectangle container = getLayoutContainer().getClientArea();

            int attach = calculateChildAttachment((GraphNode) child.getModel(),
                    getAbsoluteMouseLocation(request.getLocation()), container);
            ((PortConstraint) constraint).setAttachment(attach);
        }

        return createChangeConstraintCommand(child, constraint);
    }

    /**
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getLayoutContainer()
     */
    protected IFigure getLayoutContainer()
    {
        return getHostFigure();
    }
    
    public Command getCommand(Request request)
    {
        if (ModelerRequestConstants.REQ_AUTOLAYOUT.equals(request.getType()))
        {
            return null;
        }

        return super.getCommand(request);
    }

}