/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.util;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.VisibilityKind;

/**
 * An helper class to handle Association parameters more easily
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class AssociationHelper
{
    private Association association;

    public static final String ASSOCIATION_NAME = "association name";

    public static final String FIRST_END_CLASS = "first end class";

    public static final String FIRST_END_ROLE = "first end role";

    public static final String FIRST_END_IS_NAVIGABLE = "first end is navigable";

    public static final String FIRST_END_IS_ORDERED = "first end is ordered";

    public static final String FIRST_END_VISIBILITY_KIND = "first end visibility kind";

    public static final String FIRST_END_AGGREGATION_KIND = "first end aggregation kind";

    public static final String FIRST_END_LOWER_BOUND = "first end lower bound";

    public static final String FIRST_END_UPPER_BOUND = "first end upper bound";

    public static final String SECOND_END_CLASS = "second end class";

    public static final String SECOND_END_ROLE = "second end role";

    public static final String SECOND_END_IS_NAVIGABLE = "second end is navigable";

    public static final String SECOND_END_IS_ORDERED = "second end is ordered";

    public static final String SECOND_END_VISIBILITY_KIND = "second end visibility kind";

    public static final String SECOND_END_AGGREGATION_KIND = "second end aggregation kind";

    public static final String SECOND_END_LOWER_BOUND = "second end lower bound";

    public static final String SECOND_END_UPPER_BOUND = "second end upper bound";

    /**
     * Constructor
     * 
     * @param association the Association
     */
    public AssociationHelper(Association association)
    {
        this.association = association;
    }

    /**
     * Check if the Association is completely created
     * 
     * @return true if the Association has its two MemberEnds
     */
    public boolean isAssociationReady()
    {
        return association.isBinary();
    }

    // --------------------------------------
    // The helper methods of the Association

    public String getAssociationName()
    {
        return association.getName();
    }

    // --------------------------------------

    // --------------------------------------------
    // The helper methods of the firstEnd Property

    public Type getAssociationFirstEndClass()
    {
        return ((Property) association.getMemberEnds().get(0)).getType();
    }

    public String getAssociationFirstEndRole()
    {
        return ((Property) association.getMemberEnds().get(0)).getName();
    }

    public boolean getAssociationFirstEndIsNavigable()
    {
        return ((Property) association.getMemberEnds().get(0)).isNavigable();
    }

    public boolean getAssociationFirstEndIsOrdered()
    {
        return ((Property) association.getMemberEnds().get(0)).isOrdered();
    }

    public VisibilityKind getAssociationFirstEndVisibilityKind()
    {
        return ((Property) association.getMemberEnds().get(0)).getVisibility();
    }

    public AggregationKind getAssociationFirstEndAggregationKind()
    {
        return ((Property) association.getMemberEnds().get(0)).getAggregation();
    }

    public String getAssociationFirstEndLowerBound()
    {
        return Integer.toString(((Property) association.getMemberEnds().get(0)).getLower());
    }

    public String getAssociationFirstEndUpperBound()
    {
        int upperBound = ((Property) association.getMemberEnds().get(0)).getUpper();

        return LiteralUnlimitedNatural.UNLIMITED == upperBound ? "*" : Integer.toString(upperBound);
    }

    // --------------------------------------------

    // ---------------------------------------------
    // The helper methods of the secondEnd Property

    public Type getAssociationSecondEndClass()
    {
        return ((Property) association.getMemberEnds().get(1)).getType();
    }

    public String getAssociationSecondEndRole()
    {
        return ((Property) association.getMemberEnds().get(1)).getName();
    }

    public boolean getAssociationSecondEndIsNavigable()
    {
        return ((Property) association.getMemberEnds().get(1)).isNavigable();
    }

    public boolean getAssociationSecondEndIsOrdered()
    {
        return ((Property) association.getMemberEnds().get(1)).isOrdered();
    }

    public VisibilityKind getAssociationSecondEndVisibilityKind()
    {
        return ((Property) association.getMemberEnds().get(1)).getVisibility();
    }

    public AggregationKind getAssociationSecondEndAggregationKind()
    {
        return ((Property) association.getMemberEnds().get(1)).getAggregation();
    }

    public String getAssociationSecondEndLowerBound()
    {
        return Integer.toString(((Property) association.getMemberEnds().get(1)).getLower());
    }

    public String getAssociationSecondEndUpperBound()
    {
        int upperBound = ((Property) association.getMemberEnds().get(1)).getUpper();

        return LiteralUnlimitedNatural.UNLIMITED == upperBound ? "*" : Integer.toString(upperBound);
    }

    // ---------------------------------------------

    /**
     * Return all the informations concerning the Association
     * 
     * @return Map
     */
    public Map extractAssociationData()
    {
        Map resultMap = new HashMap();

        resultMap.put(AssociationHelper.ASSOCIATION_NAME, getAssociationName());

        resultMap.put(AssociationHelper.FIRST_END_ROLE, getAssociationFirstEndRole());
        resultMap.put(AssociationHelper.FIRST_END_IS_NAVIGABLE, new Boolean(getAssociationFirstEndIsNavigable()));
        resultMap.put(AssociationHelper.FIRST_END_IS_ORDERED, new Boolean(getAssociationFirstEndIsOrdered()));
        resultMap.put(AssociationHelper.FIRST_END_LOWER_BOUND, getAssociationFirstEndLowerBound());
        resultMap.put(AssociationHelper.FIRST_END_UPPER_BOUND,
                Integer.toString(((Property) association.getMemberEnds().get(0)).getUpper()));
        resultMap.put(AssociationHelper.FIRST_END_VISIBILITY_KIND, getAssociationFirstEndVisibilityKind());
        resultMap.put(AssociationHelper.FIRST_END_AGGREGATION_KIND, getAssociationFirstEndAggregationKind());

        resultMap.put(AssociationHelper.SECOND_END_ROLE, getAssociationSecondEndRole());
        resultMap.put(AssociationHelper.SECOND_END_IS_NAVIGABLE, new Boolean(getAssociationSecondEndIsNavigable()));
        resultMap.put(AssociationHelper.SECOND_END_IS_ORDERED, new Boolean(getAssociationSecondEndIsOrdered()));
        resultMap.put(AssociationHelper.SECOND_END_LOWER_BOUND, getAssociationSecondEndLowerBound());
        resultMap.put(AssociationHelper.SECOND_END_UPPER_BOUND,
                Integer.toString(((Property) association.getMemberEnds().get(1)).getUpper()));
        resultMap.put(AssociationHelper.SECOND_END_VISIBILITY_KIND, getAssociationSecondEndVisibilityKind());
        resultMap.put(AssociationHelper.SECOND_END_AGGREGATION_KIND, getAssociationSecondEndAggregationKind());

        return resultMap;
    }
}
