/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.uml2.uml.DataType;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Relationship;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.sysml.util.SysMLTools;
import org.topcased.sysml.Block;
import org.topcased.sysml.PackageSYSML;
import org.topcased.sysml.Dimension;
import org.topcased.sysml.FlowSpecification;
import org.topcased.sysml.ModelSYSML;
import org.topcased.sysml.Unit;
import org.topcased.sysml.ValueType;

public class BlockTools
{
    /**
     * Return a collection of all types contained in the model associated to a
     * given element
     * 
     * @param element
     * @return types (data type, class)
     */
    public static Collection getAllAvailableTypes(Element element)
    {

        return ItemPropertyDescriptor.getReachableObjectsOfType(element, UMLPackage.eINSTANCE.getType());
    }

    /**
     * Gets all the 'Type' objects of the model
     * 
     * @param container The start element to search in
     * @return The list containing all the 'Type' objects referenced in the current model.
     */
    public static List<?> getAvailablePortType(ModelSYSML model)
    {
        List<?> listAllAvailableObjects = SysMLTools.getAllObjects(model, Type.class);
        ArrayList<Object> listObjectsFiltered = new ArrayList<Object>();
        
        Iterator<?> iterator = listAllAvailableObjects.iterator();
        while(iterator.hasNext())
        {
            Object element = iterator.next();
            if(element instanceof Dimension || element instanceof Unit || element instanceof Enumeration)
            {
                
            }
            else if(element instanceof Block || element instanceof ValueType || element instanceof DataType || element instanceof FlowSpecification)
            {
                listObjectsFiltered.add(element);
            }           
        }
        return listObjectsFiltered;
    }
    /**
     * Remove relationships from a collection
     * 
     * @param c the collection
     * @return the collection without relationships
     */
    public static Collection removeRelationship(Collection c)
    {
        Collection result = new ArrayList();
        for (Iterator iterator = c.iterator(); iterator.hasNext();)
        {
            Object current = iterator.next();
            if (!(current instanceof Relationship))
            {
                result.add(current);
            }
        }
        return result;
    }

    /**
     * Return the block definition in which an element is contained
     * 
     * @param element
     * @return the block definition
     */
    public static PackageSYSML getPackageSYSML(Element element)
    {
        if (element == null || element instanceof PackageSYSML)
            return (PackageSYSML) element;
        return getPackageSYSML(element.getOwner());
    }

    /**
     * Return an array of type names.
     * 
     * @param types a Collection of Type
     * @return array of type names
     */
    public static String[] getAllAvailableTypeNames(Collection types)
    {
        String[] result = new String[types.size()];
        int index = 0;
        for (Iterator iterator = types.iterator(); iterator.hasNext();)
        {
            Type current = (Type) iterator.next();
            result[index] = getTypeFullName(current);
            index++;
        }
        return result;
    }

    /**
     * Return the customized type name
     * 
     * @param type type to customize name
     * @return customized name
     */
    public static String getTypeFullName(Type elt)
    {
        return elt.eClass().getName() + " " + elt.getName();
    }

    /**
     * Get a string representation of the origin of a SysML element
     * 
     * @param element an SysML element
     * @param parent the element which graphically contains the SysML Type
     *            element
     * @return a fully qualified string representation or empty string if
     *         element isn't contained in an external block definition
     */
    public static String getFromNotation(Element element, Element parent)
    {
        String name = "";

        PackageSYSML bdElement = getPackageSYSML(element);
        PackageSYSML bdParent = getPackageSYSML(parent);
        if (bdElement != bdParent && element.getOwner() instanceof NamedElement)
        {
            name = "(from " + ((NamedElement) element.getOwner()).getName() + ")";
        }
        return name;
    }
}
