/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.blockdefinitiondiagram.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Type;

/**
 * Provide a structure for handling Operation data <br>
 * creation : 27 may 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class OperationDataStructure
{

    /**
     * Internal class to handle a parameter
     */
    public class ParameterObject
    {

        /** Parameter name */
        private String name;

        /** Parameter type */
        private Type type;

        /** Parameter direction */
        private ParameterDirectionKind direction;

        /**
         * The constructor
         * 
         * @param n parameter name
         * @param t parameter type
         */
        public ParameterObject(String n, Type t, ParameterDirectionKind d)
        {
            name = n;
            type = t;
            direction = d;
        }

        /**
         * Get parameter name
         * 
         * @return name of the parameter
         */
        public String getName()
        {
            return name;
        }

        /**
         * Get parameter type
         * 
         * @return type of the parameter
         */
        public Type getType()
        {
            return type;
        }

        /**
         * Get parameter type
         * 
         * @return direction of the parameter
         */
        public ParameterDirectionKind getDirection()
        {
            return direction;
        }

        /**
         * Set new name for the parameter
         * 
         * @param newName the new name
         */
        public void setName(String newName)
        {
            name = newName;
        }

        /**
         * Set new type for the parameter
         * 
         * @param newType the new type
         */
        public void setType(Type newType)
        {
            type = newType;
        }

        /**
         * Set new direction for the parameter
         * 
         * @param direction the new direction
         */
        public void setDirection(ParameterDirectionKind direction)
        {
            this.direction = direction;
        }

    } // End internal class

    /** A collection for ParameterObject objects */
    private ArrayList data;

    /**
     * The constructor
     * 
     * @param operation the Operation
     */
    public OperationDataStructure(Operation operation)
    {
        data = new ArrayList();
        if (operation != null)
        {
            addAll(getParameters(operation));
        }
    }

    /**
     * Add a parameter to the structure
     * 
     * @param parameter the parameter to add
     */
    public void add(Parameter parameter)
    {
        data.add(new ParameterObject(parameter.getName(), parameter.getType(), parameter.getDirection()));
    }

    /**
     * Add a parameter to the structure
     * 
     * @param name the parameter name
     * @param type the parameter type
     * @param direction the parameter direction
     */
    public void add(String name, Type type, ParameterDirectionKind direction)
    {
        data.add(new ParameterObject(name, type, direction));
    }

    /**
     * Remove a parameter or the name or etc..; from the structure
     * 
     * @param object the object to remove
     */
    public void remove(Object object)
    {
        data.remove(object);
    }

    /**
     * Add a collection of parameters to the operation
     * 
     * @param parameters the collection of parameters to add
     */
    public void addAll(Collection parameters)
    {
        Iterator itParameters = parameters.iterator();
        while (itParameters.hasNext())
        {
            Parameter parameter = (Parameter) itParameters.next();
            add(parameter);
        }
    }

    /**
     * Get the datas
     * 
     * @return a Collection of
     */
    public Collection getData()
    {
        return data;
    }

    /**
     * Get the name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name
     */
    public String getDisplayName(Object object)
    {
        String result = ((ParameterObject) object).getName();
        if (result == null || result.length() == 0)
        {
            result = "Property name no set";
        }
        return result;
    }

    /**
     * Get the type name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name of the type
     */
    public String getDisplayType(Object object)
    {
        Type type = getType(object);
        if (type == null)
        {
            return null;
        }
        return type.getName();
    }

    /**
     * Get the direction name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name of the direction
     */
    public String getDisplayDirection(Object object)
    {
        ParameterDirectionKind direction = ((ParameterObject) object).getDirection();
        return direction.getName();
    }

    /**
     * Get the type of a given object
     * 
     * @param object OperationDataObject object
     * @return the type
     */
    public Type getType(Object object)
    {
        return ((ParameterObject) object).getType();
    }

    /**
     * Get the direction of a given object
     * 
     * @param object OperationDataObject object
     * @return the direction
     */
    public ParameterDirectionKind getDirection(Object object)
    {
        return ((ParameterObject) object).getDirection();
    }

    /**
     * Set the name of the parameter object
     * 
     * @param object a ParameterObject
     * @param name the new name
     */
    public void setName(Object object, String name)
    {
        ((ParameterObject) object).setName(name);
    }

    /**
     * Set the type of the parameter object
     * 
     * @param object a ParameterObject
     * @param type the new type
     */
    public void setType(Object object, Type type)
    {
        ((ParameterObject) object).setType(type);
    }

    /**
     * Set the direction of the parameter object
     * 
     * @param object a ParameterObject
     * @param direction the new direction
     */
    public void setDirection(Object object, ParameterDirectionKind direction)
    {
        ((ParameterObject) object).setDirection(direction);
    }

    private Collection getParameters(Operation operation)
    {
        ArrayList parameters = new ArrayList();
        for (Iterator ownedParameters = operation.getOwnedParameters().iterator(); ownedParameters.hasNext();)
        {

            Parameter ownedParameter = (Parameter) ownedParameters.next();

            if (ownedParameter.getDirection() != ParameterDirectionKind.RETURN_LITERAL)
            {
                parameters.add(ownedParameter);
            }
        }
        return parameters;
    }
}
