/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.sysml.internalblockdiagram.commands;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.uml2.uml.Port;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.DeleteGraphElementCommand;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.utils.Utils;
import org.topcased.sysml.Block;

/**
 * This command is used to refresh ports of a block
 */
public class RefreshPortsCommand extends CompoundCommand
{

    /** The modeler */
    private Modeler modeler;

    /** The CreationUtils factory to use for creating GraphElements */
    private ICreationUtils creationUtils;

    /**
     * The Constructor
     * 
     * @param modeler the modeler
     * @param blockNode the block node
     */
    public RefreshPortsCommand(Modeler modeler, GraphNode blockNode)
    {
        super("Refresh Ports Command");

        this.modeler = modeler;
        this.creationUtils = modeler.getActiveConfiguration().getCreationUtils();

        refreshPorts(blockNode);
    }

    /**
     * Refresh the Ports of a given block GraphNode. Refresh all the GraphNodes
     * linked with the model object associated with the GraphNode.
     * 
     * @param blockNode
     */
    private void refreshPorts(GraphNode blockNode)
    {
        EObject rootEObject = Utils.getElement(blockNode);
        List< ? > ports = retrievePorts(rootEObject);

        if (ports != null)
        {
            createCreateGraphNodeCommand(ports, blockNode);
            createDeleteGraphNodeCommand(ports, blockNode);
        }
    }

    /**
     * This method iterates on the ports list and returns a List of GraphNodes
     * (the ports) that are not present in the container GraphNode (the block)
     * 
     * @param ports the list of ports to synchronize
     * @param blockNode the block GraphNode
     */
    private void createCreateGraphNodeCommand(List< ? > ports, GraphNode blockNode)
    {
        // a counter that is used to place the ports that are not already
        // present
        int yPos = 30;

        for (Iterator< ? > iter = ports.iterator(); iter.hasNext();)
        {
            // retrieve the ports to display
            Port port = (Port) iter.next();

            // check if the element is already present
            if (Utils.getGraphElement(blockNode, port, true) == null)
            {
                // create the new graphNode and add it to the parentNode
                GraphNode node = (GraphNode) creationUtils.createGraphElement(port);

                add(new CreateGraphNodeCommand((DefaultEditDomain) modeler.getAdapter(DefaultEditDomain.class), node,
                        blockNode, new Point(1, yPos), node.getSize(), PositionConstants.LEFT));

                yPos = yPos + 45;
            }
        }
    }

    /**
     * This method iterates on the graphNodes contained by the block and returns
     * a List of graphNode that are not associated with a port any more.
     * 
     * @param ports the list of Ports to synchronize
     * @param blockNode the block GraphNode
     */
    private void createDeleteGraphNodeCommand(List< ? > ports, GraphNode blockNode)
    {
        // Delete feature nodes that are not associated with a feature any more
        for (Iterator< ? > iter = blockNode.getContained().iterator(); iter.hasNext();)
        {
            DiagramElement elt = (DiagramElement) iter.next();
            if (elt instanceof GraphNode)
            {
                GraphNode node = (GraphNode) elt;
                EObject obj = Utils.getElement(node);
                if (obj instanceof Port && !ports.contains(obj))
                {
                    add(new DeleteGraphElementCommand(node, true));
                }
            }
        }
    }

    /**
     * This method returns the list of ports associated with a block
     * 
     * @param block the block model object
     * @return EList the ports list
     */
    private EList< ? > retrievePorts(EObject block)
    {
        if (block instanceof Block)
        {
            return ((Block) block).getOwnedPorts();
        }
        return null;
    }
}
