/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.sysml.internalblockdiagram.dialogs;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.sysml.internalblockdiagram.util.TypeHelper;
import org.topcased.sysml.ModelSYSML;
import org.topcased.sysml.provider.SysMLItemProviderAdapterFactory;

/**
 * A Dialog used to create a new 'Type' or reference an existing 'Type' during the creation of a Port.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class ReferencePortDialog extends Dialog implements ITopcasedDialogConstants
{
        // The StructuredClassifier that will contain the Property
        private StructuredClassifier structuredClassifier;

        // Type of the new Property
        private Type refType;

        // Name of the new Type to create
        private String typeName;

        // Indicates if a new Type must be created or if an existing Type must be used.
        private boolean createType;

        // The type of the dialog  (Property or Port)
        protected String type;
        
        // SWT Objects
        private Button existingTypeBt;

        private Button newTypeBt;

        private Combo typeCb;

        private ComboViewer typeCbViewer;

        private Text typeFd;

        /**
         * Creates the Dialog window to select a Classifier to associate with the current Property or Port.
         * 
         * @param class the class that contains the State
         * @param parentShell the parent Shell
         */
        public ReferencePortDialog(StructuredClassifier structuredClass_, Shell parentShell_)
        {
            super(parentShell_);
            setShellStyle(getShellStyle() | SWT.RESIZE);
            setBlockOnOpen(true);
            structuredClassifier = structuredClass_;
            type = "Port";
        }

        /**
         * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
         */
        protected void configureShell(Shell newShell)
        {
            newShell.setText(type.concat(" Reference"));
            newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

            super.configureShell(newShell);
        }

        /**
         * Creates the dialog area
         * 
         * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
         */
        protected Control createDialogArea(Composite parent)
        {
            Composite dialogComposite = (Composite) super.createDialogArea(parent);
            createDialogComposite(dialogComposite);
            createMainGroup(dialogComposite);
            hookListeners();
            loadData();
            return dialogComposite;
        }

        /**
         * Creates the dialog Contents
         * 
         * @param composite
         */
        private void createDialogComposite(Composite composite)
        {
            GridLayout dialogLayout = new GridLayout();
            GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
            composite.setLayout(dialogLayout);
            composite.setLayoutData(dialogLayoutData);
        }

        /**
         * Creates the SWT Objects of the Dialog
         * 
         * @param parent the parent Composite
         */
        protected void createMainGroup(Composite parent)
        {
            Composite composite = new Composite(parent, SWT.NONE);

            // Add layout on composite
            composite.setLayout(new GridLayout());
            composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

            existingTypeBt = new Button(composite, SWT.RADIO);
            existingTypeBt.setText("Choose an existing Type to reference from the ".concat(type).concat(" :"));

            typeCb = new Combo(composite, SWT.NONE);
            typeCb.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
            typeCbViewer = new ComboViewer(typeCb);

            Label separationLbl = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
            separationLbl.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

            newTypeBt = new Button(composite, SWT.RADIO);
            newTypeBt.setText("Create a new Block to reference from the ".concat(type).concat(" :"));

            typeFd = new Text(composite, SWT.BORDER);
            typeFd.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        }

        /**
         * Initializes widgets value
         */
        private void loadData()
        {
            typeCbViewer.setContentProvider(new TypeContentPortProvider());
            
            ComposedAdapterFactory factory = new ComposedAdapterFactory(new AdapterFactory[] {
                    new UMLItemProviderAdapterFactory(), new SysMLItemProviderAdapterFactory()});
            ILabelProvider labelProvider = new AdapterFactoryLabelProvider(factory);       
            typeCbViewer.setLabelProvider(labelProvider);
          
            // Retrieve all the available 'Type' in the current structured classifier.
            typeCbViewer.setInput(getModelSysML(structuredClassifier));
           
            existingTypeBt.setSelection(true);
            newTypeBt.setSelection(false);
            typeFd.setEnabled(false);
        }
        
        
        
        private ModelSYSML getModelSysML(Element element)
        {
        	while(!(element.getOwner() instanceof ModelSYSML))
        	{
        		element = element.getOwner();
        	}
        	return (ModelSYSML)element.getOwner();
        }
        

        /**
         * Hooks the listeners
         */
        private void hookListeners()
        {
            existingTypeBt.addSelectionListener(new SelectionAdapter()
            {
                public void widgetSelected(SelectionEvent e)
                {
                	handleOKButton();
                }
            });

            newTypeBt.addSelectionListener(new SelectionAdapter()
            {
                public void widgetSelected(SelectionEvent e)
                {
                	handleOKButton();
                }
            });

            typeCbViewer.addSelectionChangedListener(new ISelectionChangedListener()
            {
                // This ensures that we handle selections correctly.
                public void selectionChanged(SelectionChangedEvent event)
                {
                    if (event.getSelection() instanceof IStructuredSelection)
                    {
                    	refType = (Type) ((IStructuredSelection) event.getSelection()).getFirstElement();
                    }
                }
            });
            
            typeFd.addModifyListener(new ModifyListener()
            {
            	public void modifyText(ModifyEvent e)
            	{
            		handleOKButton();
            	}
            });
            
            typeCb.addModifyListener(new ModifyListener() 
            {
            	public void modifyText(ModifyEvent e)
            	{
            		handleOKButton();
            	}
            });
        }

    	/**
    	 * Determines whether the OK button should be enable or disable.
    	 */
    	private void handleOKButton()
    	{
    		if (newTypeBt.getSelection())
    		{
    			typeCb.setEnabled(false);
    			typeFd.setEnabled(true);
    			if (!"".equals(typeFd.getText()))
    			{
    				getButton(Dialog.OK).setEnabled(true);
    			}
    			else
    			{
    				getButton(Dialog.OK).setEnabled(false);
    			}
    		}

    		if (existingTypeBt.getSelection())
    		{
    			typeCb.setEnabled(true);
    			typeFd.setEnabled(false);
    			if (!"".equals(typeCb.getText()))
    			{
    				getButton(Dialog.OK).setEnabled(true);
    			}
    			else
    			{
    				getButton(Dialog.OK).setEnabled(false);
    			}
    		}
    	}
    	
        /**
         * Returns the 'Type'
         * 
         * @return a class
         */
        public Type getType()
        {
            return refType;
        }

        /**
         * Returns the name of the new created Type
         * 
         * @return String The class name
         */
        public String getNewTypeName()
        {
            return typeName;
        }

        /**
         * Returns <code>true</code> if a new Type should be created
         * 
         * @return boolean
         */
        public boolean getCreateType()
        {
            return createType;
        }

        /**
         * @see org.eclipse.jface.dialogs.Dialog#okPressed()
         */
        protected void okPressed()
        {
            createType = newTypeBt.getSelection();
            typeName = typeFd.getText();

            super.okPressed();
        }

    	/**
    	 * Overrides the initial method to allow the OK button to be created with a disabled status.
    	 *  
    	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
    	 */
    	protected void createButtonsForButtonBar(Composite parent)
    	{
    		super.createButtonsForButtonBar(parent);
    		getButton(Dialog.OK).setEnabled(false);
    	}
    	
        /**
         * This Content Provider displays contents of collaborations. For the model, this provider delegates contents
         * computing to the model content provider. <br>
         */
        public class TypeContentPortProvider implements IStructuredContentProvider
        {
            /**
             * Constructor
             */
            public TypeContentPortProvider()
            {
                // Do nothing
            }

            /**
             * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
             */
            public Object[] getElements(Object inputElement)
            {
                return TypeHelper.getAvailablePortType((ModelSYSML) inputElement).toArray();
            }

            /**
             * @see org.eclipse.jface.viewers.IContentProvider#dispose()
             */
            public void dispose()
            {
                // Do nothing
            }

            /**
             * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
             *      java.lang.Object, java.lang.Object)
             */
            public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
            {
                // Do nothing
            }
        }
        
}
