/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.requirementdiagram.dialogs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.RequirementKind;
import org.topcased.sysml.RiskKind;
import org.topcased.sysml.VerifyMethodKind;
import org.topcased.sysml.provider.SysMLItemProviderAdapterFactory;

/**
 * Updating requirement parameters
 */
public class RequirementEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** The ID of the requirement name */
    public static final String REQUIREMENT_NAME = "requirement name";

    /** The ID of the requirement id */
    public static final String REQUIREMENT_ID = "requirement id";

    /** The ID of the requirement source */
    public static final String REQUIREMENT_SOURCE = "requirement source";

    /** The ID of the requirement text */
    public static final String REQUIREMENT_TEXT = "requirement text";

    /** The ID of the requirement kind */
    public static final String REQUIREMENT_KIND = "requirement kind";

    /** The ID of the requirement risk */
    public static final String REQUIREMENT_RISK = "requirement risk";

    /** The ID of the requirement verifyMethod */
    public static final String REQUIREMENT_VERIFY = "requirement verify";

    /** Current edited requirement */
    private Requirement _requirement;

    // SWT Objects
    private Text _requirementNameTxt;

    private Text _requirementIdTxt;

    private Text _requirementSourceTxt;

    private Text _requirementTextTxt;

    private ComboViewer _kindCb;

    private ComboViewer _riskCb;

    private ComboViewer _verifyCb;

    private Map _data;

    /**
     * Create the dialog for a given requirement
     * 
     * @param requirement the requirement to edit
     * @param parentShell the parent shell
     */
    public RequirementEditDialog(Requirement requirement, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        _requirement = requirement;
    }

    /**
     * Set the title of the new shell
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Requirement " + _requirement.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        Composite dialogComposite = (Composite) super.createDialogArea(parent);
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayoutData(dialogLayoutData);

        createRequirementGroup(dialogComposite);

        loadData();

        return dialogComposite;
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    private void createRequirementGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        Label requirementNameLbl = new Label(composite, SWT.NONE);
        requirementNameLbl.setText("Name : ");
        _requirementNameTxt = new Text(composite, SWT.BORDER);
        _requirementNameTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label requirementIdLbl = new Label(composite, SWT.NONE);
        requirementIdLbl.setText("Id : ");
        _requirementIdTxt = new Text(composite, SWT.BORDER);
        _requirementIdTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label requirementSourceLbl = new Label(composite, SWT.NONE);
        requirementSourceLbl.setText("Source : ");
        _requirementSourceTxt = new Text(composite, SWT.BORDER);
        _requirementSourceTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label requirementTextLbl = new Label(composite, SWT.NONE);
        requirementTextLbl.setText("Text : ");
        _requirementTextTxt = new Text(composite, SWT.BORDER | SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
        _requirementTextTxt.setLayoutData(new GridData(GridData.FILL_BOTH));

        ComposedAdapterFactory factory = new ComposedAdapterFactory(new AdapterFactory[] {
                new UMLItemProviderAdapterFactory(), new SysMLItemProviderAdapterFactory()});
        ILabelProvider labelProvider = new AdapterFactoryLabelProvider(factory);

        Label requirementKindLbl = new Label(composite, SWT.NONE);
        requirementKindLbl.setText("Kind : ");

        _kindCb = new ComboViewer(composite, SWT.READ_ONLY);
        _kindCb.setContentProvider(new ArrayContentProvider());
        _kindCb.setLabelProvider(labelProvider);
        _kindCb.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label requirementRiskLbl = new Label(composite, SWT.NONE);
        requirementRiskLbl.setText("Risk : ");

        _riskCb = new ComboViewer(composite, SWT.READ_ONLY);
        _riskCb.setContentProvider(new ArrayContentProvider());
        _riskCb.setLabelProvider(labelProvider);
        _riskCb.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label requirementVerifyLbl = new Label(composite, SWT.NONE);
        requirementVerifyLbl.setText("Verify Method : ");

        _verifyCb = new ComboViewer(composite, SWT.READ_ONLY);
        _verifyCb.setContentProvider(new ArrayContentProvider());
        _verifyCb.setLabelProvider(labelProvider);
        _verifyCb.getControl().setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Initialize the content of the widgets
     */
    private void loadData()
    {
        if (_requirement.getName() != null)
            _requirementNameTxt.setText(_requirement.getName());
        if (_requirement.getId() != null)
            _requirementIdTxt.setText(_requirement.getId());
        if (_requirement.getSource() != null)
            _requirementSourceTxt.setText(_requirement.getSource());
        if (_requirement.getText() != null)
            _requirementTextTxt.setText(_requirement.getText());

        _kindCb.setInput(RequirementKind.VALUES);
        _kindCb.setSelection(new StructuredSelection(_requirement.getKind()));

        _riskCb.setInput(RiskKind.VALUES);
        _riskCb.setSelection(new StructuredSelection(_requirement.getRisk()));

        _verifyCb.setInput(VerifyMethodKind.VALUES);
        _verifyCb.setSelection(new StructuredSelection(_requirement.getVerifyMethod()));
    }

    /**
     * Save the values before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        _data = new HashMap();
        try
        {
            _data.put(REQUIREMENT_NAME, _requirementNameTxt.getText());
            _data.put(REQUIREMENT_ID, _requirementIdTxt.getText());
            _data.put(REQUIREMENT_SOURCE, _requirementSourceTxt.getText());
            _data.put(REQUIREMENT_TEXT, _requirementTextTxt.getText());
            Object selectedKind = ((IStructuredSelection) _kindCb.getSelection()).getFirstElement();
            _data.put(REQUIREMENT_KIND, selectedKind);
            Object selectedRisk = ((IStructuredSelection) _riskCb.getSelection()).getFirstElement();
            _data.put(REQUIREMENT_RISK, selectedRisk);
            Object selectedVerifyMethod = ((IStructuredSelection) _verifyCb.getSelection()).getFirstElement();
            _data.put(REQUIREMENT_VERIFY, selectedVerifyMethod);
            super.okPressed();
        }
        catch (Exception e)
        {
            // TODO change this with a validation listener that disable the ok
            // button until the widgets are valid
            SysMLPlugin.log("Required fields", IStatus.WARNING);
            MessageDialog.openWarning(getShell(), "Required parameters",
                    "Some parameters are not set.\nPlease, fill those fields before validating.");
        }
    }

    /**
     * Return a map containing property data that may changed
     * 
     * @return a Map
     */
    public Map getData()
    {
        return _data;
    }
}
