/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation, Thomas Friol (Anyware Technologies)
 ******************************************************************************/

package org.topcased.modeler.sysml.util;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.MultiplicityElement;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.VisibilityKind;

public class SysMLTools
{
    /**
     * Get the list of all the applied stereotypes on the given UML2 element or
     * SysMLElement.<br>
     * 
     * @param element an UML2 element or a SysML Element
     * @return the stereotype list or empty string if no applied stereotypes
     */
    public static String getStereotypesList(Element element)
    {
        StringBuffer label = new StringBuffer();

        Iterator itStereotypes = element.getAppliedStereotypes().iterator();
        while (itStereotypes.hasNext())
        {
            Stereotype stereotype = (Stereotype) itStereotypes.next();
            label.append(stereotype.getName());
            if (itStereotypes.hasNext())
            {
                label.append(", ");
            }
        }
        return label.toString();
    }

    /**
     * Get a string representation of all the applied stereotypes on the given
     * UML2 element or SysMLElement.<br>
     * 
     * @param element an UML2 element or a SysML Element
     * @return a fully qualified string representation or empty string if no
     *         applied stereotypes
     */
    public static String getStereotypeNotation(Element element)
    {
        String stereotype = getStereotypesList(element);
        if (stereotype.length() != 0)
            return "<<" + stereotype + ">>";
        return "";
    }

    /**
     * Return a representation for a given visibility kind
     * 
     * @param visibility the visibily
     * @return a string that represents the visibility kind
     */
    public static String getVisibilityNotation(VisibilityKind visibility)
    {
        if (visibility != null)
        {
            switch (visibility.getValue())
            {
                case VisibilityKind.PUBLIC:
                    // Public
                    return "+";
                case VisibilityKind.PRIVATE:
                    // Private
                    return "-";
                case VisibilityKind.PROTECTED:
                    // Protected
                    return "#";
                case VisibilityKind.PACKAGE:
                    // Package
                    return "~";
                default:
                    return "";

            }
        }
        return "";
    }

    /**
     * Returns the multiplicity label
     * 
     * @param multiplicity the multiplicity element
     * @return the associated text
     */
    public static String getMultiplicityText(MultiplicityElement multiplicity)
    {
        StringBuffer text = new StringBuffer();

        int lower = multiplicity.getLower();
        int upper = multiplicity.getUpper();
        if (lower == upper)
        {
            if (lower != 1)
            {
                text.append("[");
                text.append(lower);
                text.append("]");
            }
        }
        else if (upper == -1)
        {
            if (lower == 0)
            {
                text.append("[*]");
            }
            else
            {
                text.append("[");
                text.append(lower);
                text.append("..*]");
            }
        }
        else
        {

            text.append("[");
            text.append(lower);
            text.append("..");
            text.append(upper);
            text.append("]");
        }

        return text.toString();
    }
    
    /**
     * Search in the given model every occurence of a type
     * 
     * @param element the element to search in
     * @param clazz the wanted type
     * @return an ArrayList containing the wanted model objects
     */
    public static ArrayList<?> getAllObjects(Element element, java.lang.Class<?> clazz)
    {
        ArrayList<Object> result = new ArrayList<Object>();

        if (clazz == null || clazz.isInstance(element))
        {
            result.add(element);
        }

        Iterator<Element> iterator = element.getOwnedElements().iterator();

        while (iterator.hasNext())
        {
            result.addAll(getAllObjects((Element) iterator.next(), clazz));
        }

        return result;

    }
}
