/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.activitydiagram.commands;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.activitydiagram.edit.ActivityNodeEditPart;
import org.topcased.modeler.uml.activitydiagram.edit.ActivityPartitionEditPart;

/**
 * Command to change the bounds of the ActivityPartition and update the list of the contained ActivityNode/ActivityEdge
 * that are covered by this Partition
 * 
 * Creation 15 mars 07
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ActivityPartitionChangeBoundsCommand extends ChangeBoundsCommand
{
    /** The list that will contains the new ActivityNode covered */
    private EList<ActivityNode> newActivityNodeList;

    /** The list that will contains the new ActivityEdge covered */
    private EList<ActivityEdge> newActivityEdgeList;

    /** The list that contains the ActivityNode covered before the ActivityPartition bounds were changed */
    private EList<ActivityNode> oldActivityNodeList;

    /** The list that contains the ActivityEdge covered before the ActivityPartition bounds were changed */
    private EList<ActivityEdge> oldActivityEdgeList;

    /** The ActivityPartition editPart */
    private EditPart childEditPart;

    /** The Diagram editPart */
    private EditPart parentEditPart;

    /** The new constraint of the ActivityPartition */
    private Rectangle constraint;

    /**
     * Constructor
     * 
     * @param node the node to resize
     * @param constraint the new rectangle that contain the figure
     * @param childEditPart the EditPart associated with the ActivityPartition
     * @param parentEditPart the container EditPart of the ActivityPartition
     */
    public ActivityPartitionChangeBoundsCommand(GraphNode node, Rectangle constraint, EditPart childEditPart, EditPart parentEditPart)
    {
        super(node, constraint);

        this.childEditPart = childEditPart;
        this.parentEditPart = parentEditPart;

        this.constraint = constraint;
    }

    /**
     * Store the old List of Lifelines and retrieve the new list
     * 
     * @see org.topcased.modeler.commands.ChangeBoundsCommand#execute()
     */
    public void execute()
    {
        // initialize the lists
        oldActivityNodeList = new BasicEList<ActivityNode>(((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getNodes());
        oldActivityEdgeList = new BasicEList<ActivityEdge>(((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getEdges());

        newActivityNodeList = new BasicEList<ActivityNode>();
        newActivityEdgeList = new BasicEList<ActivityEdge>();

        // Translate the constraint of the ActivityPartition with the offset of its parent (margins of the diagram)
        Rectangle zone = ((GraphicalEditPart) ((ActivityPartitionEditPart) childEditPart).getParent()).getContentPane().getClientArea();
        constraint.translate(zone.x, zone.y);

        // then retrieve all the new ActivityNode that intersect with the new bounds of the ActivityPartition
        for (Object editPart : parentEditPart.getChildren())
        {
            // We need to convert constraint as absolute coordinates
            if (editPart instanceof ActivityNodeEditPart && ((ActivityNodeEditPart) editPart).getFigure().intersects(constraint))
            {
                ActivityNode currentNode = (ActivityNode) ((ActivityNodeEditPart) editPart).getEObject();
                newActivityNodeList.add(currentNode);

                // In the second pass, we threat the ActivityEdges that are associated with the current ActivityNode (an
                // ActivityEdge is contained in a Partition only when both the source and the target ActivityNodes are
                // already contained by the Partition)
                for (ActivityEdge incomingEdge : currentNode.getIncomings())
                {
                    if (newActivityNodeList.contains(incomingEdge.getSource()))
                    {
                        newActivityEdgeList.add(incomingEdge);
                    }
                }
                for (ActivityEdge outgoingEdge : currentNode.getOutgoings())
                {
                    if (newActivityNodeList.contains(outgoingEdge.getTarget()))
                    {
                        newActivityEdgeList.add(outgoingEdge);
                    }
                }
            }
        }

        super.execute();
    }

    /**
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        super.redo();

        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getNodes().clear();
        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getNodes().addAll(newActivityNodeList);

        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getEdges().clear();
        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getEdges().addAll(newActivityEdgeList);
    }

    /**
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        super.undo();

        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getNodes().clear();
        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getNodes().addAll(oldActivityNodeList);

        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getEdges().clear();
        ((ActivityPartition) ((ActivityPartitionEditPart) childEditPart).getEObject()).getEdges().addAll(oldActivityEdgeList);

    }
}
