/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.activitydiagram.commands;

import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.uml2.uml.ActivityEdge;
import org.eclipse.uml2.uml.ActivityNode;
import org.eclipse.uml2.uml.ActivityPartition;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.activitydiagram.edit.ActivityNodeEditPart;

/**
 * The command to create an ActivityPartition
 * 
 * Creation 15 mars 07
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ActivityPartitionCreationCommand extends CreateGraphNodeCommand
{
    /** The EditPart that will contain the ActivityPartitionEditPart */
    private EditPart activityDiagramEditPart;

    /** The bounds of the ActivityPartition node */
    private Rectangle activityPartitionBounds;

    /**
     * Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObject the node to create
     * @param newParent the parentNode node
     * @param newContainerParent the model object that should contain the new childNode object
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param attach the newAttachment of the created node
     * @param featuresList the avalaible model element feature of the parentNode node where the model element of the
     *        childNode node can be added
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code>
     *        if objects already exist in the mode.
     * @param parentEditPart the EditPart that will contain the CombinedFragment EditPart
     */
    public ActivityPartitionCreationCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach,
            List featuresList, boolean needModelUpdate, EditPart parentEditPart)
    {
        super(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);

        this.activityPartitionBounds = new Rectangle(location == null ? new Point() : location, dimension == null ? new Dimension() : dimension);
        this.activityDiagramEditPart = parentEditPart;
    }

    /**
     * Create the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        super.redoModel();

        ActivityPartition activityPartition = (ActivityPartition) getChildEObject();

        for (Object editPart : activityDiagramEditPart.getChildren())
        {
            // In the first pass, we only threat the ActivityNodes
            if (editPart instanceof ActivityNodeEditPart && ((ActivityNodeEditPart) editPart).getFigure().intersects(activityPartitionBounds))
            {
                ActivityNode currentNode = (ActivityNode) ((ActivityNodeEditPart) editPart).getEObject();
                activityPartition.getNodes().add(currentNode);

                // In the second pass, we threat the ActivityEdges that are associated with the current ActivityNode (an
                // ActivityEdge is contained in a Partition only when both the source and the target ActivityNodes are
                // already contained by the Partition)
                for (ActivityEdge incomingEdge : currentNode.getIncomings())
                {
                    if (activityPartition.getNodes().contains(incomingEdge.getSource()))
                    {
                        activityPartition.getEdges().add(incomingEdge);
                    }
                }
                for (ActivityEdge outgoingEdge : currentNode.getOutgoings())
                {
                    if (activityPartition.getNodes().contains(outgoingEdge.getTarget()))
                    {
                        activityPartition.getEdges().add(outgoingEdge);
                    }
                }
            }
        }
    }

    /**
     * Remove the Execution Occurrence
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        super.undoModel();

        ((ActivityPartition) getChildEObject()).getNodes().clear();
        ((ActivityPartition) getChildEObject()).getEdges().clear();
    }

}
