/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.commands.update;

import java.util.Iterator;

import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Expression;
import org.eclipse.uml2.uml.Generalization;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Slot;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationDataStructure;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationHelper;

/**
 * Class that create a command in order to update operation parameters <br>
 * creation : 12 mai 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class InstanceSpecificationUpdateCommand extends Command
{

    /** The InstanceSpecification currently updated */
    private InstanceSpecification instance;

    /** New instance data */
    private InstanceSpecificationDataStructure datas;

    /** Old instance data */
    private InstanceSpecificationDataStructure oldDatas;

    /** Current instanciated class */
    private Class clazz;

    /** Old instanciated class */
    private Class oldClazz;

    /** Instance name */
    private String instanceName;

    /** Old instance name */
    private String oldInstanceName;

    /**
     * Create a command for updating InstanceSpecification values
     * 
     * @param inst the current instance specification object
     * @param instName the name of the instance
     * @param c the class to instanciate
     * @param data the new data
     * @deprecated Slots should be updated through the properties view.
     */
    public InstanceSpecificationUpdateCommand(InstanceSpecification inst, String instName, Class c, InstanceSpecificationDataStructure data)
    {
        instance = inst;
        instanceName = instName;
        clazz = c;
        datas = data;
    }

    /**
     * Get slot that defining feature is a given property
     * 
     * @param property the property
     * @return the slot if it exists or create it if it don't
     */
    private Slot getSlot(Property property)
    {
        Slot slot = null;

        Iterator iterator = instance.getSlots().iterator();
        boolean found = false;
        while (!found && iterator.hasNext())
        {
            slot = (Slot) iterator.next();
            if (slot.getDefiningFeature().equals(property))
            {
                found = true;
            }
        }
        if (!found)
        {
            slot = UMLFactory.eINSTANCE.createSlot();
            slot.setDefiningFeature(property);
            instance.getSlots().add(slot);
        }
        return slot;
    }

    /**
     * Add instance value on a given slot
     * 
     * @param slot slot to update
     * @param type type of the property that the slot defines
     * @param value value to set on slot
     */
    private void setValueOnSlot(Slot slot, Type type, String value)
    {
        slot.getValues().clear();
        if (value != null)
        {
            if (type instanceof Class)
            {
                InstanceSpecification inst = null;
                Iterator iterator = InstanceSpecificationHelper.getAvailableInstances(inst.getModel(), type).iterator();
                boolean found = false;
                // Look for instance that matches value
                while (!found && iterator.hasNext())
                {
                    inst = (InstanceSpecification) iterator.next();
                    if (inst.getName().equals(value))
                    {
                        found = true;
                    }
                }
                if (found)
                {
                    InstanceValue instValue = UMLFactory.eINSTANCE.createInstanceValue();
                    instValue.setInstance(inst);
                    slot.getValues().add(instValue);
                }
            }
            else if (type instanceof PrimitiveType)
            {
                if ("boolean".equals(type.getName().toLowerCase()) || "bool".equals(type.getName().toLowerCase()))
                {
                    LiteralBoolean bool = UMLFactory.eINSTANCE.createLiteralBoolean();
                    bool.setValue(new Boolean(value).booleanValue());
                    slot.getValues().add(bool);
                }
                else if ("integer".equals(type.getName().toLowerCase()) || "int".equals(type.getName().toLowerCase()))
                {
                    LiteralInteger inte = UMLFactory.eINSTANCE.createLiteralInteger();
                    inte.setValue(Integer.parseInt(value));
                    slot.getValues().add(inte);
                }
                else if ("string".equals(type.getName().toLowerCase()))
                {
                    LiteralString str = UMLFactory.eINSTANCE.createLiteralString();
                    str.setValue(value);
                    slot.getValues().add(str);
                }
                else
                {
                    Expression expression = UMLFactory.eINSTANCE.createExpression();
                    expression.setValue(null, null, value);
                    slot.getValues().add(expression);
                }
            }
            else
            {
                // TODO Handle other types
            }
        }
    }

    /**
     * Return the property of a class that matches a given name. If the property does not exist in the class, search it
     * within the classes that are generalizations of the given class
     * 
     * @param c the class owning the properties
     * @param displayName the name of the property
     * @param type the type of the property
     * @return the found property, null if not found
     */
    private Property searchProperty(Class c, String displayName, Type type)
    {
        if (c.getOwnedAttribute(displayName, type) != null)
        {
            return c.getOwnedAttribute(displayName, type);
        }
        else
        {
            Iterator iterator = c.getGeneralizations().iterator();
            while (iterator.hasNext())
            {
                Generalization gen = (Generalization) iterator.next();
                if (gen.getGeneral() instanceof Class)
                {
                    Property result = searchProperty((Class) gen.getGeneral(), displayName, type);
                    if (result != null)
                    {
                        return result;
                    }
                }
            }
        }
        return null;
    }

    /**
     * Get the old values
     */
    protected void getOldValues()
    {
        if (instance.getClassifiers().size() != 0)
        {
            oldClazz = (Class) instance.getClassifiers().get(0);
            oldDatas = new InstanceSpecificationDataStructure((Class) instance.getClassifiers().get(0), instance);
        }
        else
        {
            oldClazz = null;
            oldDatas = null;
        }

        oldInstanceName = instance.getName();
    }

    /**
     * Set the values on the Operation
     */
    protected void setValues()
    {
        // Reset instance object and slots
        instance.getSlots().clear();

        if (instance.getClassifiers().size() != 0)
        {
            // Set class
            instance.getClassifiers().set(0, clazz);
        }
        else
        {
            instance.getClassifiers().add(clazz);
        }

        instance.eSetDeliver(false);
        // Update name
        instance.setName(instanceName);

        // Set slots
        Iterator iterator = datas.getData().iterator();
        while (iterator.hasNext())
        {
            Object current = iterator.next();
            Property property = searchProperty(clazz, datas.getDisplayName(current), datas.getType(current));
            setValueOnSlot(getSlot(property), property.getType(), datas.getDisplayValue(current));
        }
        instance.eSetDeliver(true);
    }

    /**
     * Switch the old and new values
     */
    protected void switchValues()
    {
        // Inverse InstanceSpecificationDataStructure objects
        InstanceSpecificationDataStructure temp = datas;
        datas = oldDatas;
        oldDatas = temp;

        // Inverse class objects
        Class classTemp = clazz;
        clazz = oldClazz;
        oldClazz = classTemp;

        // Inverse instance names
        String stringTemp = instanceName;
        instanceName = oldInstanceName;
        oldInstanceName = stringTemp;
    }

    /**
     * Get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}
