/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.dialogs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.AggregationKind;
import org.eclipse.uml2.uml.Association;
import org.eclipse.uml2.uml.VisibilityKind;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.ModelerColorConstants;
import org.topcased.modeler.uml.classdiagram.util.AssociationHelper;

/**
 * Updating association parameters
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class AssociationEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** Current edited Association */
    private Association association;

    /** Properties of the Association */
    private String associationName;

    private String firstEndRole;

    private boolean firstEndIsNavigable;

    private boolean firstEndIsOrdered;

    private VisibilityKind firstEndVisibilityKind;

    private AggregationKind firstEndAggregationKind;

    private String firstEndLowerBound;

    private String firstEndUpperBound;

    private String secondEndRole;

    private boolean secondEndIsNavigable;

    private boolean secondEndIsOrdered;

    private VisibilityKind secondEndVisibilityKind;

    private AggregationKind secondEndAggregationKind;

    private String secondEndLowerBound;

    private String secondEndUpperBound;

    // SWT Objects
    private Composite dialogComposite;

    private Text associationNameTxt;

    private Label firstEndLabelLbl;

    private Text firstEndRoleTxt;

    private Button firstEndIsNavigableBt;

    private Button firstEndIsOrderedBt;

    private Text firstEndLowerBoundTxt;

    private Text firstEndUpperBoundTxt;

    private Button firstEndTypeNoneBt;

    private Button firstEndTypeCompositeBt;

    private Button firstEndTypeSharedBt;

    private Button firstEndVisibilityPackageBt;

    private Button firstEndVisibilityPrivateBt;

    private Button firstEndVisibilityProtectedBt;

    private Button firstEndVisibilityPublicBt;

    private Label firstEndValidationLbl;

    private Label secondEndLabelLbl;

    private Text secondEndRoleTxt;

    private Button secondEndIsNavigableBt;

    private Button secondEndIsOrderedBt;

    private Text secondEndLowerBoundTxt;

    private Text secondEndUpperBoundTxt;

    private Button secondEndTypeNoneBt;

    private Button secondEndTypeCompositeBt;

    private Button secondEndTypeSharedBt;

    private Button secondEndVisibilityPackageBt;

    private Button secondEndVisibilityPrivateBt;

    private Button secondEndVisibilityProtectedBt;

    private Button secondEndVisibilityPublicBt;

    private Label secondEndValidationLbl;

    private ModifyListener firstEndValidationListener = new ModifyListener()
    {
        public void modifyText(ModifyEvent e)
        {
            if (getButton(IDialogConstants.OK_ID) != null)
            {
                getButton(IDialogConstants.OK_ID).setEnabled(validateFirstEndGroup());
            }
        }
    };

    private ModifyListener secondEndValidationListener = new ModifyListener()
    {
        public void modifyText(ModifyEvent e)
        {
            if (getButton(IDialogConstants.OK_ID) != null)
            {
                getButton(IDialogConstants.OK_ID).setEnabled(validateSecondEndGroup());
            }
        }
    };

    /**
     * Create the dialog for a given association
     * 
     * @param assoc the association to edit
     * @param parentShell the owning shell
     */
    public AssociationEditDialog(Association assoc, Shell parentShell)
    {
        super(parentShell);

        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);

        this.association = assoc;
    }

    /**
     * Set the shell title
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Association - " + association.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createOperationGroup(dialogComposite);

        hookListeners();

        loadData();

        return dialogComposite;
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    protected void createOperationGroup(Composite parent)
    {
        TabFolder tabFolder = new TabFolder(parent, SWT.TOP);
        tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH));

        createFirstEndTabItem(tabFolder);
        createSecondEndTabItem(tabFolder);
        createGeneralTabItem(tabFolder);
    }

    /**
     * Create the Tab for the first end of the association
     * 
     * @param parent the owning Tab folder
     */
    private void createFirstEndTabItem(TabFolder parent)
    {
        Composite composite = createCompositeTab(parent, "Association first end");

        Group emptyGroup = createEmptyTab(composite);

        firstEndLabelLbl = createLabelTab(emptyGroup);
        firstEndRoleTxt = createRoleTab(emptyGroup);

        firstEndIsNavigableBt = createNavigableTab(emptyGroup);
        firstEndIsOrderedBt = createOrderedTab(emptyGroup);

        Group typeGroup = createGroupTab(composite, "Association Type");

        firstEndTypeNoneBt = createItemTab(typeGroup, "simple");
        firstEndTypeCompositeBt = createItemTab(typeGroup, "composition");
        firstEndTypeSharedBt = createItemTab(typeGroup, "aggregation");

        Group visibilityGroup = createGroupTab(composite, "Association Visibility");

        firstEndVisibilityPackageBt = createItemTab(visibilityGroup, "package");
        firstEndVisibilityPrivateBt = createItemTab(visibilityGroup, "private");
        firstEndVisibilityProtectedBt = createItemTab(visibilityGroup, "protected");
        firstEndVisibilityPublicBt = createItemTab(visibilityGroup, "public");

        Group cardinalityGroup = createCardinalityTab(composite);

        firstEndLowerBoundTxt = createBoundTab(cardinalityGroup, "Lower bound");
        firstEndUpperBoundTxt = createBoundTab(cardinalityGroup, "Upper bound");

        firstEndValidationLbl = createValidationMessage(composite);
    }

    /**
     * Create the Tab for the second end of the association
     * 
     * @param parent the owning Tab folder
     */
    private void createSecondEndTabItem(TabFolder parent)
    {
        Composite composite = createCompositeTab(parent, "Association second end");

        Group emptyGroup = createEmptyTab(composite);

        secondEndLabelLbl = createLabelTab(emptyGroup);
        secondEndRoleTxt = createRoleTab(emptyGroup);

        secondEndIsNavigableBt = createNavigableTab(emptyGroup);
        secondEndIsOrderedBt = createOrderedTab(emptyGroup);

        Group typeGroup = createGroupTab(composite, "Association Type");

        secondEndTypeNoneBt = createItemTab(typeGroup, "simple");
        secondEndTypeCompositeBt = createItemTab(typeGroup, "composition");
        secondEndTypeSharedBt = createItemTab(typeGroup, "aggregation");

        Group visibilityGroup = createGroupTab(composite, "Association Visibility");

        secondEndVisibilityPackageBt = createItemTab(visibilityGroup, "package");
        secondEndVisibilityPrivateBt = createItemTab(visibilityGroup, "private");
        secondEndVisibilityProtectedBt = createItemTab(visibilityGroup, "protected");
        secondEndVisibilityPublicBt = createItemTab(visibilityGroup, "public");

        Group cardinalityGroup = createCardinalityTab(composite);

        secondEndLowerBoundTxt = createBoundTab(cardinalityGroup, "Lower bound");
        secondEndUpperBoundTxt = createBoundTab(cardinalityGroup, "Upper bound");

        secondEndValidationLbl = createValidationMessage(composite);
    }

    /**
     * Create the tab item that contains main data
     * 
     * @param parent the tab folder parent
     */
    private void createGeneralTabItem(TabFolder parent)
    {
        // Create tab item and add it composite that fills it
        TabItem generalItem = new TabItem(parent, SWT.NONE);
        generalItem.setText("Options");
        Composite composite = new Composite(parent, SWT.NONE);
        generalItem.setControl(composite);

        // Add layout on composite
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        // Add widgets
        Label nameLbl = new Label(composite, SWT.NONE);
        nameLbl.setText("Association name");
        associationNameTxt = new Text(composite, SWT.BORDER);
        associationNameTxt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Create a composite tab
     * 
     * @param parent the parent tab folder
     * @param tabName the name of the table
     * @return the created composite Tab
     */
    private Composite createCompositeTab(TabFolder parent, String tabName)
    {
        // Create tab item and add it composite that fills it
        TabItem item = new TabItem(parent, SWT.NONE);
        item.setText(tabName);
        Composite composite = new Composite(parent, SWT.NONE);
        item.setControl(composite);

        // Add layout on composite
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));

        return composite;
    }

    /**
     * Create a group for the Property attributes
     * 
     * @param composite the parent composite
     * @return the group for the Property attributes
     */
    private Group createEmptyTab(Composite composite)
    {
        Group emptyGroup = new Group(composite, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        emptyGroup.setLayout(new GridLayout(2, false));
        emptyGroup.setLayoutData(gd);
        return emptyGroup;
    }

    /**
     * Create the label in the tab
     * 
     * @param composite the parent composite
     * @return the created label
     */
    private Label createLabelTab(Composite composite)
    {
        new Label(composite, SWT.NONE).setText("Classifier :");
        Label label = new Label(composite, SWT.NONE);
        label.setFont(getBoldFont());
        label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        return label;
    }

    /**
     * Return the default Font with Bold style
     * 
     * @return the default Font with a bold style
     */
    private Font getBoldFont()
    {
        FontData[] fData = Display.getDefault().getSystemFont().getFontData();
        fData[0].setStyle(SWT.BOLD);
        JFaceResources.getFontRegistry().put("BoldFont", fData);

        return JFaceResources.getFontRegistry().get("BoldFont");
    }

    /**
     * Create the role field in the tab
     * 
     * @param composite the owning tab
     * @return the created Text field
     */
    private Text createRoleTab(Composite composite)
    {
        new Label(composite, SWT.NONE).setText("Role :");
        Text role = new Text(composite, SWT.BORDER);
        role.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        return role;
    }

    /**
     * Create the navigable button
     * 
     * @param composite the parent composite
     * @return the created button
     */
    private Button createNavigableTab(Composite composite)
    {
        Button isNavigable = new Button(composite, SWT.CHECK);
        isNavigable.setText("Is navigable");
        GridData isNavGridData = new GridData();
        isNavGridData.horizontalSpan = 2;
        isNavigable.setLayoutData(isNavGridData);
        return isNavigable;
    }

    /**
     * Create the ordered button
     * 
     * @param composite the parent composite
     * @return the created button
     */
    private Button createOrderedTab(Composite composite)
    {
        Button isOrdered = new Button(composite, SWT.CHECK);
        isOrdered.setText("Is ordered");
        GridData isNavGridData = new GridData();
        isNavGridData.horizontalSpan = 2;
        isOrdered.setLayoutData(isNavGridData);
        return isOrdered;
    }

    /**
     * Create a Group with a given label
     * 
     * @param composite the parent composite
     * @param label the text of the group
     * @return the created group
     */
    private Group createGroupTab(Composite composite, String label)
    {
        Group group = new Group(composite, SWT.NONE);
        group.setText(label);
        group.setLayout(new GridLayout(2, false));
        group.setLayoutData(new GridData(GridData.FILL_BOTH));
        return group;
    }

    /**
     * Create a RadioButton with a text in the given Group
     * 
     * @param theGroup the owning group for the item
     * @param label the label of the button
     * @return the created button
     */
    private Button createItemTab(Group theGroup, String label)
    {
        Button itemBt = new Button(theGroup, SWT.RADIO);
        new Label(theGroup, SWT.NONE).setText(label);
        return itemBt;
    }

    /**
     * Create a group for the cardinality selection
     * 
     * @param composite the parent composite
     * @return the group for the cardinality selection
     */
    private Group createCardinalityTab(Composite composite)
    {
        Group cardinalityGroup = new Group(composite, SWT.NONE);
        cardinalityGroup.setText("Multiplicity");
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 2;
        cardinalityGroup.setLayout(new GridLayout(2, false));
        cardinalityGroup.setLayoutData(gd);
        return cardinalityGroup;
    }

    /**
     * Create the Text field for the cardinality
     * 
     * @param cardinalityGroup the ownng group of cardinality selection
     * @param label the name of the button
     * @return the text fiel
     */
    private Text createBoundTab(Group cardinalityGroup, String label)
    {
        new Label(cardinalityGroup, SWT.NONE).setText(label);
        Text bound = new Text(cardinalityGroup, SWT.BORDER);
        GridData layoutData = new GridData();
        layoutData.widthHint = 20;
        bound.setLayoutData(layoutData);
        return bound;
    }

    /**
     * Create a Label that will display error and warning messages of the validation step
     * 
     * @param composite the parent Composite
     * @return the created Label
     */
    private Label createValidationMessage(Composite composite)
    {
        Label validationLbl = new Label(composite, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        validationLbl.setLayoutData(gd);
        validationLbl.setForeground(ModelerColorConstants.COLOR_ERROR);
        return validationLbl;
    }

    /**
     * Add the listeners to the Buttons and the Text widgets
     */
    private void hookListeners()
    {
        firstEndLowerBoundTxt.addModifyListener(firstEndValidationListener);
        firstEndUpperBoundTxt.addModifyListener(firstEndValidationListener);
        firstEndRoleTxt.addModifyListener(firstEndValidationListener);

        secondEndLowerBoundTxt.addModifyListener(secondEndValidationListener);
        secondEndUpperBoundTxt.addModifyListener(secondEndValidationListener);
        secondEndRoleTxt.addModifyListener(secondEndValidationListener);
    }

    /**
     * Check if the informations contained in the Group are valid
     * 
     * @return true if it is OK
     */
    protected boolean validateFirstEndGroup()
    {
        if ("".equals(firstEndRoleTxt.getText()))
        {
            firstEndValidationLbl.setText("The name of the Role should not be empty.");
            return false;
        }

        try
        {
            int lowerBound = Integer.parseInt(firstEndLowerBoundTxt.getText());
            if (lowerBound < 0)
            {
                firstEndValidationLbl.setText("lowerBound should be greater or equal to 0");
                return false;
            }
        }
        catch (NumberFormatException e)
        {
            firstEndValidationLbl.setText("lowerBound should be a valid integer value");
            return false;
        }

        try
        {
            if (!"*".equals(firstEndUpperBoundTxt.getText()))
            {
                int upperBound = Integer.parseInt(firstEndUpperBoundTxt.getText());
                int lowerBound = Integer.parseInt(firstEndLowerBoundTxt.getText());
                if (upperBound < 1 || upperBound < lowerBound)
                {
                    firstEndValidationLbl.setText("upperBound can not be lower than lowerBound.");
                    return false;
                }
            }
        }
        catch (NumberFormatException e)
        {
            firstEndValidationLbl.setText("upperBound should be a valid integer value");
            return false;
        }

        firstEndValidationLbl.setText("");
        return true;
    }

    /**
     * Check if the informations contained in the Group are valid
     * 
     * @return true if it is OK
     */
    protected boolean validateSecondEndGroup()
    {
        if ("".equals(secondEndRoleTxt.getText()))
        {
            secondEndValidationLbl.setText("The name of the Role should not be empty.");
            return false;
        }

        try
        {
            int lowerBound = Integer.parseInt(secondEndLowerBoundTxt.getText());
            if (lowerBound < 0)
            {
                secondEndValidationLbl.setText("lowerBound should be greater or equal to 0");
                return false;
            }
        }
        catch (NumberFormatException e)
        {
            secondEndValidationLbl.setText("lowerBound should be a valid integer value");
            return false;
        }

        try
        {
            if (!"*".equals(secondEndUpperBoundTxt.getText()))
            {
                int upperBound = Integer.parseInt(secondEndUpperBoundTxt.getText());
                int lowerBound = Integer.parseInt(secondEndLowerBoundTxt.getText());
                if (upperBound < 1 || upperBound < lowerBound)
                {
                    secondEndValidationLbl.setText("upperBound can not be lower than lowerBound.");
                    return false;
                }
            }
        }
        catch (NumberFormatException e)
        {
            secondEndValidationLbl.setText("upperBound should be a valid integer value");
            return false;
        }

        secondEndValidationLbl.setText("");
        return true;
    }

    /**
     * Fill the fields of the association current datas
     */
    private void loadData()
    {
        AssociationHelper associationHelper = new AssociationHelper(association);

        // Main data
        associationNameTxt.setText(associationHelper.getAssociationName());

        if (associationHelper.isAssociationReady())
        {
            // First end data
            firstEndLabelLbl.setText(associationHelper.getAssociationFirstEndClass().getName());
            firstEndRoleTxt.setText(associationHelper.getAssociationFirstEndRole());
            firstEndIsNavigableBt.setSelection(associationHelper.getAssociationFirstEndIsNavigable());
            firstEndIsOrderedBt.setSelection(associationHelper.getAssociationFirstEndIsOrdered());
            firstEndLowerBoundTxt.setText(associationHelper.getAssociationFirstEndLowerBound());
            firstEndUpperBoundTxt.setText(associationHelper.getAssociationFirstEndUpperBound());

            VisibilityKind visibilityKind = associationHelper.getAssociationFirstEndVisibilityKind();
            firstEndVisibilityPackageBt.setSelection(visibilityKind.equals(VisibilityKind.PACKAGE_LITERAL));
            firstEndVisibilityPrivateBt.setSelection(visibilityKind.equals(VisibilityKind.PRIVATE_LITERAL));
            firstEndVisibilityProtectedBt.setSelection(visibilityKind.equals(VisibilityKind.PROTECTED_LITERAL));
            firstEndVisibilityPublicBt.setSelection(visibilityKind.equals(VisibilityKind.PUBLIC_LITERAL));

            AggregationKind aggregationKind = associationHelper.getAssociationFirstEndAggregationKind();
            firstEndTypeSharedBt.setSelection(aggregationKind.equals(AggregationKind.SHARED_LITERAL));
            firstEndTypeCompositeBt.setSelection(aggregationKind.equals(AggregationKind.COMPOSITE_LITERAL));
            firstEndTypeNoneBt.setSelection(aggregationKind.equals(AggregationKind.NONE_LITERAL));

            // Second end data
            secondEndLabelLbl.setText(associationHelper.getAssociationSecondEndClass().getName());
            secondEndRoleTxt.setText(associationHelper.getAssociationSecondEndRole());
            secondEndIsNavigableBt.setSelection(associationHelper.getAssociationSecondEndIsNavigable());
            secondEndIsOrderedBt.setSelection(associationHelper.getAssociationSecondEndIsOrdered());
            secondEndLowerBoundTxt.setText(associationHelper.getAssociationSecondEndLowerBound());
            secondEndUpperBoundTxt.setText(associationHelper.getAssociationSecondEndUpperBound());

            visibilityKind = associationHelper.getAssociationSecondEndVisibilityKind();
            secondEndVisibilityPackageBt.setSelection(visibilityKind.equals(VisibilityKind.PACKAGE_LITERAL));
            secondEndVisibilityPrivateBt.setSelection(visibilityKind.equals(VisibilityKind.PRIVATE_LITERAL));
            secondEndVisibilityProtectedBt.setSelection(visibilityKind.equals(VisibilityKind.PROTECTED_LITERAL));
            secondEndVisibilityPublicBt.setSelection(visibilityKind.equals(VisibilityKind.PUBLIC_LITERAL));

            aggregationKind = associationHelper.getAssociationSecondEndAggregationKind();
            secondEndTypeSharedBt.setSelection(aggregationKind.equals(AggregationKind.SHARED_LITERAL));
            secondEndTypeCompositeBt.setSelection(aggregationKind.equals(AggregationKind.COMPOSITE_LITERAL));
            secondEndTypeNoneBt.setSelection(aggregationKind.equals(AggregationKind.NONE_LITERAL));
        }
        else
        {
            // Association does not contain member ends
            firstEndTypeNoneBt.setSelection(true);
            secondEndTypeNoneBt.setSelection(true);
        }
    }

    /**
     * Save the datas before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        // Prepare general data
        associationName = associationNameTxt.getText();

        // Prepare first end data
        firstEndRole = firstEndRoleTxt.getText();
        firstEndIsNavigable = firstEndIsNavigableBt.getSelection();
        firstEndIsOrdered = firstEndIsOrderedBt.getSelection();
        firstEndLowerBound = firstEndLowerBoundTxt.getText();
        String upperBound = firstEndUpperBoundTxt.getText();
        if ("*".equals(upperBound))
        {
            upperBound = "-1";
        }
        firstEndUpperBound = upperBound;

        if (firstEndVisibilityPackageBt.getSelection())
        {
            firstEndVisibilityKind = VisibilityKind.PACKAGE_LITERAL;
        }
        else if (firstEndVisibilityPrivateBt.getSelection())
        {
            firstEndVisibilityKind = VisibilityKind.PRIVATE_LITERAL;
        }
        else if (firstEndVisibilityProtectedBt.getSelection())
        {
            firstEndVisibilityKind = VisibilityKind.PROTECTED_LITERAL;
        }
        else
        {
            firstEndVisibilityKind = VisibilityKind.PUBLIC_LITERAL;
        }

        if (firstEndTypeSharedBt.getSelection())
        {
            firstEndAggregationKind = AggregationKind.SHARED_LITERAL;
        }
        else if (firstEndTypeCompositeBt.getSelection())
        {
            firstEndAggregationKind = AggregationKind.COMPOSITE_LITERAL;
        }
        else
        {
            firstEndAggregationKind = AggregationKind.NONE_LITERAL;
        }

        // Prepare second end data
        secondEndRole = secondEndRoleTxt.getText();
        secondEndIsNavigable = secondEndIsNavigableBt.getSelection();
        secondEndIsOrdered = secondEndIsOrderedBt.getSelection();
        secondEndLowerBound = secondEndLowerBoundTxt.getText();
        upperBound = secondEndUpperBoundTxt.getText();
        if ("*".equals(upperBound))
        {
            upperBound = "-1";
        }
        secondEndUpperBound = upperBound;

        if (secondEndVisibilityPackageBt.getSelection())
        {
            secondEndVisibilityKind = VisibilityKind.PACKAGE_LITERAL;
        }
        else if (secondEndVisibilityPrivateBt.getSelection())
        {
            secondEndVisibilityKind = VisibilityKind.PRIVATE_LITERAL;
        }
        else if (secondEndVisibilityProtectedBt.getSelection())
        {
            secondEndVisibilityKind = VisibilityKind.PROTECTED_LITERAL;
        }
        else
        {
            secondEndVisibilityKind = VisibilityKind.PUBLIC_LITERAL;
        }

        if (secondEndTypeSharedBt.getSelection())
        {
            secondEndAggregationKind = AggregationKind.SHARED_LITERAL;
        }
        else if (secondEndTypeCompositeBt.getSelection())
        {
            secondEndAggregationKind = AggregationKind.COMPOSITE_LITERAL;
        }
        else
        {
            secondEndAggregationKind = AggregationKind.NONE_LITERAL;
        }

        super.okPressed();
    }

    /**
     * Return all the informations concerning an Association
     * 
     * @return a Map
     */
    public Map getAssociationData()
    {
        Map resultMap = new HashMap();

        resultMap.put(AssociationHelper.ASSOCIATION_NAME, associationName);

        resultMap.put(AssociationHelper.FIRST_END_ROLE, firstEndRole);
        resultMap.put(AssociationHelper.FIRST_END_IS_NAVIGABLE, new Boolean(firstEndIsNavigable));
        resultMap.put(AssociationHelper.FIRST_END_IS_ORDERED, new Boolean(firstEndIsOrdered));
        resultMap.put(AssociationHelper.FIRST_END_LOWER_BOUND, firstEndLowerBound);
        resultMap.put(AssociationHelper.FIRST_END_UPPER_BOUND, firstEndUpperBound);
        resultMap.put(AssociationHelper.FIRST_END_VISIBILITY_KIND, firstEndVisibilityKind);
        resultMap.put(AssociationHelper.FIRST_END_AGGREGATION_KIND, firstEndAggregationKind);

        resultMap.put(AssociationHelper.SECOND_END_ROLE, secondEndRole);
        resultMap.put(AssociationHelper.SECOND_END_IS_NAVIGABLE, new Boolean(secondEndIsNavigable));
        resultMap.put(AssociationHelper.SECOND_END_IS_ORDERED, new Boolean(secondEndIsOrdered));
        resultMap.put(AssociationHelper.SECOND_END_LOWER_BOUND, secondEndLowerBound);
        resultMap.put(AssociationHelper.SECOND_END_UPPER_BOUND, secondEndUpperBound);
        resultMap.put(AssociationHelper.SECOND_END_VISIBILITY_KIND, secondEndVisibilityKind);
        resultMap.put(AssociationHelper.SECOND_END_AGGREGATION_KIND, secondEndAggregationKind);

        return resultMap;
    }

}
