/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.dialogs;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationDataStructure;

/**
 * Updating operation parameters
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class InstanceSpecificationEditDialog extends Dialog implements ITopcasedDialogConstants
{
    /** Current edited operation */

    private InstanceSpecificationViewer attributesViewer;

    private Combo classifiers;

    private ArrayList classes;

    // SWT Objects
    private Composite dialogComposite;

    private InstanceSpecification instance;

    private Text instanceNameField;

    // Model objects
    private InstanceSpecificationDataStructure data;

    private Class clazz;

    private String instanceName;

    /**
     * Create the dialog for a given instance specification
     * 
     * @param inst the instance specification to edit
     * @param parentShell the parent shell
     * @deprecated Use the properties view to update the Slot elements
     */
    public InstanceSpecificationEditDialog(InstanceSpecification inst, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        instance = inst;
    }

    /**
     * Set the title of the shell
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("InstanceSpecification " + instance.getName());
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Creates the group
     * 
     * @param parent the parent Composite
     */
    protected void createOperationGroup(Composite parent)
    {
        parent.setLayout(new GridLayout(2, false));

        new Label(parent, SWT.NONE).setText("Instance name ");

        instanceNameField = new Text(parent, SWT.BORDER);
        if (instance.getName() != null)
        {
            instanceNameField.setText(instance.getName());
        }
        instanceNameField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        new Label(parent, SWT.NONE).setText("Is instance of");

        classifiers = new Combo(parent, SWT.NONE);
        classifiers.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        classes = UMLTools.getAllObjects(instance.getModel(), Class.class);

        String[] classesArray = new String[classes.size()];
        int i = 0;
        for (Iterator it = classes.iterator(); it.hasNext();)
        {
            Class c = (Class) it.next();
            classesArray[i++] = c.getName();
        }
        classifiers.setItems(classesArray);

        attributesViewer = new InstanceSpecificationViewer(parent, instance);
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 2;
        gd.minimumHeight = 200;
        attributesViewer.getTableViewer().getTable().setLayoutData(gd);

        classifiers.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (classifiers.getSelectionIndex() != -1)
                {
                    attributesViewer.updateViewer((Class) classes.get(classifiers.getSelectionIndex()));
                }
            }
        });

        if (instance.getClassifiers().size() != 0)
        {
            classifiers.select(classes.indexOf(instance.getClassifiers().get(0)));
            attributesViewer.updateViewer((Class) instance.getClassifiers().get(0));
        }
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createOperationGroup(dialogComposite);
        return dialogComposite;
    }

    /**
     * Save the datas before the widgets are disposed
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        data = attributesViewer.getData();
        clazz = (Class) classes.get(classifiers.getSelectionIndex());
        instanceName = instanceNameField.getText();
        super.okPressed();
    }

    /**
     * Get the InstanceSpecificationDataStructure data
     * 
     * @return the new datas
     */
    public InstanceSpecificationDataStructure getNewData()
    {
        return data;
    }

    /**
     * Get the class that the instanceSpecification instanciates
     * 
     * @return the class that the instanceSpecification instanciates
     */
    public Class getInstanciatedClass()
    {
        return clazz;
    }

    /**
     * Get the name of the instance specification
     * 
     * @return the name of the instance specification
     */
    public String getInstanceName()
    {
        return instanceName;
    }

}
