/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.dialogs;

import java.util.Arrays;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.PrimitiveType;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationDataStructure;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationHelper;

/**
 * Handle table viewer for Instance Specification update <br>
 * creation : 27 may 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class InstanceSpecificationViewer
{

    private static final int COLUMN1_WIDTH = 130;

    private static final int COLUMN2_WIDTH = 130;

    private static final int COLUMN3_WIDTH = 180;

    private static final int TEXT_FIELD_WIDTH = 60;

    /** The table viewer object */
    private TableViewer tableViewer;

    /** Editing structure */
    private CellEditor[] editors;

    /** Table column names */
    private String[] columnNames = new String[] {"Property name", "Type", "Value"};

    /** Current instance specification */
    private InstanceSpecification instance;

    /** Data structure */
    private InstanceSpecificationDataStructure dataObject;

    /** The table object */
    private Table table;

    /**
     * The constructor
     * 
     * @param parent the owning composite
     * @param inst the instance to show
     */
    public InstanceSpecificationViewer(Composite parent, InstanceSpecification inst)
    {
        instance = inst;
        createTableViewer(parent);
    }

    /**
     * Return the data structure
     * 
     * @return the data structure
     */
    public InstanceSpecificationDataStructure getData()
    {
        return dataObject;
    }

    /**
     * Create table
     * 
     * @param composite the owning composite
     * @return the created table
     */
    private Table createTable(Composite composite)
    {
        int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

        Table t = new Table(composite, style);
        TableColumn tc1 = new TableColumn(t, SWT.LEFT);
        tc1.setText(columnNames[0]);
        tc1.setWidth(COLUMN1_WIDTH);

        TableColumn tc2 = new TableColumn(t, SWT.LEFT);
        tc2.setText(columnNames[1]);
        tc2.setWidth(COLUMN2_WIDTH);

        TableColumn tc3 = new TableColumn(t, SWT.LEFT);
        tc3.setText(columnNames[2]);
        tc3.setWidth(COLUMN3_WIDTH);

        t.setHeaderVisible(true);
        t.setLinesVisible(true);

        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.grabExcessVerticalSpace = true;
        gridData.horizontalSpan = 3;
        t.setLayoutData(gridData);

        return t;
    }

    /**
     * Return the viewer of the table
     * 
     * @return the viewer of the table
     */
    public TableViewer getTableViewer()
    {
        return tableViewer;
    }

    /**
     * Create table viewer
     * 
     * @param composite the owning composite
     */
    private void createTableViewer(Composite composite)
    {
        table = createTable(composite);
        tableViewer = new TableViewer(table);

        tableViewer.setUseHashlookup(true);
        tableViewer.setColumnProperties(columnNames);

        editors = new CellEditor[3];

        TextCellEditor textEditor = new TextCellEditor(table);
        ((Text) textEditor.getControl()).setTextLimit(TEXT_FIELD_WIDTH);
        editors[0] = textEditor;

        TextCellEditor textEditor2 = new TextCellEditor(table);
        ((Text) textEditor2.getControl()).setTextLimit(TEXT_FIELD_WIDTH);
        editors[1] = textEditor2;

        editors[2] = new ComboBoxCellEditor(table, new String[] {}, SWT.NONE);

        tableViewer.setCellEditors(editors);
        tableViewer.setContentProvider(new ISContentProvider());
        tableViewer.setLabelProvider(new ISLabelProvider());
        tableViewer.setCellModifier(new ISCellModifier());
    }

    /**
     * Update the viewer with a given class
     * 
     * @param clazz the new class
     */
    public void updateViewer(Class clazz)
    {
        dataObject = new InstanceSpecificationDataStructure(clazz, instance);
        if (clazz == null)
        {
            tableViewer.setInput(null);
        }
        else
        {
            tableViewer.setInput(dataObject.getData());
        }
    }

    /**
     * Internal class that handle modification
     * 
     * @author matg
     */
    class ISCellModifier implements ICellModifier
    {
        /**
         * Can only modify the third column
         * 
         * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
         */
        public boolean canModify(Object element, String property)
        {
            boolean result = false;
            int index = Arrays.asList(columnNames).indexOf(property);
            switch (index)
            {
                case 0:
                    result = false;
                    break;
                case 1:
                    result = false;
                    break;
                case 2:
                    // If property type is not set, the user can't edit the
                    // value
                    if (dataObject.getType(element) != null)
                    {
                        result = true;
                    }
                    else
                    {
                        result = false;
                    }
                    break;
                default:
                    break;
            }
            return result;
        }

        /**
         * Return the label for the two first column
         * 
         * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
         */
        public Object getValue(Object element, String property)
        {
            Object result = null;
            int index = Arrays.asList(columnNames).indexOf(property);
            switch (index)
            {
                case 0:
                    result = dataObject.getDisplayName(element);
                    break;
                case 1:
                    result = dataObject.getDisplayType(element);
                    break;
                case 2:
                    if (Class.class.isInstance(dataObject.getType(element)))
                    {
                        String[] values = InstanceSpecificationHelper.getAvailableInstanceNames(instance.getModel(), dataObject.getType(element));
                        if (values.length == 0)
                        {
                            values = new String[] {"No available instance"};
                        }
                        String selection = dataObject.getDisplayValue(element);
                        ((ComboBoxCellEditor) editors[2]).setItems(values);
                        result = new Integer(Arrays.asList(values).indexOf(selection));
                    }
                    else if (PrimitiveType.class.isInstance(dataObject.getType(element)))
                    {
                        if ("boolean".equals(dataObject.getDisplayType(element)) || "bool".equals(dataObject.getDisplayType(element)))
                        {
                            String[] values = InstanceSpecificationHelper.getBooleanValues();
                            String selection = dataObject.getDisplayValue(element);
                            ((ComboBoxCellEditor) editors[2]).setItems(values);
                            result = new Integer(Arrays.asList(values).indexOf(selection));
                        }
                        else
                        {
                            String[] values = new String[] {};
                            String selection = dataObject.getDisplayValue(element);
                            if (selection != null)
                            {
                                values = new String[] {selection};
                            }
                            ((ComboBoxCellEditor) editors[2]).setItems(values);
                            result = new Integer(0);
                        }
                    }
                    else
                    {
                        // TODO Handle other types
                    }
                    break;
                default:
                    break;
            }
            return result;
        }

        /**
         * Handle the modification of the thirs column
         * 
         * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
         */
        public void modify(Object element, String property, Object value)
        {
            TableItem item = (TableItem) element;
            int index = Arrays.asList(columnNames).indexOf(property);
            switch (index)
            {
                case 2:
                    dataObject.updateObject(item.getData(), value, ((CCombo) ((ComboBoxCellEditor) editors[2]).getControl()).getText(), instance);
                    break;
                default:
                    break;
            }
            tableViewer.update(item.getData(), null);
        }
    }

    /**
     * Internal class that handle content
     * 
     * @author matg
     */
    class ISContentProvider implements IStructuredContentProvider
    {
        /**
         * Returns every elements
         * 
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement)
        {
            return dataObject.getData().toArray();
        }

        /**
         * nothing to do
         * 
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose()
        {
            // nothing to do
        }

        /**
         * nothing to do
         * 
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
         *      java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
        {
            // nothing to do
        }
    }

    /**
     * Internal class that handle label
     * 
     * @author matg
     */
    class ISLabelProvider extends LabelProvider implements ITableLabelProvider
    {
        /**
         * No image
         * 
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
         */
        public Image getColumnImage(Object element, int columnIndex)
        {
            return null;
        }

        /**
         * Return the label for the 2 first columns, the value for the third
         * 
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
         */
        public String getColumnText(Object element, int columnIndex)
        {
            String result = "";
            switch (columnIndex)
            {
                case 0:
                    result = dataObject.getDisplayName(element);
                    break;
                case 1:
                    result = dataObject.getDisplayType(element);
                    if (result == null || result.length() == 0)
                    {
                        result = "Property type no set";
                    }
                    break;
                case 2:
                    result = dataObject.getDisplayValue(element);
                    break;
                default:
                    break;
            }
            return result;
        }
    }
}
