/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *   Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware Technologies),
 *   Nicolas Lalevée (Anyware Technologies) - initial API and implementation 
 ******************************************************************************/

package org.topcased.modeler.uml.classdiagram.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.Type;

/**
 * Provide a structure for handling Operation data <br>
 * creation : 27 may 2005
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public class OperationDataStructure
{

    /**
     * Internal class to handle a parameter
     */
    public class ParameterObject
    {

        /** Parameter name */
        private String name;

        /** Parameter type */
        private Type type;

        /**
         * The constructor
         * 
         * @param n parameter name
         * @param t parameter type
         */
        public ParameterObject(String n, Type t)
        {
            name = n;
            type = t;
        }

        /**
         * Get property name
         * 
         * @return name of the property
         */
        public String getName()
        {
            return name;
        }

        /**
         * Get property type
         * 
         * @return type of the property
         */
        public Type getType()
        {
            return type;
        }

        /**
         * Set new name for the parameter
         * 
         * @param newName the new name
         */
        public void setName(String newName)
        {
            name = newName;
        }

        /**
         * Set new type for the parameter
         * 
         * @param newType the new type
         */
        public void setType(Type newType)
        {
            type = newType;
        }

    } // End internal class

    /** A collection for ParameterObject objects */
    private ArrayList data;

    /**
     * The constructor
     * 
     * @param operation the Operation
     */
    public OperationDataStructure(Operation operation)
    {
        data = new ArrayList();
        if (operation != null)
        {
            addAll(operation.getOwnedParameters());
        }
    }

    /**
     * Add a parameter to the structure
     * 
     * @param parameter the parameter to add
     */
    public void add(Parameter parameter)
    {
        data.add(new ParameterObject(parameter.getName(), parameter.getType()));
    }

    /**
     * Add a parameter to the structure
     * 
     * @param name the parameter name
     * @param type the parameter type
     */
    public void add(String name, Type type)
    {
        data.add(new ParameterObject(name, type));
    }

    /**
     * Remove a parameter or the name or etc..; from the structure
     * 
     * @param object the object to remove
     */
    public void remove(Object object)
    {
        data.remove(object);
    }

    /**
     * Add a collection of parameters to the operation
     * 
     * @param parameters the collection of parameters to add
     */
    public void addAll(Collection parameters)
    {
        Iterator itParameters = parameters.iterator();
        while (itParameters.hasNext())
        {
            Parameter parameter = (Parameter) itParameters.next();
            add(parameter);
        }
    }

    /**
     * Get the datas
     * 
     * @return a Collection of
     */
    public Collection getData()
    {
        return data;
    }

    /**
     * Get the name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name
     */
    public String getDisplayName(Object object)
    {
        String result = ((ParameterObject) object).getName();
        if (result == null || result.length() == 0)
        {
            result = "Property name no set";
        }
        return result;
    }

    /**
     * Get the type name of a given object
     * 
     * @param object OperationDataObject object
     * @return the name of the type
     */
    public String getDisplayType(Object object)
    {
        Type type = getType(object);
        if (type == null)
        {
            return null;
        }
        return type.getName();
    }

    /**
     * Get the type of a given object
     * 
     * @param object OperationDataObject object
     * @return the type
     */
    public Type getType(Object object)
    {
        return ((ParameterObject) object).getType();
    }

    /**
     * Set the name of the parameter object
     * 
     * @param object a ParameterObject
     * @param name the new name
     */
    public void setName(Object object, String name)
    {
        ((ParameterObject) object).setName(name);
    }

    /**
     * Set the type of the parameter object
     * 
     * @param object a ParameterObject
     * @param type the new type
     */
    public void setType(Object object, Type type)
    {
        ((ParameterObject) object).setType(type);
    }
}
