/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.commands;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.uml2.uml.EncapsulatedClassifier;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.DeleteGraphElementCommand;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.utils.Utils;

/**
 * This command is used to update the Port inserted in a Property Node.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 * 
 */
public class UpdatePortCommand extends CompoundCommand
{
	/** The GraphNode that should be updated */
	private GraphNode graphNode;

	/** The EditDomain associated */
	private EditDomain editDomain;

	private ICreationUtils creationUtils;

	/** Default location for the port placement : */
	private static final int PORT_POSITION = 2; // this value corresponds to the top border of the current property

	/**
	 * The Constructor
	 * 
	 * @param domain the EditDomain
	 * @param node the GraphNode
	 * @param utils the creationUtils factory
	 */
	public UpdatePortCommand(EditPart host, GraphNode node)
	{
		super("Update Ports contained in Property");

		graphNode = node;
		editDomain = host.getViewer().getEditDomain();
		creationUtils = ((ModelerGraphicalViewer) host.getViewer()).getModelerEditor().getActiveConfiguration().getCreationUtils();

		initialize();
	}

	/**
	 * Initialize
	 */
	private void initialize()
	{
		EObject eObject = Utils.getElement(graphNode);

		List listPorts = new ArrayList();

		deleteAllPorts();

		if (eObject instanceof Property && ((Property) eObject).getType() != null)
		{
			Type typeProperty = ((Property) eObject).getType();

			if (typeProperty instanceof EncapsulatedClassifier)
			{
				listPorts = ((EncapsulatedClassifier) typeProperty).getOwnedPorts();

				addNewPorts(listPorts);
			}
		}
	}

	/**
	 * Adds eventual ports of the classifier representing by
	 * 
	 * @param listPorts a list of elements
	 */
	private void addNewPorts(List listPorts)
	{
		// initial position
		int posX = 8;

		for (Iterator iter = listPorts.iterator(); iter.hasNext();)
		{
			// retrieve the port to display on the property
			Port port = (Port) iter.next();

			if (getGraphicalRepresentation(port) == null)
			{
				// creates the new graphNode and add it to the parentNode
				GraphNode node = (GraphNode) creationUtils.createGraphElement(port);

				// the command is added
				add(new CreateGraphNodeCommand(editDomain, node, graphNode, new Point(posX, 1), node.getSize(), PORT_POSITION));

				// determines the position of the next port if it exists
				posX += 25;
			}
		}
	}

	/**
	 * Deletes all Ports of the current graphNode
	 */
	private void deleteAllPorts()
	{
		for (Iterator iter = graphNode.getContained().iterator(); iter.hasNext();)
		{
			GraphElement elt = (GraphElement) iter.next();
			EObject object = Utils.getElement(elt);
			if (object instanceof Port)
			{
				add(new DeleteGraphElementCommand(elt, true));
			}
		}
	}

	/**
	 * Gets the graphNode associated with the model object in the current selected editPart
	 * 
	 * @param modelObject the model object to display
	 * @return the GraphNode or null if it does not exist
	 */
	private GraphNode getGraphicalRepresentation(EObject modelObject)
	{
		for (Iterator iter = graphNode.getContained().iterator(); iter.hasNext();)
		{
			GraphElement elt = (GraphElement) iter.next();
			if (Utils.getElement(elt) == modelObject && elt instanceof GraphNode)
			{
				return (GraphNode) elt;
			}
		}
		return null;
	}
}
