/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.figures;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.swt.SWT;
import org.topcased.draw2d.figures.ComposedLabel;
import org.topcased.draw2d.figures.EditableLabel;
import org.topcased.draw2d.figures.IContainerFigure;
import org.topcased.draw2d.figures.ILabel;
import org.topcased.draw2d.figures.ILabelFigure;
import org.topcased.draw2d.figures.Label;
import org.topcased.draw2d.layout.BorderAttachedLayout;
import org.topcased.draw2d.layout.CenterLayout;

/**
 * Represents the 'Property' Figure.<br>
 * This figure is represented by a simple rectangle composed by two distinct layouts.<br>
 * Two layouts are used to correctly attached Port figure to the Property figure borders.<br>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class PropertyFigure extends Figure implements IContainerFigure, ILabelFigure
{
	/** The label displayed inside the figure */
	private ILabel label;

	/** The external figure */
	private IFigure contentPane;

	/** The internal figure */
	private CustomRectangleFigure innerRectangle;

	/**
	 * Constructor
	 * 
	 * @generated
	 */
	public PropertyFigure()
	{
		// internal figure
		setLayoutManager(new StackLayout());
		add(createBackgroundFigure());
		
		// external figure
		contentPane = new Figure();
		contentPane.setLayoutManager(new BorderAttachedLayout());
		add(contentPane);
	}

	/**
	 * Draws an innerRectangle that will be the real background figure. The Anchor will be attached to this inner rectangle.<br>
	 * The innerRectangle is also the container of the HeaderComponent and the Label of the Component that are arranged in a toolbarLayout.
	 * 
	 * @return the figure
	 */
	protected IFigure createBackgroundFigure()
	{
		IFigure rectangle = new Figure();
		rectangle.setLayoutManager(new BorderLayout());
		rectangle.setBorder(new MarginBorder(7));

		innerRectangle = new CustomRectangleFigure();
		innerRectangle.setLayoutManager(new CenterLayout());
		innerRectangle.setBorder(new MarginBorder(10));
		innerRectangle.setOpaque(true);
		label = createLabel();
		innerRectangle.add(label);

		rectangle.add(innerRectangle, new Integer(PositionConstants.CENTER));

		return rectangle;
	}

	/**
	 * Returns the figure drawn behind the label. In this case, we only return the inner rectangle drawn.
	 * 
	 * @return the background figure
	 */
	public IFigure getBackgroundFigure()
	{
		return innerRectangle;
	}

	/**
	 * Sets the border line style
	 * 
	 * @param lineStyle The line border style
	 */
	public void setBorderStyle(int lineStyle)
	{
		innerRectangle.setBorderStyle(lineStyle);
	}

	/**
	 * Creates the label of the figure.<br>
	 * <b>Subclasses can override this method to customize the appearance of the label (for example you can return a {@link ComposedLabel} instead)</b>
	 * 
	 * @return the label of the figure
	 * @generated NOT
	 */
	protected ILabel createLabel()
	{
		// The first part represents the editable label i.e the name of the instance
		// The second part represents the separator ':' and the type of the instance
		// The last part represents the cardinalities
		return new ComposedLabel(new EditableLabel(), new Label(), new Label(), true);
	}

	/**
	 * Gets the figure label
	 * 
	 * @see org.topcased.draw2d.figures.ILabelFigure#getLabel()
	 */
	public ILabel getLabel()
	{
		return label;
	}

	/**
	 * Gets the content pane
	 * 
	 * @see org.topcased.draw2d.figures.IContainerFigure#getContentPane()
	 */
	public IFigure getContentPane()
	{
		return contentPane;
	}
	
	/**
	 * Extended class of RectangleFigure allowing to manage outline style of the inner container.
	 * 
	 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
	 * 
	 */
	private class CustomRectangleFigure extends RectangleFigure
	{
		/** Default line style to plot */
		private int style = Graphics.LINE_SOLID;

		/** Constant representing the custom line style */
		private final int[] CUSTOM_DASH = new int[] {6,7};
		
		/**
		 * Default constructor
		 */
		public CustomRectangleFigure()
		{
			super();
		}

		/**
		 * Constructor
		 */
		public CustomRectangleFigure(int borderStyle)
		{
			super();
			style = borderStyle;
		}

		/**
		 * Sets the border style to adopt
		 * 
		 * @param borderStyle The border style to set (Dash, Line, Dot...)
		 */
		public void setBorderStyle(int borderStyle)
		{
			style = borderStyle;
		}

		/**
		 * Draws the outline graphics with the corresponding outline style.<br>
		 * There are mainly two possibilities : solid line or custom dashed line. 
		 * 
		 * 
		 * @param graphics The graphics object 
		 * @see org.eclipse.draw2d.RectangleFigure#outlineShape(org.eclipse.draw2d.Graphics)
		 */
		protected void outlineShape(Graphics graphics)
		{
			if (style == SWT.LINE_CUSTOM)
			{
				// here we use our personal dashed line instead of default SWT.DASH_LINE
				graphics.setLineDash(CUSTOM_DASH);
			}
			else 
			{
				graphics.setLineStyle(style);
			}
			super.outlineShape(graphics);
		};
	}
	
	
}