/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.modeler.uml.compositestructuresdiagram.policies;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.Port;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.commands.CreateTypedEdgeCommand;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy;
import org.topcased.modeler.uml.compositestructuresdiagram.commands.ConnectorEdgeCreationCommand;
import org.topcased.modeler.utils.SourceTargetData;
import org.topcased.modeler.utils.Utils;

/**
 * Connector edge creation.
 * Method <b>isValidConnector</b> defines what kinds of connector links are allowed and forbidden.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class ConnectorEdgeCreationEditPolicy extends AbstractEdgeCreationEditPolicy
{
	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#createCommand(org.eclipse.gef.EditDomain, org.topcased.modeler.di.model.GraphEdge,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected CreateTypedEdgeCommand createCommand(EditDomain domain, GraphEdge edge, GraphElement source)
	{
		return new ConnectorEdgeCreationCommand(domain, edge, source);
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkEdge(org.topcased.modeler.di.model.GraphEdge)
	 * @generated
	 */
	protected boolean checkEdge(GraphEdge edge)
	{
		return Utils.getElement(edge) instanceof Connector;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkSource(org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkSource(GraphElement source)
	{
		return Utils.getElement(source) instanceof Property;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkTargetForSource(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected boolean checkTargetForSource(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		if (sourceObject instanceof Property && targetObject instanceof Property)
		{
			return (isValidConnector(source, target) && areInSameClassifier(source, target));
		}
		return false;
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#checkCommand(org.eclipse.gef.commands.Command)
	 * @generated
	 */
	protected boolean checkCommand(Command command)
	{
		return (command instanceof ConnectorEdgeCreationCommand);
	}

	/**
	 * @see org.topcased.modeler.edit.policies.AbstractEdgeCreationEditPolicy#getSourceTargetData(org.topcased.modeler.di.model.GraphElement,
	 *      org.topcased.modeler.di.model.GraphElement)
	 * @generated
	 */
	protected SourceTargetData getSourceTargetData(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		if (sourceObject instanceof Property && targetObject instanceof Property)
		{
			{
				return new SourceTargetData(false, false, SourceTargetData.SOURCE_CONTAINER, "org.eclipse.uml2.uml.Class",
						"ownedConnector", null, null, null, null, null, null);
			}
		}
		return null;
	}

	/**
	 * Indicates if the source and the target are in the same StructuredClassifier<br>
	 * This means that it is not possible to draw a Connector between two differents Classifier (e.g Collaboration to Class or Class to Class).
	 * 
	 * @param source The source node
	 * @param target The target node
	 * @return <code>true</code> if the two nodes are in the same classifier, <code>false</code> otherwise.
	 */
	private boolean areInSameClassifier(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		EObject eSourceContainerObject = Utils.getElement((GraphNode) source.eContainer());
		EObject eTargetContainerObject = Utils.getElement((GraphNode) target.eContainer());

		// find the container for the source
		if (sourceObject instanceof Port && eSourceContainerObject instanceof Property)
		{
			eSourceContainerObject = eSourceContainerObject.eContainer();
		}

		// find the container for the target
		if (targetObject instanceof Port && eTargetContainerObject instanceof Property)
		{
			eTargetContainerObject = eTargetContainerObject.eContainer();
		}

		// determines if it is the same classifier or not
		return eSourceContainerObject.equals(eTargetContainerObject);
	}

	/**
	 * A Connector between a Port and a Property is allowed only if the following conditions are respected.<br>
	 * In others words, it does not make sense to draw a Connector whether :<br>
	 * <ul>
	 * <li>the source element equals the target element (not reflexive link)</li>
	 * <li>the source container equals the target element (Connector from Port to Property)</li>
	 * <li>the target container equals the source element (Connector from Property to Port)</li>
	 * <li>the source container equals the target container (Connector from Port to Port belonging to the same Property)</li>
	 * </ul>
	 * 
	 * @param source The source node
	 * @param target The target node
	 * @return <code>true</code> if the Connector between source and target may be drawn, <code>false</code> otherwise.
	 */
	private boolean isValidConnector(GraphElement source, GraphElement target)
	{
		EObject sourceObject = Utils.getElement(source);
		EObject targetObject = Utils.getElement(target);

		// Does source element equals target element ?
		if (!sourceObject.equals(targetObject))
		{
			if (sourceObject instanceof Port && targetObject instanceof Port)
			{
				// Does the source container equals the target container ? => source Port and target Port are in the same Property
				return !source.eContainer().equals(target.eContainer());
			}

			if (sourceObject instanceof Property && targetObject instanceof Port)
			{
				// Does the target container equals the source element ? => Property contains the Port
				return !target.eContainer().equals(source);
			}

			if (targetObject instanceof Property && sourceObject instanceof Port)
			{
				// Does the source container equals the target element ? => Port is contained by the Property
				return !source.eContainer().equals(target);
			}
			
			return true;
		}
		return false;
	}

}