/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.modeler.uml.importer;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.uml2.uml.Model;
import org.topcased.java.reverse.Java2UMLConverter;
import org.topcased.modeler.tools.DiagramFileInitializer;

/**
 * This class creates a Class Diagram from a Java Project (Package or Java
 * Project)
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class Java2ClassDiagramConverter
{
    private static final String UML_EXTENSION = "uml";
    
    private static final String DIAGRAM_ID = "org.topcased.modeler.uml.classdiagram";

    /**
     * Constructor
     */
    public Java2ClassDiagramConverter()
    {
        // Do nothing
    }

    /**
     * Convert an Package or a Java project into a UML2 model
     * 
     * @param project the JavaProject
     * @param modelName The name of the created model
     * @param container the container of the generated files
     * @param monitor  the progress monitor
     * @throws JavaModelException an error occured during reverse engineering
     * @throws IOException an error occured during creating files
     */
    public void createDiagram(IJavaProject project, String modelName, IContainer container, IProgressMonitor monitor)
            throws JavaModelException, IOException
    {
        assert project != null;
        assert modelName != null;
        assert container != null && container.exists();
        assert monitor != null;

        monitor.beginTask("Reverse Engineering", 5);

        String modelFileName = modelName + "." + UML_EXTENSION;

        IFile modelFile = container.getFile(new Path(modelFileName));

        // Reverse Engineering
        Java2UMLConverter converter = new Java2UMLConverter();
        Model model = converter.convert(project);
        monitor.worked(1);

        // Save reversed UML model
        saveModelFile(model, modelFile);
        monitor.worked(1);

        DiagramFileInitializer initializer = new DiagramFileInitializer();
        initializer.createDiagram(model, DIAGRAM_ID, modelName, true, new SubProgressMonitor(monitor, 3));

        monitor.done();
    }

    private void saveModelFile(Model model, IFile modelFile) throws IOException
    {
        // Create a resource set
        ResourceSet resourceSet = new ResourceSetImpl();

        // Get the URI of the model file.
        URI fileURI = URI.createPlatformResourceURI(modelFile.getFullPath().toString());

        // Create a resource for this file.
        Resource emfResource = resourceSet.createResource(fileURI);

        // Add the initial model object to the contents.
        if (model != null)
        {
            emfResource.getContents().add(model);
        }

        // Save the contents of the resource to the file
        // system.
        Map options = new HashMap();
        emfResource.save(options);
    }
}
