/******************************************************************************************
 * Copyright (c) 2007 Communication & System (C-S).
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Sebastien Gabel (CS) 
 *    
 *******************************************************************************************/
package org.topcased.modeler.uml.internal.properties.sections.operation.behavior;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.OpaqueBehavior;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.utils.LabelHelper;

/**
 * This Composite contains the necessary widgets for editing a OpaqueBehavior in the UML editor
 * 
 * Creation 10 april 2007
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class OperationBehaviorComposite extends Composite
{
	/** The OpaqueBehavior that is currently edited */
	private OpaqueBehavior opaqueBehavior;

	/** The Operation that is currently edited */
	private Operation operation;

	private MixedEditDomain mixedEditDomain;

	/** The widgetFactory to use to create the widgets */
	private TabbedPropertySheetWidgetFactory widgetFactory;

	/** widgets */
	private CCombo languageCombo;

	private Text codeArea;

	private Button removeBtn;

	private Button addBtn;

	/**
	 * Enumeration of available languages.
	 */
	public enum Language 
	{
		C("C"), CPP("C++"), JAVA("Java"), ADA("Ada"), PYTHON("Python");

		private String name;

		/**
		 * Constructor
		 * 
		 * @param name_ The language name
		 */
		private Language(String name_)
		{
			name = name_;
		}

		/**
		 * Getter for the language name
		 */
		public String getName()
		{
			return name;
		}
	};

	/**
	 * The constructor
	 * 
	 * @param parent a widget which will be the parent of the new instance (cannot be null)
	 * @param style the style of widget to construct
	 * @param wFactory the widgetFactory to use to create the different widgets
	 */
	OperationBehaviorComposite(Composite parent, int style, TabbedPropertySheetWidgetFactory wFactory)
	{
		super(parent, style);

		widgetFactory = wFactory;
		setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		final GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		setLayout(layout);

		widgetFactory.adapt(this);
		createContents(this);
	}

	/**
	 * Creates the Composite composed of a set of widgets necessary to edit an Opaquebehavior
	 * 
	 * @param parent the parent Composite
	 */
	protected void createContents(Composite parent)
	{
		createComboComposite(parent);

		createEditionAreaComposite(parent);

		createButtonsComposite(parent);
	}

	/**
	 * Set the Operation and update the content of the table
	 * 
	 * @param operation the Operation model object
	 */
	public void setOperation(Operation operation)
	{
		this.operation = operation;

		setOpaqueBehavior();
	}

	/**
	 * Sets an OpaqueBehavior when switching from an operation to another one (and vice-versa)
	 */
	private void setOpaqueBehavior()
	{
		opaqueBehavior = getOpaqueBehavior();
		languageCombo.select(0);
		updateWidgets();
	}

	/**
	 * Allows to update fields and buttons.
	 */
	private void updateWidgets()
	{
		if (opaqueBehavior != null && opaqueBehavior.getLanguages().contains(languageCombo.getText()))
		{
			int index = opaqueBehavior.getLanguages().indexOf(languageCombo.getText());
			codeArea.setText(opaqueBehavior.getBodies().get(index));
			canRemove();
		}
		else
		{
			codeArea.setText("");
			canAdd();
		}
	}

    /**
     * Method called each time a feature of the current Opaque Behavior evolve.
     */
    public void updateModel()
    {
    	mixedEditDomain.getEMFEditingDomain().getCommandStack().execute(
                SetCommand.create(mixedEditDomain.getEMFEditingDomain(), opaqueBehavior, UMLPackage.eINSTANCE.getNamedElement_Name(), null));
    }
    
	/**
	 * Sets the MixedEditDomain
	 * 
	 * @param editDomain the MixedEditDomain
	 */
	public void setMixedEditDomain(MixedEditDomain editDomain)
	{
		mixedEditDomain = editDomain;
	}

	/**
	 * Creates the last composite containing the Add button and the Remove button.
	 * 
	 * @param parent The parent composite
	 */
	private void createButtonsComposite(Composite parent)
	{
		final GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = 0;
		layout.marginHeight = 0;

		final Composite btnsComposite = widgetFactory.createComposite(parent, SWT.NONE);
		btnsComposite.setLayout(layout);
		btnsComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		addBtn = widgetFactory.createButton(btnsComposite, "Add", SWT.PUSH);
		addBtn.setLayoutData(new GridData(75, SWT.DEFAULT));
		addBtn.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				addCodeAndLanguage();
			}
		});

		removeBtn = widgetFactory.createButton(btnsComposite, "Remove", SWT.PUSH);
		removeBtn.setLayoutData(new GridData(75, SWT.DEFAULT));
		removeBtn.setEnabled(false);
		removeBtn.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				removeCodeAndLanguage();
			}
		});
	}

	/**
	 * Creates the first line containing a Label and a Combo Box.
	 * 
	 * @param parent The parent composite
	 */
	private void createComboComposite(Composite parent)
	{
		final GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = 0;

		final Composite comboComposite = widgetFactory.createComposite(parent, SWT.NONE);
		comboComposite.setLayout(layout);
		comboComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		final Label comboLabel = widgetFactory.createLabel(comboComposite, "Language : ", SWT.NONE);
		comboLabel.setLayoutData(new GridData(100, SWT.DEFAULT));

		languageCombo = widgetFactory.createCCombo(comboComposite, SWT.NONE);
		languageCombo.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		languageCombo.setEditable(false);
		for (Language current : Language.values())
		{
			languageCombo.add(current.getName());
		}
		languageCombo.select(0);

		languageCombo.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				updateWidgets();
			}
		});
	}

	/**
	 * Enables to add a Language and the associated code if necessary.<br>
	 * First of all, the OpaqueBehavior is created if it does not exist. Then the language is added to the Language list. The code in the Body list.
	 */
	private void addCodeAndLanguage()
	{
		if (!opaqueBehaviorAlreadyExists())
		{
			createOpaqueBehavior();
		}

		// adds the existing code at the specific index
		opaqueBehavior.getBodies().add(codeArea.getText());
		// opaqueBehavior.getBodies().add(code.get());
		opaqueBehavior.getLanguages().add(languageCombo.getText());
		canRemove();
		updateModel();
	}

	/**
	 * Removes an existing code and language stored inside an OpaqueBehavior Object.
	 */
	private void removeCodeAndLanguage()
	{
		// replace the existing code at the specific index
		int index = opaqueBehavior.getLanguages().indexOf(languageCombo.getText());
		opaqueBehavior.getBodies().remove(index);
		opaqueBehavior.getLanguages().remove(languageCombo.getText());

		// fields are reseted
		codeArea.setText("");
		languageCombo.select(0);
		canAdd();

		if (opaqueBehavior.getBodies().isEmpty() && opaqueBehavior.getLanguages().isEmpty())
		{
			// object is removed
			removeOpaqueBehavior();
		}
		else
		{
			updateModel();
		}
	}

	/**
	 * Creates the code area where the user can enter its own code related to a given language.
	 * 
	 * @param parent The parent composite
	 */
	private void createEditionAreaComposite(Composite parent)
	{
		final Composite editComposite = widgetFactory.createComposite(parent, SWT.NONE);
		editComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		editComposite.setLayout(new GridLayout());

		widgetFactory.createLabel(editComposite, "Body : ", SWT.NONE);

		codeArea = widgetFactory.createText(editComposite, "", SWT.WRAP | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		codeArea.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		codeArea.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				hookListeners();
			}
		});
	}

	/**
	 * Gets a new child instance of type 'Opaque Behavior'.
	 * 
	 * @return a new child instance.
	 */
	private EObject getNewChild()
	{
		OpaqueBehavior newOpaqueBehavior = UMLFactory.eINSTANCE.createOpaqueBehavior();
		LabelHelper.INSTANCE.initName(mixedEditDomain, operation.eContainer(), newOpaqueBehavior);
		return newOpaqueBehavior;
	}

	/**
	 * Indicates if an OpaqueBehavior already exists for this operation.
	 * 
	 * @return <code>true</code> if it exists, <code>false</code> otherwise.
	 */
	private boolean opaqueBehaviorAlreadyExists()
	{
		if (operation != null)
		{
			for (Behavior behavior : operation.getMethods())
			{
				if (behavior instanceof OpaqueBehavior)
				{
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Gets the OpaqueBehavior object associated to the current Operation<br>
	 * If the current operation does not exist, the method returns <code>null</code>.
	 * 
	 * @return an OpaqueBehavior object execpt if the current operation has no Behavior at this time.
	 */
	protected OpaqueBehavior getOpaqueBehavior()
	{
		if (operation != null)
		{
			for (Behavior behavior : operation.getMethods())
			{
				if (behavior instanceof OpaqueBehavior)
				{
					return (OpaqueBehavior) behavior;
				}
			}
		}
		return null;
	}

	/**
	 * Creates a new OpaqueBehavior related to the current Operation Object
	 */
	private void createOpaqueBehavior()
	{
		// Gets the parent Classifier of the current Operation
		Class parentObject = (Class) operation.getOwner();
		opaqueBehavior = (OpaqueBehavior) getNewChild();

		// Sets the relation with the current Operation
		opaqueBehavior.setSpecification(operation);

		// Attaches the created OpaqueBehavior object to the current Classifier containing the Operation.
		parentObject.getOwnedBehaviors().add(opaqueBehavior);
	}

	/**
	 * Removes an existing OpaqueBehavior attached to an operation.
	 */
	private void removeOpaqueBehavior()
	{
		// Object is removed from the model
		Class parentObject = (Class) operation.getOwner();
		parentObject.getOwnedBehaviors().remove(opaqueBehavior);
		operation.getMethods().remove(opaqueBehavior);
	}

	/**
	 * Implements the text area listener aims to enter information about language.
	 */
	private void hookListeners()
	{
		// The OpaqueBehavior object must already exist
		if (opaqueBehaviorAlreadyExists())
		{
			// The current language must at least being edited in past
			if (opaqueBehavior.getLanguages().contains(languageCombo.getText()))
			{
				int index = opaqueBehavior.getLanguages().indexOf(languageCombo.getText());
				
				// The content of the text area must be different from the content store inside the object 
				if (!opaqueBehavior.getBodies().get(index).equals(codeArea.getText()))
				{
					opaqueBehavior.getBodies().set(index, codeArea.getText());
					updateModel();
				}
			}
		}
	}

	/**
	 * Handles the buttons Add and Remove in case where it is possible to add code and language.
	 */
	private void canAdd()
	{
		addBtn.setEnabled(true);
		removeBtn.setEnabled(false);
	}

	/**
	 * Handles the buttons Add and Remove in case where it is possible to remove code and language.
	 */
	private void canRemove()
	{
		addBtn.setEnabled(false);
		removeBtn.setEnabled(true);
	}
}
