/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEnd;
import org.eclipse.uml2.uml.MessageEvent;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.MessageSort;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.ReceiveOperationEvent;
import org.eclipse.uml2.uml.ReceiveSignalEvent;
import org.eclipse.uml2.uml.SendOperationEvent;
import org.eclipse.uml2.uml.SendSignalEvent;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.SignalEvent;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.SequenceDiagramEditPart;
import org.topcased.modeler.uml.sequencediagram.util.MessageArgument;
import org.topcased.modeler.utils.Utils;

/**
 * Utility class to manipulate UML EObjects for the sequence diagram.
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public final class SequenceUtils
{
    /**
     * The step where the message becomes rectilinear
     */
    public static final int ALIGNEMENT_MESSAGE_STEP = 20;

    /**
     * Contructor private as it is a utility class
     */
    private SequenceUtils()
    {
        // utility class : no default constructor
    }

    /**
     * Insert an OccurrenceSpecification to the lifeline at the correct
     * postition If there is another occurrenceSpecification corresponding a
     * message which is not drawn on the diagram, the new
     * occurrenceSpecification is inserted before it.
     * 
     * @param edge the edge of the new occurrenceSpecification
     * @param figure the figure of the edge (or node for lost or found)
     * @param newOccurrenceSpecification the new OccurrenceSpecification to
     *            insert
     * @param lifeline the lifeline
     * @param newPos the position of the occurrenceSpecification on the lifeline
     */
    public static void insertOccurrenceSpecification(GraphEdge edge, IFigure figure,
            MessageOccurrenceSpecification newOccurrenceSpecification, Lifeline lifeline, Point newPos)
    {
        EList covereds = lifeline.getCoveredBys();
        EList fragments = lifeline.getInteraction().getFragments();

        Iterator it = fragments.iterator();

        Point newP = new Point(newPos);

        GraphNode edgeContainer = (GraphNode) edge.getContainer();

        /*
         * search the index and position of the occurrenceSpecification just
         * before the new occurrenceSpecification
         */
        int beforePos = -1;
        int beforeIndex = -1;

        while (it.hasNext())
        {
            Object obj = it.next();
            if (obj instanceof MessageOccurrenceSpecification && covereds.contains(obj))
            {
                MessageOccurrenceSpecification occurrenceSpecification = (MessageOccurrenceSpecification) obj;
                Point p = getOccurrenceSpecificationPosition(figure, edgeContainer, occurrenceSpecification);
                if (p != null)
                {
                    int pos = p.y;
                    if (pos < newP.y && beforePos < pos)
                    {
                        beforeIndex = fragments.indexOf(occurrenceSpecification);
                        beforePos = pos;
                    }
                }
            }
        }

        /*
         * Add the new event just after the 'before' event If there is no
         * 'before' event, the place the new event to the first position
         * (-1+1==0)
         */

        fragments.add(beforeIndex + 1, newOccurrenceSpecification);

    }

    /**
     * Get the occurrenceSpecification position on the lifeline
     * 
     * @param figure the figure owning the anchor of the occurrenceSpecification
     * @param edgeContainer the graph node that contains every edge
     * @param occurrenceSpecification the occurrenceSpecification
     * 
     * @return the position on the lifeline
     */
    public static Point getOccurrenceSpecificationPosition(IFigure figure, GraphNode edgeContainer,
            MessageOccurrenceSpecification occurrenceSpecification)
    {
        // int numanchor = 0;
        // Message message = event.getSendMessage();
        // if (message == null)
        // {
        // message = event.getReceiveMessage();
        // numanchor = 1;
        // }

        Message message = occurrenceSpecification.getMessage();
        if (message == null)
        {
            return null;
        }
        int numanchor = occurrenceSpecification.equals(message.getSendEvent()) ? 0 : 1;

        GraphElement elem = Utils.getGraphElement(edgeContainer, message);

        if (elem == null)
        {
            // the corresponding elem is not drawn is the diagram
            return null;
        }

        Point pos;

        if (elem instanceof GraphEdge)
        {
            GraphConnector connector = (GraphConnector) ((GraphEdge) elem).getAnchor().get(numanchor);
            pos = new Point(connector.getPosition());
        }
        else
        {
            pos = new Point(((GraphNode) elem).getPosition());
        }

        pos.y += figure.getBounds().y;

        return pos;
    }

    /**
     * Return the relative position of the source of the message
     * 
     * @param msgEdit the edit part of the message
     * 
     * @return the relative source position
     */
    public static Point getSourcePosition(MessageEditPart msgEdit)
    {
        GraphEdge msgEdge = (GraphEdge) msgEdit.getModel();
        GraphConnector connector = (GraphConnector) msgEdge.getAnchor().get(0);
        Point pos = connector.getPosition();

        return pos.getCopy();
    }

    /**
     * Return the relative position of the target of the message
     * 
     * @param msgEdit the edit part of the message
     * 
     * @return the relative target position
     */
    public static Point getTargetPosition(MessageEditPart msgEdit)
    {
        GraphEdge msgedge = (GraphEdge) msgEdit.getModel();
        GraphConnector connector = (GraphConnector) msgedge.getAnchor().get(1);
        Point pos = connector.getPosition();

        return pos.getCopy();
    }

    /**
     * Return the absolute position of the source of the message
     * 
     * @param msgEdit the edit part of the message
     * 
     * @return the absolute source position
     */
    public static Point getSourceAbsolutePosition(MessageEditPart msgEdit)
    {
        Point pos = getSourcePosition(msgEdit);

        BaseEditPart srcEdit = (BaseEditPart) msgEdit.getSource();
        IFigure figure;
        if (srcEdit instanceof SequenceDiagramEditPart)
        {
            figure = (IFigure) srcEdit.getFigure().getChildren().get(0);
        }
        else
        {
            figure = srcEdit.getFigure();
        }
        pos.translate(figure.getBounds().getLocation());

        return pos;
    }

    /**
     * Return the absolute position of the target of the message
     * 
     * @param msgEdit the edit part of the message
     * 
     * @return the absolute target position
     */
    public static Point getTargetAbsolutePosition(MessageEditPart msgEdit)
    {
        Point pos = getTargetPosition(msgEdit);

        BaseEditPart targetEdit = (BaseEditPart) msgEdit.getTarget();
        IFigure figure;
        if (targetEdit instanceof SequenceDiagramEditPart)
        {
            figure = (IFigure) targetEdit.getFigure().getChildren().get(0);
        }
        else
        {
            figure = targetEdit.getFigure();
        }
        pos.translate(figure.getBounds().getLocation());

        return pos;
    }

    /**
     * Align a position of an anchor to the opposite anchor if they are on
     * "approximatively" on the same ordinate
     * 
     * @param oppositepos absolute position of the opposite anchor
     * @param pos position of the anchor the align
     * @return the difference of the aligned position and the initial position
     */
    public static int alignPosAnchor(Point pos, Point oppositepos)
    {
        if (Math.abs(oppositepos.y - pos.y) < ALIGNEMENT_MESSAGE_STEP)
        {
            int diff = oppositepos.y - pos.y;
            pos.y = oppositepos.y;
            return diff;
        }
        return 0;
    }

    /**
     * Set the start and finish OccurrenceSpecification of an
     * ExecutionOccurrence. The start ExecutionOccurrence will be the top
     * OccurrenceSpecification on the ExecutionOccurrence. The finish
     * ExecutionOccurrence will be the bottom OccurrenceSpecification on the
     * ExecutionOccurrence.
     * 
     * @param execEdit the edit part of the ExecutionSpecification
     */
    public static void updateStartAndFinishOccurrenceSpecification(BehaviorExecutionSpecificationEditPart execEdit)
    {
        GraphNode execNode = (GraphNode) execEdit.getModel();

        BehaviorExecutionSpecification exec = (BehaviorExecutionSpecification) Utils.getElement(execNode);

        MessageOccurrenceSpecification firstOccurrenceSpecification = null;
        int firstY = 0;
        MessageOccurrenceSpecification lastOccurrenceSpecification = null;
        int lastY = 0;

        Iterator itAnchors = execNode.getAnchorage().iterator();
        while (itAnchors.hasNext())
        {
            GraphConnector connector = (GraphConnector) itAnchors.next();
            MessageOccurrenceSpecification occurrenceSpecification = getMessageOccurrenceSpecification(connector);

            int y = connector.getPosition().y;

            if (firstOccurrenceSpecification == null)
            {
                firstOccurrenceSpecification = occurrenceSpecification;
                firstY = y;
                lastOccurrenceSpecification = occurrenceSpecification;
                lastY = y;
            }
            else
            {
                if (y < firstY)
                {
                    firstY = y;
                    firstOccurrenceSpecification = occurrenceSpecification;
                }
                else if (y > lastY)
                {
                    lastY = y;
                    lastOccurrenceSpecification = occurrenceSpecification;
                }
            }

        }

        exec.setStart(firstOccurrenceSpecification);
        exec.setFinish(lastOccurrenceSpecification);
    }

    /**
     * Return the connector corresponding to the occurrenceSpecification
     * 
     * @param occurrenceSpecification the occurrenceSpecification
     * @param edgeContainer the container of the edge (the diagram node)
     * 
     * @return the graph connector of the event
     */
    public static GraphConnector getGraphConnector(MessageOccurrenceSpecification occurrenceSpecification,
            GraphNode edgeContainer)
    {
        // int numanchor = 0;
        // Message message = event.getSendMessage();
        // if (message == null)
        // {
        // message = event.getReceiveMessage();
        // numanchor = 1;
        // }

        Message message = occurrenceSpecification.getMessage();
        if (message == null)
        {
            return null;
        }
        int numanchor = occurrenceSpecification.equals(message.getSendEvent()) ? 0 : 1;

        GraphElement elem = Utils.getGraphElement(edgeContainer, message);

        if (elem == null)
        {
            // the corresponding elem is not drawn is the diagram
            return null;
        }

        return (GraphConnector) ((GraphEdge) elem).getAnchor().get(numanchor);
    }

    /**
     * Return the occurrenceSpecification corresponding to the connector
     * 
     * @param connector the connector
     * 
     * @return the occurrenceSpecification of the graph connector
     */
    public static MessageOccurrenceSpecification getMessageOccurrenceSpecification(GraphConnector connector)
    {
        GraphEdge edge = (GraphEdge) connector.getGraphEdge().get(0);
        int index = edge.getAnchor().indexOf(connector);

        Message message = (Message) Utils.getElement(edge);
        MessageOccurrenceSpecification occurrenceSpecification;

        if (index == 0)
        {
            occurrenceSpecification = (MessageOccurrenceSpecification) message.getSendEvent();
        }
        else
        {
            occurrenceSpecification = (MessageOccurrenceSpecification) message.getReceiveEvent();
        }
        return occurrenceSpecification;
    }

    /**
     * Create a new graph connector
     * 
     * @param p the position of the connector
     * @param element the receiving graph element
     * @param graphEdge the owning graph edge
     * 
     * @return the new graph connector
     */
    public static GraphConnector createGraphConnector(Point p, GraphElement element, GraphEdge graphEdge)
    {
        GraphConnector connector = DiagramInterchangeFactory.eINSTANCE.createGraphConnector();
        connector.setPosition(p);
        element.getAnchorage().add(connector);
        connector.getGraphEdge().add(graphEdge);

        return connector;
    }

    /**
     * Remove a graph connector from the model
     * 
     * @param connector the connector to remove
     * @param element the element wich receive the connector
     * @param graphedge the owning graph edge
     */
    public static void removeGraphConnector(GraphConnector connector, GraphElement element, GraphEdge graphedge)
    {
        EList anchors = element.getAnchorage();
        anchors.remove(connector);
        connector.getGraphEdge().remove(graphedge);
    }

    /**
     * Update an occurrenceSpecification
     * 
     * @param edge the egde of the occurrenceSpecification
     * @param changedOccurrenceSpecification the changed occurrenceSpecification
     * @param newPos the new position of the occurrenceSpecification
     * @param oldNodeEdit the old receiving node
     * @param newNodeEdit the new receiving node
     */
    public static void updateOccurrenceSpecification(GraphEdge edge,
            MessageOccurrenceSpecification changedOccurrenceSpecification, Point newPos, BaseEditPart oldNodeEdit,
            BaseEditPart newNodeEdit)
    {
        GraphNode newNode = (GraphNode) newNodeEdit.getModel();
        GraphNode oldNode = (GraphNode) oldNodeEdit.getModel();

        Message message = (Message) Utils.getElement(edge);

        Lifeline newLifeline;
        Lifeline oldLifeline;

        if (isCall(message) || isReplyMessage(message))
        {
            newLifeline = (Lifeline) Utils.getElement(newNode.getContainer());
            oldLifeline = (Lifeline) Utils.getElement(oldNode.getContainer());
        }
        else
        {
            newLifeline = (Lifeline) Utils.getElement(newNode);
            oldLifeline = (Lifeline) Utils.getElement(oldNode);
        }
        // remove it form the list
        oldLifeline.getInteraction().getFragments().remove(changedOccurrenceSpecification);
        // and re-insert it at the right place
        insertOccurrenceSpecification(edge, newNodeEdit.getFigure(), changedOccurrenceSpecification, newLifeline,
                newPos);

        changedOccurrenceSpecification.getCovereds().remove(oldLifeline);
        changedOccurrenceSpecification.getCovereds().add(newLifeline);

        if (isCall(message) || isReplyMessage(message))
        {
            updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) newNodeEdit);
            if (oldNode != newNode)
            {
                updateStartAndFinishOccurrenceSpecification((BehaviorExecutionSpecificationEditPart) oldNodeEdit);
            }
        }

    }

    /**
     * Get the text representing the typical call of an the operation This
     * function will return a string with the operation name and a sequence of
     * '-' for each parameter. For example, a function with 3 arguments :
     * function(-, -, -)
     * 
     * @param operation the operation
     * @return the call text
     */
    public static String getOperationCallText(Operation operation)
    {
        StringBuffer buffer = new StringBuffer(operation.getLabel());

        buffer.append('(');

        int nbparam = operation.getOwnedParameters().size();
        if (nbparam != 0)
        {
            for (int j = 0; j < nbparam - 1; j++)
            {
                buffer.append("-, ");
            }
            buffer.append('-');
        }
        buffer.append(')');

        return buffer.toString();
    }

    /**
     * Return true if the message is a Signal Message
     * 
     * @param message the message to check
     * @return true if the message is a Signal Message
     */
    public static boolean isSignal(Message message)
    {
        return message.getMessageSort().getValue() == MessageSort.ASYNCH_SIGNAL;
    }

    /**
     * Return true if the message is a Call Message
     * 
     * @param message the message to check
     * @return true if the message is a Call Signal
     */
    public static boolean isCall(Message message)
    {
        int sort = message.getMessageSort().getValue();
        return sort == MessageSort.ASYNCH_CALL || sort == MessageSort.SYNCH_CALL;
    }

    /**
     * Return true if the message is a SynchronousCall Message
     * 
     * @param message the message to check
     * @return true if the message is a SynchronousCall Message
     */
    public static boolean isSynchCallMessage(Message message)
    {
        return message.getMessageSort().getValue() == MessageSort.SYNCH_CALL;
    }

    /**
     * Return true if the message is a reply message
     * 
     * @param message the Message to check
     * @return true if the message is a reply message
     */
    public static boolean isReplyMessage(Message message)
    {
        return MessageSort.REPLY == message.getMessageSort().getValue();
    }

    /**
     * Return true if the message is a Create Message
     * 
     * @param message the Message to check
     * @return true if the message is a Create Message
     */
    public static boolean isCreateMessage(Message message)
    {
        return MessageSort.CREATE_MESSAGE == message.getMessageSort().getValue();
    }

    /**
     * Return true if the message is a Delete Message
     * 
     * @param message the Message to check
     * @return true if the message is a Delete Message
     */
    public static boolean isDeleteMessage(Message message)
    {
        return MessageSort.DELETE_MESSAGE == message.getMessageSort().getValue();
    }

    /**
     * Get the collection of arguments of the signature of a message. The
     * signature must be an Operation or a Signal. The contents of the returned
     * Collection are of type MessageArgument
     * 
     * @param message the Message
     * @return a Collection of MessageArgument
     */
    public static Collection getMessageArguments(Message message)
    {
        NamedElement signature = message.getSignature();
        ArrayList messageArguments = new ArrayList();

        if (signature != null)
        {
            // initialize the two lists that should be synchronized
            ValueSpecification[] arguments = (ValueSpecification[]) message.getArguments().toArray();
            TypedElement[] attributes;

            if (signature instanceof Operation)
            {
                attributes = (TypedElement[]) ((Operation) signature).getOwnedParameters().toArray();
            }
            else
            {
                attributes = (TypedElement[]) ((Signal) signature).getOwnedAttributes().toArray();
            }

            // check if the two lists have the same size
            if (arguments.length == attributes.length)
            {
                for (int i = 0; i < arguments.length; i++)
                {
                    messageArguments.add(new MessageArgument(arguments[i], attributes[i]));
                }
            }
        }

        return messageArguments;
    }

    /**
     * Complete the list of MessageArgument by the lacking parameters values,
     * and remove the unexisting ones. This function is used when the parameters
     * of the operation has change, so we need to update the arguments of a
     * Message.
     * 
     * @param newArguments the new collection of MessageArgument
     * @param oldArguments the old collection of MessageArgument
     * @deprecated The update of the Arguments should be done step by step
     */
    public static void updateArguments(Collection newArguments, Collection oldArguments)
    {
        Iterator itArguments = newArguments.iterator();
        while (itArguments.hasNext())
        {
            MessageArgument arg = (MessageArgument) itArguments.next();
            Iterator itOldArguments = oldArguments.iterator();
            while (itOldArguments.hasNext())
            {
                ValueSpecification oldValue = (ValueSpecification) itOldArguments.next();
                if (oldValue.getName().equals(arg.getName()))
                {
                    if (oldValue.getType() == null)
                    {
                        arg.setStringValue(null);
                    }
                    else
                    {
                        arg.setStringValue(getStringValue(oldValue));
                    }
                    break;
                }
            }
        }
    }

    /**
     * This method is used to set the signature of a MessageEvent
     * 
     * @param messEvent the MessageEvent
     * @param signature the newSignature. This may be a null Signature.
     */
    public static void setMessageEventSignature(MessageEvent messEvent, NamedElement signature)
    {
        if (messEvent instanceof CallEvent)
        {
            ((CallEvent) messEvent).setOperation((Operation) signature);
        }
        else if (messEvent instanceof ReceiveOperationEvent)
        {
            ((ReceiveOperationEvent) messEvent).setOperation((Operation) signature);
        }
        else if (messEvent instanceof SendOperationEvent)
        {
            ((SendOperationEvent) messEvent).setOperation((Operation) signature);
        }
        else if (messEvent instanceof SignalEvent)
        {
            ((SignalEvent) messEvent).setSignal((Signal) signature);
        }
        else if (messEvent instanceof ReceiveSignalEvent)
        {
            ((ReceiveSignalEvent) messEvent).setSignal((Signal) signature);
        }
        else if (messEvent instanceof SendSignalEvent)
        {
            ((SendSignalEvent) messEvent).setSignal((Signal) signature);
        }
    }

    /**
     * Get the MessageEvent associated with a Message. The MessageEvent will be
     * associated with the Signature of the Message. So we first search the
     * SendEvent and then the ReceiveEvent in the case of the SendEvent does not
     * exist.
     * 
     * @param msg the Message
     * @return the MessageEvent that will be associated with the Signature of
     *         the Message
     */
    public static MessageEvent getMessageEvent(Message msg)
    {
        MessageEvent msgEvent = null;

        MessageEnd sendEvent = msg.getSendEvent();

        if (sendEvent instanceof MessageOccurrenceSpecification)
        {
            Event event = ((MessageOccurrenceSpecification) sendEvent).getEvent();

            if (event instanceof MessageEvent)
            {
                msgEvent = (MessageEvent) event;
            }
        }

        if (msgEvent == null)
        {
            MessageEnd receiveEvent = msg.getReceiveEvent();

            if (receiveEvent instanceof MessageOccurrenceSpecification)
            {
                Event event = ((MessageOccurrenceSpecification) receiveEvent).getEvent();

                if (event instanceof MessageEvent)
                {
                    msgEvent = (MessageEvent) event;
                }
            }
        }

        return msgEvent;
    }

    /**
     * Get the notation of the message
     * 
     * @param message the message
     * @return the notation of the message
     */
    public static String getMessageNotation(Message message)
    {
        StringBuffer buffer = new StringBuffer();

        NamedElement signature = message.getSignature();

        if (signature != null)
        {
            buffer.append(signature.getName());

            buffer.append('(');

            Iterator itArguments = message.getArguments().iterator();
            while (itArguments.hasNext())
            {
                ValueSpecification valueSpec = (ValueSpecification) itArguments.next();

                boolean isString = valueSpec.getType() instanceof PrimitiveType
                        && "String".equals(valueSpec.getType().getName());

                if (isString)
                {
                    buffer.append('"');
                }

                buffer.append(getStringValue(valueSpec));

                if (isString)
                {
                    buffer.append('"');
                }

                if (itArguments.hasNext())
                {
                    buffer.append(", ");
                }
            }

            buffer.append(')');
        }
        // In the case that no Signature is associated with the Message, the
        // name property of the Message is displayed.
        else if (message.getName() != null)
        {
            buffer.append(message.getName());
        }

        return buffer.toString();
    }

    /**
     * Return the string representation of the ValueSpecification
     * 
     * @param valueSpec the ValueSpecification
     * @return the string representation of the ValueSpecification
     */
    public static String getStringValue(ValueSpecification valueSpec)
    {
        if (valueSpec instanceof LiteralBoolean)
        {
            return Boolean.toString(((LiteralBoolean) valueSpec).booleanValue());
        }
        else if (valueSpec instanceof LiteralInteger)
        {
            return Integer.toString(((LiteralInteger) valueSpec).integerValue());
        }
        else if (valueSpec instanceof LiteralString)
        {
            return ((LiteralString) valueSpec).stringValue();
        }
        // else if (valueSpec instanceof Expression)
        // {
        // return ((Expression) valueSpec).getBody();
        // }
        else if (valueSpec instanceof InstanceValue)
        {
            if (((InstanceValue) valueSpec).getInstance() != null)
            {
                return ((InstanceValue) valueSpec).getInstance().getName();
            }
            return "";
        }
        else if (valueSpec instanceof LiteralNull)
        {
            return "Null";
        }
        else if (valueSpec instanceof LiteralUnlimitedNatural)
        {
            return String.valueOf(((LiteralUnlimitedNatural) valueSpec).integerValue());
        }

        return "-";
    }
}
