/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.anchors;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.uml.sequencediagram.figures.BehaviorExecutionSpecificationFigure;

/**
 * The anchor of the Execution Occurrence
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class BehaviorExecutionSpecificationAnchor extends AbstractConnectionAnchor
{
    private Point refPoint;

    /**
     * Constructs an anchor with the given <i>owner</i> figure.
     * 
     * @param owner The owner figure
     * @param cursor the position of the cursor
     * @param oppositePos the position of the opposite anchor
     */
    public BehaviorExecutionSpecificationAnchor(IFigure owner, Point cursor, Point oppositePos)
    {
        super(owner);
        computeRefPointFromCursor(cursor, oppositePos);
    }

    /**
     * Contructor
     * 
     * @param owner the owner figure
     * @param pos the position of the anchor
     */
    public BehaviorExecutionSpecificationAnchor(IFigure owner, Point pos)
    {
        super(owner);
        // fix bug when undoing a delete from model (pos can be null);
        refPoint = (pos != null) ? pos.getCopy() : new Point();
    }

    /**
     * Compute the reference point
     * 
     * @param pCursor the position of the cursor
     * @param oppositePos the position of the opposite anchor
     */
    protected void computeRefPointFromCursor(Point pCursor, Point oppositePos)
    {
        BehaviorExecutionSpecificationFigure fig = (BehaviorExecutionSpecificationFigure) getOwner();

        refPoint = pCursor.getCopy();

        refPoint.x = fig.getBounds().getCenter().x;

        if (oppositePos != null)
        {
            SequenceUtils.alignPosAnchor(refPoint, oppositePos);
        }

        if (refPoint.y > fig.getBounds().getBottom().y)
        {
            refPoint.y = fig.getBounds().getBottom().y;
        }
        if (refPoint.y < fig.getBounds().getTop().y)
        {
            refPoint.y = fig.getBounds().getTop().y;
        }

        refPoint.translate(fig.getLocation().getNegated());
    }

    /**
     * Return the reference point.
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
     */
    public Point getLocation(Point reference)
    {
        return getReferencePoint();
    }

    /**
     * Returns the bounds of this LifeLineAnchor's owner. Subclasses can override this method to adjust the box the
     * anchor can be placed on. For instance, the owner figure may have a drop shadow that should not be included in the
     * box.
     * 
     * @return The bounds of this LifeLineAnchor's owner
     */
    protected Rectangle getBox()
    {
        return getOwner().getBounds();
    }

    /**
     * The reference point is the abolute position of the anchor
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getReferencePoint()
     */
    public Point getReferencePoint()
    {
        Point p = refPoint.getCopy();
        p.x = getOwner().getBounds().width / 2;
        p.translate(((BehaviorExecutionSpecificationFigure) getOwner()).getLocation());
        getOwner().translateToAbsolute(p);
        return p;
    }

    /**
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj)
    {
        if (obj instanceof LifelineAnchor)
        {
            LifelineAnchor other = (LifelineAnchor) obj;
            return other.getOwner() == getOwner() && other.getBox().equals(getBox());
        }
        return false;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
        if (getOwner() != null)
        {
            return getOwner().hashCode();
        }
        else
        {
            return super.hashCode();
        }
    }

}
