/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.anchors;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.topcased.modeler.uml.sequencediagram.figures.SequenceDiagramFigure;

/**
 * The anchor of the Sequence Diagram
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 */
public class SequenceDiagramAnchor extends AbstractConnectionAnchor
{
    private Point refPoint;

    /**
     * Constructs a SequenceDiagramAnchor with the given <i>owner</i> figure.
     * 
     * @param owner The owner figure
     * @param cursor the position of the cursor
     * @param oppositePos the position of the opposite anchor
     */
    public SequenceDiagramAnchor(IFigure owner, Point cursor, Point oppositePos)
    {
        super(owner);
        computeRefPointFromCursor(cursor, oppositePos);
    }

    /**
     * Contructor
     * 
     * @param owner the owner figure
     * @param pos the position of the anchor
     */
    public SequenceDiagramAnchor(IFigure owner, Point pos)
    {
        super(owner);
        refPoint = pos.getCopy();
    }

    /**
     * Compute the reference point
     * 
     * @param pCursor the position of the cursor
     * @param oppositePos the position of the opposite anchor
     */
    protected void computeRefPointFromCursor(Point pCursor, Point oppositePos)
    {
        refPoint = pCursor.getCopy();

        if (oppositePos != null)
        {
            refPoint.y = oppositePos.y;
        }

        refPoint.translate(getSequenceDiagramFigure().getLocation().getNegated());
    }

    /**
     * Return the figure of the sequence diagram
     * 
     * @return the figure of the sequence diagram
     */
    protected SequenceDiagramFigure getSequenceDiagramFigure()
    {
        IFigure owner = getOwner();
        if (!(owner instanceof SequenceDiagramFigure))
        {
            owner = (IFigure) owner.getChildren().get(0);
        }
        return (SequenceDiagramFigure) owner;
    }

    /**
     * The location is the reference point
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
     */
    public Point getLocation(Point reference)
    {
        return getReferencePoint();
    }

    /**
     * Returns the bounds of this LifelineAnchor's owner. Subclasses can override this method to adjust the box the
     * anchor can be placed on. For instance, the owner figure may have a drop shadow that should not be included in the
     * box.
     * 
     * @return The bounds of this LifelineAnchor's owner
     */
    protected Rectangle getBox()
    {
        return getOwner().getBounds();
    }

    /**
     * The reference point is the abolute position of the anchor
     * 
     * @see org.eclipse.draw2d.ConnectionAnchor#getReferencePoint()
     */
    public Point getReferencePoint()
    {
        Point p = refPoint.getCopy();
        p.translate(getSequenceDiagramFigure().getLocation());
        getSequenceDiagramFigure().translateToAbsolute(p);
        return p;
    }

    /**
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj)
    {
        if (obj instanceof SequenceDiagramAnchor)
        {
            SequenceDiagramAnchor other = (SequenceDiagramAnchor) obj;
            return other.getOwner() == getOwner();
        }
        return false;
    }

    /**
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
        if (getOwner() != null)
        {
            return getOwner().hashCode();
        }
        else
        {
            return super.hashCode();
        }
    }

}
