/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands;

import java.util.Iterator;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.uml2.uml.CombinedFragment;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.uml.sequencediagram.edit.CombinedFragmentEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.LifelineEditPart;

/**
 * Command to change the bounds of the combined fragment and update the list of the lifelines that are covered by this
 * CombinedFragment
 * 
 * Creation 8 juin 06
 * 
 * @author jlescot
 */
public class CombinedFragmentChangeBoundsCommand extends ChangeBoundsCommand
{
    /** The list that will contains the new Lifelines covered */
    private EList newList;

    /**
     * The list that contains the Lifelines covered before the combinedFragment bounds were changed
     */
    private EList oldList;

    /** The CombinedFrament editPart */
    private EditPart childEditPart;

    /** The Diagram editPart */
    private EditPart parentEditPart;

    /** The new constraint of the CombinedFragment */
    private Rectangle constraint;

    /**
     * Constructor
     * 
     * @param node the node to resize
     * @param constraint the new rectangle that contain the figure
     * @param childEditPart the EditPart associated with the CombinedFragment
     * @param parentEditPart the container EditPart of the CombinedFragment
     */
    public CombinedFragmentChangeBoundsCommand(GraphNode node, Rectangle constraint, EditPart childEditPart, EditPart parentEditPart)
    {
        super(node, constraint);

        this.childEditPart = childEditPart;
        this.parentEditPart = parentEditPart;

        this.constraint = constraint;
    }

    /**
     * Store the old List of Lifelines and retrieve the new list
     * 
     * @see org.topcased.modeler.commands.ChangeBoundsCommand#execute()
     */
    public void execute()
    {
        // initialize the lists
        oldList = new BasicEList(((CombinedFragment) ((CombinedFragmentEditPart) childEditPart).getEObject()).getCovereds());
        newList = new BasicEList();

        // Translate the constraint of the CombinedFragment with the offset of
        // its parent (margins of the diagram)
        Rectangle zone = ((GraphicalEditPart) ((CombinedFragmentEditPart) childEditPart).getParent()).getContentPane().getClientArea();
        constraint.translate(zone.x, zone.y);

        // then retrieve all the new Lifelines that intersect with the new
        // bounds of the combinedFragment
        Iterator iterEditParts = parentEditPart.getChildren().iterator();
        while (iterEditParts.hasNext())
        {
            EditPart editPart = (EditPart) iterEditParts.next();
            // We need to convert constraint as absolute coordinates
            if (editPart instanceof LifelineEditPart && ((LifelineEditPart) editPart).getFigure().intersects(constraint))
            {
                newList.add(((LifelineEditPart) editPart).getEObject());
            }
        }

        super.execute();
    }

    /**
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        super.redo();

        ((CombinedFragment) ((CombinedFragmentEditPart) childEditPart).getEObject()).getCovereds().clear();
        ((CombinedFragment) ((CombinedFragmentEditPart) childEditPart).getEObject()).getCovereds().addAll(newList);
    }

    /**
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        super.undo();

        ((CombinedFragment) ((CombinedFragmentEditPart) childEditPart).getEObject()).getCovereds().clear();
        ((CombinedFragment) ((CombinedFragmentEditPart) childEditPart).getEObject()).getCovereds().addAll(oldList);
    }
}
