/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.DestructionEvent;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * Command to create the Stop
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class DestructionEventCreationCommand extends CreateGraphNodeCommand
{
    private GraphNode lifelineNode;

    private OccurrenceSpecification occurrenceSpecification;

    private Event oldEvent;

    /**
     * Contructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObject the node to create
     * @param newParent the parentNode node
     * @param newContainerParent the model object that should contain the new childNode object
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param featuresList the avalaible model element feature of the parentNode node where the model element of the
     *        childNode node can be added
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code>
     *        if objects already exist in the mode.
     */
    public DestructionEventCreationCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, List featuresList,
            boolean needModelUpdate)
    {
        super(domain, newObject, newParent, newContainerParent, location, dimension, PositionConstants.NONE, featuresList, needModelUpdate);

        this.lifelineNode = newParent;
    }

    /**
     * Retrieve the OccurrenceSpecification that will be associated with the DestructionEvent and store the old Event
     * associated with it
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#execute()
     */
    public void execute()
    {
        Lifeline lifeline = (Lifeline) Utils.getElement(lifelineNode);
        occurrenceSpecification = getLastOccurrenceSpecification(lifeline);
        oldEvent = occurrenceSpecification.getEvent();

        super.execute();
    }

    /**
     * Set the DestructionEvent reference to the OccurrenceSpecification
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        super.redoModel();

        LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());

        occurrenceSpecification.setEvent((DestructionEvent) getChildEObject());
    }

    /**
     * Remove the DestructionEvent reference and set the old Event reference
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        super.undoModel();

        occurrenceSpecification.setEvent(oldEvent);
    }

    /**
     * The occurenceSpecification should be retrieved from the ORDERED list of fragments of the Interaction
     * 
     * @param lifeline the Lifeline on which the DestructionEvent is about to be added
     * @return the last OccurrenceSpecification covered by the Lifeline that should be associated with the
     *         DestructionEvent
     */
    private OccurrenceSpecification getLastOccurrenceSpecification(Lifeline lifeline)
    {
        // 
        Iterator itFragments = lifeline.getInteraction().getFragments().iterator();
        while (itFragments.hasNext())
        {
            InteractionFragment fragment = (InteractionFragment) itFragments.next();
            if (fragment instanceof MessageOccurrenceSpecification && lifeline.equals(((MessageOccurrenceSpecification) fragment).getCovered(null)))
            {
                occurrenceSpecification = (OccurrenceSpecification) fragment;
            }
        }
        return occurrenceSpecification;
    }
}
