/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Property;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.utils.LabelHelper;

/**
 * Command to create a Lifeline
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class LifelineCreationCommand extends CreateGraphNodeCommand
{

    private ConnectorEnd connectorend;

    private Property property;

    private boolean createproperty;

    private boolean createconnectorend;

    private Classifier classifier;

    /**
     * Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param newObject the node to create
     * @param newParent the parentNode node
     * @param newContainerParent the model object that should contain the new childNode object
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param attach the newAttachment of the created node
     * @param featuresList the avalaible model element feature of the parentNode node where the model element of the
     *        childNode node can be added
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code>
     *        if objects already exist in the mode.
     */
    public LifelineCreationCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dimension, int attach, List featuresList,
            boolean needModelUpdate)
    {
        super(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
    }

    /**
     * Create the Lifeline
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), getChildEObject());

        Interaction interaction = (Interaction) getParentEObject();

        ((Lifeline) getChildEObject()).setInteraction(interaction);

        property = interaction.getOwnedAttribute(null, classifier);
        createproperty = property == null;

        // if a such property doesn't exist, create it
        if (createproperty)
        {
            property = interaction.createOwnedAttribute(null, classifier);
            LabelHelper.INSTANCE.initName(getEditDomain(), interaction, property);
        }

        // search a connectorend with the role set to the found property
        connectorend = getConnectorEnd(interaction, property);

        createconnectorend = connectorend == null;

        // if a such property doesn't exist, create it
        if (createconnectorend)
        {
            connectorend = ((Connector) interaction.getOwnedConnectors().get(0)).createEnd();
            connectorend.setRole(property);
            LabelHelper.INSTANCE.initName(getEditDomain(), (EObject) interaction.getOwnedConnectors().get(0), connectorend);
        }

        Lifeline lifeline = (Lifeline) getChildEObject();
        lifeline.setRepresents(property);
    }

    /**
     * Return the property own by the collaboration which is typed by the classifier. If the property is not found,
     * create it.
     * 
     * @param collaboration the owning collaboration
     * @param c the wanted classifer
     * 
     * @return the searched property
     * @deprecated use the getOwnedAttribute() method of the StructuredClassifier class defined in UML
     */
    protected Property getProperty(Collaboration collaboration, Classifier c)
    {
        Property prop = null;
        Iterator itattributes = collaboration.getOwnedAttributes().iterator();
        while (prop == null && itattributes.hasNext())
        {
            Object att = itattributes.next();
            if (att instanceof Property)
            {
                Property p = (Property) att;
                if (p.getType().getName().equals(c.getName()))
                {
                    prop = p;
                }
            }
        }
        return prop;
    }

    /**
     * Return the connector end which has the correct property as role, owned by a connector int the interaction
     * 
     * @param interaction the owning interaction
     * @param prop the property
     * 
     * @return the ConnectorEnd
     */
    protected ConnectorEnd getConnectorEnd(Interaction interaction, Property prop)
    {
        EList connectors = interaction.getOwnedConnectors();
        Connector connector;
        if (connectors.isEmpty())
        {
            connector = interaction.createOwnedConnector(null);
            LabelHelper.INSTANCE.initName(getEditDomain(), interaction, connector);
            return null;
        }

        Iterator itconnectors = connectors.iterator();
        ConnectorEnd end = null;
        while (end == null && itconnectors.hasNext())
        {
            Iterator itends = ((Connector) itconnectors.next()).getEnds().iterator();
            while (end == null && itends.hasNext())
            {
                ConnectorEnd e = (ConnectorEnd) itends.next();
                if (e.getRole() == prop)
                {
                    end = e;
                }
            }
        }
        return end;
    }

    /**
     * Remove the Lifeline
     * 
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        Lifeline lifeline = (Lifeline) getChildEObject();
        lifeline.setRepresents(null);

        if (createconnectorend)
        {
            connectorend.setRole(null);
            ((Connector) connectorend.getOwner()).getEnds().remove(connectorend);
        }

        if (createproperty)
        {
            ((Classifier) property.getOwner()).getAttributes().remove(property);
        }

    }

}
