/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.creation;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.GraphNodeEditPart;
import org.topcased.modeler.editor.GraphElementCreationFactory;
import org.topcased.modeler.editor.ModelerGraphicalViewer;
import org.topcased.modeler.uml.sequencediagram.commands.MessageEdgeCreationCommand;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;

/**
 * Command to create a Message and an ExecutionOccurrence
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class MessageAndExecutionCreationCommand extends Command
{
    private static final int OFFSET = 10;

    /**
     * The Command that is used to create the Message and its OccurenceSpecifications associated
     */
    private MessageEdgeCreationCommand messageCommand;

    /**
     * The Command that is used to create the ExecutionSpecification of the target Lifeline
     */
    private Command execCommand;

    /** The viewer */
    private final EditPartViewer editViewer;

    private final Point location;

    private GraphNode execNode;

    /**
     * Constructor
     * 
     * @param viewer the viewer
     * @param loc the location of the cursor
     * @param messageCmd the command of creation of the message
     */
    public MessageAndExecutionCreationCommand(EditPartViewer viewer, Point loc, MessageEdgeCreationCommand messageCmd)
    {
        this.editViewer = viewer;
        this.location = loc;
        location.y -= OFFSET;

        this.messageCommand = messageCmd;

        CreateRequest createRequest = new CreateRequest();
        createRequest.setLocation(location);

        GraphElementCreationFactory factory = new GraphElementCreationFactory(((ModelerGraphicalViewer) viewer).getModelerEditor().getActiveConfiguration().getCreationUtils(),
                UMLPackage.eINSTANCE.getBehaviorExecutionSpecification());
        createRequest.setFactory(factory);

        GraphNodeEditPart lifeEdit = (GraphNodeEditPart) messageCmd.getTargetEditPart();
        if (lifeEdit instanceof BehaviorExecutionSpecificationEditPart)
        {
            lifeEdit = (GraphNodeEditPart) lifeEdit.getParent();
        }
        execCommand = lifeEdit.getCommand(createRequest);

        execNode = (GraphNode) createRequest.getNewObject();
        messageCommand.setTarget(execNode);
    }

    /**
     * Set up the message command to target on the created Execution Occurrence
     */
    protected void setupMessageCommand()
    {
        GraphNodeEditPart targetEdit = (GraphNodeEditPart) editViewer.getEditPartRegistry().get(execNode);
        messageCommand.setTargetEditPart(targetEdit);

        ((GraphicalEditPart) targetEdit.getParent()).getFigure().validate();
        targetEdit.getFigure().invalidateTree();

        // Dimension oldDim = targetEdit.getFigure().getSize();
        //
        // Point oppositePos = messageCommand.getSourcePos();
        //
        // Point pos = location.getCopy();
        // targetEdit.getFigure().translateToRelative(pos);
        //
        // SequenceUtils.alignPosAnchor(pos, oppositePos);
        // pos = new Point(ExecutionOccurrenceEditPart.DEFAULT_WIDTH / 2,
        // OFFSET);
        //
        // messageCommand.setTargetPos(pos);
    }

    /**
     * Only if the execution can be created
     * 
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return execCommand != null;
    }

    /**
     * Execute the command to create the ExecutionSpecification and the Message
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        execCommand.execute();
        setupMessageCommand();
        messageCommand.execute();
    }

    /**
     * Redo the command to create the ExecutionSpecification and the Message
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        execCommand.redo();
        setupMessageCommand();
        messageCommand.redo();
    }

    /**
     * Undo the both command
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        messageCommand.undo();
        execCommand.undo();
    }

}
