/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.reconnect;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.edit.GraphEdgeEditPart;

/**
 * Command that moves a graph edge
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class GraphEdgeMoveCommand extends Command
{

    private final GraphEdgeEditPart graphEdgeEdit;

    private final Point newLocation;

    private final Point initialLocation;

    private Point oldSourcePosition;

    private Point newSourcePosition;

    private Point oldTargetPosition;

    private Point newTargetPosition;

    private GraphConnector source;

    private GraphConnector target;

    /**
     * Constructor
     * 
     * @param edgeEdit the edit part of the graph edge to move
     * @param initialPos the initial position of the connection
     * @param newPos the new position of the connection
     */
    public GraphEdgeMoveCommand(GraphEdgeEditPart edgeEdit, Point initialPos, Point newPos)
    {
        this.graphEdgeEdit = edgeEdit;
        this.initialLocation = initialPos;
        this.newLocation = newPos;
    }

    /**
     * Return the edit part of the graph edge
     * 
     * @return the edit part of the graph edge
     */
    public GraphEdgeEditPart getGraphEdgeEdit()
    {
        return graphEdgeEdit;
    }

    /**
     * Return the new position of the source
     * 
     * @return the new position of the source
     */
    public Point getNewSourcePosition()
    {
        return newSourcePosition;
    }

    /**
     * Return the new position of the target
     * 
     * @return the new position of the target
     */
    public Point getNewTargetPosition()
    {
        return newTargetPosition;
    }

    /**
     * Get the old and new values
     * 
     */
    protected void getValues()
    {
        int delta = newLocation.y - initialLocation.y;

        GraphEdge edge = (GraphEdge) graphEdgeEdit.getModel();
        EList anchors = edge.getAnchor();
        source = (GraphConnector) anchors.get(0);
        target = (GraphConnector) anchors.get(1);

        oldSourcePosition = source.getPosition();
        newSourcePosition = oldSourcePosition.getCopy();
        newSourcePosition.y += delta;

        oldTargetPosition = target.getPosition();
        newTargetPosition = oldTargetPosition.getCopy();
        newTargetPosition.y += delta;
    }

    /**
     * Set the values on the edge
     * 
     */
    protected void setValues()
    {
        source.setPosition(newSourcePosition);
        target.setPosition(newTargetPosition);

        setModelValues();
    }

    /**
     * Swicth between the new and old values
     * 
     */
    protected void switchValues()
    {
        Point p = oldSourcePosition;
        oldSourcePosition = newSourcePosition;
        newSourcePosition = p;

        p = oldTargetPosition;
        oldTargetPosition = newTargetPosition;
        newTargetPosition = p;
    }

    /**
     * Set the values on the model. Should be overrided by sub classes
     * 
     */
    protected void setModelValues()
    {
        // nothing to do
    }

    /**
     * Get the old values then set the new values
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getValues();
        setValues();
    }

    /**
     * Set the new values
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Set the old values
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}
