/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.reconnect;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.BaseEditPart;
import org.topcased.modeler.edit.GraphEdgeEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;

/**
 * Comand to reconnect the graph edge
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 */
public class GraphEdgeReconnectCommand extends Command
{
    private GraphNode newNode;

    private GraphNode oldNode;

    private GraphEdge edge;

    private boolean isMovingStartAnchor;

    private GraphConnector connector;

    private Point newPos;

    private Point oldPos;

    private BaseEditPart newNodeEdit;

    private GraphEdgeEditPart graphEdgeEdit;

    private BaseEditPart oldNodeEdit;

    /**
     * Constructor
     * 
     * @param newEdit the edit part of the new receiving node
     * @param edgeEdit the edit part of the graph edge
     * @param isStart set it to true if the source is reconnected
     * @param newP the new position of the connector
     */
    public GraphEdgeReconnectCommand(BaseEditPart newEdit, MessageEditPart edgeEdit, boolean isStart, Point newP)
    {
        edge = (GraphEdge) edgeEdit.getModel();
        newPos = newP;
        isMovingStartAnchor = isStart;
        newNodeEdit = newEdit;
        graphEdgeEdit = edgeEdit;
    }

    /**
     * Return if this is start anchor which is moved
     * 
     * @return true if the start anchor is moved
     */
    public boolean isMovingStartAnchor()
    {
        return isMovingStartAnchor;
    }

    /**
     * Get the graph edge
     * 
     * @return the graph edge
     */
    public GraphEdge getGraphEdge()
    {
        return edge;
    }

    /**
     * Get the new node
     * 
     * @return the new graph node
     */
    public GraphNode getNewNode()
    {
        return newNode;
    }

    /**
     * Get the edit part of the new node
     * 
     * @return the edit part of the new node
     */
    public BaseEditPart getNewNodeEdit()
    {
        return newNodeEdit;
    }

    /**
     * Get the old graph node
     * 
     * @return the old node
     */
    public GraphNode getOldNode()
    {
        return oldNode;
    }

    /**
     * Get the edit part of the old node
     * 
     * @return the edit part of the old node
     */
    public BaseEditPart getOldNodeEdit()
    {
        return oldNodeEdit;
    }

    /**
     * Get the edit part of the edge
     * 
     * @return the edit part of the edge
     */
    public GraphEdgeEditPart getGraphEdgeEdit()
    {
        return graphEdgeEdit;
    }

    /**
     * Get the new position
     * 
     * @return the new position
     */
    public Point getNewPos()
    {
        return newPos;
    }

    /**
     * Set the new position
     * 
     * @param newP the new position
     */
    public void setNewPos(Point newP)
    {
        this.newPos = newP;
    }

    /**
     * Get the old position
     * 
     * @return the old position
     */
    public Point getOldPos()
    {
        return oldPos;
    }

    /**
     * Get the old values
     * 
     */
    protected void getOldValues()
    {
        newNode = (GraphNode) newNodeEdit.getModel();

        EList connectors = edge.getAnchor();

        if (isMovingStartAnchor)
        {
            connector = (GraphConnector) connectors.get(0);
        }
        else
        {
            connector = (GraphConnector) connectors.get(1);
        }
        oldNode = (GraphNode) connector.getGraphElement();

        oldNodeEdit = (BaseEditPart) graphEdgeEdit.getViewer().getEditPartRegistry().get(oldNode);

        oldPos = connector.getPosition();
    }

    /**
     * Set the new values
     * 
     */
    protected void setValues()
    {
        if (oldNode == newNode)
        {
            // the node doesn't change, just update the connector position
            connector.setPosition(newPos);
        }
        else
        {
            // the node change
            // remove from old node
            oldNode.getAnchorage().remove(connector);

            // add the connector to the new node
            newNode.getAnchorage().add(connector);

            // update the position of the connector
            connector.setPosition(newPos);
        }

        setModelValues();
    }

    /**
     * Set the values on the model. Should be customized by sub-classes
     */
    protected void setModelValues()
    {
        // nothing to do
    }

    /**
     * Swicth bewteen the old and new values
     * 
     */
    protected void switchValues()
    {
        GraphNode tempNode = oldNode;
        oldNode = newNode;
        newNode = tempNode;

        BaseEditPart tempNodeEdit = oldNodeEdit;
        oldNodeEdit = newNodeEdit;
        newNodeEdit = tempNodeEdit;

        Point tempPos = oldPos;
        oldPos = newPos;
        newPos = tempPos;
    }

    /**
     * get the old values and set the new ones
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        getOldValues();
        setValues();
    }

    /**
     * Graphically reconnect the message
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        switchValues();
        setValues();
    }

    /**
     * Undo the reconnection graphically
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        switchValues();
        setValues();
    }

}