/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.update;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.utils.LabelHelper;

/**
 * Command to update the lifeline parameters
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * @deprecated Use the UpdateLifelineRepresentsCommand instead
 */
public class LifelineUpdateCommand extends Command
{

    /** The Lifeline currently updated */
    private Lifeline lifeline;

    /** Current instanciated classifier */
    private Classifier classifier;

    private EditDomain editDomain;

    private boolean createProperty;

    private boolean createConnectorEnd;

    private Property property;

    private ConnectorEnd connectorend;

    private ConnectableElement oldProperty;

    /**
     * Create a command for updating InstanceSpecification values
     * 
     * @param life the current lifeline object
     * @param classif the new classifier that represents the lifeline
     * @param domain the edit domain
     */
    public LifelineUpdateCommand(Lifeline life, Classifier classif, EditDomain domain)
    {
        lifeline = life;
        classifier = classif;
        editDomain = domain;
        oldProperty = lifeline.getRepresents();
    }

    /**
     * Return the property own by the collaboration which is typed by the classifier. If the property is not found,
     * create it.
     * 
     * @param collaboration the owning collaboration
     * @param c the wanted classifer
     * 
     * @return the searched property
     * @deprecated use the getOwnedAttribute() method of the StructuredClassifier class defined in UML
     */
    protected Property getProperty(Collaboration collaboration, Classifier c)
    {
        Property prop = null;
        Iterator itAttributes = collaboration.getOwnedAttributes().iterator();
        while (prop == null && itAttributes.hasNext())
        {
            Object att = itAttributes.next();
            if (att instanceof Property)
            {
                Property p = (Property) att;
                if (p.getType().getName().equals(c.getName()))
                {
                    prop = p;
                }
            }
        }
        return prop;
    }

    /**
     * Return the connector end which has the correct property as role, owned by a connector int the interaction
     * 
     * @param interaction the owning interaction
     * @param prop the property
     * 
     * @return the ConnectorEnd
     */
    protected ConnectorEnd getConnectorEnd(Interaction interaction, Property prop)
    {
        EList connectors = interaction.getOwnedConnectors();
        Connector connector;
        if (connectors.isEmpty())
        {
            connector = interaction.createOwnedConnector(null);
            LabelHelper.INSTANCE.initName(editDomain, interaction, connector);
            return null;
        }

        Iterator itconnectors = connectors.iterator();
        ConnectorEnd end = null;
        while (end == null && itconnectors.hasNext())
        {
            Iterator itends = ((Connector) itconnectors.next()).getEnds().iterator();
            while (end == null && itends.hasNext())
            {
                ConnectorEnd e = (ConnectorEnd) itends.next();
                if (e.getRole() == prop)
                {
                    end = e;
                }
            }
        }
        return end;
    }

    /**
     * just redo
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        redo();
    }

    /**
     * Update the lifeline parameters
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        if (classifier == null)
        {
            lifeline.setRepresents(null);
            createProperty = false;
            createConnectorEnd = false;
            return;
        }

        // Retrieve the interaction from the Lifeline
        Interaction interaction = lifeline.getInteraction();

        // Try to get the first Property of the given type
        property = interaction.getOwnedAttribute(null, classifier);
        createProperty = property == null;

        // If a such property doesn't exist, create it
        if (createProperty)
        {
            property = interaction.createOwnedAttribute(null, classifier);
            LabelHelper.INSTANCE.initName(editDomain, interaction, property);
        }

        // Search a ConnectorEnd with the role set to the found property
        connectorend = getConnectorEnd(interaction, property);

        createConnectorEnd = connectorend == null;

        // if a such property doesn't exist, create it
        if (createConnectorEnd)
        {
            connectorend = ((Connector) interaction.getOwnedConnectors().get(0)).createEnd();
            connectorend.setRole(property);
        }

        lifeline.setRepresents(property);
    }

    /**
     * Undo the update
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if (createConnectorEnd)
        {
            connectorend.setRole(null);
            ((Connector) connectorend.getOwner()).getEnds().remove(connectorend);
        }

        if (createProperty)
        {
            ((StructuredClassifier) property.getOwner()).getOwnedAttributes().remove(property);
        }

        lifeline.setRepresents(oldProperty);
    }

}
