/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.update;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Connector;
import org.eclipse.uml2.uml.ConnectorEnd;
import org.eclipse.uml2.uml.Interaction;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.StructuredClassifier;
import org.topcased.modeler.utils.LabelHelper;

/**
 * Command for updating the represents property associated with a Lifeline
 * 
 * Creation 13 juin 06
 * 
 * @author jlescot
 */
public class UpdateLifelineRepresentsCommand extends Command
{
    /** The Lifeline to update */
    private Lifeline lifeline;

    /** The Classifier to associate */
    private Classifier classifier;

    private EditDomain editDomain;

    private boolean createProperty;

    private boolean createConnectorEnd;

    /**
     * The Property used to link the Classifier with the Lifeline through the Represents property
     */
    private ConnectableElement newProperty;

    private ConnectorEnd connectorend;

    private ConnectableElement oldProperty;

    /**
     * Create a command for updating InstanceSpecification values
     * 
     * @param lifeline the current lifeline object
     * @param classifier the new classifier that represents the lifeline
     * @param domain the edit domain
     */
    public UpdateLifelineRepresentsCommand(Lifeline lifeline, Classifier classifier, EditDomain domain)
    {
        this.lifeline = lifeline;
        this.classifier = classifier;
        this.editDomain = domain;
    }

    /**
     * Return the connector end which has the correct property as role, owned by a connector in the interaction
     * 
     * @param interaction the owning interaction
     * @param prop the property
     * 
     * @return the ConnectorEnd
     */
    protected ConnectorEnd getConnectorEnd(Interaction interaction, ConnectableElement prop)
    {
        EList connectors = interaction.getOwnedConnectors();
        Connector connector;
        if (connectors.isEmpty())
        {
            connector = interaction.createOwnedConnector(null);
            LabelHelper.INSTANCE.initName(editDomain, interaction, connector);
            return null;
        }

        Iterator itconnectors = connectors.iterator();
        ConnectorEnd end = null;
        while (end == null && itconnectors.hasNext())
        {
            Iterator itends = ((Connector) itconnectors.next()).getEnds().iterator();
            while (end == null && itends.hasNext())
            {
                ConnectorEnd e = (ConnectorEnd) itends.next();
                if (e.getRole() == prop)
                {
                    end = e;
                }
            }
        }
        return end;
    }

    /**
     * Store the old Property associated with the represents property of the Lifeline and then make the redo
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldProperty = lifeline.getRepresents();

        redo();
    }

    /**
     * Update the lifeline parameters
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        if (classifier == null)
        {
            lifeline.setRepresents(null);
            createProperty = false;
            createConnectorEnd = false;
            return;
        }

        // Retrieve the interaction from the Lifeline
        Interaction interaction = lifeline.getInteraction();

        // Try to get the first Property of the given type
        newProperty = interaction.getOwnedAttribute(null, classifier);
        createProperty = newProperty == null;

        // If a such property doesn't exist, create it
        if (createProperty)
        {
            newProperty = interaction.createOwnedAttribute(null, classifier);
            LabelHelper.INSTANCE.initName(editDomain, interaction, newProperty);
        }

        // Search a ConnectorEnd with the role set to the found property
        connectorend = getConnectorEnd(interaction, newProperty);

        createConnectorEnd = connectorend == null;

        // if a such property doesn't exist, create it
        if (createConnectorEnd)
        {
            connectorend = ((Connector) interaction.getOwnedConnectors().get(0)).createEnd();
            connectorend.setRole(newProperty);
        }

        lifeline.setRepresents(newProperty);
    }

    /**
     * Undo the update
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        if (createConnectorEnd)
        {
            connectorend.setRole(null);
            ((Connector) connectorend.getOwner()).getEnds().remove(connectorend);
        }

        if (createProperty)
        {
            ((StructuredClassifier) newProperty.getOwner()).getOwnedAttributes().remove(newProperty);
        }

        lifeline.setRepresents(oldProperty);
    }
}
