/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.commands.update;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageEvent;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Signal;
import org.eclipse.uml2.uml.TypedElement;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.modeler.uml.sequencediagram.SequenceUtils;
import org.topcased.modeler.utils.LabelHelper;

/**
 * Command for updating the Signature associated with a Message
 * 
 * Creation 14 juin 06
 * 
 * @author jlescot
 */
public class UpdateMessageSignatureCommand extends Command
{
    /** The Message to update */
    private Message message;

    private String oldMessageName;

    private String newMessageName;

    private EditDomain editDomain;

    private boolean createMessageEvent;

    /** The new NamedElement that represents the Signature of the Message */
    private NamedElement newSignature;

    /** The old NamedElement that represents the Signature of the Message */
    private NamedElement oldSignature;

    /** The MessageEvent that will be associated with the Signature */
    private MessageEvent messageEvent;

    private List oldArguments;

    private List newArguments;

    /**
     * Create a command for updating the Signature of a given Message
     * 
     * @param message the current message object
     * @param newSignature the new namedElement that represents the signature of the message
     * @param domain the edit domain
     */
    public UpdateMessageSignatureCommand(Message message, NamedElement newSignature, EditDomain domain)
    {
        this.message = message;
        this.newSignature = newSignature;
        this.editDomain = domain;
    }

    /**
     * Store the old Property associated with the represents property of the Lifeline and then make the redo
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        oldSignature = message.getSignature();

        // The oldMessageName is the name of the message before it is modified
        oldMessageName = message.getName();

        // The newMessageName is the name of the newSignature or the empty
        // String if the signature is null
        newMessageName = newSignature == null ? "" : newSignature.getName();

        // get the arguments associated with the Signatures
        oldArguments = new ArrayList(message.getArguments());
        newArguments = initializeArguments(newSignature);

        messageEvent = SequenceUtils.getMessageEvent(message);
        createMessageEvent = messageEvent == null;

        if (createMessageEvent)
        {
            // We need to create the MessageEvent and associate it with the
            // Message
            if (SequenceUtils.isSignal(message))
            {
                messageEvent = UMLFactory.eINSTANCE.createSignalEvent();
            }
            else
            {
                messageEvent = UMLFactory.eINSTANCE.createCallEvent();
            }

            LabelHelper.INSTANCE.initName(editDomain, message.getNearestPackage(), messageEvent);
        }

        redo();
    }

    /**
     * Update the message parameters
     * 
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
        if (createMessageEvent)
        {
            // We add the message that was created in the execute() method
            message.getNearestPackage().getPackagedElements().add(messageEvent);
        }

        // Update the Signature of the MessageEvent
        SequenceUtils.setMessageEventSignature(messageEvent, newSignature);

        // Update the arguments of the Message
        message.getArguments().clear();
        message.getArguments().addAll(newArguments);

        // Update the name of the Message
        message.setName(newMessageName);
    }

    /**
     * Undo the update
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
        SequenceUtils.setMessageEventSignature(messageEvent, oldSignature);

        if (createMessageEvent)
        {
            message.getNearestPackage().getPackagedElements().remove(messageEvent);
        }

        // Update the arguments of the Message
        message.getArguments().clear();
        message.getArguments().addAll(oldArguments);

        // update the name of the Message
        message.setName(oldMessageName);
    }

    /**
     * This method create all the necessary arguments that will match a given signature
     * 
     * @param signature the Signature
     * @return a List containing all the ValueSpecifications that will represent the function signature
     */
    private List initializeArguments(NamedElement signature)
    {
        List arguments = new ArrayList();
        List attributes = new ArrayList();

        // Retrieve the Properties or Parameters from the Signature
        if (signature instanceof Operation)
        {
            attributes = ((Operation) signature).getOwnedParameters();
        }
        else if (signature instanceof Signal)
        {
            attributes = ((Signal) signature).getOwnedAttributes();
        }

        // Iterates on all the Properties or Parameters and create the
        // representative Argument
        Iterator itAttributes = attributes.iterator();
        while (itAttributes.hasNext())
        {
            TypedElement attribute = (TypedElement) itAttributes.next();

            // Manage the Primitive types
            if (attribute.getType() instanceof PrimitiveType)
            {
                if (attribute.getType().getName().equals("Boolean"))
                {
                    if (attribute instanceof Property)
                    {
                        arguments.add(((Property) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_BOOLEAN));
                    }
                    else if (attribute instanceof Parameter)
                    {
                        arguments.add(((Parameter) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_BOOLEAN));
                    }
                }
                else if (attribute.getType().getName().equals("Integer"))
                {
                    if (attribute instanceof Property)
                    {
                        arguments.add(((Property) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_INTEGER));
                    }
                    else if (attribute instanceof Parameter)
                    {
                        arguments.add(((Parameter) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_INTEGER));
                    }
                }
                else if (attribute.getType().getName().equals("String"))
                {
                    if (attribute instanceof Property)
                    {
                        arguments.add(((Property) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                    }
                    else if (attribute instanceof Parameter)
                    {
                        arguments.add(((Parameter) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                    }
                }
                // -RFU- traitement des autres types primitifs comme String Literal
                else 
                {
                	if (attribute instanceof Property)
                	{
                		arguments.add(((Property) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                	}
                	else if (attribute instanceof Parameter)
                	{
                		arguments.add(((Parameter) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                	}
                }
            }
            
            // -RFU- Manage the Enumeration types
            else if (attribute.getType() instanceof org.eclipse.uml2.uml.Enumeration)
            {
            	InstanceValue instanceValue = UMLFactory.eINSTANCE.createInstanceValue();
            	instanceValue.setName(attribute.getName());
            	instanceValue.setType(attribute.getType());
                arguments.add(instanceValue);
            }
            // Manage the Class types
            else if (attribute.getType() instanceof org.eclipse.uml2.uml.Class)
            {
                InstanceValue instanceValue = UMLFactory.eINSTANCE.createInstanceValue();
                // -RFU- rajout du nom
                instanceValue.setName(attribute.getName());
                
                instanceValue.setType(attribute.getType());
                arguments.add(instanceValue);
            }

            // Manage any other type
            // -RFU- traiter les autres types comme des String Literal
            else
            {
            	if (attribute instanceof Property)
                {
                    //arguments.add(((Property) attribute).createDefaultValue(null, null, UMLPackage.Literals.LITERAL_NULL));
                	arguments.add(((Property) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                }
                else if (attribute instanceof Parameter)
                {
                    arguments.add(((Parameter) attribute).createDefaultValue(attribute.getName(), attribute.getType(), UMLPackage.Literals.LITERAL_STRING));
                }
            }
        }

        return arguments;
    }
}
