// ------------------------------------------------
// $Id: ArgumentsComposite.java,v 1.2 2007-05-25 11:34:54 jako Exp $
// (c) Anyware Technologies 2007 www.anyware-tech.com
// ------------------------------------------------
package org.topcased.modeler.uml.sequencediagram.composites;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ValueSpecification;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.uml.sequencediagram.editingsupport.DelegatingEditingSupport;
import org.topcased.modeler.uml.sequencediagram.labelproviders.ValueSpecificationValueLabelProvider;
import org.topcased.modeler.uml.sequencediagram.properties.sections.MessageSignaturePropertySection;
import org.topcased.modeler.uml.sequencediagram.providers.ValueSpecificationContentProvider;

/**
 * The composite responsible for editing of the message's arguments.
 * 
 * @author jrme
 */
public class ArgumentsComposite extends Composite
{
    /**
     * The table viewer used for editing the arguments. The input is a list of
     * the arguments of the currently selected message.
     */
    private TableViewer viewer;

    /**
     * The editing support used to modify an argument value. Kept as a private
     * field so as to be able to update it through
     * {@link #setEditDomain(MixedEditDomain)}.
     */
    private final DelegatingEditingSupport editingSupport;;

    /**
     * Constructor : creates the viewer. Uses a parameter to get the preferred
     * column width from the form section so as to keep a uniform layout.
     * 
     * @param parent
     * @param style
     * @param columnWidth The preferred column width in pixels
     */
    public ArgumentsComposite(Composite parent, int style, int columnWidth)
    {
        super(parent, style);
        this.setLayout(new GridLayout(1, false));
        viewer = new TableViewer(this, SWT.FULL_SELECTION | SWT.SINGLE | SWT.V_SCROLL | SWT.BORDER);

        GridData data = new GridData(GridData.FILL_BOTH);
        data.verticalSpan = 15;
        viewer.getTable().setLayoutData(data);

        viewer.setContentProvider(new ValueSpecificationContentProvider());
        viewer.getTable().setLinesVisible(true);
        viewer.getTable().setHeaderVisible(true);

        TableViewerColumn column;
        column = createTableViewerColumn(viewer, "Property Name", new ValueSpecificationNameLabelProvider(), null, 0);
        column.getColumn().setWidth(columnWidth);

        column = createTableViewerColumn(viewer, "Type", new ValueSpecificationTypeLabelProvider(), null, 1);
        column.getColumn().setWidth(columnWidth);

        editingSupport = new DelegatingEditingSupport(viewer);
        column = createTableViewerColumn(viewer, "Value", new ValueSpecificationValueLabelProvider(), editingSupport, 2);
        column.getColumn().setWidth(columnWidth);
    }

    /**
     * Convenience method used to create a {@link TableViewerColumn} with the
     * given parameters in the viewer passed as an argument.
     * 
     * @param viewer
     * @param columnTitle
     * @param labelProvider
     * @param editingSupport
     * @param index
     * @return the {@link TableViewerColumn} created.
     */
    private TableViewerColumn createTableViewerColumn(final TableViewer viewer, final String columnTitle,
            final ColumnLabelProvider labelProvider, final EditingSupport editingSupport, final int index)
    {
        TableViewerColumn column;
        column = new TableViewerColumn(viewer, SWT.CENTER, index);
        column.getColumn().setText(columnTitle);
        column.setLabelProvider(labelProvider);
        if (editingSupport != null)
        {
            column.setEditingSupport(editingSupport);
        }

        return column;
    }

    /**
     * Label provider returning the name of a {@link ValueSpecification}
     * 
     */
    protected class ValueSpecificationNameLabelProvider extends ColumnLabelProvider
    {
        @Override
        public String getText(Object element)
        {
            // special for calls : the name is null
            if (element instanceof ValueSpecification)
            {
                if (((EObject) element).eContainer() instanceof Message)
                {
                    Message message = (Message) ((EObject) element).eContainer();
                    if (message.getSendEvent() instanceof MessageOccurrenceSpecification)
                    {
                        if (((MessageOccurrenceSpecification) message.getSendEvent()).getEvent() instanceof CallEvent)
                        {
                            CallEvent event = (CallEvent) ((MessageOccurrenceSpecification) message.getSendEvent()).getEvent();
                            Operation operation = event.getOperation();
                            int indexCurrentArgument = message.getArguments().indexOf((ValueSpecification) element);
                            Parameter currentParameter = operation.getOwnedParameters().get(indexCurrentArgument);
                            return currentParameter.getName();

                        }
                    }
                }

                return ((ValueSpecification) element).getName();
            }

            return "";
        }

        @Override
        public Image getImage(Object element)
        {
            return null;
        }
    }

    /**
     * Label provider returning the type of a {@link ValueSpecification}
     * 
     */
    protected class ValueSpecificationTypeLabelProvider extends ColumnLabelProvider
    {
        @Override
        public String getText(Object element)
        {
            if (element instanceof ValueSpecification)
            {
                if (((ValueSpecification) element).getType() != null)
                {
                    return ((ValueSpecification) element).getType().getName();
                }
                return "";
            }

            return "";
        }

        @Override
        public Image getImage(Object element)
        {
            return null;
        }
    }

    /**
     * Called from the {@link MessageSignaturePropertySection} when an update is
     * needed. Changes the viewer input to the new input.
     * 
     * @param message
     */
    public void updateViewer(Message message)
    {
        if (viewer.getTable() != null && !viewer.getTable().isDisposed())
        {
            viewer.setInput(message);

        }
    }

    /**
     * Called from {@link MessageSignaturePropertySection} to update the editing
     * domain. Only passes it to the editing support, not needed here.
     * 
     * @param editDomain
     */
    public void setEditDomain(MixedEditDomain editDomain)
    {
        this.editingSupport.setEditDomain(editDomain);
    }
}
