/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.dialogs;

import java.util.ArrayList;

import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.uml.UMLImageRegistry;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.sequencediagram.commands.creation.ClassifierCreationCommand;

/**
 * Dialog to create a classifier in the model This dialog will offer the choice of created type :<br> - Actor<br> -
 * Class<br>
 * and the choice where ut will be created (in which package).
 */
public class ClassifierCreateDialog extends Dialog implements ITopcasedDialogConstants
{
    // SWT Objects
    private Composite dialogComposite;

    private Text classifierNameText;

    private Model model;

    private Tree packagesTree;

    private TreeViewer packagesViewer;

    private ArrayList packages;

    private Button classButton;

    private Button actorButton;

    private Command command;

    /** The classifier that has been created when the OK Button has been pressed */
    private Classifier classifier;

    /**
     * Constructor
     * 
     * @param m the UML2 root element
     * @param parentShell the owning shell
     */
    public ClassifierCreateDialog(Model m, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        this.model = m;
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Classifier creation");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents(Composite parent)
    {
        Control control = super.createContents(parent);

        // Disable the OK Button until a name is typed
        getButton(IDialogConstants.OK_ID).setEnabled(false);

        return control;
    }

    /**
     * Create the complete area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createDialogComposite();

        createNameEntry(dialogComposite);
        createTypeGroup(dialogComposite);
        createPackageGroup(dialogComposite);

        loadData();
        hookListeners();

        return dialogComposite;
    }

    /**
     * Create the Dialog Contents
     */
    private void createDialogComposite()
    {
        GridLayout dialogLayout = new GridLayout();
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayout(dialogLayout);
        dialogComposite.setLayoutData(dialogLayoutData);
    }

    /**
     * Create the name input entry
     * 
     * @param parent the owning area
     */
    protected void createNameEntry(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label label = new Label(composite, SWT.NONE);
        label.setText("Name : ");

        classifierNameText = new Text(composite, SWT.BORDER);
        classifierNameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Create the area of the type buttons
     * 
     * @param parent the owning area
     */
    protected void createTypeGroup(Composite parent)
    {
        Group grp = new Group(parent, SWT.NONE);
        grp.setText("Select the Type");
        grp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        grp.setLayout(new GridLayout());

        classButton = new Button(grp, SWT.RADIO);
        classButton.setSelection(true);
        classButton.setText("Class");
        classButton.setImage(UMLImageRegistry.getImage("CLASS"));
        classButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        actorButton = new Button(grp, SWT.RADIO);
        actorButton.setSelection(false);
        actorButton.setText("Actor");
        actorButton.setImage(UMLImageRegistry.getImage("ACTOR"));
        actorButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Create the list of package
     * 
     * @param parent the owning area
     */
    protected void createPackageGroup(Composite parent)
    {
        Group grp = new Group(parent, SWT.NONE);
        grp.setText("Select the container Package");
        grp.setLayoutData(new GridData(GridData.FILL_BOTH));
        grp.setLayout(new GridLayout());

        packagesTree = new Tree(grp, SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        packagesTree.setLayoutData(new GridData(GridData.FILL_BOTH));

        packagesViewer = new TreeViewer(packagesTree);
        packagesViewer.setContentProvider(new TreeArrayContentProvider());
        packagesViewer.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));
    }

    /**
     * Initialize the widgets
     */
    private void loadData()
    {
        packages = UMLTools.getAllObjects(model, Package.class);
        packagesViewer.setInput(packages);
        packagesViewer.setSelection(new StructuredSelection(packages.get(0)));
    }

    /**
     * Hook the listeners associated with the widgets
     */
    private void hookListeners()
    {
        classifierNameText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                getButton(OK).setEnabled(classifierNameText.getText().length() != 0);
            }
        });
    }

    /**
     * Save the viewers datas before the dialogs disposing
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        Package selectedPackage = (Package) ((IStructuredSelection) packagesViewer.getSelection()).getFirstElement();
        String classifierName = classifierNameText.getText();
        if (classButton.getSelection())
        {
            classifier = UMLFactory.eINSTANCE.createClass();
        }
        else if (actorButton.getSelection())
        {
            classifier = UMLFactory.eINSTANCE.createActor();
        }

        if (classifier != null)
        {
            classifier.setName(classifierName);
            command = new ClassifierCreationCommand(selectedPackage, classifier);
        }

        super.okPressed();
    }

    /**
     * Return the created Classifier (Class or Actor)
     * 
     * @return Classifier
     */
    public Classifier getCreatedClassifier()
    {
        return classifier;
    }

    /**
     * Return the creation command
     * 
     * @return the command
     */
    public Command getCommand()
    {
        return command;
    }

    /**
     * Wrapper to adapt the ArrayContentProvider to a TreeViewer
     * 
     * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
     */
    private class TreeArrayContentProvider extends ArrayContentProvider implements ITreeContentProvider
    {
        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
         */
        public Object[] getChildren(Object parentElement)
        {
            return new Object[0];
        }

        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
         */
        public Object getParent(Object element)
        {
            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
         */
        public boolean hasChildren(Object element)
        {
            return false;
        }
    }

}
