/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.dialogs;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.InteractionOperatorKind;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;

/**
 * Dialog for creating a new Combined Fragment This dialog will propose the different intercation operators, and will
 * ask (if necessary, depending of the choosen operator) for the number of operand to create
 */
public class CombinedFragmentCreateDialog extends Dialog implements ITopcasedDialogConstants
{
    private static final HashMap INTERACTION_OPERATORS = new HashMap();

    private static final int LABEL_NUMBEROFOPERAND_WIDTH = 80;

    static
    {
        INTERACTION_OPERATORS.put("Alternative", new Integer(InteractionOperatorKind.ALT));
        INTERACTION_OPERATORS.put("Assertion", new Integer(InteractionOperatorKind.ASSERT));
        INTERACTION_OPERATORS.put("Break", new Integer(InteractionOperatorKind.BREAK));
        INTERACTION_OPERATORS.put("Consider", new Integer(InteractionOperatorKind.CONSIDER));
        INTERACTION_OPERATORS.put("Critical", new Integer(InteractionOperatorKind.CRITICAL));
        INTERACTION_OPERATORS.put("Ignore", new Integer(InteractionOperatorKind.IGNORE));
        INTERACTION_OPERATORS.put("Loop", new Integer(InteractionOperatorKind.LOOP));
        INTERACTION_OPERATORS.put("Negative", new Integer(InteractionOperatorKind.NEG));
        INTERACTION_OPERATORS.put("Optional", new Integer(InteractionOperatorKind.OPT));
        INTERACTION_OPERATORS.put("Parallele", new Integer(InteractionOperatorKind.PAR));
        INTERACTION_OPERATORS.put("Sequence", new Integer(InteractionOperatorKind.SEQ));
        INTERACTION_OPERATORS.put("Strict", new Integer(InteractionOperatorKind.STRICT));
    }

    // SWT Objects
    private Composite dialogComposite;

    private List operatorsViewer;

    private int selectedoperator;

    private Text numberOfOperandText;

    private int numberOfOperand;

    private Label numberOfOperandErrorLabel;

    /**
     * Create the dialog box
     * 
     * @param parentShell the parent shell owning the dialog
     */
    public CombinedFragmentCreateDialog(Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        selectedoperator = -1;
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Combined fragment creation");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog Contents
     */
    private void createDialogComposite()
    {
        GridLayout dialogLayout = new GridLayout();
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayout(dialogLayout);
        dialogComposite.setLayoutData(dialogLayoutData);
    }

    /**
     * create the area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createDialogComposite();
        createOperatorsGroup(dialogComposite);
        createNumberOfOperandGroup(dialogComposite);
        return dialogComposite;
    }

    /**
     * Create the view of the list of interaction operators
     * 
     * @param parent the parent composite to contain this group
     */
    protected void createOperatorsGroup(Composite parent)
    {
        new Label(parent, SWT.NONE).setText("Select a Interaction Operator");
        operatorsViewer = new List(parent, SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        operatorsViewer.setLayoutData(new GridData(GridData.FILL_BOTH));

        Iterator it = INTERACTION_OPERATORS.entrySet().iterator();
        while (it.hasNext())
        {
            operatorsViewer.add((String) it.next());
        }

        operatorsViewer.addSelectionListener(new SelectionListener()
        {
            public void widgetDefaultSelected(SelectionEvent e)
            {
                updateNumberOfOperand();
            }

            public void widgetSelected(SelectionEvent e)
            {
                updateNumberOfOperand();
            }
        });
        operatorsViewer.select(0);
    }

    /**
     * Update the number of format depending of the choosen operator As the UML2 spec says, if the choosen operator is
     * an opt, a loop, a break or a neg, the number of operand is 1
     */
    protected void updateNumberOfOperand()
    {
        int operator = ((Integer) INTERACTION_OPERATORS.get(operatorsViewer.getSelection()[0])).intValue();
        switch (operator)
        {
            case InteractionOperatorKind.OPT:
            case InteractionOperatorKind.LOOP:
            case InteractionOperatorKind.BREAK:
            case InteractionOperatorKind.NEG:
                numberOfOperandText.setText("1");
                numberOfOperandText.setEnabled(false);
                break;
            default:
                numberOfOperandText.setEnabled(true);
                break;
        }
    }

    /**
     * Create the view of the list of interaction operators
     * 
     * @param parent the parent composite to contain this group
     */
    protected void createNumberOfOperandGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        composite.setLayout(layout);
        composite.setFont(parent.getFont());

        GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
        composite.setLayoutData(gridData);

        Label label = new Label(composite, SWT.NONE);
        label.setText("Number of operand : ");

        gridData = new GridData();
        gridData.widthHint = LABEL_NUMBEROFOPERAND_WIDTH;
        label.setLayoutData(gridData);

        numberOfOperandText = new Text(composite, SWT.BORDER);

        gridData = new GridData(GridData.FILL_HORIZONTAL);
        numberOfOperandText.setLayoutData(gridData);
        numberOfOperandText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                verifyNumberOfOperand();
            }
        });

        numberOfOperandErrorLabel = new Label(composite, SWT.NONE);
        gridData = new GridData(GridData.FILL_HORIZONTAL);
        gridData.horizontalSpan = 2;
        numberOfOperandErrorLabel.setText("");
        numberOfOperandErrorLabel.setFont(parent.getFont());
        numberOfOperandErrorLabel.setForeground(new Color(parent.getDisplay(), 255, 0, 0));
    }

    /**
     * Verify if the number of operand is correct. It verifies if it is a integer and if this integer is positive and
     * non null. In case of error, the OK button is disabled and an error notification is displayed just under the text
     * area.
     */
    protected void verifyNumberOfOperand()
    {
        try
        {
            numberOfOperand = Integer.parseInt(numberOfOperandText.getText());

            if (numberOfOperand < 0)
            {
                getButton(OK).setEnabled(false);
                numberOfOperandErrorLabel.setText("Error : the number must be positive");
            }
            else if (numberOfOperand == 0)
            {
                getButton(OK).setEnabled(false);
                numberOfOperandErrorLabel.setText("Error : the number mustn't be null");
            }
            else
            {
                getButton(OK).setEnabled(true);
                numberOfOperandErrorLabel.setText("");
            }
        }
        catch (NumberFormatException e)
        {
            getButton(OK).setEnabled(false);
            numberOfOperandErrorLabel.setText("Error : enter a valid number");
        }
    }

    /**
     * Save the viewers datas before the dialogs disposing
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        selectedoperator = ((Integer) INTERACTION_OPERATORS.get(operatorsViewer.getSelection()[0])).intValue();
        super.okPressed();
    }

    /**
     * Return the selected Interaction Operator. See org.eclipse.uml2.InteractionOperator for valid returned values
     * 
     * @return ALT, ASSERT, BREAK, CONSIDER, CRITICAL, IGNORE, LOOP, NEG, OPT, PAR, SEQ or STRICT (see
     *         org.eclipse.uml2.InteractionOperator)
     */
    public int getSelectedOperator()
    {
        return selectedoperator;
    }
}
