/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.dialogs;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.uml2.uml.Actor;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.Property;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;
import org.topcased.modeler.ModelerPlugin;
import org.topcased.modeler.commands.CompoundCommand;
import org.topcased.modeler.uml.UMLTools;
import org.topcased.modeler.uml.sequencediagram.commands.update.LifelineUpdateCommand;

/**
 * Dialog which will be drawn for the edition of a lifeline.
 * 
 * @deprecated Use the Tabbed Properties View instead
 */
public class LifelineEditDialog extends Dialog implements ITopcasedDialogConstants
{

    private static final int CREATECLASSIFIER_ID = 0;

    // SWT Objects
    private Composite dialogComposite;

    private List classifierViewer;

    private ArrayList classifiers;

    private String[] classifiersNames;

    private Lifeline lifeline;

    private int firstSelected;

    private EditDomain editDomain;

    private CompoundCommand command;

    /**
     * Consructor
     * 
     * @param life the lifeline to edit
     * @param domain the edit domain
     * @param parentShell the owning parent
     */
    public LifelineEditDialog(Lifeline life, EditDomain domain, Shell parentShell)
    {
        super(parentShell);
        setBlockOnOpen(true);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        lifeline = life;
        editDomain = domain;
        command = new CompoundCommand();
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("Classifier Selection");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Add updates
     * 
     * @see org.eclipse.jface.window.Window#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents(Composite parent)
    {
        Control ctrl = super.createContents(parent);
        firstSelected = updateClassifierNames();
        return ctrl;
    }

    /**
     * Create the Dialog Contents
     */
    private void createDialogComposite()
    {
        GridLayout dialogLayout = new GridLayout();
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        dialogLayoutData.widthHint = DEFAULT_DIALOG_WIDTH;
        dialogLayoutData.heightHint = DEFAULT_DIALOG_HEIGHT;
        dialogComposite.setLayout(dialogLayout);
        dialogComposite.setLayoutData(dialogLayoutData);
    }

    /**
     * Create the view of the list of classifier
     * 
     * @param parent the owning composite
     */
    protected void createClassifierGroup(Composite parent)
    {
        new Label(parent, SWT.NONE).setText("Select a classifier :");

        classifierViewer = new List(parent, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
        classifierViewer.setLayoutData(new GridData(GridData.FILL_BOTH));
    }

    /**
     * Create the view of the list of classifier
     * 
     * @param parent the owning composite
     */
    protected void createNewClassifierGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        composite.setLayout(layout);
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_CENTER | GridData.VERTICAL_ALIGN_CENTER);
        composite.setLayoutData(data);
        composite.setFont(parent.getFont());

        createButton(composite, IDialogConstants.CLIENT_ID + CREATECLASSIFIER_ID, "New classifier", false);
    }

    /**
     * Update the list of classifier
     * 
     * @return the index of the selected classifier
     */
    protected int updateClassifierNames()
    {
        // get the classifier of the lifeline
        Property property = (Property) lifeline.getRepresents();
        String classifierName = null;
        if (property != null)
        {
            classifierName = UMLTools.getCompleteName(property.getType());
        }

        ArrayList list = UMLTools.getAllObjects(lifeline.getModel(), Classifier.class);

        classifiers = sortClassifiers(list);

        classifiersNames = new String[classifiers.size() + 1];
        int i = 1;
        int select = 0;
        Iterator it = classifiers.iterator();
        while (it.hasNext())
        {
            classifiersNames[i] = UMLTools.getCompleteName((Element) it.next());
            if (classifiersNames[i].equals(classifierName))
            {
                select = i;
            }
            i++;
        }

        // add the first item : no classifier
        classifiers.add(0, null);
        classifiersNames[0] = "NONE";

        classifierViewer.setItems(classifiersNames);
        classifierViewer.select(select);

        Button okbutton = getButton(IDialogConstants.OK_ID);
        okbutton.setEnabled(i != 0);

        return select;
    }

    /**
     * Sort the list of classifiers by their types. First there are the classes, then the actors and the rest
     * 
     * @param list the unsorted list of classifiers
     * 
     * @return the sorted array list of classifiers
     */
    protected ArrayList sortClassifiers(ArrayList list)
    {
        ArrayList sortedlist = new ArrayList(list.size());

        sortedlist.addAll(filterClassifiers(list, Class.class));
        sortedlist.addAll(filterClassifiers(list, Actor.class));

        ArrayList templist = new ArrayList(list);
        templist.removeAll(sortedlist);

        sortedlist.addAll(templist);
        return sortedlist;
    }

    /**
     * Filter a list of classifiers : only the instance of the specified class are kept.
     * 
     * @param list the list of classifiers
     * @param clazz the class type to filter
     * @return the filtered classifiers
     */
    protected ArrayList filterClassifiers(ArrayList list, java.lang.Class clazz)
    {
        ArrayList filteredlist = new ArrayList();

        Iterator it = list.iterator();
        while (it.hasNext())
        {
            Classifier classifier = (Classifier) it.next();
            if (clazz.isInstance(classifier))
            {
                filteredlist.add(classifier);
            }
        }
        return filteredlist;
    }

    /**
     * create the area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        // Dialog
        dialogComposite = (Composite) super.createDialogArea(parent);
        createDialogComposite();
        createClassifierGroup(dialogComposite);
        createNewClassifierGroup(dialogComposite);
        return dialogComposite;
    }

    /**
     * Handle the "Create classifer" button
     * 
     * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
     */
    protected void buttonPressed(int buttonId)
    {
        super.buttonPressed(buttonId);
        if (buttonId == IDialogConstants.CLIENT_ID + CREATECLASSIFIER_ID)
        {
            ClassifierCreateDialog dialog = new ClassifierCreateDialog(lifeline.getModel(), ModelerPlugin.getActiveWorkbenchShell());
            if (dialog.open() == Window.OK)
            {
                Command c = dialog.getCommand();
                if (c != null)
                {
                    c.execute();
                    command.addExecuted(c);
                    updateClassifierNames();
                }
            }
        }
    }

    /**
     * Save the viewers datas before the dialogs disposing
     * 
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        int index = classifierViewer.getSelectionIndex();

        Classifier selectedClassifier = (Classifier) classifiers.get(index);

        if (index != firstSelected)
        {
            command.add(new LifelineUpdateCommand(lifeline, selectedClassifier, editDomain));
        }
        super.okPressed();
    }

    /**
     * Undo the
     * 
     * @see org.eclipse.jface.dialogs.Dialog#cancelPressed()
     */
    protected void cancelPressed()
    {
        command.undo();
        super.cancelPressed();
    }

    /**
     * Return the command to update the lifeline
     * 
     * @return the lifeline command
     */
    public Command getCommand()
    {
        return command;
    }

}
