/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.sequencediagram.editingsupport;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.swt.widgets.Composite;

/**
 * When subclassing, you should override {@link #doSetValue(Object, Object)} and
 * {@link #doGetValue(Object)}, since these two methods allow one to work
 * directly with objects from the model (otherwise a combo cell editor works
 * with strings).
 * 
 * We assume all the items are different from one another.
 * 
 * Every overriden method has been made public because we want to have access to
 * it from the delegating support.
 * 
 * @author jrme
 */
public abstract class ComboBoxEditingSupport extends EditingSupport
{
    private final Object[] items;

    private final String[] itemsLabels;

    /**
     * Constructs a new {@link ComboBoxEditingSupport} for the given
     * <code>items</code>, having the given <code>itemsLabels</code>.
     * 
     * @param viewer
     * @param items
     * @param itemsLabels
     */
    public ComboBoxEditingSupport(ColumnViewer viewer, Object[] items, String[] itemsLabels)
    {
        super(viewer);
        this.items = items;
        this.itemsLabels = itemsLabels;

        if (items.length != itemsLabels.length)
        {
            throw new IllegalArgumentException("");
        }
    }

    /**
     * Unused constructor, here since {@link EditingSupport} is abstract.
     * 
     * @param viewer
     */
    private ComboBoxEditingSupport(ColumnViewer viewer)
    {
        super(viewer);
        items = null;
        itemsLabels = null;
    }

    @Override
    public boolean canEdit(Object element)
    {
        return true;
    }

    @Override
    public CellEditor getCellEditor(Object element)
    {
        return new ComboBoxCellEditor((Composite) this.getViewer().getControl(), itemsLabels);
    }

    @Override
    public Object getValue(Object element)
    {
        Object value = this.doGetValue(element);
        int index = 0;
        if (value != null)
        {
            for (Object item : items)
            {
                if (item == value)
                {
                    break;
                }
                ++index;
            }
        }
        return new Integer(index);
    }

    @Override
    public void setValue(Object element, Object value)
    {
        int index = (Integer) value;
        if (index >= 0)
        {
            this.doSetValue(element, items[index]);
            this.getViewer().update(element, null);

        }
        else
        {
            // no value selected, do nothing.
        }
    }

    protected abstract Object doGetValue(Object element);

    protected abstract void doSetValue(Object element, Object value);

}
