/*******************************************************************************
 * Copyright (c) 2007 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.sequencediagram.editingsupport;

import java.util.Collection;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.InstanceValue;
import org.eclipse.uml2.uml.LiteralBoolean;
import org.eclipse.uml2.uml.LiteralInteger;
import org.eclipse.uml2.uml.LiteralNull;
import org.eclipse.uml2.uml.LiteralString;
import org.eclipse.uml2.uml.LiteralUnlimitedNatural;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.uml2.uml.util.UMLSwitch;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.uml.classdiagram.util.InstanceSpecificationHelper;
import org.topcased.modeler.uml.sequencediagram.composites.ArgumentsComposite;

/**
 * Editing support used for editing a {@link ValueSpecification} value. Uses an
 * internal switch to delegate to the appropriate EditingSupport since we can't
 * have multiple inheritance.
 * 
 * Because of the lack of multiple inheritance, since we need access to the
 * corresponding methods in the delegated editing support, we use a cast to
 * {@link TextEditingSupport} or {@link ComboBoxEditingSupport} in order to
 * access the methods made public (eg canEdit is protected in
 * {@link EditingSupport}, but {@link TextEditingSupport} overrides it and
 * makes it public).
 * 
 * @author jrme
 */
public class DelegatingEditingSupport extends EditingSupport
{
    /**
     * Internal switch returning appropriate EditingSupports
     */
    private final UMLSwitch<EditingSupport> switcher;

    /**
     * EditDomain used for modifying the model.
     */
    private MixedEditDomain editDomain;

    public DelegatingEditingSupport(ColumnViewer viewer)
    {
        super(viewer);
        switcher = new ValueSpecificationEditingSupport(viewer);
    }

    /**
     * Called from the {@link ArgumentsComposite} to set the EditDomain.
     * 
     * @param editDomain
     */
    public void setEditDomain(MixedEditDomain editDomain)
    {
        this.editDomain = editDomain;
    }

    /**
     * Delegates to the right EditingSupport.
     * 
     * @see org.eclipse.jface.viewers.EditingSupport#canEdit(java.lang.Object)
     */
    @Override
    protected boolean canEdit(Object element)
    {
        EditingSupport editingSupport = switcher.doSwitch((EObject) element);
        boolean canEdit = false;
        if (editingSupport instanceof TextEditingSupport)
        {
            canEdit = ((TextEditingSupport) editingSupport).canEdit(element);
        }
        else if (editingSupport instanceof ComboBoxEditingSupport)
        {
            canEdit = ((ComboBoxEditingSupport) editingSupport).canEdit(element);
        }
        return canEdit;
    }

    /**
     * Delegates to the right EditingSupport.
     * 
     * @see org.eclipse.jface.viewers.EditingSupport#getCellEditor(java.lang.Object)
     */
    @Override
    protected CellEditor getCellEditor(Object element)
    {
        EditingSupport editingSupport = switcher.doSwitch((EObject) element);
        CellEditor cellEditor = null;
        if (editingSupport instanceof TextEditingSupport)
        {
            cellEditor = ((TextEditingSupport) editingSupport).getCellEditor(element);
        }
        else if (editingSupport instanceof ComboBoxEditingSupport)
        {
            cellEditor = ((ComboBoxEditingSupport) editingSupport).getCellEditor(element);
        }
        return cellEditor;
    }

    /**
     * Delegates to the right EditingSupport.
     * 
     * @see org.eclipse.jface.viewers.EditingSupport#getValue(java.lang.Object)
     */
    @Override
    protected Object getValue(Object element)
    {
        EditingSupport editingSupport = switcher.doSwitch((EObject) element);
        Object value = null;
        if (editingSupport instanceof TextEditingSupport)
        {
            value = ((TextEditingSupport) editingSupport).getValue(element);
        }
        else if (editingSupport instanceof ComboBoxEditingSupport)
        {
            value = ((ComboBoxEditingSupport) editingSupport).getValue(element);
        }
        return value;
    }

    /**
     * Delegates to the right EditingSupport.
     * 
     * @see org.eclipse.jface.viewers.EditingSupport#setValue(java.lang.Object,
     *      java.lang.Object)
     */
    @Override
    protected void setValue(Object element, Object value)
    {
        EditingSupport editingSupport = switcher.doSwitch((EObject) element);
        Object oldValue = getValue(element);
        if (oldValue != value)
        {
            if (editingSupport instanceof TextEditingSupport)
            {
                ((TextEditingSupport) editingSupport).setValue(element, value);
            }
            else if (editingSupport instanceof ComboBoxEditingSupport)
            {
                ((ComboBoxEditingSupport) editingSupport).setValue(element, value);
            }
        }
    }

    /**
     * Switch on ValueSpecification subtypes that returns an appropriate
     * EditingSupport.
     * 
     * @author jrme
     */
    private class ValueSpecificationEditingSupport extends UMLSwitch<EditingSupport>
    {
        private final ColumnViewer viewer;

        public ValueSpecificationEditingSupport(ColumnViewer viewer)
        {
            this.viewer = viewer;
        }

        @Override
        public EditingSupport caseInstanceValue(InstanceValue object)
        {
            Collection<InstanceSpecification> possibleInstances = InstanceSpecificationHelper.getAvailableInstances(
                    object.getNearestPackage(), object.getType());
            String[] comboItems = new String[possibleInstances.size()];
            int i = 0;
            for (InstanceSpecification instanceSpecification : possibleInstances)
            {
                comboItems[i] = instanceSpecification.getName();
                ++i;
            }

            EditingSupport editingSupport = new ComboBoxEditingSupport(this.viewer, possibleInstances.toArray(),
                    comboItems)
            {
                @Override
                public Object doGetValue(Object element)
                {
                    return ((InstanceValue) element).getInstance();
                }

                @Override
                public void doSetValue(Object element, Object value)
                {
                    Command command = new SetCommand(editDomain.getEMFEditingDomain(), (EObject) element,
                            UMLPackage.eINSTANCE.getInstanceValue_Instance(), value);
                    editDomain.getEMFEditingDomain().getCommandStack().execute(command);
                }
            };
            return editingSupport;
        }

        @Override
        public EditingSupport caseLiteralBoolean(LiteralBoolean object)
        {
            Object[] possibleValues = new Boolean[] {true, false};
            String[] comboItems = new String[] {"true", "false"};
            return new ComboBoxEditingSupport(this.viewer, possibleValues, comboItems)
            {
                @Override
                protected Object doGetValue(Object element)
                {
                    return ((LiteralBoolean) element).isValue();
                }

                @Override
                protected void doSetValue(Object element, Object value)
                {
                    Command command = SetCommand.create(editDomain.getEMFEditingDomain(), element,
                            UMLPackage.eINSTANCE.getLiteralBoolean_Value(), value);
                    editDomain.getEMFEditingDomain().getCommandStack().execute(command);

                }
            };
        }

        @Override
        public EditingSupport caseLiteralInteger(LiteralInteger object)
        {
            return new TextEditingSupport(this.viewer)
            {
                @Override
                public Object getValue(Object element)
                {
                    return String.valueOf(((LiteralInteger) element).getValue());
                }

                @Override
                public void setValue(Object element, Object value)
                {
                    int n = Integer.parseInt(((String) value));
                    Command command = SetCommand.create(editDomain.getEMFEditingDomain(), element,
                            UMLPackage.eINSTANCE.getLiteralInteger_Value(), n);
                    editDomain.getEMFEditingDomain().getCommandStack().execute(command);

                }

            };
        }

        @Override
        public EditingSupport caseLiteralNull(LiteralNull object)
        {
            return new EditingSupport(this.viewer)
            {
                @Override
                public boolean canEdit(Object element)
                {
                    return false;
                }

                @Override
                public CellEditor getCellEditor(Object element)
                {
                    return null;
                }

                @Override
                public Object getValue(Object element)
                {
                    return null;
                }

                @Override
                public void setValue(Object element, Object value)
                {
                }
            };
        }

        @Override
        public EditingSupport caseLiteralString(LiteralString object)
        {
            return new TextEditingSupport(this.viewer)
            {
                @Override
                public Object getValue(Object element)
                {
                    String stringValue = ((LiteralString) element).getValue();
                    if (stringValue == null)
                    {
                        stringValue = "";
                    }
                    return stringValue;
                }

                @Override
                public void setValue(Object element, Object value)
                {
                    Command command = SetCommand.create(editDomain.getEMFEditingDomain(), element,
                            UMLPackage.eINSTANCE.getLiteralString_Value(), value);
                    editDomain.getEMFEditingDomain().getCommandStack().execute(command);

                }
            };
        }

        @Override
        public EditingSupport caseLiteralUnlimitedNatural(LiteralUnlimitedNatural object)
        {
            return new TextEditingSupport(this.viewer)
            {
                @Override
                public Object getValue(Object element)
                {
                    return String.valueOf(((LiteralUnlimitedNatural) element).getValue());
                }

                @Override
                public void setValue(Object element, Object value)
                {
                    int n = Integer.parseInt((String) value);
                    Command command = SetCommand.create(editDomain.getEMFEditingDomain(), element,
                            UMLPackage.eINSTANCE.getLiteralUnlimitedNatural_Value(), n);
                    editDomain.getEMFEditingDomain().getCommandStack().execute(command);

                }
            };
        }
    }

}
