package org.topcased.modeler.uml.sequencediagram.figures.decorations;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.geometry.Transform;

/**
 * A particular decoration used for the lost messages (MessageKind = LOST). An isArrowFilled is displayed with a circle.
 * The isArrowFilled may be filled (Synchronous message) or not (Asynchronous message).
 * 
 * Creation 12 juin 06
 * 
 * @author jlescot
 */
public class MessageLostDecoration extends Figure implements RotatableDecoration
{
    private static final PointList ARROW_TIP = new PointList();

    private static final Point CIRCLE_POINT_TIP = new Point(-1, 0);

    private static final Dimension CIRCLE_SIZE_TIP = new Dimension(2, 2);

    private static final int LINE_WIDTH = 1;

    static
    {
        ARROW_TIP.addPoint(-4, -1);
        ARROW_TIP.addPoint(-2, 0);
        ARROW_TIP.addPoint(-4, 1);
    }

    private Point location;

    private Transform transform;

    private PointList arrowPoints;

    private Point circlePoint;

    private Dimension circleSize;

    private double xScale = 1;

    private double yScale = 1;

    private final boolean isArrowFilled;

    /**
     * Constructor
     * 
     * @param isArrowFilled determine whether the arrow should be filled
     */
    public MessageLostDecoration(boolean isArrowFilled)
    {
        this.isArrowFilled = isArrowFilled;
        location = new Point();
        transform = new Transform();
    }

    /**
     * @see org.eclipse.draw2d.Figure#getBounds()
     */
    public Rectangle getBounds()
    {
        if (bounds == null)
        {
            updatePoints();

            bounds = new Rectangle(location, new Dimension(0, 0));

            bounds = bounds.getUnion(arrowPoints.getBounds());

            Point p = Point.SINGLETON;
            p.setLocation(circlePoint);
            p.x += circleSize.width / 2;
            p.y += circleSize.height / 2;
            bounds.union(p);
            p.x -= circleSize.width;
            p.y -= circleSize.height;
            bounds.union(p);

            bounds.expand(LINE_WIDTH / 2, LINE_WIDTH / 2);
        }
        return bounds;
    }

    /**
     * Sets the amount of scaling to be done along X and Y axes on the MessageLostDecoration's template.
     * 
     * @param x X scaling
     * @param y Y scaling
     */
    public void setScale(double x, double y)
    {
        arrowPoints = null;
        circlePoint = null;
        circleSize = null;
        bounds = null;
        transform.setScale(x, y);
        xScale = x;
        yScale = y;
    }

    /**
     * @see org.eclipse.draw2d.Figure#setLocation(org.eclipse.draw2d.geometry.Point)
     */
    public void setLocation(Point p)
    {
        arrowPoints = null;
        circlePoint = null;
        circleSize = null;
        bounds = null;
        location.setLocation(p);
        transform.setTranslation(p.x, p.y);
        super.setLocation(p);
    }

    /**
     * @see org.eclipse.draw2d.RotatableDecoration#setReferencePoint(org.eclipse.draw2d.geometry.Point)
     */
    public void setReferencePoint(Point p)
    {
        arrowPoints = null;
        circlePoint = null;
        circleSize = null;
        bounds = null;
        Point pt = Point.SINGLETON;
        pt.setLocation(p);
        pt.negate().translate(location);
        setRotation(Math.atan2(pt.y, pt.x));
    }

    /**
     * Sets the angle by which rotation is to be done on the PolygonDecoration.
     * 
     * @param angle Angle of rotation
     */
    public void setRotation(double angle)
    {
        arrowPoints = null;
        circlePoint = null;
        circleSize = null;
        bounds = null;
        transform.setRotation(angle);
    }

    /**
     * Update the points of the Decoration with the actual parameters
     */
    protected void updatePoints()
    {
        if (arrowPoints == null)
        {
            arrowPoints = new PointList();
            for (int i = 0; i < ARROW_TIP.size(); i++)
            {
                arrowPoints.addPoint(transform.getTransformed(ARROW_TIP.getPoint(i)));
            }
        }

        if (circlePoint == null)
        {
            circlePoint = new Point(transform.getTransformed(CIRCLE_POINT_TIP));
        }
        if (circleSize == null)
        {
            circleSize = new Dimension((int) xScale * CIRCLE_SIZE_TIP.width, (int) yScale * CIRCLE_SIZE_TIP.height);
        }
    }

    /**
     * @see org.eclipse.draw2d.Figure#paintFigure(org.eclipse.draw2d.Graphics)
     */
    protected void paintFigure(Graphics graphics)
    {
        updatePoints();

        graphics.pushState();
        graphics.setBackgroundColor(graphics.getForegroundColor());

        if (isArrowFilled)
        {
            graphics.fillPolygon(arrowPoints);
        }

        graphics.fillOval(circlePoint.x - circleSize.width / 2, circlePoint.y - circleSize.height / 2, circleSize.width, circleSize.height);

        graphics.popState();
    }

    /**
     * @see org.eclipse.draw2d.Figure#paintBorder(org.eclipse.draw2d.Graphics)
     */
    protected void paintBorder(Graphics graphics)
    {
        updatePoints();

        graphics.pushState();
        graphics.setLineWidth(LINE_WIDTH);

        if (!isArrowFilled)
        {
            graphics.drawPolyline(arrowPoints);
        }

        graphics.drawOval(circlePoint.x - circleSize.width / 2, circlePoint.y - circleSize.height / 2, circleSize.width, circleSize.height);

        graphics.popState();
    }

}
