/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies), Nicolas Lalevée (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.modeler.uml.sequencediagram.policies;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.UnexecutableCommand;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.DestructionEvent;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.InteractionFragment;
import org.eclipse.uml2.uml.Lifeline;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.topcased.draw2d.layout.PortConstraint;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy;
import org.topcased.modeler.uml.sequencediagram.commands.ExecutionChangeBoundsCommand;
import org.topcased.modeler.uml.sequencediagram.commands.creation.BehaviorExecutionSpecificationCreationCommand;
import org.topcased.modeler.uml.sequencediagram.commands.creation.DestructionEventCreationCommand;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.DestructionEventEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Edit policy that handles the executions and stop on lifeline
 * 
 * @author <a href="mailto:nicolas.lalevee@anyware-tech.com">Nicolas Lalevée</a>
 * 
 */
public class LifelineLayoutEditPolicy extends ModelerLayoutEditPolicy
{
    /**
     * Default contructor. <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated
     */
    public LifelineLayoutEditPolicy()
    {
        super();
    }

    /**
     * Create a stop or execution creation command
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getCreateCommand(org.eclipse.gef.EditDomain,
     *      org.topcased.modeler.di.model.GraphNode, org.topcased.modeler.di.model.GraphNode,
     *      org.eclipse.emf.ecore.EObject, org.eclipse.draw2d.geometry.Point, org.eclipse.draw2d.geometry.Dimension,
     *      int, java.util.List, boolean)
     */
    protected Command getCreateCommand(EditDomain domain, GraphNode newObject, GraphNode newParent, EObject newContainerParent, Point location, Dimension dim, int attach, List featuresList,
            boolean needModelUpdate)
    {
        Dimension dimension = dim;
        EObject obj = Utils.getElement(newObject);
        if (obj instanceof DestructionEvent)
        {
            dimension = new Dimension(DestructionEventEditPart.DEFAULT_WIDTH, DestructionEventEditPart.DEFAULT_HEIGHT);
            return new DestructionEventCreationCommand(domain, newObject, newParent, newContainerParent, location, dimension, featuresList, needModelUpdate);
        }
        else if (obj instanceof BehaviorExecutionSpecification)
        {
            location.x = (newParent.getSize().width - BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH) / 2;
            if (dimension != null)
            {
                dimension.width = BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH;
            }
            else
            {
                dimension = new Dimension(BehaviorExecutionSpecificationEditPart.DEFAULT_WIDTH, BehaviorExecutionSpecificationEditPart.DEFAULT_HEIGHT);
            }
            return new BehaviorExecutionSpecificationCreationCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
        }
        return super.getCreateCommand(domain, newObject, newParent, newContainerParent, location, dimension, attach, featuresList, needModelUpdate);
    }

    /**
     * Check if a destructionEvent can be associated with a given Lifeline. The Lifeline must be covered at least by one
     * MessageOccurrenceSpecification and no fragments should already be associated with a DestructionEvent
     * 
     * @param parent the parent EObject
     * @return true if a destructionEvent can be created
     */
    private boolean canCreateDestructionEvent(EObject parent)
    {
        boolean hasMessageOccurrenceSpecification = false;
        boolean hasNoDestructionEvent = true;

        if (parent instanceof Lifeline)
        {
            Iterator itFragments = ((Lifeline) parent).getCoveredBys().iterator();
            while (itFragments.hasNext())
            {
                InteractionFragment fragment = (InteractionFragment) itFragments.next();
                if (fragment instanceof MessageOccurrenceSpecification)
                {
                    hasMessageOccurrenceSpecification = true;
                    if (((MessageOccurrenceSpecification) fragment).getEvent() instanceof DestructionEvent)
                    {
                        hasNoDestructionEvent = false;
                    }
                }
            }
        }

        return hasMessageOccurrenceSpecification && hasNoDestructionEvent;
    }

    /**
     * Only execution and stop are allowed
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#isValid(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.ecore.EObject)
     */
    protected boolean isValid(EObject child, EObject parent)
    {
        return child != null && parent != null && (child instanceof BehaviorExecutionSpecification || (child instanceof DestructionEvent && canCreateDestructionEvent(parent)));
    }

    /**
     * The container is not the lifeline but the interaction
     * 
     * @see org.topcased.modeler.edit.policies.ModelerLayoutEditPolicy#getParentContainerEObject(GraphElement,
     *      DiagramElement)
     */
    protected EObject getParentContainerEObject(GraphElement parent, DiagramElement child)
    {
        Lifeline lifeline = (Lifeline) Utils.getElement(parent);
        EObject childEObject = Utils.getElement((GraphElement) child);
        if (childEObject instanceof BehaviorExecutionSpecification)
        {
            return lifeline.getInteraction();
        }
        else if (childEObject instanceof Event)
        {
            return lifeline.getInteraction().getNearestPackage();
        }

        return null;
    }

    /**
     * Mustn't move in the x direction
     * 
     * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getMoveChildrenCommand(org.eclipse.gef.Request)
     */
    protected Command getMoveChildrenCommand(Request request)
    {
        ChangeBoundsRequest req = (ChangeBoundsRequest) request;
        req.getMoveDelta().x = 0;
        return super.getMoveChildrenCommand(request);
    }

    /**
     * The resizing is forbidden in the 'width' way for the executions
     * 
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#getResizeChildrenCommand(org.eclipse.gef.requests.ChangeBoundsRequest)
     */
    protected Command getResizeChildrenCommand(ChangeBoundsRequest request)
    {
        switch (request.getResizeDirection())
        {
            case PositionConstants.EAST:
            case PositionConstants.WEST:
            case PositionConstants.SOUTH_EAST:
            case PositionConstants.SOUTH_WEST:
            case PositionConstants.NORTH_EAST:
            case PositionConstants.NORTH_WEST:
                return UnexecutableCommand.INSTANCE;
            default:
                return super.getResizeChildrenCommand(request);
        }
    }

    /**
     * Don't allow the resize if some message are present
     * 
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest,
     *      org.eclipse.gef.EditPart, java.lang.Object)
     */
    protected Command createChangeConstraintCommand(ChangeBoundsRequest request, EditPart child, Object constraint)
    {
        if (child instanceof BehaviorExecutionSpecificationEditPart)
        {
            int maxy = 0;
            Iterator itAnchors = ((GraphNode) child.getModel()).getAnchorage().iterator();
            while (itAnchors.hasNext())
            {
                GraphConnector connector = (GraphConnector) itAnchors.next();
                int y = connector.getPosition().y;
                if (y > maxy)
                {
                    maxy = y;
                }
            }

            if (((Rectangle) constraint).height < maxy)
            {
                return null;
            }
        }
        return super.createChangeConstraintCommand(request, child, constraint);
    }

    /**
     * Create custom command
     * 
     * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart,
     *      java.lang.Object)
     */
    protected Command createChangeConstraintCommand(EditPart child, Object constraint)
    {
        GraphNode node = (GraphNode) child.getModel();

        if (constraint instanceof PortConstraint)
        {
            return new ExecutionChangeBoundsCommand(node, (PortConstraint) constraint);
        }
        else if (isValidConstraint((GraphicalEditPart) getHost(), (Rectangle) constraint))
        {
            return new ExecutionChangeBoundsCommand(node, (Rectangle) constraint);
        }

        return null;
    }

}