// ------------------------------------------------
// $Id: SequenceDeleteModelObjectEditPolicy.java,v 1.1 2007-05-24 15:36:39 jako
// Exp $
// (c) Anyware Technologies 2007 www.anyware-tech.com
// ------------------------------------------------
package org.topcased.modeler.uml.sequencediagram.policies;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.uml2.uml.BehaviorExecutionSpecification;
import org.eclipse.uml2.uml.Event;
import org.eclipse.uml2.uml.ExecutionOccurrenceSpecification;
import org.eclipse.uml2.uml.GeneralOrdering;
import org.eclipse.uml2.uml.Message;
import org.eclipse.uml2.uml.MessageOccurrenceSpecification;
import org.eclipse.uml2.uml.OccurrenceSpecification;
import org.topcased.modeler.ModelerRequestConstants;
import org.topcased.modeler.commands.CompoundCommand;
import org.topcased.modeler.commands.DeleteModelCommand;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy;
import org.topcased.modeler.editor.MixedEditDomain;
import org.topcased.modeler.editor.Modeler;
import org.topcased.modeler.uml.sequencediagram.edit.BehaviorExecutionSpecificationEditPart;
import org.topcased.modeler.uml.sequencediagram.edit.MessageEditPart;
import org.topcased.modeler.utils.Utils;

/**
 * Custom policy for handling the deletion of Message and
 * BehavioExecutionSpecification instances.
 * 
 * @author jrme
 */
public class SequenceDeleteModelObjectEditPolicy extends DeleteModelObjectEditPolicy
{
    /**
     * This policy understands delete requests for {@link MessageEditPart} and
     * {@link BehaviorExecutionSpecificationEditPart}.
     * 
     * @see org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy#understandsRequest(org.eclipse.gef.Request)
     */
    @Override
    public boolean understandsRequest(Request request)
    {
        if (!super.understandsRequest(request))
        {
            return false;
        }
        if (ModelerRequestConstants.REQ_DELETE_MODEL_OBJECT.equals(request.getType()))
        {
            if (getHost() instanceof MessageEditPart || getHost() instanceof BehaviorExecutionSpecificationEditPart)
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Delegates the command creation to
     * {@link #getCommandForMessage(MixedEditDomain, Diagrams, Message)} when
     * the host is a {@link MessageEditPart} and
     * {@link #getCommandForBehaviorExecution(MixedEditDomain, Diagrams, BehaviorExecutionSpecification)}
     * when it is a {@link BehaviorExecutionSpecificationEditPart}.
     * 
     * We need to pass the domain and diagrams objects to those methods since
     * they're needed to create instances of {@link DeleteModelCommand}.
     * 
     * @see org.topcased.modeler.edit.policies.DeleteModelObjectEditPolicy#getCommand(org.eclipse.gef.Request)
     */
    @Override
    public Command getCommand(Request request)
    {
        if (!understandsRequest(request))
        {
            return null;
        }

        GraphElement element = (GraphElement) getHost().getModel();
        EObject model = Utils.getElement(element);

        assert (getHost().getViewer().getEditDomain() instanceof MixedEditDomain);
        MixedEditDomain domain = ((MixedEditDomain) getHost().getViewer().getEditDomain());
        Diagrams diagrams = ((Modeler) domain.getEditorPart()).getDiagrams();

        if (model != null && domain != null && diagrams != null)
        {
            if (getHost() instanceof MessageEditPart)
            {
                assert (model instanceof Message);
                return getCommandForMessage(domain, diagrams, (Message) model);
            }

            if (getHost() instanceof BehaviorExecutionSpecificationEditPart)
            {
                assert (model instanceof BehaviorExecutionSpecification);
                return getCommandForBehaviorExecution(domain, diagrams, (BehaviorExecutionSpecification) model);
            }
        }

        return null;
    }

    /**
     * Computes the {@link OccurrenceSpecification} to delete along with the
     * behavior.
     * 
     * @param behaviorExecutionSpecification the
     *            {@link BehaviorExecutionSpecification} to delete.
     * @return {@link Set} of {@link OccurrenceSpecification} the occurrences
     *         that should be deleted with the
     *         {@link BehaviorExecutionSpecification} given in parameter.
     */
    private Set<OccurrenceSpecification> getOccurrencesToDeleteFromBehavior(
            BehaviorExecutionSpecification behaviorExecutionSpecification)
    {
        OccurrenceSpecification startOccurrence = behaviorExecutionSpecification.getStart();
        OccurrenceSpecification endOccurrence = behaviorExecutionSpecification.getFinish();

        HashSet<OccurrenceSpecification> occurrencesToDelete = new HashSet<OccurrenceSpecification>();
        HashSet<OccurrenceSpecification> occurrencesAfterStart = new HashSet<OccurrenceSpecification>();

        // this could happen after all messages have been deleted
        if (startOccurrence != null)
        {
            occurrencesToDelete.add(startOccurrence);
            for (GeneralOrdering ordering : startOccurrence.getToAfters())
            {
                occurrencesAfterStart.add(ordering.getAfter());
            }
        }

        // this happens when no result message is sent back.
        if (endOccurrence != null)
        {
            occurrencesToDelete.add(endOccurrence);
            for (GeneralOrdering ordering : endOccurrence.getToBefores())
            {
                if (occurrencesAfterStart.contains(ordering.getBefore()))
                {
                    occurrencesToDelete.add(ordering.getBefore());
                }
            }
        }
        return occurrencesToDelete;
    }

    /**
     * Returns the command for deleting a behavior in the model.
     * 
     * @param domain
     * @param diagrams
     * @param message
     * @return {@link Command} the command for deleting a behavior in the model.
     */
    private Command getCommandForBehaviorExecution(MixedEditDomain domain, Diagrams diagrams,
            BehaviorExecutionSpecification behaviorExecutionSpecification)
    {
        CompoundCommand command = new CompoundCommand();

        command.add(new DeleteModelCommand(domain, diagrams, behaviorExecutionSpecification));

        for (OccurrenceSpecification occurrence : getOccurrencesToDeleteFromBehavior(behaviorExecutionSpecification))
        {
            if (occurrence instanceof MessageOccurrenceSpecification)
            {
                MessageOccurrenceSpecification messageReceiveOccurrence = (MessageOccurrenceSpecification) occurrence;
                Message message = messageReceiveOccurrence.getMessage();
                command.add(getCommandForMessage(domain, diagrams, message));
            }
            else if (occurrence instanceof ExecutionOccurrenceSpecification)
            {
                ExecutionOccurrenceSpecification executionOccurrence = (ExecutionOccurrenceSpecification) occurrence;
                command.add(new DeleteModelCommand(domain, diagrams, executionOccurrence));
            }
            else
            {
                assert (false);
            }

        }

        return command;
    }

    /**
     * Returns the command for deleting a message in the model.
     * 
     * It deletes the message and the SendEvent and ReceiveEvent associated, as
     * well as the call event if there is one.
     * 
     * @param domain
     * @param diagrams
     * @param message
     * @return {@link Command} the command for deleting a message in the model.
     */
    private Command getCommandForMessage(MixedEditDomain domain, Diagrams diagrams, Message message)
    {
        CompoundCommand command = new CompoundCommand();

        DeleteModelCommand deleteSendEventCommand = new DeleteModelCommand(domain, diagrams, message.getSendEvent());
        DeleteModelCommand deleteReceiveEventCommand = new DeleteModelCommand(domain, diagrams,
                message.getReceiveEvent());
        DeleteModelCommand deleteMessageCommand = new DeleteModelCommand(domain, diagrams, message);

        // this deals with deleting the events associated with the message.
        // 
        Event sendEvent = null;
        if (message.getSendEvent() instanceof MessageOccurrenceSpecification)
        {
            MessageOccurrenceSpecification messageOccurrence = (MessageOccurrenceSpecification) message.getSendEvent();
            sendEvent = messageOccurrence.getEvent();
            if (sendEvent != null)
            {
                command.add(new DeleteModelCommand(domain, diagrams, sendEvent));
            }
        }

        if (message.getReceiveEvent() instanceof MessageOccurrenceSpecification)
        {
            MessageOccurrenceSpecification messageOccurrence = (MessageOccurrenceSpecification) message.getReceiveEvent();
            Event event = messageOccurrence.getEvent();
            if (event != null && event != sendEvent)
            {
                command.add(new DeleteModelCommand(domain, diagrams, event));
            }

        }

        // if (message.getReceiveEvent() instanceof
        // MessageOccurrenceSpecification)
        // {
        // MessageOccurrenceSpecification messageOccurrence =
        // (MessageOccurrenceSpecification) message.getReceiveEvent();
        // Event event = messageOccurrence.getEvent();
        // if (event != null)
        // {
        // command.add(new DeleteModelCommand(domain, diagrams, event));
        // }
        // }

        command.add(deleteReceiveEventCommand);
        command.add(deleteSendEventCommand);
        command.add(deleteMessageCommand);
        return command;
    }

}
