/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.commands;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditDomain;
import org.eclipse.jface.window.Window;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.UMLFactory;
import org.topcased.modeler.commands.CreateGraphNodeCommand;
import org.topcased.modeler.commands.IDialogCommand;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.ICreationUtils;
import org.topcased.modeler.uml.statemachinediagram.STMPlugin;
import org.topcased.modeler.uml.statemachinediagram.dialogs.ReferenceStateMachineDialog;
import org.topcased.modeler.utils.LabelHelper;
import org.topcased.modeler.utils.Utils;

/**
 * <b>SubmachineState creation in UML</b><br>
 * This command is used to reference the StateMachine that is associated with the State.
 * 
 * Creation 28 juin 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class CreateSubmachineStateCommand extends CreateGraphNodeCommand implements IDialogCommand
{
    /** The StateMachine */
    private StateMachine stateMachine;

    /** The name of the stateMachine to create */
    private String stateMachineName;

    /** Indicate if the stateMachine had to be created */
    private boolean createStateMachine;

    /**
     * The Constructor
     * 
     * @param domain the EditDomain (used to init the name)
     * @param creation the CreationUtils class (used to create the model object)
     * @param newObject the node to create
     * @param newParent the parent node
     * @param location the location of the created node
     * @param dimension the size of the created node
     * @param pos
     * @param needModelUpdate <code>true</code> if the model must be modified during this command, <code>false</code> if objects already exist in the mode.
     */
    public CreateSubmachineStateCommand(EditDomain domain, ICreationUtils creation, GraphNode newObject, GraphNode newParent, Point location, Dimension dimension, int pos, boolean needModelUpdate)
    {
        super(domain, newObject, newParent, Utils.getElement(newParent), location, dimension, pos, null, needModelUpdate);
    }

    /**
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#redoModel()
     */
    protected void redoModel()
    {
        // create the newly created model objects or just reference the selected
        // Component.
        if (createStateMachine)
        {
            stateMachine = UMLFactory.eINSTANCE.createStateMachine();
            stateMachine.setName(stateMachineName);
            ((Region) getParentEObject()).getNearestPackage().getPackagedElements().add(stateMachine);
        }

        State state = (State) getChildEObject();
        // Initialize the name of the SubmachineState
        String curName = LabelHelper.INSTANCE.getName(getEditDomain(), state);
        if (curName == null || "".equals(curName))
        {
            LabelHelper.INSTANCE.initName(getEditDomain(), getParentEObject(), state);
        }

        ((Region) getParentEObject()).getSubvertices().add(state);
        state.setSubmachine(stateMachine);
    }

    /**
     * @see org.topcased.modeler.commands.CreateGraphNodeCommand#undoModel()
     */
    protected void undoModel()
    {
        if (createStateMachine)
        {
            ((Region) getParentEObject()).getNearestPackage().getPackagedElements().remove(stateMachine);
        }
        ((Region) getParentEObject()).getSubvertices().remove(getChildEObject());
    }

    // private void updateFeatures()
    // {
    // // a counter that is used to place the ports at the left
    // int pos = 0;
    //
    // for (Iterator iter = ((Subcomponent)
    // getChildEObject()).getXAllFeature().iterator(); iter.hasNext();)
    // {
    // // retrieve the features to display
    // Feature feature = (Feature) iter.next();
    //
    // if (feature instanceof Port || feature instanceof ComponentAccess)
    // {
    // GraphNode implNode = (GraphNode)
    // creationUtils.createGraphElement(feature);
    // implNode.setPosition(new Point(1, 10 + (20 * pos)));
    // DIUtils.setProperty(implNode, ModelerPropertyConstants.PORT_POSITION,
    // StringConverter.asString(PositionConstants.LEFT));
    // getChildNode().getContained().add(implNode);
    // pos++;
    // }
    // }
    // if (getChildNode().getSize() != null && getChildNode().getSize().height <
    // 30 + pos * 20)
    // {
    // getChildNode().setSize(new Dimension(getChildNode().getSize().width, 30 +
    // pos * 20));
    // }
    // }

    /**
     * @see org.topcased.modeler.commands.IDialogCommand#openDialog()
     */
    public boolean openDialog()
    {
        // Display a dialog that allow the user to reference the StateMachine to
        // associate with the State
        ReferenceStateMachineDialog dialog = new ReferenceStateMachineDialog((Region) getParentEObject(), STMPlugin.getActiveWorkbenchShell());
        if (dialog.open() == Window.OK)
        {
            createStateMachine = dialog.getCreateStateMachine();
            stateMachine = dialog.getNewStateMachine();
            stateMachineName = dialog.getNewStateMachineName();

            return true;
        }

        return false;
    }

}
