/*******************************************************************************
 * Copyright (c) 2006 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.dialogs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Region;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.edit.providers.UMLItemProviderAdapterFactory;
import org.topcased.facilities.dialogs.ITopcasedDialogConstants;

/**
 * A Dialog used to reference a StateMachine at the creation of a SubMachineState
 * 
 * Creation 28 juin 06
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ReferenceStateMachineDialog extends Dialog implements ITopcasedDialogConstants
{
    // The Region that contains the SubmachineState
    private Region region;

    // Store the stateMachine that was choosed
    private StateMachine stateMachine;

    private String stateMachineName;

    private boolean createStateMachine;

    // SWT Objects
    private Button existingStateMachineBt;

    private Button newStateMachineBt;

    private Combo stateMachineCb;

    private ComboViewer stateMachineCbViewer;

    private Text stateMachineFd;

    /**
     * Create the Dialog window to select a componentClassifier to associate with the current subComponent.
     * 
     * @param region the region that contains the State
     * @param parentShell the parent Shell
     */
    public ReferenceStateMachineDialog(Region region, Shell parentShell)
    {
        super(parentShell);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        setBlockOnOpen(true);
        this.region = region;
    }

    /**
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell newShell)
    {
        newShell.setText("StateMachine Reference");
        newShell.setMinimumSize(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT);

        super.configureShell(newShell);
    }

    /**
     * Create the Dialog area
     * 
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent)
    {
        Composite dialogComposite = (Composite) super.createDialogArea(parent);
        createDialogComposite(dialogComposite);
        createMainGroup(dialogComposite);
        hookListeners();
        loadData();
        return dialogComposite;
    }

    /**
     * Create the Dialog Contents
     * 
     * @param composite
     */
    private void createDialogComposite(Composite composite)
    {
        GridLayout dialogLayout = new GridLayout();
        GridData dialogLayoutData = new GridData(GridData.FILL_BOTH);
        composite.setLayout(dialogLayout);
        composite.setLayoutData(dialogLayoutData);
    }

    /**
     * Creates the SWT Objects of the Dialog
     * 
     * @param parent the parent Composite
     */
    protected void createMainGroup(Composite parent)
    {
        Composite composite = new Composite(parent, SWT.NONE);

        // Add layout on composite
        GridLayout layout = new GridLayout();
        GridData layoutData = new GridData(GridData.FILL_BOTH);
        composite.setLayout(layout);
        composite.setLayoutData(layoutData);

        existingStateMachineBt = new Button(composite, SWT.RADIO);
        existingStateMachineBt.setText("Choose an existing StateMachine to reference from the SubMachineState :");

        stateMachineCb = new Combo(composite, SWT.NONE);
        stateMachineCb.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        stateMachineCbViewer = new ComboViewer(stateMachineCb);

        Label separationLbl = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
        separationLbl.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        newStateMachineBt = new Button(composite, SWT.RADIO);
        newStateMachineBt.setText("Create a new StateMachine to reference from the SubmachineState :");

        stateMachineFd = new Text(composite, SWT.BORDER);
        stateMachineFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    }

    /**
     * Initialize widgets value
     */
    private void loadData()
    {
        stateMachineCbViewer.setContentProvider(new StateMachineContentProvider());
        stateMachineCbViewer.setLabelProvider(new AdapterFactoryLabelProvider(new UMLItemProviderAdapterFactory()));
        // Retrieve all the available StateMachines
        stateMachineCbViewer.setInput(region.getNearestPackage().getPackagedElements());

        existingStateMachineBt.setSelection(true);
        newStateMachineBt.setSelection(false);
        stateMachineFd.setEnabled(false);
    }

    /**
     * Hook the listeners
     */
    private void hookListeners()
    {
        existingStateMachineBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (existingStateMachineBt.getSelection())
                {
                    stateMachineCb.setEnabled(true);
                    stateMachineFd.setEnabled(false);
                }
            }
        });

        newStateMachineBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (newStateMachineBt.getSelection())
                {
                    stateMachineCb.setEnabled(false);
                    stateMachineFd.setEnabled(true);
                }
            }
        });

        stateMachineCbViewer.addSelectionChangedListener(new ISelectionChangedListener()
        {
            // This ensures that we handle selections correctly.
            //
            public void selectionChanged(SelectionChangedEvent event)
            {
                if (event.getSelection() instanceof IStructuredSelection)
                {
                    stateMachine = (StateMachine) ((IStructuredSelection) event.getSelection()).getFirstElement();
                }
            }
        });
    }

    /**
     * Return the new StateMachine
     * 
     * @return a StateMachine
     */
    public StateMachine getNewStateMachine()
    {
        return stateMachine;
    }

    /**
     * Return the name of the StateMachine to create
     * 
     * @return String
     */
    public String getNewStateMachineName()
    {
        return stateMachineName;
    }

    /**
     * Return true if a StateMachine should be created
     * 
     * @return boolean
     */
    public boolean getCreateStateMachine()
    {
        return createStateMachine;
    }

    /**
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    protected void okPressed()
    {
        createStateMachine = newStateMachineBt.getSelection();
        stateMachineName = stateMachineFd.getText();

        super.okPressed();
    }

    /**
     * This Content Provider displays contents of StateMachines. For the model, this provider delegates contents
     * computing to the model content provider. <br>
     * 
     * Creation 28 juin 06
     * 
     * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
     */
    public class StateMachineContentProvider implements IStructuredContentProvider
    {
        /**
         * Constructor
         */
        public StateMachineContentProvider()
        {
            // Do nothing
        }

        /**
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement)
        {
            if (inputElement instanceof EList)
            {
                if (!((EList) inputElement).isEmpty())
                {
                    List result = new ArrayList();
                    for (Iterator iter = ((EList) inputElement).iterator(); iter.hasNext();)
                    {
                        PackageableElement packElt = (PackageableElement) iter.next();

                        // filter the StateMachine model objects only
                        if (packElt instanceof StateMachine)
                        {
                            result.add(packElt);
                        }
                    }
                    return result.toArray();
                }
            }
            return new Object[0];
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose()
        {
            // Do nothing
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
         *      java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
        {
            // Do nothing
        }

    }

}
