/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.uml.statemachinediagram.internal.properties.sections.graphics;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.topcased.modeler.commands.ChangeBoundsCommand;
import org.topcased.modeler.commands.ChangeGraphElementPresentationCommand;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.DiagramInterchangePackage;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.editor.MixedEditDomain;

/**
 * The section used to change the orientation of a Fork (or Join) element in the graphics tab.<br>
 * 
 * Creation 2 mars 07
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class ForkAndJoinOrientationSection extends AbstractPropertySection
{
    /**
     * The checkButton control for the section.
     */
    private Button checkButton;

    /**
     * The current selected diagram element or the first graphical element in the selection when multiple objects are
     * selected.
     */
    private DiagramElement diagElt;

    /**
     * The current select EditPart
     */
    private EditPart selectedEditPart;

    /**
     * Listener for the model notifications
     */
    private Adapter modelListener = new EContentAdapter()
    {
        public void notifyChanged(Notification notification)
        {
            super.notifyChanged(notification);
            handleModelChanged(notification);
        }
    };

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#setInput(org.eclipse.ui.IWorkbenchPart,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void setInput(IWorkbenchPart part, ISelection selection)
    {
        super.setInput(part, selection);
        Assert.isTrue(selection instanceof IStructuredSelection);
        if (((IStructuredSelection) selection).getFirstElement() instanceof EditPart)
        {
            selectedEditPart = (EditPart) ((IStructuredSelection) selection).getFirstElement();
            Object elt = selectedEditPart.getModel();
            if (elt instanceof DiagramElement)
            {
                removeListener();
                diagElt = (DiagramElement) elt;
                addListener();
            }
        }
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#aboutToBeShown()
     */
    public void aboutToBeShown()
    {
        addListener();
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#aboutToBeHidden()
     */
    public void aboutToBeHidden()
    {
        removeListener();
    }

    /**
     * Remove the listener
     */
    protected void removeListener()
    {
        if (diagElt == null)
        {
            return;
        }
        if (diagElt.eAdapters().contains(getModelListener()))
        {
            diagElt.eAdapters().remove(getModelListener());
        }
    }

    /**
     * Add the listener
     */
    protected void addListener()
    {
        if (diagElt == null)
        {
            return;
        }
        if (!diagElt.eAdapters().contains(getModelListener()))
        {
            diagElt.eAdapters().add(getModelListener());
        }
    }

    /**
     * @return the diagElt
     */
    protected DiagramElement getDiagElt()
    {
        return diagElt;
    }

    /**
     * @return the selectedEditPart
     */
    protected EditPart getSelectedEditPart()
    {
        return selectedEditPart;
    }

    /**
     * Return the model listener
     * 
     * @return Adapter
     */
    protected Adapter getModelListener()
    {
        return modelListener;
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#createControls(org.eclipse.swt.widgets.Composite,
     *      org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
     */
    public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage)
    {
        super.createControls(parent, tabbedPropertySheetPage);
        Composite composite = getWidgetFactory().createFlatFormComposite(parent);
        FormData data;

        checkButton = getWidgetFactory().createButton(composite, "isVertical", SWT.CHECK);
        data = new FormData();
        data.left = new FormAttachment(0, 0);
        data.right = new FormAttachment(100, 0);
        data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
        checkButton.setLayoutData(data);

        checkButton.addSelectionListener(new SelectionListener()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleCheckButtonModified();
            }

            public void widgetDefaultSelected(SelectionEvent e)
            {
                // Do nothing
            }
        });

        refresh();
    }

    /**
     * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#refresh()
     */
    public void refresh()
    {
        super.refresh();

        if (!checkButton.isDisposed())
        {
            // Initialize the orientation
            if (getSelectedEditPart() != null)
            {
                checkButton.setSelection("default".equals(((GraphElement) ((AbstractEditPart) getSelectedEditPart()).getModel()).getSemanticModel().getPresentation()));
            }
        }
    }

    /**
     * Handle the button modified event.
     */
    protected void handleCheckButtonModified()
    {
        MixedEditDomain mixedEditDomain = (MixedEditDomain) getPart().getAdapter(MixedEditDomain.class);
        CompoundCommand compoundCommand = new CompoundCommand();
        compoundCommand.add(new ChangeGraphElementPresentationCommand((GraphElement) getDiagElt(), checkButton.getSelection() ? "default" : "horizontal"));
        int newX = ((GraphNode) getDiagElt()).getPosition().x + ((GraphNode) getDiagElt()).getSize().width / 2 - ((GraphNode) getDiagElt()).getSize().height / 2;
        int newY = ((GraphNode) getDiagElt()).getPosition().y + ((GraphNode) getDiagElt()).getSize().height / 2 - ((GraphNode) getDiagElt()).getSize().width / 2;
        compoundCommand.add(new ChangeBoundsCommand((GraphNode) getDiagElt(), new Rectangle(new Point(newX, newY), ((GraphNode) getDiagElt()).getSize().getTransposed())));
        mixedEditDomain.getCommandStack().execute(compoundCommand);
    }

    /**
     * This method is called when an event occured on the model objects
     * 
     * @param msg the event notification
     */
    protected void handleModelChanged(Notification msg)
    {
        Object notifier = msg.getNotifier();
        if (notifier.equals(getDiagElt()) && msg.getFeatureID(getDiagElt().getClass()) == DiagramInterchangePackage.SEMANTIC_MODEL_BRIDGE__PRESENTATION)
        {
            refresh();
        }
    }

}
