/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/
package org.topcased.modeler.utils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.model.management.IAnnotationConstants;
import org.topcased.model.management.UuidHelper;

/**
 * Provides utilies to manage resources <br>
 * creation : 8 juin 2005
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 */
public final class ResourceUtils
{

    private static final String[] READ_ONLY_RESOURCE_SCHEME = {"pathmap"};

    private static Map registeredResources = new HashMap();

    /**
     * Hidden constructor
     */
    private ResourceUtils()
    {
        // Do nothing, just hide
    }

    /**
     * Return true if the given resource is read only
     * 
     * @param resource the resource to check
     * @return <code>true</code> if the resource cannot be written
     */
    public static boolean isReadOnly(Resource resource)
    {
        URI uri = resource.getURI();
        Boolean isReadOnly = (Boolean) registeredResources.get(uri);

        if (isReadOnly == null)
        {
            isReadOnly = Boolean.valueOf(isSchemeReadOnly(uri.scheme()) || isPluginModel(uri));

            registeredResources.put(uri, isReadOnly);
        }
        return isReadOnly.booleanValue();
    }

    /**
     * Check if the URI points to a plugin
     * 
     * @param uri the URI
     * @return <code>true</code> if the resource is in a plugin
     */
    private static boolean isPluginModel(URI uri)
    {
        return uri.toString().startsWith("platform:/plugin");
    }

    /**
     * Check if the scheme represents an unwriteable protocol
     * 
     * @param scheme the scheme
     * @return <code>true</code> if the scheme cannot be written
     */
    private static boolean isSchemeReadOnly(String scheme)
    {
        return Arrays.asList(READ_ONLY_RESOURCE_SCHEME).contains(scheme);
    }

    /**
     * Return true if the given resource is the model resource
     * 
     * @param resource
     * @return <code>true</code> if it is the main model resource
     */
    public static boolean isModelResource(Resource resource)
    {
        return resource.getResourceSet().getResources().indexOf(resource) == 1;
    }

    /**
     * Add UUID on elements : elements can only be modified if the class
     * inherits from EModelElement
     * 
     * @param modelObject the model to modify
     */
    public static void addUUID(EObject modelObject)
    {
        // Add UUID on this object
        addUUIDForEObject(modelObject);

        // Add UUID on children
        Iterator itChildren = modelObject.eAllContents();
        while (itChildren.hasNext())
        {
            addUUIDForEObject((EObject) itChildren.next());
        }
    }

    /**
     * Add UUID on elements : elements can only be modified if the class
     * inherits from EModelElement
     * 
     * @param modelObject the model to modify
     */
    private static void addUUIDForEObject(EObject modelObject)
    {
        if (modelObject instanceof EModelElement && !(modelObject instanceof EAnnotation))
        {
            // Create Helper
            UuidHelper helper = new UuidHelper();
            helper.initUUID((EModelElement) modelObject, EcoreUtil.generateUUID());
        }
    }
}
