/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies), Thomas Friol (Anyware
 * Technologies) - initial API and implementation
 ******************************************************************************/
package org.topcased.modeler.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.RangeModel;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.topcased.modeler.ModelerPropertyConstants;
import org.topcased.modeler.di.model.Diagram;
import org.topcased.modeler.di.model.DiagramElement;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.EMFSemanticModelBridge;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;
import org.topcased.modeler.di.model.GraphNode;
import org.topcased.modeler.di.model.Property;
import org.topcased.modeler.di.model.SimpleSemanticModelElement;
import org.topcased.modeler.di.model.util.DIUtils;
import org.topcased.modeler.diagrams.model.Diagrams;
import org.topcased.modeler.diagrams.model.util.DiagramsUtils;
import org.topcased.modeler.evaluator.EvaluatorException;
import org.topcased.modeler.evaluator.extension.EvaluatorsManager;
import org.topcased.modeler.graphconf.Constraint;
import org.topcased.modeler.graphconf.DiagramGraphConf;
import org.topcased.modeler.graphconf.EdgeGraphConf;
import org.topcased.modeler.graphconf.NodeGraphConf;
import org.topcased.modeler.internal.ModelerPlugin;

/**
 * Helper class to manage the graph model. <br>
 * creation : 10 d�c. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public final class Utils
{
    /**
     * Constructor
     */
    private Utils()
    {
        // Do nothing
    }

    /**
     * Returns the EList associated with the given feature for the given EMF object.
     * 
     * @param owner The EMF Object
     * @param feat The list feature
     * @return the EList or <code>null</code> if the feature cannot be found or it is not a multiple feature.
     */
    public static EList getOwnerList(EObject owner, EStructuralFeature feat)
    {
        return owner.eClass().getEAllStructuralFeatures().contains(feat) && feat.isMany() ? (EList) owner.eGet(feat) : null;
    }

    /**
     * Returns the model object linked with the given graph element (If the bridge is an EMFSemanticModelBridge)
     * 
     * @param graphElt the graph element
     * @return the model object associated with the given graph element or <code>null</code> if unable to find a model
     *         object.
     */
    public static EObject getElement(GraphElement graphElt)
    {
        if (graphElt != null && (graphElt.getSemanticModel() instanceof EMFSemanticModelBridge))
        {
            return ((EMFSemanticModelBridge) graphElt.getSemanticModel()).getElement();
        }

        return null;
    }

    /**
     * Returns the GraphElement child associated with the given model object
     * 
     * @param container the graph container
     * @param model the model EObject
     * @return The GraphElement associated with the model object or <code>null</code> if no child can be found.
     */
    public static GraphElement getGraphElement(GraphElement container, EObject model)
    {
        return getGraphElement(container, model, false);
    }

    /**
     * Returns the GraphElement child associated with the given model object
     * 
     * @param container the graph container
     * @param model the model EObject
     * @param deepSearch a boolean indicating if the search process must be recursive
     * @return The GraphElement associated with the model object or <code>null</code> if no child can be found.
     */
    public static GraphElement getGraphElement(GraphElement container, EObject model, boolean deepSearch)
    {
        ListIterator it = container.getContained().listIterator();
        while (it.hasNext())
        {
            DiagramElement diagramElt = (DiagramElement) it.next();
            if (diagramElt instanceof GraphElement)
            {
                GraphElement graphElt = (GraphElement) diagramElt;
                EObject object = getElement(graphElt);
                if (object != null && object.equals(model))
                {
                    return graphElt;
                }
                if (deepSearch)
                {
                    GraphElement result = getGraphElement(graphElt, model, deepSearch);
                    if (result != null)
                    {
                        return result;
                    }
                }
            }
        }

        return null;
    }

    /**
     * Returns the GraphElement children associated with the given model object
     * 
     * @param container the graph container
     * @param model the model EObject
     * @param deepSearch a boolean indicating if the search process must be recursive
     * @return The list of GraphElements associated with the model object or an Empty list if no child can be found.
     */
    public static List getGraphElements(GraphElement container, EObject model, boolean deepSearch)
    {
        // use this method in other search algorithms
        List children = new ArrayList();
        ListIterator it = container.getContained().listIterator();
        while (it.hasNext())
        {
            DiagramElement diagramElt = (DiagramElement) it.next();
            if (diagramElt instanceof GraphElement)
            {
                GraphElement graphElt = (GraphElement) diagramElt;
                EObject object = getElement(graphElt);
                if (object != null && object.equals(model))
                {
                    children.add(graphElt);
                }
                if (deepSearch)
                {
                    GraphElement result = getGraphElement(graphElt, model, deepSearch);
                    if (result != null)
                    {
                        children.add(result);
                    }
                }
            }
        }

        return children;
    }

    /**
     * Returns the graphElements associated with the given EObject in all the diagrams
     * 
     * @param diagrams the Diagrams object
     * @param model the searched EObject
     * @return the List of associated GraphElements
     */
    public static GraphElement[] getGraphElements(Diagrams diagrams, EObject model)
    {
        List results = new ArrayList();

        Iterator it = DiagramsUtils.findAllDiagrams(diagrams).iterator();
        while (it.hasNext())
        {
            Diagram diagram = (Diagram) it.next();
            GraphElement elt = getGraphElement(diagram, model, true);
            if (elt != null)
            {
                results.add(elt);
            }
        }

        return (GraphElement[]) results.toArray(new GraphElement[results.size()]);
    }

    /**
     * Returns the type info associated with the given graph element (If the bridge is a SimpleSemanticModelBridge)
     * 
     * @param graphElt the graph element
     * @return the type info associated with the given graph element or <code>null</code> if unable to find a type
     *         info (it is not a SimpleSemanticModelBridge).
     */
    public static String getTypeInfo(GraphElement graphElt)
    {
        if (graphElt != null && (graphElt.getSemanticModel() instanceof SimpleSemanticModelElement))
        {
            return ((SimpleSemanticModelElement) graphElt.getSemanticModel()).getTypeInfo();
        }

        return null;
    }

    /**
     * Creates a graph node NOT linked with a model object
     * 
     * @param type the identifier of the node
     * @return the graph node
     */
    public static GraphNode createGraphNode(String type)
    {
        return createGraphNode(type, "default");
    }

    /**
     * Creates a graph node NOT linked with a model object
     * 
     * @param type the identifier of the node
     * @param presentation the presentation of the graphical element
     * @return the graph node
     */
    public static GraphNode createGraphNode(String type, String presentation)
    {
        GraphNode node = DiagramInterchangeFactory.eINSTANCE.createGraphNode();
        SimpleSemanticModelElement element = DiagramInterchangeFactory.eINSTANCE.createSimpleSemanticModelElement();
        element.setTypeInfo(type);
        element.setPresentation(presentation);
        node.setSemanticModel(element);
        return node;
    }

    /**
     * Creates a graph edge NOT linked with a model object
     * 
     * @param type the identifier of the edge
     * @return the graph edge
     */
    public static GraphEdge createGraphEdge(String type)
    {
        return createGraphEdge(type, "default");
    }

    /**
     * Creates a graph edge NOT linked with a model object
     * 
     * @param type the identifier of the edge
     * @param presentation the presentation of the graphical element
     * @return the graph edge
     */
    public static GraphEdge createGraphEdge(String type, String presentation)
    {
        GraphEdge edge = DiagramInterchangeFactory.eINSTANCE.createGraphEdge();
        SimpleSemanticModelElement element = DiagramInterchangeFactory.eINSTANCE.createSimpleSemanticModelElement();
        element.setTypeInfo(type);
        element.setPresentation(presentation);
        edge.setSemanticModel(element);
        return edge;
    }

    /**
     * Returns the background Color of the Figure
     * 
     * @param graphElement
     * @return Color
     */
    public static Color getBackgroundColor(GraphElement graphElement)
    {
        return getColor(graphElement, ModelerPropertyConstants.BACKGROUND_COLOR);
    }

    /**
     * Returns the foreground Color of the Figure
     * 
     * @param graphElement
     * @return Color
     */
    public static Color getForegroundColor(GraphElement graphElement)
    {
        return getColor(graphElement, ModelerPropertyConstants.FOREGROUND_COLOR);
    }

    /**
     * Get the Color corresponding to a Property with a particular key
     * 
     * @param graphElement
     * @param key ("backgroundColor" or "foregroundColor")
     * @return Color
     */
    private static Color getColor(GraphElement graphElement, String key)
    {
        Property property = DIUtils.getProperty(graphElement, key);
        if (property != null)
        {
            RGB colorRGB = StringConverter.asRGB(property.getValue());
            if (colorRGB != null)
            {
                Color color = JFaceResources.getColorRegistry().get(StringConverter.asString(colorRGB));
                if (color == null)
                {
                    JFaceResources.getColorRegistry().put(StringConverter.asString(colorRGB), colorRGB);
                    color = JFaceResources.getColorRegistry().get(StringConverter.asString(colorRGB));
                }
                return color;
            }
        }
        return null;
    }

    /**
     * Get the Font corresponding to a DiagramElement
     * 
     * @param diagElement
     * @return Font
     */
    public static Font getFont(DiagramElement diagElement)
    {
        Property property = DIUtils.getProperty(diagElement, ModelerPropertyConstants.FONT);
        if (property != null)
        {
            FontData fontData = StringConverter.asFontData(property.getValue());
            if (fontData != null)
            {
                FontData[] fontDataList = new FontData[1];
                fontDataList[0] = fontData;
                JFaceResources.getFontRegistry().put(StringConverter.asString(fontData), fontDataList);
                // the get on a font always return a font even if this font is
                // not registered
                Font font = JFaceResources.getFontRegistry().get(StringConverter.asString(fontData));

                return font;
            }
        }
        return null;
    }

    /**
     * Get the target GraphElement for the given connection
     * 
     * @param gEdge the connection
     * @return the target graph element
     */
    public static GraphElement getTarget(GraphEdge gEdge)
    {
        GraphConnector toConnector = (GraphConnector) gEdge.getAnchor().get(gEdge.getAnchor().size() - 1);
        return toConnector.getGraphElement();
    }

    /**
     * Get the source GraphElement for the given connection
     * 
     * @param gEdge the connection
     * @return the source graph element
     */
    public static GraphElement getSource(GraphEdge gEdge)
    {
        GraphConnector fromConnector = (GraphConnector) gEdge.getAnchor().get(0);
        return fromConnector.getGraphElement();
    }

    /**
     * Check if an object is a child of an other object
     * 
     * @param parent the parent graph element
     * @param child the possible child element
     * @return a boolean ;)
     */
    public static boolean isChild(GraphElement parent, GraphElement child)
    {
        GraphElement childParent = child.getContainer();
        if (childParent == null)
        {
            return false;
        }
        if (childParent == parent)
        {
            return true;
        }

        return isChild(parent, childParent);
    }

    /**
     * Get the list of the incoming/outgoing connections
     * 
     * @param gElt the graphical element
     * @return the list of connections
     */
    public static List getEdges(GraphElement gElt)
    {
        List connections = new ArrayList();
        Iterator it = gElt.getAnchorage().iterator();
        while (it.hasNext())
        {
            GraphConnector connector = (GraphConnector) it.next();
            Iterator it2 = connector.getGraphEdge().iterator();
            while (it2.hasNext())
            {
                GraphEdge edge = (GraphEdge) it2.next();
                connections.add(edge);
            }
        }
        return connections;
    }

    /**
     * Get the list of the outgoing connection
     * 
     * @param gElt the source element
     * @return the list of source connections
     */
    public static List<GraphEdge> getSourceEdges(GraphElement gElt)
    {
        List<GraphEdge> srcConnections = new ArrayList<GraphEdge>();
        Iterator it = gElt.getAnchorage().iterator();
        while (it.hasNext())
        {
            GraphConnector connector = (GraphConnector) it.next();
            Iterator it2 = connector.getGraphEdge().iterator();
            while (it2.hasNext())
            {
                GraphEdge edge = (GraphEdge) it2.next();
                if (connector == edge.getAnchor().get(0))
                {
                    srcConnections.add(edge);
                }
            }
        }
        return srcConnections;
    }

    /**
     * Get the list of the incoming connection
     * 
     * @param gElt the source element
     * @return the list of target connections
     */
    public static List getTargetEdges(GraphElement gElt)
    {
        List tgtConnections = new ArrayList();
        Iterator it = gElt.getAnchorage().iterator();
        while (it.hasNext())
        {
            GraphConnector connector = (GraphConnector) it.next();
            Iterator it2 = connector.getGraphEdge().iterator();
            while (it2.hasNext())
            {
                GraphEdge edge = (GraphEdge) it2.next();
                if (edge.getAnchor().size() == 2 && connector == edge.getAnchor().get(1))
                {
                    tgtConnections.add(edge);
                }
            }
        }
        return tgtConnections;
    }

    /**
     * Used to get the model object that is the root of the current diagram
     * 
     * @param graphElement
     * @return the root model object of the current diagram
     */
    public static EObject getDiagramModelObject(GraphElement graphElement)
    {
        return Utils.getElement(getDiagram(graphElement));
    }

    /**
     * Used to get the Diagram node of a given graphElement
     * 
     * @param graphElement
     * @return the current Diagram
     */
    public static Diagram getDiagram(GraphElement graphElement)
    {
        if (graphElement.getContainer() != null)
        {
            return getDiagram(graphElement.getContainer());
        }
        return (Diagram) graphElement;
    }

    /**
     * Used to get the Diagrams of GraphElements
     * 
     * @param graphElements the elements to have the diagram
     * @return the current Diagram
     */
    public static Diagram[] getDiagrams(GraphElement[] graphElements)
    {
        List diagrams = new ArrayList();

        for (int i = 0; i < graphElements.length; i++)
        {
            Diagram diag = getDiagram(graphElements[i]);
            if (diag != null && !diagrams.contains(diag))
            {
                diagrams.add(diag);
            }
        }

        return (Diagram[]) diagrams.toArray(new Diagram[diagrams.size()]);
    }

    /**
     * This api will expose the diagram at the location given in absolute co-ordinates.
     * 
     * @param canvas
     * @param location
     */
    public static void exposeLocation(FigureCanvas canvas, Point location)
    {
        Point loc = location.getCopy();
        int padding = 50;
        if (loc.x >= 0)
        {
            loc.x += padding;
        }
        else
        {
            location.x -= padding;
        }

        if (loc.y >= 0)
        {
            loc.y += padding;
        }
        else
        {
            loc.y -= padding;
        }

        int viewPortXLocation = canvas.getBounds().x; // getViewport().getHorizontalRangeModel().getValue();
        int viewPortXExtent = canvas.getBounds().x + canvas.getBounds().width; // getViewport().getHorizontalRangeModel().getExtent();

        int viewPortYLocation = canvas.getBounds().y; // getViewport().getVerticalRangeModel().getValue();
        int viewPortYExtent = canvas.getBounds().y + canvas.getBounds().height; // Viewport().getVerticalRangeModel().getExtent();

        int deltaX = 0;
        int deltaY = 0;

        if (loc.x < viewPortXLocation)
        {
            deltaX = loc.x - viewPortXLocation;
        }
        else if (loc.x > viewPortXExtent)
        {
            deltaX = loc.x - viewPortXExtent;
        }

        if (loc.y < viewPortYLocation)
        {
            deltaY = loc.y - viewPortYLocation;
        }
        else if (loc.y > viewPortYExtent)
        {
            deltaY = loc.y - viewPortYExtent;
        }

        RangeModel hRange = canvas.getViewport().getHorizontalRangeModel();
        RangeModel vRange = canvas.getViewport().getVerticalRangeModel();
        if ((deltaX != 0) || (deltaY != 0))
        {
            canvas.getViewport().setIgnoreScroll(true);
            int x = hRange.getValue() + deltaX;
            int y = vRange.getValue() + deltaY;
            if (x < 0)
            {
                x = 0;
            }
            if (y < 0)
            {
                y = 0;
            }
            canvas.scrollSmoothTo(x, y);
            canvas.getViewport().setIgnoreScroll(false);
        }
    }

    /**
     * Search in the DiagramGraphConf the first NodeGraphConf with the corresponding EClass and presentation that
     * validate all the constraints when they exist
     * 
     * @param diagGraphConf the DiagramGraphConf that contains all the NodeGraphConf objects
     * @param modelObject the model object that should correspond to a NodeGraphConf
     * @param presentation the presentation to match
     * @return the first NodeGraphConf that match all the constraints
     */
    public static NodeGraphConf getNodeGraphConf(DiagramGraphConf diagGraphConf, EObject modelObject, String presentation)
    {
        NodeGraphConf nodeGraphConf = null;
        boolean isFound = false;

        // Get the list of the NodeGraphConf that belongs to a given EClass and
        // presentation
        List allNodes = diagGraphConf.getListNodeGraphConf(modelObject.eClass(), presentation);

        for (Iterator itNodes = allNodes.iterator(); itNodes.hasNext() && !isFound;)
        {
            nodeGraphConf = (NodeGraphConf) itNodes.next();
            boolean validateConstraints = true;

            // Iterates on all the existing constraints that should
            for (Iterator itConstraints = nodeGraphConf.getConstraint().iterator(); itConstraints.hasNext();)
            {
                Constraint constraint = (Constraint) itConstraints.next();

                // all the constraints should be valid
                try
                {
                    validateConstraints = validateConstraints && EvaluatorsManager.getInstance().evaluate(modelObject, constraint.getRule(), constraint.getLanguage());
                }
                catch (EvaluatorException ee)
                {
                    // A problem occured during the evaluation of the
                    // constraint. We consider that the constraint is not valid
                    // and that the object is compliant with it.
                    // Then Just log
                    ModelerPlugin.log(ee);
                }
            }
            if (validateConstraints)
            {
                // a NodeGraphConf that validate all the constraints was found
                isFound = true;
            }
        }

        // TODO Iterates on the ESuperTypes of the EClass associated with the
        // modelObject to check whether a configuration corresponding to a
        // parent object may be used
        return nodeGraphConf;
    }

    /**
     * Search in the DiagramGraphConf the first EdgeGraphConf with the corresponding EClass and presentation that
     * validate all the constraints when they exist
     * 
     * @param diagGraphConf the DiagramGraphConf that contains all the EdgeGraphConf objects
     * @param modelObject the model object that should correspond to an EdgeGraphConf
     * @param presentation the presentation to match
     * @return the first EdgeGraphConf that match all the constraints
     */
    public static EdgeGraphConf getEdgeGraphConf(DiagramGraphConf diagGraphConf, EObject modelObject, String presentation)
    {
        EdgeGraphConf edgeGraphConf = null;
        boolean isFound = false;

        // Get the list of the EdgeGraphConf that belongs to a given EClass and
        // presentation
        List allEdges = diagGraphConf.getListEdgeGraphConf(modelObject.eClass(), presentation);

        for (Iterator itEdges = allEdges.iterator(); itEdges.hasNext() && !isFound;)
        {
            edgeGraphConf = (EdgeGraphConf) itEdges.next();
            boolean validateConstraints = true;

            // Iterates on all the existing constraints that should
            for (Iterator itConstraints = edgeGraphConf.getConstraint().iterator(); itConstraints.hasNext();)
            {
                Constraint constraint = (Constraint) itConstraints.next();

                // all the constraints should be valid
                try
                {
                    validateConstraints = validateConstraints && EvaluatorsManager.getInstance().evaluate(modelObject, constraint.getRule(), constraint.getLanguage());
                }
                catch (EvaluatorException ee)
                {
                    // A problem occured during the evaluation of the
                    // constraint. We consider that the constraint is not valid
                    // and that the object is compliant with it.
                    // Then Just log
                    ModelerPlugin.log(ee);
                }
            }
            if (validateConstraints)
            {
                // an EdgeGraphConf that validate all the constraints was found
                isFound = true;
            }
        }

        // TODO Iterates on the ESuperTypes of the EClass associated with the
        // modelObject to check whether a configuration corresponding to a
        // parent object may be used
        return edgeGraphConf;
    }
}