/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: David Sciamma (Anyware Technologies), Mathieu Garcia (Anyware
 * Technologies), Jacques Lescot (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.modeler.wizards;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.topcased.modeler.extensions.DiagramDescriptor;
import org.topcased.modeler.extensions.DiagramsManager;
import org.topcased.modeler.extensions.TemplateDescriptor;
import org.topcased.modeler.extensions.TemplatesManager;
import org.topcased.modeler.internal.ModelerPlugin;
import org.topcased.modeler.preferences.ModelerPreferenceConstants;

/**
 * This page displays the new file information inputs.
 * 
 * <br>
 * creation : 13 dec. 2004
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma </a>
 */
public abstract class DiagramsPage extends WizardPage
{
    private static final String BROWSE_TEXT = "...";

    /**
     * Contains the resource that is initially selected when the wizard is opened. In the case of a file with an invalid
     * extension, we store the container of the file. This variable is then used to load the corresponding data in the
     * wizard.
     */
    private IResource selectedResource;

    private boolean allowInit;

    // SWT objects
    private Button existingModelBt;

    private Button newModelBt;

    private Group newModelGrp;

    private Text directoryFd;

    private Button directoryBt;

    private Text nameFd;

    private ComboViewer templateCbViewer;

    private Group existingModelGrp;

    private Text modelFd;

    private Button modelBt;

    private TreeViewer viewer;

    private ComboViewer rootDiagramCbViewer;

    private Button initializeBt;

    private ModifyListener validationListener = new ModifyListener()
    {
        public void modifyText(ModifyEvent e)
        {
            setPageComplete(validatePage());
        }
    };

    /**
     * Constructor
     * 
     * @param pageName
     * @param selection
     * @deprecated Use the constructor that enables to specify whether the Diagram may be initialized
     */
    public DiagramsPage(String pageName, IStructuredSelection selection)
    {
        this(pageName, selection, false);
    }

    /**
     * Constructor
     * 
     * @param pageName the page title
     * @param selection the selected object
     * @param allowDiagramInitialization
     */
    public DiagramsPage(String pageName, IStructuredSelection selection, boolean allowDiagramInitialization)
    {
        super(pageName);
        setPageComplete(false);

        Object sel = selection.getFirstElement();
        if (sel != null && sel instanceof IResource)
        {
            if (sel instanceof IFile)
            {
                // check if the selected file as the expected extension
                if (((IFile) sel).getFileExtension().equals(getFileExtension()))
                {
                    selectedResource = (IFile) sel;
                }
                else
                {
                    selectedResource = ((IResource) sel).getParent();
                }
            }
            else
            {
                selectedResource = (IResource) sel;
            }
        }
        else
        {
            selectedResource = ModelerPlugin.getWorkspace().getRoot();
        }

        this.allowInit = allowDiagramInitialization;
    }

    /**
     * Creation of the wizard page
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        Composite mainComp = new Composite(parent, SWT.NONE);
        mainComp.setLayout(new GridLayout());
        mainComp.setLayoutData(new GridData(GridData.FILL_BOTH));
        mainComp.setFont(parent.getFont());

        createDestinationGroup(mainComp);

        // Add listeners
        hookListeners();

        // Initialize widgets value
        loadData();

        // Show description on opening
        setErrorMessage(null);
        setMessage(null);
        setControl(mainComp);

        setPageComplete(validatePage());
    }

    /**
     * Create the Destination group
     * 
     * @param parent the Composite
     */
    protected void createDestinationGroup(Composite parent)
    {
        // Group activated when the user choose to create a new Model file
        newModelBt = new Button(parent, SWT.RADIO);
        newModelBt.setText("Create a new Model");
        newModelBt.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        createNewModelGroup(parent);

        // Group activated when the user choose to create from an existing Model
        existingModelBt = new Button(parent, SWT.RADIO);
        existingModelBt.setText("Create from an existing Model");
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.verticalIndent = 20;
        existingModelBt.setLayoutData(gd);

        createExistingModelGroup(parent);
    }

    private void createNewModelGroup(Composite parent)
    {
        newModelGrp = new Group(parent, SWT.NONE);
        newModelGrp.setLayout(new GridLayout(3, false));
        newModelGrp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        // The destination directory
        Label directoryLbl = new Label(newModelGrp, SWT.NONE);
        directoryLbl.setText("Directory : ");

        directoryFd = new Text(newModelGrp, SWT.BORDER);
        directoryFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        directoryFd.setEnabled(false);

        directoryBt = new Button(newModelGrp, SWT.PUSH);
        directoryBt.setText(BROWSE_TEXT);

        // The new name of the model file to create
        Label nameLbl = new Label(newModelGrp, SWT.NONE);
        nameLbl.setText("Model name : ");

        nameFd = new Text(newModelGrp, SWT.BORDER);
        GridData layoutNameFd = new GridData(GridData.FILL_HORIZONTAL);
        layoutNameFd.horizontalSpan = 2;
        nameFd.setLayoutData(layoutNameFd);

        // Template selection
        Label templLbl = new Label(newModelGrp, SWT.NONE);
        templLbl.setText("Template : ");

        templateCbViewer = new ComboViewer(newModelGrp, SWT.READ_ONLY | SWT.BORDER);
        GridData layoutTemplateCbViewer = new GridData(GridData.FILL_HORIZONTAL);
        layoutTemplateCbViewer.horizontalSpan = 2;
        templateCbViewer.getCombo().setLayoutData(layoutTemplateCbViewer);
    }

    private void createExistingModelGroup(Composite parent)
    {
        existingModelGrp = new Group(parent, SWT.NONE);
        existingModelGrp.setLayout(new GridLayout(3, false));
        existingModelGrp.setLayoutData(new GridData(GridData.FILL_BOTH));

        // Selection of the existing model file
        Label modelLbl = new Label(existingModelGrp, SWT.NONE);
        modelLbl.setText("Model : ");

        modelFd = new Text(existingModelGrp, SWT.BORDER);
        modelFd.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        modelFd.setEnabled(false);

        modelBt = new Button(existingModelGrp, SWT.PUSH);
        modelBt.setText(BROWSE_TEXT);

        // The TreeViewer of the model
        Label viewerLbl = new Label(existingModelGrp, SWT.NONE);
        viewerLbl.setText("Select : ");

        viewer = new TreeViewer(existingModelGrp, SWT.SINGLE | SWT.BORDER);
        GridData gdViewer = new GridData(GridData.FILL_BOTH);
        gdViewer.horizontalSpan = 2;
        gdViewer.minimumHeight = 100;
        viewer.getTree().setLayoutData(gdViewer);

        Label rootDiagramLbl = new Label(existingModelGrp, SWT.NONE);
        rootDiagramLbl.setText("Root Diagram : ");

        rootDiagramCbViewer = new ComboViewer(existingModelGrp, SWT.READ_ONLY | SWT.BORDER);
        GridData layoutRootDiagramCbViewer = new GridData(GridData.FILL_HORIZONTAL);
        layoutRootDiagramCbViewer.horizontalSpan = 2;
        rootDiagramCbViewer.getCombo().setLayoutData(layoutRootDiagramCbViewer);

        if (allowInit)
        {
            initializeBt = new Button(existingModelGrp, SWT.CHECK);
            initializeBt.setText("Initialize the diagram with existing model objects");
            GridData gdInit = new GridData(GridData.FILL_HORIZONTAL);
            gdInit.horizontalSpan = 3;
            initializeBt.setLayoutData(gdInit);
        }
    }

    /**
     * Initialize widgets value
     */
    private void loadData()
    {
        // Set the label provider
        templateCbViewer.setLabelProvider(new LabelProvider()
        {
            public String getText(Object element)
            {
                // Return the resolution's label.
                if (element instanceof TemplateDescriptor)
                {
                    TemplateDescriptor diag = (TemplateDescriptor) element;
                    return diag.getName();
                }
                return "";
            }
        });

        // retrieve the templates applicable for a given Topcased editor
        TemplateDescriptor[] templateDescriptors = TemplatesManager.getInstance().getTemplates(getEditorID());

        // Set the content provider
        templateCbViewer.setContentProvider(new ArrayContentProvider());
        templateCbViewer.setInput(templateDescriptors);

        // Set the initial selection
        TemplateDescriptor initialTemplate = TemplatesManager.getInstance().find(getDefaultTemplateId());
        if (initialTemplate != null && getEditorID().equals(initialTemplate.getEditorId()))
        {
            templateCbViewer.setSelection(new StructuredSelection(initialTemplate), true);
        }

        // Set the label provider
        rootDiagramCbViewer.setLabelProvider(new LabelProvider()
        {
            public String getText(Object element)
            {
                // Return the resolution's label.
                if (element instanceof DiagramDescriptor)
                {
                    DiagramDescriptor diag = (DiagramDescriptor) element;
                    return diag.getName();
                }
                return "";
            }
        });

        // Set the content provider
        rootDiagramCbViewer.setContentProvider(new ArrayContentProvider());

        if (selectedResource instanceof IFile)
        {
            existingModelBt.setSelection(true);
            newModelGrp.setEnabled(false);

            modelFd.setText(selectedResource.getFullPath().toString());

            if (loadModelFile())
            {
                handleRootModelObjectChanged();
                existingModelBt.setFocus();
            }
        }
        else
        {
            newModelBt.setSelection(true);
            existingModelGrp.setEnabled(false);

            directoryFd.setText(selectedResource.getFullPath().toString());
            nameFd.setText("DefaultName");
            nameFd.setFocus();
        }

        if (allowInit)
        {
            initializeBt.setSelection(ModelerPlugin.getDefault().getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM));
        }
    }

    /**
     * Add the listeners to the Buttons and the Text widgets
     */
    private void hookListeners()
    {
        directoryFd.addModifyListener(validationListener);
        nameFd.addModifyListener(validationListener);
        templateCbViewer.getCombo().addModifyListener(validationListener);

        modelFd.addModifyListener(validationListener);
        rootDiagramCbViewer.getCombo().addModifyListener(validationListener);

        // Button listeners
        directoryBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleDirectoryChoose();
            }
        });

        modelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleModelChoose();
            }
        });

        existingModelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (existingModelBt.getSelection())
                {
                    newModelGrp.setEnabled(false);
                    existingModelGrp.setEnabled(true);
                }
                setPageComplete(validatePage());
            }
        });

        newModelBt.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                if (newModelBt.getSelection())
                {
                    newModelGrp.setEnabled(true);
                    existingModelGrp.setEnabled(false);
                }
                setPageComplete(validatePage());
            }
        });

        viewer.addSelectionChangedListener(new ISelectionChangedListener()
        {
            public void selectionChanged(SelectionChangedEvent event)
            {
                handleRootModelObjectChanged();
                setPageComplete(validatePage());
            }
        });

        if (initializeBt != null)
        {
            initializeBt.addSelectionListener(new SelectionAdapter()
            {
                /**
                 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
                 */
                public void widgetSelected(SelectionEvent e)
                {
                    ModelerPlugin.getDefault().getPreferenceStore().setValue(ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM, initializeBt.getSelection());
                }
            });
        }
    }

    /**
     * Handle the directory choice button action
     */
    protected void handleDirectoryChoose()
    {
        ContainerSelectionDialog dialog = new ContainerSelectionDialog(getShell(), (IContainer) ModelerPlugin.getWorkspace().getRoot().findMember(directoryFd.getText()), false,
                "Choose the destination directory :");
        if (dialog.open() == Window.OK)
        {
            Object[] results = dialog.getResult();
            if (results.length == 1 && results[0] instanceof IPath)
            {
                IPath newPath = (IPath) results[0];
                directoryFd.setText(newPath.toString());
            }
        }
    }

    /**
     * Handle the model choice button action
     */
    protected void handleModelChoose()
    {
        // The initial directory is always the Workspace
        String initialDirectory = ModelerPlugin.getWorkspace().getRoot().getFullPath().toString();

        ResourceSelectionDialog dialog = new ResourceSelectionDialog(getShell(), ModelerPlugin.getWorkspace().getRoot().findMember(initialDirectory), "Choose an existing model file :");
        if (dialog.open() == Window.OK)
        {
            Object[] results = dialog.getResult();
            if (results.length == 1 && results[0] instanceof IFile)
            {
                modelFd.setText(((IFile) results[0]).getFullPath().toString());

                // update the other widgets
                loadModelFile();
            }
        }
    }

    /**
     * Handle the change of the root model object. Update the available Diagrams
     */
    protected void handleRootModelObjectChanged()
    {
        if (!viewer.getSelection().isEmpty())
        {
            EObject currentSelectedEObject = (EObject) AdapterFactoryEditingDomain.unwrap(((IStructuredSelection) viewer.getSelection()).getFirstElement());

            if (rootDiagramCbViewer.getContentProvider() != null)
            {
                // retrieve the diagrams available for a given model object
                DiagramDescriptor[] diagramDescriptors = DiagramsManager.getInstance().getDiagrams(currentSelectedEObject, getEditorID());
                rootDiagramCbViewer.setInput(diagramDescriptors);

                // Set the initial selection
                rootDiagramCbViewer.setSelection(new StructuredSelection(diagramDescriptors), true);
            }
        }
    }

    /**
     * Load the modelFile selected and update the associated widgets
     * 
     * @return true if the file was loaded successfully, return false otherwise.
     */
    protected boolean loadModelFile()
    {
        URI uri = URI.createPlatformResourceURI(modelFd.getText(), true);
        Resource resource = getResourceSet().getResource(uri, true);

        if (resource != null)
        {
            ComposedAdapterFactory adapterFactory = getAdapterFactory();

            // Set the tree providers of the model file contents
            AdapterFactoryContentProvider adapterContentProvider = new AdapterFactoryContentProvider(adapterFactory);
            adapterContentProvider.inputChanged(viewer, null, null);
            viewer.setContentProvider(new WizardContentProvider(adapterContentProvider));
            viewer.setLabelProvider(new AdapterFactoryLabelProvider(adapterFactory));

            viewer.setInput(resource.getContents());
            viewer.refresh();
            viewer.setSelection(new StructuredSelection(resource.getContents()), true);

            return true;
        }
        return false;
    }

    /**
     * Return the ResourceSet used to store the model and diagram file. By default create a new ResourceSet.
     * 
     * @return ResourceSet
     */
    protected ResourceSet getResourceSet()
    {
        return new ResourceSetImpl();
    }

    /**
     * Check if the informations contained in the page are OK
     * 
     * @return true is OK
     */
    protected boolean validatePage()
    {
        // erase all previous messages
        setMessage(null);
        setErrorMessage(null);

        if (isNewModel())
        {
            return validateNewModelGroup();
        }
        else
        {
            return validateExistingModel();
        }
    }

    /**
     * Validate the group if the existing model option is chosen
     * 
     * @return <code>true</code> if data is valid
     */
    private boolean validateExistingModel()
    {
        // the user has choosen an existing model
        if (modelFd.getText() == null || "".equals(modelFd.getText()))
        {
            setErrorMessage("You have to choose an existing model file.");
            return false;
        }

        if (!((IFile) ModelerPlugin.getWorkspace().getRoot().findMember(modelFd.getText())).getFileExtension().equals(getFileExtension()))
        {
            setErrorMessage("Invalid model file. The model must be a " + getFileExtension() + " model.");
            return false;
        }

        if (diagramFileExist())
        {
            setMessage("The diagrams file already exists. It will be overwritten !", IMessageProvider.WARNING);
        }

        if (rootDiagramCbViewer.getSelection() == null || rootDiagramCbViewer.getSelection().isEmpty())
        {
            setErrorMessage("You must select a valid model object that will be associated with a Diagram.");
            return false;
        }

        return true;
    }

    /**
     * Validate the group if the new model option is chosen
     * 
     * @return <code>true</code> if data is valid
     */
    private boolean validateNewModelGroup()
    {
        IContainer container = getSelectedIContainer();
        if (container == null || container instanceof IWorkspaceRoot)
        {
            setErrorMessage("The destination is not a valid container.");
            return false;
        }

        if (getModelName() == null || "".equals(getModelName()))
        {
            setErrorMessage("Model name cannot be empty.");
            return false;
        }
        else
        {
            validateExistingFiles();
        }

        if (getTemplateId() == null)
        {
            setErrorMessage("You must select a Template.");
            return false;
        }

        return true;
    }

    /**
     * Check if the created files does not already exist
     */
    private void validateExistingFiles()
    {
        if (modelFileExist())
        {
            if (diagramFileExist())
            {
                setMessage("The model file and the diagram file already exist. They will be overwritten !", IMessageProvider.WARNING);
            }
            else
            {
                setMessage("The model file already exists. It will be overwritten !", IMessageProvider.WARNING);
            }
        }
        else if (diagramFileExist())
        {
            setMessage("The diagram file already exists. It will be overwritten !", IMessageProvider.WARNING);
        }
    }

    private boolean modelFileExist()
    {
        if (isNewModel())
        {
            String path = directoryFd.getText() + File.separator + getModelNameWithoutExtension(nameFd.getText()) + "." + getFileExtension();
            IResource resource = ModelerPlugin.getWorkspace().getRoot().findMember(path);
            return resource != null;
        }
        else
        {
            IResource resource = ModelerPlugin.getWorkspace().getRoot().findMember(modelFd.getText());
            return resource != null;
        }
    }

    private boolean diagramFileExist()
    {
        if (isNewModel())
        {
            String path = directoryFd.getText() + File.separator + getModelNameWithoutExtension(nameFd.getText()) + "." + getFileExtension() + "di";
            IResource resource = ModelerPlugin.getWorkspace().getRoot().findMember(path);
            return resource != null;
        }
        else
        {
            IResource resource = ModelerPlugin.getWorkspace().getRoot().findMember(modelFd.getText() + "di");
            return resource != null;
        }
    }

    /**
     * Returns if the diagram must be initialized with model objects
     * 
     * @return <code>true</code> if diagram must be initialized
     */
    public boolean isInitialized()
    {
        if (allowInit)
        {
            return ModelerPlugin.getDefault().getPreferenceStore().getBoolean(ModelerPreferenceConstants.P_INITIALIZE_DIAGRAM);
        }

        return false;
    }

    /**
     * Returns the selected destination container
     * 
     * @return the selected container
     */
    public IContainer getSelectedIContainer()
    {
        if (isNewModel())
        {
            return (IContainer) ModelerPlugin.getWorkspace().getRoot().findMember(directoryFd.getText());
        }
        else
        {
            return ((IFile) ModelerPlugin.getWorkspace().getRoot().findMember(modelFd.getText())).getParent();
        }
    }

    /**
     * Returns the selected model name
     * 
     * @return the model name. Returns null if no model file name is set.
     */
    public String getModelName()
    {
        if (isNewModel())
        {
            if (nameFd != null && !nameFd.isDisposed())
            {
                return getModelNameWithoutExtension(nameFd.getText());
            }
        }
        else
        {
            if (modelFd != null && !modelFd.isDisposed())
            {
                IResource resource = ModelerPlugin.getWorkspace().getRoot().findMember(modelFd.getText());
                return getModelNameWithoutExtension(resource.getName());
            }
        }
        return null;
    }

    /**
     * Returns the root EObject of the model file
     * 
     * @return the EObject
     */
    public EObject getRootEObject()
    {
        return ((EList<EObject>) viewer.getInput()).get(0);
    }

    /**
     * Returns the selected EObject that will be associated with the diagrams file. If no object is selected, returns
     * the rootEObject of the model
     * 
     * @return the EObject
     */
    public EObject getDiagramEObject()
    {
        if (((IStructuredSelection) viewer.getSelection()).getFirstElement() == null)
        {
            return getRootEObject();
        }
        else
        {
            return (EObject) AdapterFactoryEditingDomain.unwrap(((IStructuredSelection) viewer.getSelection()).getFirstElement());
        }
    }

    /**
     * Returns the name of the modelFile without extension
     * 
     * @param fullName the name with (or without) extension
     * @return the name of the modelFile without extension
     */
    protected String getModelNameWithoutExtension(String fullName)
    {
        String[] nameSplitted = fullName.split("." + getFileExtension());
        return nameSplitted[0];
    }

    /**
     * Returns the selected template id
     * 
     * @return the selected template id. Returns null if none found
     */
    public String getTemplateId()
    {
        if (templateCbViewer != null)
        {
            IStructuredSelection selection = (IStructuredSelection) templateCbViewer.getSelection();
            if (selection != null && !selection.isEmpty())
            {
                return ((TemplateDescriptor) selection.getFirstElement()).getId();
            }
        }
        return null;
    }

    /**
     * Returns the selected diagram id
     * 
     * @return the selected diagram id. Returns null if none found
     */
    public String getDiagramId()
    {
        if (rootDiagramCbViewer != null)
        {
            IStructuredSelection selection = (IStructuredSelection) rootDiagramCbViewer.getSelection();
            if (selection != null && !selection.isEmpty())
            {
                return ((DiagramDescriptor) selection.getFirstElement()).getId();
            }
        }
        return "";
    }

    /**
     * Return the choice of creation made by the user
     * 
     * @return true if the model file is not created
     */
    public boolean isNewModel()
    {
        return newModelBt.getSelection();
    }

    /**
     * Subclasses must implement this method to return the editorID on which the templates should be filtered
     * 
     * @return the string that represent an editorID
     */
    public abstract String getEditorID();

    /**
     * Subclasses must implement this method to return the file extension
     * 
     * @return the string that represent the model file extension
     */
    public abstract String getFileExtension();

    /**
     * Subclasses must implement this method to return the ComposedAdapterFactory
     * 
     * @return the ComposedAdapterFactory
     */
    public abstract ComposedAdapterFactory getAdapterFactory();

    /**
     * Subclasses must implement this <code>IWizard</code> method to perform any special finish processing for their
     * wizard.
     * 
     * @return the ComposedAdapterFactory
     */
    public abstract String getDefaultTemplateId();

}
