/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * ModelExtensibleSearchQuery.java
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import java.util.Collection;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.ui.IWorkingSet;
import org.topcased.search.core.l10n.Messages;

/**
 * Describes an abstract model search query to be subclassed by users. This abtraction describes the fact each query needs parameters common
 *  to all kind of queries such as :
 *  
 *  <p>
 *  <li>queryText
 *  <li>modelSearchEngineID
 *  <li>selectedScope
 *  <li>selection
 *  </p> 
 * <br>
 * 
 *  Users wanting to describe such query parameters must extends this class. It allows to describe arbitrary parameters using a Object map.
 *  
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class AbstractModelExtensibleSearchQuery implements ISearchQuery, INamedQuery {
    private ModelExtensibleSearchResult modelSearchResult;
    private AbstractModelExtensibleSearchParameters modelSearchParameters;
    
	private String queryKind;
	private String queryText;
	
	protected Object root;
	protected List<?> participantElements;
	protected Collection<?> validParticipantMetaElements;

    public AbstractModelExtensibleSearchQuery(AbstractModelExtensibleSearchParameters parameters) {
        modelSearchResult = new ModelExtensibleSearchResult(this);
        modelSearchParameters = parameters;
		queryKind = getModelSearchParameters().getQueryKind();
		queryText = getModelSearchParameters().getQueryText();
		participantElements = getModelSearchParameters().getParticipantElements();
   }
    
    /**
     * @see org.eclipse.search.ui.ISearchQuery#canRerun()
     */
    public boolean canRerun() {
        return true;
    }
    
    /**
     * @see org.eclipse.search.ui.ISearchQuery#canRunInBackground()
     */
    public boolean canRunInBackground() {
        return true;
    }
    
	protected void processSearchResultMatching(IResource resource, Object o) {
		ModelExtensibleSearchResultEntry mesre = new ModelExtensibleSearchResultEntry((IFile)resource, o);
		mesre.addMatchingFeature(o);
		getModelSearchResult().addItem((IFile)resource, mesre);
	}

    
    /**
     * @see org.eclipse.search.ui.ISearchQuery#getLabel()
     */
    public String getLabel() {
        String label = getName();
        
        label += Messages.getString("AbstractModelExtensibleSearchQuery.searchForMessage1") + getQueryText() + Messages.getString("AbstractModelExtensibleSearchQuery.searchForMessage2"); //$NON-NLS-1$ //$NON-NLS-2$
        int searchScope = getContainer().getSelectedScope();
        switch (searchScope) {
            case ISearchPageContainer.WORKSPACE_SCOPE:
                label += Messages.getString("AbstractModelExtensibleSearchQuery.workspaceScopeMessage"); //$NON-NLS-1$
                break;
            case ISearchPageContainer.SELECTION_SCOPE:
                label += Messages.getString("AbstractModelExtensibleSearchQuery.selectionScopeMessage"); //$NON-NLS-1$
                break;
            case ISearchPageContainer.WORKING_SET_SCOPE:
                label += Messages.getString("AbstractModelExtensibleSearchQuery.workingsetMessage"); //$NON-NLS-1$
                break;
            case ISearchPageContainer.SELECTED_PROJECTS_SCOPE:
                label += Messages.getString("AbstractModelExtensibleSearchQuery.projectsScopeMessage"); //$NON-NLS-1$
                break;
        }

        if (modelSearchResult != null) {
            int nbMatches = modelSearchResult.getTotalMatches();
            label += Messages.getString("AbstractModelExtensibleSearchQuery.matchesMessage1") + nbMatches + Messages.getString("AbstractModelExtensibleSearchQuery.matchesMessage2") + ((nbMatches > 1)?Messages.getString("AbstractModelExtensibleSearchQuery.matchesMessage3"):Messages.getString("AbstractModelExtensibleSearchQuery.matchesMessage4")) + Messages.getString("AbstractModelExtensibleSearchQuery.matchesMessage5"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
        }

        return label;
    }

    /**
     * @see org.eclipse.search.ui.ISearchQuery#getSearchResult()
     */
    public ISearchResult getSearchResult() {
        return modelSearchResult;
	}

    // returns eclipse's search page container.
    // usefull to retrieve scope settings
    private ISearchPageContainer getContainer() {
        return modelSearchParameters.getContainer();
    }
    
    // returns Model search query in terms in participants.
	private IModelSearchScope getModelSearchScope() {
		ModelSearchScopeFactory scopeFactory = ModelSearchScopeFactory.getInstance();
		
		switch (getContainer().getSelectedScope()) {
			case ISearchPageContainer.WORKSPACE_SCOPE:
			{
				return scopeFactory.createModelWorkspaceScope(modelSearchParameters);
			}
			case ISearchPageContainer.SELECTION_SCOPE:
			{	
				return  scopeFactory.createModelSearchScope(modelSearchParameters, getContainer().getSelection());
			}
			case ISearchPageContainer.SELECTED_PROJECTS_SCOPE: 
			{
				String[] projectNames= getContainer().getSelectedProjectNames();
				return  scopeFactory.createModelProjectSearchScope(modelSearchParameters, projectNames);
			}
			case ISearchPageContainer.WORKING_SET_SCOPE: 
			{				
				IWorkingSet[] workingSets= getContainer().getSelectedWorkingSets();
				// should not happen - just to be sure
				if (workingSets == null || workingSets.length < 1)
					return null;
				
				return  scopeFactory.createModelSearchScope(modelSearchParameters, workingSets);
			}
		}
		// Obviously, this is a void scope ^^
		return new ModelSearchScope();
	}

    
    /**
     * @see org.eclipse.search.ui.ISearchQuery#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    public IStatus run(IProgressMonitor monitor) throws OperationCanceledException {
        getModelSearchResult().clean();
    	IModelSearchScope scope = getModelSearchScope();
        for (IResource participant : scope.getParticipants()) {
        	if (!search(participant).equals(Status.OK_STATUS)) {
        		return Status.CANCEL_STATUS;
        	}
        }
        return new Status(IStatus.OK, Messages.getString("AbstractModelExtensibleSearchQuery.modelSearchFinishedMessage"), IStatus.OK, Messages.getString("AbstractModelExtensibleSearchQuery.noResultMessage"), null); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * Users wanting to get results must override this method.
     * @param resource A valid participant IResource according to {@link IModelResourceValidator} checkings
     * coming from search page selection. 
     */
    protected IStatus search(IResource resource) {
    	return Status.OK_STATUS;
    }
    
	public AbstractModelExtensibleSearchParameters getModelSearchParameters() {
		return modelSearchParameters;
	}
	public ModelExtensibleSearchResult getModelSearchResult() {
		return modelSearchResult;
	}
	public String getQueryKind() {
		return queryKind;
	}
	public void setQueryKind(String queryKind) {
		this.queryKind = queryKind;
	}
	public String getQueryText() {
		return queryText;
	}
	public void setQueryText(String queryText) {
		this.queryText = queryText;
	}
	public ModelSearchQueryTextualExpressionMatchingHelper getTextualExpressionMatchingHelper() {
		return ModelSearchQueryTextualExpressionMatchingHelper.getInstance();
	}
	public String getName() {
		return Messages.getString("AbstractModelExtensibleSearchQuery.modelNameMessage"); //$NON-NLS-1$
	}
}
