/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * ModelExtensibleSearchResult.java
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.search.ui.ISearchQuery;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.search.ui.ISearchResultListener;
import org.eclipse.search.ui.SearchResultEvent;
import org.eclipse.swt.graphics.Image;
import org.osgi.framework.Bundle;
import org.topcased.search.Activator;
import org.topcased.search.core.l10n.Messages;

/**
 * Defines a model search result to be computed by the eclipse search mechanism in order to 
 * contribute content to the eclipse search view. 
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
/**
 * Defines a model search result to be computed by the eclipse search mechanism in order to 
 * contribute content to the eclipse search view. 
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public final class ModelExtensibleSearchResult implements ISearchResult {
	public final static String modelSearchResultImagePath = "icons/full/esearch.gif"; //$NON-NLS-1$
	public final static String modelSearchResultImageID = "modelSearchResultImageID"; //$NON-NLS-1$
    private ISearchQuery searchQuery;
    private Map <IFile, Collection<ModelExtensibleSearchResultEntry>> searchEntries;
    private Collection<ISearchResultListener> searchResultListenrs;
    private ImageDescriptor modelSearchResultImageDescriptor;
    
    public ModelExtensibleSearchResult(AbstractModelExtensibleSearchQuery query) {
        searchQuery = query;
        searchEntries = new HashMap <IFile, Collection<ModelExtensibleSearchResultEntry>>();
        searchResultListenrs = new ArrayList <ISearchResultListener>();
        modelSearchResultImageDescriptor = getModelSearchResultImageDescriptor();
    }
    private ImageDescriptor getModelSearchResultImageDescriptor() {
	    ImageDescriptor desc = ImageDescriptor.getMissingImageDescriptor();
	    Bundle bundle = Platform.getBundle(Activator.getDefault().getBundle().getSymbolicName());
	    URL url = null;
	    if (bundle != null){
	        url = FileLocator.find(bundle, new Path(modelSearchResultImagePath), null);
	        desc = ImageDescriptor.createFromURL(url);
	    }
	    modelSearchResultImageDescriptor = desc;
	    return desc;
    }
    public void addItem(IFile file, ModelExtensibleSearchResultEntry item) {
        Collection <ModelExtensibleSearchResultEntry> itemsForFile = searchEntries.get(file);
        if (itemsForFile == null) {
            itemsForFile = new ArrayList <ModelExtensibleSearchResultEntry>();
        }
        if (!itemsForFile.contains(item)) {
            itemsForFile.add(item);
        }
        searchEntries.put(file, itemsForFile);
        fireItemAdded(item);
    }
    public int getTotalMatches() {
        int result = 0;
        for (Collection <ModelExtensibleSearchResultEntry> currentMatches : searchEntries.values()) {
            for (ModelExtensibleSearchResultEntry item : currentMatches) {
                result += item.getMatches().size();
            }
        }
        return result;
    }
    public int getMatchesForFile(IFile file) {
        int result = 0;
        Collection <ModelExtensibleSearchResultEntry> matches = searchEntries.get(file);
        if (matches != null) {
            for (ModelExtensibleSearchResultEntry item : matches) {
                result += item.getMatches().size();
            }
        }
        return result;
    }
    private void fireItemAdded(ModelExtensibleSearchResultEntry item) {
        SearchResultEvent searchResultEvent = new ModelExtensibleSearchResultEvent(this);
        for (ISearchResultListener listener : searchResultListenrs) {
            listener.searchResultChanged(searchResultEvent);
        }
    }
    public Map <IFile, Collection <ModelExtensibleSearchResultEntry>> getItems()  {
        return searchEntries;
    }
    public void addListener(ISearchResultListener l) {
        if (!searchResultListenrs.contains(l)) {
            searchResultListenrs.add(l);
        }
    }
    public void removeListener(ISearchResultListener l) {
        searchResultListenrs.remove(l);
    }
    public void clean() {
        searchEntries = new HashMap <IFile, Collection <ModelExtensibleSearchResultEntry>>();
    }
	public ImageDescriptor getImageDescriptor() {
		return modelSearchResultImageDescriptor;
	}
	public Image getImage() {
		return getModelSearchResultImageDescriptor().createImage();
	}
	public ISearchQuery getQuery() {
		return searchQuery;
	}
	public String getLabel() {
		return " " + getQuery().getLabel(); //$NON-NLS-1$
	}
	public String getTooltip() {
		return Messages.getString("ModelExtensibleSearchResult.queryResultMessage3"); //$NON-NLS-1$
	}
}