/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.engine;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceProxy;
import org.eclipse.core.resources.IResourceProxyVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkingSet;
import org.topcased.search.Activator;
import org.topcased.search.core.services.ModelExtensibleSearchEngineExtensionManager;
import org.topcased.search.core.services.ModelSearchEngineDescriptor;

public final class ModelSearchScopeFactory {
	// shared ModelSearchScopeFactory instance
	static ModelSearchScopeFactory instance;
	
	/**
	 * Singleton access to the ModelSearchScopeFactory instance.
	 * @return New ModelSearchScopeFactory instance or previously created one
	 */
	public static ModelSearchScopeFactory getInstance() {
		return instance==null?instance=new ModelSearchScopeFactory():instance;
	}
	
	/*
	 * Populates {@link ModelSearchScope} with potential resource participants for the 
	 * given resource.
	 */
	private IModelSearchScope collectModelSearchScopeParticipants(IResource resource, ModelSearchScopeResourceVisitor visitor) {
		try {
			resource.accept(visitor, IResource.DEPTH_INFINITE);
		} catch (CoreException e) {
			Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "ModelSearchScopeVisitor failed to walk workspace root !", e));
		}
		return visitor.getModelSearchScope();
	}
	
	/**
	 * Populates {@link ModelSearchScope} with potential resource participants for the 
	 * workspace root.
	 * 
	 * @param params Current search query parameters
	 * @return Populated {@link ModelSearchScope} if any potential participant matching
	 */
	public IModelSearchScope createModelWorkspaceScope(AbstractModelExtensibleSearchParameters params) {
		return collectModelSearchScopeParticipants(ResourcesPlugin.getWorkspace().getRoot(), new ModelSearchScopeResourceVisitor(new ModelSearchScope(), params));
	}
	
	/**
	 * Populates {@link ModelSearchScope} with potential resource participants for the 
	 * given workspace selection.
	 * 
	 * @param params Current search query parameters
	 * @param selection IStructuredSelection coming from current search scope selection
	 * @return Populated {@link ModelSearchScope} if any potential participant matching
	 */
	public IModelSearchScope createModelSearchScope(AbstractModelExtensibleSearchParameters params, Object selection) {
		IModelSearchScope scope = new ModelSearchScope();
		ModelSearchScopeResourceVisitor visitor = new ModelSearchScopeResourceVisitor(scope, params);
		
		if (selection instanceof IStructuredSelection) {
			for (Object o : ((IStructuredSelection)selection).toList()) {
				if (o instanceof IAdaptable) {
					Object resource = ((IAdaptable)o).getAdapter(IResource.class);
					if (resource instanceof IResource)  {
						scope = collectModelSearchScopeParticipants((IResource)resource, visitor);
					}
				}
			}
		}
		return scope;
	}
	
	/**
	 * Populates {@link ModelSearchScope} with potential resource participants for the 
	 * given Project names array.
	 * 
	 * @param params Current search query parameters
	 * @param projectNames Project names array coming from current search scope selection
	 * @return Populated {@link ModelSearchScope} if any potential participant matching
	 */
	public IModelSearchScope createModelProjectSearchScope(AbstractModelExtensibleSearchParameters params, String[] projectNames) {
		IModelSearchScope scope = new ModelSearchScope();
		ModelSearchScopeResourceVisitor visitor = new ModelSearchScopeResourceVisitor(scope, params);
		
		for (String projectName : projectNames) {
			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
			if (project != null && project.isAccessible()) {
				scope = collectModelSearchScopeParticipants(project, visitor);
			}
		}
		return scope;
	}

	/**
	 * Populates {@link ModelSearchScope} with potential resource participants for the 
	 * given {@link IWorkingSet} array.
	 * 
	 * @param params Current search query parameters
	 * @param workingSets {@link WorkingSet} array coming from current search scope selection
	 * @return Populated {@link ModelSearchScope} if any potential participant matching
	 */
	public IModelSearchScope createModelSearchScope(AbstractModelExtensibleSearchParameters params, IWorkingSet[] workingSets) {
		IModelSearchScope scope = new ModelSearchScope();
		ModelSearchScopeResourceVisitor visitor = new ModelSearchScopeResourceVisitor(scope, params);
		
		for (IWorkingSet workingSet : workingSets) {
			for (IAdaptable adaptable : workingSet.getElements()) {
				IProject project = (IProject) adaptable.getAdapter(IProject.class);
				if (project != null && project.isAccessible()) {
					scope = collectModelSearchScopeParticipants(project, visitor);
				}
			}
		}
		return scope;
	}
	
	/**
	 * Recursively visit ModelResourceScope IResource tree adding each potential 
	 * participants according to modelSearchEngine extension point resource
	 * validator.
	 *  
	 * @author lucas.bigeardel@anyware-tech.com
	 *
	 */
	class ModelSearchScopeResourceVisitor implements IResourceProxyVisitor  {
		IModelSearchScope modelSearchScope;
		AbstractModelExtensibleSearchParameters modelSearchParameters;
		
		/**
		 * Visitor looking for search potential participants.
		 * 
		 * @param scope Search scope to populate with potential participants
		 * @param params Current search query parameters
		 */
		public ModelSearchScopeResourceVisitor(IModelSearchScope scope, AbstractModelExtensibleSearchParameters params) {
			modelSearchScope = scope;
			modelSearchParameters = params;
		}
		
		/**
		 * Visit each resource proxy and add it if potential search participant.
		 */
		public boolean visit(IResourceProxy proxy) throws CoreException {
			switch (proxy.getType()) {
				case IResource.FILE:
					IResource resource = proxy.requestResource();
					if (isParticipantCurrentSearchEngineValid(resource)) {
						modelSearchScope.addParticipant(resource);
					}
					break;
				default:
					break;
			}
			return true;
		}
		public AbstractModelExtensibleSearchParameters getModelSearchParameters() {
			return modelSearchParameters;
		}
		public IModelSearchScope getModelSearchScope() {
			return modelSearchScope;
		}
		
		/**
		 * Evaluates whether a resource is a potential search participant or not.
		 * 
		 * @param resource Currently visited resource
		 * @return true if resource is potential search participant according to the 
		 * contributed validator, false otherwise
		 */
		private boolean isParticipantCurrentSearchEngineValid(IResource resource) {
			ModelSearchEngineDescriptor searchEngineDescriptor = ModelExtensibleSearchEngineExtensionManager.getInstance().find(modelSearchParameters.getModelSearchEngineID());
			return searchEngineDescriptor.getModelResourceValidator().check(resource);
		}
	}
}
