/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.core.services;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.topcased.search.core.engine.IModelResourceValidator;
import org.topcased.search.core.factories.IModelSearchQueryFactory;
import org.topcased.search.core.factories.IModelSearchQueryParametersFactory;

/**
 * Data coming from each modelSearchEngine extension point contribution is stored in this class.
 * 
 * It allows user to cache data coming from extension point registry once and keep it at runtime time in a single place.
 *  
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public final class ModelSearchEngineDescriptor {
	public final static String MODEL_SEARCH_ENGINE_ID = "id";
	public final static String MODEL_SEARCH_ENGINE_LABEL_ID = "label";
	public final static String MODEL_SEARCH_ENGINE_QUERY_CLASS_ID = "searchQueryFactory";
	public final static String MODEL_SEARCH_ENGINE_PARAMETERS_CLASS_ID = "queryParametersFactory";
	public final static String MODEL_SEARCH_ENGINE_RESOURCE_VALIDATOR_CLASS_ID = "modelResourceValidator";
	
	String ID;
	String label;
	IModelSearchQueryFactory modelSearchQueryFactory;
	IModelSearchQueryParametersFactory modelSearchParametersFactory;
	IModelResourceValidator modelResourceValidator;

	private IConfigurationElement configElement;

	public ModelSearchEngineDescriptor(IConfigurationElement element) throws CoreException {
		super();
		configElement = element;
		load();
	}
	private void load()throws CoreException {
		ID = configElement.getAttribute(MODEL_SEARCH_ENGINE_ID);
		label = configElement.getAttribute(MODEL_SEARCH_ENGINE_LABEL_ID);
		
		modelSearchQueryFactory = (IModelSearchQueryFactory)configElement.createExecutableExtension(MODEL_SEARCH_ENGINE_QUERY_CLASS_ID);
		
		modelSearchParametersFactory = (IModelSearchQueryParametersFactory)configElement.createExecutableExtension(MODEL_SEARCH_ENGINE_PARAMETERS_CLASS_ID);
		
		modelResourceValidator = (IModelResourceValidator)configElement.createExecutableExtension(MODEL_SEARCH_ENGINE_RESOURCE_VALIDATOR_CLASS_ID);
		
        // Sanity check.
        if (ID == null 
        		|| label == null
        		|| modelSearchParametersFactory == null
        		|| modelSearchQueryFactory == null
        		|| modelResourceValidator == null
                ) 
        {
            throw new CoreException(new Status(IStatus.ERROR, configElement
                    .getContributor().getName(), IStatus.OK,
                    "Invalid extension (missing id, label or class name): " + ID,
                    null));
        }
	}
	public String getID() {
		return ID;
	}
	public String getLabel() {
		return label;
	}
	public IModelSearchQueryParametersFactory getModelSearchParametersFactory() {
		return modelSearchParametersFactory;
	}
	public IModelSearchQueryFactory getModelSearchQueryFactory() {
		return modelSearchQueryFactory;
	}
	public IModelResourceValidator getModelResourceValidator() {
		return modelResourceValidator;
	}
}
