/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.engine;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.topcased.search.core.engine.AbstractModelExtensibleSearchParameters;
import org.topcased.search.core.engine.AbstractModelExtensibleSearchQuery;
import org.topcased.search.core.engine.ModelSearchQueryTextualExpressionMatchingHelper;
import org.topcased.search.ecore.utils.ECoreModelLoaderUtil;

import ca.ecliptical.emf.xpath.EMFXPath;

public class ECoreModelSearchQuery extends AbstractModelExtensibleSearchQuery {
	public final static String CASE_SENSITIVE_SEARCH = "searchCaseSensitive";
	public final static String REGEX_SEARCH = "searchRegularExpression";
	
	public ECoreModelSearchQuery(AbstractModelExtensibleSearchParameters parameters) {
		super(parameters);
	}
	
	@SuppressWarnings("unchecked") //$NON-NLS-1$
	protected IStatus search(IResource resource) {
		IStatus status = super.search(resource);
		
		if (status.equals(Status.OK_STATUS)) {
			try {
			//FIXME: LB to find a way to unload model after each model search query
				root = ECoreModelLoaderUtil.openFile(resource, false);
				validParticipantMetaElements = ECoreMetaModelIntrospector.discriminateValidMetaElements((EObject)root, ECoreSupportedElements.getSupportedElements(participantElements));
			
				if ("org.topcased.search.ecorePlainTextSearchQueryTab".equals(getQueryKind())) {
					ecorePlainTextModelSearch(resource);
				} else if ("org.topcased.search.ecoreXPathSearchQueryTab".equals(getQueryKind())) {
					ecoreXPathModelSearch(resource);
				}
			} catch (Exception e) {
				return Status.CANCEL_STATUS;
			}
		}
		return status;
	}

	
	protected int getSearchKind() {
		boolean caseSensitiveSearch;
		boolean regexSearch;
		
		caseSensitiveSearch = ((Boolean)getModelSearchParameters().getData(CASE_SENSITIVE_SEARCH)).booleanValue();
		regexSearch = ((Boolean)getModelSearchParameters().getData(REGEX_SEARCH)).booleanValue();
		
		int searchKind = ModelSearchQueryTextualExpressionMatchingHelper.NORMAL_TEXT;
		if (caseSensitiveSearch) {
			searchKind = ModelSearchQueryTextualExpressionMatchingHelper.CASE_SENSITIVE;
		}
		if (regexSearch) {
			searchKind = ModelSearchQueryTextualExpressionMatchingHelper.REGULAR_EXPRESSION;
		}
		return searchKind;
	}
	
	// textual search mode
	private void ecorePlainTextModelSearch(IResource resource) {
		String qText = getQueryText();
		qText = (qText==""&&getSearchKind()==ModelSearchQueryTextualExpressionMatchingHelper.NORMAL_TEXT)?"*":qText;
		
		// discriminating according to participant meta elements selection
		for (Object o : validParticipantMetaElements) {
		    // to avoid duplicate results, the current element should be contained by the current resource
			if (o instanceof ENamedElement && ((ENamedElement) o).eResource().getURI().equals(URI.createPlatformResourceURI(resource.getFullPath().toOSString(), false))) {
				String ecoreElementName = ((ENamedElement)o).getName();
				ecoreElementName = ecoreElementName!=null?ecoreElementName:"";
				if (getTextualExpressionMatchingHelper().match(ecoreElementName, qText, getSearchKind())) {
					processSearchResultMatching(resource, o);
				}
			}
		}
	}
	
	//xpath query mode
	protected void ecoreXPathModelSearch(IResource resource) {
		EMFXPath xPath = new EMFXPath(getQueryText());
		List nodes = xPath.selectNodes(root);
		List matchedNodes = new ArrayList<Object>();
		for (Object o : nodes) {
			if (o instanceof ENamedElement && !matchedNodes.contains(o)) {
				ENamedElement eNamedElement = (ENamedElement)o;
				// discriminating according to participant meta elements selection
				if (validParticipantMetaElements.contains(eNamedElement)) {
					matchedNodes.add(o);
					processSearchResultMatching(resource, o);
				}
			}
		}
	}
	
	@Override
	public String getName() {
		if ("org.topcased.search.ecorePlainTextSearchQueryTab".equals(getQueryKind())) {
			switch(getSearchKind()) {
			case ModelSearchQueryTextualExpressionMatchingHelper.NORMAL_TEXT:
				return " ECore Normal Text";
			case ModelSearchQueryTextualExpressionMatchingHelper.CASE_SENSITIVE:
				return " ECore Case Sensitive";
			case ModelSearchQueryTextualExpressionMatchingHelper.REGULAR_EXPRESSION:
				return " ECore Regular Expression";
			default:
				return "";
			}
		} else if ("org.topcased.search.ecoreXPathSearchQueryTab".equals(getQueryKind())) {
			return " Ecore Xpath";
		}
		return " ECore ";
	}
}
