/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ecore.ui.areas;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.PlatformUI;
import org.topcased.search.core.engine.ModelSearchQueryTextualExpressionMatchingHelper;
import org.topcased.search.ecore.engine.ECoreModelSearchQuery;
import org.topcased.search.ecore.ui.Activator;
import org.topcased.search.ui.areas.IModelSearchQueryArea;
import org.topcased.search.ui.pages.ModelExtensibleSearchPage;
import org.topcased.search.ui.services.QueryTabDescriptor;
import org.topcased.search.ui.utils.ModelSearchImages;

/**
 * Defines UI for ECore models Text based queries.
 * 
 * Different Modes are supported :
 * <p>
 * <li>Textual Search
 * <li>Regular Expression Search
 * <li>Case Sensitive Search
 * </p>
 * <br>
 * <p>
 * <b>Textual Search</b>
 * <ol>Textual Search allowing to use *, ? characters.</ol>
 * </p>
 * 
 * <p><b>Regular Expression Search</b></p>
 * <ol>Regular Expression Search fully support Java Regex Patterns. @see <a href="http://java.sun.com/j2se/1.4.2/docs/api/java/util/regex/Pattern.html">http://java.sun.com/j2se/1.4.2/docs/api/java/util/regex/Pattern.html</a></ol>
 * 
 * <p>
 * <b>Case Sensitive Search</b>
 * <ol>Textual Search matching strictly according to case.</ol>
 * </p>
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ECoreTextModelSearchQueryArea extends Composite implements IModelSearchQueryArea {
	// Messages
	private final static String EXPLANATORY_SEARCH_JOKERS_TEXT = "(* = any string, ? = any character, \\ = escape for literals : * ? \\)";
	private final static String REGEX_EXPRESSION_ERROR_MESSAGE = "Regular Expresion not valid !";
	private final static String REGEX_EXPLANATION_LINK_MESSAGE = "<a>Query Expression Help</a>";

	// Dialog settings
	private static final String ECORE_TEXT_MODEL_SEARCH_AREA_DIALOG_SECTION_ID  = "ECoreTextModelSearchQueryArea";
	private static final String QUERY_LAST_SEARCHES_LIST_DIALOG_SETTINGS_ID = "queryLastSearchesList";
	private static final String REGEX_CHECKBOX_DIALOG_SETTINGS_ID = "regExCheckbox";
	private static final String CASE_SENSISTIVE_CHECKBOX_DIALOG_SETTINGS_ID = "caseSensitiveCheckbox";
	
	private String ecoreDilaogSettingsFile = "ecoreQueryAreaTextualSearch.settings";
	
	// Search queries queue size
	private static int QUERY_LAST_SEARCHES_LIST_CAPACITY = 12;
	
	// Conrtols
	ModelExtensibleSearchPage searchPage;
	Combo searchQueryTextCombo;
	Label searchQueryLabel,searchQueryExplanatoryLabel;
	Link javaRegexPatternUrlLink;
	Button searchCaseSensitiveCheckBox, searchRegularExpressionCheckBox;
	List<String> lastQueriesList; 
	boolean hasAValidRegex = true;
	
	public ECoreTextModelSearchQueryArea(Composite parent, ModelExtensibleSearchPage page, int style) {
		super(parent, style);
		PlatformUI.getWorkbench().getHelpSystem().setHelp(this.getControl(), "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		searchPage = page;
		createControl(parent);
		lastQueriesList = new ArrayList<String>(QUERY_LAST_SEARCHES_LIST_CAPACITY);
	}
	
	//
	// Validate whether a regex is valid or not according to PatternException catching mechanism
	//
	private boolean validateRegex() {
		if (searchRegularExpressionCheckBox.getSelection()) {
			try {
				ModelSearchQueryTextualExpressionMatchingHelper.createPattern(searchQueryTextCombo.getText(), searchCaseSensitiveCheckBox.getSelection(), true);
			} catch (Exception e) {
				searchQueryExplanatoryLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_RED));
				searchQueryExplanatoryLabel.setText(REGEX_EXPRESSION_ERROR_MESSAGE);
				hasAValidRegex = false;
				validateStatus();
				return hasAValidRegex;
			}
			searchQueryExplanatoryLabel.setText("");
		} else {
			searchQueryExplanatoryLabel.setText(EXPLANATORY_SEARCH_JOKERS_TEXT);
		}
		searchQueryExplanatoryLabel.setForeground(getControl().getForeground());
		hasAValidRegex = true;
		return validateStatus();
	}
	
	//
	// Create graphicla content for current area
	//
	private void createControl(Composite parent) {
		setLayout(new GridLayout(3, false));
		setLayoutData(new GridData(GridData.FILL_BOTH));
		
		searchQueryLabel = new Label(this, SWT.NONE);
		searchQueryLabel.setText("Text Query:");
		searchQueryLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.NONE, false, false, 3, 1));
		
		searchQueryTextCombo = new Combo(this, SWT.DROP_DOWN);
		searchQueryTextCombo.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false, 2, 1));
		searchQueryTextCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent evt) {
				validateRegex();
			}
		});		
		
		searchCaseSensitiveCheckBox = new Button(this, SWT.CHECK);
		searchCaseSensitiveCheckBox.setText("Case Sensitive");
		searchCaseSensitiveCheckBox.setLayoutData(new GridData(SWT.LEFT, SWT.NONE, false, false, 1, 1));
		searchCaseSensitiveCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				searchQueryTextCombo.forceFocus();
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		
		searchQueryExplanatoryLabel = new Label(this, SWT.WRAP);
		searchQueryExplanatoryLabel.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false, 2, 1));
		searchQueryExplanatoryLabel.setText(EXPLANATORY_SEARCH_JOKERS_TEXT);
		
		searchRegularExpressionCheckBox = new Button(this, SWT.CHECK);
		searchRegularExpressionCheckBox.setText("Regular Expression");
		searchRegularExpressionCheckBox.setLayoutData(new GridData(SWT.LEFT, SWT.NONE, false, false, 1, 1));
		searchRegularExpressionCheckBox.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				searchQueryExplanatoryLabel.setText(!searchRegularExpressionCheckBox.getSelection()?EXPLANATORY_SEARCH_JOKERS_TEXT:"");
				searchCaseSensitiveCheckBox.setEnabled(!searchRegularExpressionCheckBox.getSelection());
				validateRegex();
				searchQueryTextCombo.forceFocus();
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		
		javaRegexPatternUrlLink = new Link(this, SWT.NONE);
		javaRegexPatternUrlLink.setText(REGEX_EXPLANATION_LINK_MESSAGE);
		javaRegexPatternUrlLink.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent evt) {
				try {
					PlatformUI.getWorkbench().getHelpSystem().displayHelp("org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
				} catch (Exception e) {
					Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "Impossible to open Web Browser !", e));
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(searchQueryLabel, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		PlatformUI.getWorkbench().getHelpSystem().setHelp(searchQueryTextCombo, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		PlatformUI.getWorkbench().getHelpSystem().setHelp(searchCaseSensitiveCheckBox, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		PlatformUI.getWorkbench().getHelpSystem().setHelp(searchQueryExplanatoryLabel, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		PlatformUI.getWorkbench().getHelpSystem().setHelp(searchRegularExpressionCheckBox, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		PlatformUI.getWorkbench().getHelpSystem().setHelp(javaRegexPatternUrlLink, "org.topcased.search.ecore.ui.ECoreTextModelSearchQueryArea");
		
		validateRegex();
	}
	public Control getControl() {
		return this;
	}
	public Map<String, Object> getData() {
		return dataMap;
	}
	public String getQuery() {
		return searchQueryTextCombo.getText();
	}
	
	//
	// General dialog settings loading
	//
	public void loadDialogSettings() {
		//Dialog settings for Query Tab
    	IDialogSettings settings = Activator.getDefault().getDialogSettings();
    	
		try {
			settings.load(ecoreDilaogSettingsFile);
		} catch (IOException e) {
			Activator.getDefault().getLog().log(new Status(IStatus.INFO, Activator.PLUGIN_ID, 0, "Impossible to load dialog settings !", e));
		}

        IDialogSettings ecoreTextQuerySectionDialogSettings;
        
        if ((ecoreTextQuerySectionDialogSettings = settings.getSection(ECORE_TEXT_MODEL_SEARCH_AREA_DIALOG_SECTION_ID))==null) {
        	ecoreTextQuerySectionDialogSettings = settings.addNewSection(ECORE_TEXT_MODEL_SEARCH_AREA_DIALOG_SECTION_ID);
        }

    	String[] lastQueries = ecoreTextQuerySectionDialogSettings.getArray(QUERY_LAST_SEARCHES_LIST_DIALOG_SETTINGS_ID);
    	lastQueries = lastQueries==null?new String[0]:lastQueries;
    	searchQueryTextCombo.setItems(lastQueries);
    	lastQueriesList.addAll(Arrays.asList(lastQueries));
    	if (lastQueries!=null&&lastQueries.length>0) {
    		searchQueryTextCombo.select(0);
    	} else {
    		searchQueryTextCombo.setText("*");
    	}
    	searchQueryTextCombo.setFocus();
    	
    	boolean regexCheckboxSelection = ecoreTextQuerySectionDialogSettings.getBoolean(REGEX_CHECKBOX_DIALOG_SETTINGS_ID);
    	searchRegularExpressionCheckBox.setSelection(regexCheckboxSelection);
    	
    	boolean caseSensitiveCheckboxSelection = ecoreTextQuerySectionDialogSettings.getBoolean(CASE_SENSISTIVE_CHECKBOX_DIALOG_SETTINGS_ID);
    	searchCaseSensitiveCheckBox.setSelection(caseSensitiveCheckboxSelection);
    	searchCaseSensitiveCheckBox.setEnabled(!regexCheckboxSelection);
    	searchQueryExplanatoryLabel.setText(!regexCheckboxSelection?EXPLANATORY_SEARCH_JOKERS_TEXT:"");
		searchQueryTextCombo.setFocus();
	}

	//
	// General dialog settings storage
	//
	public void storeDialogSettings() {
		//Dialog settings for Query Tab
    	IDialogSettings settings = Activator.getDefault().getDialogSettings();
        IDialogSettings ecoreTextQuerySectionDialogSettings;
        
        if ((ecoreTextQuerySectionDialogSettings = settings.getSection(ECORE_TEXT_MODEL_SEARCH_AREA_DIALOG_SECTION_ID))==null) {
        	ecoreTextQuerySectionDialogSettings = settings.addNewSection(ECORE_TEXT_MODEL_SEARCH_AREA_DIALOG_SECTION_ID);
        }
        
    	ecoreTextQuerySectionDialogSettings.put(QUERY_LAST_SEARCHES_LIST_DIALOG_SETTINGS_ID, lastQueriesList.toArray(new String[0]));
    	ecoreTextQuerySectionDialogSettings.put(REGEX_CHECKBOX_DIALOG_SETTINGS_ID, searchRegularExpressionCheckBox.getSelection());
    	ecoreTextQuerySectionDialogSettings.put(CASE_SENSISTIVE_CHECKBOX_DIALOG_SETTINGS_ID, searchCaseSensitiveCheckBox.getSelection());
    	
    	try {
			settings.save(ecoreDilaogSettingsFile);
		} catch (IOException e) {
			Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "Impossible to store dialog settings !", e));
		}
	}
	
	//
	// Query preparation, called before triggering the search
	//
	public void prepare() {
		String queryText = searchQueryTextCombo.getText();
		if (queryText != null && !queryText.equals("")) {
			if (lastQueriesList.size()>=QUERY_LAST_SEARCHES_LIST_CAPACITY) {
				lastQueriesList.remove(lastQueriesList.size()-1);
			}
			lastQueriesList.add(0, queryText);
		}
		dataMap.clear();
		dataMap.put(ECoreModelSearchQuery.CASE_SENSITIVE_SEARCH, new Boolean(!searchCaseSensitiveCheckBox.isEnabled()?false:searchCaseSensitiveCheckBox.getSelection()));
		dataMap.put(ECoreModelSearchQuery.REGEX_SEARCH, new Boolean(searchRegularExpressionCheckBox.getSelection()));
	}
	
	//
	// Validate area status according to inputs, specially regex
	//
	public boolean validateStatus() {
		boolean status = hasAValidRegex && !getQuery().equals("");
		searchPage.getContainer().setPerformActionEnabled(status);
		
		TabItem qTabItem = searchPage.getCurrentlySelectedQueryTabItem();
		if (qTabItem!=null) {
			QueryTabDescriptor qTabDescriptor = searchPage.getCurrentlySelectedQueryDescriptor(); 
			qTabItem.setImage((!status)?ModelSearchImages.getErrorImage():qTabDescriptor.getImage());
		}
		return hasAValidRegex;
	}
}
