/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.pages;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.search.ui.ISearchPage;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.search.ui.NewSearchUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.topcased.search.core.engine.AbstractModelExtensibleSearchParameters;
import org.topcased.search.core.engine.AbstractModelExtensibleSearchQuery;
import org.topcased.search.core.services.ModelExtensibleSearchEngineExtensionManager;
import org.topcased.search.core.services.ModelSearchEngineDescriptor;
import org.topcased.search.ui.Activator;
import org.topcased.search.ui.areas.IModelSearchArea;
import org.topcased.search.ui.areas.IModelSearchParticipantArea;
import org.topcased.search.ui.services.ModelExtensibleSearchParticipantTabExtensionManager;
import org.topcased.search.ui.services.ModelExtensibleSearchQueryTabExtensionManager;
import org.topcased.search.ui.services.ParticipantTabDescriptor;
import org.topcased.search.ui.services.QueryTabDescriptor;

/**
 * Defines a page contributed to the org.eclipse.search.searchPages extension point.
 * 
 * <p>
 * This pages exposes three different areas :
 * <bl>
 * <li>query area
 * <li>participant area
 * <li>scope area
 * </bl>
 * </p>
 * <br>
 * <b>Query Area</b>
 * <p>
 * The query area displays a TabFolder containing a tab for every query contributed to modelSearchQueryTab the extension point. 
 * </p>
 * <br>
 * <b>Participant Area</b>
 * <p>
 * The participant area displays a TabFolder containing a tab for every query contributed to modelSearchParticipantTab the extension point. 
 * </p>
 * <br>
 * <b>Scope Area</b>
 * <p>
 * The Scope area displays a group containing possibles workspace search scopes. 
 * </p>
 * <br>
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public final class ModelExtensibleSearchPage extends DialogPage implements ISearchPage {
	// Dialog Settings
	private final static String QUERY_SECTION_DIALOG_SETTINGS_ID = "QuerySection";
	private final static String PARTICIPANTS_SECTION_DIALOG_SETTINGS_ID = "PartcipantsSection";
	private final static String SELECTED_QUERY_ID = "SelectedQueryTabID";
	private final static String SELECTED_PARTICIPANT_ID = "SelectedPartcipantTabID";
	
	// controls
	private String participantTabID, queryTabID;
	private TabItem participantTabItemToSelect, queryTabItemToSelect;

	/** Extension point managers */
	private ModelExtensibleSearchQueryTabExtensionManager queryAreaExtensionManager;
	private ModelExtensibleSearchParticipantTabExtensionManager participantAreaExtensionManager;
	private ModelExtensibleSearchEngineExtensionManager searchEngineExtensionManager;
	
	/** tab folders which items are respectively contributed by modelSearchQueryTab & modelSearchParticipantTab */
	private TabFolder searchQueryTabFolder, searchParticipantTabFolder;
	
	/** map searchEngineID <-> ModelSearchEngineDescriptor */
	private Map<String, ModelSearchEngineDescriptor> searchEngineDescriptorsMap;

	/** ISearchPageContainer allows to get resource selection */
	private ISearchPageContainer searchPageContainer;
	
	/** Dialog settings for Model Search main page */
	private IDialogSettings settings;

	public ModelExtensibleSearchPage() {
		queryAreaExtensionManager = ModelExtensibleSearchQueryTabExtensionManager.getInstance();
		participantAreaExtensionManager = ModelExtensibleSearchParticipantTabExtensionManager.getInstance();
		searchEngineExtensionManager = ModelExtensibleSearchEngineExtensionManager.getInstance();
		searchEngineDescriptorsMap = new HashMap<String, ModelSearchEngineDescriptor>();
		settings = Activator.getDefault().getDialogSettings();
	}
	
	/* 
	 * Creates the top query configuration area 
	 */ 
	protected void createSearchQueryArea(Composite parent) {
		searchQueryTabFolder = new TabFolder(parent, SWT.TOP);
		
		GridLayout gl = new GridLayout();
		searchQueryTabFolder.setLayout(gl);
		
		GridData data = new GridData(GridData.FILL_BOTH);
		data.verticalIndent = 3;
		data.widthHint= convertWidthInCharsToPixels(50);
		data.minimumWidth= convertWidthInCharsToPixels(50);
		data.heightHint= convertWidthInCharsToPixels(15);
		data.minimumHeight= convertWidthInCharsToPixels(15);
		
		searchQueryTabFolder.setLayoutData(data);
		
		assert queryAreaExtensionManager.getModelSearchQueryAreas() != null : "No query area contributed to modelSearchQueryTab extension point";
		assert queryAreaExtensionManager.getModelSearchQueryAreas()[0] != null : "At least one area should be contributed to modelSearchQueryTab extension point";

    	// Create Query Tabs
		for (QueryTabDescriptor queryAreaDescriptor : queryAreaExtensionManager.getModelSearchQueryAreas()) {
			IModelSearchArea area = queryAreaDescriptor.createArea(searchQueryTabFolder, this);

			TabItem queryTabItem = new TabItem(searchQueryTabFolder, SWT.NONE);
			queryTabItem.setText(queryAreaDescriptor.getLabel());
			queryTabItem.setToolTipText(queryAreaDescriptor.getTooltip());
			queryTabItem.setControl(area.getControl());
			queryTabItem.setData(queryAreaDescriptor);
			queryTabItem.setImage(queryAreaDescriptor.getImage());
			
			if (queryTabID == queryAreaDescriptor.ID) {
				queryTabItemToSelect = queryTabItem;
			}
			area.loadDialogSettings();
		}
	}

	/* 
	 * Creates the middle model search participant configuration area 
	 */ 
	protected void createParticipantTabsArea(Composite parent) {
		searchParticipantTabFolder = new TabFolder(parent, SWT.TOP);
		searchParticipantTabFolder.setLayout(new GridLayout());
		
		GridData data = new GridData(GridData.FILL_BOTH);
		data.widthHint= convertWidthInCharsToPixels(50);
		data.heightHint= convertWidthInCharsToPixels(5);

		searchParticipantTabFolder.setLayoutData(data);
		
		assert participantAreaExtensionManager.getModelSearchParticipantDescriptors() != null : "No query area contributed to modelSearchParticipantTab extension point";
		assert participantAreaExtensionManager.getModelSearchParticipantDescriptors()[0] != null : "At least one area should be contributed to modelSearchParticipantTab extension point";

    	
    	// create participant tabs
		for (ParticipantTabDescriptor participantAreaDescriptor : participantAreaExtensionManager.getModelSearchParticipantDescriptors()) {
			IModelSearchArea area = participantAreaDescriptor.createArea(searchParticipantTabFolder, this);
			
			TabItem participantTabItem = new TabItem(searchParticipantTabFolder, SWT.NONE);
			participantTabItem.setText(participantAreaDescriptor.getLabel());
			participantTabItem.setToolTipText(participantAreaDescriptor.getTooltip());
			participantTabItem.setControl(area.getControl());
			participantTabItem.setImage(participantAreaDescriptor.getImage());
			participantTabItem.setData(participantAreaDescriptor);
			
			if (participantTabID == participantAreaDescriptor.ID) {
				participantTabItemToSelect = participantTabItem;
			}
			area.loadDialogSettings();
		}
	}
	
	// Associate an instance of modesl search engine to its ID according to extension point declaration
	private void initModelSearchEngineMap() {
		for (ModelSearchEngineDescriptor modelSearchEngineDescriptor : searchEngineExtensionManager.getModelSearchEngines()) {
			searchEngineDescriptorsMap.put(modelSearchEngineDescriptor.getID(), modelSearchEngineDescriptor);
		}
	}
	
	public void createControl(Composite parent) {
		getContainer().setPerformActionEnabled(false);
		
		SashForm sashForm = new SashForm(parent, SWT.VERTICAL);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalIndent = 3;
		data.verticalIndent = 3;
		data.widthHint= convertWidthInCharsToPixels(50);
		data.heightHint= convertWidthInCharsToPixels(15);

		sashForm.setLayoutData(data);
		
		createSearchQueryArea(sashForm);
		createParticipantTabsArea(sashForm);
		
		initModelSearchEngineMap();
		
		loadDialogSettings();
		
		sashForm.setWeights(new int[]{50, 50});
		
		setControl(parent);
		
		hookListeners();
		
		validateInput();
	}
	
	/* 
	 * Called whenever user clicks on the search button 
	 */
    private AbstractModelExtensibleSearchQuery newQuery() {
    	ParticipantTabDescriptor participant =  getCurrentlySelectedParticipantDescriptor();
    	QueryTabDescriptor query = getCurrentlySelectedQueryDescriptor();
    	ModelSearchEngineDescriptor searchEngine = getSearchEngineDescriptorFromPageSelection(query, participant);
    	
        AbstractModelExtensibleSearchParameters parameters = searchEngine.getModelSearchParametersFactory().createSearchQueryParameters();
        
        parameters.setModelSearchEngineID(searchEngine.getID());
        parameters.setQueryText(query.getQueryModelSearchArea().getQuery());
        parameters.setContainer(getContainer());
        parameters.setQueryKind(query.ID);
        
        IModelSearchArea area = participant.getParticipantModelSearchArea();
        if (area instanceof IModelSearchParticipantArea) {
        	parameters.setParticipantElements(((IModelSearchParticipantArea)area).getSelectedParticipants());
        }
        
        Map<String, Object> dataMap = query.getQueryModelSearchArea().getData();
        for (String key : dataMap.keySet()) {
        	parameters.setData(key, dataMap.get(key));
        }
        
        return searchEngine.getModelSearchQueryFactory().createModelSearchQuery(parameters);
    }
	private void storeQueryTabsDialogSettings(IDialogSettings settings) {
        IDialogSettings querySectionDialogSettings;
        if ((querySectionDialogSettings = settings.getSection(QUERY_SECTION_DIALOG_SETTINGS_ID))==null) {
        	querySectionDialogSettings = settings.addNewSection(QUERY_SECTION_DIALOG_SETTINGS_ID);
        }
    	String selectedQueryTabID = getCurrentlySelectedQueryDescriptor().getID();
        querySectionDialogSettings.put(SELECTED_QUERY_ID, selectedQueryTabID);
        
        // store dialog settings for all query tabs
		for (TabItem tabItem : searchQueryTabFolder.getItems()) {
			QueryTabDescriptor queryAreaDescriptor = (QueryTabDescriptor) tabItem.getData();
			queryAreaDescriptor.getQueryModelSearchArea().storeDialogSettings();
		}
	}
	private void storeParticipantTabsDialogSettings(IDialogSettings settings) {
        IDialogSettings participantSectionDialogSettings;
        if ((participantSectionDialogSettings = settings.getSection(PARTICIPANTS_SECTION_DIALOG_SETTINGS_ID))==null) {
        	participantSectionDialogSettings = settings.addNewSection(PARTICIPANTS_SECTION_DIALOG_SETTINGS_ID);
        }
    	String selectedParticipantTabID = getCurrentlySelectedParticipantDescriptor().getID();
    	participantSectionDialogSettings.put(SELECTED_PARTICIPANT_ID, selectedParticipantTabID);

        // store dialog settings for all participant tabs
		for (TabItem tabItem : searchParticipantTabFolder.getItems()) {
			ParticipantTabDescriptor participantAreaDescriptor = (ParticipantTabDescriptor) tabItem.getData();
			participantAreaDescriptor.getParticipantModelSearchArea().storeDialogSettings();
		}
	}
    private void saveDialogSettings() {
        storeQueryTabsDialogSettings(settings);
        storeParticipantTabsDialogSettings(settings);
    }
    private void loadQueryTabsDialogSettings() {
        IDialogSettings querySectionDialogSettings;
        if ((querySectionDialogSettings = settings.getSection(QUERY_SECTION_DIALOG_SETTINGS_ID))==null) {
        	querySectionDialogSettings = settings.addNewSection(QUERY_SECTION_DIALOG_SETTINGS_ID);
        }

    	queryTabID = querySectionDialogSettings.get(SELECTED_QUERY_ID);
    	queryTabItemToSelect = null;

    	// create participant tabs
		for (TabItem tabItem : searchQueryTabFolder.getItems()) {
			QueryTabDescriptor queryAreaDescriptor = (QueryTabDescriptor) tabItem.getData();
			if (queryTabID != null && queryTabID.equals(queryAreaDescriptor.ID)) {
				queryTabItemToSelect = tabItem;
			}
		}

		if (queryTabItemToSelect != null) {
			searchQueryTabFolder.setSelection(queryTabItemToSelect);
		} else {
			if (searchQueryTabFolder.getItemCount()>0) {
				searchQueryTabFolder.setSelection(0);
			}
		}
    }
    private void loadParticipantTabsDialogSettings() {
        IDialogSettings participantSectionDialogSettings;
        if ((participantSectionDialogSettings = settings.getSection(PARTICIPANTS_SECTION_DIALOG_SETTINGS_ID))==null) {
        	participantSectionDialogSettings = settings.addNewSection(PARTICIPANTS_SECTION_DIALOG_SETTINGS_ID);
        }
    	participantTabID = participantSectionDialogSettings.get(SELECTED_PARTICIPANT_ID);
    	participantTabItemToSelect = null;

    	// create participant tabs
		for (TabItem tabItem : searchParticipantTabFolder.getItems()) {
			ParticipantTabDescriptor participantAreaDescriptor = (ParticipantTabDescriptor) tabItem.getData();
			if (participantTabID != null && participantTabID.equals(participantAreaDescriptor.ID)) {
				participantTabItemToSelect = tabItem;
			}
		}

		if (participantTabItemToSelect != null) {
			searchParticipantTabFolder.setSelection(participantTabItemToSelect);
		} else {
			if (searchParticipantTabFolder.getItemCount()>0) {
				searchParticipantTabFolder.setSelection(0);
			}
		}
    }
    private void loadDialogSettings() {
    	loadQueryTabsDialogSettings();
    	loadParticipantTabsDialogSettings();
    }
    private void prepareQueries() {
    	// prepare queries tabs
		for (TabItem tabItem : searchQueryTabFolder.getItems()) {
			QueryTabDescriptor queryAreaDescriptor = (QueryTabDescriptor) tabItem.getData();
			queryAreaDescriptor.getQueryModelSearchArea().prepare();
		}
    }
    private void prepareParticipants() {
     	// prepare participant tabs
		for (TabItem tabItem : searchParticipantTabFolder.getItems()) {
			ParticipantTabDescriptor participantAreaDescriptor = (ParticipantTabDescriptor) tabItem.getData();
			participantAreaDescriptor.getParticipantModelSearchArea().prepare();
		}
    }
    private void prepareQuery() {
    	prepareQueries();
    	prepareParticipants();
    }
    private boolean validateQueryTabs() {
    	// valid queries tabs
		for (TabItem tabItem : searchQueryTabFolder.getItems()) {
			QueryTabDescriptor queryAreaDescriptor = (QueryTabDescriptor) tabItem.getData();
			if (!queryAreaDescriptor.getQueryModelSearchArea().validateStatus()) {
				return false;
			}
		}
		return true;
    }
    private boolean vaidateParticipantTabs() {
     	// valid participant tabs
		for (TabItem tabItem : searchParticipantTabFolder.getItems()) {
			ParticipantTabDescriptor participantAreaDescriptor = (ParticipantTabDescriptor) tabItem.getData();
			if(!participantAreaDescriptor.getParticipantModelSearchArea().validateStatus()) {
				return false;
			}
		}
    	return true;
    }
    
    private boolean validateInput() {
    	boolean status = validateQueryTabs() && vaidateParticipantTabs();
    	getContainer().setPerformActionEnabled(status);
    	return status;
    }
    
    
    
	/** 
	 * Called whenever user clicks on the search button 
	 */
	public boolean performAction() {
        try {
        	// Prepare data and settings before fire a new Search Query
        	prepareQuery();
        
            // Process the search operation in background
            NewSearchUI.runQueryInBackground(newQuery());

            // Store Dialog Settings between two queries
        	saveDialogSettings();

        } catch (Exception e) {
			Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "addExtension failed !", e));
            return false;
        }
        return true;
	}
	
	
	/** 
	 * Returns current particpant configuration coming from participant area controls settings 
	 */
	public ParticipantTabDescriptor getCurrentlySelectedParticipantDescriptor() {
		assert searchParticipantTabFolder.getSelection() != null : "selection shouldn't be null";
		assert searchParticipantTabFolder.getSelection().length == 1 : "selection should contain at least one participant";
		assert searchParticipantTabFolder.getSelection()[0] != null : "selected participant shouldn't be null";
		
		return (ParticipantTabDescriptor)searchParticipantTabFolder.getSelection()[0].getData();
	}
	
	/** 
	 * Returns current particpant configuration coming from query area controls settings 
	 */
	public QueryTabDescriptor getCurrentlySelectedQueryDescriptor() {
		assert searchQueryTabFolder.getSelection() != null : "selection shouldn't be null";
		assert searchQueryTabFolder.getSelection().length == 1 : "selection should contain at least one query";
		assert searchQueryTabFolder.getSelection()[0] != null : "selected query shouldn't be null";
		
		return (QueryTabDescriptor)searchQueryTabFolder.getSelection()[0].getData();
	}
	
	/** 
	 * Returns current particpant tab item coming from participant area controls settings 
	 */
	public TabItem getCurrentlySelectedParticipantTabItem() {
		if (searchParticipantTabFolder == null ||
			searchParticipantTabFolder.getSelection() == null ||
			searchParticipantTabFolder.getSelection().length < 1 ||
			searchParticipantTabFolder.getSelection()[0] == null) return null;
		
		return searchParticipantTabFolder.getSelection()[0];
	}
	
	/** 
	 * Returns current particpant tab item coming from query area controls settings 
	 */
	public TabItem getCurrentlySelectedQueryTabItem() {
		if (searchQueryTabFolder == null ||
			searchQueryTabFolder.getSelection() == null ||
			searchQueryTabFolder.getSelection().length < 1 ||
			searchQueryTabFolder.getSelection()[0] == null) return null;
			
		return searchQueryTabFolder.getSelection()[0];
	}
	
	/**
	 * Returns search engine associated with given (query, particpant) tabs selection. 
	 */
	public ModelSearchEngineDescriptor getSearchEngineDescriptorFromPageSelection(QueryTabDescriptor q, ParticipantTabDescriptor p)  {
		return searchEngineDescriptorsMap.get(p.searchEngineID);
	}

	/** 
	 * Check tabs selection integrity according to modelSearchParticipantTab & modelSearchQueryTab current selections 
	 */
	private void hookListeners() {
		searchParticipantTabFolder.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				TabItem[] particpantTabSelection = searchParticipantTabFolder.getSelection();
				if (particpantTabSelection != null) {
					for (TabItem pItem : particpantTabSelection) {
						ParticipantTabDescriptor desc = (ParticipantTabDescriptor) pItem.getData();
						List<String> qTabIDs = desc.getReferencedQueryTabIDs();
						for (TabItem qItem : searchQueryTabFolder.getItems()) {
							if (qItem.getControl() instanceof Composite) {
								boolean visible = qTabIDs.contains(((QueryTabDescriptor)qItem.getData()).getID());
								qItem.getControl().setEnabled(visible);
								for (Control c : ((Composite)qItem.getControl()).getChildren()) {
									c.setEnabled(visible);
								}
								qItem.getControl().update();
								qItem.getControl().redraw();
							}
							searchQueryTabFolder.redraw();
						}
					}
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {}
		});
	}
	
	/**
	 * Sets the search page's container.
	 * @param searchPageContainer the container to set
	 */
	public void setContainer(ISearchPageContainer c) {
		searchPageContainer = c;
		searchPageContainer.setPerformActionEnabled(false);
	}
	
	/**
	 * Return Page Container for the current model search page.
	 * @return The current model search page Page Container, null otherwise.
	 */
	public ISearchPageContainer getContainer() {
		return searchPageContainer;
	}
}
