/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.pages;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.provider.ComposeableAdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.emf.workspace.util.WorkspaceSynchronizer;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.search.ui.IContextMenuConstants;
import org.eclipse.search.ui.ISearchResult;
import org.eclipse.search.ui.ISearchResultListener;
import org.eclipse.search.ui.ISearchResultPage;
import org.eclipse.search.ui.ISearchResultViewPart;
import org.eclipse.search.ui.SearchResultEvent;
import org.eclipse.search2.internal.ui.OpenSearchPreferencesAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.Page;
import org.topcased.search.core.engine.ModelExtensibleSearchResult;
import org.topcased.search.core.engine.ModelExtensibleSearchResultEntry;
import org.topcased.search.core.engine.ModelExtensibleSearchResultEvent;
import org.topcased.search.ui.Activator;
import org.topcased.search.ui.areas.IModelElementEditorSelectionHandler;
import org.topcased.search.ui.l10n.Messages;
import org.topcased.search.ui.services.ModelExtensibleSearchParticipantTabExtensionManager;
import org.topcased.search.ui.services.ParticipantTabDescriptor;
import org.topcased.search.ui.utils.ModelSearchImages;
import org.topcased.search.ui.utils.ModelSearchResultItemProvider;

/**
 * The actual model search result page. 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ModelExtensibleSearchResultPage extends Page implements ISearchResultPage, ISearchResultListener {
    private String fId;
    
    private TreeViewer resultTree;
    private ISearchResultViewPart modelSearchResultViewPart;
  
    // Search selection wrappers
    private ModelExtensibleSearchResult modelSearchResult;

    // Adapter factories based content/label providers
    private ComposedAdapterFactory adapterFactory;
    private ModelSearchResultItemProvider searchResultItemProvider;
    private ILabelProvider searchResultLabelProvider;
    private List<ComposeableAdapterFactory> composeableAdapterFactories;
    private List<IModelElementEditorSelectionHandler> modelElementEditorSelectionHandlerList;
    
    /**
     * Creates a Model Search Page with result tree
     */
	public ModelExtensibleSearchResultPage() {
    	ModelExtensibleSearchParticipantTabExtensionManager modelExtensibleSearchParticipantTabExtensionManager = ModelExtensibleSearchParticipantTabExtensionManager.getInstance();
    	
    	ParticipantTabDescriptor[] participantTabDescriptors = modelExtensibleSearchParticipantTabExtensionManager.getModelSearchParticipantDescriptors();
    	
    	composeableAdapterFactories = new ArrayList<ComposeableAdapterFactory>();
    	modelElementEditorSelectionHandlerList = new ArrayList<IModelElementEditorSelectionHandler>();
		
    	// Add AdapterFactoryLabelAdapters
    	for (ParticipantTabDescriptor participantTabDescriptor : participantTabDescriptors) {
    		composeableAdapterFactories.add(participantTabDescriptor.getElementComposeableAdapterFactory());
    		modelElementEditorSelectionHandlerList.add(participantTabDescriptor.getModelElementEditorSelectionHandler());
    	}
    	composeableAdapterFactories.add(new ResourceItemProviderAdapterFactory());
    	
		adapterFactory = new ComposedAdapterFactory(composeableAdapterFactories);
		
		searchResultItemProvider = new ModelSearchResultItemProvider();
		searchResultLabelProvider = new AdapterFactoryLabelProvider(adapterFactory){
			@Override
			public String getColumnText(Object object, int columnIndex) {
				if (object instanceof ModelExtensibleSearchResultEntry) {
					ModelExtensibleSearchResultEntry entry = (ModelExtensibleSearchResultEntry) object;
					EObject eObject = (EObject) entry.getSource();
					return super.getColumnText(eObject, columnIndex);
				} else if (object instanceof IFile) {
					IFile file = (IFile)object;
					int nbMatches = modelSearchResult.getMatchesForFile(file);
					return file.getFullPath().toString() + Messages.getString("ModelExtensibleSearchResultPage.matchText1")+ nbMatches +  Messages.getString("ModelExtensibleSearchResultPage.matchTex2") + (nbMatches>1?Messages.getString("ModelExtensibleSearchResultPage.matchTex3"):Messages.getString("ModelExtensibleSearchResultPage.matchTex4"))+ Messages.getString("ModelExtensibleSearchResultPage.matchTex5"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				}
				return super.getColumnText(object, columnIndex);
			}
			@Override
			public Image getColumnImage(Object object, int columnIndex) {
				if (object instanceof ModelExtensibleSearchResultEntry) {
					return super.getColumnImage(((ModelExtensibleSearchResultEntry) object).getSource(), columnIndex);
				} else if (object instanceof IFile) {
					return ModelSearchImages.getFileImage();
				}
				return super.getColumnImage(object, columnIndex);
			}
		};
    }

    public void createControl(Composite parent) {
        resultTree = new TreeViewer(parent /*mainComposite*/, SWT.NONE);
        resultTree.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));

        resultTree.setContentProvider(searchResultItemProvider);
        resultTree.setLabelProvider(searchResultLabelProvider);
        resultTree.addDoubleClickListener(new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                IStructuredSelection sel = (IStructuredSelection) event.getSelection();
                if (!sel.isEmpty()) {
                    openEditor(sel.getFirstElement());
                }
            }
        });

    }
    private void openEditor(Object destination) {
        if (destination == null) {
            return;
        }

        IFile file = null;
        if (destination instanceof IFile) {
            file = (IFile) destination;
        } else {
            Object object = ((ModelExtensibleSearchResultEntry) destination).getSource();
            if (object instanceof EObject) {
            	file = WorkspaceSynchronizer.getFile(((EObject)object).eResource());
            } else {
            	file = ((ModelExtensibleSearchResultEntry) destination).getFile();
            }
        }

        IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        if (dw != null) {
            IWorkbenchPage page = dw.getActivePage();

            if (page != null) {
                try  {
                    // open the default editor (a better solution would consist in trying to open an EMF editor when possible)
                    EditorPart editorPart = (EditorPart) IDE.openEditor(page, file, true);

                    if (destination instanceof ModelExtensibleSearchResultEntry) {
                        
                        EObject eobject = (EObject) ((ModelExtensibleSearchResultEntry) destination).getSource();
                        
                        // check whether the selected EditorPart implements the IEditingDomainProvider interface
                        if (editorPart instanceof IEditingDomainProvider)
                        {
                            EditingDomain editingDomain = ((IEditingDomainProvider) editorPart).getEditingDomain();
                            
                            Object object = editingDomain.getResourceSet().getEObject(EcoreUtil.getURI(eobject), true);
                            
                            for (IModelElementEditorSelectionHandler modelElementEditorSelectionHandler : modelElementEditorSelectionHandlerList) {
                            	if (modelElementEditorSelectionHandler.isCompatibleModelElementEditorSelectionHandler(editorPart)) {
                            		modelElementEditorSelectionHandler.handleModelSearchElementSelection(editorPart, object);
                            	}
                            }
                        }
                    }
                } catch (PartInitException e) {
                    Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "Editor model element Selection failed !", e));
                }
            }
        }
    }
    public Control getControl() {
        return resultTree.getControl();
    }
    public void setFocus() {
        if (resultTree != null && resultTree.getControl() != null)
        	resultTree.getControl().setFocus();
    }
    public void setInput(ISearchResult search, Object viewState) {
        if (search instanceof ModelExtensibleSearchResult) {
            search.addListener(this);
            modelSearchResult = (ModelExtensibleSearchResult) search;
            resultTree.setInput(search);
            resultTree.refresh();
            getViewPart().updateLabel();
        }
    }
    public void setViewPart(ISearchResultViewPart part) {
        modelSearchResultViewPart = part;
    }
    private ISearchResultViewPart getViewPart() {
        return modelSearchResultViewPart;
    }
    public Object getUIState() {
        // empty implementation
        return null;
    }
    public void init(IPageSite pageSite) {
        super.init(pageSite);
        getSite().setSelectionProvider(null);
        // add something to avoid the empty menu
        IMenuManager menuManager = pageSite.getActionBars().getMenuManager();
        menuManager.appendToGroup(IContextMenuConstants.GROUP_PROPERTIES, new OpenSearchPreferencesAction());
    }
    public void saveState(IMemento memento) {
        // do nothing
    }
    public void restoreState(IMemento memento) {
        // do nothing
    }
    
    public void setID(String id) {
        fId = id;
    }
    public String getID() {
        return fId;
    }
    
    public String getLabel() {
        return modelSearchResult == null?"":modelSearchResult.getLabel(); //$NON-NLS-1$
    }
    @Override
    public void makeContributions(IMenuManager menuManager,
    		IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {

    	
    	// FIXME: LB to add a toggle button to handle graphical/non graphical editors selection open 
    	//toolBarManager.add();
    	
    	super.makeContributions(menuManager, toolBarManager, statusLineManager);
    }
    
    public void searchResultChanged(SearchResultEvent e) {
        if (e instanceof ModelExtensibleSearchResultEvent) {
        	ModelExtensibleSearchResultEvent ssre = (ModelExtensibleSearchResultEvent) e;
            modelSearchResult = (ModelExtensibleSearchResult) ssre.getSearchResult();
            // Update view must be done inside the UI thread.
            Display.getDefault().syncExec(new Runnable() {
                public void run() {
                    resultTree.setInput(modelSearchResult);
                    resultTree.refresh(false);
                    getViewPart().updateLabel();
                }
            });
        }
    }
}
