/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.ui.services;

import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.topcased.facilities.extensions.AbstractExtensionManager;
import org.topcased.search.ui.Activator;

/**
 * Listen to extension point registry to handle modelSearchQueryTab contribution lifcycle.
 * 
 * @author lucas.bigeardel@anyware-tech.com
 *
 */
public class ModelExtensibleSearchQueryTabExtensionManager extends AbstractExtensionManager {
	private final static String bundleId = Activator.getDefault().getBundle().getSymbolicName();
	public final static String TOPCASED_UI_MODEL_SEARCH_QUERY_TABS_EXT_POINT_ID = bundleId + ".modelSearchQueryTab";

	/** the shared instance */
	private static ModelExtensibleSearchQueryTabExtensionManager manager;

	/**
	 * A set that will only ever contain AbstractModelSearchQueryArea.
	 */
	private SortedSet<QueryTabDescriptor> queryContributions = new TreeSet<QueryTabDescriptor>(new Comparator<QueryTabDescriptor>() {
		public int compare(QueryTabDescriptor o1, QueryTabDescriptor o2) {
			return o1.getID().compareTo(o2.getID());
		}
	});

	
	protected ModelExtensibleSearchQueryTabExtensionManager() {
		super(TOPCASED_UI_MODEL_SEARCH_QUERY_TABS_EXT_POINT_ID);
		readRegistry();
	}

	/**
	 * Get the shared instance.
	 * 
	 * @return the validators manager
	 */
	public static ModelExtensibleSearchQueryTabExtensionManager getInstance() {
		return (manager == null)?manager = new ModelExtensibleSearchQueryTabExtensionManager():manager;
	}

	/**
	 * Find a descriptor in the registry.
	 * @param id the searched validator id
	 * @return the validator or <code>null</code> if not found
	 */
	public QueryTabDescriptor find(String id) {
		for (QueryTabDescriptor desc : queryContributions) {
			if (id.equals(desc.getID())) {
				return desc;
			}
		}
		return null;
	}

	/**
	 * Get an enumeration of validator descriptors.
	 * @return The registered validators
	 */
	public QueryTabDescriptor[] getModelSearchQueryAreas() {
		return (QueryTabDescriptor[]) queryContributions.toArray(new QueryTabDescriptor[queryContributions.size()]);
	}

	/**
	 * @see org.topcased.facilities.extensions.AbstractExtensionManager#addExtension(org.eclipse.core.runtime.IExtension)
	 */
	protected void addExtension(IExtension extension) {
		IConfigurationElement[] elements = extension.getConfigurationElements();
		for (int i = 0; i < elements.length; i++) {
			IConfigurationElement confElt = elements[i];
			try {
				queryContributions.add(new QueryTabDescriptor(confElt));
			} catch (CoreException ce) {
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, bundleId, 0, "addExtension failed !", ce));
			}
		}
	}

	/**
	 * @see org.topcased.facilities.extensions.AbstractExtensionManager#removeExtension(org.eclipse.core.runtime.IExtension)
	 */
	protected void removeExtension(IExtension extension) {
		IConfigurationElement[] elements = extension.getConfigurationElements();
		for (int i = 0; i < elements.length; i++) {
			IConfigurationElement confElt = elements[i];
			String id = confElt.getAttribute(QueryTabDescriptor.QUERY_AREA_ID);
			QueryTabDescriptor descriptor = find(id);
			queryContributions.remove(descriptor);
		}
	}
}
