
/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Lucas Bigeardel (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/

package org.topcased.search.uml2.engine;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ocl.query.Query;
import org.eclipse.emf.ocl.query.QueryFactory;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.search.core.engine.AbstractModelExtensibleSearchParameters;
import org.topcased.search.core.engine.ModelSearchQueryTextualExpressionMatchingHelper;
import org.topcased.search.ecore.engine.ECoreMetaModelIntrospector;
import org.topcased.search.ecore.engine.ECoreModelSearchQuery;
import org.topcased.search.ecore.utils.ECoreModelLoaderUtil;
import org.topcased.search.uml2.Activator;
import org.topcased.search.uml2.l10n.Messages;

import ca.ecliptical.emf.xpath.EMFXPath;

public final class UML2ModelSearchQuery extends ECoreModelSearchQuery {
	final static class UML2SupportedElements {
		private static List<EClassifier> getUML2EClassifiersLiterals() {
			List<EClassifier> eclassifiersLiteralsList = new ArrayList<EClassifier>();
			
			try {
				for (Object o : UMLPackage.eINSTANCE.getEClassifiers()) {
					if (o instanceof ENamedElement ) {
						eclassifiersLiteralsList.add((EClassifier)o);
					}
				}
			} catch (Throwable t) {
				Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, "impossible to walk UML2 classifiers", t));
			}
			return eclassifiersLiteralsList;
		}

		public static List<EClassifier> getSupportedElements(List<? extends Object> participantEClassList) {
			ArrayList<EClassifier> definitiveMetaElementParticipantList = new ArrayList<EClassifier>();
			for (EClassifier eClass : getUML2EClassifiersLiterals()) {
				if (participantEClassList.contains(eClass)) {
					definitiveMetaElementParticipantList.add(eClass);
				}
			}
			return definitiveMetaElementParticipantList;
		}
	}
	
	public UML2ModelSearchQuery(AbstractModelExtensibleSearchParameters parameters) {
		super(parameters);
	}
	
	protected IStatus search(IResource resource) {
		IStatus status = super.search(resource);
		if (status.equals(Status.OK_STATUS)) {
			try {
				root = ECoreModelLoaderUtil.openFile(resource, false);
				validParticipantMetaElements = ECoreMetaModelIntrospector.discriminateValidMetaElements((EObject)root, UML2SupportedElements.getSupportedElements(participantElements));

				if ("org.topcased.search.ecorePlainTextSearchQueryTab".equals(getQueryKind())) {
					uml2PlainTextModelSearch(resource);
				} else if ("org.topcased.search.ecoreXPathSearchQueryTab".equals(getQueryKind())) {
					uml2XPathModelSearch(resource);
				} else if ("org.topcased.search.umlOclSearchQueryTab".equals(getQueryKind())) {
					uml2OCLModelSearch(resource);
				}
			} catch (Exception e) {
				return Status.CANCEL_STATUS;
			}
		}
		return status;
			
	}
	
	// textual search mode
	private void uml2PlainTextModelSearch(IResource resource) {
		for (Object o : validParticipantMetaElements) {
			if (o instanceof NamedElement) {
				String umlElementName = ((NamedElement)o).getName();
				umlElementName = umlElementName!=null?umlElementName:"";
				if (getTextualExpressionMatchingHelper().match(umlElementName, getQueryText(), getSearchKind())) {
					processSearchResultMatching(resource, o);				
				}
			}
		}
	}
	
	// xpath query mode
	private void uml2XPathModelSearch(IResource resource) {
		EMFXPath xPath = new EMFXPath(getQueryText());
		List nodes = xPath.selectNodes(root);
		List matchedNodes = new ArrayList<Object>();
		for (Object o : nodes) {
			if (o instanceof NamedElement && !matchedNodes.contains(o)) {
				matchedNodes.add(o);
				processSearchResultMatching(resource, o);
			}
		}
	}
	
	// FIME: LB to add ocl query mode management
	private void uml2OCLModelSearch(IResource resource) {
		if (root instanceof EClassifier) {
			Query uml2OclQuery = QueryFactory.eINSTANCE.createQuery(getQueryText(), (EClassifier)root);
			if (uml2OclQuery.check(root)) {
//				uml2OclQuery.
				//processMatching(resource, o);
			}
		}
	}

	
	@Override
	public String getName() {
		if ("org.topcased.search.ecorePlainTextSearchQueryTab".equals(getQueryKind())) {
			switch(getSearchKind()) {
				case ModelSearchQueryTextualExpressionMatchingHelper.NORMAL_TEXT:
					return Messages.getString("UML2ModelSearchQuery.UML2NormalTextMessage");
				case ModelSearchQueryTextualExpressionMatchingHelper.CASE_SENSITIVE:
					return Messages.getString("UML2ModelSearchQuery.UML2CaseSensitiveMessage");
				case ModelSearchQueryTextualExpressionMatchingHelper.REGULAR_EXPRESSION:
					return Messages.getString("UML2ModelSearchQuery.UML2RegularExpressionMessage");
				default:
					return "";
			}
		} else if ("org.topcased.search.ecoreXPathSearchQueryTab".equals(getQueryKind())) {
			return Messages.getString("UML2ModelSearchQuery.UML2XPathMessage");
		}
		return Messages.getString("UML2ModelSearchQuery.UML2Message");
	}
}
