/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.changeuri.dialogs;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewFileResourceWizard;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.ServicesManager;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * Defines the dialog where the user will be able to change namespace values.<br>
 * This dialog is composed by an input field, an output field and a table containing the namespace values.<br>
 * Once the input model is set, the namespaces contained in the root node are read.
 */
public class ChangeURIDialog extends Dialog
{
	// Table with the namespaces read from the model
	private Table parameters = null;

	// Represents the model to read
	private Text inFileText = null;

	// Represents the model to write
	private Text outFileText = null;

	// Constante defining namespace
	private static final String XMLNS_EXTENSION = "xmlns";

	/**
	 * Constructor
	 * 
	 * @param parent The parent Shell
	 */
	public ChangeURIDialog(Shell parent)
	{
		super(parent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	protected void configureShell(Shell newShell)
	{
		super.configureShell(newShell);
		newShell.setText("Change URI Dialog");
		newShell.setMinimumSize(600, 400);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createDialogArea(Composite superComp)
	{
		Composite parent = new Composite(superComp, SWT.NONE);
		parent.setLayout(new GridLayout(4, false));

		final GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
		parent.setLayoutData(data);

		final Label inFileLabel = new Label(parent, SWT.NONE);
		inFileLabel.setText("Input file :");

		final GridData inData = new GridData(SWT.FILL, SWT.NONE, true, false);
		inFileText = new Text(parent, SWT.SINGLE | SWT.BORDER);
		inFileText.setLayoutData(inData);

		final Button inFileBrowseButton = new Button(parent, SWT.PUSH);
		inFileBrowseButton.setText("Browse...");
		inFileBrowseButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
				String result = dialog.open();
				if (result != null && !result.equals(""))
				{
					inFileText.setText(result);
					detectNamespacesInModel(inFileText.getText());
				}
			}
		});

		final Button inFileWorkspaceButton = new Button(parent, SWT.PUSH);
		inFileWorkspaceButton.setText("Workspace...");
		inFileWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				ResourceSelectionDialog dialog = new ResourceSelectionDialog(getShell(), ResourcesPlugin.getWorkspace().getRoot(),
						"Select the input model");
				int result = dialog.open();
				if (result == Window.OK && dialog.getResult().length > 0)
				{
					Object[] selectedArray = dialog.getResult();
					Object selectedObject = selectedArray[0];
					if (selectedObject instanceof IFile)
					{
						IFile resourceFile = (IFile) selectedObject;
						inFileText.setText(resourceFile.getLocation().toOSString());
						detectNamespacesInModel(inFileText.getText());
					}
				}
			}
		});

		final Label outFileLabel = new Label(parent, SWT.NONE);
		outFileLabel.setText("Output file :");

		final GridData outData = new GridData(SWT.FILL, SWT.NONE, true, false);
		outFileText = new Text(parent, SWT.SINGLE | SWT.BORDER);
		outFileText.setLayoutData(outData);

		final Button outFileBrowseButton = new Button(parent, SWT.PUSH);
		outFileBrowseButton.setText("Browse...");
		outFileBrowseButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog dialog = new FileDialog(getShell(), SWT.SAVE);
				String result = dialog.open();
				if (result != null && !result.equals(""))
				{
					outFileText.setText(result);
					updateOKButton();
				}
			}
		});

		final Button outFileWorkspaceButton = new Button(parent, SWT.PUSH);
		outFileWorkspaceButton.setText("Workspace...");
		outFileWorkspaceButton.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				BasicNewFileResourceWizard newFileWizard = new BasicNewFileResourceWizard();
				newFileWizard.init(PlatformUI.getWorkbench(), StructuredSelection.EMPTY);

				WizardDialog dialog = new WizardDialog(getShell(), newFileWizard)
				{
					protected void finishPressed()
					{
						WizardNewFileCreationPage wizardPage = (WizardNewFileCreationPage) getCurrentPage();
						IPath filePath = wizardPage.getContainerFullPath().append(wizardPage.getFileName());
						IFile resourceFile = ResourcesPlugin.getWorkspace().getRoot().getFile(filePath);
						outFileText.setText(resourceFile.getLocation().toOSString());
						setReturnCode(OK);
						getWizard().dispose();
						getWizard().setContainer(null);
						super.close();
					}
				};
				if (dialog.open() == Window.OK)
				{
					updateOKButton();
				}
			}
		});

		final GridData tableGridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		tableGridData.horizontalSpan = 4;

		// Table containing the parameters
		parameters = new Table(parent, SWT.BORDER);
		parameters.setLayoutData(tableGridData);
		parameters.setLinesVisible(true);
		parameters.setHeaderVisible(true);
		parameters.getVerticalBar().setEnabled(true);

		// First column : keys
		final TableColumn namespaceID = new TableColumn(parameters, SWT.NONE);
		namespaceID.setWidth(280);
		namespaceID.setText("Namespace ID");

		// Second column : values
		final TableColumn namespaceValue = new TableColumn(parameters, SWT.NONE);
		namespaceValue.setWidth(280);
		namespaceValue.setText("Namespace Value");

		final TableEditor editor = new TableEditor(parameters);
		editor.horizontalAlignment = SWT.LEFT;
		editor.grabHorizontal = true;
		editor.minimumWidth = 50;

		// editing only the second column
		final int EDITABLECOLUMN = 1;

		parameters.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				// Clean up any previous editor control
				Control oldEditor = editor.getEditor();
				if (oldEditor != null)
					oldEditor.dispose();

				// Identify the selected row
				TableItem item = (TableItem) e.item;
				if (item == null)
					return;

				// The control that will be the editor must be a child of the Table
				Text newEditor = new Text(parameters, SWT.NONE);
				newEditor.setText(item.getText(EDITABLECOLUMN));
				newEditor.addModifyListener(new ModifyListener()
				{
					public void modifyText(ModifyEvent me)
					{
						Text text = (Text) editor.getEditor();
						editor.getItem().setText(EDITABLECOLUMN, text.getText());
					}
				});
				newEditor.selectAll();
				newEditor.setFocus();
				editor.setEditor(newEditor, item, EDITABLECOLUMN);
			}
		});

		return parent;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	protected void okPressed()
	{
		// Prepare and call the service aimed to change URIs
		IService changeUriService = ServicesManager.getInstance().getService("org.topcased.service.changeuri.service");

		// Browse the table in order to build the map
		Map<String, String> namespaces = new HashMap<String, String>();
		TableItem[] items = parameters.getItems();
		for (int i = 0; i < items.length; i++)
		{
			namespaces.put(items[i].getText(0), items[i].getText(1));
		}

		Map<String, Object> params = new HashMap<String, Object>();
		params.put("InFilePath", inFileText.getText());
		params.put("OutFilePath", outFileText.getText());
		params.put("NSURI", namespaces);

		changeUriService.serviceRun(params);

		super.okPressed();
	}

	/**
	 * Parse the file to detect the different namespaces
	 * 
	 * @param modelToParse The model to read
	 */
	private void detectNamespacesInModel(String modelToParse)
	{
		// All the table items are removed from the table
		parameters.removeAll();

		// Read the model and extract the different namespaces used
		readNamespaceInputFile(new File(modelToParse));

		// Tries to update the OK button
		updateOKButton();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.dialogs.Dialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent)
	{
		// The button OK is disabled when it is created
		super.createButtonsForButtonBar(parent);
		getButton(IDialogConstants.OK_ID).setEnabled(false);
	}

	/**
	 * Updates the OK button only if the input and output fields are set.
	 */
	private void updateOKButton()
	{
		if (!"".equals(inFileText.getText()) && !"".equals(outFileText.getText()))
		{
			getButton(IDialogConstants.OK_ID).setEnabled(true);
		}
	}

	/**
	 * Reads namespace contained in the root node of the model.
	 * 
	 * @param model The model to parse
	 */
	private void readNamespaceInputFile(File model)
	{
		DocumentBuilderFactory factoryIn = DocumentBuilderFactory.newInstance();
		factoryIn.setIgnoringComments(true);
		factoryIn.setCoalescing(true);
		factoryIn.setIgnoringElementContentWhitespace(true);
		factoryIn.setNamespaceAware(true);

		try
		{
			DocumentBuilder parser = factoryIn.newDocumentBuilder();

			// Document
			Document document = parser.parse(model);
			NamedNodeMap attNodes = document.getFirstChild().getAttributes();
			for (int i = 0; i < attNodes.getLength(); i++)
			{
				Node currentHeader = attNodes.item(i);
				// Looks for all the attributes with an 'xmlns' namespace
				if (currentHeader.getPrefix() != null && currentHeader.getPrefix().equals(XMLNS_EXTENSION))
				{
					TableItem item = new TableItem(parameters, SWT.NONE);
					item.setText(new String[] { currentHeader.getLocalName(), currentHeader.getNodeValue() });
				}
			}
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "Parser configuration exception", IStatus.ERROR, e);
		}
		catch (Exception e)
		{
			CorePlugin.log(this.getClass(), "The input file does not seem to be a valid model.", IStatus.ERROR, e);
		}
	}
}
