/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.oclchecker.services.external;

import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.ocl.expressions.OCLExpression;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.helper.CheckResultModelHelper;
import org.topcased.externalbus.core.CorePlugin;
import org.topcased.externalbus.core.IExternalService;
import org.topcased.externalbus.core.helper.ExternalServiceHelper;
import org.topcased.service.oclchecker.helper.OCLServiceHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ExternalStatisticService implements IExternalService
{

    private static final String MMURI = "MMURI";

    private static final String RULE = "Rule";

    private static final String CONTEXT = "Context";

    private static final String PACKAGE = "Package";

    private static final String MODEL = "Model";

    /**
     * @see org.topcased.externalbus.core.IExternalService#externalServiceRun(java.util.Map)
     */
    public Object externalServiceRun(Map<String, Object> parameters)
    {
        String docResponse = "ExternalStatisticService";
        if (checkServiceParameters(parameters))
        {
            // initializes parameters.
            Element modelAsElement = (Element) parameters.get(ExternalStatisticService.MODEL);
            String contextPackage = (String) parameters.get(ExternalStatisticService.PACKAGE);
            String context = (String) parameters.get(ExternalStatisticService.CONTEXT);
            String ruleText = (String) parameters.get(ExternalStatisticService.RULE);
            String mmUri = (String) parameters.get(ExternalStatisticService.MMURI);

            try
            {
            	String modelAsString = ExternalServiceHelper.documentToString(modelAsElement);
                EPackage metamodelPackage = OCLServiceHelper.getMetaModelPackage(mmUri);
                if (metamodelPackage != null)
                {
                    metamodelPackage = getModelPackage(contextPackage, metamodelPackage);
                    EClass contextClass = (EClass) metamodelPackage.getEClassifier(context);
                    OCLExpression<EClassifier> rule = OCLServiceHelper.createOCLQuery(mmUri, context, ruleText);
                    docResponse = statisticServiceFromStringModel(ruleText, metamodelPackage, contextClass, rule, modelAsString);
                }
                else
                {
                	CorePlugin.log(getClass(), "The specified metamodel is not registered in EMF registry", IStatus.WARNING);
                }
            }
            catch (Exception e)
            {
                CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
            }
        }
        return docResponse;
    }

    /**
     * @param ruleText The 
     * @param metamodelPackage
     * @param contextClass
     * @param rule
     * @param modelAsString
     * @return
     */
    private String statisticServiceFromStringModel(String ruleText, EPackage metamodelPackage, EClass contextClass, OCLExpression<EClassifier> rule, String modelAsString)
    {
        try
        {
        	Resource resource = getDynamicResource();
        	resource.load(new URIConverter.ReadableInputStream(modelAsString), null);
            
            EObject model = resource.getContents().get(0);
            Object resultObject = applyStatisticService(contextClass, rule, model);

            Model resultModel = CheckResultModelHelper.createCheckResulModel(model, metamodelPackage, contextClass, ruleText);
            CheckResultModelHelper.getFirstRule(resultModel).setResult(resultObject.toString());
            
            return saveModel(resultModel);
        }
        catch (IOException e)
        {
        	CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
        }
        return null;
    }

    /**
     * @param modelAsString
     * @return
     */
    private Resource getDynamicResource()
    {
        ResourceSet resourceSet = new ResourceSetImpl();
        Resource resource = resourceSet.createResource(URI.createURI("*.xml"));
        return resource;
    }

    /**
     * @param contextClass
     * @param rule
     * @param model
     * @return
     */
    private Object applyStatisticService(EClass contextClass, OCLExpression<EClassifier> rule, EObject model)
    {
        IService statisticService = ServicesManager.getInstance().getService(IServiceCst.OCL_STATISTIC);
        HashMap<String, Object> checkParameters = new HashMap<String, Object>();
        checkParameters.put(MODEL, model);
        checkParameters.put("OCLRule", rule);
        checkParameters.put(ExternalStatisticService.CONTEXT, contextClass);
        Object resultObject = statisticService.serviceRun(checkParameters);
        return resultObject;
    }

    /**
     * Saves the dynamic model
     * 
     * @param modelResult
     */
    private String saveModel(Model modelResult)
    {
        Resource resource = getDynamicResource();
        resource.getContents().add(modelResult);

        String modelResultString = resourceToString(resource, "UTF-8");
        Document modelDocument = null;
        try
        {
            modelDocument = ExternalServiceHelper.stringToDom(modelResultString);
        }
        catch (Exception e)
        {
            try
            {
	            modelDocument = ExternalServiceHelper.sendErrorParameterDocument();
            }
            catch (ParserConfigurationException e1)
            {
	            e1.printStackTrace();
            }
        }
        return modelResultString;
    }



    /**
     * Gets the model package 
     * 
     * @param contextPackage
     * @param metamodelPackage
     * @return
     */
    private EPackage getModelPackage(String contextPackage, EPackage metamodelPackage)
    {
        if (contextPackage != null && !contextPackage.equals(""))
        {
            String[] subpackages = contextPackage.split(":");
            List<String> packagePrefixes = new ArrayList<String>();
            for (int i = 0; i < subpackages.length; i++)
            {
                if (!subpackages[i].equals(""))
                {
                    packagePrefixes.add(subpackages[i]);
                }
            }
            if (packagePrefixes.size() != 0)
            {
                if (metamodelPackage.getNsPrefix().equals(packagePrefixes.get(0)))
                {
                    EPackage tempPackage = metamodelPackage;
                    boolean packageError = false;
                    for (int i = 1; i < packagePrefixes.size() && !packageError; i++)
                    {
                        List<EPackage> subPackages = tempPackage.getESubpackages();
                        boolean packageFound = false;
                        for (int j = 0; j < subPackages.size() && !packageFound; j++)
                        {
                            EPackage subi = subPackages.get(j);
                            if (subi.getNsPrefix().equals(packagePrefixes.get(i)))
                            {
                                packageFound = true;
                                tempPackage = subi;
                            }
                        }
                        if (!packageFound)
                        {
                            packageError = true;
                        }
                    }
                    if (!packageError)
                    {
                        metamodelPackage = tempPackage;
                    }
                }
            }
        }
        return metamodelPackage;
    }

    /**
     * @param rs
     * @param encoding
     * @return
     */
    public String resourceToString(Resource rs, String encoding)
    {
        Map<String, String> options = new HashMap<String, String>();
        // change as needed according to the type of resource you are using
        options.put(XMIResource.OPTION_ENCODING, "UTF-8");

        StringWriter sw = new StringWriter();
        URIConverter.WriteableOutputStream uws = new URIConverter.WriteableOutputStream(sw, encoding);
        try
        {
            rs.save(uws, options);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        String resourceString = sw.toString();

        return resourceString;
    }

    /**
     * Checks parameters for the external statistic service. 
     * 
     * @param parameters A map of required parameters
     */
    private boolean checkServiceParameters(Map<String, Object> parameters)
    {
        boolean isParameter = true;
        if (!parameters.containsKey(MODEL))
        {
            CorePlugin.log(this.getClass(), "The 'Model' parameter was not found.", IStatus.ERROR);
            isParameter = false;
        }

        if (!parameters.containsKey(ExternalStatisticService.RULE))
        {
            CorePlugin.log(this.getClass(), "The 'Rule' parameter was not found.", IStatus.ERROR);
            isParameter = false;
        }

        if (!parameters.containsKey(ExternalStatisticService.CONTEXT))
        {
            CorePlugin.log(this.getClass(), "The 'Rule' parameter was not found.", IStatus.ERROR);
            isParameter = false;
        }

        if (!parameters.containsKey(ExternalStatisticService.MMURI))
        {
            CorePlugin.log(this.getClass(), "The 'MMURI' parameter was not found.", IStatus.ERROR);
            isParameter = false;
        }

        if (!(parameters.get(MODEL) instanceof Element))
        {
            CorePlugin.log(this.getClass(), "The 'Model' parameter must be a model.", IStatus.ERROR);
            isParameter = false;
        }

        if (!(parameters.get(ExternalStatisticService.RULE) instanceof String))
        {
            CorePlugin.log(this.getClass(), "The 'Rule' parameter must be a string.", IStatus.ERROR);
            isParameter = false;
        }

        if (parameters.containsKey("package") && !(parameters.get(ExternalStatisticService.PACKAGE) instanceof String))
        {
            CorePlugin.log(this.getClass(), "The 'Package' parameter must be a string.", IStatus.ERROR);
            isParameter = false;
        }

        if (!(parameters.get(ExternalStatisticService.CONTEXT) instanceof String))
        {
            CorePlugin.log(this.getClass(), "The 'Context' parameter must be a string.", IStatus.ERROR);
            isParameter = false;
        }

        if (!(parameters.get(ExternalStatisticService.MMURI) instanceof String))
        {
            CorePlugin.log(this.getClass(), "The 'MMURI' parameter must be a string.", IStatus.ERROR);
            isParameter = false;
        }
        return isParameter;
    }
}