/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.plugincustomization.epf;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;

/**
 * Transform a Eclipse Preferences File (.epf) in a plugin_customization.ini
 * Filter file .epf in '/instance/'. Copy line to line even if the line begin by
 * '/instance/'. (Not copy '/instance/')
 * 
 */
public class TransformEpf
{
    // Path of the Eclipse Preferences file (.epf)
    private String aPathEpf = "";

    // Path of the file 'plugin_customization.ini'
    private String aPathPci = "";

    // Filter on string 'instance'
    private static String INSTANCE = "/instance/";

    /**
     * Constructor
     * 
     * @param pathEpf : path of the ecplise preferences file (.epf)
     * @param pathPci : path of the file plugin_customization.ini
     */
    public TransformEpf(String pathEpf, String pathPci)
    {
        aPathEpf = pathEpf;
        aPathPci = pathPci;
    }

    /**
     * Create 'plugin_customization.ini' file Transform a Eclipse Preferences
     * File (.epf) in a 'plugin_customization.ini'. Copy line to line even if
     * the line begin by '/instance/'. (But not copy '/instance/')
     * 
     * @return true if 'plugin_customization.ini' is created
     */
    public boolean CreatePluginCustomization()
    {
        // Bloc declaration
        boolean isCreated = false;
        boolean isEpfPath = false;
        boolean isPciPath = false;
        // Verify Epf Path
        isEpfPath = isValidEpfPath();
        // Verify Pci Path (repository)
        isPciPath = isValidPciPath();
        if (isEpfPath && isPciPath)
        {
            isCreated = TransfomEpfToPci();
        }
        else
        {
            if (!isEpfPath)
            {
                CorePlugin.log(this.getClass(), PlugincustomizationPlugin.getResourceString("TransformEpf.log.incorrectpath.epf"),
                        IStatus.ERROR);

            }
        }

        return isCreated;
    }

    /**
     * @return true if the path is valid and exists
     */
    private boolean isValidEpfPath()
    {
        boolean isValid = false;
        File file;
        if (aPathEpf != null && !aPathEpf.equals(""))
        {
            // Exist
            file = new File(aPathEpf);
            if (file.exists() && file.isFile())
            {
                // '.epf' file
                String epfString = aPathEpf.substring(aPathEpf.lastIndexOf('.'));
                if (epfString.equals(".epf"))
                {
                    isValid = true;
                }
            }
        }
        return isValid;
    }

    /**
     * @return true if the path (repository) is valid and exists
     */
    private boolean isValidPciPath()
    {
        boolean isValid = false;
        File file;
        if (aPathPci != null && !aPathPci.equals(""))
        {
            file = new File(aPathPci);
            if (file.getParentFile().exists())
            {
                isValid = true;
            }
            if (file.exists() && file.isFile())
            {
                isValid = MessageDialog.openQuestion(new Shell(), "TOPCASED Question","The file " + aPathPci + " already exists, do you want to overwrite it?" );
            }
            
            
        }
        return isValid;
    }

    /**
     * Transform '.efp file in a 'plugin-customization.ini' file '.epf' ->
     * aPathEpf 'plugin-customization.ini' -> aPathPci
     * 
     * @return true if the transformation is done
     */
    private boolean TransfomEpfToPci()
    {

        // Bloc declaration
        boolean isTransform = false;
        // '.epf' File
        BufferedReader bufferedReader;
        // 'plugin_customization.ini' File
        BufferedWriter bufferedWriter;

        try
        {

            // Open '.epf' file
            bufferedReader = new BufferedReader(new FileReader(aPathEpf));
            // Open 'plugin_customization.ini' file (create it if does not
            // exist)
            bufferedWriter = new BufferedWriter(new FileWriter(aPathPci));

            // Add Header line in Plugin Customization
            HeaderPCI(bufferedWriter);

            // Copy and Paste when "/instance/"
            CopyFileWithFilter(INSTANCE, bufferedReader, bufferedWriter);

            // Close 'plugin_customization.ini' file
            bufferedWriter.close();
            // Close '.epf' File
            bufferedReader.close();
            isTransform = true;

        }
        catch (FileNotFoundException e)

        {
            CorePlugin.log(this.getClass(), PlugincustomizationPlugin.getResourceString("TransformEpf.log.exception.filenotfind") + e.getMessage(), IStatus.ERROR);
        }

        catch (IOException e1)

        {
            CorePlugin.log(this.getClass(), PlugincustomizationPlugin.getResourceString("TransformEpf.log.exception.io") + e1.getMessage(), IStatus.ERROR);
        }
        return isTransform;

    }

    /**
     * Read file1 (bufferedReader) and Write in file2 (bufferWritter). If lines
     * of file1 begin with 'filterString', the string is copied and pasted in
     * file2 without prefix 'filterString'.
     * 
     * @param filterString : filter
     * @param bufferedReader : file to read
     * @param bufferWriter : file to write
     * @throws IOException
     */
    private void CopyFileWithFilter(String filterString, BufferedReader bufferedReader, BufferedWriter bufferWriter)
            throws IOException
    {
        String copyLine = "";

        String line = null;
        while ((line = bufferedReader.readLine()) != null)
        {
            line = line.trim();
            if (line.startsWith(filterString))
            {
                // Copy Line
                copyLine = line.substring(filterString.length());
                while (copyLine.endsWith("\\"))
                {
                    copyLine = copyLine.substring(0, copyLine.length() - 1);
                    copyLine = copyLine.concat(bufferedReader.readLine());
                }
                // Write
                WriteLine(copyLine, bufferWriter);
            }
        }
    }

    /**
     * Write line 'line' in the file define by 'bufferWritter'
     * 
     * @param line : line to write
     * @param bufferWritter : destination
     * @throws IOException
     */
    private void WriteLine(String line, BufferedWriter bufferWritter) throws IOException
    {
        bufferWritter.write(line);
        bufferWritter.newLine();
    }

    /**
     * Write header line in the Plugin Customization
     * @param bufferWritter
     * @throws IOException
     */
    private void HeaderPCI(BufferedWriter bufferWritter) throws IOException
    {
        bufferWritter.write(PlugincustomizationPlugin.getResourceString("TransformEpf.HeaderPCI.line"));
        bufferWritter.newLine();
        bufferWritter.write(PlugincustomizationPlugin.getResourceString("TransformEpf.HeaderPCI.message.created"));
        bufferWritter.newLine();
        bufferWritter.write(PlugincustomizationPlugin.getResourceString("TransformEpf.HeaderPCI.message.objective"));
        bufferWritter.newLine();
        bufferWritter.write(PlugincustomizationPlugin.getResourceString("TransformEpf.HeaderPCI.line"));
        bufferWritter.newLine();
        bufferWritter.newLine();
    }

}
