/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.plugincustomization.wizard;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.topcased.service.plugincustomization.PlugincustomizationPlugin;
import org.topcased.service.plugincustomization.services.PluginCustomizationCreation;

/**
 * PluginCustomizationPage is a Wizard page.<br>
 * This interface permits : 
 * <ul>
 * <li>create 'plugin-customization.ini' giving the location of his creation</li>
 * <li>using the EPF file taken by the interface EclipsePreferencePage.</li>
 * </ul>
 */
public class PluginCustomizationPage extends WizardPage
{
    // Wizard
    private PluginCustomizationWizard wizard;

    // Page Finish
    private boolean finish = false;

    // Path of Plugin-Customization.ini directory
    private String aPathPCIDirectory = "";

    // Text Path of Plugin-Customization.ini directory
    private Text textPathPci = null;

    // Name of the File (Plugin-Customization.ini)
    private static String NAMEPCI = File.separator + PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.file.name");

    /**
     * Constructor
     * 
     * @param pageName : name of the page
     * @param pciWizard : wizard which call this page
     */
    protected PluginCustomizationPage(String pageName, PluginCustomizationWizard pciWizard)
    {
        super(pageName);
        wizard = pciWizard;
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent)
    {
        final Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout());
        setControl(composite);

        // Label
        createLabel(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.label"), composite);

        // Browse 'plugin_customization.ini'
        createBrowsePci(composite);

        setPageComplete(false);
    }

    /**
     * Finish : calls SERVCIE 'PluginCustomizationCreation' which calls 'TransformEpf' 
     * 
     * @return true if the page can be finished
     */
    public boolean finish()
    {
        String epf = wizard.getEpfPath();
        String pci = wizard.getPciPath();

        // CALL SERVICE PluginCustomizationCreation
        Map<String, Object> mapInput = new HashMap<String, Object>();
        PluginCustomizationCreation createPCI = new PluginCustomizationCreation();
        mapInput.put(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.parameter.EPFPath"), epf);
        mapInput.put(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.parameter.PCIPath"), pci);
        createPCI.serviceRun(mapInput);

        return finish;
    }

    /**
     * Create Section : path template repository
     * 
     * @param parent
     */
    private void createBrowsePci(Composite parent)
    {
        final Composite browseComposite = new Composite(parent, SWT.NONE);
        browseComposite.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
        browseComposite.setLayout(new GridLayout(2, false));
        
        textPathPci = new Text(browseComposite, SWT.BORDER | SWT.READ_ONLY);
        textPathPci.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
        textPathPci.setText(aPathPCIDirectory);        
        
        final Button browseButton = new Button(browseComposite, SWT.PUSH);
        browseButton.setText(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.button.browse"));
        browseButton.addSelectionListener(new SelectionAdapter()
        {
            public void widgetSelected(SelectionEvent e)
            {
                handleBrowseEpf();
            }
        });
    }

    /**
     * Browse directory for 'plugin_customization.ini' in a Directory Dialog
     * 
     */
    private void handleBrowseEpf()
    {
        DirectoryDialog dialogBrowse = new DirectoryDialog(getShell());
        dialogBrowse.setFilterPath(aPathPCIDirectory);
        String dialogOpen = dialogBrowse.open();
        // Browse Dialog is opened
        if (dialogOpen != null)
        {
            // Take the path of the repository
            aPathPCIDirectory = dialogOpen;
            handlePathPci();
        }
    }

    /**
     * Display the path pci in the text field
     */
    private void handlePathPci()
    {
        if (!aPathPCIDirectory.equals(""))
        {
            finish = true;
            setPageComplete(true);
            // Display in the text filed
            textPathPci.setText(aPathPCIDirectory);
            // Give parameter to Wizard
            wizard.setPciPath(aPathPCIDirectory + NAMEPCI);
            // Un-Display message
            wizard.getPluginCustomizationPage().setMessage(PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.message.description"));
        }
        else
        {
            // Display message
            wizard.getPluginCustomizationPage().setMessage(
                    PlugincustomizationPlugin.getResourceString("PluginCustomizationPage.message.missing.pathPCI"),
                    WizardPage.ERROR);
        }
    }

    /**
     * Create Label
     * 
     * @param text
     * @param parent
     */
    private void createLabel(String text, Composite parent)
    {
        final Label label = new Label(parent, SWT.NONE);
        label.setText(text);
    }
}
