/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.projecttotemplate.infos;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.projecttotemplate.infos.util.InfosTemplateProjectToTemplate;
import org.topcased.service.projecttotemplate.infos.util.ProjectToInfosTemplateProject;

/**
 * All informations which can be find in a "template file project" ('template.xml'). InfosTemplateProject is composed of severals Files and Directories it-self
 * composed of Files and Directories.
 * 
 * <template name="..." pathzip="..."> <project oldProjectName=...> <directory name="..."> [directory * | file *] </directory> * <file name="..."/> * </project>
 * </template>
 * 
 */
public class InfosTemplateProject
{
	// *********************
	// Declaration bloc
	// *********************
	// Infos Directory Map
	private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>();

	// Infos File Map
	private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

	// Template Name
	private String templateName = "";

	// Old Project Name
	private String oldProjectName = "";

	// Template Path ZIP File
	private String templatePathZipFile = "";

	// **********
	// XML TAG
	// **********
	/**
	 * Tag <template>
	 */
	public static final String TEMPLATE = "template";

	/**
	 * Tag <directory>
	 */
	public static final String DIRECTORY = "directory";

	/**
	 * Tag <file>
	 */
	public static final String FILE = "file";

	/**
	 * Tag <project>
	 */
	public static final String PROJECT = "project";

	// ***************
	// XML Attributs
	// ***************
	/**
	 * Attribut <template name=...>
	 */
	public static final String TEMPLATENAME = "name";

	/**
	 * Attribut <template name=...>
	 */
	public static final String TEMPLATEPATHZIPFILE = "pathzip";

	/**
	 * Attribut <project oldProjectName=...>
	 */
	public static final String PROJECTOLDNAME = "oldProjectName";

	/**
	 * Attribut <directory name=...>
	 */
	public static final String DIRECTORYNAME = "name";

	/**
	 * Attribut <file name=...>
	 */
	public static final String FILENAME = "name";

	/**
	 * Attribut <file path=...>
	 */
	public static final String FILEPATH = "path";

	/**
	 * Constructor
	 */
	public InfosTemplateProject()
	{
		// do nothing
	}

	/**
	 * Constructor : use to create a InfosTemplateProject from an existing directory
	 * 
	 * @param nameTemplate : template name
	 * @param directoryPath : directory path
	 * @param nameOldProject : old project name
	 */
	public InfosTemplateProject(String nameTemplate, String directoryPath)
	{
		boolean success = true;
		ProjectToInfosTemplateProject project = new ProjectToInfosTemplateProject();
		File fileProject = new File(directoryPath);

		oldProjectName = fileProject.getName();
		templateName = nameTemplate;
		if (directoryPath != null && !directoryPath.equals(""))
		{
			success = project.getInfosTemplateProjectFromDirectory(directoryPath);
			mapInfosDirectory = project.getMapInfosDirectory();
			mapInfosFile = project.getMapInfosFile();
			if (!success)
			{
				CorePlugin.log(this.getClass(), "Problem : The project selected is not read corectly. Internal error.", IStatus.ERROR);
			}
		}
		else
		{
			CorePlugin.log(this.getClass(), "The path of the project is incorrect : 'null' or 'void'", IStatus.ERROR);
		}
	}

	/**
	 * Constructor
	 * 
	 * @param nameTemplate : template name
	 * @param oldNameProject : old projectName
	 * @param pathZipFile
	 * @param infosDirectoryMap : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 * @param infosFileMap : Map of InfosFile (key=name ; value=InfosFile)
	 */
	public InfosTemplateProject(String nameTemplate, String oldNameProject, String pathZipFile,
			Map<String, InfosDirectory> infosDirectoryMap, Map<String, InfosFile> infosFileMap)
	{
		mapInfosDirectory = infosDirectoryMap;
		templatePathZipFile = pathZipFile;
		mapInfosFile = infosFileMap;
		templateName = nameTemplate;
		oldProjectName = oldNameProject;
	}

	/**
	 * Set template name
	 * 
	 * @param nameTemplate
	 */
	public void setTemplateName(String nameTemplate)
	{
		templateName = nameTemplate;
	}

	/**
	 * Set template zip file
	 * 
	 * @param pathZipFile
	 */
	public void setTemplatePathZipFile(String pathZipFile)
	{
		templatePathZipFile = pathZipFile;
	}

	/**
	 * Set oldNameFile
	 * 
	 * @param name the new name to set
	 */
	public void setOldProjectName(String name)
	{
		oldProjectName = name;
	}

	/**
	 * Set InfosDirectory Map
	 * 
	 * @param infosDirectoryMap : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 */
	public void setInfosDirectoryMap(Map<String, InfosDirectory> infosDirectoryMap)
	{
		mapInfosDirectory = infosDirectoryMap;
	}

	/**
	 * Set infosFileMap Map
	 * 
	 * @param infosFileMap : Map of infosFileMap (key=name ; value=InfosFile)
	 */
	public void setInfosFileMap(Map<String, InfosFile> infosFileMap)
	{
		mapInfosFile = infosFileMap;
	}

	/**
	 * Get templateName
	 * 
	 * @return template name
	 */
	public String getTemplateName()
	{
		return templateName;
	}

	/**
	 * Get template path zip file
	 * 
	 * @return path Zip File
	 */
	public String getTemplatePathZipFile()
	{
		return templatePathZipFile;
	}

	/**
	 * Get mapInfosDirectory
	 * 
	 * @return mapInfosDirectory : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 */
	public Map<String, InfosDirectory> getMapInfosDirectory()
	{
		return mapInfosDirectory;
	}

	/**
	 * Get mapInfosFile
	 * 
	 * @return mapInfosFile : Map of InfosFile (key=name ; value=InfosFile)
	 */
	public Map<String, InfosFile> getMapInfosFile()
	{
		return mapInfosFile;
	}

	/**
	 * Getter for the old project name
	 * 
	 * @return
	 */
	public String getOldProjectName()
	{
		return oldProjectName;
	}

	/**
	 * Create a zip file from the selected directory
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 * @return true if the zip file is created
	 */
	public boolean createZipFile(String pathSource, String pathDestination)
	{
		InfosTemplateProjectToTemplate infosTP = new InfosTemplateProjectToTemplate(this, oldProjectName);
		return infosTP.createZipFile(pathSource, pathDestination);
	}

	/**
	 * Create a zip file from the selected directory + and only with files selected.
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 */
	public void createZipFileSelected(String pathSource, String pathDestination)
	{
		InfosTemplateProjectToTemplate infosTP = new InfosTemplateProjectToTemplate(this, oldProjectName);
		infosTP.createZipFileSelected(pathSource, pathDestination);
	}

	/**
	 * Install the project. Create in the directory 'projectPath' the tree of the current project template. Unzip the Zip file (*.zip) define in
	 * InfosTemplateProject
	 * 
	 * @param projectPath : project path
	 * @return true if the project is correctly install
	 */
	/*
	 * public boolean installProject(String projectPath) { InfosTemplateProjectToInstallProject infosTP = new
	 * InfosTemplateProjectToInstallProject(mapInfosDirectory, mapInfosFile, templatePathZipFile, projectPath); return infosTP.installProject(); }
	 */

	/**
	 * Take informations about the template project templatePath (.xml); Initialize private Map : - 'mapInfosDirectory'(name, InfosDirectory), - 'mapInfosFile'
	 * (name, InfosFile)
	 * 
	 * @param templatePath
	 */
	/*
	 * public void getInfosTemplateProject(String templatePath) { TemplateToInfosTemplateProject template = new TemplateToInfosTemplateProject();
	 * template.getInfosTemplateProject(templatePath); mapInfosDirectory = template.getMapInfosDirectory(); templatePathZipFile =
	 * template.getTemplatePathZipFile(); mapInfosFile = template.getMapInfosFile(); templateName = template.getTemplateName(); templatePathZipFile =
	 * template.getTemplatePathZipFile(); }
	 */

	/**
	 * Create the template Project (.xml + .zip )about the template project templatePath
	 * 
	 * @param templatePath : path of the xml file (.xml)
	 * @param zipFilePath : path of the zip file (.zip)
	 * @return true if the template project is correctly created
	 */
	public boolean createTemplateProject(String templatePath, String zipFilePath)
	{
		InfosTemplateProjectToTemplate infosTP = new InfosTemplateProjectToTemplate(this, oldProjectName);
		return infosTP.createTemplateProject(templatePath, zipFilePath);
	}

}
