/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.projecttotemplate.wizard;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.topcased.service.projecttotemplate.ProjectToTemplatePlugin;
import org.topcased.service.projecttotemplate.infos.InfosDirectory;
import org.topcased.service.projecttotemplate.infos.InfosFile;
import org.topcased.service.projecttotemplate.infos.InfosTemplateProject;
import org.topcased.service.projecttotemplate.preferences.ProjectToTemplatePreferencePage;
import org.topcased.service.projecttotemplate.service.CreateTemplate;
import org.topcased.service.projecttotemplate.structure.TemplateProjectTree;
import org.topcased.service.projecttotemplate.structure.TemplateProjectTreeContentProvider;
import org.topcased.service.projecttotemplate.structure.TemplateProjectTreeLabelProvider;

/**
 * Display the wizard to create a template (xml file) from a project selected in the current WorkSpace.
 */
public class ProjectToTemplatePage extends WizardPage
{
	/**
	 * Wizard projectToTemplate
	 */
	ProjectToTemplateWizard wizard;

	/**
	 * Finish indicator
	 */
	boolean finish;

	/**
	 * Tree viewer : project content
	 */
	private TreeViewer treeViewer;

	/**
	 * Tree TemplateProject
	 */
	private TemplateProjectTree aTemplateProjectTree = null;

	/**
	 * Map Infos TemplateProject : InfosDirectory
	 */
	private Map<String, InfosDirectory> aMapDirectory = new HashMap<String, InfosDirectory>();

	/**
	 * Map Infos TemplateProject : InfosFile
	 */
	private Map<String, InfosFile> aMapFile = new HashMap<String, InfosFile>();

	/**
	 * Class InfosTemplateProject to construct the project from template : *.xml file
	 */
	private InfosTemplateProject aInfosTemplateProject = new InfosTemplateProject();

	/**
	 * Path of repository of template (all *.xml) "aTemplateXMLDirectory" must come from "Preferences"
	 */
	private String aTemplateXMLDirectory = "";

	/**
	 * Template name
	 */
	private String templateName = "DefaultNameProject";

	/**
	 * Project path : Workspace
	 */
	private String pathProject = Platform.getInstanceLocation().getURL().getPath();

	/**
	 * Constructor
	 * 
	 * @param projectPath : project path which will be transformed in a temaplate (xml file)
	 * @param projectToTemplate : wizard
	 */
	public ProjectToTemplatePage(IPath projectPath, ProjectToTemplateWizard projectToTemplate)
	{
		super("ProjectToTemplatePage");
		templateName = projectPath.toFile().getName();
		pathProject = projectPath.toString();

		aInfosTemplateProject = new InfosTemplateProject(templateName, pathProject);
		aMapDirectory = aInfosTemplateProject.getMapInfosDirectory();
		aMapFile = aInfosTemplateProject.getMapInfosFile();
		wizard = projectToTemplate;
	}

	/**
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout(1, false);
		composite.setLayout(gridLayout);
		setControl(composite);

		// Group projet
		createGroupProject(composite);

		// Composite Tree
		Composite compositeTree = new Composite(composite, SWT.NONE);
		GridLayout gridLayout2 = new GridLayout(2, false);
		compositeTree.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER | GridData.FILL_BOTH));
		compositeTree.setLayout(gridLayout2);

		createTreeViewer(compositeTree);

		handleFinish();
	}

	/**
	 * Create the TreeViewer
	 * 
	 * @param parent
	 */
	private void createTreeViewer(Composite parent)
	{
		treeViewer = new TreeViewer(parent, SWT.BORDER);
		final Tree tree = treeViewer.getTree();
		final GridData gridDataTree = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridDataTree.verticalSpan = 2;
		tree.setLayoutData(gridDataTree);
		treeViewer.setContentProvider(new TemplateProjectTreeContentProvider());
		treeViewer.setLabelProvider(new TemplateProjectTreeLabelProvider());

		// New TemplateProjectTree
		aTemplateProjectTree = new TemplateProjectTree(null, aMapDirectory, aMapFile, templateName, pathProject, true);
		aMapDirectory = aTemplateProjectTree.getMapInfosDirectory();
		aMapFile = aTemplateProjectTree.getMapInfosFile();

		treeViewer.setInput(new Object[] { aTemplateProjectTree });

		// Is it finish ?
		handleFinish();

	}

	/**
	 * Create Group : template name group
	 * 
	 * @param parent
	 */
	private void createGroupProject(Composite parent)
	{
		Group groupTemplateName = new Group(parent, SWT.NONE);
		final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		final Text templateNameText = new Text(groupTemplateName, SWT.BORDER);
		final GridData gridDataPName = new GridData(GridData.FILL_HORIZONTAL);
		groupTemplateName.setLayout(new GridLayout());
		groupTemplateName.setText(ProjectToTemplatePlugin.getResourceString("ProjectToTemplatePage.grouptemplate"));
		groupTemplateName.setLayoutData(gridData);

		templateNameText.setText(templateName);
		templateNameText.selectAll();
		templateNameText.setLayoutData(gridDataPName);
		templateNameText.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				handleTemplateName(templateNameText);
			}
		});
	}

	/**
	 * Install project : install the template in the ProjectPath (workspace)
	 * 
	 * @return finish page value
	 */
	public boolean finish()
	{
		String templateNameTrim = templateName.trim();
		String[] strings = templateName.split("");
		templateNameTrim = "";
		for (int i = 0; i < strings.length; i++)
		{
			String s = strings[i].trim();
			templateNameTrim = templateNameTrim + s;
		}

		String basePath = aTemplateXMLDirectory + File.separator + templateNameTrim;
		aInfosTemplateProject.setTemplatePathZipFile(basePath + ".zip"); // Not necessary
		aInfosTemplateProject.setTemplateName(templateName);

		// Call SERVICE CREATE TEMPLATE
		Map<String, Object> mapInput = new HashMap<String, Object>();
		mapInput.put("TemplateName", templateName);
		mapInput.put("PathProject", pathProject);
		mapInput.put("PathTemplateZip", basePath + ".zip");
		mapInput.put("PathTemplateXml", basePath + ".xml");
		CreateTemplate createTemplateService = new CreateTemplate();
		createTemplateService.serviceRun(mapInput);

		return finish;
	}

	/**
	 * Handle Finish
	 */
	public void handleFinish()
	{
		aTemplateXMLDirectory = ProjectToTemplatePlugin.getDefault().getPreferenceStore().getString(
				ProjectToTemplatePreferencePage.TEMPLATE_REPOSITORY_PATH);
		if (!templateName.equals("") && !aTemplateXMLDirectory.equals(""))
		{
			finish = true;
			setPageComplete(true);
			wizard.getProjectToTemplatePage().setMessage(ProjectToTemplatePlugin.getResourceString("ProjectToTemplatePage.message"));
		}
		else
		{
			wizard.getProjectToTemplatePage().setMessage(ProjectToTemplatePlugin.getResourceString("ProjectToTemplatePage.message.error"),
					WizardPage.ERROR);
			finish = false;
			setPageComplete(false);
		}
	}

	/**
	 * Take le value (which is changing) of the project name text area
	 * 
	 * @param nameTemplate : template name
	 */
	private void handleTemplateName(Text nameTemplate)
	{
		templateName = nameTemplate.getText();
		// Refresh tree
		aTemplateProjectTree.setName(templateName);
		treeViewer.setInput(new Object[] { aTemplateProjectTree });
		handleFinish();
	}
}