/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.templatetoproject.infos;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.topcased.service.templatetoproject.infos.util.InfosTemplateProjectToInstallProject;
import org.topcased.service.templatetoproject.infos.util.TemplateToInfosTemplateProject;

/**
 * All informations which can be find in a "template file project" ('template.xml'). <br>
 * InfosTemplateProjOect is composed of severals Files and Directories it-self composed of Files and Directories.
 * 
 * <code>
 * <template name="..." pathzip="...">
 *  <project oldProjectName="..">
 *   <directory name="..."> 
 *   [directory * | file *] 
 *   </directory> 
 *   <file name="..."/> *
 * </project> 
 * </template>
 * </code>
 * 
 */
public class InfosTemplateProject
{
	/**
	 * Declaration bloc
	 */
	// Infos Directory Map
	private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>();

	// Infos File Map
	private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>();

	// Infos Variables Map
	private Map<String, String> mapVariables = new TreeMap<String, String>();

	// Template Name
	private String templateName = "";

	// Old Project Name
	private String oldProjectName = "";

	// Template Path ZIP File
	private String templatePathZipFile = "";

	// XML TAG
	/**
	 * Tag <template>
	 */
	public static final String TEMPLATE = "template";

	/**
	 * Tag <directory>
	 */
	public static final String DIRECTORY = "directory";

	/**
	 * Tag <file>
	 */
	public static final String FILE = "file";

	/**
	 * Tag <project>
	 */
	public static final String PROJECT = "project";

	// XML Attributs
	/**
	 * Attribut <template name=...>
	 */
	public static final String TEMPLATENAME = "name";

	/**
	 * Attribut <template name=...>
	 */
	public static final String TEMPLATEPATHZIPFILE = "pathzip";

	/**
	 * Attribut <project oldProjectName=...>
	 */
	public static final String PROJECTOLDNAME = "oldProjectName";

	/**
	 * Attribut <directory name=...>
	 */
	public static final String DIRECTORYNAME = "name";

	/**
	 * Attribut <file name=...>
	 */
	public static final String FILENAME = "name";

	/**
	 * Attribut <file path=...>
	 */
	public static final String FILEPATH = "path";

	public InfosTemplateProject()
	{
		// do nothing
	}

	/**
	 * Constructor
	 * 
	 * @param nameTemplate
	 * @param pathZipFile
	 * @param infosDirectoryMap : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 * @param infosFileMap : Map of InfosFile (key=name ; value=InfosFile)
	 */
	public InfosTemplateProject(String nameTemplate, String pathZipFile, Map<String, InfosDirectory> infosDirectoryMap,
			Map<String, InfosFile> infosFileMap)
	{
		mapInfosDirectory = infosDirectoryMap;
		templatePathZipFile = pathZipFile;
		mapInfosFile = infosFileMap;
		templateName = nameTemplate;
	}

	/**
	 * Set template name
	 * 
	 * @param nameTemplate
	 */
	public void setTemplateName(String nameTemplate)
	{
		templateName = nameTemplate;
	}

	/**
	 * Set old project name
	 * 
	 * @param oldNameproject
	 */
	public void setOldProjectName(String oldNameproject)
	{
		oldProjectName = oldNameproject;
	}

	/**
	 * Set template zip file
	 * 
	 * @param pathZipFile
	 */
	public void setTemplatePathZipFile(String pathZipFile)
	{
		templatePathZipFile = pathZipFile;
	}

	/**
	 * Set InfosDirectory Map
	 * 
	 * @param infosDirectoryMap : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 */
	public void setInfosDirectoryMap(Map<String, InfosDirectory> infosDirectoryMap)
	{
		mapInfosDirectory = infosDirectoryMap;
	}

	/**
	 * Set infosFileMap Map
	 * 
	 * @param infosFileMap : Map of infosFileMap (key=name ; value=InfosFile)
	 */
	public void setInfosFileMap(Map<String, InfosFile> infosFileMap)
	{
		mapInfosFile = infosFileMap;
	}

	/**
	 * Set variables vector
	 * 
	 * @param templateVar : vector containing all the template variables read.
	 */
	public void setTemplateVariables(Map<String, String> templateVar)
	{
		mapVariables = templateVar;
	}

	/**
	 * Get templateName
	 * 
	 * @return template name
	 */
	public String getTemplateName()
	{
		return templateName;
	}

	/**
	 * Get template path zip file
	 * 
	 * @return path Zip File
	 */
	public String getTemplatePathZipFile()
	{
		return templatePathZipFile;
	}

	/**
	 * Getter for the old project name
	 * 
	 * @return
	 */
	public String getOldProjectName()
	{
		return oldProjectName;
	}

	/**
	 * Get mapInfosDirectory
	 * 
	 * @return mapInfosDirectory : Map of InfosDirectory (key=name ; value=InfosDirectory)
	 */
	public Map<String, InfosDirectory> getMapInfosDirectory()
	{
		return mapInfosDirectory;
	}

	/**
	 * Get mapInfosFile
	 * 
	 * @return mapInfosFile : Map of InfosFile (key=name ; value=InfosFile)
	 */
	public Map<String, InfosFile> getMapInfosFile()
	{
		return mapInfosFile;
	}

	/**
	 * Get variables map
	 * 
	 * @return variables : a map containing all the variables read from a given xml file
	 */
	public Map<String, String> getMapTemplateVariables()
	{
		return mapVariables;
	}

	public void setMapTemplateVariable(String name, String value)
	{
		mapVariables.put(name, value);
	}

	/**
	 * Install the project. Create in the directory 'projectPath' the tree of the current project template. Unzip the Zip file (*.zip) define in
	 * InfosTemplateProject
	 * 
	 * @param projectPath : project path
	 * @return true if the project is correctly install
	 */
	public boolean installProject(String projectPath)
	{
		InfosTemplateProjectToInstallProject infosTP = new InfosTemplateProjectToInstallProject(mapInfosDirectory, mapInfosFile,
				mapVariables, templatePathZipFile, projectPath, oldProjectName);
		return infosTP.installProject();
	}

	/**
	 * Take informations about the template project templatePath (.xml); Initialize private Map : - 'mapInfosDirectory'(name, InfosDirectory), - 'mapInfosFile'
	 * (name, InfosFile)
	 * 
	 * @param templatePath
	 */
	public void getInfosTemplateProject(String templatePath)
	{
		TemplateToInfosTemplateProject template = new TemplateToInfosTemplateProject();
		template.getInfosTemplateProject(templatePath);
		mapInfosDirectory = template.getMapInfosDirectory();
		templatePathZipFile = template.getTemplatePathZipFile();
		mapInfosFile = template.getMapInfosFile();
		templateName = template.getTemplateName();
		templatePathZipFile = template.getTemplatePathZipFile();
		oldProjectName = template.getOldProjectName();
		mapVariables = template.getMapVariables();

	}

	/**
	 * This method replaces all the variables previously found in a template xml file by their value.
	 */
	/*
	 * public void replaceVariableByValue() { Iterator it = mapVariables.keySet().iterator();
	 * 
	 * while (it.hasNext()) { String variableName = (String)it.next(); String variableValue = (String)mapVariables.get(variableName);
	 *  // Test on the zip file path if (templatePathZipFile.equals(variableName)) { templatePathZipFile = variableValue; }
	 *  // Test on the old project name if (oldProjectName.equals(variableName)) { oldProjectName = variableValue; }
	 *  // Test on the template name if (templateName.equals(variableName)) { templateName = variableValue; }
	 *  // Test on the file structure if (mapInfosFile.containsKey(variableName)) { InfosFile fileToRename = (InfosFile)mapInfosFile.get(variableName);
	 * fileToRename.setFileName(variableValue); }
	 *  // Test on the directory structure if (mapInfosDirectory.containsKey(variableName)) { InfosDirectory directoryToRename =
	 * (InfosDirectory)mapInfosDirectory.get(variableName); directoryToRename.setDirectoryName(variableValue); } } }
	 */

	/**
	 * Create the template Project (.xml + .zip )about the template project templatePath
	 * 
	 * @param templatePath : path of the xml file (.xml)
	 * @param zipFilePath : path of the zip file (.zip)
	 * @return true if the template project is correctly created
	 */
	/*
	 * public boolean createTemplateProject(String templatePath, String zipFilePath) { InfosTemplateProjectToTemplate infosTP = new
	 * InfosTemplateProjectToTemplate(this); return infosTP.createTemplateProject(templatePath, zipFilePath); }
	 */

	/**
	 * Create a zip file from the selected directory
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 * @return true if the zip file is created
	 */
	/*
	 * public boolean createZipFile(String pathSource, String pathDestination) { InfosTemplateProjectToTemplate infosTP = new
	 * InfosTemplateProjectToTemplate(this); return infosTP.createZipFile(pathSource, pathDestination); }
	 */

	/**
	 * Create a zip file from the selected directory + and only with files selected.
	 * 
	 * @param pathSource : path of the zip file to create(.zip)
	 * @param pathDestination : path of the directory to zip
	 */
	/*
	 * public void createZipFileSelected(String pathSource, String pathDestination) { InfosTemplateProjectToTemplate infosTP = new
	 * InfosTemplateProjectToTemplate(this); infosTP.createZipFileSelected(pathSource, pathDestination); }
	 */

	/**
	 * Constructor : use to create a InfosTemplateProject from an existing directory
	 * 
	 * @param nameTemplate : template name
	 * @param directoryPath : directory path
	 */
	/*
	 * public InfosTemplateProject(String nameTemplate, String directoryPath) { boolean success = true; ProjectToInfosTemplateProject project = new
	 * ProjectToInfosTemplateProject();
	 * 
	 * templateName = nameTemplate; if (directoryPath != null && !directoryPath.equals("")) { success =
	 * project.getInfosTemplateProjectFromDirectory(directoryPath); mapInfosDirectory = project.getMapInfosDirectory(); mapInfosFile =
	 * project.getMapInfosFile(); if (!success) { CorePlugin.log(this.getClass(), "Problem : The project selected is not read corectly. It's a internal error.",
	 * IStatus.ERROR); } } else { CorePlugin.log(this.getClass(), "The path of the project is incorrect : 'null' or 'void'", IStatus.ERROR); } }
	 */

}
