/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************/
package org.topcased.service.templatetoproject.infos.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.templatetoproject.infos.InfosDirectory;
import org.topcased.service.templatetoproject.infos.InfosFile;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * InfosTemplateProjectToInstallProject install a project with all informations come from the class InfosTemplateProject. (All informations come from a template
 * file (.xml))
 * 
 * InfosTemplateProjectToInstallProject is used by InfosTemplateProject.
 * 
 * 
 */
public class InfosTemplateProjectToInstallProject
{
	// Infos Directory Map
	private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>(); 

	// Infos File Map
	private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>(); 

	// Infos Variables Map
	private Map<String, String> mapVariable = new TreeMap<String, String>(); 

	// Template Path ZIP File
	private String templatePathZipFile = "";

	// Template : path project
	private String templatePathProject = "";

	// Project name
	private String projectName = "";

	// old Project Name
	private String oldProjectName = "";

	// TAG <projectDescription> : '.project'
	private static String PROJECTDESCIPTION = "projectDescription";

	// TAG <name> : '.project'
	private static String PROJECTNAME = "name";

	// TAG <comment> : '.project'
	private static String PROJECTCOMMENT = "comment";

	// TAG <projects> : '.project'
	private static String PROJECTPROJECTS = "projects";

	// TAG <buildSpec> : '.project'
	private static String PROJECTBUILDSPEC = "buildSpec";

	// TAG <nature> : '.project'
	private static String PROJECTNATURE = "nature";

	/**
	 * Constructor
	 * 
	 * @param aMapInfosDirectory
	 * @param aMapInfosFile
	 * @param aTemplatePathZipFile
	 * @param aTemplatePathProject
	 * @param oldNameProject : old project name
	 */
	public InfosTemplateProjectToInstallProject(Map<String, InfosDirectory> aMapInfosDirectory, Map<String, InfosFile> aMapInfosFile, Map<String, String> aMapInfosVariable,
			String aTemplatePathZipFile, String aTemplatePathProject, String oldNameProject)
	{
		mapInfosDirectory = aMapInfosDirectory;
		mapInfosFile = aMapInfosFile;
		mapVariable = aMapInfosVariable;
		templatePathZipFile = aTemplatePathZipFile;
		templatePathProject = aTemplatePathProject;
		projectName = new File(templatePathProject).getName();
		oldProjectName = oldNameProject;
	}

	// **************************************************************
	//
	// Install the project with InfosTemplateProject
	//
	// **************************************************************
	/**
	 * Install the project. Create in the directory 'projectPath' the tree of the current project template. Unzip the Zip file (*.zip) define in
	 * InfosTemplateProject
	 * 
	 * @return true if the project is correctly installed
	 * 
	 */
	public boolean installProject()
	{
		boolean continu = true;
		boolean isOKreturn = false;
		boolean isInstallZipOK = true;
		boolean isInstallFiles = true;
		boolean isInstallDirectories = true;
		boolean isModified = true;

		File projectDirectory = new File(templatePathProject);
		if (continu)
		{
			boolean doInstallZip = true;
			boolean doInstallXml = false;
			boolean doModifyContent = true;

			// Create project if it does not exits yet
			projectDirectory.mkdirs();

			// Unzip the template archive
			isInstallZipOK = unzip(templatePathZipFile, templatePathProject, doInstallZip);

			// Install Files
			isInstallFiles = installFiles(mapInfosFile, templatePathProject, "", doInstallXml);

			// Install Directories
			isInstallDirectories = installDirectories(mapInfosDirectory, templatePathProject, "", doInstallXml);

			// Modify ".project"
			isModified = modifyProjectFile();

			if (doModifyContent)
			{
				File projectRespository = new File(templatePathProject);
				changeContentFile(projectRespository, oldProjectName, projectName);
			}

			// Open it in WorkSpace
			openProjectInWorkSpace();

			// Display Informations
			if (isInstallZipOK & isInstallFiles & isInstallDirectories)
			{
				// CorePlugin.log(this.getClass(), "The project is successfully
				// installed in : " + templatePathProject, IStatus.INFO);

				isOKreturn = true;
				if (!isModified)
				{
					CorePlugin.log(this.getClass(), "Problem when modify '.project'.", IStatus.ERROR);
				}
			}
			else
			{
				isOKreturn = true;
				if (!isInstallZipOK && doInstallZip)
				{
					CorePlugin.log(this.getClass(), "Problem when install the unzip '" + templatePathZipFile + "' in "
							+ templatePathProject, IStatus.ERROR);
					isOKreturn = false;
				}
				if (!isInstallFiles && doInstallXml)
				{
					CorePlugin.log(this.getClass(), "Problem when install Files in " + templatePathProject, IStatus.ERROR);
					isOKreturn = false;
				}
				if (!isInstallDirectories && doInstallXml)
				{
					CorePlugin.log(this.getClass(), "Problem when install Directories in " + templatePathProject, IStatus.ERROR);
					isOKreturn = false;
				}
				if (!isModified)
				{
					CorePlugin.log(this.getClass(), "Problem when modify '.project'.", IStatus.ERROR);
					isOKreturn = false;
				}
			}
		}
		return isOKreturn;
	}

	/**
	 * Unzip file 'pathZipFile' in the directory 'pathToUnzip'
	 * 
	 * @param pathZipFile
	 * @param pathToUnzip
	 * @param go : unzip if 'go' is true
	 * @return true if the unzip works
	 */
	private boolean unzip(String pathZipFile, String pathToUnzip, boolean go)
	{
		boolean successUnzip = false;
		if (go)
		{
			File pathFile = new File(pathZipFile);
			if (pathFile.exists())
			{
				try
				{
					// UnZip all files
					ZipInputStream input = new ZipInputStream(new FileInputStream(pathFile));

					String zipEntryName = null;
					ZipEntry zipEntry = null;
					while ((zipEntry = input.getNextEntry()) != null)
					{
						zipEntryName = zipEntry.getName();

						// Modify the name of the zip path entry if it contains a '%' character.
						zipEntryName = zipEntryName.replaceAll("%", "");

						// Extract and replace if necessary
						String fileName = replaceVariableByValueInPathFile(zipEntryName);
						File fileOutput = new File(pathToUnzip + File.separator + fileName);

						// Change the name of the file if it is necessary
						String fileNameModify = modifyChain(fileOutput.getName(), oldProjectName, projectName);
						if (!fileName.equals(fileOutput.getName()))
						{
							String newFileModify = fileOutput.getParentFile() + File.separator + fileNameModify;
							fileOutput = new File(newFileModify);
						}

						// Creation of parent repository if does not exist
						File repositoryParent = fileOutput.getParentFile();
						repositoryParent.mkdirs();

						// Writing data
						FileOutputStream output = new FileOutputStream(fileOutput);
						byte[] buffer = new byte[1024];
						int length;

						while ((length = input.read(buffer)) != -1)
						{
							output.write(buffer, 0, length);
						}
						output.close();
					}
					input.close();

					successUnzip = true;

				}
				catch (FileNotFoundException e)
				{
					CorePlugin.log(this.getClass(), "File Not Found Exception for " + pathToUnzip + ". " + e.getMessage(), IStatus.ERROR);
				}
				catch (IOException e)
				{
					CorePlugin.log(this.getClass(), "IO Exception " + pathToUnzip + ". " + e.getMessage(), IStatus.ERROR);
				}
				catch (Exception e)
				{
					CorePlugin.log(this.getClass(), "Exception " + pathToUnzip + ". " + e.getMessage(), IStatus.ERROR);
				}
			}
			else
			{
				CorePlugin.log(this.getClass(), "The path " + pathZipFile + " does not exist. It is not unzip in " + pathToUnzip,
						IStatus.INFO);
			}
		}
		return successUnzip;
	}

	/**
	 * Install File indicate in the Map 'mapInfoFile' in the directory 'projectPath/relativePath'
	 * 
	 * @param mapInfoFile : Map of InfoFile (key=name, value = InfosFile)
	 * @param projectPath
	 * @param relativePath
	 * @param go : install files if go is 'true'
	 * @return true if the Files are successfully installed
	 */
	private boolean installFiles(Map<String, InfosFile> mapInfoFile, String projectPath, String relativePath, boolean go)
	{
		boolean isOK = true;
		if (go)
		{
			InfosFile aInfoF = new InfosFile();
			String aRelativePath = "";

			// Create Repository of the file path if it does not exist.
			String newFilePath = projectPath + File.separator + relativePath;
			File newFile = new File(newFilePath);
			newFile.getParentFile().mkdirs();

			// Loop on 'mapInfoFile'
			Iterator<String> iter = mapInfoFile.keySet().iterator();
			while (iter.hasNext() & isOK)
			{
				String info = iter.next();
				aInfoF = (InfosFile) mapInfoFile.get(info);
				aRelativePath = aInfoF.getFilePath();
				isOK = installFile(projectPath + File.separator + aRelativePath);
			}
		}
		return isOK;
	}

	/**
	 * Create the file 'pathFileToCreate'
	 * 
	 * @param pathFileToCreate
	 * @return true if the File is successfully created
	 */
	private boolean installFile(String pathFileToCreate)
	{
		File newFile = new File(pathFileToCreate);

		// Change the name of the file if it is necessary
		String fileName = modifyChain(newFile.getName(), oldProjectName, projectName);
		if (!fileName.equals(newFile.getName()))
		{
			String newFileModify = newFile.getParentFile() + File.separator + fileName;
			newFile = new File(newFileModify);
		}

		try
		{
			newFile.createNewFile();
		}
		catch (IOException e)
		{
			CorePlugin.log(this.getClass(), "IO Exception : the file " + pathFileToCreate + " is not create." + e.getMessage(),
					IStatus.ERROR);
			return false;
		}
		return true;
	}

	/**
	 * Install Directory indicate in the Map 'mapInfoDirectory' in the directory 'projectPath/relativePath'
	 * 
	 * @param mapInfoDirectory : Map of InfoDirectory (key=name, value = InfosDirectory)
	 * @param projectPath
	 * @param relativePath
	 * @param go : install directories if 'go' is true
	 * @return true if the Directories are successfully installed
	 */
	private boolean installDirectories(Map<String, InfosDirectory> mapInfoDirectory, String projectPath, String relativePath, boolean go)
	{
		boolean isOKreturn = true;
		if (go)
		{
			Map<String, InfosDirectory> aMapInfoD = new HashMap<String, InfosDirectory>();
			Map<String, InfosFile> aMapInfoF = new HashMap<String, InfosFile>();
			InfosDirectory aInfoD = new InfosDirectory();
//			InfosFile aInfoF = new InfosFile();
			String aRelativePath = "";
			// Create Repository path
			String newDirectoryPath = projectPath + File.separator + relativePath;
			File newDirectory = new File(newDirectoryPath);
			newDirectory.mkdirs();
			// Loop on 'mapInfoDirectory'
			Iterator<String> iter = mapInfoDirectory.keySet().iterator();
			while (iter.hasNext() & isOKreturn)
			{
				String infoDir = iter.next();
				aInfoD = mapInfoDirectory.get(infoDir);
				aMapInfoD = aInfoD.getMapInfosDirectory();
				aRelativePath = aInfoD.getDirectoryPath();
				aMapInfoF = aInfoD.getMapInfosFile();
				isOKreturn = installDirectories(aMapInfoD, projectPath, aRelativePath, true);
				if (isOKreturn)
				{
					isOKreturn = installFiles(aMapInfoF, projectPath, aRelativePath, true);
				}
//				else if (mapInfoDirectory.get(key) instanceof InfosFile)
//				{
//					aInfoF = (InfosFile) mapInfoDirectory.get(key);
//					aRelativePath = aInfoF.getFilePath();
//					isOKreturn = installFile(projectPath + File.separator + aRelativePath);
//				}
			}
		}
		else
		{
			isOKreturn = false;
		}

		return isOKreturn;
	}

	/**
	 * Open project (templatePathProject) in the Workspace
	 */
	public void openProjectInWorkSpace()
	{
		// bloc declaration
		boolean isOpen = false;

		try
		{
			if (templatePathProject.equals(ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString() + File.separator
					+ projectName))
			{
				IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
				project.create(new NullProgressMonitor());
				project.open(new NullProgressMonitor());
				isOpen = true;
			}
			else
			{
				IProjectDescription ip = ResourcesPlugin.getWorkspace().loadProjectDescription(
						new Path(templatePathProject + File.separator + IProjectDescription.DESCRIPTION_FILE_NAME));
				IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(ip.getName());
				project.create(ip, null);
				project.open(new NullProgressMonitor());
				isOpen = true;
			}

		}
		catch (CoreException e)
		{
			CorePlugin.log(this.getClass(), "Core Exception : " + e + ". If the project already exists in the workSpace, 'refresh' it.",
					IStatus.ERROR);
		}

		if (isOpen)
		{
			CorePlugin.log(this.getClass(), projectName + " is opened in the Workspace.", IStatus.INFO);
		}
		else
		{
			CorePlugin.log(this.getClass(), projectName + " can not be opened in the Workspace.", IStatus.ERROR);
		}
	}

	/**
	 * 
	 * @param ancestorName : the Name of the ancestor to found
	 * @param childrenNode : the node whose content to replace
	 * @return boolean
	 */
	private boolean ancestor(String ancestorName, Node childrenNode)
	{
		boolean ancestorFound = false;
		while (!ancestorFound && childrenNode.getParentNode() != null)
		{
			if ((childrenNode.getParentNode().getNodeName().compareTo(ancestorName)) == 0)
			{
				ancestorFound = true;
			}
			childrenNode = childrenNode.getParentNode();
		}
		return ancestorFound;
	}

	/**
	 * Modify tag <name> in the file '.project'
	 * 
	 * @return true if '.project' is correctly modified
	 */
	private boolean modifyProjectFile()
	{
		// bloc declaration
		boolean isModified = false;
		String pathFile = templatePathProject + File.separator + ".project";
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setIgnoringComments(true);
		factory.setCoalescing(true);
		// Parser
		try
		{
			DocumentBuilder parser = factory.newDocumentBuilder();
			// Document
			Document document = parser.parse(new File(pathFile + File.separator));

			// <projectDescription>
			NodeList projectdescriptorList = document.getElementsByTagName(PROJECTDESCIPTION);
			int nbNodeReq = projectdescriptorList.getLength();
			for (int i = 0; i < nbNodeReq; i++)
			{
				Element projectdescriptorNode = (Element) projectdescriptorList.item(i);
				// <name>
				NodeList projectNameList = projectdescriptorNode.getElementsByTagName(PROJECTNAME);
				int nbProject = projectNameList.getLength();
				for (int k = 0; k < nbProject; k++)
				{
					Node projectNameNode = projectNameList.item(k);
					if (!ancestor("buildSpec", projectNameNode))
					{
						projectNameNode.getFirstChild().setNodeValue(projectName);
					}
				}
			}
			// Xml write
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();

			// Indent the XML document
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");

			DOMSource sourceDom = new DOMSource(document);
			StreamResult result = new StreamResult(pathFile + File.separator);
			transformer.transform(sourceDom, result);

			isModified = true;
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
		}
		catch (SAXException e)
		{
			CorePlugin.log(this.getClass(), pathFile + " is void. '.project' file is filled.", IStatus.WARNING);
			isModified = createProjectFile(pathFile, projectName);
		}
		catch (IOException e)
		{
			CorePlugin.log(this.getClass(), pathFile + " does not exist. '.project' is created.", IStatus.WARNING);
			isModified = createProjectFile(pathFile, projectName);
		}
		catch (TransformerConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "Error while writing file : " + e.getMessage(), IStatus.ERROR);
		}
		catch (TransformerException e)
		{
			CorePlugin.log(this.getClass(), "Error while writing file : " + e.getMessage(), IStatus.ERROR);
		}
		return isModified;
	}

	/**
	 * Create Project File '.project' <projectDescription><name/><comment/><project/><buildSpec/><nature/></projectDescription>
	 * 
	 * @param projectPath : path of the project
	 * @param projectNameString : name of the project
	 * @return true if the project is correctly created
	 */
	private boolean createProjectFile(String projectPath, String projectNameString)
	{
		boolean isCreate = false;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			// Tag creation : <projectDescription>
			Element projectDescptionTag = document.createElement(PROJECTDESCIPTION);
			document.appendChild(projectDescptionTag);

			// Tag <name>...</name>
			Element nameTag = document.createElement(PROJECTNAME);
			projectDescptionTag.appendChild(nameTag);
			nameTag.setNodeValue(projectNameString);

			// Tag <comment>
			Element commentTag = document.createElement(PROJECTCOMMENT);
			projectDescptionTag.appendChild(commentTag);

			// Tag <project>
			Element projectTag = document.createElement(PROJECTPROJECTS);
			projectDescptionTag.appendChild(projectTag);

			// Tag <buildSpec>
			Element buildSpecTag = document.createElement(PROJECTBUILDSPEC);
			projectDescptionTag.appendChild(buildSpecTag);

			// Tag <nature>
			Element natureTag = document.createElement(PROJECTNATURE);
			projectDescptionTag.appendChild(natureTag);

			// Xml write
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer();
			// Indent the XML document
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3");
			DOMSource sourceDom = new DOMSource(document);
			StreamResult result = new StreamResult(projectPath);
			transformer.transform(sourceDom, result);

			isCreate = true;
		}
		catch (ParserConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "Parse Configuration Exception" + e.getMessage(), IStatus.ERROR);
		}
		catch (TransformerConfigurationException e)
		{
			CorePlugin.log(this.getClass(), "Transformer Configuration Exception" + e.getMessage(), IStatus.ERROR);
		}
		catch (TransformerException e)
		{
			CorePlugin.log(this.getClass(), "Transformer Exception" + e.getMessage(), IStatus.ERROR);
		}
		catch (Exception e)
		{
			CorePlugin.log(this.getClass(), "Exception " + e.getMessage(), IStatus.ERROR);
		}
		return isCreate;
	}

	/**
	 * If the chain 'chain' contains 'oldName', it is replaced by 'newName'
	 * 
	 * @param chain
	 * @param oldName : name of the old project
	 * @param newName : name of the new project (template)
	 * @return title or a other string if the tiltle contain the old project name
	 */
	private String modifyChain(String chain, String oldName, String newName)
	{
		String chainReturn = chain;
		// "chain" contains "oldName"?
		int index = chain.indexOf(oldName);
		if (index != -1)
		{
			String begin = chain.substring(0, index);
			String end = chain.substring(index + oldName.length());
			// recursive call on the last part of the string
			chainReturn = begin + newName + modifyChain(end, oldName, newName);
		}
		return chainReturn;
	}

	private void changeContentFile(File file, String oldValue, String newValue)
	{
		try
		{
			// Recursive call if it is a directory
			if (file.isDirectory())
			{
				File[] files = file.listFiles();
				for (int i = 0; i < files.length; i++)
				{
					changeContentFile(files[i], oldValue, newValue);
				}
				return;
			}

			// No treatement if it is a ASCII or BINARY file
			if (file.getName().endsWith(".jar") || file.getName().endsWith(".zip") || file.getName().endsWith(".gif")
					|| file.getName().endsWith(".jpg") || file.getName().endsWith(".jpeg") || file.getName().endsWith(".dll")
					|| file.getName().endsWith(".swf") || file.getName().endsWith(".png") || file.getName().endsWith(".bmp")
					|| file.getName().endsWith(".rar") || file.getName().endsWith(".pdf") || file.getName().endsWith(".class")
					|| file.getName().endsWith(".svgz"))
			{
				return;
			}

			FileInputStream is = new FileInputStream(file);
			InputStreamReader reader = new InputStreamReader(is);

			StringBuffer outBuffer = new StringBuffer();
			char[] buffer = new char[1024];
			int read = 0;

			while ((read = reader.read(buffer)) != -1)
			{
				// read = reader.read(buffer);
				StringBuffer paramBuffer = new StringBuffer();
				paramBuffer.append(buffer, 0, read);
				String content = paramBuffer.toString();
				String chainModify = modifyChain(content, oldValue, newValue);
				outBuffer.append(chainModify);
			}

			FileOutputStream fos = new FileOutputStream(file);
			fos.write(outBuffer.toString().getBytes());
			fos.flush();
			fos.close();
			reader.close();
			is.close();
		}
		catch (Exception e)
		{
			CorePlugin.log(this.getClass(), "Exception " + e.getMessage(), IStatus.ERROR);
		}
	}

	/**
	 * This function search piece of string and replace it according to the values defined by the user.
	 * 
	 * @param path represents the path to process, to replace
	 * @return the new path replaced with the custom values
	 */
	public String replaceVariableByValueInPathFile(String path)
	{
		for (String var : mapVariable.keySet())
		{
			// Take the key value without the % char
			String varKey = var.replaceAll("%", "");
			path = path.replaceAll(varKey, mapVariable.get(var));
		}
		return path;
	}
}
