/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.templatetoproject.infos.util;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IStatus;
import org.topcased.bus.core.CorePlugin;
import org.topcased.service.templatetoproject.infos.InfosDirectory;
import org.topcased.service.templatetoproject.infos.InfosFile;
import org.topcased.service.templatetoproject.infos.InfosTemplateProject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * TemplateToInfosTemplateProject get information from a xml file. All
 * information which can be find in a "template file project" ('template.xml').
 * TemplateToInfosTemplateProject is composed of severals Files and Directories
 * it-self composed of Files and Directories.
 * 
 * <template name="..." pathzip="..."> <project> <directory
 * name="..."> [directory * | file *] </directory> * <file name="..."/> *
 * </project> </template> TemplateToInfosTemplateProject is used by
 * InfosTemplateProject
 * 
 * 
 */
public class TemplateToInfosTemplateProject
{

    /**
     * Declaration bloc
     */
    // Infos Directory Map
    private Map<String, InfosDirectory> mapInfosDirectory = new HashMap<String, InfosDirectory>(); 

    // Infos File Map
    private Map<String, InfosFile> mapInfosFile = new HashMap<String, InfosFile>(); 
    
    // Infos Variable Map
    private Map<String, String> mapVariables = new TreeMap<String, String>(); 

    // Template Name
    private String templateName = "";

    // Template Path ZIP File
    private String templatePathZipFile = "";

    // Old project name
    private String oldProjectName = "";
    
    // Character used to declare template variable
    private static final String VARCHAR = "%";
    
    /**
     * Get templateName
     * 
     * @return template name
     */
    public String getTemplateName()
    {
        return templateName;
    }
    
    /**
     * Get oldProjectName
     * 
     * @return old Project name
     */
    public String getOldProjectName()
    {
        return oldProjectName;
    }
    

    /**
     * Get template path zip file
     * 
     * @return path Zip File
     */
    public String getTemplatePathZipFile()
    {
        return templatePathZipFile;
    }

    /**
     * Get mapInfosDirectory
     * 
     * @return mapInfosDirectory : Map of InfosDirectory (key=name ;
     *         value=InfosDirectory)
     */
    public Map<String, InfosDirectory> getMapInfosDirectory()
    {
        return mapInfosDirectory;
    }

    /**
     * Get mapInfosFile
     * 
     * @return mapInfosFile : Map of InfosFile (key=name ; value=InfosFile)
     */
    public Map<String, InfosFile> getMapInfosFile()
    {
        return mapInfosFile;
    }
    
    /**
     * Get variables map
     * @return variables which is a hashmap containing all the variables found in the xml file 
     */
    public Map<String, String> getMapVariables()
    {
    	return mapVariables;
    }

    // **************************************************************
    //
    // Build InfosTemplateProject from a XML file
    //
    // **************************************************************
    /**
     * Take informations about the template project templatePath; Initialize
     * private Map : - 'mapInfosDirectory'(name, InfosDirectory), -
     * 'mapInfosFile' (name, InfosFile)
     * 
     * @param templatePath
     * @return InfosTemplateProject
     */
    public InfosTemplateProject getInfosTemplateProject(String templatePath)
    {
        InfosTemplateProject infosTemplateProject = new InfosTemplateProject();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        // Parser
        try
        {
            DocumentBuilder parser = factory.newDocumentBuilder();
            // Document
            Document document = parser.parse(new File(templatePath + File.separator));

            // <template name="..." ><project oldProjectName="..."> <file name= ... >
            // <repository name=...>)
            NodeList templateList = document.getElementsByTagName(InfosTemplateProject.TEMPLATE);
            int nbNodeReq = templateList.getLength();
            for (int i = 0; i < nbNodeReq; i++)
            {
                Element templateNode = (Element) templateList.item(i);
                templateName = templateNode.getAttribute(InfosTemplateProject.TEMPLATENAME);
                templatePathZipFile = templateNode.getAttribute(InfosTemplateProject.TEMPLATEPATHZIPFILE);
                
                // <project oldProjectName="...">
                NodeList projectList = templateNode.getElementsByTagName(InfosTemplateProject.PROJECT);
                int nbProject = projectList.getLength();
                for (int k = 0; k < nbProject; k++)
                {
                    Element projectNode = (Element) projectList.item(k);
                    oldProjectName = projectNode.getAttribute(InfosTemplateProject.PROJECTOLDNAME);
                    infosTemplateProject.setOldProjectName(oldProjectName);
                    
                    // Is it a variable ? i.e something with the following pattern %name%
                    findVariable(oldProjectName);
                    	
                    // <file name ="..." path="...">
                    mapInfosFile = getInfosFile(projectNode, "");

                    // <directory name="...">
                    mapInfosDirectory = getInfosDirectory(mapInfosDirectory, projectNode, "");
                }
            }

        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);

        }
        catch (SAXException e)
        {
            CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage(), IStatus.ERROR);
        }
        infosTemplateProject.setInfosDirectoryMap(mapInfosDirectory);
        infosTemplateProject.setInfosFileMap(mapInfosFile);
        infosTemplateProject.setTemplateName(templateName);
        infosTemplateProject.setTemplatePathZipFile(templatePathZipFile);
        infosTemplateProject.setTemplateVariables(mapVariables);
        return infosTemplateProject;
    }

    /**
     * Get InfosFile : take informations at the Tag <file>
     * 
     * @param tagParent : parent TAG
     * @param path : path in tree
     * @return Map of InfosFile (key=name ; value=InfosFile)
     */
    private Map<String, InfosFile> getInfosFile(Element tagParent, String path)
    {
        Map<String, InfosFile> mapIF = new HashMap<String, InfosFile>();
        String filePath = "";
        String fileName = "";
        // <file name ="..." path="...">
        NodeList fileList = tagParent.getChildNodes();
        int nbFile = fileList.getLength();
        for (int j = 0; j < nbFile; j++)
        {
            Node fileNode = fileList.item(j);
            if (fileNode.getNodeType() != Node.TEXT_NODE)
            {
                if (fileNode.getNodeName().compareTo(InfosTemplateProject.FILE) == 0)
                {
                    Element fileNodeElement = (Element) fileNode;
                    fileName = fileNodeElement.getAttribute(InfosTemplateProject.FILENAME);
                    // Is filename a variable ?
                    findVariable(fileName);
                    
                    filePath = path + File.separator + fileName;
                    InfosFile infosFile = new InfosFile(fileName, filePath);
                    mapIF.put(infosFile.getFileName(), infosFile);
                }
            }
        }
        return mapIF;
    }

    /**
     * Get InfosDirectory : take informations at the Tag <directory>
     * 
     * @param mapInfoDirectory : Map of InfoDirectory (key=name, value =
     *            InfosDirectory)
     * @param tagParent : parent TAG
     * @param path : path in tree
     * @return Map of InfoDirectory (key=name, value = InfosDirectory)
     */
    private Map<String, InfosDirectory> getInfosDirectory(Map<String, InfosDirectory> mapInfoDirectory, Element tagParent, String path)
    {
        Map<String, InfosDirectory> mapID = new HashMap<String, InfosDirectory>();
        String directoryName = "";
        String directoryPath = "";
        // NodeList repositoryList = tagParent.getElementsByTagName(DIRECTORY);
        NodeList repositoryList = tagParent.getChildNodes();
        int nbDirectory = repositoryList.getLength();
        for (int j = 0; j < nbDirectory; j++)
        {
            // Element tagDirectory = (Element)repositoryList.item(j);
            Node tagDirectory = repositoryList.item(j);
            if (tagDirectory.getNodeType() != Node.TEXT_NODE)
            {
                if (tagDirectory.getNodeName().compareTo(InfosTemplateProject.DIRECTORY) == 0)
                {
                    Element tagDirectoryElement = (Element) tagDirectory;
                    directoryName = tagDirectoryElement.getAttribute(InfosTemplateProject.DIRECTORYNAME);
                    
                    // Is directory name a template variable ?
                    findVariable(directoryName);
                    
                    directoryPath = path + File.separator + directoryName;
                    // <directory>
                    Map<String, InfosDirectory> mapInfoDirectory2 = getInfosDirectory(mapInfoDirectory, tagDirectoryElement, directoryPath); // /
                    mapInfoDirectory = getInfosDirectory(mapInfoDirectory, tagDirectoryElement, directoryPath);

                    // <file>
                    Map<String, InfosFile> mapInfoFile2 = getInfosFile(tagDirectoryElement, directoryPath);
                    InfosDirectory infosDirectory = new InfosDirectory(directoryName, directoryPath, mapInfoDirectory2, mapInfoFile2);

                    mapID.put(infosDirectory.getDirectoryName(), infosDirectory);
                }
            }
        }
        return mapID;
    }
    
    /**
     * This method determines whether the given string contains template variable.
     * If it is the case, the variable name is added to the corresponding hashmap
     * @param string The string to evaluate 
     */
    public void findVariable(String string)
    {
    	String result = "";
    	String toProcess = string;
    	boolean finished = false;
    	while (!finished)
    	{
    		try
    		{
    			result = toProcess.substring(toProcess.indexOf(VARCHAR), toProcess.indexOf(VARCHAR, toProcess.indexOf(VARCHAR) + 1) + 1);
	    		toProcess = toProcess.substring(toProcess.indexOf(result) + result.length(), toProcess.length());
	    		// The variable is added 
	    		if (result.length() > 2)
	    		{
	    			mapVariables.put(result,"");
	    		}
	    		else
	    		{
	    			CorePlugin.log(this.getClass(), "Strange template variable found. Check your XML file", IStatus.WARNING);
	    		}
	    	}
	    	catch(StringIndexOutOfBoundsException ex)
	    	{
	    		finished = true; 
	    	}
    	}
    }

}
