/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.m2m.atl.engine.AtlEMFModelHandler;
import org.eclipse.m2m.atl.engine.AtlLauncher;
import org.eclipse.m2m.atl.engine.AtlModelHandler;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;
import org.topcased.bus.core.CorePlugin;

/**
 * Prepares and launches a Transformation.
 */
public class Transformation
{
	private static final String ATT_URI = "uri:";
	
	private static final String ATT_EXT = "ext:";
	
	private AtlEMFModelHandler amh = (AtlEMFModelHandler) AtlModelHandler.getDefault(AtlEMFModelHandler.AMH_EMF);

	private Map<String, ASMModel> transfoModels = new HashMap<String, ASMModel>();

	private Map<String, String> resultModels = new HashMap<String, String>();

	/**
	 * Constructor
	 * 
	 * @param idTransformation The full identifier of the transformation to perform
	 * @param models
	 * @param pathProject
	 * @param xmlModels
	 * @param monitor
	 */
	public Transformation(String idTransformation, Map<String, String> models, IPath pathProject, boolean xmlModels,
			IProgressMonitor monitor)
	{
		TransformationInfo trans = TransformationHelper.getInstance().getTransfoInfoById(idTransformation);
		if (trans != null)
		{
			List<File> fileToDelete = new ArrayList<File>();
			try
			{
				Map<String, String> inputMap = new HashMap<String, String>();
				Map<String, String> outputMap = new HashMap<String, String>();
				Map<String, String> pathMap = new HashMap<String, String>();

				for (String inParam : trans.getListInParams())
				{
					String mmName = trans.getInParamMetamodel(inParam);
					inputMap.put(inParam, mmName);
					String inputFilename = models.get(inParam);

					if (xmlModels)
					{
						if (monitor != null)
						{
							monitor.subTask("XML Adapter process ...");
							monitor.worked(5);
						}
						Xmi1toXmi2 xmi1toxmi2 = new Xmi1toXmi2(inputFilename, inputFilename + ".xmi", "ISO-8859-1", mmName);
						String result = xmi1toxmi2.runImport();
						if (result.equals("OK"))
						{
							inputFilename = inputFilename.concat(".xmi");
							fileToDelete.add(new File(inputFilename));
						}
					}
					if (monitor != null)
					{
						monitor.subTask("Preparing transformation parameters ...");
						monitor.worked(20);
					}
					pathMap.put(inParam, ATT_EXT.concat(inputFilename));
					// CLC add : to modify if xml file
					models.put(inParam, inputFilename);

					// in case of an already registered metamodel we should invoke it through its Uri.
					if (TransformationHelper.isAlreadyLoadedInEMFRegistry(trans.getMetamodelId(mmName)))
					{
						pathMap.put(mmName, ATT_URI.concat(trans.getMetamodelId(mmName)));
					}
					else
					{
						// else we need to load the metamodel. ATL makes it.
						pathMap.put(mmName, ATT_EXT.concat(trans.getMetamodelFile(mmName).getAbsolutePath()));
					}
				}

				for (String outParam : trans.getListOutParams())
				{
					String mmName = trans.getOutParamMetamodel(outParam);
					outputMap.put(outParam, mmName);
					pathMap.put(outParam, pathProject.toString() + IPath.SEPARATOR + models.get(outParam));

					// in case of an already registered metamodel we should invoke it through its Uri.
					if (TransformationHelper.isAlreadyLoadedInEMFRegistry(trans.getMetamodelId(mmName)))
					{
						pathMap.put(mmName, ATT_URI.concat(trans.getMetamodelId(mmName)));
					}
					else
					{
						// else we need to load the metamodel. ATL makes it.
						pathMap.put(mmName, ATT_EXT.concat(trans.getMetamodelFile(mmName).getAbsolutePath()));
					}
				}
				if (monitor != null)
				{
					monitor.worked(5);
				}
				// CLC : ATL Launch new version, to avoid conflict with editors
				provideInModelsWithMetamodel(trans, models);
				provideOutModelsWithMetamodel(trans, models, pathProject);

				URL mAsmUrl = null;
				try
				{
					mAsmUrl = trans.getRuleFile().toURI().toURL();
				}
				catch (MalformedURLException e1)
				{
					CorePlugin.log("Bad URL conversion on transformation rule file", IStatus.ERROR);
					CorePlugin.log(e1);
				}

				if (monitor != null)
				{
					monitor.subTask("Launching transformation ...");
				}

				AtlLauncher.getDefault().launch(mAsmUrl, Collections.EMPTY_MAP, transfoModels, Collections.EMPTY_MAP, Collections.EMPTY_LIST, Collections.EMPTY_MAP);
				if (monitor != null)
				{
					monitor.worked(15);
					monitor.subTask("Saving result model(s) ...");
				}
				saveOutModels(trans, models, pathProject);
				if (monitor != null)
				{
					monitor.worked(5);
				}
				// retrieve the models produced by the transformation
				for (String outParam : trans.getListOutParams())
				{
					resultModels.put(outParam, models.get(outParam));
				}
			}
			catch (Exception e)
			{
				CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
			}

			for (File tmpFile : fileToDelete)
			{
				tmpFile.delete();
			}
			trans = null;
		}
	}

	/**
	 * Fills the transfoModels table for ATL transformation with the IN models
	 * 
	 * @param trans : TransformationInfo parameters
	 * @param models : Models involved in the transformation
	 */
	private void provideInModelsWithMetamodel(TransformationInfo trans, Map<String, String> models)
	{
		ASMModel modelIn = null;
		ASMModel metaIn = null;
		transfoModels.put("MOF", amh.getMof());
		for (String inParam : trans.getListInParams())
		{
			String mmName = trans.getInParamMetamodel(inParam);

			URL urlMM = null;
			// get the metamodel associated
			try
			{
				if (trans.getMetamodelFile(mmName) != null)
				{
					urlMM = trans.getMetamodelFile(mmName).toURI().toURL();
				}
			}
			catch (MalformedURLException e1)
			{
				CorePlugin.log(e1);
			}

			// load metamodel
			InputStream inStream = null;
			try
			{
				if (urlMM != null)
				{
					inStream = urlMM.openStream();
				}
			}
			catch (IOException e)
			{
				CorePlugin.log(e);
			}

			// define the ASMModel
			// in case of an already registered metamodel we should invoke it through its Uri.
			if (TransformationHelper.isAlreadyLoadedInEMFRegistry(trans.getMetamodelId(mmName)))
			{
				metaIn = amh.loadModel(mmName, amh.getMof(), ATT_URI.concat(trans.getMetamodelId(mmName)));
			}
			else
			{
				// else we need to load the metamodel. ATL makes it.
				metaIn = amh.loadModel(mmName, amh.getMof(), inStream);
			}

			// Put the ASMModel for the transformation
			transfoModels.put(mmName, metaIn);

			URL urlModel = null;
			// get the metamodel associated
			try
			{
				File inFile = new File((String) models.get(inParam));
				urlModel = inFile.toURI().toURL();
			}
			catch (MalformedURLException e1)
			{
				CorePlugin.log(e1);
			}
			
			// load metamodel
			InputStream inModelStream = null;
			try
			{
				inModelStream = urlModel.openStream();
			}
			catch (IOException e)
			{
				CorePlugin.log(e);
			}

			// define the ASMModel

			modelIn = amh.loadModel(inParam, metaIn, inModelStream);
			transfoModels.put(inParam, modelIn);
		}
	}

	/**
	 * Fills the transfoModels table for ATL transformation with the OUT models
	 * 
	 * @param trans : TransformationInfo parameters
	 * @param models : Models involved in the transformation
	 * @param pathProject : Path project given as full path
	 */
	private void provideOutModelsWithMetamodel(TransformationInfo trans, Map<String, String> models, IPath pathProject)
	{
		ASMModel modelOut = null;
		ASMModel metaOut = null;
		transfoModels.put("MOF", amh.getMof());
		for (String outParam : trans.getListOutParams())
		{
			String mmName = trans.getOutParamMetamodel(outParam);
			// in case of an already registered metamodel we should invoke it through its Uri.
			if (TransformationHelper.isAlreadyLoadedInEMFRegistry(trans.getMetamodelId(mmName)))
			{
				metaOut = amh.loadModel(mmName, amh.getMof(), ATT_URI.concat(trans.getMetamodelId(mmName)));
			}
			// else we need to load the metamodel. ATL makes it.
			else
			{
				URL urlMM = null;
				// get the metamodel associated
				try
				{
					urlMM = trans.getMetamodelFile(mmName).toURI().toURL();
				}
				catch (MalformedURLException e1)
				{
					CorePlugin.log(e1);
				}
				// load metamodel
				InputStream inStream = null;
				try
				{
					inStream = urlMM.openStream();
				}
				catch (IOException e)
				{
					CorePlugin.log(e);
				}
				metaOut = amh.loadModel(mmName, amh.getMof(), inStream);
			}

			// Put the ASMModel for the transformation
			transfoModels.put(mmName, metaOut);

			// define the ASMModel for output
			modelOut = amh.newModel(models.get(outParam), metaOut);
			transfoModels.put(outParam, modelOut);
		}
	}

	/**
	 * Allows to save all the models produced by the transformation in the targetted project
	 * 
	 * @param trans : TransformationInfo parameters
	 * @param models : Models involved in the transformation
	 * @param pathProject : Path project given as full path
	 */
	private void saveOutModels(TransformationInfo trans, Map<String, String> models, IPath pathProject)
	{
		for (String outParam : trans.getListOutParams())
		{
			amh.saveModel(transfoModels.get(outParam), pathProject.toString() + "/" + models.get(outParam));
		}
	}
}
