/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EPackage;
import org.osgi.framework.Bundle;
import org.topcased.bus.core.CorePlugin;

/**
 * Helper for getting information about all the available Topcased transformation.<br>
 * This class will be called :
 * <ul>
 * <li>by the wizard to determine the missing parameters</li>
 * <li>before launching an ATL transformation.</li>
 * </ul>
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 *
 */
public class TransformationHelper
{
	/** Constant representing an ID */
	private static final String ID = "id";
	
	/** Constant representing the URI */
	private static final String URI = "uri";
	
	/** Constant representing the NAME */
	private static final String NAME = "name";
	
	/** The singleton object */
	private static TransformationHelper transformationInfoCatalog = null;

	/** Map containing information about all the available Topcased transformation by ID */
	private Map<String, TransformationInfo> mapTransfoId;	
	
	/** Map containing information about all the available Topcased transformation by Name */
	private Map<String, TransformationInfo> mapTransfoName;	
	
	/**
	 * Private constructor building a map containing information about all the available Topcased mapTransfoId.
	 */
	private TransformationHelper()
	{
		getTopcasedTransformations();
	}
	
	/**
	 * Returns the shared instance
	 * 
	 * @return the singleton
	 */
	public static TransformationHelper getInstance()
	{
		if (transformationInfoCatalog == null)
		{
		   transformationInfoCatalog = new TransformationHelper();
		}
		return transformationInfoCatalog;
	}
	
	/**
	 * Gets information about a given <b>transformation id</b>. 
	 * 
	 * @param transfoId the full identifier of a transformation
	 * @return Information object about the given transformation
	 */
	public TransformationInfo getTransfoInfoById(String transfoId)
	{
		return mapTransfoId.get(transfoId);
	}
	
	/**
	 * Gets information <b>by id</b> about all registered (available) Topcased transformations. 
	 * 
	 * @return The map with all information of each registered Topcased transformation
	 */
	public Map<String, TransformationInfo> getTransfoInfoById()
	{
		return mapTransfoId;
	}
	
	/**
	 * Gets information about a given <b>transformation name</b>. 
	 * 
	 * @param transfoName the full name of a transformation
	 * @return Information object about the given transformation
	 */
	public TransformationInfo getTransfoInfoByName(String transfoName)
	{
		return mapTransfoName.get(transfoName);
	}
	
	/**
	 * Gets information <b>by name</b> about all registered (available) Topcased transformations. 
	 * 
	 * @return The map with all information of each registered Topcased transformation
	 */
	public Map<String, TransformationInfo> getTransfoInfoByName()
	{
		return mapTransfoName;
	}
	
	/**
	 * This method is called during initialization of the singleton.<br>
	 * Information about all available Topcased mapTransfoId get into a map.
	 */
	private void getTopcasedTransformations()
	{
		mapTransfoId = new HashMap<String, TransformationInfo>();
		mapTransfoName = new HashMap<String, TransformationInfo>();
		IExtensionPoint extensionPoint = Platform.getExtensionRegistry().getExtensionPoint(CorePlugin.getDefault().getBundle().getSymbolicName(), "Transformation");
		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length; i++)
		{
			IExtension extension = extensions[i];
			IConfigurationElement[] elements = extension.getConfigurationElements();
			for (int j = 0; j < elements.length; j++)
			{
				String transfoId = elements[j].getAttribute(ID);
				String transfoName = elements[j].getAttribute(NAME);
				TransformationInfo transformationInfo = new TransformationInfo(transfoId, transfoName);
				// Adds collected information into the dedicated maps.
				mapTransfoId.put(transfoId, transformationInfo);
				mapTransfoName.put(transfoName, transformationInfo);
				findRuleInformation(transformationInfo, elements[j]);
				findInParamsInformation(transformationInfo, elements[j]);
				findOutParamsInformation(transformationInfo, elements[j]);
			}
		}
	}
	
	/**
	 * Finds and sets information about the rule file which must be used during the transformation.
	 * 
	 * @param transInfo The information transformation object
	 * @param config
	 */
	private void findRuleInformation(TransformationInfo transInfo, IConfigurationElement config)
	{
		IConfigurationElement[] ruleFile = config.getChildren("ruleFile");
		// this suppose only one transformation
		URL location = null;
		for (int k = 0; k < ruleFile.length; k++)
		{
			IExtension extension = config.getDeclaringExtension();
			Bundle bundle = Platform.getBundle(extension.getNamespaceIdentifier());
			location = FileLocator.find(bundle, new Path(ruleFile[k].getAttribute("filename")), null);
			try
			{
				location = FileLocator.resolve(location);
			}
			catch (IOException e)
			{
				CorePlugin.log(e);
			}
			transInfo.setRuleFile(new File(location.getFile()));
		}
	}
	
	/** 
	 * Finds and sets information about the required input parameters for performing the transformation.
	 * 
	 * @param transInfo The information transformation object
	 * @param config 
	 */
	private void findInParamsInformation(TransformationInfo transInfo, IConfigurationElement config)
	{
		IConfigurationElement[] inParams = config.getChildren("inParam");
		for (int i = 0; i < inParams.length; i++)
		{
			IConfigurationElement[] metamodel = inParams[i].getChildren("transformationMetamodel");
			for (int j = 0; j < metamodel.length; j++)
			{
				String mmName = metamodel[j].getAttribute(NAME);
				transInfo.addInParams(inParams[i].getAttribute(NAME), mmName);
				transInfo.setMetamodelId(mmName, metamodel[j].getAttribute(ID));
				transInfo.setMetamodelHandler(mmName, metamodel[j].getAttribute("modelHandler"));
				setMetamodelFile(transInfo, metamodel[j], "in");
			}
		}
	}
	
	/** 
	 * Finds and sets information about the required output parameters for performing the transformation.
	 * 
	 * @param transInfo The information transformation object
	 * @param config 
	 */
	private void findOutParamsInformation(TransformationInfo transInfo, IConfigurationElement config)
	{
		IConfigurationElement[] outParams = config.getChildren("outParam");
		for (int i = 0; i < outParams.length; i++)
		{
			IConfigurationElement[] metamodel = outParams[i].getChildren("transformationMetamodel");
			for (int j = 0; j < metamodel.length; j++)
			{
				String mmName = metamodel[j].getAttribute(NAME);
				transInfo.addOutParams(outParams[i].getAttribute(NAME), mmName);
				transInfo.setMetamodelId(mmName, metamodel[j].getAttribute(ID));
				transInfo.setMetamodelHandler(mmName, metamodel[j].getAttribute("modelHandler"));
				setMetamodelFile(transInfo, metamodel[j], "out");
			}
		}
	}
	
	/**
	 * Sets metamodel file according to the metamodel id and its type.
	 * 
	 * @param transfoInfo Information about the current transformation
	 * @param mmConfig A configuration element representing a metamodel
	 * @param kindMM The kind of the current metamodel : input or output
	 */
	private void setMetamodelFile(TransformationInfo transfoInfo, IConfigurationElement mmConfig, String kindMM)
	{
		String mmId = mmConfig.getAttribute(ID);
		if (!isAlreadyLoadedInEMFRegistry(mmId))
		{
			String mmName = mmConfig.getAttribute(NAME);
			File mm = retrieveMetamodel(mmId);
			if (mm != null && mm.exists())
			{
				transfoInfo.setMetamodelFile(mmName, mm);
			}
			else
			{
				if ("out".equals(kindMM))
				{
					transfoInfo.setMissingOutMetamodelName(mmName);
				}
				else
				{
					transfoInfo.setMissingInMetamodelName(mmName);
				}
			}
		}	
	}
	
	/**
	 * Searchs into the EMF registry the metamodel given in parameter. 
	 * 
	 * @param metamodelURI The URI of the metamodel to consult into the EMF registry
	 * @return <code>true</code> if the metamodel was found, <code>false</code> otherwise.
	 */
	public static boolean isAlreadyLoadedInEMFRegistry(String metamodelURI)
	{
		return EPackage.Registry.INSTANCE.containsKey(metamodelURI);
	}
	
	/**
	 * Retrieves the .ecore file corresponding to the given metamodel uri
	 */
	private File retrieveMetamodel(String mmId)
	{
		File mm = null;
		boolean found = false;
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(CorePlugin.getDefault().getBundle().getSymbolicName(), "Metamodels");
		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length && !(found); i++)
		{
			IExtension extension = extensions[i];
			IConfigurationElement[] elements = extension.getConfigurationElements();
			for (int j = 0; j < elements.length && !(found); j++)
			{
				if (elements[j].getAttribute(URI).equals(mmId))
				{
					found = true;
					Bundle bundle = Platform.getBundle(extension.getNamespaceIdentifier());
					URL location = FileLocator.find(bundle, new Path(elements[j].getAttribute("MetamodelPath")), null);
					try
					{
						location = FileLocator.resolve(location);
					}
					catch (Exception e)
					{
						return null;
					}
					mm = new File(location.getFile());
				}
			}
		}
		return mm;
	}
}
