/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation.sildexsam.services;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.IServiceCst;
import org.topcased.bus.core.ServicesManager;
import org.topcased.sam.Automaton;
import org.topcased.sam.Model;
import org.topcased.sam.ModelContent;
import org.topcased.sam.MultiPort;
import org.topcased.sam.Port;
import org.topcased.sam.SAMFactory;
import org.topcased.sam.System;

/**
 * Defines the SILDEX to SAM import service.
 * 
 * @author <a href="mailto:christophe.le-camus@c-s.fr">Christophe LE CAMUS</a>
 */
public class SildexToSamImportService implements IService
{
	private static final IWorkspaceRoot WORKSPACE_ROOT = ResourcesPlugin.getWorkspace().getRoot();
	
	private Model modelroot = null;

	private ModelContent content = null;
	
	/**
	 * @see org.topcased.bus.core.IService#serviceRun(java.util.Map)
	 */
	public Object serviceRun(Map<String, Object> parameters)
	{

		if (!parameters.containsKey("IN"))
		{
			CorePlugin.log("The parameter IN must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("OUT"))
		{
			CorePlugin.log("The parameter OUT must be set.", IStatus.ERROR);
		}

		if (!parameters.containsKey("Path"))
		{
			CorePlugin.log("The parameter Path must be set.", IStatus.ERROR);
		}

		if (!(parameters.get("IN") instanceof String))
		{
			CorePlugin.log("The parameter IN must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("OUT") instanceof String))
		{
			CorePlugin.log("The parameter OUT must be a String.", IStatus.ERROR);
		}

		if (!(parameters.get("Path") instanceof IPath))
		{
			CorePlugin.log("The parameter Path must be a IPath.", IStatus.ERROR);
		}

		String beginTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Entering Sildex to SAM import service at ".concat(beginTime), IStatus.INFO);

		String inpath = (String) parameters.get("IN");
		IPath outpath = new Path((String) parameters.get("OUT"));
		outpath = outpath.addFileExtension("sam");

		IPath path = (IPath) parameters.get("Path");

		HashMap<String, String> models = new HashMap<String, String>();
		models.put("IN", inpath);
		models.put("OUT", outpath.toOSString());

		HashMap<String, Object> serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("Id", "org.topcased.import.sildexsam.transformation");
		serviceParameters.put("Models", models);
		serviceParameters.put("IsXml", new Boolean(true));
		serviceParameters.put("ProjectPath", path);

		IService transformation = ServicesManager.getInstance().getService(IServiceCst.TRANSFORMATION);
		transformation.serviceRun(serviceParameters);

		IService readService = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
		serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("XMIFile", path.append(outpath).toString());
		Object result = readService.serviceRun(serviceParameters);

		try
		{
			// generates directory
			IPath libPath = path.append("lib");
			SildexToSamImportService.generateDirectory(libPath);

			// models export
			modelroot = (Model) result;
			content = modelroot.getModelContent();
			String rootModelName = "";
			if (content instanceof System)
			{
				System systemRoot = (System) content;
				rootModelName = systemRoot.getName();
				exploreSystems(systemRoot, getFolder(libPath).getFullPath());
			}
			else if (content instanceof Automaton)
			{
				Automaton automatonRoot = (Automaton) content;
				rootModelName = automatonRoot.getName();
				exploreAutomaton(automatonRoot, getFolder(libPath).getFullPath());
			}

			IFile file = WORKSPACE_ROOT.getFile(libPath.append(rootModelName).addFileExtension("sam"));

			// refreshes workspace before and after
			refreshWokspace();
			IFile fileToDelete = WORKSPACE_ROOT.getFile(path.append(outpath));
			fileToDelete.delete(true, null);
			file.copy(path.append(outpath), true, null);
			file.delete(true, null);
			refreshWokspace();
			
		}
		catch (CoreException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}

		String endTime = new SimpleDateFormat("yyyy-MM-dd hh'h'mm").format(new Date());
		CorePlugin.log("Leaving Sildex to SAM import service at ".concat(endTime), IStatus.INFO);

		return result;
	}

	/**
	 * Refreshes the workspace
	 * 
	 * @throws CoreException If a problem occurs during refreshment
	 */
	private static void refreshWokspace() throws CoreException
	{
		WORKSPACE_ROOT.refreshLocal(IResource.DEPTH_INFINITE, null);
	}
	
	/**
	 * Gets a folder from the current workspace
	 * 
	 * @param path Represents the path of the resource to find.
	 * @return the IFolder representing the current directory.
	 */
	private static IFolder getFolder(IPath path)
	{
		return WORKSPACE_ROOT.getFolder(path);
	}

	/**
	 * Generates a directory into the new created project.
	 * 
	 * @param projectPath the path of the folder to create
	 * @throws CoreException
	 */
	private static void generateDirectory(IPath projectPath) throws CoreException
	{
		IFolder folder = getFolder(projectPath);
		if (!folder.exists())
		{
			folder.create(true, true, null);
		}
	}

	/**
	 * Explores System object model
	 * 
	 * @param system : the SAM system to explore
	 * @param path : workspace project/directory to save models
	 * @return
	 */
	private System exploreSystems(ModelContent system, IPath path)
	{
		ModelContent anElement = null;
		for (Iterator<ModelContent> it = ((System) system).getListElements().iterator(); it.hasNext();)
		{
			anElement = it.next();
			System aSystem = null;
			if (anElement instanceof System)
			{
				aSystem = (System) anElement;
				exploreSystems(aSystem, path);
			}
			if (anElement instanceof Automaton)
			{
				EObject modelSaved = saveModels(path, (Automaton) anElement);
				modifyModelContent((Automaton) anElement, (Model) modelSaved);
			}
		}
		if (system != null)
		{
			EObject modelSaved = saveModels(path, system);
			system = modifyModelContent(system, (Model) modelSaved);
		}

		return (System) system;
	}

	/**
	 * Explores Automaton model objects
	 * 
	 * @param automaton : the SAM automaton to explore
	 * @param path : workspace project/directory to save models
	 * @return
	 */
	private Automaton exploreAutomaton(ModelContent automate, IPath path)
	{
		if (automate != null)
		{
			EObject modelSaved = saveModels(path, automate);
			automate = modifyModelContent(automate, (Model) modelSaved);
		}

		return (Automaton) automate;
	}

	/**
	 * 
	 * @param mdc : The System or the Automaton which ports and multiports have to be modified
	 * @param external : the external model already saved
	 */
	private ModelContent modifyModelContent(ModelContent mdc, Model external)
	{

		EObject searchExternalElt = identifyModelContent(external);
		if (mdc instanceof System)
		{
			System sys = (System) mdc;
			sys.setIsInstanceOf((System) searchExternalElt);
			// explore listPorts + explore multiPort
			EList<Port> listPort = ((System) searchExternalElt).getListPorts();
			refactorPorts(sys, listPort);
			EList<MultiPort> listMultiPort = ((System) searchExternalElt).getListMultiPort();
			refactorMultiPorts(sys, listMultiPort);

		}
		if (mdc instanceof Automaton)
		{
			Automaton autom = (Automaton) mdc;
			autom.setIsInstanceOf((Automaton) searchExternalElt);
			// Check if explore listPorts + explore multiPort
			EList<Port> listPort = ((Automaton) searchExternalElt).getListPorts();
			refactorPorts(autom, listPort);
			EList<MultiPort> listMultiPort = ((Automaton) searchExternalElt).getListMultiPort();
			refactorMultiPorts(autom, listMultiPort);
		}
		return mdc;
	}

	/**
	 * Adds the instanceof reference towards external MultiPorts
	 * 
	 * @param mdc : The System or the Automaton for which MultiPorts are instanceof external MultiPorts
	 * @param listMultiPort : list of MultiPorts of the external ModelContent (System or Automaton)
	 */
	private void refactorMultiPorts(ModelContent mdc, EList<MultiPort> listMultiPort)
	{
		if (mdc instanceof System)
		{
			for (Iterator<MultiPort> it = ((System) mdc).getListMultiPort().iterator(); it.hasNext();)
			{
				MultiPort multiPort = it.next();
				MultiPort externalPort = findMultiPort(multiPort, listMultiPort);
				multiPort.setIsInstanceOf(externalPort);
			}
		}
		if (mdc instanceof Automaton)
		{
			for (Iterator<MultiPort> it = ((Automaton) mdc).getListMultiPort().iterator(); it.hasNext();)
			{
				MultiPort multiPort = it.next();
				MultiPort externalPort = findMultiPort(multiPort, listMultiPort);
				multiPort.setIsInstanceOf(externalPort);
			}
		}
	}

	/**
	 * Adds the instanceof reference towards external Ports
	 * 
	 * @param mdc : The System or the Automaton for which Ports are instanceof external Ports
	 * @param listPort : list of Ports of the external ModelContent (System or Automaton)
	 */
	private void refactorPorts(ModelContent mdc, EList<Port> listPort)
	{
		if (mdc instanceof System)
		{
			for (Iterator<Port> it = ((System) mdc).getListPorts().iterator(); it.hasNext();)
			{
				Port port = it.next();
				Port externalPort = findPort(port, listPort);
				port.setIsInstanceOf(externalPort);
			}
		}
		if (mdc instanceof Automaton)
		{
			for (Iterator<Port> it = ((Automaton) mdc).getListPorts().iterator(); it.hasNext();)
			{
				Port port = it.next();
				Port externalPort = findPort(port, listPort);
				port.setIsInstanceOf(externalPort);
			}
		}
	}

	/**
	 * Finds a MultiPort in the external MultiPorts model's list whose name is equal to the multiPort
	 * 
	 * @param multiPort : multiPort whose name has to be found in the external list
	 * @param listMultiPort : external list of multiPorts
	 * @return : the identified multiPort
	 */
	private MultiPort findMultiPort(MultiPort multiPort, EList<MultiPort> listMultiPort)
	{

		boolean notFound = true;
		MultiPort identifiedMultiPort = null;
		Iterator<MultiPort> itMultiPort = listMultiPort.iterator();
		while (itMultiPort.hasNext() && notFound)
		{
			MultiPort extPort = itMultiPort.next();
			if (multiPort.getName().compareTo(extPort.getName()) == 0)
			{
				notFound = false;
				identifiedMultiPort = extPort;
			}
		}
		return identifiedMultiPort;
	}

	/**
	 * Finds a Port in the external Ports model's list whose name is equal to the Port
	 * 
	 * @param Port : Port whose name has to be found in the external list
	 * @param listPort : external list of Ports
	 * @return : the identified Port
	 */
	private Port findPort(Port port, EList<Port> listPort)
	{

		boolean notFound = true;
		Port identifiedPort = null;
		Iterator<Port> itPort = listPort.iterator();
		while (itPort.hasNext() && notFound)
		{
			Port extPort = itPort.next();
			if (port.getName().compareTo(extPort.getName()) == 0)
			{
				notFound = false;
				identifiedPort = extPort;
			}
		}
		return identifiedPort;
	}

	/**
	 * Returns the modelContent (System or Automaton) of the external SAM model
	 * 
	 * @param external : the external SAM model
	 * @return
	 */
	private ModelContent identifyModelContent(Model external)
	{
		return external.getModelContent();
	}

	/**
	 * Saves the model at path location
	 * 
	 * @param path : the path of the workspace directory
	 * @param eobj : the ModelContent to be saved as an external file
	 * @return the saved SAM model
	 */
	private EObject saveModels(IPath path, EObject eobj)
	{
		IPath pathFile = new Path("");
		if (eobj instanceof System)
		{
			pathFile = path.append(((System) eobj).getName()).addFileExtension("sam");
		}
		
		if (eobj instanceof Automaton)
		{
			pathFile = path.append(((Automaton) eobj).getName()).addFileExtension("sam");
		}
		
		Model modelSam = SAMFactory.eINSTANCE.createModel();
		ModelContent modelContent = (ModelContent) EcoreUtil.copy(eobj);
		modelSam.setModelContent(modelContent);
		
		IService is = ServicesManager.getInstance().getService(IServiceCst.WRITE_EMF_XMI);
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("PathFileString", WORKSPACE_ROOT.getLocation().append(pathFile).toString());
		param.put("Model", modelSam);

		// call the write service
		is.serviceRun(param);
		
		IService isRead = ServicesManager.getInstance().getService(IServiceCst.READ_XMI_EMF);
		Map<String, Object> serviceParameters = new HashMap<String, Object>();
		serviceParameters.put("XMIFile", pathFile.toString());
		
		return (EObject) isRead.serviceRun(serviceParameters);
	}
}
