/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.transformation.wizard;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.topcased.bus.core.CorePlugin;
import org.topcased.bus.core.Service;
import org.topcased.bus.core.ServicesManager;
import org.topcased.bus.core.parameter.Type;
import org.topcased.service.transformation.TransformationHelper;
import org.topcased.service.transformation.TransformationInfo;
import org.topcased.service.transformation.TransformationPlugin;

/**
 * This page is displayed when the import or an export wizard is called.
 * 
 * @author <a href="mailto:sebastien.gabel@c-s.fr">Sebastien GABEL</a>
 */
public class TransformationPage extends WizardPage
{
	/** The main shell */
	private Shell shell;

	/** The main container */
	private Composite container;

	/** The combo containing the list of available transformations */
	private Combo transCombo;

	/** Map storing the name of the model parameters and its values given by the user */
	private Map<String, String> modelParamsMap = new HashMap<String, String>();

	/** Map storing the name of the metamodel parameters and its value */
	private Map<String, String> metamodelParamsMap = new HashMap<String, String>();

	/** Selected Path */
	private IPath pathProject = null;
	
	/** Kind of this page : import or export */
	private String kind = null;

	/**
	 * Constructor
	 * 
	 * @param selectedPath The selection made by the user. It represents a IFolder or a IFile.
	 * @param kindPage The kind of the page : import or export
	 */
	protected TransformationPage(IPath selectedPath, String kindPage)
	{
		super(kindPage);
		pathProject = selectedPath;
		kind = kindPage;
	}

	/**
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent)
	{
		shell = parent.getShell();
		shell.setMinimumSize(500, 430);

		// Layout and Data definition
		final GridLayout gridMainComposite = new GridLayout(2, false);
		gridMainComposite.horizontalSpacing = 5;
		gridMainComposite.verticalSpacing = 5;

		// Main composite
		container = new Composite(parent, SWT.NONE);
		container.setLayout(gridMainComposite);
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		if (pathProject != null)
		{
			final Label selectLabel = new Label(container, SWT.NONE);
			selectLabel.setText(TransformationPlugin.getResourceString("TransformationPage.mainLabel"));

			transCombo = new Combo(container, SWT.READ_ONLY);
			transCombo.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
			transCombo.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(SelectionEvent e)
				{
					String transfoName = transCombo.getItem(transCombo.getSelectionIndex());
					TransformationInfo transfoInfo = TransformationHelper.getInstance().getTransfoInfoByName(transfoName);
					redrawFields(transfoInfo.getId());
					canFinish();
				}
			});

			for (Type type : ServicesManager.getInstance().getAllType().values())
			{
				if (kind.equals(ImportWizard.IMPORT_KIND))
				{
					if (type.getParentIdType() != null && type.getParentIdType().equals("org.topcased.service.import"))
					{
						transCombo.add(type.getNameType());
					}
				}
				
				if (kind.equals(ExportWizard.EXPORT_KIND))
				{
					if (type.getParentIdType() != null &&  type.getParentIdType().equals("org.topcased.service.export"))
					{
						transCombo.add(type.getNameType());
					}
				}
			}
			drawFields(null);
		}
		else
		{
			Label label = new Label(container, SWT.NONE);
			label.setText("TransformationPage.missing.project");
		}

		setControl(container);
		setPageComplete(false);
		shell.layout(true, true);
	}

	/**
	 * Redraws the fields displayed
	 * 
	 * @param transfoId The identifier of the transformation to apply
	 */
	private void redrawFields(String transfoId)
	{
		metamodelParamsMap.clear();
		modelParamsMap.clear();

		for (Control control : container.getChildren())
		{
			if (control instanceof Group)
			{
				control.dispose();
			}
		}
		drawFields(transfoId);
		shell.layout(true, true);
	}

	/**
	 * Draws the different required Groups according to the selected transformation.
	 * 
	 * @param transfoId The identifier of the transformation to apply
	 */
	private void drawFields(String transfoId)
	{
		// Gets info about the transformation to prepare
		TransformationInfo info = TransformationHelper.getInstance().getTransfoInfoById(transfoId);

		final GridData inModelParamsData = new GridData(SWT.FILL, SWT.NONE, true, false);
		inModelParamsData.horizontalSpan = 2;
		inModelParamsData.minimumHeight = 50;
		inModelParamsData.heightHint = 50;

		final Group inModelParams = new Group(container, SWT.SHADOW_IN);
		inModelParams.setLayout(new GridLayout(1, false));
		inModelParams.setText(TransformationPlugin.getResourceString("TransformationPage.group.input.model"));
		inModelParams.setLayoutData(inModelParamsData);

		if (info != null)
		{
			for (String inParam : info.getListInParams())
			{
				drawFieldsForGroupParameters(inParam, inModelParams, modelParamsMap);
			}

			// process missing input metamodel
			if (!info.getMissingInMetamodelName().isEmpty())
			{
				final GridData inMetamodelParamsData = new GridData(SWT.FILL, SWT.NONE, true, false);
				inMetamodelParamsData.horizontalSpan = 2;
				inMetamodelParamsData.minimumHeight = 50;
				inMetamodelParamsData.heightHint = 50;

				final Group inMetamodelParams = new Group(container, SWT.SHADOW_IN);
				inMetamodelParams.setLayout(new GridLayout(1, false));
				inMetamodelParams.setText(TransformationPlugin.getResourceString("TransformationPage.group.input.mm"));
				inMetamodelParams.setLayoutData(inMetamodelParamsData);

				// At least one input metamodels is missing...
				for (String inMissing : info.getMissingInMetamodelName())
				{
					drawFieldsForGroupParameters(inMissing, inMetamodelParams, metamodelParamsMap);
				}
			}
		}

		final GridData outModelParamsData = new GridData(SWT.FILL, SWT.NONE, true, false);
		outModelParamsData.horizontalSpan = 2;
		outModelParamsData.minimumHeight = 50;
		outModelParamsData.heightHint = 50;

		final Group outModelParams = new Group(container, SWT.SHADOW_IN);
		outModelParams.setLayout(new GridLayout(1, false));
		outModelParams.setText(TransformationPlugin.getResourceString("TransformationPage.group.output.model"));
		outModelParams.setLayoutData(outModelParamsData);

		if (info != null)
		{
			for (final String outParam : info.getListOutParams())
			{
				final Composite comp = new Composite(outModelParams, SWT.NONE);
				comp.setLayout(new GridLayout(2, false));
				comp.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

				final Label outLabel = new Label(comp, SWT.NONE);
				outLabel.setText(outParam);

				final Text outText = new Text(comp, SWT.BORDER);
				outText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
				outText.setToolTipText(TransformationPlugin.getResourceString("TransformationPage.tips.output"));
				modelParamsMap.put(outParam, outText.getText());
				outText.addModifyListener(new ModifyListener()
				{
					public void modifyText(ModifyEvent e)
					{
						modelParamsMap.put(outParam, outText.getText());
						canFinish();
					}
				});
			}

			// Process missing output metamodel
			if (!info.getMissingOutMetamodelName().isEmpty())
			{
				final GridData outMetamodelParamsData = new GridData(SWT.FILL, SWT.NONE, true, false);
				outMetamodelParamsData.horizontalSpan = 2;
				outMetamodelParamsData.minimumHeight = 50;
				outMetamodelParamsData.heightHint = 50;

				final Group outMetamodelParams = new Group(container, SWT.SHADOW_IN);
				outMetamodelParams.setLayout(new GridLayout(1, false));
				outMetamodelParams.setText(TransformationPlugin.getResourceString("TransformationPage.group.output.mm"));
				outMetamodelParams.setLayoutData(outMetamodelParamsData);

				// At least one output metamodels is missing...
				for (String outMissing : info.getMissingOutMetamodelName())
				{
					drawFieldsForGroupParameters(outMissing, outMetamodelParams, metamodelParamsMap);
				}
			}
		}
	}

	/**
	 * Draws the composite enabling to set input parameters.<br>
	 * This composite contains : a short label, a text field and
	 * 
	 * @param paramName The name of the parameter that can be a model or a metamodel
	 * @param group The graphical group where the new composite must be drawn
	 * @param paramMap The map storing a parameter and its value
	 */
	private void drawFieldsForGroupParameters(final String paramName, Group group, final Map<String, String> paramMap)
	{
		final Composite comp = new Composite(group, SWT.NONE);
		comp.setLayout(new GridLayout(3, false));
		comp.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));

		final Label label = new Label(comp, SWT.NONE);
		label.setText(paramName);

		final Text text = new Text(comp, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text.addModifyListener(new ModifyListener()
		{
			public void modifyText(ModifyEvent e)
			{
				paramMap.put(paramName, text.getText());
				canFinish();
			}
		});

		final GridData inBrowseDataLayout = new GridData();
		inBrowseDataLayout.widthHint = 85;

		final Button browseButton = new Button(comp, SWT.PUSH);
		browseButton.setLayoutData(inBrowseDataLayout);
		browseButton.setText(TransformationPlugin.getResourceString("TransformationPage.button.label"));
		browseButton.setToolTipText(TransformationPlugin.getResourceString("TransformationPage.tips.button"));
		browseButton.addSelectionListener(new SelectionAdapter()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				FileDialog dialog = new FileDialog(shell);
				String result = dialog.open();
				text.setText(result == null ? "" : result);
				canFinish();
			}
		});
	}

	/**
	 * Performs the finish action in launching the selected transformation.
	 * 
	 * @return <code>true</code> if the transformation occurred without errors, <code>false</code> otherwise.
	 */
	public boolean performFinish()
	{
		// Fill missing information about metamodel before starting transformation
		String transfoName = transCombo.getItem(transCombo.getSelectionIndex());
		final TransformationInfo info = TransformationHelper.getInstance().getTransfoInfoByName(transfoName);
		for (String metamodel : metamodelParamsMap.keySet())
		{
			info.setMetamodelFile(metamodel, new File(metamodelParamsMap.get(metamodel)));
		}

		Job job = new Job(TransformationPlugin.getResourceString("TransformationPage.progress.title"))
		{
			@Override
			protected IStatus run(IProgressMonitor monitor)
			{
				monitor.worked(IProgressMonitor.UNKNOWN);
				monitor.setTaskName(TransformationPlugin.getResourceString("TransformationPage.progress.label"));
				
				if (monitor.isCanceled()) 
				{
					return Status.CANCEL_STATUS;
				}

				try
				{
					// Launches the transformation application
					for (Service service : ServicesManager.getInstance().getAllServices().values())
					{
						Type type = ServicesManager.getInstance().getAllType().get(service.getIdType());
						if (type.getNameType().equals(info.getName()))
						{
							Map<String, Object> parameters = new HashMap<String, Object>();
							for (String model : modelParamsMap.keySet())
							{
								parameters.put(model, modelParamsMap.get(model));
							}
							parameters.put("Path", pathProject);
							service.getClass_().serviceRun(parameters);
						}
					}
				}
				catch (Exception e)
				{
					MessageDialog.openError(shell, TransformationPlugin.getResourceString("TransformationPage.progress.label"), "Error during ".concat(info.getName()));
					CorePlugin.log(getClass(), "Error during ".concat(info.getName()), IStatus.ERROR, e);
				}
				finally
				{
					monitor.done();
				}

				return Status.OK_STATUS;
			}
		};
	    job.setUser(true);
	    job.schedule();
		
		return true;
	}

	/**
	 * Indicates whether the transformation can start.<br>
	 * During this step all the required parameters are checked.
	 * 
	 * @return if all information required for the transformation are set, <code>false</code> otherwise.
	 */
	public boolean canFinish()
	{
		if (modelParamsMap.isEmpty())
		{
			setMessage(TransformationPlugin.getResourceString("TransformationPage.missing.field"), WizardPage.ERROR);
			setPageComplete(false);
			return isPageComplete();
		}

		// Fill in metamodel structure
		Map<String, String> allParams = new HashMap<String, String>();
		allParams.putAll(metamodelParamsMap);
		allParams.putAll(modelParamsMap);

		// Check
		for (String model : allParams.keySet())
		{
			if ("".equals(allParams.get(model)))
			{
				setMessage(TransformationPlugin.getResourceString("TransformationPage.missing.field"), WizardPage.ERROR);
				setPageComplete(false);
				return isPageComplete();
			}
		}

		setMessage(TransformationPlugin.getResourceString("TransformationPage.import.ok"), WizardPage.INFORMATION);
		setPageComplete(true);
		return isPageComplete();
	}
}