/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS),
*    Sbastien Gabel (CS), 
*******************************************************************************/ 

package org.topcased.service.updatesite;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class CheckXMLValidity
{
	/**
	 * This class is a tool aimed to work around XML file. 
	 * It checks that the content of the feature repository (locale, FTP, SSH) correponds to the elements described in XML file (site.xml).
	 * Two kinds of methods are important : one detects problems between repository and XML file. 
	 * The other solves the problems in deleting the elements which are not relevant in the XML file.    
	 */

    // Constantes using to parse and extract information from 'site.xml'  
	private static final String FEATURE_TAG = "feature";
    
    private static final String SITE_TAG = "site";

    private static final String URL_ATTRIBUTE = "url";
    
    // Reference to the map containing the features name.
    private static Map mapContentRepositoryPluginFeatureUS;
    
    // Reference to the current UpdateSiteWizard
    private static UpdateSiteWizard wizard;
    
    // Reference to the repositoy path which can represent a FTP, SSH or locale connection.
    private static String path;
    
    
    /**
     * Constructor of the class. 
     */
    public CheckXMLValidity()
    {
    	path = "";
    	mapContentRepositoryPluginFeatureUS = new HashMap();
    }
    
    /**
     * Setter for the repository path.
     * @param p represents the path of the repository 
     */
    public static void setPath(String p)
    {
    	path = p;
    }
    
    /**
     * Getter for the repository path. 
     * @return the path of the (temp) repository. 
     */
    public static String getPath()
    {
    	return path;
    }
    
    /**
     * Setter of the hashmap RepositoryPluginFeaturesUS. 
     * @param content is the hashmap containing the names of the available TA features.
     */
    public static void setMapContentRepository(Map content)
    {
    	mapContentRepositoryPluginFeatureUS = content;
    }
    
    /**
     * Getter of the the hashmap RepositoryPluginFeaturesUS.  
     * @return the map containing the names of the available TA features. 
     */
    public static Map getMapContentRepository()
    {
    	return mapContentRepositoryPluginFeatureUS;
    }
	
    /**
     *  Setter of the wizard object
     *  @param w represents the current wizard used for this operation.
     */
    public static void setWizard(UpdateSiteWizard w)
    {
    	wizard = w;
    }
    
    /**
     * Getter of the current wizard object in use.
     * @return a reference to the current wizard in use. 
     */
    public static UpdateSiteWizard getWizard()
    {
    	return wizard;
    }
    
    /**
     * This method tests if the XML content matchs with the repository content.
     * 
     * @return true if the XML description matchs with the repository content, false otherwise.
     */
    private static boolean checkXMLContentWithRepository()
    {
        boolean contentIsGood = true;
    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        
        try
        {
            DocumentBuilder parser = factory.newDocumentBuilder();
            File xml = new File(path + File.separator + "site.xml");
            if (xml.exists())
            {
	            Document document = parser.parse(xml);
	            NodeList siteList = document.getElementsByTagName(SITE_TAG);
	            
	            for (int i = 0; i < siteList.getLength(); i++)
	            {
	                Element node = (Element) siteList.item(i);
	                
	                // We list all the features described in site.xml
	                NodeList featureList = node.getElementsByTagName(FEATURE_TAG);
	                for (int j = 0; j < featureList.getLength(); j++)
	                {
	                	Element featureElement = (Element) featureList.item(j);
	                    String url = featureElement.getAttribute(URL_ATTRIBUTE);
	                    String name = url.substring(url.indexOf("/") + 1, url.length());
	                    
	                    // Test that the content between the XML file and repository are the same
	                    if(!mapContentRepositoryPluginFeatureUS.containsKey(name))
	                    {
	                    	contentIsGood = false;
	                    	CorePlugin.log("The feature " + name + " is described in site.xml but it has not been found in target repository. Please check its physical content." , IStatus.WARNING);
	                    }
	                }
	            }
            }
        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log("ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
        }
        catch (SAXException e)
        {
            CorePlugin.log("Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log("Error while opening file " + path + "site.xml", IStatus.ERROR);
        }
    	return contentIsGood ;
    }
    
    /**
     * This method arrange the XML file according to the user choice. 
     * The nodes describing a feature not present in the directory are removed from the XML flow.  
     * The result is a new XML. 
     */
    private static void arrangeXMLFile()
    {
    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);
        
        try
        {
        	DocumentBuilder parser = factory.newDocumentBuilder();
            Document document = parser.parse(new File(path + File.separator + "site.xml"));
            NodeList siteList = document.getElementsByTagName(SITE_TAG);
            
            for (int i = 0; i < siteList.getLength() ; i++)
            {
                Element site = (Element) siteList.item(i);
                
                // We list all the features described in site.xml
                NodeList featureList = site.getElementsByTagName(FEATURE_TAG);
                for (int j=0; j < featureList.getLength(); j++)
                {
                	Element featureElement = (Element) featureList.item(j);
                    String url = featureElement.getAttribute(URL_ATTRIBUTE);
                    String name = url.substring(url.indexOf("/") + 1, url.length());
                    
                    if(!mapContentRepositoryPluginFeatureUS.containsKey(name))
                    {   	
                    	// remove the text node (which contains a CR) just after the feature node.
                    	site.removeChild(featureElement.getNextSibling());  
                    	// remove the feature node which does not exist
                    	site.removeChild(featureElement);
                    	j--;
                    }
                }
            }
            
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();
            // Indent the XML document
            transformer.setOutputProperty(OutputKeys.INDENT, "yes"); 
            transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "3"); 
            // Write the new DOM in the former XML file 
            DOMSource sourceDom = new DOMSource(document);
            StreamResult result = new StreamResult(path + File.separator + "site.xml");
            transformer.transform(sourceDom, result);
        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log("ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
        }
        catch (SAXException e)
        {
            CorePlugin.log("Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log("Error while opening file : " + path + File.separator
                    + "site.xml" + ". The file site.xml does not exist.", IStatus.INFO);
        }  
        catch (TransformerConfigurationException e)
        {
            CorePlugin.log("Transformer Configuration Exception" + e.getMessage(), IStatus.ERROR);
        }
        catch (TransformerException e)
        {
            CorePlugin.log("Transformer Exception" + e.getMessage(), IStatus.ERROR);
        }       
    }  
    
    /**
     * This is the main method aimed to execute the test.
     * @param currentWizard represents the current wizard used.
     * @param repositoryPath is the path where the US features are stored.
     * @param mapFeatures is the hashmap containing the name of the TA features available. 
     */    
    public static void startTest(UpdateSiteWizard currentWizard, String repositoryPath, Map mapFeatures)
    {
    	wizard = currentWizard;
    	mapContentRepositoryPluginFeatureUS = mapFeatures;
    	path = repositoryPath;
    	startTest();
    }
    
    /**
     * This is the main method aimed to execute the test. 
     */
    public static void startTest()
    {
        if (!checkXMLContentWithRepository())
        {
            MessageBox msgBox = new MessageBox(wizard.getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
            msgBox.setText("Mismatch between XML file and feature directory");
            msgBox.setMessage("The XML file describes features which are not in the target repository : "+ path +"feature.\n" + "See Error Log for more details about features.\n " + "This problem can be solved. However some information contained in XML file will be lost !\nDo you want to solve the problem between the repository and the XML file ?");
            int button = msgBox.open();
            if (button == SWT.YES)
            {
            	arrangeXMLFile();
            }
        }	
    }
}
