/******************************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************************/ 
package org.topcased.service.updatesite;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.topcased.bus.core.CorePlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * 
 * This class allow to see information about a given feature :
 * - name
 * - id
 * - version
 * - label
 * - provider name
 * - informations
 */
public class PopupDialog extends Dialog
{
    private Table tableFeatureInfo;

    private String featurePath;

    // XML Tags
    private static final String FEATURE = "feature";

    private static final String DESCRIPTION = "description";

    // XML Attributs
    private static final String ID = "id";

    private static final String LABEL = "label";

    private static final String VERSION = "version";

    private static final String PROVIDER = "provider-name";

    // Label
    private static final String TITLE = "Feature information";

    /**
     * Constructor
     * 
     * @param parentShell
     * @param featurePath
     */
    public PopupDialog(Shell parentShell, String featurePath)
    {
        super(parentShell);
        setShellStyle(SWT.CLOSE | SWT.TITLE | SWT.RESIZE);
        this.featurePath = featurePath;
    }

    /**
     * Set the title of the popup dialog : "Informations"
     * 
     * @param newShell
     */
    protected void configureShell(Shell newShell)
    {
        super.configureShell(newShell);
        newShell.setText(TITLE);
        newShell.setSize(600,350);
    }

    /**
     * Initialize the buttons of the dialog : only the button 'OK'
     * 
     * @param parent
     */
    protected void createButtonsForButtonBar(Composite parent)
    {
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    }

    /**
     * Create the controls on the dialog
     * 
     * @param parent
     * @return Control
     */
    protected Control createDialogArea(Composite parent)
    {
    	Composite container = (Composite) super.createDialogArea(parent);
        container.setLayout(new GridLayout());

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);

        File fileToParse = null;
        File propertyFile = null;
        boolean isJarFile = false;
        if (featurePath.endsWith(".jar"))
        {
            isJarFile = true;
        }

        // Extract 'feature.xml' from the jar file to a temporary file
        File pathFile = new File(featurePath);
        if (isJarFile)
        {
            try
            {
                fileToParse = FilesTreatment.createTempFileFromJar(pathFile, "feature.xml", FilesTreatment.FEATURE_XML);
                propertyFile = FilesTreatment.createTempFileFromJar(pathFile, "feature.properties", FilesTreatment.FEATURE_PROPERTIES);
            }
            catch (IOException e1)
            {
                CorePlugin.log(this.getClass(), "IOException : " + e1.getMessage(), IStatus.ERROR);
            }
        }
        else
        {
            fileToParse = new File(featurePath + File.separator + "feature.xml");
            propertyFile = new File(featurePath + File.separator + "feature.properties");
        }

        // Parser
        try
        {
            DocumentBuilder parser = factory.newDocumentBuilder();
            // Document
            Document document = parser.parse(fileToParse);
            NodeList featureList = document.getElementsByTagName(FEATURE);
            int nbNode = featureList.getLength();
            for (int i = 0; i < nbNode; i++)
            {
                Element node = (Element) featureList.item(i);
                String id = node.getAttribute(ID);
                String label = node.getAttribute(LABEL);
                String version = node.getAttribute(VERSION);
                String providerName = node.getAttribute(PROVIDER);

                NodeList descriptionList = node.getElementsByTagName(DESCRIPTION);
                int nbDescription = descriptionList.getLength();
                String description = null;
                for (int j = 0; j < nbDescription; j++)
                {
                    Element descriptionElement = (Element) descriptionList.item(i);
                    description = descriptionElement.getFirstChild().getNodeValue().trim();
                }

                // Label information
                Label text = new Label(container, SWT.NONE);
                text.setText("Information about " + FilesTreatment.analyzeString(label, propertyFile) + " feature.");

                // Display table
                tableFeatureInfo = new Table(container, SWT.BORDER);
                tableFeatureInfo.setLayoutData(new GridData(GridData.FILL_BOTH));
                tableFeatureInfo.setLinesVisible(true);
                tableFeatureInfo.setHeaderVisible(true);

                final TableColumn tableColumn_label_feature = new TableColumn(tableFeatureInfo, SWT.NONE);
                tableColumn_label_feature.setWidth(120);
                tableColumn_label_feature.setText("Label");

                final TableColumn tableColumn_value_feature = new TableColumn(tableFeatureInfo, SWT.NONE);
                tableColumn_value_feature.setWidth(400);
                tableColumn_value_feature.setText("Value");

                tableFeatureInfo.removeAll();
                TableItem item = new TableItem(tableFeatureInfo, 0);
                item.setText("Feature : ");
                item.setText(1, featurePath);
                tableFeatureInfo.showItem(item);

                item = new TableItem(tableFeatureInfo, 0);
                item.setText("Id : ");
                item.setText(1, FilesTreatment.analyzeString(id, propertyFile));
                tableFeatureInfo.showItem(item);

                item = new TableItem(tableFeatureInfo, 0);
                item.setText("Label : ");
                item.setText(1, FilesTreatment.analyzeString(label, propertyFile));
                tableFeatureInfo.showItem(item);

                item = new TableItem(tableFeatureInfo, 0);
                item.setText("Version : ");
                item.setText(1, FilesTreatment.analyzeString(version, propertyFile));
                tableFeatureInfo.showItem(item);

                item = new TableItem(tableFeatureInfo, 0);
                item.setText("Provider name : ");
                item.setText(1, FilesTreatment.analyzeString(providerName, propertyFile));
                tableFeatureInfo.showItem(item);

                item = new TableItem(tableFeatureInfo, 0);
                item.setText("Description : ");
                item.setText(1, FilesTreatment.analyzeString(description, propertyFile));

                tableFeatureInfo.showItem(item);
                tableFeatureInfo.redraw();

            }
        }
        catch (ParserConfigurationException e)
        {
            CorePlugin.log(this.getClass(), "ParserConfiguration error : " + e.getMessage(), IStatus.ERROR);
        }
        catch (SAXException e)
        {
            CorePlugin.log(this.getClass(), "Error while parsing document : " + e.getMessage(), IStatus.ERROR);
        }
        catch (IOException e)
        {
            CorePlugin.log(this.getClass(), "Error while opening file : " + e.getMessage() + ". Maybe is not a feature.", IStatus.ERROR);
        }

        return container;
    }
}
