/*******************************************************************************
* Copyright (c) 2005 AIRBUS FRANCE.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), 
*    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
*******************************************************************************/ 
package org.topcased.service.updatesiteproject;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.ftp.FtpException;
import org.eclipse.ftp.IDirectoryEntry;
import org.eclipse.ftp.internal.FTPClient;
import org.topcased.bus.core.CorePlugin;

import com.sshtools.j2ssh.SftpClient;
import com.sshtools.j2ssh.sftp.FileAttributes;
import com.sshtools.j2ssh.sftp.SftpFile;

/**
 * 
 * This class groups some files treatments. They are used in the diffrent WizardPages
 */
public class FilesTreatment
{
    /**
     * Define the file type as feature.xml
     */
    public static final int FEATURE_XML = 0;

    /**
     * Define the file type as plugin.xml
     */
    public static final int PLUGIN_XML = 1;

    /**
     * Define the file type as feature.properties
     */
    public static final int FEATURE_PROPERTIES = 3;

    /**
     * Define the file type as plugin.properties
     */
    public static final int PLUGIN_PROPERTIES = 2;

    /**
     * Create 'tempName' file XML/properties which is the
     * 'feature.xml'/'plugin.xml'/'feature.properties'/'plugin.properties' from
     * the jarFile(pathFile)
     * 
     * @param pathFile : jarFile
     * @param tempName : name of the temporaty
     *            'feature.xml'/'plugin.xml'/'feature.properties'/'plugin.properties'
     * @param fileType : identify the type of file which must be read :
     *            FEATURE_XML/PLUGIN_XML/FEATURE_PROPERTIES/PLUGIN_PROPERTIES
     * @return fileToParse or null if the file do not exist
     * @throws IOException
     * @throws FileNotFoundException
     */
    public static File createTempFileFromJar(File pathFile, String tempName, int fileType)
            throws FileNotFoundException, IOException
    {
        // Unjar file
        String featureName = pathFile.getName().substring(0, pathFile.getName().lastIndexOf('.'));
        JarInputStream input = new JarInputStream(new FileInputStream(pathFile));
        if (input == null)
        {
            return null;
        }
        // Moving to the needed jarEntry
        JarEntry nextEntry = input.getNextJarEntry();
        String jarEntryName = nextEntry.getName();

        String finalPath = "";

        switch (fileType)
        {
            case (FEATURE_XML):
                finalPath =  featureName + File.separator + "feature.xml";
                break;
            case (PLUGIN_XML):
                finalPath =  featureName + File.separator + "plugin.xml";
                break;
            case (FEATURE_PROPERTIES):
                finalPath =  featureName + File.separator + "feature.properties";
                break;
            case (PLUGIN_PROPERTIES):
                finalPath =  featureName + File.separator + "plugin.properties";
                break;
            default:
                return null;
        }

        while (!(finalPath.indexOf(jarEntryName)!=-1) && nextEntry != null)
        {
            nextEntry = input.getNextJarEntry();
            if (nextEntry != null)
            {
                jarEntryName = nextEntry.getName();
            }
        }
        if (!(finalPath.indexOf(jarEntryName)!=-1) && nextEntry == null)
        {
            return null;
        }

        // File to Parse : extract
        File fileToParse = File.createTempFile(tempName, "");
        FileOutputStream output = new FileOutputStream(fileToParse);
        byte[] buffer = new byte[1024];
        int length;
        while ((length = input.read(buffer)) != -1)
        {
            output.write(buffer, 0, length);
        }
        input.close();
        output.close();
        return fileToParse;
    }

    /**
     * Give automaticaly the state of the file taking its source and destination
     * 
     * @param srcDst the SourceDestination of the file
     * @param typeFile the type of the file : plugin or feature (not used)
     */
    public static void autoState(SourceDestination srcDst, int typeFile)
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setCoalescing(true);

        String featurePath = srcDst.getName();
        String featurePathJar = srcDst.getName();
        if (!featurePathJar.endsWith(".jar"))
        {
            featurePathJar = featurePathJar.concat(".jar");
        }

        if (!isFileExists(srcDst.getSource()) && !isFileExists(srcDst.getSource() + ".jar"))
        {
            srcDst.setState(SourceDestination.MISSING);
        }
        else
        {
            if (isFileExists(srcDst.getDestination() + featurePath)
                    || isFileExists(srcDst.getDestination() + featurePathJar))
            {
                srcDst.setState(SourceDestination.STILL_EXIST);
            }
            else
            {
                srcDst.setState(SourceDestination.DONT_EXIST);
            }
        }
    }

    /**
     * Search if the file exists at the path given in parameter
     * 
     * @param path of the file (directory or jar file)
     * @return boolean true if the file exists at the path, false otherwise
     */
    public static boolean isFileExists(String path)
    {
        File fileTest = new File(path);
        if (fileTest.exists())
        {
            return true;
        }
        return false;
    }

    /**
     * Analyze String. If the String begin with '%', the string returned is the
     * corresponding value define in pathFileProperty else it is the String
     * itself
     * 
     * @param property
     * @param pathFileProperty
     * @return String
     * @throws IOException
     */
    public static String analyzeString(String property, File pathFileProperty) throws IOException
    {
        String result = "";
        if (property.length()>0 && property.charAt(0) == '%')
        {
            property = property.substring(1);
            BufferedReader bufferedReader = new BufferedReader(new FileReader(pathFileProperty));
            String line = null;
            while ((line = bufferedReader.readLine()) != null && result.equals(""))
            {
                line = line.trim();
                if (line.startsWith(property))
                {
                    result = line.substring(line.indexOf('=') + 1);
                    while (result.endsWith("\\"))
                    {
                        result = result.substring(0, result.length() - 1);
                        result = result.concat(bufferedReader.readLine());
                    }
                }
            }
            bufferedReader.close();
            return result;
        }
        else
        {
            return property;
        }
    }

    /**
     * Search if the given file exists without any special version
     * 
     * @param path of the file
     * @return true if the file exists, false otherwise
     */
    public static boolean isFileExistsWithoutVersion(String path)
    {
        // CorePlugin.log(this.getClass(), path, IStatus.ERROR);

        File repository = new File(path.substring(0, path.lastIndexOf(File.separator)));

        final String withoutVersionPath;
        if (path.lastIndexOf("_") != -1)
        {
            withoutVersionPath = path.substring(0, path.lastIndexOf('_'));
        }
        else
        {
            withoutVersionPath = path;
        }

        // CorePlugin.log(this.getClass(), withoutVersionPath, IStatus.WARNING);

        File[] repositoryContent = repository.listFiles((new FileFilter()
        {
            public boolean accept(File pathname)
            {
                boolean acceptReturn = false;
                String jarFile = pathname.toString();
                if (jarFile.lastIndexOf("_") != -1)
                {
                    jarFile = jarFile.substring(0, jarFile.lastIndexOf('_'));
                }

                if (jarFile.equals(withoutVersionPath))// || jarFile.equals(withoutVersionPath+".jar"))
                {
                    acceptReturn = true;
                }
                return acceptReturn;
            }
        }));

        // CorePlugin.log(this.getClass(), repositoryContent[0].getPath(),
        // IStatus.INFO);

        if (repositoryContent!=null&&repositoryContent.length != 0)
        {
            return true;
        }
        return false;
    }

    /**
     * Search a file with the same name and an other version ( like name_X.Y.Z)
     * and return the path of this file
     * 
     * @param path of the file with or without version (name or name_X.Y.Z)
     * @return the new path of a file with a version or null if any file exists
     */
    public static String getFileWithVersionFromNoVersion(String path)
    {
        File repository = new File(path.substring(0, path.lastIndexOf(File.separator)));

        final String withoutVersionPath;
        if (path.lastIndexOf("_") != -1)
        {
            withoutVersionPath = path.substring(0, path.lastIndexOf('_'));
        }
        else
        {
            withoutVersionPath = path;
        }

        File[] repositoryContent = repository.listFiles((new FileFilter()
        {
            public boolean accept(File pathname)
            {
                boolean acceptReturn = false;
                String jarFile = pathname.toString();
                if (jarFile.lastIndexOf("_") != -1)
                {
                    jarFile = jarFile.substring(0, jarFile.lastIndexOf('_'));
                }

                if (jarFile.equals(withoutVersionPath))
                {

                    acceptReturn = true;
                }
                return acceptReturn;
            }
        }));

        if (repositoryContent!=null&&repositoryContent.length != 0)
        {
            return repositoryContent[0].getPath();
        }
        return null;
    }

    /**
     * 
     * @param parentPath
     * @param pathDestination
     * @param file
     * @param clientFTP
     * @param monitor
     * @return
     */
    public static boolean copyFileFromFTP(final String parentPath, final String pathDestination,
            final IDirectoryEntry file, final FTPClient clientFTP, IProgressMonitor monitor)
    {
        InputStream fileIn;

        try
        {
            if (file.hasDirectorySemantics())
            {
                String path = parentPath + file.getName() + File.separator;
                String pathDest = pathDestination + file.getName() + File.separator;

                File dir = new File(pathDest);
                dir.mkdir();
                dir.deleteOnExit();

                IDirectoryEntry[] filesDir = clientFTP.listFiles(path, false, monitor);

                for (int j = 0; j < filesDir.length; j++)
                {
                    copyFileFromFTP(path, pathDest, filesDir[j], clientFTP, monitor);
                }
            }
            else
            {
                fileIn = clientFTP.getFile(parentPath + file.getName(), true, (long) 0, monitor);

                try
                {
                    File temp = new File(pathDestination + file.getName());
                    temp.deleteOnExit();
                    FileOutputStream outputFile = new FileOutputStream(temp);
                    // Copy
                    byte[] buffer = new byte[1024];
                    int length;
                    while ((length = fileIn.read(buffer)) != -1)
                    {
                        outputFile.write(buffer, 0, length);
                    }
                    fileIn.close();
                    outputFile.close();
                }
                catch (FileNotFoundException e)
                {
                	CorePlugin.log(e);
                    return false;
                }
                catch (IOException e)
                {
                	CorePlugin.log(e);
                    return false;
                }
            }
        }
        catch (FtpException e)
        {
        	CorePlugin.log(e);
        }
        return true;
    }

    /**
     * Copy the given file into the pathDestination, from a FTP server to local destination
     * @param parentPath of the file
     * @param pathDestination the destination path
     * @param file the specified file
     * @param clientFTP to remote FTP server
     * @param monitor the progress monitor or null
     * @return
     */
    public static boolean copyPluginFeatureFromFTP(final String parentPath, final String pathDestination,
            final IDirectoryEntry file, final FTPClient clientFTP, IProgressMonitor monitor) throws FtpException, IOException
    {
        try
        {
	    	InputStream fileIn;
	    	if (file.hasDirectorySemantics())
	        {
	            String path = parentPath + file.getName() + File.separator;
	            String pathDest = pathDestination + file.getName() + File.separator;
	
	            File dir = new File(pathDest);
	            dir.mkdir();
	            dir.deleteOnExit();
	
	            IDirectoryEntry[] filesDir = clientFTP.listFiles(path, false, monitor);
	            if (filesDir.length == 0)
	            {
	                // end system is different from the source system
	                if (File.separator.equals("\\"))
	                {
	                    filesDir = clientFTP.listFiles(path = path.replace('\\', '/'), false, monitor);
	                }
	                else
	                {
	                    filesDir = clientFTP.listFiles(path = path.replace('/', '\\'), false, monitor);
	                }
	            }
	            for (int j = 0; j < filesDir.length; j++)
	            {
	                if (filesDir[j].getName().endsWith(".xml") || filesDir[j].getName().endsWith(".properties"))
	                {
	                    fileIn = clientFTP.getFile(path + filesDir[j].getName(), true, (long) 0, monitor);
	
	                    try
	                    {
	                        File temp = new File(pathDest + filesDir[j].getName());
	                        temp.deleteOnExit();
	                        FileOutputStream outputFile = new FileOutputStream(temp);
	                        // Copy
	                        byte[] buffer = new byte[1024];
	                        int length;
	                        while ((length = fileIn.read(buffer)) != -1)
	                        {
	                            outputFile.write(buffer, 0, length);
	                        }
	                        fileIn.close();
	                        outputFile.close();
	                    }
	                    catch (FileNotFoundException e)
	                    {
	                    	CorePlugin.log(e);
	                        return false;
	                    }
	                }
	            }
	        }
	        else
	        {
	        	//File fileLength = new File(parentPath + file.getName());
	            //fileIn = clientFTP.getFile(parentPath + file.getName(), true, fileLength.length(), monitor);
	        	fileIn = clientFTP.getFile(parentPath + file.getName(), true, (long)0, monitor);
	
	            try
	            {
	                File temp = new File(pathDestination + file.getName());
	                temp.deleteOnExit();
	                FileOutputStream outputFile = new FileOutputStream(temp);
	                // Copy
	                byte[] buffer = new byte[1024];
	                int length;
	                while ((length = fileIn.read(buffer)) != -1)
	                {
	                    outputFile.write(buffer, 0, length);
	                }
	                fileIn.close();
	                outputFile.close();
	            }
	            catch (FileNotFoundException e)
	            {
	            	CorePlugin.log(e);
	                return false;
	            }
	            catch (IOException e)
	            {
	            	CorePlugin.log(e);
	                return false;
	            }
	        }
        }
	    catch (FtpException e)
        {
	    	CorePlugin.log(e);
        }
        return true;
    }

    /**
     * Empty the given file or directory.
     * If the file is a directory, all the files which it contains are emptied.
     * @param file the specified file or directory
     */
    public static boolean emptyFile(File file)
    {
        File[] files = file.listFiles();

        if (files != null)
        {
            for (int i = 0; i < files.length; i++)
            {
                if (files[i].isDirectory())
                {
                    emptyFile(files[i]);
                }
                else
                {
                    files[i].delete();
                }
            }
        }

        return file.delete();
    }
	
    /**
     * CreateSiteUpdateSiteWizardTreatment
     * @param file
     * @param pathDest
     * @param clientSFTP
     */
    public static void copyPluginFeatureFromSFTP(SftpFile file, String pathDest, SftpClient clientSFTP) throws IOException
    {
        if (file.isDirectory())
        {
        	CorePlugin.log((((SftpFile) file).getFilename()),IStatus.INFO);
            if (!file.getFilename().equals("..") && !file.getFilename().equals("."))
            {
                clientSFTP.mkdir(pathDest + file.getFilename());

                String path = file.getAbsolutePath();

                List files = null;
                try
                {
                    files = clientSFTP.ls(path);
                }
                catch (IOException e)
                {
                    // end system is different from the source system
                    if (File.separator.equals("\\"))
                    {
                        try
                        {
                            files = clientSFTP.ls(path = path.replace('\\', '/'));
                        }
                        catch (IOException e1)
                        {
                        	CorePlugin.log(e);
                        	CorePlugin.log(e1);
                        }
                    }
                    else
                    {
                        try
                        {
                            files = clientSFTP.ls(path = path.replace('/', '\\'));
                        }
                        catch (IOException e2)
                        {
                        	CorePlugin.log(e);
                        	CorePlugin.log(e2);
                        }
                    }
                }

                for (int i = 0; i < files.size(); i++)
                {
                	CorePlugin.log(((SftpFile) files.get(i)).getFilename(),IStatus.INFO);
                    if (((SftpFile) files.get(i)).getFilename().endsWith(".xml")
                            || ((SftpFile) files.get(i)).getFilename().endsWith(".properties"))
                    {
                        clientSFTP.get(((SftpFile) files.get(i)).getAbsolutePath(), pathDest + file.getFilename()
                                + File.separator + ((SftpFile) files.get(i)).getFilename());
                    }
                }
            }
        }
        else
        {
            clientSFTP.get(file.getAbsolutePath(), pathDest + file.getFilename());
        }
    }
    
    /**
     * Empty all files referenced by the given map
     * Not empty the files which are still installed
     * @param map containing SourceDestination with the stat attribut initialized
     */
    public static void emptyFilesFromMap(Map map)
    {
        Iterator it = map.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String) it.next();
            if (map.get(key) instanceof SourceDestination)
            {
                SourceDestination sd = (SourceDestination) map.get(key);

                if (sd.getState() != SourceDestination.STILL_EXIST && sd.getState() != SourceDestination.MISSING)
                {
                    File emptyFile = new File(sd.getDestination()+sd.getName());
                    if (!FilesTreatment.emptyFile(emptyFile))
                    {
                        if (sd.getName().endsWith(".jar"))
                        {
                            emptyFile = new File(sd.getDestination()+sd.getName().substring(0, sd.getName().indexOf(".jar")));
                            FilesTreatment.emptyFile(emptyFile);
                        }
                        else
                        {
                            emptyFile = new File(sd.getDestination()+sd.getName()+".jar");
                            FilesTreatment.emptyFile(emptyFile);
                        }
                    }
                }
            }
        }
    }
	
	/**
     * Empty all files referenced by the given map
     * Not empty the files which are still installed
     * @param map containing SourceDestination with the stat attribut initialized
     */
    public static void emptyFilesFTPFromMap(FTPClient clientFTP, Map map, String path)
    {
        Iterator it = map.keySet().iterator();
        while (it.hasNext())
        {
            String key = (String) it.next();
            if (map.get(key) instanceof SourceDestination)
            {
                SourceDestination sd = (SourceDestination) map.get(key);

                if (sd.getState() != SourceDestination.STILL_EXIST && sd.getState() != SourceDestination.MISSING)
                {
					try
					{
						clientFTP.deleteFile(path+sd.getName(), null);
					}
					catch (FtpException e)
					{
						try
						{
							clientFTP.deleteFile(path+sd.getName()+".jar", null);
						}
						catch (FtpException e1)
						{
							CorePlugin.log(e1);
						}
					}
                }
            }
        }
    }
	
    public static long getFileLength(File file)
    {
        File[] files = file.listFiles();

        if (files != null)
        {
            long filesLength = 0;
            for (int i = 0; i < files.length; i++)
            {
                if (files[i].isDirectory())
                {
                    filesLength += FilesTreatment.getFileLength(files[i]);
                }
                else
                {
                    filesLength += files[i].length();
                }
            }
            return filesLength;
        }
        return file.length();
    }
    
    public static long getFileLengthFTP(FTPClient clientFTP, IDirectoryEntry file, String path)
    {
        try
        {
            if (file.hasDirectorySemantics())
            {
                IDirectoryEntry[] files = clientFTP.listFiles(path, false, null);
                if (files.length == 0)
                {
                    // end system is different from the source system
                    if (File.separator.equals("\\"))
                    {
                        files = clientFTP.listFiles(path = path.replace('\\', '/'), false, null);
                    }
                    else
                    {
                        files = clientFTP.listFiles(path = path.replace('/', '\\'), false, null);
                    }
                }
                
                long filesLength = 0;
                for (int i = 0; i < files.length; i++)
                {
                    if (files[i].hasDirectorySemantics())
                    {
                        filesLength += FilesTreatment.getFileLengthFTP(clientFTP, files[i], path + files[i].getName());
                    }
                    else
                    {
                        filesLength += files[i].getSize();
                    }
                }
                return filesLength;
            }
            else
            {
                return file.getSize();
            }
        }
        catch (FtpException e)
        {
            return 0;
        }
    }
    
    public static long getFileLengthSSH(SftpFile file, SftpClient clientSFTP)
    {
        FileAttributes fileProp = file.getAttributes();
        
        if (fileProp.isDirectory())
        {
            String path = file.getAbsolutePath();
            List files = null;
            try
            {
                files = clientSFTP.ls(path);
            }
            catch (IOException e)
            {
                // end system is different from the source system
                if (File.separator.equals("\\"))
                {
                    try
                    {
                        path = path.replace('/', '\\');
                        files = clientSFTP.ls(path);
                    }
                    catch (IOException e1)
                    {
                    	CorePlugin.log(e);
                    	CorePlugin.log(e1);
                    }
                }
                else
                {
                    try
                    {
                        path = path.replace('/', '\\');
                        files = clientSFTP.ls(path);
                    }
                    catch (IOException e2)
                    {
                    	CorePlugin.log(e);
                    	CorePlugin.log(e2);
                    }
                }
            }
    
            if (files != null)
            {
                long filesLength = 0;
                for (int i = 0; i < files.size(); i++)
                {
                    FileAttributes filesProp = ((SftpFile)files.get(i)).getAttributes();
                    if (filesProp.isDirectory())
                    {
                        filesLength += FilesTreatment.getFileLengthSSH((SftpFile)files.get(i), clientSFTP);
                    }
                    else
                    {
                        filesLength += fileProp.getSize().longValue();
                    }
                }
                return filesLength;
            }
            return 0;
        }
        else
        {
            return fileProp.getSize().longValue();
        }
    }
}
