/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.service.writeemfxmi;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.topcased.modeler.diagrams.model.Diagrams;

/**
 * Drives the model. Acts as the model entry point.
 */
public class WriteModelManager
{
	/**
	 * In EMF a resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Gives access to the top level object contained in the resource.
	 */
	private EObject model = null;

	/**
	 * Factory
	 */
	private AdapterFactory adapterFactory;

	/**
	 * Returns the resource containing the network. This uses lazy initialization.
	 * 
	 * @param path
	 * @return the associated resource
	 */
	public Resource getResource(IPath path)
	{
		if (resource == null)
		{
			ResourceSet resSet = getResourceSet();
			resource = resSet.getResource(URI.createPlatformResourceURI(path.toString(), false), true);
		}
		return resource;
	}

	/**
	 * Creates a resource to contain the network. The resource file does not exist yet.
	 * 
	 * @param path
	 * @return the associated resource
	 */
	private Resource createResource(IPath path)
	{
		if (resource == null)
		{
			ResourceSet resSet = getResourceSet();
			resource = resSet.createResource(URI.createPlatformResourceURI(path.toString(), false));
		}
		return resource;
	}

	/**
	 * Returns the resource set.
	 * 
	 * @return a new ResourceSet
	 */
	private ResourceSet getResourceSet()
	{
		// Obtains a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Creates a new model.
	 * 
	 * @param path the model path
	 * @return the main object
	 */
	public EObject createModel(IPath path)
	{
		createResource(path);
		return null;
	}

	/**
	 * Loads the content of the model from the file.
	 * 
	 * @param path
	 * @throws IOException
	 */
	public void load(IPath path) throws IOException
	{
		getResource(path);
		// FIXME : change template of Map
		resource.load(new HashMap<Object, Object>());
	}

	/**
	 * reloads the content of the model from the file.
	 * 
	 * @param path
	 * @throws IOException
	 */
	public void reload(IPath path) throws IOException
	{
		getResource(path).unload();
		load(path);
	}

	/**
	 * Saves the content of the model to the file.
	 * 
	 * @param path
	 * @param diagrams
	 * @throws IOException
	 */
	public void save(IPath path, EObject diagrams) throws IOException
	{
		// Save from __Minibus doSave(IProgressMonitor monitor)
		ResourceSetImpl rsrcSet2 = new ResourceSetImpl();

		// Resource 1 : *.___di
		URI uri = URI.createFileURI(path.toString());
		Resource rsc = rsrcSet2.createResource(uri);
		rsc.getContents().clear();
		rsc.getContents().add(0, diagrams);

		// Add Christophe
		if (path.getFileExtension().equals("di"))
		{
			// Resource 2 : *.___
			URI uri2 = URI.createPlatformResourceURI(path.removeFileExtension().toString(), false);
			Resource resource2 = rsrcSet2.createResource(uri2); 
			if (diagrams instanceof Diagrams)
			{
				resource2.getContents().add(0, ((Diagrams) diagrams).getModel());
			}
		}

		// Save Resource 1 et Resource 2
		for (Resource currentResource : rsrcSet2.getResources())
		{
			currentResource.save(Collections.EMPTY_MAP);
		}
	}

	/**
	 * Saves the content of the model to a new file.
	 * 
	 * @param path
	 * @param theModel The model to save
	 * @throws IOException
	 */
	public void saveAs(IPath path, EObject theModel) throws IOException
	{
		model = theModel;
		createResource(path);
		getResource(path);
		setModel();
		resource.save(new HashMap<Object, Object>());
	}

	/**
	 * Gets the top level object.
	 * 
	 * @return the edited model
	 */
	public EObject getModel()
	{
		if (model == null && !resource.getContents().isEmpty())
		{
			model = resource.getContents().get(0);
		}
		return model;
	}

	/**
	 * Sets the model in the EMF resource
	 */
	public void setModel()
	{
		// Cleans resources content and add the model
		resource.getContents().clear();
		resource.getContents().add(model);
	}

	/**
	 * Returns the Adapter factory for the current model
	 * 
	 * @return the factory
	 */
	public AdapterFactory getAdapterFactory()
	{
		if (adapterFactory == null)
		{
			adapterFactory = new ReflectiveItemProviderAdapterFactory();
		}
		return adapterFactory;
	}
}
