package org.topcased.sysml.tools.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.emf.edit.ui.celleditor.FeatureEditorDialog;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.editor.SysMLEditor;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.SysMLPackage;
import org.topcased.sysml.tools.commands.EMFDesignElementsCoverCommand;
import org.topcased.sysml.tools.commands.GEFDesignElementsCoverCommand;
import org.topcased.sysml.tools.operations.DesignElementsCoverOperations;

/**
 * Action enabling to choose design elements that satisfy a requirement
 */
public class DesignElementsCoverAction implements IObjectActionDelegate
{

    private Requirement requirement;

    private IWorkbenchPart activePart;

    private ILabelProvider labelProvider = new LabelProvider()
    {
        public String getText(Object elem)
        {
            if (elem instanceof NamedElement)
            {
                return ((NamedElement) elem).eClass().getName() + " " + ((NamedElement) elem).getName();
            }
            return null;
        }
    };

    /**
     * selection changed
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     * @param action {@link IAction}
     * @param sel {@link ISelection}
     */
    public void selectionChanged(IAction action, ISelection sel)
    {
        requirement = null;
        action.setEnabled(false);
        if (sel instanceof IStructuredSelection)
        {
            Object selectedObject = ((StructuredSelection) sel).getFirstElement();
            if (selectedObject instanceof Requirement)
            {
                requirement = (Requirement) selectedObject;
                action.setEnabled(true);
            }
        }
    }

    /**
     * Show a dialog enabling to choose the list of design elements that satisfy
     * the current requirement
     * 
     * @param currentCoveringDesignElements the current list of design elements
     *            that satisfy the requirement
     * @return the list of chosen design elements
     */
    private List chooseCoveringDesignElements(List currentCoveringDesignElements)
    {
        List choiceOfValues = new ArrayList();
        choiceOfValues.addAll(DesignElementsCoverOperations.getAvailableDesignElements(requirement));

        Collections.sort(choiceOfValues, new Comparator()
        {
            public int compare(Object o1, Object o2)
            {
                return labelProvider.getText(o1).compareTo(labelProvider.getText(o2));
            }
        });

        FeatureEditorDialog dialog = new FeatureEditorDialog(SysMLPlugin.getActiveWorkbenchWindow().getShell(),
                labelProvider, requirement, SysMLPackage.eINSTANCE.getRequirement(), currentCoveringDesignElements,
                "Design elements cover", choiceOfValues);
        dialog.open();

        if (Window.OK == dialog.getReturnCode())
        {
            return dialog.getResult();
        }

        return null;
    }

    /**
     * run
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     * @param action {@link IAction}
     */
    public void run(IAction action)
    {
        if (requirement != null)
        {
            List < ? > currentDesignElements = DesignElementsCoverOperations.getCoveringDesignElements(requirement);
            List < ? > result = chooseCoveringDesignElements(currentDesignElements);
            if (result != null)
            {
                CommandStack gefCommandStack = (CommandStack) activePart.getAdapter(CommandStack.class);
                if (gefCommandStack != null)
                {
                    EditDomain gefEditDomain = (EditDomain) activePart.getAdapter(DefaultEditDomain.class);
                    GEFDesignElementsCoverCommand command = new GEFDesignElementsCoverCommand(gefEditDomain,
                            requirement, result, currentDesignElements);
                    if (command.canExecute())
                        gefCommandStack.execute(command);
                }
                else if (activePart instanceof SysMLEditor)
                {
                    SysMLEditor editor = (SysMLEditor) activePart;
                    editor.getEditingDomain().getCommandStack().execute(
                            new EMFDesignElementsCoverCommand(requirement, result, currentDesignElements));
                }
            }
        }
    }

    /**
     * set the ActivePart
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     * @param action IAction
     * @param targetPart IWorkbenchPart
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        activePart = targetPart;
    }
}
