/**
 * http://jautodoc.sourceforge.net/
 */
package org.topcased.sysml.tools.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.emf.edit.ui.celleditor.FeatureEditorDialog;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.uml2.uml.NamedElement;
import org.topcased.modeler.sysml.SysMLPlugin;
import org.topcased.modeler.sysml.editor.SysMLEditor;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.SysMLPackage;
import org.topcased.sysml.tools.commands.EMFTestCaseCoverCommand;
import org.topcased.sysml.tools.commands.GEFTestCaseCoverCommand;
import org.topcased.sysml.tools.operations.TestCaseCoverOperations;

/**
 * Action enabling to choose test case that cover a requirement
 */
public class TestCaseCoverAction implements IObjectActionDelegate
{

    private Requirement requirement;

    private IWorkbenchPart activePart;

    private ILabelProvider labelProvider = new LabelProvider()
    {
        public String getText(Object elem)
        {
            if (elem instanceof NamedElement)
            {
                return ((NamedElement) elem).eClass().getName() + " " + ((NamedElement) elem).getName();
            }
            return null;
        }
    };

    /**
     * Selection changed.
     * 
     * @param action IAction
     * @param sel ISelection
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection sel)
    {
        requirement = null;
        action.setEnabled(false);
        if (sel instanceof IStructuredSelection)
        {
            Object selectedObject = ((StructuredSelection) sel).getFirstElement();
            if (selectedObject instanceof Requirement)
            {
                requirement = (Requirement) selectedObject;
                action.setEnabled(true);
            }
        }
    }

    /**
     * Show a dialog enabling to choose the list of test case that verify the
     * current requirement
     * 
     * @param currentCoveringTestCases the current list of test cases that
     *            verify the requirement
     * @return the list of chosen test case
     */
    private List < ? > chooseCoveringTestCase(List < ? > currentCoveringTestCases)
    {
        List < ? > choiceOfValues = new ArrayList();
        choiceOfValues.addAll(TestCaseCoverOperations.getAvailableTestCases(requirement));

        Collections.sort(choiceOfValues, new Comparator()
        {
            public int compare(Object o1, Object o2)
            {
                return labelProvider.getText(o1).compareTo(labelProvider.getText(o2));
            }
        });

        FeatureEditorDialog dialog = new FeatureEditorDialog(SysMLPlugin.getActiveWorkbenchWindow().getShell(),
                labelProvider, requirement, SysMLPackage.eINSTANCE.getRequirement(), currentCoveringTestCases,
                "Test case cover", choiceOfValues);
        dialog.open();

        if (Window.OK == dialog.getReturnCode())
        {
            return dialog.getResult();
        }

        return null;
    }

    /**
     * run 
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     * @param action IAction
     */
    public void run(IAction action)
    {
        if (requirement != null)
        {
            List < ? > currentTestCases = TestCaseCoverOperations.getCoveringTestCases(requirement);
            List < ? > result = chooseCoveringTestCase(currentTestCases);
            if (result != null)
            {
                CommandStack gefCommandStack = (CommandStack) activePart.getAdapter(CommandStack.class);
                if (gefCommandStack != null)
                {
                    EditDomain gefEditDomain = (EditDomain) activePart.getAdapter(DefaultEditDomain.class);
                    GEFTestCaseCoverCommand command = new GEFTestCaseCoverCommand(gefEditDomain, requirement, result,
                            currentTestCases);
                    if (command.canExecute())
                        gefCommandStack.execute(command);
                }
                else if (activePart instanceof SysMLEditor)
                {
                    SysMLEditor editor = (SysMLEditor) activePart;
                    editor.getEditingDomain().getCommandStack().execute(
                            new EMFTestCaseCoverCommand(requirement, result, currentTestCases));
                }
            }
        }
    }

    /**
     * set the ActivePart
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     * @param action {@link IAction}
     * @param targetPart {@link IWorkbenchPart}
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        activePart = targetPart;
    }
}
