/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
******************************************************************************/
package org.topcased.sysml.tools.operations;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.UMLPackage;
import org.topcased.sysml.Requirement;
import org.topcased.sysml.PackageSYSML;
import org.topcased.sysml.Satisfy;
import org.topcased.sysml.SysMLFactory;
import org.topcased.sysml.SysMLPackage;
import org.topcased.sysml.tools.SysMLRequirementsTools;

/**
 * The Class DesignElementsCoverOperations.
 */
public class DesignElementsCoverOperations
{

    /**
     * Return the list of design elements that can satisfy a requirement A query
     * based on the following OCL expression NamedElement.allInstances()
     * 
     * @param requirement the requirement
     * 
     * @return the list of design elements
     */
    public static List getAvailableDesignElements(Requirement requirement)
    {
        return new UniqueEList(ItemPropertyDescriptor.getReachableObjectsOfType(requirement,
                UMLPackage.eINSTANCE.getNamedElement()));
    }

    /**
     * Return the list of design elements that satisfy a requirement A query
     * based on the following OCL expression
     * Satisfy.allInstances()->select(supplier->includes(self)).client->asSet()
     * 
     * @param requirement the requirement
     * 
     * @return the list of design elements
     */
    public static List getCoveringDesignElements(Requirement requirement)
    {
        return requirement.getSatisfiedBy();
    }

    /**
     * Remove a design element from the design elements satisfying a requirement
     * (i.e remove the satisfy links between the design element and the
     * requirement)
     * 
     * @param requirement the requirement
     * @param designElement the design element to remove from the design
     *            elements satisfying the requirement
     */
    public static void removeCoveringDesignElement(Requirement requirement, NamedElement designElement)
    {
        List satisfyLinksToRemove = getSatisfyLinks(requirement, designElement);
        for (Iterator it = satisfyLinksToRemove.iterator(); it.hasNext();)
        {
            ((Satisfy) it.next()).destroy();
        }
    }

    /**
     * Add a design element to the design elements satisfying a requirement (i.e
     * add a satisfy link between the design element and the requirement)
     * 
     * @param requirement the requirement
     * @param designElement the design element to add to the design elements
     *            satisfying the requirement
     * 
     * @return the created satisfy link
     */
    public static Satisfy addCoveringDesignElement(Requirement requirement, NamedElement designElement)
    {
        Satisfy satisfy = SysMLFactory.eINSTANCE.createSatisfy();
        satisfy.getClients().add(designElement);
        satisfy.getSuppliers().add(requirement);
        PackageSYSML rd = SysMLRequirementsTools.getPackageSYSML(requirement);
        if (rd != null)
        {
            rd.getOwnedSatisfy().add(satisfy);
        }
        return satisfy;
    }

    /**
     * Returns all the satisfy links that match a design element to a
     * requirement.
     * 
     * @param requirement the requirement
     * @param designElement the design element
     * 
     * @return result return a list of result
     */
    public static List getSatisfyLinks(Requirement requirement, NamedElement designElement)
    {
        Collection allSatisfy = ItemPropertyDescriptor.getReachableObjectsOfType(requirement,
                SysMLPackage.eINSTANCE.getSatisfy());
        List result = new ArrayList();
        for (Iterator it = allSatisfy.iterator(); it.hasNext();)
        {
            Satisfy satisfyLink = (Satisfy) it.next();
            if (satisfyLink.getClients().contains(designElement) && satisfyLink.getSuppliers().contains(requirement))
                result.add(satisfyLink);
        }
        return result;
    }
}
