package org.topcased.traceability.analyzers.openOffice;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

import tte.exceptions.ConversionException;

public class ODTConverter {

	private File inFile;
	private File outFile;
	private String arguments;
	private File contentFile;
	private File stylesFile;
	private Document domDocument;
	private Element rootNode;
	
	final int BUFFER = 2048;

	/**
	 * This method is a basic OpenOffice Writer document transformer to TopCased
	 * TRAMway document file format.
	 * 
	 * @param inFile
	 *            is the odt file.
	 * @param outFile
	 *            is the xml results file.
	 * @param arguments
	 *            is a String that defines which odt style has to be converted
	 *            in TRAMway object reprsentation. ex:
	 *            ReqID->Requirement;ReqText->Text;...
	 * @throws ConversionException
	 *             when conversion fails
	 */
	public static void convertDocumentToTRAMwayFormat_0_1_0(File inFile,
			File outFile, String arguments, ttm.Document document) throws ConversionException {
		new ODTConverter(inFile, outFile, arguments, document)
				.basicODTtoTRAMwayTransformation();
	};

	private ODTConverter(File anInFile, File anOutFile, String someArguments, ttm.Document document)
			throws ConversionException {
		inFile = anInFile;
		outFile = anOutFile;
		arguments = someArguments;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			domDocument = builder.newDocument();
			domDocument.setXmlVersion("1.0");
			domDocument.setXmlStandalone(true);
			domDocument.setDocumentURI(outFile.toURI().toString());
			rootNode = domDocument.createElement("ttm:Document");
			rootNode.setAttribute("xmlns:ttm","http://org.topcased.traceability.model");
			rootNode.setAttribute("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance");
			rootNode.setAttribute("ident", document.getIdent());
			rootNode.setAttribute("shortDescription", document.getShortDescription());
			rootNode.setAttribute("filename",document.getFilename());
			domDocument.appendChild(rootNode);
		} catch (ParserConfigurationException e) {
			throw new ConversionException(e.getMessage());
		}
	}

	private void basicODTtoTRAMwayTransformation() throws ConversionException {
		ODTunzip();
		OOXPathExtractor ooExtractor = new OOXPathExtractor();
		HashMap<String, String> stylesAsso = ooExtractor
				.extractStylesInFile(stylesFile);
		stylesAsso = ooExtractor.completeStylesFromContentFile(stylesAsso,
				contentFile);

		HashMap<String, String> searchedStyles = getSearchedStyles();
		
		NodeList nodes = ooExtractor.extractNodesWithStyleName(contentFile);
		Element currentParent = rootNode;
		int currentDepth = 0;
		Integer[] numbers = new Integer[]{0,0,0,0,0,0,0,0,0};
		for (int i = 0; i < nodes.getLength(); i++) {
			Node node = nodes.item(i);
			NamedNodeMap attrs = node.getAttributes();
			String internalStyleName = attrs.getNamedItem("text:style-name").getNodeValue();
			String styleDisplayName = stylesAsso.get(internalStyleName);
			String type = searchedStyles.get(internalStyleName);
			if (styleDisplayName != null || type != null) {
				if (type == null) {
					String secondSearch = stylesAsso.get(styleDisplayName);
					if (secondSearch != null) {
						styleDisplayName = secondSearch;
					}
					type = searchedStyles.get(styleDisplayName);
				}
				if (type != null) {
					if(type.matches("Section@\\d+")) {
						while(currentParent.getAttribute("xsi:type").equals("ttm:Requirement")) {
							currentParent = ((Element)(currentParent.getParentNode()));
							//currentDepth -=1;
						}
						Integer localDepth = new Integer(type.split("@")[1]);
						while (localDepth <= currentDepth ) {
							currentParent = (Element) currentParent.getParentNode();
							currentDepth -= 1;
						}
						currentDepth = localDepth;
						String shortDescription = node.getTextContent();
						if(shortDescription != null) {
							numbers[currentDepth-1] +=1;
							String number = "";
							for(int j = 0; j < currentDepth; j++) {
								number += numbers[j].toString();
								number += ".";
							}
							for(int k = currentDepth; k<9; k++){
								numbers[k]=0;
							}
							Element domElt = domDocument.createElement("children");
							domElt.setAttribute("xsi:type", "ttm:Section");						
							domElt.setAttribute("ident", number);					
							domElt.setAttribute("shortDescription", shortDescription);					
							currentParent.appendChild(domElt);
							currentParent = domElt;
						}
						
					} else if(type.equals("Requirement")) {
						if(currentParent.getAttribute("xsi:type").equals("ttm:Requirement"))
							currentParent = (Element) currentParent.getParentNode();
						String ident = node.getTextContent();
						if(ident != null && ident.length()!=0) {
							Element domElt = domDocument.createElement("children");
							domElt.setAttribute("xsi:type", "ttm:Requirement");						
							domElt.setAttribute("ident", ident);					
							currentParent.appendChild(domElt);
							currentParent = domElt;
						}
					} else if(type.equals("CoverLink")) {
						String ident = node.getTextContent();
						if(ident != null && ident.length()!=0) {
							Element domElt = domDocument.createElement("coverLinks");
							domElt.setAttribute("xsi:type", "ttm:CoverLink");						
							domElt.setAttribute("referencedID", ident);					
							currentParent.appendChild(domElt);
						}
					} else if(type.equals("NavigationLink")) {
						String ident = node.getTextContent();
						if(ident != null && ident.length()!=0) {
							Element domElt = domDocument.createElement("navigationLinks");
							domElt.setAttribute("xsi:type", "ttm:NavigationLink");						
							domElt.setAttribute("referencedID", ident);					
							currentParent.appendChild(domElt);
						}
					} else if(type.matches("Attribute@.+")) {
						String ident = node.getTextContent();
						if(ident != null && ident.length()!=0) {
							Element domElt = domDocument.createElement("attributes");
							domElt.setAttribute("xsi:type", "ttm:Attribute");	
							domElt.setAttribute("name", type.split("@")[1]);
							domElt.setAttribute("value", ident);					
							currentParent.appendChild(domElt);
						}
					} else if(type.equals("Text")) {
						String value = node.getTextContent();
						if(value != null && value.length()!=0) {
							Element domElt = domDocument.createElement("texts");
							domElt.setAttribute("xsi:type", "ttm:Text");						
							domElt.setAttribute("value", value);					
							currentParent.appendChild(domElt);
						}
					}
					
				}
			}
		}

		OutputFormat doc = new OutputFormat(domDocument);
		doc.setIndent(0);
		doc.setIndenting(true);
		doc.setEncoding("ISO-8859-1");
		FileOutputStream stream;
		try {
			stream = new FileOutputStream(outFile);
		} catch (FileNotFoundException e) {
			throw new ConversionException("Cannot find temporary file");
		}
		XMLSerializer sauv = new XMLSerializer(stream, doc);
		try {
			sauv.serialize(domDocument);
		} catch (IOException e) {
			throw new ConversionException("Cannot write file for TRAMway");
		}
		try {
			stream.close();
		} catch (IOException e) {
			throw new ConversionException("Cannot save intermediate file for TRAMway");
		}
		contentFile.delete();
		stylesFile.delete();
	}

	private void ODTunzip() throws ConversionException {
		try {
			BufferedOutputStream dest = null;
			FileInputStream fis = new FileInputStream(inFile);
			ZipInputStream zis = new ZipInputStream(
					new BufferedInputStream(fis));
			ZipEntry entry;
			while ((entry = zis.getNextEntry()) != null) {
				String name = entry.getName();
				if (name.equals("content.xml")) {
					contentFile = new File(name);
					int count;
					byte data[] = new byte[BUFFER];
					// write the files to the disk
					FileOutputStream fos = new FileOutputStream(contentFile);
					dest = new BufferedOutputStream(fos, BUFFER);
					while ((count = zis.read(data, 0, BUFFER)) != -1) {
						dest.write(data, 0, count);
					}
					dest.flush();
					dest.close();
				}
				if (name.equals("styles.xml")) {
					stylesFile = new File(name);
					int count;
					byte data[] = new byte[BUFFER];
					// write the files to the disk
					FileOutputStream fos = new FileOutputStream(stylesFile);
					dest = new BufferedOutputStream(fos, BUFFER);
					while ((count = zis.read(data, 0, BUFFER)) != -1) {
						dest.write(data, 0, count);
					}
					dest.flush();
					dest.close();
				}
			}
			zis.close();
		} catch (Exception e) {
			throw new ConversionException(e.getMessage());
		}
	}

	private HashMap<String, String> getSearchedStyles() {
		// Regexp regExp = new Regexp("\\[\\s*([^\\])");
		HashMap<String, String> searchedStyles = new HashMap<String, String>();
		Pattern pattern = Pattern.compile("([\\w @]+?=[^\\]]+)");
		Matcher matcher = pattern.matcher(arguments);
		while (matcher.find()) {
			String res = matcher.group();
			String[] splitted = res.split("=");
			searchedStyles.put(splitted[1], splitted[0]);
		}
		return searchedStyles;
	}

}
