package org.topcased.traceability.hmi.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;
import org.topcased.traceability.hmi.TtmImageRegistry;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.editor.TtmMultipleEditor;
import org.topcased.traceability.hmi.runtime.ResourceLoader;

import tte.exceptions.FilesNeedToBeReloadException;
import tte.impl.Engine;
import ttm.Project;

public class AnalyseActionDelegate extends Action implements IEditorActionDelegate, IViewActionDelegate, Runnable
{    
    
    /**
     * Comment for <code>selectedModel</code>
     */
    private Project selectedProject= null;

	private TtmMultipleEditor activeEditor;
    
	private boolean delegateForceReload;
    
		
    public AnalyseActionDelegate()
    {
        this(null);
        delegateForceReload = false;
    }
    
    /**
     * Constructor with the selectedProject used by the action
     * @param selectedProject
     */
    public AnalyseActionDelegate(Project selectedProject)
    {
    	this(selectedProject, false);
    }
    
    /**
     * Constructor with the selectedProject used by the action, 
     * and a boolean indicating whether documents must
     * be reloaded without checking or prompting user for confirmation
     * @param selectedProject
     * @param forceReload should we force reloading of all documents ?
     */
    public AnalyseActionDelegate(Project selectedProject, boolean forceReload)
    {
        super("Analyse project",TtmImageRegistry.getImageDescriptor("ANALYSE_ACTION"));
        setSelectedProject(selectedProject);
        delegateForceReload = forceReload;
    }
    
    
    
    public void setActiveEditor(IAction action, IEditorPart targetEditor)
    {
    	if (targetEditor instanceof TtmMultipleEditor) {
    		activeEditor = (TtmMultipleEditor)targetEditor;
    	}
    	else {
    		activeEditor = null;
    	}
    }
    
    public void resolveActiveEditor() {
    	if (activeEditor == null) {
    		try {
    			IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
    			if (editor instanceof TtmMultipleEditor) {
    				activeEditor = (TtmMultipleEditor)editor;
    			}
    		}
    		catch (Exception e) {
    			//Some resources are not available and can involve NullPointerException here.
    		}
    	}
    }
    
    
    public void run()
    {
        run(this);
    }

    public void run(IAction action)
    {
        Shell activeShell = Display.getDefault().getActiveShell();
    	
        try {
            IRunnableWithProgress op = new AnalyseAction(delegateForceReload);
            new ProgressMonitorDialog(activeShell).run(true, true, op);
         } catch (InvocationTargetException e) {
            // handle exception
         } catch (InterruptedException e) {
            // handle cancelation
         }
         
         resolveActiveEditor();
         
         if (activeEditor != null) {
        	 activeEditor.updateViews();
        	 activeEditor.startChangeChecker();
         }
         
    }

    private Project getProjectFromEditor() {
//    	URI resourceURI = ResourceLoadURIHelper.createURI((FileEditorInput)activeEditor.getEditorInput());
//    	EditingDomain domain = activeEditor.getEditingDomain();
//    	Resource resource = domain.getResourceSet().getResource(resourceURI, true);
//    	Diagrams diagrams = (Diagrams) resource.getContents().get(0);
//    	return (Project) diagrams.getModel();
    	return ((TtmMultipleEditor)activeEditor).getProject();
    }
    
    public void selectionChanged(IAction action, ISelection selection)
    {  
    	if (activeEditor != null) {
    		setSelectedProject(getProjectFromEditor());
    	}
    	else if (selection != null && !selection.isEmpty() && selection instanceof TreeSelection) {
            Object strSelection = ((TreeSelection)selection).getFirstElement();
            if(strSelection != null)
            {
                //Package explorer view
                if(strSelection instanceof IResource)
                {
                    if (((IResource)strSelection).getFileExtension().equalsIgnoreCase(TtmPlugin.MODEL_EXTENSION))
                    {
                        setSelectedProject((Project)ResourceLoader.getInstance().load((IResource)strSelection));
                    }
                }                  
            }

    	}
        action.setEnabled(selectedProject != null);
       
    }
    
    /**
     * 
     * Display a progess bar during the analysis of all the documents of the model
     * <br>
     * creation : 13 mars 07
     *
     * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
     */
    class AnalyseAction implements IRunnableWithProgress
    {
    	private boolean forceReload;

    	public AnalyseAction() {
    		this.forceReload = false;
    	}
    	
		public AnalyseAction(boolean forceReload) {
    		this.forceReload = forceReload;
    	}
    	
    	public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
        {
    		
            if(getSelectedProject() != null)
            {
            	final Engine engine = new Engine();
    			
            	if (forceReload) {
            		analyseProjectForcingReload(engine);
            	} else {
            		analyseProjectReloadingIfNecessary(engine);
            	}
            }
            else {
            	//TODO: ???
            }
        }

		private void analyseProjectForcingReload(final Engine engine) {
			try {
				engine.reloadProjectForcing(selectedProject, true);
				
			} catch (final Exception e) { 	
			
				Display.getDefault().asyncExec(new Runnable(){
					public void run() {
						MessageDialog.openError(TtmPlugin.getActiveWorkbenchShell(), "Resource Load", e.getMessage());                    			
					}
				});
			}
			
		}

		private void analyseProjectReloadingIfNecessary(final Engine engine) {
			
			try
			{
			    engine.reloadProject(selectedProject);
			   
			}
			catch (FilesNeedToBeReloadException e)
			{
				Display.getDefault().asyncExec(new Runnable(){
					public void run() {
			        	
			        	String[] buttons = {"Yes", "No", "Cancel"};
			            MessageDialog askReloadDlg = new MessageDialog(TtmPlugin.getActiveWorkbenchShell(), 
			            		"Reload modified documents", null,
			                    "Some documents in project have been changed since the last analysis.\nDo you want to reload them?", 
			                    MessageDialog.QUESTION, buttons, 1);
			            askReloadDlg.open();
			            int returnCode = askReloadDlg.getReturnCode();
			            if (returnCode == 3)
			                return;
			            try
			            {
			                engine.reloadProjectForcing(selectedProject, returnCode == 0);
			               
			            }
			            catch (final Exception e2)
			            {
			            	MessageDialog.openError(TtmPlugin.getActiveWorkbenchShell(), "Resource Load", e2.getMessage());                    			
			            }
					}
				});
			}
			catch (final Exception e)
			{
				Display.getDefault().asyncExec(new Runnable(){
					public void run() {
						MessageDialog.openError(TtmPlugin.getActiveWorkbenchShell(), "Resource Load", e.getMessage());                    			
					}
				});
			}
		}
    }

    public void init(IViewPart view)
    {
       
    }


    /**
     * Getter for selected Project
     * @return Project selectedProject
     */
    public Project getSelectedProject()
    {
        return selectedProject;
    }

    /**
     * Setter for selectedProject
     * @param selectedModel
     */
    public void setSelectedProject(Project selectedModel)
    {
        this.selectedProject = selectedModel;
    }

}
