package org.topcased.traceability.hmi.contextmenu;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Menu;

/**
 * creation : 13 june 07
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 *
 * A class that handles a context menu on a viewer
 * 
 * @see ContextAction
 */
public class ContextMenuManager {
	
	/** the viewer that displays this menu */
	private Viewer viewer;
	
	/** actions and classes associations */
	private Map<Class, List<ContextAction>> actions;

	/**
	 * Default constructor
	 * @param viewer
	 */
	public ContextMenuManager(Viewer viewer) {
		this.viewer = viewer;
		actions = new HashMap<Class, List<ContextAction>>();
		
		MenuManager menuMgr = new MenuManager();
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener(){
        	public void menuAboutToShow(IMenuManager manager) {
        		Object selected = getSelection();
				for (ContextAction action: getActions(selected)) {
					manager.add(action);
					action.setSelection(selected);
				}
        	}
        });
        Menu menu = menuMgr.createContextMenu(viewer.getControl());
        viewer.getControl().setMenu(menu);
	}
	
	/**
	 * Add an action available for the given class
	 * @param action
	 * @param clazz
	 */
	public void addAction(ContextAction action, Class clazz) {
		addActions(new ContextAction[]{ action }, clazz);
	}
	
	/**
	 * Add an action available for the given classes
	 * @param action
	 * @param classes
	 */
	public void addAction(ContextAction action, Class[] classes) {
		for (Class clazz: classes) {
			addAction(action, clazz);
		}
	}
	
	/**
	 * Add several actions available for a given class
	 * @param actions
	 * @param clazz
	 */
	public void addActions(ContextAction[] actions, Class clazz) {
		List<ContextAction> actionsForClass = this.actions.get(clazz);
		if (actionsForClass == null) {
			actionsForClass = new ArrayList<ContextAction>();
			this.actions.put(clazz, actionsForClass);
		}
		actionsForClass.addAll(Arrays.asList(actions));
	}
	
	/**
	 * Return all actions available for a given object 
	 * @param object
	 * @return
	 */
	private List<ContextAction> getActions(Object object) {
		List<ContextAction> availableActions = new ArrayList<ContextAction>();
		
		for (Class c: actions.keySet()) {
			if (c.isInstance(object)) {
				availableActions.addAll(actions.get(c));
			}
		}
		return availableActions;
	}
	
	/**
	 * Return the current selection in the viewer
	 * @return
	 */
	private Object getSelection() {
		if (viewer.getSelection() instanceof StructuredSelection) {
			return ((StructuredSelection)viewer.getSelection()).getFirstElement();
		}
		return null;
	}
}
