package org.topcased.traceability.hmi.editor;

import java.io.File;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.part.FileEditorInput;
import org.topcased.traceability.hmi.TtmPlugin;
import org.topcased.traceability.hmi.actions.AnalyseActionDelegate;
import org.topcased.traceability.hmi.util.FileNameUtils;

import ttm.Project;

/**
 * Task that checks regularly for the changes on documents of a project, to
 * re-launch the analysis when needed.
 * 
 * @author phtrivier
 * 
 */
public class DocumentChangeChecker
{

    class CustomAnalysis implements Runnable
    {

        private Project project;

        public CustomAnalysis(Project p)
        {
            this.project = p;
        }

        public void run()
        {
            
            boolean dirty = ttmMultipleEditor.hasDiagramChanged();
            boolean reanalyse = false;

            Shell shell = Display.getDefault().getActiveShell();

            if (dirty)
            {

                if (MessageDialog.openConfirm(shell, "Save current model",
                        "Some files have changed on disk since last analysis. "
                                + "Do you want to save your diagram, reload the files, and relaunch the analysis ?"))
                {
                    // Save the diagram
                    ttmMultipleEditor.doSave(new ProgressMonitorDialog(shell).getProgressMonitor());

                    // Prepare the analysis
                    reanalyse = true;
                }
            }
            else
            {

                if (MessageDialog.openConfirm(shell, "Relaunch analysis",
                        "Some files have changed on disk since last analysis. "
                                + "Do you want to reload them and relaunch the analysis ?"))
                {

                    reanalyse = true;
                }
            }
            if (reanalyse)
            {
                new AnalyseActionDelegate(project, true).run();
            }
            else
            {
                stop();
            }

        }

    }

    private TtmMultipleEditor ttmMultipleEditor;       

    public DocumentChangeChecker(TtmMultipleEditor ttmMultipleEditor)
    {
        this.ttmMultipleEditor = ttmMultipleEditor;

    }

    public void stop()
    {
        if (t != null)
        {
            t.cancel();
            t = null;
        }

    }

    Timer t;

    public void start()
    {        
        if (t == null && isJobActivated())
        {     
            t = new Timer();

            int interval = loadInterval();

            t.scheduleAtFixedRate(new TimerTask()
            {
                @Override
                public void run()
                {

                    Project project = null;

                    try
                    {
                        project = ttmMultipleEditor.getProject();

                        // If the documents for this project have changed since
                        // last time
                        if (project != null && hasProjectDocumentChanged(project))
                        {
                            Display.getDefault().asyncExec(new CustomAnalysis(project));
                        }

                    }
                    catch (NullPointerException e)
                    {
                        project = null;
                    }

                }

                private boolean hasProjectDocumentChanged(Project project)
                {
                    long lastAnalysisDate = getLastAnalysisDate();

                    for (ttm.Document doc : project.getDocuments())
                    {
                        File docFile = new File(doc.getFilename());                        
                        if (lastAnalysisDate < docFile.lastModified())
                        {                            
                            return true;
                        }                        
                    }
                    return false;
                }

            }, 0, interval * 1000);
            // TODO : Make the time between two checks a property of the plugin

        }
    }

    /**
     * Interval between two checks, in seconds. Interval is read from preferences.
     * @return
     */
    private int loadInterval()
    {
        return TtmPlugin.getDefault().getPluginPreferences().getInt(TtmPlugin.DOCUMENT_CHECK_INTERVAL_PREFERENCE);
    }
    
    /**
     * 
     * @return true if the job is active
     */
    private boolean isJobActivated()
    {
        return TtmPlugin.getDefault().getPluginPreferences().getBoolean(TtmPlugin.DOCUMENT_CHECK_ACTIVATION_PREFERENCE);
    }

    /**
     * Date at which this editor's project has been analysis.
     * @return
     */
    protected long getLastAnalysisDate()
    {

        long res = 0;
        IEditorInput editorInput = ttmMultipleEditor.getEditorInput();
        if (editorInput instanceof FileEditorInput)
        {

            FileEditorInput fileInput = (FileEditorInput) editorInput;

            IPath path = fileInput.getPath();
            IPath resultPath = FileNameUtils.getResultFileFullPath(path);
            File resultIoFile = resultPath.toFile();

            res = resultIoFile.lastModified();

        }

        return res;
    }

}
