package org.topcased.traceability.hmi.modeler.commands;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.commands.Command;
import org.topcased.modeler.di.model.DiagramInterchangeFactory;
import org.topcased.modeler.di.model.GraphConnector;
import org.topcased.modeler.di.model.GraphEdge;
import org.topcased.modeler.di.model.GraphElement;

/**
 * The command to delete an Inheritance Link
 * 
 * Creation : 19 may 2006
 * 
 * @author <a href="mailto:jacques.lescot@anyware-tech.com">Jacques LESCOT</a>
 */
public class DeleteCoverLinkCommand extends Command
{

    private static final int SOURCE_ANCHOR = 0;

    private static final int TARGET_ANCHOR = 1;

    private GraphEdge deletedEdge;

    private GraphElement source;

    private GraphElement target;

    private GraphElement parent;

    /**
     * The Constructor
     */
    public DeleteCoverLinkCommand()
    {
        super("Delete Inheritance Link");
    }

    /**
     * Check if the command can be executed
     * 
     * @see org.eclipse.gef.commands.Command#canExecute()
     */
    public boolean canExecute()
    {
        return true;
    }

    /**
     * @see org.eclipse.gef.commands.Command#redo()
     */
    public void redo()
    {
    
    	if (parent != null)
        {
            EList anchors = deletedEdge.getAnchor();
            GraphConnector sourceConnector = (GraphConnector) anchors.get(0);
            GraphConnector targetConnector = (GraphConnector) anchors.get(1);

            // remove the connection from the parent element
            parent.getContained().remove(deletedEdge);

            // remove the connection from its different Connectors
            deletedEdge.getAnchor().remove(targetConnector);
            deletedEdge.getAnchor().remove(sourceConnector);
       } 
    }

     /**
     * Removes the connection
     * 
     * @see org.eclipse.gef.commands.Command#execute()
     */
    public void execute()
    {
        parent = deletedEdge.getContainer();
        redo();
    }

    /**
     * Restores the connection
     * 
     * @see org.eclipse.gef.commands.Command#undo()
     */
    public void undo()
    {
    if (source == target)
        {
            // Creates anchor if needed
            if (source.getAnchorage().isEmpty())
            {
                GraphConnector connector1 = DiagramInterchangeFactory.eINSTANCE.createGraphConnector();
                source.getAnchorage().add(connector1);
            }
            if (source.getAnchorage().size() < 2)
            {
                GraphConnector connector2 = DiagramInterchangeFactory.eINSTANCE.createGraphConnector();
                source.getAnchorage().add(connector2);
            }
            GraphConnector srcConnector = (GraphConnector) source.getAnchorage().get(0);
            GraphConnector tgtConnector = (GraphConnector) source.getAnchorage().get(1);

            // add the edge to the graphical view
            srcConnector.getGraphEdge().add(deletedEdge);
            tgtConnector.getGraphEdge().add(deletedEdge);
        }
        else
        {
            // Creates anchor if needed
            if (source.getAnchorage().isEmpty())
            {
                GraphConnector connector = DiagramInterchangeFactory.eINSTANCE.createGraphConnector();
                source.getAnchorage().add(connector);
            }
            GraphConnector srcConnector = (GraphConnector) source.getAnchorage().get(0);

            if (target.getAnchorage().isEmpty())
            {
                GraphConnector connector = DiagramInterchangeFactory.eINSTANCE.createGraphConnector();
                target.getAnchorage().add(connector);
            }
            GraphConnector tgtConnector = (GraphConnector) target.getAnchorage().get(0);

            // add the edge to the graphical view
            srcConnector.getGraphEdge().add(deletedEdge);
            tgtConnector.getGraphEdge().add(deletedEdge);
        }
	
        if (parent != null)
        {
            parent.getContained().add(deletedEdge);
        }

    }

    /**
     * Set the edit part that must be deleted
     * 
     * @param model the graph object to be deleted
     * @return the delete command
     */
    public Command setPartToBeDeleted(Object model)
    {
        if (model instanceof GraphEdge)
        {
            deletedEdge = (GraphEdge) model;
            setSource(deletedEdge);
            setTarget(deletedEdge);
            return this;
        }

        return null;
    }

    /**
     * Retrieve the source Node from the given edge
     * 
     * @param edge the GraphEdge concerned by the Command
     */
    private void setSource(GraphEdge edge)
    {
        this.source = (GraphElement) ((GraphConnector) edge.getAnchor().get(SOURCE_ANCHOR)).eContainer();
        // this.source = (GraphNode) edge.getAnchor().get(SOURCE_ANCHOR).eContainer();
    }

    /**
     * Get the sourceNode of the link
     * 
     * @return the sourceNode
     */
    public GraphElement getSource()
    {
        return source;
    }

    /**
     * Retrieve the target Node from the given edge
     * 
     * @param edge the GraphEdge concerned by the Command
     */
    private void setTarget(GraphEdge edge)
    {
		if (edge.getAnchor().size() == 1)
        {
            // the target Node is the same as source Node
            this.target = (GraphElement) ((GraphConnector) edge.getAnchor().get(SOURCE_ANCHOR)).eContainer();
        }
        else
        {
            this.target = (GraphElement) ((GraphConnector) edge.getAnchor().get(TARGET_ANCHOR)).eContainer();
        }
    }
}
