/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.ui;

import java.util.List;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import ttm.Attribute;
import ttm.IdentifiedElement;

/**
 * Created on 21 fvr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * A composite that handles the display of attributes and text values
 * 
 */
public class PropertiesComposite extends Composite
{

    private Text text;

    private Table table;

    private TableViewer viewer;

    /**
     * Default constructor
     * 
     * @param parent
     * @param style
     */
    public PropertiesComposite(Composite parent, int style)
    {
        super(parent, style);

        GridLayout layout = new GridLayout(1, true);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        setLayout(layout);

        createContents(this);

    }

    /**
     * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
     */
    public void setBackground(Color color)
    {
        super.setBackground(color);
        for (Control control : getChildren())
        {
            control.setBackground(color);
        }
    }

    protected void createContents(Composite parent)
    {
        table = createTable(parent);
        viewer = new TableViewer(table);
        viewer.setUseHashlookup(true);
        viewer.setContentProvider(new PropertiesContentProvider());
        viewer.setLabelProvider(new PropertiesLabelProvider());

        text = new Text(parent, SWT.BORDER | SWT.MULTI | SWT.WRAP);
        text.setEditable(false);
        GridData textLayoutData = new GridData(GridData.FILL_BOTH);
        textLayoutData.horizontalSpan = 2;
        textLayoutData.widthHint = 2;
        text.setLayoutData(textLayoutData);
    }

    private Table createTable(Composite composite)
    {
        int style = SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.HIDE_SELECTION;

        final Table table = new Table(composite, style);

        TableColumn column1 = new TableColumn(table, SWT.LEFT);
        column1.setText("Attribute");
        column1.setWidth(100);

        TableColumn column2 = new TableColumn(table, SWT.LEFT);
        column2.setText("Value");
        column2.setWidth(200);

        table.setLinesVisible(true);

        GridData gridData = new GridData(GridData.FILL_BOTH);
        table.setLayoutData(gridData);
        return table;
    }

    /**
     * Update the content of the composite according the given model object
     * 
     * @param object
     */
    public void updateContent(Object object)
    {

        text.setText("");

        if (object != null)
        {
            String textValue = "";

            if (object instanceof IdentifiedElement)
            {
                IdentifiedElement element = (IdentifiedElement) object;

                List<ttm.Text> textes = element.getTexts();
                StringBuffer buffer = new StringBuffer();
                for (ttm.Text text : textes)
                {
                    buffer.append(text.getValue());
                }
                textValue = buffer.toString();

                viewer.setInput(object);
            }

            text.setText(textValue);
        }
        else
        {
            viewer.setInput(null);
        }
    }

    /**
     * @see org.eclipse.swt.widgets.Widget#dispose()
     */
    public void dispose()
    {
        text.dispose();
        super.dispose();
    }

    /**
     * Handle the content of the table
     */
    class PropertiesContentProvider implements IStructuredContentProvider
    {

        /**
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements(Object inputElement)
        {
            if (inputElement instanceof IdentifiedElement)
            {
                IdentifiedElement ie = (IdentifiedElement) inputElement;
                List<Attribute> attributes = ie.getAttributes();

                String[][] result = new String[attributes.size() + 2][2];
                result[0][0] = "Identification";
                result[0][1] = ie.getIdent();
                result[1][0] = "Short description";
                result[1][1] = ie.getShortDescription();
                int i = 2;
                for (Attribute attr : attributes)
                {
                    result[i][0] = attr.getName();
                    result[i][1] = attr.getValue();
                    i++;
                }
                return result;
            }
            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose()
        {
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
         *      java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
        {
        }
    }

    /**
     * Handle label provider for the table
     */
    class PropertiesLabelProvider extends LabelProvider implements ITableLabelProvider
    {

        /**
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
         */
        public Image getColumnImage(Object element, int columnIndex)
        {
            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
         */
        public String getColumnText(Object element, int columnIndex)
        {
            return ((String[]) element)[columnIndex];
        }

    }

}
