/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;

import ttm.CoverLink;
import ttm.CoverPoint;
import ttm.Document;
import ttm.Element;
import ttm.HierarchicalElement;
import ttm.Project;
import ttm.Requirement;
import ttm.RuleInstance;
import ttm.Section;
import ttm.Text;

/**
 * 
 * Created on 13 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * An helper arround model objects
 * 
 */
public class ModelHelper
{

    /**
     * Build a list that contains all cover links under a given element
     * 
     * @param element
     * @return the list of cover link
     */
    public static List<CoverLink> getAllCoverLinksUnder(HierarchicalElement element)
    {
        List<CoverLink> links = new ArrayList<CoverLink>();
        links.addAll(element.getCoverLinks());
        List<HierarchicalElement> children = element.getChildren();
        for (HierarchicalElement ce : children)
        {
            links.addAll(getAllCoverLinksUnder(ce));
        }
        return links;
    }
    
    public static List<CoverLink> getAllIncomingCoverLinksUnder(Requirement element)
    {
        List<CoverLink> links = new ArrayList<CoverLink>();
        links.addAll(element.getIncomingCoverLinks());
        List<HierarchicalElement> children = element.getChildren();
        for (HierarchicalElement ce : children)
        {
        	if (ce instanceof Requirement) {
        		links.addAll(getAllIncomingCoverLinksUnder((Requirement)ce));
        	}
        }
        return links;
    }
    
    
    public static List<Requirement> getRecursiveCoveringRequirements(Requirement requirement) {
    	List<Requirement> requirements = new ArrayList<Requirement>();
    	for (CoverLink link: getAllCoverLinksUnder(requirement)) {
    		if (link.getTo() != null)
            {
                HierarchicalElement ce = link.getTo();
                if (ce instanceof Requirement) {
                	requirements.add((Requirement)ce);
                }
            }
    	}
    	return requirements;
    }
    
    public static List<HierarchicalElement> getRecursiveCoveredRequirements(Requirement requirement) {
    	List<HierarchicalElement> children = new ArrayList<HierarchicalElement>();
    	for (CoverLink link: getAllIncomingCoverLinksUnder(requirement)) {
    		if (link.getFrom() != null)
            {
                HierarchicalElement ce = link.getFrom();
               	children.add(ce);
            }
    	}
    	
    	//In this loop, we remove sections if a requirement of its children is 
    	//also in the previous calculated list.
    	List<HierarchicalElement> sectionsToRemove = new ArrayList<HierarchicalElement>();
    	for (HierarchicalElement he: children) {
    		if (he instanceof Section) {
    			for (Requirement req: getRequirements((Section)he)) {
    				if (children.contains(req)) {
    					sectionsToRemove.add(he);
    					break;
    				}
    			}
    		}
    	}
    	
    	children.removeAll(sectionsToRemove);
    	return children;
    }
    
    public static List<Requirement> getRequirements(Section section) {
    	List<Requirement> requirements = new ArrayList<Requirement>();
    	for (Object child: section.getChildren()) {
        	if (child instanceof Section) {
        		Section subSection = (Section)child;
        		requirements.addAll(getRequirements(subSection));
        	}
        	else {
        		Requirement requirement = (Requirement)child;
        		requirements.add(requirement);
        	}
        }
    	return requirements;
    }
    
    public static void updateTreeWithIncomingCoverLinks(Tree tree, Requirement requirement) {
        List<CoverLink> links = requirement.getIncomingCoverLinks();
        for (CoverLink link : links)
        {
            List<Object> parents = new ArrayList<Object>();
            if (link.getFrom() != null)
            {
                HierarchicalElement ce = link.getFrom();
                while (ce != null)
                {
                    parents.add(0, ce);
                    ce = ce.getParent();
                }
                tree.addBranch(parents);
            }
        }
    }

	public static Float getCoverageResult(CoverPoint coverPoint) {
		
		Document coveredDocument = coverPoint.getCoveredDocument();
		Map<Object, Float> coverageResults = coveredDocument.getCoverageResults();
		
		Float globalCoverage = coverageResults.get(coverPoint);
		
		return globalCoverage;
	}

	public static int getNbRequirements(Document coveredDocument) {
		int res = 0;
		Document document = coveredDocument.getDocument();
		EList children = document.getChildren();
		for (Object object : children) {
			Section section = (Section) object;
			res += getRequirements(section).size();
		}
		return res;
	}

	public static float getCoverageResult(Document covered,
			Document covering) {
		Map<Object, Float> coverageResults = covered.getCoverageResults();
		Float ratio = coverageResults.get(covering);
		if (ratio == null) {
			ratio = new Float(0);
		}
		return ratio.floatValue();
	}
	
	public static float getCoverageResult(Document covered,
			CoveringDocument covering) {
		Map<Object, Float> coverageResults = covered.getCoverageResults();
		Float ratio = coverageResults.get(covering.getWrappedDocument());
		if (ratio == null) {
			ratio = new Float(0);
		}
		return ratio.floatValue();
	}

	/**
	 * Extract the requirement associated to a rule, 
	 * browsing up the rule structure if required.
	 * @param rule
	 * @return
	 */
	public static Element getElementFromRule(RuleInstance rule) {
		
		Element result = null;
		
		List<Object> parentsBrowsed = new ArrayList<Object>();
		
		Element current = rule.getElement();
		boolean found = false;
		while (!found && current != null && !parentsBrowsed.contains(current)) {
			
			if (current instanceof Requirement || current instanceof Section) {
				result = current;
				found = true;
			} else if (current instanceof Element) {
				parentsBrowsed.add(result);
				current = (Element) current.eContainer();
			} else {
				current = null;
			}
			
		}
		return result;

	}

	public static List<Requirement> getRequirements(Project project) {
		List<Requirement> res = new ArrayList<Requirement>();
		
		List<Document> documents = project.getDocuments();
		for (Document document : documents) {
		
			res.addAll(getRequirements(document));
			
		}
		
		
		return res;
		
		
	}

	private static List<Requirement> getRequirements(Document document) {
		List<Requirement> res = new ArrayList<Requirement>();
		EList children = document.getChildren();
		for (Object child : children) {
			
			Section section = (Section) child;
			
			res.addAll(getRequirements(section));
			
		}
		return res;
		
		
	}

	public static String getRequirementText(Requirement r) {
		StringBuffer buf = new StringBuffer();
		EList texts = r.getTexts();
		for (Object object : texts) {
			Text text = (Text) object;
			buf.append(text.getValue());
		}
		return buf.toString();
	}
	
	
	
}
