/*******************************************************************************
 * Copyright (c) 2007 Anyware Technologies. All rights reserved. This program
 * and the accompanying materials are made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Mathieu Garcia (Anyware Technologies) - initial API and
 * implementation
 ******************************************************************************/
package org.topcased.traceability.hmi.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.topcased.traceability.hmi.TtmPlugin;

import ttm.CoverPoint;

/**
 * Created on 21 f�vr. 07
 * 
 * @author <a href="mailto:mathieu.garcia@anyware-tech.com">Mathieu Garcia</a>
 * 
 * A data structure that maps a tree
 * 
 */
public class Tree
{

    /**
     * An internal class to map a node of the tree
     */
    private class TreeNode
    {

        private Object object;

        private List<TreeNode> children;

        private TreeNode parent;

        private TreeNode(Object object, TreeNode parent)
        {
            this.parent = parent;
            this.object = object;
            children = new ArrayList<TreeNode>();
        }

        private Object getObject()
        {
            return object;
        }

        private void addChild(TreeNode node)
        {
            children.add(node);
        }

        private List<TreeNode> getChildren()
        {
            return children;
        }

        private TreeNode getParent()
        {
            return parent;
        }
        
        private void removeChild(TreeNode node) {
        	children.remove(node);
        }
        
        private void removeChildren() {
        	children.clear();
        }
    }

    private Map<Object, TreeNode> indexes;

    private List<Object> roots;
    
    private List<Object> duplicatedEntries;

    /**
     * Default constructor
     */
    public Tree()
    {
        indexes = new HashMap<Object, TreeNode>();
        roots = new ArrayList<Object>();
        duplicatedEntries = new ArrayList<Object>();
    }

    /**
     * Add to the tree some roots
     * 
     * @param roots
     */
    public void addRoots(List<Object> roots)
    {
        for (Object object : roots)
        {
            TreeNode node = indexes.get(object);
            if (node == null)
            {
                node = new TreeNode(object, null);
                indexes.put(object, node);
                this.roots.add(object);
            }
        }
    }
    
    /**
     * Add to the tree a branch. Missing nodes are created
     * 
     * @param branch
     */
    public void addBranch(List<Object> branch)
    {
        if (!branch.isEmpty())
        {
            TreeNode parent = null;
            for (Iterator<Object> iter = branch.iterator(); iter.hasNext();)
            {
                Object element = iter.next();
                TreeNode node = indexes.get(element);
                if (node == null)
                {
                    node = new TreeNode(element, parent);
                    indexes.put(element, node);
                    if (parent == null)
                    {
                        roots.add(element);
                    }
                    else
                    {
                        parent.addChild(node);
                    }
                }
                parent = node;
            }
        }
    }

    /**
     * Add to the tree under the given parent object, a given child
     * 
     * @param parent
     * @param child
     */
    public void addNode(Object parent, Object child)
    {
        TreeNode parentNode = indexes.get(parent);
        if (parentNode == null)
        {
            throw new IllegalArgumentException("Parent object does not belong to this tree");
        }
        TreeNode childNode = indexes.get(child);
        if (childNode != null)
        {
        	duplicatedEntries.add(child);
        	TtmPlugin.log(child + " has been already added to this tree. A new insertion may turn it inconsistent.", IStatus.WARNING);
            //throw new IllegalArgumentException("Child object already exits");
        }
        childNode = new TreeNode(child, parentNode);
        parentNode.addChild(childNode);
        indexes.put(child, childNode);
    }

    public void removeNode(Object parent, Object child) {
    	TreeNode parentNode = indexes.get(parent);
        if (parentNode == null)
        {
            throw new IllegalArgumentException("Parent object does not belong to this tree");
        }
        TreeNode childNode = indexes.get(child);
        if (childNode != null)
        {
        	indexes.remove(child);
            parentNode.removeChild(childNode); 
        }
    }
    
    public void removeNodes(Object parent) {
    	TreeNode parentNode = indexes.get(parent);
        if (parentNode == null)
        {
            throw new IllegalArgumentException("Parent object does not belong to this tree");
        }
        for (TreeNode childNode: parentNode.getChildren()) {
        	Object child = childNode.getObject();
        	indexes.remove(child);
        }
        parentNode.removeChildren();
    }
    
    /**
     * Return true if the given object belongs to this tree
     * 
     * @param object
     * @return true if model object is present.
     */
    public boolean contains(Object object)
    {
        return indexes.containsKey(object);
    }

    /**
     * Return the model object parent of the given object
     * 
     * @param object
     * @return a model object
     */
    public Object getParent(Object object)
    {
        TreeNode node = indexes.get(object);
        if (node == null)
        {
            throw new IllegalArgumentException("Object does not belong to this tree");
        }
        TreeNode parentNode = node.getParent();
        if (parentNode == null)
        {
            return null; // Given object is root
        }
        return parentNode.getObject();
    }

    /**
     * Get the roots of the tree
     * 
     * @return an array of the top elements
     */
    public Object[] getRoots()
    {
        return roots.toArray(new Object[roots.size()]);
    }

    /**
     * Get the children of the given object
     * 
     * @param parent
     * @return an array
     */
    public Object[] getChildren(Object parent)
    {
    	if (duplicatedEntries.contains(parent)) {
    		TtmPlugin.log(parent + " is known as duplicated entries in this tree. Result of this method may be inconsistent.", IStatus.WARNING);
    	}
    	List<Object> children = listChildren(parent);
        return children.toArray(new Object[children.size()]);
    }

    /**
     * Get the children of the given object
     * 
     * @param parent
     * @return a list
     */
    public List<Object> listChildren(Object parent)
    {
        TreeNode node = indexes.get(parent);
        if (node == null)
        {
            throw new IllegalArgumentException("Parent object does not belong to this tree");
        }

        List<TreeNode> childNodes = node.getChildren();
        List<Object> list = new ArrayList<Object>();
        for (TreeNode tn : childNodes)
        {
            list.add(tn.getObject());
        }
        return list;
    }

    private String contentToString(Object parent, int level)
    {
        StringBuffer buffer = new StringBuffer();
        String levelSpace = "";
        for (int i = 0; i < level; i++)
        {
            levelSpace += " ";
        }
        for (Object object : getChildren(parent))
        {
            buffer.append(levelSpace);
            buffer.append(object);
            buffer.append("\n");
            buffer.append(contentToString(object, level + 5));
            buffer.append("\n");
        }
        return buffer.toString();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        for (Object object : roots)
        {
            buffer.append(object);
            buffer.append("\n");
            buffer.append(contentToString(object, 5));
            buffer.append("\n");
        }
        return buffer.toString();
    }

	/**
	 * Insert a cover point node as a root node, parent
	 * of its document node.
 	 * The old root nodes corresponding to documents are
     * made 'non root' nodes.
 	 */
	public void putRootAsParent(CoverPoint newRoot, List oldRoots) {
	
		TreeNode newRootNode = null;
		
		for (Object oldRoot : oldRoots) {
			
			TreeNode oldRootNode = indexes.get(oldRoot);
			if (oldRootNode != null ) {
				
				// Insert the root node if not already done.
				// We make sure that the root is only added if it
				// has some child in the tree.
				if (newRootNode == null) {
					List<Object> addedRoots = new ArrayList<Object>();
					addedRoots.add(newRoot);
					addRoots(addedRoots);
					newRootNode = indexes.get(newRoot);
				}		
				
				newRootNode.addChild(oldRootNode);
			 
				roots.remove(oldRoot);
			}
			
		}
		
	}

}
