/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.validation.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EValidator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.topcased.validation.core.internal.ValidationPlugin;

/**
 * This class find the validation markers and return a multiple status containing all the messages
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class MarkerUtil
{
    /** Marker ID */
    public static final String MARKER_ID = EValidator.MARKER;

    /**
     * Get an IStatus from the markers associated with the given EObject
     * 
     * @param object the EObject
     * @return a MultiStatus composed by the list of status from markers
     */
    public static IStatus getStatus(EObject object)
    {
        MultiStatus status = new MultiStatus(ValidationPlugin.getId(), IStatus.OK, "Validation Problems", null);

        IMarker[] markers = getMarkers(object);
        for (int i = 0; i < markers.length; i++)
        {
            IMarker marker = markers[i];
            IStatus markerStatus = getStatusForMarker(marker);
            if (markerStatus != null)
            {
                status.add(markerStatus);
            }
        }

        return status;
    }

    /**
     * Returns the markers associated with the given EObject
     * 
     * @param object the EObject
     * @return the list of markers
     */
    private static IMarker[] getMarkers(EObject object)
    {
        List markerList = new ArrayList();

        String objectURI = EcoreUtil.getURI(object).toString();
        ResourceSet set = object.eResource().getResourceSet();

        if (set != null)
        {
            List copiedResources = new ArrayList(set.getResources());
            Iterator it = copiedResources.iterator();
            while (it.hasNext())
            {
                Resource res = (Resource) it.next();
                IFile file = getFile(res);
                if (file != null)
                {
                    try
                    {
                        IMarker[] markers = file.findMarkers(MARKER_ID, true, IResource.DEPTH_ZERO);
                        for (int i = 0; i < markers.length; i++)
                        {
                            IMarker marker = markers[i];

                            String uriAttribute = (String) marker.getAttribute(EValidator.URI_ATTRIBUTE);
                            // This is a marker for this object
                            if (uriAttribute != null && uriAttribute.equals(objectURI))
                            {
                                markerList.add(marker);
                            }
                        }
                    }
                    catch (CoreException ce)
                    {
                        // Ignore bad files
                    }
                }
            }
        }

        return (IMarker[]) markerList.toArray(new IMarker[markerList.size()]);
    }

    /**
     * Transform a marker into an IStatus
     * 
     * @param marker The EMF marker
     * @return the IStatus or <code>null</code> if no marker is found
     */
    private static IStatus getStatusForMarker(IMarker marker)
    {
        IStatus result = null;
        try
        {
            Object severityAttribute = marker.getAttribute(IMarker.SEVERITY);
            String message = (String) marker.getAttribute(IMarker.MESSAGE);
            if (severityAttribute != null && severityAttribute instanceof Integer)
            {
                if (((Integer) severityAttribute).intValue() == IMarker.SEVERITY_ERROR)
                {
                    result = new Status(IStatus.ERROR, ValidationPlugin.getId(), IStatus.OK, message, null);
                }
                else if (((Integer) severityAttribute).intValue() == IMarker.SEVERITY_WARNING)
                {
                    result = new Status(IStatus.WARNING, ValidationPlugin.getId(), IStatus.OK, message, null);
                }
            }
        }
        catch (CoreException ce)
        {
            ValidationPlugin.log(ce);
            // Don't display a message box to increase usability
        }

        return result;
    }

    /**
     * Return the IFile associated with the given resource
     * 
     * @param resource the EMF resource
     * @return the containing IFile or <code>null</code> if the resource is not an IFile
     */
    private static IFile getFile(Resource resource)
    {
        URI uri = resource.getURI();
        uri = resource.getResourceSet().getURIConverter().normalize(uri);
        String scheme = uri.scheme();
        if ("platform".equals(scheme) && uri.segmentCount() > 1 && "resource".equals(uri.segment(0)))
        {
            StringBuffer platformResourcePath = new StringBuffer();
            for (int j = 1; j < uri.segmentCount(); ++j)
            {
                platformResourcePath.append('/');
                platformResourcePath.append(uri.segment(j));
            }
            return ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(platformResourcePath.toString()));
        }
        return null;
    }

    /**
     * This is a temporary method copied from the EMF EclipseResourceUtil to keep compatibility from 3.2M6 to 3.2RC4.<br>
     * It should be removed in the next versions
     * 
     * @param file
     * @param diagnostic
     * @deprecated
     */
    public static void createMarkers(IFile file, Diagnostic diagnostic)
    {
        EObject eObject = null;
        List data = diagnostic.getData();
        if (!data.isEmpty())
        {
            Object target = data.get(0);
            if (target instanceof EObject)
            {
                eObject = (EObject) target;
            }
        }

        if (diagnostic.getChildren().isEmpty())
        {
            try
            {
                IMarker marker = file.createMarker(EValidator.MARKER);
                int severity = diagnostic.getSeverity();
                if (severity < Diagnostic.WARNING)
                {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO);
                }
                else if (severity < Diagnostic.ERROR)
                {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
                }
                else
                {
                    marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
                }
                marker.setAttribute(IMarker.MESSAGE, diagnostic.getMessage());
                if (eObject != null)
                {
                    marker.setAttribute(EValidator.URI_ATTRIBUTE, EcoreUtil.getURI(eObject).toString());
                }
            }
            catch (CoreException exception)
            {
                ValidationPlugin.log(exception);
            }
        }
        else
        {
            String parentMessage = diagnostic.getMessage() + ". ";
            for (Iterator i = diagnostic.getChildren().iterator(); i.hasNext();)
            {
                Diagnostic childDiagnostic = (Diagnostic) i.next();
                try
                {
                    IMarker marker = file.createMarker(EValidator.MARKER);
                    int severity = childDiagnostic.getSeverity();
                    if (severity < Diagnostic.WARNING)
                    {
                        marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_INFO);
                    }
                    else if (severity < Diagnostic.ERROR)
                    {
                        marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
                    }
                    else
                    {
                        marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
                    }
                    marker.setAttribute(IMarker.MESSAGE, parentMessage + childDiagnostic.getMessage());
                    if (eObject != null)
                    {
                        marker.setAttribute(EValidator.URI_ATTRIBUTE, EcoreUtil.getURI(eObject).toString());
                    }
                }
                catch (CoreException exception)
                {
                    ValidationPlugin.log(exception);
                }
            }
        }
    }
}
