/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.validation.core;

import java.util.Iterator;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EObjectValidator;
import org.topcased.validation.core.extension.ValidatorDescriptor;
import org.topcased.validation.core.extension.ValidatorsManager;
import org.topcased.validation.core.internal.ValidationPlugin;

/**
 * Class that validates an EMF model using registered validators from the <i>org.topcased.validation.core.validators</i>
 * extension point. <br>
 * creation : 13 janv. 2006
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class ModelValidator
{

    /**
     * Validate an EObject with several kind of validation (EMF, OCL...)
     * 
     * @param model the EObject to validate
     * @param progressMonitor the progress bar
     * @return the list of diagnostics
     */
    public Diagnostic validate(EObject model, final IProgressMonitor progressMonitor)
    {

        BasicDiagnostic result = new BasicDiagnostic(EObjectValidator.DIAGNOSTIC_SOURCE, 0, "Root diagnostic", new Object[] {model});

        ValidatorDescriptor[] descriptors = ValidatorsManager.getInstance().getValidators();

        progressMonitor.beginTask("Validation", descriptors.length * 10);

        for (int i = 0; i < descriptors.length; i++)
        {
            String uri = model.eClass().getEPackage().getNsURI();
            String validatorURI = descriptors[i].getURI();
            if (validatorURI == null || validatorURI.equals(uri))
            {
                try
                {
                    IValidator validator = descriptors[i].getValidator();
                    validator.validate(model, result, new SubProgressMonitor(progressMonitor, 50));
                }
                catch (CoreException ce)
                {
                    ValidationPlugin.log(ce);
                }
            }
        }

        progressMonitor.done();
        return getUniqueDiagnostics(result);
    }

    /**
     * Iterates on all the Diagnostics contained by the BasicDiagnostic and removes those which appears more than once.
     * 
     * @param basicDiagnostic The initial list of Diagnostics
     * @return a Diagnostic with only unique Diagnostics
     */
    private Diagnostic getUniqueDiagnostics(BasicDiagnostic basicDiagnostic)
    {
        BasicDiagnostic result = new BasicDiagnostic(EObjectValidator.DIAGNOSTIC_SOURCE, 0, "Root diagnostic", basicDiagnostic.getData().toArray());

        // Iterate on all the diagnostics found
        Iterator itDiag = basicDiagnostic.getChildren().iterator();
        while (itDiag.hasNext())
        {
            Diagnostic currentDiag = (Diagnostic) itDiag.next();
            boolean alreadyExist = false;

            // check if the Diagnostic has not been already added
            for (Iterator itUniqueDiag = result.getChildren().iterator(); itUniqueDiag.hasNext() && !alreadyExist;)
            {
                Diagnostic uniqueDiag = (Diagnostic) itUniqueDiag.next();

                // check the equality of the two Diagnostics
                if (currentDiag.getSeverity() == uniqueDiag.getSeverity() && currentDiag.getCode() == uniqueDiag.getCode() && currentDiag.getSource().equals(uniqueDiag.getSource())
                        && currentDiag.getMessage().equals(uniqueDiag.getMessage()))
                {
                    alreadyExist = true;
                }
            }

            if (!alreadyExist)
            {
                result.add(currentDiag);
            }
        }

        return result;
    }
}
