/*******************************************************************************
 * Copyright (c) 2005 Anyware Technologies.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies),
 *    Jacques Lescot (Anyware Technologies) - initial API and implementation
 *******************************************************************************/

package org.topcased.validation.emft.internal;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.validation.model.EvaluationMode;
import org.eclipse.emf.validation.model.IConstraintStatus;
import org.eclipse.emf.validation.service.IBatchValidator;
import org.eclipse.emf.validation.service.ModelValidationService;
import org.topcased.validation.core.IValidator;

/**
 * This validator adds the extensible EMFT validation process to the Topcased validation framework
 * 
 * @author <a href="david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class BatchValidator implements IValidator
{

    /** The id of the validation errors */
    public static final String DIAGNOSTIC_SOURCE = "org.topcased.validation.emft.BatchValidator";

    /**
     * @see org.topcased.validation.core.IValidator#validate(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.common.util.DiagnosticChain, org.eclipse.core.runtime.IProgressMonitor)
     */
    public boolean validate(EObject eObject, DiagnosticChain diagnostics, IProgressMonitor monitor)
    {
        IBatchValidator validator = (IBatchValidator) ModelValidationService.getInstance().newValidator(EvaluationMode.BATCH);
        validator.setIncludeLiveConstraints(true);

        final IStatus status = validator.validate(eObject);

        fillDiagnostic(status, diagnostics);

        return true;
    }

    /**
     * Transform a validation status into an EMF diagnostic
     * 
     * @param validationStatus the validation status
     * @param diagnostics the diagnostic to complete
     */
    private void fillDiagnostic(IStatus validationStatus, DiagnosticChain diagnostics)
    {
        if (validationStatus.isOK())
        {
            return;
        }

        if (validationStatus.isMultiStatus())
        {
            IStatus[] children = validationStatus.getChildren();
            for (int i = 0; i < children.length; i++)
            {
                if (children[i] instanceof IConstraintStatus)
                {
                    fillDiagnostic((IConstraintStatus) children[i], diagnostics);
                }
            }
        }
        else if (validationStatus instanceof IConstraintStatus)
        {
            fillDiagnostic((IConstraintStatus) validationStatus, diagnostics);
        }
    }

    /**
     * Transform a constraint status into an EMF diagnostic
     * 
     * @param status the constraint status
     * @param diagnostics the diagnostic to complete
     */
    private static void fillDiagnostic(IConstraintStatus status, DiagnosticChain diagnostics)
    {

        if (diagnostics != null && status != null)
        {
            if (!status.matches(IStatus.ERROR | IStatus.WARNING | IStatus.CANCEL))
            {
                return;
            }

            int level = Diagnostic.WARNING;
            switch (status.getSeverity())
            {
                case IStatus.WARNING:
                    level = Diagnostic.WARNING;
                    break;
                case IStatus.ERROR:
                case IStatus.CANCEL:
                    level = Diagnostic.ERROR;
                    break;
                default:
                    break;
            }

            diagnostics.add(new BasicDiagnostic(level, DIAGNOSTIC_SOURCE, 1, status.getMessage(), new Object[] {status.getTarget()}));
        }
    }
}
