/*******************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Sciamma (Anyware Technologies)
 *    - initial API and implementation
 *******************************************************************************/
package org.topcased.validation.ocl.internal;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.BasicDiagnostic;
import org.eclipse.emf.common.util.DiagnosticChain;
import org.eclipse.emf.ecore.EObject;
import org.topcased.bus.core.IService;
import org.topcased.bus.core.ServicesManager;
import org.topcased.checkresultmodel.Context;
import org.topcased.checkresultmodel.Item;
import org.topcased.checkresultmodel.Model;
import org.topcased.checkresultmodel.Package;
import org.topcased.checkresultmodel.Rule;
import org.topcased.validation.core.IValidator;
import org.topcased.validation.ocl.extension.OCLFileDescriptor;
import org.topcased.validation.ocl.extension.OCLMetamodelManager;

/**
 * Validates a model using OCL files registered with the extension point <br>
 * creation : 16 janv. 2006
 * 
 * @author <a href="mailto:david.sciamma@anyware-tech.com">David Sciamma</a>
 */
public class OCLMetamodelValidator implements IValidator
{
    public static final String DIAGNOSTIC_SOURCE = "org.topcased.validation.ocl.OCLMetamodelValidator";

    public static final int RULE = 1;

    /**
     * Calidates an EObject using the given OCL file
     * 
     * @param model the model to validate
     * @param oclFile the file that conatins the OCL rules
     * @param monitor the progress monitor
     * @return The validation results
     */
    protected Model validate(EObject model, String oclFile, IProgressMonitor monitor)
    {
        Model resultModel = null;

        // Call the validation from the OCL Checker provided by the bus
        IService checkerService = ServicesManager.getInstance().getService("org.topcased.service.oclchecker.fileCheckService");

        if (checkerService != null)
        {
            HashMap parameters = new HashMap();
            parameters.put("Model", model);
            parameters.put("OCLFile", oclFile);
            Object result = checkerService.serviceRun(parameters);
            if (result instanceof Model)
            {
                resultModel = (Model) result;
            }
        }

        return resultModel;
    }

    /**
     * Fill the diagnostic result with the given CheckModel Result
     * 
     * @param result the validation result
     * @param level the level od the produced problems
     * @param diagnostics the translated result
     */
    private void fillDiagnostic(Model result, int level, DiagnosticChain diagnostics)
    {
        Iterator itPackages = result.getContextPackages().iterator();
        while (itPackages.hasNext())
        {
            Package pack = (Package) itPackages.next();

            Iterator itContexts = pack.getContext().iterator();
            while (itContexts.hasNext())
            {
                Context context = (Context) itContexts.next();

                Iterator itRules = context.getRules().iterator();
                while (itRules.hasNext())
                {
                    Rule rule = (Rule) itRules.next();

                    Iterator itItems = rule.getRejectedItems().iterator();
                    while (itItems.hasNext())
                    {
                        Item item = (Item) itItems.next();

                        fillDiagnostic(rule, item, level, diagnostics);
                    }
                }
            }
        }
    }

    /**
     * Add the error for the given rule and the given item in the validation result
     * 
     * @param rule the failing rule
     * @param item the erroneous item
     * @param level the level od the produced problems
     * @param diagnostics the validation result
     */
    private void fillDiagnostic(Rule rule, Item item, int level, DiagnosticChain diagnostics)
    {
        EObject eObject = null;
        if (item != null)
        {
            eObject = item.getReference();
        }

        if (diagnostics != null && rule != null && eObject != null)
        {
            StringBuffer message = new StringBuffer("Error for rule : ");
            message.append(rule.getRuleText());

            diagnostics.add(new BasicDiagnostic(level, DIAGNOSTIC_SOURCE, RULE, message.toString(), new Object[] {eObject}));
        }
    }

    /**
     * Validates an EObject using the given OCL file and translates the result into a Diagnostic
     * 
     * @param eObject the object to validate
     * @param oclFile the file containing OCL rules
     * @param level The level of the produced problems
     * @param diagnostics the validation result
     * @param monitor the progress monitor
     */
    protected void validate(EObject eObject, String oclFile, int level, DiagnosticChain diagnostics, IProgressMonitor monitor)
    {
        Model result = validate(eObject, oclFile, monitor);
        if (result != null)
        {
            fillDiagnostic(result, level, diagnostics);
        }
    }

    /**
     * @see org.topcased.validation.core.IValidator#validate(org.eclipse.emf.ecore.EObject,
     *      org.eclipse.emf.common.util.DiagnosticChain, org.eclipse.core.runtime.IProgressMonitor)
     */
    public boolean validate(EObject eObject, DiagnosticChain diagnostics, IProgressMonitor monitor)
    {
        // call the extension point manager
        String metamodelURI = eObject.eClass().getEPackage().getNsURI();
        OCLFileDescriptor[] rules = OCLMetamodelManager.getInstance().getFiles(metamodelURI);
        monitor.beginTask("OCL Validation", rules.length);
        for (int i = 0; i < rules.length; i++)
        {
            validate(eObject, rules[i].getFile(), rules[i].getLevel(), diagnostics, monitor);
            monitor.worked(1);
        }
        monitor.done();
        return true;
    }
}
