/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.web.server;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.topcased.externalbus.core.CorePlugin;
import org.topcased.externalbus.core.ExternalServicesManager;
import org.topcased.externalbus.core.IExternalService;
import org.topcased.web.services.ExternalServiceType;
import org.topcased.web.services.INPUTPARAMETER;
import org.topcased.web.xml.ExternalServiceBinder;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Handler for external services. This is the interface between soapServer and topcased application.
 * 
 * @author <a href="mailto:david.brouard@c-s.fr">David BROUARD</a>
 * 
 */
public class ExternalServiceHandler
{

	private ExternalServiceType _externalServiceInstance_;

	private final static String GET_ALL_SERVICES = "getAllServices";

	/**
	 * instantiate an external Service from it xml String representation.
	 * 
	 * @param Xmldefinition_ - the xml string representing the external service.
	 */
	public void setExternalServiceDescription(String Xmldefinition_)
	{
		ExternalServiceBinder l_binder = new ExternalServiceBinder();
		_externalServiceInstance_ = l_binder.getExternalServiceFromXml(Xmldefinition_);
	}

	/**
	 * Retrieve parameters from externalService and Use the external service bus to call the external service.
	 * 
	 * @return response of the internal service call.
	 * @throws ExternalServiceException
	 */
	public String callExternalService() throws ExternalServiceException
	{
		String externalServiceReturn = null;
		if (_externalServiceInstance_ != null)
		{
			String l_serviceName = _externalServiceInstance_.getSERVICENAME();
			CorePlugin.log(getClass(), "Calling external service " + l_serviceName, IStatus.INFO);
			if (l_serviceName.equalsIgnoreCase(GET_ALL_SERVICES))
			{
				Document externalServiceDoc = ExternalServicesManager.getInstance().getExternalService();
				externalServiceReturn = documentToString(externalServiceDoc);
			}
			else
			{
				IExternalService l_externalService = getExternalService();
				// Launch ExternalService indicated by the service Id
				CorePlugin.log(getClass(), "Server launchs : " + l_serviceName, IStatus.INFO);
				Map<String, Object> l_externalServiceParameters = buildExternalServiceParameters();
				externalServiceReturn = callExternalService(l_externalService, l_externalServiceParameters);
				CorePlugin.log(getClass(), "End " + l_serviceName, IStatus.OK);
			}
		}
		else
		{

		}
		return externalServiceReturn;
	}

	/**
	 * Call external service.
	 * 
	 * @param externalServiceParameters_ - parameters of the external service.
	 * @return The Document corresponding of the external service response.
	 * @throws ExternalServiceException - if the external service response is not a document.
	 */
	private String callExternalService(IExternalService iExternalService, Map<String, Object> externalServiceParameters_)
	        throws ExternalServiceException
	{
		String externalServiceResponse = null;
		Object externalServiceReturn = iExternalService.externalServiceRun(externalServiceParameters_);
		if (externalServiceReturn instanceof String)
		{
			externalServiceResponse = (String) externalServiceReturn;

		}
		else if (externalServiceReturn == null)
		{
			StringBuilder exceptionString = new StringBuilder("External service return is NULL.");
			exceptionString.append("\n Arguments are : " + getArgumentsAsString(externalServiceParameters_));
			throw new ExternalServiceException(exceptionString.toString());
		}
		else
		{
			throw new ExternalServiceException("Return of an external service call must be a String class = "
			        + externalServiceReturn.getClass().getName());
		}
		return externalServiceResponse;
	}

	private String getArgumentsAsString(Map<String, Object> externalServiceParameters_)
	{
		StringBuilder argumentAsString = new StringBuilder();
		for (String key : externalServiceParameters_.keySet())
		{
			argumentAsString.append(key + "=" + externalServiceParameters_.get(key) + "\n");
		}
		return argumentAsString.toString();
	}

	/**
	 * Get external service selected by it's name.
	 * 
	 * @throws ExternalServiceException - if no service with the correct name was found.
	 */
	private IExternalService getExternalService() throws ExternalServiceException
	{
		// Take parameters for the ExternalService indicated by the
		// "serviceId"
		IExternalService l_iExternalService = ExternalServicesManager.getInstance().getExternalServiceClass(
		        _externalServiceInstance_.getSERVICENAME());
		if (l_iExternalService == null)
		{
			// the external service name isn't correct.
			throw new ExternalServiceException("Can't find service with name = " + _externalServiceInstance_.getSERVICENAME());
		}
		return l_iExternalService;
	}

	/**
	 * Build external service parameter Map. String get from the soap message are translated with their mime type.
	 * 
	 * @return
	 */
	private Map<String, Object> buildExternalServiceParameters()
	{
		HashMap<String, Object> l_parametersMap = new HashMap<String, Object>();
		for (INPUTPARAMETER l_parameter : _externalServiceInstance_.getINPUTPARAMETERS().getINPUTPARAMETER())
		{
			if ((l_parameter.getType().equals("text/xml")) || (l_parameter.getType().equalsIgnoreCase("node")))
			{
				// create a document from String.
				try
				{
					Document l_docParameter = stringToDom(l_parameter.getValue());

					l_parametersMap.put(l_parameter.getName(), l_docParameter.getDocumentElement());
					l_parametersMap.put(l_parameter.getName() + "debug", l_parameter.getValue());
				}
				catch (SAXException e)
				{
					CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
				}
				catch (ParserConfigurationException e)
				{
					CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
				}
				catch (IOException e)
				{
					CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
				}
			}
			else if ((l_parameter.getType().equals("text/plain")) || (l_parameter.getType().equalsIgnoreCase("string")))
			{
				l_parametersMap.put(l_parameter.getName(), l_parameter.getValue());
			}
		}
		return l_parametersMap;
	}

	/**
	 * Create a Document instance from an xml String.
	 * 
	 * @param xmlSource - String in the xml format.
	 * @return
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * @throws IOException
	 */
	private Document stringToDom(String xmlSource) throws SAXException, ParserConfigurationException, IOException
	{
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		return builder.parse(new InputSource(new StringReader(xmlSource)));
	}

	public ExternalServiceType getServiceType()
	{
		return _externalServiceInstance_;
	}

	/**
	 * Convert a xml document to a String.
	 * 
	 * @param node
	 * @return
	 */
	private String documentToString(Node node)
	{
		try
		{
			Source source = new DOMSource(node);
			StringWriter stringWriter = new StringWriter();
			Result result = new StreamResult(stringWriter);
			TransformerFactory factory = TransformerFactory.newInstance();
			Transformer transformer = factory.newTransformer();
			transformer.transform(source, result);
			return stringWriter.getBuffer().toString();
		}
		catch (TransformerConfigurationException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}
		catch (TransformerException e)
		{
			CorePlugin.log(getClass(), e.getMessage(), IStatus.ERROR, e);
		}
		return "Error";
	}
}
