/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.web.server.SOAP;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.SelectorProvider;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.Iterator;
import java.util.Set;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPMessage;

/**
 * Listenes to SOAP message in the server channel.<br>
 * This listener waits for a client connection.<br>
 * It reads from client stream and builds SOAP message.<br>
 * It sends the SOAP message to a _callback which processes the SOAP message.<br>
 * 
 * @author <a href="mailto:david.brouard@c-s.fr">David BROUARD</a>
 */
public class SOAPListenerImpl implements SOAPListener
{
	/**
	 * Selector for the connection key.
	 */
	private Selector _selector;

	/**
	 * Callback used to process SOAP message. It's called after a soap message is extract from client stream.
	 */
	private SOAPCallback _callback;

	/**
	 * Thread waiting for connection.
	 */
	private Thread _startThread;

	public void initMsgLoop(int port, SOAPCallback callback) throws IOException
	{

		this._callback = callback;
		ServerSocketChannel serverChannel = ServerSocketChannel.open();
		serverChannel.configureBlocking(false);
		InetAddress lh = InetAddress.getLocalHost();
		InetSocketAddress isa = new InetSocketAddress(lh, port);
		serverChannel.socket().bind(isa);
		_selector = SelectorProvider.provider().openSelector();
		serverChannel.register(_selector, SelectionKey.OP_ACCEPT);
	}

	private void startMsgLoop() throws IOException, ClosedChannelException, InterruptedException
	{
		try
		{
			// Wait for an event
			while (true)
			{
				if (_selector.select() == 0)
				{
					continue;
				}
				// Get list of selection keys with pending events
				Set<SelectionKey> keys = _selector.selectedKeys();
				Iterator<SelectionKey> it = keys.iterator();
				// Process each key
				while (it.hasNext())
				{
					SelectionKey key = (SelectionKey) it.next();
					it.remove(); // remove the key
					int kro = key.readyOps();
					if ((kro & SelectionKey.OP_READ) == SelectionKey.OP_READ)
						doRead(key);
					if ((kro & SelectionKey.OP_WRITE) == SelectionKey.OP_WRITE)
						doWrite(key);
					if ((kro & SelectionKey.OP_ACCEPT) == SelectionKey.OP_ACCEPT)
						doAccept(key);
				}
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{

			System.out.println(getClass().getName() + " startMsgLoop() End");
		}
	}

	public void startMsgLoopInThread()
	{
		_startThread = new Thread(new Runnable()
		{
			public void run()
			{
				try
				{
					startMsgLoop();
				}
				catch (IOException ie)
				{
					ie.printStackTrace();
				}
				catch (InterruptedException ine)
				{
					ine.printStackTrace();
				}
			}
		});
		_startThread.setName("SOAPServerThread");
		_startThread.start();
	}

	public void stopMsgLoopInThread()
	{
		_startThread.interrupt();
	}

	private void doAccept(SelectionKey sk)
	{
		System.out.println(getClass().getName() + " doAccept");

		ServerSocketChannel sc = (ServerSocketChannel) sk.channel();
		SocketChannel usc = null;
		ByteBuffer data;
		try
		{

			usc = sc.accept();
			usc.configureBlocking(false);
			ChannelClosure l_channelClosure = new ChannelClosure(sk);
			data = ByteBuffer.allocate(8192);
			data.position(data.limit()); // looks like write complete
			usc.register(_selector, SelectionKey.OP_READ, l_channelClosure);
		}
		catch (IOException re)
		{
			fail(re, "registration error");
		}
	}

	/**
	 * Reads buffer from channel. Decode the Soap message in the stream and send it to a soapCallBack.
	 * 
	 * @param sk - channel selection key to read from and to send the response message.
	 * @throws IOException
	 */
	private void doRead(SelectionKey sk) throws IOException
	{
		// System.out.println(getClass().getName() + " doRead");
		SocketChannel sc = (SocketChannel) sk.channel();
		ByteBuffer byteBuffer = ByteBuffer.allocate(1024);
		ChannelClosure l_channelClosure = (ChannelClosure) sk.attachment();

		sc.read(byteBuffer);
		byteBuffer.flip();
		String result = this.decode(byteBuffer);
		l_channelClosure.append(result);
		if (l_channelClosure.gotFullRequest())
		{
			_callback.onMessage(l_channelClosure.getMessage(), sk);
		}
	}

	private void doWrite(SelectionKey sk)
	{
		System.out.println(getClass().getName() + " doWrite");
		SocketChannel sc = (SocketChannel) sk.channel();
		ByteBuffer data = (ByteBuffer) sk.attachment();

		if (data.hasRemaining())
		{
			try
			{
				sc.write(data);
			}
			catch (IOException e)
			{
				e.printStackTrace();
				close(sc);
			}
		}
		if (!data.hasRemaining())
			sk.interestOps(SelectionKey.OP_READ);
	}

	private void close(SocketChannel sc)
	{
		try
		{
			sc.close();
		}
		catch (IOException ce)
		{
			fail(ce, "close failed");
		}
	}

	/**
	 * Decode bytebuffer to a String.
	 * 
	 * @param byteBuffer - bytebuffer to decode.
	 * @return String corresponding to byteBuffer datas.
	 * @throws CharacterCodingException if bytebuffer character encoding is not UTF-8.
	 */
	private String decode(ByteBuffer byteBuffer) throws CharacterCodingException
	{

		Charset charset = Charset.forName("UTF-8");
		CharsetDecoder decoder = charset.newDecoder();
		CharBuffer charBuffer = decoder.decode(byteBuffer);
		String result = charBuffer.toString();
		return result;
	}

	private void fail(Exception e, String s)
	{
		if (e != null)
			e.printStackTrace();
		if (s != null)
			System.out.println(s);
		System.exit(0);
	}

	public class ChannelClosure
	{
		private SelectionKey _selectionKey;

		private StringBuffer buffer;

		private SimpleHTTP http;

		public ChannelClosure(SelectionKey key_)
		{
			this._selectionKey = key_;
			this.buffer = new StringBuffer();
			this.http = new SimpleHTTP(buffer);
		}

		public SelectionKey getKeyChannel()
		{
			return this._selectionKey;
		}

		public void append(String part)
		{
			buffer.append(part);
		}

		public boolean gotFullRequest()
		{
			http.updateRequest();
			return http.gotFullRequest();
		}

		public SOAPMessage getMessage()
		{
			return http.getContent();
		}
	}

	public class SimpleHTTP
	{
		private StringBuffer requestStr;

		private boolean gotFull;

		private int curIndex;

		private int lineBegIndex;

		private boolean gotHeaders;

		private MimeHeaders headers;

		private int contentLength = -1;

		private String xmlData;

		private String lastHdrAttr;

		private String lastHdrValue;

		private SOAPMessage msg;

		public SimpleHTTP(StringBuffer requestStr)
		{
			this.requestStr = requestStr;
			headers = new MimeHeaders();
			this.curIndex = 0;
		}

		public void updateRequest()
		{
			while (!gotHeaders && curIndex < requestStr.length())
			{
				char ch = requestStr.charAt(curIndex);
				if (ch == '\n' && curIndex > 0 && requestStr.charAt(curIndex - 1) == '\r')
				{

					String hdr = requestStr.substring(lineBegIndex, curIndex - 1);
					addHeaders(hdr);
					if (lineBegIndex + 1 == curIndex)
					{
						gotHeaders = true;
					}
					lineBegIndex = curIndex + 1;
				}
				curIndex++;
			}
			if (gotHeaders && requestStr.length() >= curIndex + contentLength)
			{
				this.gotFull = true;
				xmlData = requestStr.substring(curIndex, requestStr.length());
			}
		}

		public boolean gotFullRequest()
		{
			return gotFull;
		}

		public SOAPMessage getContent()
		{
			if (msg == null)
			{
				try
				{
					MessageFactory msgFactory = MessageFactory.newInstance();
					msg = msgFactory.createMessage(headers, new ByteArrayInputStream(xmlData.getBytes()));
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}
			return msg;
		}

		private void addHeaders(String hdr)
		{
			int colonIndex = hdr.indexOf(':');
			if (colonIndex == -1)
			{
				if (lastHdrAttr == null)
				{
					return;
				}
				lastHdrValue += hdr;
				// System.out.println("LastHdrAttr=<"+lastHdrAttr+">");
				// System.out.println("LastHdrValu=<"+lastHdrValue+">");
				headers.setHeader(lastHdrAttr, lastHdrValue);
			}
			else
			{
				lastHdrAttr = hdr.substring(0, colonIndex);
				lastHdrValue = hdr.substring(colonIndex + 2, hdr.length());
				// System.out.println("LastHdrAttr=<"+lastHdrAttr+">");
				// System.out.println("LastHdrValu=<"+lastHdrValue+">");
				headers.addHeader(lastHdrAttr, lastHdrValue);
			}
			if (lastHdrAttr.equalsIgnoreCase("Content-Length"))
			{
				contentLength = new Integer(lastHdrValue).intValue();
			}
		}
	}

}
