/******************************************************************************************
 * Copyright (c) 2005 AIRBUS FRANCE.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Christophe Le Camus (CS), Marion Feau (CS), Guillaume Jolly (CS), Sbastien Gabel (CS) 
 *    Petre Bazavan (AEIC), Vincent Combet (CS) - initial API and implementation
 *******************************************************************************************/
package org.topcased.web.server.SOAP;

import java.io.IOException;
import java.nio.channels.SelectionKey;

import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

import org.topcased.web.server.ExternalServiceException;
import org.topcased.web.server.ExternalServiceHandler;
import org.w3c.dom.Document;

/**
 * The SOAP Server is a SOAP message manager.<br>
 * It receives SOAP message from a soap listener and launch a new thread to process it.<br>
 * 
 * @author <a href="mailto:david.brouard@c-s.fr">David BROUARD</a>
 * 
 */
public class SOAPServer implements SOAPCallback
{
    /**
     * Listener which is bind to the SOAP server port and create soap Message.
     */
    private SOAPListener httpServer;

    /**
     * Handler that will call external service web.
     */
    private ExternalServiceHandler _externalServiceHandler;

    /**
     * Create a SOAP Server listening on port port_.
     * 
     * @param port_ - socket port listened by SOAP Server.
     */
    public SOAPServer(int port_)
    {
        httpServer = new SOAPListenerImpl();
        _externalServiceHandler = new ExternalServiceHandler();
        start(port_);
    }

    /**
     * Start listening on port port_.
     * 
     * @param port_
     */
    private void start(int port_)
    {
        try
        {
            httpServer.initMsgLoop(port_, this);
            httpServer.startMsgLoopInThread();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Set external service handler.
     * 
     * @param externalServiceHandler_ - the external service handler.
     */
    public void setExternalServiceHandler(ExternalServiceHandler externalServiceHandler_)
    {
        _externalServiceHandler = externalServiceHandler_;
    }

    /**
     * Init the selection key (ip + port) on wich the response of the soap message will be sent.
     * 
     * @param key - init the selection key on the soap sender.
     * @return - A SOAPSenderImpl which assume the network part of the response to the soap message.
     */
    protected SOAPSenderImpl initSoapResponseSender(SelectionKey key)
    {
        SOAPSenderImpl l_soapSender = new SOAPSenderImpl();
        l_soapSender.setKey(key);
        return l_soapSender;
    }

    /**
     * @see org.topcased.web.server.SOAP.SOAPCallback#onMessage(SOAPMessage, SelectionKey)
     */
    public void onMessage(SOAPMessage requestMsg_, SelectionKey key)
    {
        System.out.println(getClass().getName() + " on Message");
        SOAPSenderImpl l_soapSender = initSoapResponseSender(key);
        MessageProcessor proc = new MessageProcessor(requestMsg_, l_soapSender);
        // Start a new thread to process this client's request
        new Thread(proc, "Message Processor").start();
    }

    /**
     * Process the SOAPMessage in a different thread of the soap server : extract datas from the message and call the
     * external service.
     * 
     * @author dbrouard
     * 
     */
    public class MessageProcessor implements Runnable
    {
        /**
         * SOAP message to be processed.
         */
        private SOAPMessage _requestMsg;

        /**
         * Sender of the response of the message.
         */
        private SOAPSenderImpl _responseSender;

        public MessageProcessor(SOAPMessage requestMsg_, SOAPSenderImpl responseSender_)
        {
            this._requestMsg = requestMsg_;
            _responseSender = responseSender_;
        }

        public void run()
        {
            try
            {
                System.out.println("Received Client Message...");
                _requestMsg.writeTo(System.out);
                extractSoapContent();
            }
            catch (SOAPException se)
            {
                se.printStackTrace();
            }
            catch (IOException ie)
            {
                ie.printStackTrace();
            }
        }

        public void close()
        {

        }

        /**
         * Extract service and attachment from soapMessage.
         * 
         * @throws SOAPException
         * 
         */
        private void extractSoapContent() throws SOAPException
        {
            // message to be sended.
            SOAPMessage msg = null;
            SOAPPart sp = _requestMsg.getSOAPPart();
            SOAPEnvelope envelope = sp.getEnvelope();
            SOAPBody body = envelope.getBody();

            // xml string describing the service.
            SOAPElement documentElement = (SOAPElement) body.getFirstChild();
            if (documentElement == null)
            {
                msg = createFaultMessage("Can't find SOAPElement body.getFirstChild() in " + _requestMsg.toString());
            }
            else
            {
                String l_externalServiceXml = documentElement.getValue();

                // init the external service.
                _externalServiceHandler.setExternalServiceDescription(l_externalServiceXml);

                try
                {
                    // call external service.
                    Object externalServiceResponse = _externalServiceHandler.callExternalService();
                    // create msg response from external service response.
                    msg = createMessage(externalServiceResponse, _externalServiceHandler.getServiceType().getOUTPUTPARAMETER().getType());
                }
                catch (NullPointerException e)
                {
                    e.printStackTrace();
                    msg = createFaultMessage(e.toString());
                }
                catch (ExternalServiceException e)
                {
                    e.printStackTrace();
                    msg = createFaultMessage(e.getMessage());

                }
                catch (SOAPException e)
                {
                    e.printStackTrace();
                    msg = createFaultMessage(e.getMessage());
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
                finally
                {
                    _responseSender.sendMessage(msg);
                }
            }
        }

        /**
         * Create a fault message to be sent to the client.
         * 
         * @param faultReason_
         * @return
         * @throws SOAPException
         */
        private SOAPMessage createFaultMessage(String faultReason_) throws SOAPException
        {
            MessageFactory mf = MessageFactory.newInstance();
            SOAPMessage msg = mf.createMessage();
            SOAPPart sp = msg.getSOAPPart();
            SOAPEnvelope envelope = sp.getEnvelope();
            SOAPBody body = envelope.getBody();
            SOAPFault l_fault = body.addFault();
            if (faultReason_ == null)
            {
                faultReason_ = "";
            }
            l_fault.setFaultString(faultReason_);

            return msg;
        }

        /**
         * Create SOAP message
         * 
         * @param serviceResponse_
         * @param serviceResponseType_
         * @return
         * @throws SOAPException
         */
        public SOAPMessage createMessage(Object serviceResponse_, String serviceResponseType_) throws SOAPException
        {
            // GetExternalService

            // Xml write
            // printDocumentResponse(serviceResponse_);
            MessageFactory mf = MessageFactory.newInstance();
            SOAPMessage msg = mf.createMessage();

            SOAPPart sp = msg.getSOAPPart();
            SOAPEnvelope envelope = sp.getEnvelope();
            SOAPBody body = envelope.getBody();
            QName elementName = new QName("externalServiceResponse");
            SOAPBodyElement element = body.addBodyElement(elementName);

            if (serviceResponseType_.equalsIgnoreCase("text/xml"))
            {
                if (serviceResponse_ instanceof Document)
                {
                    Document doc = (Document) serviceResponse_;
                    element.appendChild(doc);
                    // body.addDocument(doc);
                }
                else if (serviceResponse_ instanceof String)
                {
                    String response = (String) serviceResponse_;
                    element.addTextNode(response);

                }
            }

            return msg;
        }
    }
}
