package tte;

import tte.exceptions.AnalysisException;
import tte.exceptions.CannotCreateProjectFileException;
import tte.exceptions.FilesNeedToBeReloadException;
import tte.exceptions.MustBeTTMKindFileException;
import tte.exceptions.ProjectFileAlreadyDefinedException;
import tte.exceptions.ReportGenerationException;
import tte.exceptions.ResourceLoadingException;
import ttm.Project;

/**
 * The <b>Factory</b> interface.
 * provides some methods to interact with the TopCased Traceability Tool model
 * @see org.topcased.traceability
 */
public interface IEngine {

    /**
     * Creates a new project in the filename passed in parameter.
     * @param filename that will contain the traceability project structure
     * @throws ProjectFileAlreadyDefinedException indicates the project that
     *   the user want to create already exists
     * @throws CannotCreateProjectFileException indicates that an exception
     *  occured during the project creation
     */
    void newProject(String filename)
        throws ProjectFileAlreadyDefinedException,
               CannotCreateProjectFileException;

    /**
     * @param filename that will contain the traceability project structure
     * @throws ProjectFileAlreadyDefinedException indicates the project that
     *   the user want to create already exists
     * @throws CannotCreateProjectFileException indicates that an exception
     *  occured during the project creation
     */
    void newProjectWithOverwrite(String filename)
    throws ProjectFileAlreadyDefinedException,
           CannotCreateProjectFileException;

    /**
     * @return filename of the project
     */
    String getProjectFilename();

    /**
     * @param the project the user want to reload
     * @throws FilesNeedToBeReloadException indicates that some files in project
     *  have changed since the last reloading action
     * @throws ResourceLoadingException indicates the loading
     *  of resource failed
     * @throws AnalysisException indicates that exceptions occured
     *  during project analysis
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     */
    void reloadProject(Project aProject)
        throws FilesNeedToBeReloadException,
        ResourceLoadingException,
        MustBeTTMKindFileException,
        AnalysisException;

    /**
     * @param the project the user want to reload
     * @param update boolean indicates if the user want to update modified file
     * during reloading action
     * @throws ResourceLoadingException indicates the loading
     *  of resource failed
     * @throws AnalysisException indicates that exceptions occured
     *  during project analysis
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     */
    void reloadProjectForcing(Project aProject, Boolean update)
        throws ResourceLoadingException,
        MustBeTTMKindFileException,
        AnalysisException;

    /**
     * @param the project which report the user want to produce
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     * @throws ReportGenerationException indicates that some exception occured
     *  during report generation
     */
    void printReport(Project aProject, String outputStr)
        throws ReportGenerationException,
        MustBeTTMKindFileException;

    /**
     * @param the project which report the user want to produce
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     * @throws ReportGenerationException indicates that some exception occured
     *  during report generation
     */
    void printRulesReport(Project aProject, String outputStr)
        throws ReportGenerationException,
        MustBeTTMKindFileException;

}
