package tte.analyzers;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;

import tte.exceptions.AnalysisException;
import tte.exceptions.AnalyzerMethodException;
import tte.impl.Engine;

/**
 * Class that represents the analyzer instanciated from the parsing
 *  of an extension to DocumentAnalyzer extension point.
 * @author laurentlb
 *
 */
/**
 * @author laurentlb
 *
 */
public class ExtendedAnalyzer extends AbstractExtendedClass{

    /**
     * name of xml attribute that represents the name of document anaylzer
     *  in the extension declaration.
     */
    private static final String ATT_ID = "id";
 
    /**
     * name of xml attribute that represents the name of document anaylzer
     *  in the extension declaration.
     */
    private static final String ATT_NAME = "name";

    /**
     * name of xml attribute that represents the class
     *  which will be instanciated to do the conversion
     *   of the source document into the intermediate one.
     */
    private static final String ATT_OWNING_CLASS = "owningClass";

    /**
     * list of analyzer conversion methods.
     */
    private List < ExtendedAnalyzerConversionMethod > conversionMethods;

    /**
     * list of analyzer conversion methods.
     */
    private List < ExtendedAnalyzerEditionMethod > editionMethods;

    /**
     * analyzer id.
     */
    private String id;
    /**
     * analyzer name.
     */
    private String name;
    /**
     * the configuration element.
     */
    private final IConfigurationElement configElement;


    /**
     * Instanciates a new analyzer.
     * @param anId is the analyzer's id
     * @param aName is the analyzer's name
     */
    public ExtendedAnalyzer(final IConfigurationElement aConfigElement) {
    	configElement = aConfigElement;
        id = getAttribute(aConfigElement, ATT_ID, null);
        name = getAttribute(aConfigElement, ATT_NAME, null);
        this.conversionMethods = new ArrayList < ExtendedAnalyzerConversionMethod > ();
        this.editionMethods = new ArrayList < ExtendedAnalyzerEditionMethod > ();
    }

    /**
     * Adds a method to the analyzer.
     * @param method added
     */
    public final void addConversionMethod(final ExtendedAnalyzerConversionMethod method) {
        conversionMethods.add(method);
    }

    /**
     * Adds a edition method to the analyzer.
     * @param method added
     */
    public final void addEditionMethod(final ExtendedAnalyzerEditionMethod method) {
        editionMethods.add(method);
    }

    /**
     * Gets the analyzer's id.
     * @return the analyzer's id
     */
    public final String getID() {
        return id;
    }

    /**
     * Gets the analyzer's name.
     * @return the analyzer's name
     */
    public final String getName() {
        return name;
    }

    /**
     * Indicates if the analyzer has conversion methods.
     * @return if the analyzer has conversion methods
     */
    public final boolean hasConversionMethods() {
        return !conversionMethods.isEmpty();
    }

    /**
     * Indicates if the analyzer has edition methods.
     * @return if the analyzer has edition methods
     */
    public final boolean hasEditionMethods() {
        return !editionMethods.isEmpty();
    }

    /**
     * Gets the conversion methods which id is the id in parameter.
     * @param anID corresponds to the searched method's id
     * @return the analyzer method
     */
    public final ExtendedAnalyzerConversionMethod getConversionMethodWithID(final String anID) {
        String methId = anID;
        if (anID == null || anID.length() == 0 ) {
            methId = Engine.DEFAULT_ANALYZER_DEFAULT_CONVERSION_METHOD_ID;
        }
        for (Object element : conversionMethods) {
            ExtendedAnalyzerConversionMethod meth = (ExtendedAnalyzerConversionMethod) element;
            if (meth.getID().equals(methId)) {
                return meth;
            }
        }
        return null;
    }

    /**
     * Gets the edition methods which id is the id in parameter.
     * @param anID corresponds to the searched method's id
     * @return the analyzer method
     */
    public final ExtendedAnalyzerEditionMethod getEditionMethodWithID(final String anID) {
        String methId = anID;
        if (anID == null || anID.length() == 0 ) {
            methId = Engine.DEFAULT_ANALYZER_DEFAULT_EDITION_METHOD_ID;
        }
        for (Object element : editionMethods) {
            ExtendedAnalyzerEditionMethod meth = (ExtendedAnalyzerEditionMethod) element;
            if (meth.getID().equals(methId)) {
                return meth;
            }
        }
        return null;
    }
    
    
    /**
     * Gets an instance of OwningClass of this method.
     * @return an instance of OwningClass of this method
     * @throws AnalyzerMethodException if an exception occurs
     *  during instanciation of OwningClass
     */
    public IAnalyzer getAnalyzerOwningClassExecutableExtension()
        throws AnalysisException {
        Object obj;
        try {
            obj = configElement.createExecutableExtension(ATT_OWNING_CLASS);
        } catch (CoreException e) {
            throw new AnalysisException(
                    "Cannot instanciate owning class '"
                    + ATT_OWNING_CLASS
                    + "' of extension "
                    + configElement
                        .getDeclaringExtension()
                        .getUniqueIdentifier());
        }
        if (obj instanceof IAnalyzer) {
            IAnalyzer analyzerExec = (IAnalyzer) obj;
            return analyzerExec;
        }
        return null;
    }
}
