package tte.analyzers;

import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IConfigurationElement;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


import tte.exceptions.ConversionException;
import tte.exceptions.AnalyzerMethodException;

/**
 * Class that represents the analyzer method instanciated from the parsing
 *  of an extension to DocumentAnalyzer extension point.
 * @author laurentlb
 *
 */
public class ExtendedAnalyzerConversionMethod extends AbstractExtendedClass {

    /**
     * name of xml attribute that represents the name of method
     *  in the extension declaration.
     */
    private static final String ATT_NAME = "name";

    /**
     * name of xml attribute that represents the version of method
     *  in the extension declaration.
     */
    private static final String ATT_VERSION = "version";


    /**
     * name of xml attribute that represents the arguments
     *  that can be passed to the method to do the conversion.
     */
    private static final String ATT_ARGUMENTS = "arguments";
    
    /**
     * the method analyzer
     */
    private final IAnalyzer analyzer;
    /**
     * the name of the method.
     */
    private final String name;

    /**
     * the version of the method.
     */
    private final String version;

    /**
     * the configuration element.
     */
    private final IConfigurationElement configElement;

    /**
     * arguments of the method.
     */
    private final String arguments;

    /**
     * Creates a new instance of analyzer method.
     * @param aConfigElement contains the information from extension parsing
     *  to create this method
     */
    public ExtendedAnalyzerConversionMethod(final IConfigurationElement aConfigElement, IAnalyzer parentAnalyzer) {
        configElement = aConfigElement;
        analyzer = parentAnalyzer;
        name = getAttribute(configElement, ATT_NAME, null);
        version = getAttribute(configElement, ATT_VERSION, "0.0.0");
        arguments = configElement.getAttribute(ATT_ARGUMENTS);
    }

    /**
     * Gets the id of the method, computed from the name and the version
     *  values.
     * @return the computed id of the method (<b>name</b>_<b>version</b>)
     */
    public final String getID() {
        return name + "_" + version;
    }

    /**
     * Gets the conversion error message.
     * @param doc
     * @return the error message
     */
    private final String getConversionErrorMessage(ttm.Document document) {
    	return ("The converted file is not a TRAMway XML file. The document "+document.getIdent()+" has not been converted");
    }
    
    /**
     * Runs the conversion.
     * @param sourceFile is the file that will be converted into XML
     * @param destFile is the XML file complete during the conversion
     * @param documentArguments is the specific arguments that come from Document definition
     * @throws AnalyzerMethodException if an exception occurs
     *  during instanciation of OwningClass
     * @throws ConversionException if an exception occurs during the conversion
     */
    public final void executeConversion(
            final File sourceFile, final File destFile, 
            final ExtendedAnalyzer extAnalyzer, ttm.Document document)
            throws AnalyzerMethodException, ConversionException {
            
            analyzer.executeConversionMethodID(
                        getID(),
                        sourceFile,
                        destFile,
                        arguments,
                        document);
            addConversionHeaderToDestinationFile(sourceFile, destFile, extAnalyzer, document);
    }
    
    private void addConversionHeaderToDestinationFile(final File sourceFile,
                        final File destFile, final ExtendedAnalyzer analyzer, ttm.Document document) 
    	throws ConversionException {
        DocumentBuilderFactory factory =
            DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try
        {
            builder = factory.newDocumentBuilder();
            Document xmlDoc = builder.parse( destFile );
            Element rootElt = xmlDoc.getDocumentElement();
            rootElt.setAttribute("analyzerID", analyzer.getID());
            rootElt.setAttribute("conversionMethodID", getID());
            rootElt.setAttribute("analyzerArguments", document.getAnalyzerArguments());
            rootElt.setAttribute("lastModifiedDate",String.valueOf(sourceFile.lastModified()));
            Source source = new DOMSource(xmlDoc);
            Result result = new StreamResult(destFile);
            TransformerFactory transFactory = TransformerFactory.newInstance();
            Transformer transformer = transFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
            transformer.transform(source, result);        
        }
        catch (ParserConfigurationException e)
        {
            throw new ConversionException(getConversionErrorMessage(document));
        }
        catch (SAXException e)
        {
            throw new ConversionException(getConversionErrorMessage(document));
        }
        catch (IOException e)
        {
            throw new ConversionException(getConversionErrorMessage(document));
        }
        catch (TransformerConfigurationException e)
        {
            throw new ConversionException(getConversionErrorMessage(document));
        }
        catch (TransformerException e)
        {
            throw new ConversionException(getConversionErrorMessage(document));
        }
        
    }


}
