//------------------------------------------------
// $Id: DocumentAnalyserManager.java,v 1.3 2007-05-28 09:57:52 matg Exp $
// (c) Anyware Technologies 2007    www.anyware-tech.com
//------------------------------------------------
package tte.extension;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IRegistryChangeEvent;
import org.eclipse.core.runtime.IRegistryChangeListener;
import org.eclipse.core.runtime.Platform;

import tte.TTEnginePlugin;

/**
 * This class is used to manage all the extension of the 'documentAnalyzer' extension point 
 * <br>
 * creation : 5 avr. 07
 *
 * @author <a href="mailto:mickael.gerard@anyware-tech.com">Mickael Gerard</a>
 */
public class DocumentAnalyserManager implements IRegistryChangeListener
{

    private static DocumentAnalyserManager instance;
    
    /**
     * Map Containing all the extension of documentAnalyzer
     */
    private Map<String,DocumentAnalyserExtension> contributorExtensions;
    
    private static final String ATT_ID = "id"; //$NON-NLS-1$

    private static final String ATT_NAME = "name"; //$NON-NLS-1$
    
    private static final String ATT_VERSION = "version"; //$NON-NLS-1$

    private static final String ATT_ATTRIBUTES = "attributes"; //$NON-NLS-1$
            
    private static final String ELEM_DOCUMENT_ANALYZER = "documentAnalyzer"; //$NON-NLS-1$
    
    public static final String CONVERSION_METHOD = "conversionMethod";
    
    public static final String EDITION_METHOD = "editionMethod";

    
    private DocumentAnalyserManager()
    {
        Platform.getExtensionRegistry().addRegistryChangeListener(this);
        contributorExtensions = new HashMap<String,DocumentAnalyserExtension>();
    }
    
    /**
     * @return a DocumentAnalyserManager
     */
    public static DocumentAnalyserManager getInstance()
    {
        if(instance == null)
        {
            instance = new DocumentAnalyserManager();
        }
        return instance;
    }
    
    /**
     * MenuContributor extensions getter.
     * 
     * @return Map <ID, MenuContributorExtension>.
     */
    public Map<String,DocumentAnalyserExtension> getContributorExtensions()
    {
        _initialize();

        return contributorExtensions;
    }

    /**
     * Get a DocumentAnalyserExtension if it exist in the map <code>null</code>
     * if no.
     * 
     * @param analyserId
     * 
     * @return DocumentAnalyserExtension
     */
    public DocumentAnalyserExtension getDocumentAnalyserExtensionByExtensionId(String analyserId)
    {
        _initialize();

        DocumentAnalyserExtension contributorExt = (DocumentAnalyserExtension) contributorExtensions
                .get(analyserId);

        return contributorExt;
    }
    
    /**
     * Get a DocumentAnalyserExtension if it exist in the map <code>null</code>
     * if no.
     * 
     * @param analyserId (pluginId)
     * 
     * @return DocumentAnalyserExtension
     */
    public DocumentAnalyserExtension getDocumentAnalyserExtensionByIdentification(String analyserIdentification)
    {
        _initialize();
        DocumentAnalyserExtension ret = null;
        if(contributorExtensions != null && !contributorExtensions.isEmpty())
        {
            for(DocumentAnalyserExtension analyser : contributorExtensions.values())
            {
                if(analyser.getId() != null && analyser.getId().equalsIgnoreCase(analyserIdentification))                    
                {
                    ret = analyser;
                    break;
                }
            }
        }

        return ret;
    }


    private void _initialize()
    {
        if (contributorExtensions.isEmpty())
        {
            retrieveDocumentAnalysers();
        }
    }

    /**
     * 
     */
    private void retrieveDocumentAnalysers()
    {
        // Iterate on extension points.
        IExtensionRegistry registry = Platform.getExtensionRegistry();
        IExtensionPoint point = registry.getExtensionPoint(TTEnginePlugin.PLUGIN_ID,
                TTEnginePlugin.DOCUMENT_ANALYSER_EXT_ID);

        if(point != null)
        {
            IExtension[] extensions = point.getExtensions();
            for (int i = 0; i < extensions.length; i++)
            {
                addDocumentAnalyserExtension(extensions[i]);
            }
        }
    }

    /**
     * @param extension
     */
    private void addDocumentAnalyserExtension(IExtension extension)
    {
        IConfigurationElement[] elements = extension.getConfigurationElements();

        // Iterates on all the nodes
        for (int j = 0; j < elements.length; j++)
        {
            IConfigurationElement extElement = elements[j]; 
            
            //Retrieve information from conversionMethod element
            if(extElement.getName().equalsIgnoreCase(ELEM_DOCUMENT_ANALYZER))
            {
                if (_checkDocumentAnalyzerTags(extElement))
                {
                    String id = extElement.getAttribute(ATT_ID);
                                            
                    // The menuContributor is not yet defined.
                    if (!contributorExtensions.containsKey(id))
                    {
                        DocumentAnalyserExtension documentAnalyser = new DocumentAnalyserExtension(id);
                       
                        if (extElement.getAttribute(ATT_NAME) != null)
                        {                           
                            documentAnalyser.setName(extElement.getAttribute(ATT_NAME));
                        }
                        
                        IConfigurationElement[] conversionMethods = extElement.getChildren();
                        if(conversionMethods != null && conversionMethods.length > 0)                            
                        {
                            for(IConfigurationElement methodElement : conversionMethods)
                            {
                            	if (methodElement.getName().equals(CONVERSION_METHOD) && _checkMethodTags(methodElement)) {
                        			ConversionMethodExtension methodExtension = new ConversionMethodExtension();
                                    methodExtension.setName(methodElement.getAttribute(ATT_NAME));
                                    methodExtension.setVersion(methodElement.getAttribute(ATT_VERSION));
                                    methodExtension.setArguments(methodElement.getAttribute(ATT_ATTRIBUTES));
                                    documentAnalyser.addConversionMethodExtension(methodExtension);   
                            		
                            	}
                            	else if (methodElement.getName().equals(EDITION_METHOD) && _checkMethodTags(methodElement)) {
                            		EditionMethodExtension methodExtension = new EditionMethodExtension();
                                    methodExtension.setName(methodElement.getAttribute(ATT_NAME));
                                    methodExtension.setVersion(methodElement.getAttribute(ATT_VERSION));
                                    documentAnalyser.addEditionMethodExtension(methodExtension);
                            	}
                            	
                                
                            }
                        }
                                                                        
                        // Store it in the cache
                        contributorExtensions.put(id, documentAnalyser);                    
                    }
                }
            }
        }
    }

    /**
     * Only for the tag : ATT_ID, ATT_NAME
     * 
     * @param documentAnalyzer
     *            element
     * 
     * @return <code>true</code> if the Tags exist
     */
    private boolean _checkDocumentAnalyzerTags(IConfigurationElement documentAnalyserElement)
    {
        String id = documentAnalyserElement.getAttribute(ATT_ID);
        String name = documentAnalyserElement.getAttribute(ATT_NAME);               

        return (!"".equals(id) && !"".equals(name));
    }
    
    /**
     * Only for the tag : ATT_NAME, ATT_VERSION
     * 
     * @param conversionMethodElement
     *            element
     * 
     * @return <code>true</code> if the Tags exist
     */
    private boolean _checkMethodTags(IConfigurationElement conversionMethodElement)
    {        
        String name = conversionMethodElement.getAttribute(ATT_NAME);
        String version = conversionMethodElement.getAttribute(ATT_VERSION);
        
        return (!"".equals(name)&& !"".equals(version));
    }
    
    public void registryChanged(IRegistryChangeEvent event)
    {      
        
    }
    
    

}
