package tte.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


import tte.IEngine;
import tte.TTEnginePlugin;
import tte.analyzers.ExtendedAnalyzer;
import tte.analyzers.ExtendedAnalyzerConversionMethod;
import tte.analyzers.ExtendedAnalyzerEditionMethod;
import tte.analyzers.IAnalyzer;
import tte.exceptions.AnalysisException;
import tte.exceptions.AnalyzerMethodException;
import tte.exceptions.CannotCreateProjectFileException;
import tte.exceptions.EditionException;
import tte.exceptions.FilesNeedToBeReloadException;
import tte.exceptions.MustBeTTMKindFileException;
import tte.exceptions.ProjectFileAlreadyDefinedException;
import tte.exceptions.ReportGenerationException;
import tte.exceptions.ResourceLoadingException;
import tte.rules.ExtendedRule;
import tte.util.FileUtils;

import ttm.Attribute;
import ttm.AttributeOwner;
import ttm.CoverLink;
import ttm.HierarchicalElement;
import ttm.IdentifiedElement;
import ttm.NavigationLink;
import ttm.Project;
import ttm.Requirement;
import ttm.Rule;
import ttm.RuleInstance;
import ttm.Text;
import ttm.TtmFactory;
import ttm.TtmPackage;
import ttm.util.TtmResourceFactoryImpl;

/**
 * An implementation of the '<em><b>IEngine</b></em>' interface.
 *
 */
/**
 * @author laurentlb
 *
 */
public class Engine implements IEngine {

    /**
     * id of the documentAnalyzers extension point.
     */
    public static final String ANALYZER_EXT_POINT_ID  =
            "org.topcased.traceability.engine.documentAnalyzers";

    /**
     * id of the default documentAnalyzers extension.
     */
    public static final String DEFAULT_ANALYZER_ID =
        ANALYZER_EXT_POINT_ID + ".default";

    /**
     * id of the default conversion method of the default
     *  documentAnalyzers extension.
     */
    public static final String DEFAULT_ANALYZER_DEFAULT_CONVERSION_METHOD_ID =
        "defaultConversion_1.0.0";

    /**
     * id of the default edition method of the default
     *  documentAnalyzers extension.
     */
    public static final String DEFAULT_ANALYZER_DEFAULT_EDITION_METHOD_ID =
        "defaultEdition_1.0.0";

    /**
     * map that will defined the emf xml saved format.
     */
    private static final Map<String, Object> TTM_MAP = new HashMap < String, Object > ();

    /**
     * name of the rule tag in RulesSet extension point.
     */
    private static final String TAG_RULE = "rule";

    /**
     * id of the rulesSet extension point.
     */
    private static final String RULES_SET_EXT_POINT_ID  =
        "org.topcased.traceability.engine.rulesSet";

    /**
     * name of the document analyzer tag in documentAnalyzers extension point.
     */
    private static final String TAG_DOCUMENT_ANALYZER = "documentAnalyzer";

    /**
     * name of the conversion method tag in documentAnalyzers extension point.
     */
    private static final String TAG_CONVERSION_METHOD = "conversionMethod";

    /**
     * name of the edition method tag in documentAnalyzers extension point.
     */
    private static final String TAG_EDITION_METHOD = "editionMethod";

    /**
     * list of rules from extensions.
     */
    private static ExtendedRule[] cachedRules;

    /**
     * list of rules from extensions.
     */
    private static ExtendedAnalyzer[] cachedAnalyzers;


    /**
     * current project results file.
     */
    private File resultsFile = null;
    
    /**
     * current project file.
     */
    private File projectFile = null;
   /**
     * the current resource set of the engine.
     * @see getResourceSet()
     */
    private ResourceSet resourceSet = null;

    /**
     * the current resource of the engine.
     */
    private Resource resource = null;

    /**
     * the current project of the engine.
     */
    private Project currentProject = null;

    private void affectCurrentProject(Project aProject) {
    	currentProject = aProject;
		projectFile = new File(currentProject.getTtmFilename());
		resultsFile = new File(projectFile.getAbsolutePath() + "results");	
    }

    /*
     * Constructor
     */
    /**
     * Creates a new instance of Engine.
     */
    public Engine() {
        super();
    }

    /**
     * Gets the Map needed for the emf xml saved format.
     * @return the Map needed for the emf xml saved format
     */
    public static Map<String, Object> ttmMap() {
     return TTM_MAP;
    }

    /*
     * Cached Elements
     */

    /**
     * Cleans static elements that are in cache.
     */
    public static void cleanCachedElements() {
        if (cachedRules != null) {
            cleanCachedRules();
        }
    }

    /**
     * Cleans cachedRules getted from extensions.
     */
    private static void cleanCachedRules() {
        for (int i = 0; i < cachedRules.length; i++) {
            ((ttm.Rule) cachedRules[i]).getRuleInstances().clear();
        }
    }


    /*
     * Accessing
     */
    /**
     * @return the filename of the last loaded project
     * @see tte.IEngine#getProjectFilename()
     */
    public final String getProjectFilename() {
        return resource.getURI().toFileString();
    }


    /**
     * Init the current resource set.
     */
    private void initResourceSet() {
        resourceSet = new ResourceSetImpl();
        resourceSet
            .getResourceFactoryRegistry()
            .getExtensionToFactoryMap()
            .put(Resource.Factory.Registry.DEFAULT_EXTENSION
                    , new XMIResourceFactoryImpl());

        resourceSet
            .getPackageRegistry()
            .put(TtmPackage.eNS_URI, TtmPackage.eINSTANCE);       
    }
    
    /**
     * Initilize resource set and gets it.
     * @return the new current resource set
     */
    private ResourceSet getNewResourceSet() {
        initResourceSet();
        return resourceSet;
    }
    
    /**
     * gets or creates current resourceSet.
     * @return the current resource set
     */
    private ResourceSet getResourceSet() {
        if (resourceSet == null) {
            initResourceSet();
        }
        return resourceSet;
    }

    /*
     * Project Methods
     */

    /**
     * Creates a new project in the filename passed in parameter,
     *  overwritten the file if required.
     * @param filename that will contain the traceability project structure
     * @param overwrite indicates if the project file will be overwritten
     *  if already exists
     * @throws ProjectFileAlreadyDefinedException indicates the project that
     *   the user want to create already exists
     * @throws CannotCreateProjectFileException indicates that an exception
     *  occured during the project creation
     */
    private void newProject(final String filename, final Boolean overwrite)
        throws ProjectFileAlreadyDefinedException,
        CannotCreateProjectFileException {
        File file = new File(filename);
        if (file.exists() && !overwrite) {
            throw new ProjectFileAlreadyDefinedException();
        } else {
            try {
                file.createNewFile();
            } catch (IOException e) {
                throw new CannotCreateProjectFileException(
                        file.getAbsolutePath());
            }
        }

        URI uri = URI.createFileURI(file.getAbsolutePath());
        TtmResourceFactoryImpl factory = new TtmResourceFactoryImpl();
        resource = factory.createResource(uri);
        getNewResourceSet().getResources().add(resource);

        Project newProject = TtmFactory.eINSTANCE.createProject();
        if (newProject != null) {
           affectCurrentProject(newProject);
           resource.getContents().add(currentProject);
        }
    }

    /**
     * {@inheritDoc}
     * @see tte.IEngine#newProject(java.lang.String)
     */
    public final void newProject(final String filename)
        throws ProjectFileAlreadyDefinedException,
        CannotCreateProjectFileException {
        newProject(filename, false);
    }

    /**
     * @see tte.IEngine#newProjectWithOverwrite(java.lang.String)
     * {@inheritDoc}
     */
    public final void newProjectWithOverwrite(final String filename)
        throws ProjectFileAlreadyDefinedException,
        CannotCreateProjectFileException {
        newProject(filename, true);
    }
    

    /**
     * {@inheritDoc}
     * @see tte.IEngine#reloadProject(java.lang.String)
     */
    public final void reloadProject(final Project aProject)
        throws FilesNeedToBeReloadException,
        ResourceLoadingException,
        MustBeTTMKindFileException,
        AnalysisException {
    	affectCurrentProject(aProject);
        loadResourceAndTreatIfNecessary(projectFile.getAbsolutePath());
    }

    /**
     * {@inheritDoc}
     * @see tte.IEngine#reloadProjectForcing(java.lang.String,
     *                                   java.lang.Boolean)
     */
    public final void reloadProjectForcing(final Project aProject,
            final Boolean update)
        throws ResourceLoadingException,
        MustBeTTMKindFileException,
        AnalysisException {
    	affectCurrentProject(aProject);
        if (resultsFile.exists() && !update) {
            loadResource(resultsFile.getAbsolutePath());
        } else {
            analyseProject();
        }
    }

    /*
     * Working Methods
     */

    /**
     * @param the project uri
     * loads resource from current project results file
     *  and converts it if it has changed.
     * @throws FilesNeedToBeReloadException indicates that some files in project
     *  have changed since the last reloading action
     * @throws ResourceLoadingException indicates the loading
     *  of resource failed
     * @throws AnalysisException indicates that exceptions occured
     *  during project analysis
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     */
    private void loadResourceAndTreatIfNecessary(String fileURI)
        throws FilesNeedToBeReloadException,
        ResourceLoadingException,
        AnalysisException, MustBeTTMKindFileException {
    	if (fileURI == null) {
    		throw new AnalysisException("Project filename is null. Cannot load the project.",1);
    	}
        if (resultsFile.exists()) {
            loadResource(resultsFile.getAbsolutePath());
            if (checkFilesDate(fileURI)) {
                throw new FilesNeedToBeReloadException();
            }
        } else {
            analyseProject();
        }
    }

    /**
     * Loads resource from the filename in parameter.
     * @param filename of the resource file that will be loaded
     * @throws ResourceLoadingException indicates the loading
     *  of resource failed
     */
    private void loadResource(final String filename)
        throws ResourceLoadingException {
        URI uri = URI.createFileURI(filename);
        try {
            resource = getResourceSet().getResource(uri, true);
        } catch (WrappedException exception) {
            throw new ResourceLoadingException(uri);
        }
    }

    /**
     * Instanciates the Project analyzer and runs analysis.
     * @throws AnalysisException indicates that exceptions occured
     *  during project analysis
     */
    private void analyseProject() throws AnalysisException {
        ProjectAnalyzer analyzer = new ProjectAnalyzer(
                projectFile.getAbsolutePath());
        resource = analyzer.runAnalysis(getNewResourceSet());

    }

    /**
     * @param fileURI the project file URI
     * Compares the date of the document filename
     *  with its date during last conversion.
     * @return if the file of project has been modified
     *  since the last loading action
     * @throws MustBeTTMKindFileException indicates that the extension of
     *  filename in parameter is wrong
     * @throws AnalysisException 
     */
    private boolean checkFilesDate(String fileURI) throws MustBeTTMKindFileException, AnalysisException {
    	
    	File preAnalysisModelFile = new File(projectFile.getAbsolutePath() + "~");
    	
    	try {
	    	if (!preAnalysisModelFile.exists() || !FileUtils.contentsEqual(projectFile, preAnalysisModelFile)) {
	    		FileUtils.copyFile(projectFile, preAnalysisModelFile);
	    		return true;
	    	}
    	}
    	catch (IOException e) {
    		TTEnginePlugin.displayError("Unable to check model state", e);
    	}
    	
//        if (projectFile.lastModified() > resultsFile.lastModified()) {
//            return true;
//        } else {
            for (Object obj : resource.getContents()) {
                if (obj instanceof Project) {
                    Project proj = (Project) obj;
                    for (ttm.Document doc : proj.getDocuments()) {
                        File docFile = new File(doc.getFilename());
                        if (doc.getLastModifiedDate() < docFile.lastModified())
                        {
                            return true;
                        }
                    }
                }
            }
//        }
        return false;
    }

    /*
     * Edition
     */

    public final void editElement(ttm.Element element) 
    	throws EditionException, AnalyzerMethodException {
    	ttm.Document document = element.getDocument();
    	ExtendedAnalyzer analyzer = Engine.getAnalyzerWithID(document
				.getAnalyzerID());
		if (analyzer == null) {
			throw new EditionException("No available analyzer for document "
					+ document.getIdent() + ". Edition failed.");
		}
		ExtendedAnalyzerEditionMethod editionMethod = 
			analyzer.getEditionMethodWithID(document.getEditionMethodID());
		if(editionMethod == null) {
			throw new EditionException("Edition method with ID " 
					+ document.getEditionMethodID()
					+ " does not exists in document " 
					+ document.getIdent() + " definition. Edition failed.");
		}
		editionMethod.executeEdition(document, element);
    }
    /*
     * Reports methods
     */
    /**
     * {@inheritDoc}
     * @see tte.IEngine#printReport(java.lang.String)
     */
    public final void printReport(final Project aProject,
    		final String outputFileStr)
        throws ReportGenerationException,
        MustBeTTMKindFileException {
    	affectCurrentProject(aProject);
    	//String projectPath = projectFile.getAbsolutePath();
        //File file = new File(projectPath.substring(0,
        //		projectPath.lastIndexOf(".")) + ".xml");
        File file = new File(outputFileStr);
    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try {
            builder = factory.newDocumentBuilder();
            Document domDocument = builder.newDocument();
            domDocument.setXmlVersion("1.0");
            domDocument.setXmlStandalone(true);

            loadResource(resultsFile.getAbsolutePath());

            printReportOn(domDocument);
            
            printDOMDocument(file, domDocument);

        } catch (ParserConfigurationException e) {
            throw new ReportGenerationException(
                    "Cannot create the document builder: "
                    + e.getMessage());
        } catch (ResourceLoadingException e) {
            throw new ReportGenerationException(
                    "Cannot load resource: "
                    + e.getMessage());
        }
    }

    /**
     * {@inheritDoc}
     * @see tte.IEngine#printRulesReport(java.lang.String)
     */
    public final void printRulesReport(final Project aProject,
    		final String outputFileStr)
        throws ReportGenerationException,
        MustBeTTMKindFileException {

    	affectCurrentProject(aProject);
        File file = new File(outputFileStr);
    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try {
            builder = factory.newDocumentBuilder();
            Document domDocument = builder.newDocument();
            domDocument.setXmlVersion("1.0");
            domDocument.setXmlStandalone(true);

            loadResource(resultsFile.getAbsolutePath());

            printRulesReportOn(domDocument);
            
            printDOMDocument(file, domDocument);

        } catch (ParserConfigurationException e) {
            throw new ReportGenerationException(
                    "Cannot create the document builder: "
                    + e.getMessage());
        } catch (ResourceLoadingException e) {
            throw new ReportGenerationException(
                    "Cannot load resource: "
                    + e.getMessage());
        } 
    }

    /**
     * Prints the DOM Document structure to the result file
     * @param file the written file
     * @param domDocument the Dom Document
     * @throws ReportGenerationException
     * @throws MustBeTTMKindFileException
     */
    private void printDOMDocument(File file, Document domDocument) 
                    throws ReportGenerationException, MustBeTTMKindFileException  {
       try {
    
           Source source = new DOMSource(domDocument);
           Result result = new StreamResult(file.getAbsolutePath());
           TransformerFactory fabrique = TransformerFactory.newInstance();
           Transformer transformer = fabrique.newTransformer();
           transformer.setOutputProperty(OutputKeys.METHOD, "xml");
           transformer.setOutputProperty(OutputKeys.INDENT, "yes");
           transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
           transformer.transform(source, result);
       }  catch (TransformerConfigurationException e) {
           throw new ReportGenerationException(
                   "Report has been generated"
                   + " but not tranformed to UTF-8 format: "
                   + e.getMessage());
       } catch (TransformerException e) {
           throw new ReportGenerationException(
                   "Report has been generated "
                   + "but not tranformed to UTF-8 format: "
                   + e.getMessage());
       }
   }
    
    /**
     * Prints information from current project results file to XML Document.
     * @param domDoc DOM document that will contains XML report structure.
     * @throws ResourceLoadingException
     */
    private void printReportOn(final org.w3c.dom.Document domDoc) {
        Project prj = (Project) resource.getContents().get(0);
        Element prjElt = domDoc.createElement(prj
                            .eClass()
                            .getName()
                            .toLowerCase());
        HashMap<String, Integer> numericResults = new HashMap<String, Integer>();
        prjElt.setAttribute("name", prj.getIdent());
        String descr;
        descr = prj.getShortDescription();
        if (descr  != null) {
            prjElt.setAttribute("description", descr);
        }
        for (Object element : prj.getDocuments()) {
            ttm.Document doc = (ttm.Document) element;
            numericResults.put("requirements", 0);
            numericResults.put("covered", 0);
            numericResults.put("uncovered", 0);
            Element docElt = domDoc
                                .createElement(doc.eClass()
                                                    .getName()
                                                    .toLowerCase());
            docElt.setAttribute("name", doc.getIdent());
            descr = doc.getShortDescription();
            if (descr != null) {
                docElt.setAttribute("description", descr);
            }
            Float gcr = doc.getGlobalCoverageRatio();
            if ( gcr != null) {
                docElt.setAttribute("globalCoverageRatio", gcr.toString()+"%");
                Float ucgr = (Float) ((new Float(100.0)) - gcr); 
                docElt.setAttribute("uncoveredCoverageRatio", ucgr.toString()+"%");
            }
            docElt.setAttribute("filename", doc.getFilename());
            appendCoveringDocumentsOfDocument(doc,docElt,domDoc);
            appendDocElementsToDocEltOfDomDoc(doc, docElt, domDoc, numericResults);
            
            docElt.setAttribute("requirementsNumber", numericResults.get("requirements").toString());
            docElt.setAttribute("coveredNumber", numericResults.get("covered").toString());
            docElt.setAttribute("uncoveredNumber", numericResults.get("uncovered").toString());
            
            
            prjElt.appendChild(docElt);
        }
        domDoc.appendChild(prjElt);
    }
    
    /**
     * Completes the document with the covering document coverage ratios.
     * @param doc the covered document
     * @param docElt the DOM element corresponding to the document
     * @param domDoc the DOM document
     */
    private void appendCoveringDocumentsOfDocument(final ttm.Document doc,
    		final Element docElt, final org.w3c.dom.Document domDoc ) {
        for (Iterator<ttm.Document> coveringDocuments = 
        		doc.getCoveringDocuments().iterator();
        		coveringDocuments.hasNext();) {
			ttm.Document coveringDocument = 
				(ttm.Document) coveringDocuments.next();
			Element objElt = 
				domDoc.createElement("coveringDocument");
			objElt.setAttribute("name", coveringDocument.getIdent());
			objElt.setAttribute("coverageRatio", doc.getCoverageResults().get(coveringDocument).toString());
			docElt.appendChild(objElt);
		}   	
    }
    /**
     * Completes DOM document element with document elements.
     * @param doc the document that contents is added in DOM document
     * @param docElt the parent element in DOM document structure
     * @param domDoc DOM structure
     */
    private void appendDocElementsToDocEltOfDomDoc(final ttm.Document doc,
            final Element docElt, final org.w3c.dom.Document domDoc,
            HashMap<String, Integer> numericResults) {
        for (Iterator<EObject> elts = doc.eAllContents(); elts.hasNext();) {
            Object obj = elts.next();
            if (obj instanceof IdentifiedElement) {
                IdentifiedElement element = (IdentifiedElement) obj;
                Element objElt = domDoc
                                    .createElement(element
                                                        .eClass()
                                                        .getName()
                                                        .toLowerCase());
                objElt.setAttribute("ident", element.getIdent());
                if (element.getShortDescription() != null) {
                    objElt.setAttribute("description",
                                    element.getShortDescription());
                }
                printTextsOfElement(element, domDoc, objElt);
                printAttributesOfOwner((AttributeOwner)obj, objElt);
                if (element instanceof Requirement) {
                    numericResults.put("requirements", numericResults.get("requirements")+1);
                    Requirement req = (Requirement) element;
                    if(req.getIncomingCoverLinks().isEmpty() && !req.getDocument().getCoveringDocuments().isEmpty()) {
                        numericResults.put("uncovered", numericResults.get("uncovered")+1);                       
                    } else {
                        numericResults.put("covered", numericResults.get("covered")+1);                                           	
                    }
                    printCoverLinksOfReq(req, domDoc, objElt);
                }
                docElt.appendChild(objElt);
            }
        }

    }

    /**
     * Prints texts of param <b>req</b> in DOM element <b>objElt</b>
     *  of DOM document <b>domDoc</b>.
     * @param elt is the requirement from which texts will be printed
     * @param domDoc is the DOM Document
     * @param objElt is the DOM Element the represents the <b>elt</b>
     */
    private void printTextsOfElement(
            final IdentifiedElement elt ,
            final Document domDoc,
            final Element objElt) {
        for (Object element : elt.getTexts()) {
        	Text txt = (Text) element;
        	Element txtElt = domDoc.createElement("text");
        	txtElt.setTextContent(txt.getValue());
        	objElt.appendChild(txtElt);
        }

    }
    
    /**
     * Prints coverlinks of param <b>req</b> in DOM element <b>objElt</b>
     *  of DOM document <b>domDoc</b>.
     * @param req is the requirement from which coverlinks will be printed
     * @param domDoc is the DOM Document
     * @param objElt is the DOM Element the represents the <b>req</b>
     */
    private void printCoverLinksOfReq(
            final Requirement req ,
            final Document domDoc,
            final Element objElt) {
        for (Object element : req.getIncomingCoverLinks()) {
		CoverLink cvLnk = (CoverLink) element;
		HierarchicalElement conElt = cvLnk.getFrom();
		Element covElt = domDoc.createElement("coveredBy");
		covElt.setAttribute("ident", conElt.getIdent());
		if (conElt.getShortDescription() != null) {
		    covElt.setAttribute("description"
		            , conElt.getShortDescription());
		}
		covElt.setAttribute("class"
		        , conElt.eClass().getName().toLowerCase());
		covElt.setAttribute("documentName", conElt.getDocument().getIdent());
		covElt.setAttribute("document", conElt.getDocument().getFilename());
		printAttributesOfOwner((AttributeOwner)cvLnk, objElt);
		if (conElt instanceof Requirement) {
		    printRequirementCoverLinks((Requirement) conElt, covElt);
		}
		objElt.appendChild(covElt);
      }

    }

    /**
     * Prints the coverlinks of the requirement in parameter.
     * @param req is the requirement which coverlinks will be printed
     * @param parentElt is the DOM parent element
     */
    private void printRequirementCoverLinks(final Requirement req
            , final Element parentElt) {
        for (Object element : req.getIncomingCoverLinks()) {
         HierarchicalElement conElt = ((CoverLink) element).getFrom();
         Element covElt = parentElt
		                .getOwnerDocument()
		                .createElement("coveredBy");
         covElt.setAttribute("ident", conElt.getIdent());
 		covElt.setAttribute("class"
		        , conElt.eClass().getName().toLowerCase());
 		if (conElt.getShortDescription() != null) {
		covElt
		    .setAttribute("description", conElt.getShortDescription());
         }
         covElt.setAttribute("documentName", conElt.getDocument().getIdent());
         covElt.setAttribute("document", conElt.getDocument().getFilename());
         if (covElt instanceof Requirement) {
		printRequirementCoverLinks((Requirement) conElt, covElt);
         }
         parentElt.appendChild(covElt);
      }
    }
    
    /**
     * Prints the attributes of attribute owner
     * @param owner the attribute owner
     * @param parentDomElt the DOM element parent in DOM Structure
     */
    private void printAttributesOfOwner(final AttributeOwner owner,
                                        final Element parentDomElt) {
        for (Object attrIter : owner.getAttributes()){
            Attribute attr = (Attribute)attrIter;
            Element domAttr = parentDomElt
                                .getOwnerDocument()
                                .createElement("attribute");
            domAttr.setAttribute("name", attr.getName());
            domAttr.setAttribute("value", attr.getValue());
            parentDomElt.appendChild(domAttr);
        }
    }
 
    /**
     * Prints information from current project results file to XML Document.
     * @param domDoc DOM document that will contains XML report structure.
     * @throws ResourceLoadingException
     */
    private void printRulesReportOn(final org.w3c.dom.Document domDoc) {
        Project prj = (Project) resource.getContents().get(0);
        
        Element rules = domDoc.createElement("rules");
        rules.setAttribute("name", "Rules Report");
        
        for (Object element : prj.getRules()) {
            Rule rule = (Rule)element;
            Element domRule = domDoc.createElement("rule");
            domRule.setAttribute("name", rule.getName());
            printRuleInstances(domRule, rule, domDoc);
            rules.appendChild(domRule);
        }
        domDoc.appendChild(rules);
    }
    
    /**
     * Prints rule instances to DOM Document
     * @param domRule parent rule in DOM Documetn
     * @param rule parent rule in the model
     * @param domDoc DOM Document
     */
    private void printRuleInstances(Element domRule, Rule rule, Document domDoc) {
        for(Object ruleInstIter : rule.getRuleInstances()) {
            RuleInstance ruleInstance = (RuleInstance)ruleInstIter;
            ttm.Element elt = ruleInstance.getElement();
            Element domRuleInstance;
            if (elt instanceof IdentifiedElement) {
               domRuleInstance = domDoc.createElement("element");
               if (rule.getName().equals("Undefined Requirement")) {
                   HierarchicalElement cElt = (HierarchicalElement) elt;
                   for (Iterator<?> it = cElt.getCoverLinks().iterator(); it.hasNext();) {
                       CoverLink cvl = (CoverLink) it.next();
                       if (cvl.isUnresolved()) {
                           domRuleInstance.setAttribute("identifier", cvl.getReferencedID());
                           domRuleInstance.setAttribute("parent", ((IdentifiedElement)elt).getIdent());
                       }
                   }                	
               } else {
            	   domRuleInstance.setAttribute("identifier", ((IdentifiedElement)elt).getIdent());
                   domRuleInstance.setAttribute("description", ((IdentifiedElement)elt).getShortDescription());
               }
               domRule.appendChild(domRuleInstance);

            }
            if (elt instanceof Attribute) {
                domRuleInstance = domDoc.createElement("attribute");
                domRuleInstance.setAttribute("name", ((Attribute)elt).getName());
                domRuleInstance.setAttribute("description", ((Attribute)elt).getValue());  
                domRule.appendChild(domRuleInstance);
            }
            if (elt instanceof CoverLink) {
                domRuleInstance = domDoc.createElement("coverlink");
                domRuleInstance.setAttribute("from", ((CoverLink)elt).getFrom().getIdent());
                domRuleInstance.setAttribute("to", ((CoverLink)elt).getTo().getIdent());  
                domRule.appendChild(domRuleInstance);               
            }
            if (elt instanceof NavigationLink) {
                domRuleInstance = domDoc.createElement("navigationlink");
                domRuleInstance.setAttribute("from", ((NavigationLink)elt).getFrom().getIdent());
                domRuleInstance.setAttribute("to", ((NavigationLink)elt).getTo().getIdent());  
                domRule.appendChild(domRuleInstance);               
            }
        }
    }

    /*
     * Rules from Extensions Methods
     *
     */

    /**
     * gets the rules availables from extensions to RulesSet extension point
     *  and add them to the project rule definition.
     * @param aProject is the project in which rules definition will be added
     * @return an array of rules getted from RulesSet extensions
     */
    public static ExtendedRule[] getRules(final Project aProject) {
        if (aProject == null) {
            return null;
        }
        ExtendedRule[] rules = getCachedRules();
        if (aProject.getRules().isEmpty()) {
            for (int i = 0; i < rules.length; i++) {
                aProject.addRuleDefinition(rules[i]);
            }
        }
        return rules;
    }

    /**
     * Gets the rules defined in RulesSet extensions.
     * @return an array of rules getted from RulesSet extensions
     */
    private static ExtendedRule[] getCachedRules() {
        if (cachedRules != null) {
            return cachedRules;
        }
        IExtension[] exts = Platform
                                .getExtensionRegistry()
                                .getExtensionPoint(RULES_SET_EXT_POINT_ID)
                                .getExtensions();
        List < ExtendedRule > found = new ArrayList < ExtendedRule >();

        for (int i = 0; i < exts.length; i++) {
            IConfigurationElement[] configElements =
                    exts[i].getConfigurationElements();
            for (int j = 0; j < configElements.length; j++) {
                ExtendedRule proxy = parseRule(configElements[j]);
                if (proxy != null) {
                    found.add(proxy);
                }
            }
        }
        cachedRules = found.toArray(new ExtendedRule[found.size()]);
        return cachedRules;
    }

    /**
     * Instanciates a Rule from the extension parsing.
     * @param configElement is the element getted from extension analysis
     * @return the Rule from extension parsing
     */
    private static ExtendedRule parseRule(
            final IConfigurationElement configElement) {
        if (!configElement.getName().equals(TAG_RULE)) {
            return null;
        }
        return new ExtendedRule(configElement);
     }

    /*
     * analyzers from Extensions
     *
     */
    /**
     * Gets analyzer from extensions to DocumentAnalyzer extension point
     *  which id is given in parameter.
     * @param anID is the id of the analyzer requested
     * @return the analyzer corresponding to the id in parameter
     */
    public static ExtendedAnalyzer getAnalyzerWithID(final String anID) {
        ExtendedAnalyzer[] analyzers = Engine.getAnalyzers();
        String id = anID;
        if (anID == null || anID.length() == 0) {
            id = Engine.DEFAULT_ANALYZER_ID;
        }
        for (int i = 0; i < analyzers.length; i++) {
            ExtendedAnalyzer analyzer = analyzers[i];
            if (analyzer.getID().equals(id)) {
                return analyzer;
            }
        }
        return null;
    }

    /**
     * Gets analyzers from extensions to DocumentAnalyzer extension point.
     * @return an array of analyzers from extensions to
     *  DocumentAnalyzer extension point
     */
    public static ExtendedAnalyzer[] getAnalyzers() {
        if (cachedAnalyzers != null) {
            return cachedAnalyzers;
        }
        
        IExtension[] exts = Platform
                                .getExtensionRegistry()
                                .getExtensionPoint(ANALYZER_EXT_POINT_ID)
                                .getExtensions();
        List < ExtendedAnalyzer > found =
                        new ArrayList < ExtendedAnalyzer > ();

        for (int i = 0; i < exts.length; i++) {
            IExtension ext = exts[i];
            IConfigurationElement[] configElements =
                ext.getConfigurationElements();
            for(int j = 0; j < configElements.length; j++) {
            	IConfigurationElement confElt = configElements[j];
            	ExtendedAnalyzer extAnalyzer = parseDocumentAnalyzer(confElt);

            	IConfigurationElement[] analyzerConversionConfigElements = confElt.getChildren(TAG_CONVERSION_METHOD);
            	for (int k = 0; k < analyzerConversionConfigElements.length; k++) {
					ExtendedAnalyzerConversionMethod method = parseAnalyzerConversionMethod(extAnalyzer, analyzerConversionConfigElements[k]);
            		if (method != null) {
            			extAnalyzer.addConversionMethod(method);
            		}
            	}
            	if (extAnalyzer.hasConversionMethods()) {
            		found.add(extAnalyzer);
            	}
            	IConfigurationElement[] analyzerEditionConfigElements = confElt.getChildren(TAG_EDITION_METHOD);
            	for (int k = 0; k < analyzerEditionConfigElements.length; k++) {
					ExtendedAnalyzerEditionMethod method = parseAnalyzerEditionMethod(extAnalyzer, analyzerEditionConfigElements[k]);
            		if (method != null) {
            			extAnalyzer.addEditionMethod(method);
            		}
            	}
            	if (extAnalyzer.hasConversionMethods()) {
            		found.add(extAnalyzer);
            	}
           }
        }
        cachedAnalyzers = found.toArray(new ExtendedAnalyzer[found.size()]);
        return cachedAnalyzers;

    }

    /**
     * Instanciates a AnalyzerMethod from the extension parsing.
     * @param configElement is the element getted from extension analysis
     * @return the Analyzer method from extension parsing
     */
    private static ExtendedAnalyzer parseDocumentAnalyzer(
            final IConfigurationElement configElement) {
        if (!configElement.getName().equals(TAG_DOCUMENT_ANALYZER)) {
            return null;
        }
        return new ExtendedAnalyzer(configElement);
    }
        
    /**
     * Instanciates a AnalyzerConversionMethod from the extension parsing.
     * @param configElement is the element getted from extension analysis
     * @return the Analyzer Conversion method from extension parsing
     * @throws AnalysisException 
     */
    private static ExtendedAnalyzerConversionMethod parseAnalyzerConversionMethod(
    		final ExtendedAnalyzer extAnalyzer,
            final IConfigurationElement configElement) 
    {
        IAnalyzer analyzerExec;
		try {
			analyzerExec = extAnalyzer.getAnalyzerOwningClassExecutableExtension();
		} catch (AnalysisException e) {
			TTEnginePlugin
				.displayError(e.getMessage() , e);
			return null;
		}
        return new ExtendedAnalyzerConversionMethod(configElement, analyzerExec);
    }

    /**
     * Instanciates a AnalyzerEditionMethod from the extension parsing.
     * @param configElement is the element getted from extension analysis
     * @return the Analyzer Edition method from extension parsing
     * @throws AnalysisException 
     */
    private static ExtendedAnalyzerEditionMethod parseAnalyzerEditionMethod(
    		final ExtendedAnalyzer extAnalyzer,
            final IConfigurationElement configElement) 
    {
        IAnalyzer analyzerExec;
		try {
			analyzerExec = extAnalyzer.getAnalyzerOwningClassExecutableExtension();
		} catch (AnalysisException e) {
			TTEnginePlugin
				.displayError(e.getMessage() , e);
			return null;
		}
        return new ExtendedAnalyzerEditionMethod(configElement, analyzerExec);
    }

}
