package tte.impl;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.Diagnostician;

import tte.analyzers.ExtendedAnalyzer;
import tte.analyzers.ExtendedAnalyzerConversionMethod;
import tte.exceptions.AnalysisException;
import tte.exceptions.AnalyzerMethodException;
import tte.exceptions.ConversionException;
import ttm.AbstractLink;
import ttm.Attribute;
import ttm.CoverLink;
import ttm.CoverPoint;
import ttm.Document;
import ttm.Element;
import ttm.HierarchicalElement;
import ttm.NavigationLink;
import ttm.Project;
import ttm.Requirement;
import ttm.RuleInstance;
import ttm.Text;
import ttm.TtmFactory;
import ttm.TtmPackage;

/**
 * Class defining methods to do the analysis of a Topcased Traceability Model
 * project.
 * 
 */
public class ProjectAnalyzer {
	/**
	 * current project file.
	 */
	private File projectFile;

	/**
	 * current project.
	 */
	private Project currentProject = null;

	/**
	 * list of documents in current project.
	 */
	private List<Document> projectDocuments;

	/**
	 * current resource set.
	 */
	private ResourceSet currentResourceSet = null;

	/**
	 * current resource.
	 */
	private Resource currentResource = null;

	/**
	 * Map using during analysis to keep unresolved links.
	 */
	private HashMap<String, Set<AbstractLink>> unresolvedAbstractLinks = new HashMap<String, Set<AbstractLink>>();

	/**
	 * Map containing requirements list per document.
	 */
	private HashMap<String, Set<Requirement>> reqsByDocName = new HashMap<String, Set<Requirement>>();

	/**
	 * Creates a new instance of ProjectAnalyzer.
	 * 
	 * @param projectFnString
	 *            the project filename
	 */
	public ProjectAnalyzer(final String projectFnString) {
		projectFile = new File(projectFnString);
	}

	/**
	 * <!-- begin-user-doc --> Prints diagnostics with indentation. <!--
	 * end-user-doc -->
	 * 
	 * @param diagnostic
	 *            the diagnostic to print.
	 * @param indent
	 *            the indentation for printing.
	 * @throws AnalysisException
	 */
	protected static void printDiagnostic(final Diagnostic diagnostic,
			final String indent, StringBuffer buff) throws AnalysisException {
		System.out.print(indent);
		String mess = diagnostic.getMessage();
		System.out.println(mess);
		buff.append(indent + mess + "\n");
		for (Object element : diagnostic.getChildren()) {
			printDiagnostic((Diagnostic) element, indent + "  ", buff);
		}
	}

	/**
	 * Runs the analysis.
	 * 
	 * @param resourceSet
	 *            the resource set used during analysis
	 * @return the resource created during analysis
	 * @throws AnalysisException
	 *             when exception occurs during analysis
	 */
	public final Resource runAnalysis(final ResourceSet resourceSet)
			throws AnalysisException {
		currentResourceSet = resourceSet;
		if (projectFile == null) {
			throw new AnalysisException("ProjectAnalyzer: instance variable "
					+ "'projectFile' is null", 1);
		}
		if (!(projectFile.isFile())) {
			throw new AnalysisException(
					"ProjectAnalyzer: project file does not exists", 1);
		}

		/*
		 * Construct the URI for the instance file. The argument is treated as a
		 * file path only if it denotes an existing file. Otherwise, it's
		 * directly treated as a URL.
		 */
		URI uri = URI.createFileURI(projectFile.getAbsolutePath());

		// Demand load resource for this file.
		//
		currentResource = currentResourceSet.getResource(uri, true);
		for (Object obj : currentResource.getContents()) {
			if (obj instanceof Project) {
				currentProject = (Project) obj;
				analyzeCurrentProject();
			}
		}
		return currentResource;
	}

	/**
	 * Makes analysis of currentProject.
	 * 
	 * @throws AnalysisException
	 *             if document resolution raised an exception
	 */
	private void analyzeCurrentProject() throws AnalysisException {
		currentProject.cleanCachedElements();
		
		String projectName = currentProject.getIdent();
		if (projectName == null || "".equals(projectName)) {
			currentProject.setIdent("defaultIdent");
		}
		
		Diagnostic diagnostic = Diagnostician.INSTANCE.validate(currentProject);
		if (diagnostic.getSeverity() != Diagnostic.OK) {
			StringBuffer buff = new StringBuffer();
			printDiagnostic(diagnostic, "", buff);
			throw new AnalysisException(buff.toString(), 1);
		}
		Engine.cleanCachedElements();
		currentProject.getCoverageResults().clear();
		projectDocuments = currentProject.getDocuments();

		resolveDocuments();

		for (Object element : currentProject.getDocuments()) {
			Document document = (Document) element;
			resolveAbstractLinks(document);
			affectCoverageRatiosForDoc(document);
		}

		// Validate the contents of full project resource.

		for (Iterator<EObject> elts = currentProject.eAllContents(); elts
				.hasNext();) {
			EObject elt = (EObject) elts.next();
			if (elt instanceof Element) {
				checkRulesFor((Element) elt);
			}
		}
		diagnostic = Diagnostician.INSTANCE.validate(currentProject);
		if (diagnostic.getSeverity() != Diagnostic.OK) {
			StringBuffer buff = new StringBuffer();
			printDiagnostic(diagnostic, "", buff);
			throw new AnalysisException(buff.toString(), 1);
		}

		// Save project results
		saveAsResults();

	}

	/**
	 * Does the analysis of each document in the project.
	 * 
	 * @throws AnalysisException
	 *             when exception occurs during analysis.
	 */
	private void resolveDocuments() throws AnalysisException {
		StringBuffer buf = new StringBuffer();
		for (Object element : projectDocuments) {
			Document doc = (Document) element;
			if (!doc.getChildren().isEmpty()) {
				doc.getChildren().clear();
			}
			String currentFilename = doc.getFilename();
			try {
				// long time1 = System.currentTimeMillis();
				resolveDocument(doc);
				// long time2 = System.currentTimeMillis();
				/*
				 * System.out.println("conversion of " + doc.getIdent() + " -> " +
				 * (time2-time1) + "ms");
				 */
			} catch (AnalyzerMethodException e) {
				buf.insert(0, currentFilename + ": " + e.getMessage() + "\n");
			} catch (ConversionException e) {
				buf.insert(0, currentFilename + ": " + e.getMessage() + "\n");
			} catch (AnalysisException e) {
				buf.insert(0, currentFilename + ": " + e.getMessage() + "\n");
			}
		}
		if (buf.length() > 0) {
			throw new AnalysisException(buf.toString(), 2);
		}
	}

	/**
	 * Makes the resolution of abstract links for the document in parameter.
	 * 
	 * @param doc
	 *            the document which link are tried to be resolved
	 */
	private void resolveAbstractLinks(final Document doc) {
		for (Iterator<EObject> elts = doc.eAllContents(); elts.hasNext();) {
			EObject obj = (EObject) elts.next();
			currentProject.addCachedElement(obj.eClass().getClassifierID(),
					(Object) obj);
			if (obj instanceof HierarchicalElement) {
				HierarchicalElement element = (HierarchicalElement) obj;
				Set<AbstractLink> abstrlnks = unresolvedAbstractLinks
						.get(element.getIdent());
				if (abstrlnks != null) {
					for (Object element2 : abstrlnks) {
						AbstractLink abstrlnk = (AbstractLink) element2;
						resolveAbstractLinkToElement(abstrlnk, element);
					}
				}
			}
		}
	}

	/**
	 * Affects abstract link target.
	 * 
	 * @param abstrlnk
	 *            the abstract link which target will be defined
	 * @param element
	 *            the element that will be the target of abstract link
	 */
	private void resolveAbstractLinkToElement(final AbstractLink abstrlnk,
			final HierarchicalElement element) {
		if (abstrlnk.isUnresolved()) {
			if (abstrlnk instanceof CoverLink) {
				CoverLink covlnk = (CoverLink) abstrlnk;
				if (element instanceof Requirement) {
					Requirement req = (Requirement) element;
					covlnk.setUnresolved(false);
					covlnk.setTo(req);
					completeChildRequirementsWithCoverLink(req, covlnk);
				}
			}
			if (abstrlnk instanceof NavigationLink) {
				NavigationLink lnk = (NavigationLink) abstrlnk;
				lnk.setUnresolved(false);
				lnk.setTo(element);
			}
		}
	}

	/**
	 * Add a new cover link between 2 elements, if it does not already exist.
	 * 
	 * @param fromElt
	 *            the source element of the cover link
	 * @param toReq
	 *            the target requirement of the cover link
	 * @param searchedID
	 *            the identifier that allow to know if there is already
	 *            coverlink between the 2 elements
	 * @param unresolved
	 *            the status of the cover link
	 */
	private void addCoverLinkIfNecessaryBetween(HierarchicalElement fromElt,
			Requirement toReq, String searchedID, Boolean unresolved) {
		if (!fromElt.isCoveringElementID(searchedID)) {
			CoverLink newCvLnk = TtmFactory.eINSTANCE.createCoverLink();
			newCvLnk.setFrom(fromElt);
			newCvLnk.setTo(toReq);
			newCvLnk.setReferencedID(searchedID);
			newCvLnk.setUnresolved(unresolved);
		}

	}

	/**
	 * Add cover link to child requirements of a parent requirement.
	 * 
	 * @param parentReq
	 *            the parent requirement
	 * @param cvLnk
	 *            the cover link from which we create new one
	 */
	private void completeChildRequirementsWithCoverLink(Requirement parentReq,
			CoverLink cvLnk) {
		for (Object obj : parentReq.getChildren()) {
			HierarchicalElement elt = (HierarchicalElement) obj;
			if (elt instanceof Requirement) {
				addCoverLinkIfNecessaryBetween(cvLnk.getFrom(),
						(Requirement) elt, ((Requirement) elt).getIdent(),
						false);
			}
		}
	}

	/**
	 * Makes the resolution of a document.
	 * 
	 * @param aDoc
	 *            the document that will be resolved
	 * @throws AnalyzerMethodException
	 *             if analyzer method instance creation fails
	 * @throws ConversionException
	 *             if an exception occurs during conversion
	 * @throws AnalysisException
	 *             if an exception occurs during document analysis
	 */
	private void resolveDocument(final Document aDoc)
			throws AnalyzerMethodException, ConversionException,
			AnalysisException {
		String docFilename = aDoc.getFilename();
		URL url = null;
		File sourceFile = new File(docFilename);
		if ( !sourceFile.exists()) {
			try {
				url = FileLocator.resolve(new URL(docFilename));
				sourceFile = new File(url.getFile());
			} catch (MalformedURLException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		} 
		String sourceFileName = sourceFile.getName();
		File destFile;
		try {
			String fn = sourceFileName.substring(0, sourceFileName
					.lastIndexOf("."));
			destFile = File.createTempFile(fn, ".xml");
		} catch (IOException e) {
			throw new AnalysisException(
					"Cannot create intermediate file for document "
							+ aDoc.getIdent() + ". Conversion failed.");
		}

	ExtendedAnalyzer analyzer = Engine.getAnalyzerWithID(aDoc
				.getAnalyzerID());
		if (analyzer == null) {
			destFile.delete();
			throw new AnalysisException("No available analyzer for document "
					+ aDoc.getIdent() + ". Conversion failed.");
		}
		ExtendedAnalyzerConversionMethod conversionMethod = analyzer.getConversionMethodWithID(aDoc
				.getConversionMethodID());

		if (conversionMethod == null) {
			destFile.delete();
			throw new AnalysisException("Conversion method has not been found for "
					+ aDoc.getIdent() + ". Conversion failed.");
		}
		conversionMethod.executeConversion(sourceFile, destFile, analyzer, aDoc);

		destFile.deleteOnExit();
		try {
			URI docURI = URI.createFileURI(destFile.getAbsolutePath());
			Resource resource = currentResourceSet.getResource(docURI, true);
			aDoc.setLastModifiedDate(sourceFile.lastModified());
			for (Object element : resource.getContents()) {
				Document externalDoc = (Document) element;
				completeDocumentWithExternalDocContents(aDoc, externalDoc);
			}
		} catch (WrappedException e) {
			destFile.delete();
			throw new AnalysisException("Document " + aDoc.getIdent()
					+ " has not been converted due to the following error: "
					+ e.getMessage(), 1);
		}
	}

	/**
	 * Completes the Document in results with the contents from Source document
	 * conveted file.
	 * 
	 * @param aDoc
	 *            the document in results project.
	 * @param externalDoc
	 *            the source document contents after conversion.
	 */
	private void completeDocumentWithExternalDocContents(final Document aDoc,
			final Document externalDoc) {
		aDoc.setAnalyzerArguments(externalDoc.getAnalyzerArguments());
		aDoc.setAnalyzerID(externalDoc.getAnalyzerID());
		aDoc.setConversionMethodID(externalDoc.getConversionMethodID());
		aDoc.setLastModifiedDate(externalDoc.getLastModifiedDate());
		affectExternalDocElementToDocument(externalDoc, aDoc);
		/*
		 * Addition of unresolved coverlinks into hashMap to make resolution in
		 * the next step
		 */
		Set<Requirement> docReqs = new HashSet<Requirement>();
		for (Iterator<EObject> iter = aDoc.eAllContents(); iter.hasNext();) {
			Object obj = iter.next();
			if (obj instanceof Element) {
				Element element = (Element) obj;
				switch (element.eClass().getClassifierID()) {
				case TtmPackage.COVER_LINK:
					addAbstractLink(element);
					break;
				case TtmPackage.NAVIGATION_LINK:
					addAbstractLink(element);
					break;
				case TtmPackage.REQUIREMENT:
					docReqs.add((Requirement) element);
					completeRequirementFromParent((Requirement) element);
					break;
				default:
					break;
				}
			}
		}
		reqsByDocName.put(aDoc.getIdent(), docReqs);
	}

	/**
	 * Affects elements of converted source document to current document.
	 * 
	 * @param externalDoc
	 *            the source document after conversion
	 * @param aDoc
	 *            the current document in the project results
	 */
	private void affectExternalDocElementToDocument(Document externalDoc,
			Document aDoc) {
		LinkedList<HierarchicalElement> toAddElts = new LinkedList<HierarchicalElement>();
		for (Object docChild : externalDoc.getChildren()) {
			HierarchicalElement child = (HierarchicalElement) docChild;
			toAddElts.addLast(child);
		}
		for (Object child : toAddElts) {
			HierarchicalElement element = (HierarchicalElement) child;
			element.setParent(aDoc);
		}

	}

	/**
	 * Add, if parent is requirement, the parent information. Information is the
	 * parent's attributes and textes and coverLinks
	 * 
	 * @param req
	 *            the current added requirement
	 */
	private void completeRequirementFromParent(final Requirement req) {
		if (req.getParent() instanceof Requirement) {
			for (Requirement parentReq : ((Requirement) req)
					.getAllParentRequirements()) {
				for (Object attrIter : parentReq.getAttributes()) {
					Attribute attr = (Attribute) attrIter;
					Attribute newAttr = TtmFactory.eINSTANCE.createAttribute();
					newAttr.setParent(req);
					newAttr.setName(attr.getName());
					newAttr.setValue(attr.getValue());

				}
				for (Object textIter : parentReq.getTexts()) {
					Text txt = (Text) textIter;
					Text newTxt = TtmFactory.eINSTANCE.createText();
					newTxt.setParent(req);
					newTxt.setValue(txt.getValue());
				}
				for (Object cvLnkIter : parentReq.getCoverLinks()) {
					CoverLink cvLnk = (CoverLink) cvLnkIter;
					addCoverLinkIfNecessaryBetween(req, cvLnk.getTo(), cvLnk
							.getReferencedID(), cvLnk.isUnresolved());
				}
			}
		}
	}

	/**
	 * Adds abstract links to an element.
	 * 
	 * @param element
	 *            the element that will contain new abstract links
	 */
	private void addAbstractLink(final Element element) {
		AbstractLink cvlnk = (AbstractLink) element;
		if (cvlnk.isUnresolved()) {
			String cvRefID = cvlnk.getReferencedID();
			Set<AbstractLink> currentAbstrLinks = unresolvedAbstractLinks
					.get(cvRefID);
			if (currentAbstrLinks == null) {
				currentAbstrLinks = new HashSet<AbstractLink>();
			}
			currentAbstrLinks.add(cvlnk);
			unresolvedAbstractLinks.put(cvRefID, currentAbstrLinks);
		}
	}

	/**
	 * Saves results in the ttmresults file.
	 * 
	 * @throws AnalysisException
	 *             when exception occurs when trying to saveresults
	 */
	private void saveAsResults() throws AnalysisException {
		try {
			FileOutputStream fno = new FileOutputStream(new File(projectFile
					.getAbsolutePath()
					+ "results"));
			currentResource.save(fno, Engine.ttmMap());
			fno.close();
		} catch (FileNotFoundException e) {
			System.err.println("Cannot open \"" + projectFile.getAbsolutePath()
					+ "results\" to save results. Please check its rights");
		} catch (IOException e) {
			throw new AnalysisException("Cannot save the resuls. "
					+ e.getMessage(), 1);
		}
	}

	/**
	 * Computes and affects coverage ratio to the document in parameter.
	 * 
	 * @param aDoc
	 *            the document that will contains the coverage ratio
	 */
	private void affectCoverageRatiosForDoc(final Document aDoc) {
		Set<Requirement> docReqs = reqsByDocName.get(aDoc.getIdent());
		int docNbReqs = docReqs.size();
		Map<Document, Set<Requirement>> coveredReqsByCovDoc = new HashMap<Document, Set<Requirement>>();
		if (currentProject.isAnyDocumentCovered()) {
			// All documents can be covered by any other document
			for (Document doc : projectDocuments) {
				coveredReqsByCovDoc.put(doc, null);
			}
		} else {
			// Only covering document in project configuratio can be covered
			for (Document coveringDoc : aDoc.getCoveringDocuments()) {
				coveredReqsByCovDoc.put(coveringDoc, null);
			}
			if (aDoc.isCoversItself()) {
				coveredReqsByCovDoc.put(aDoc, null);
			}
		}
		fillCoveredReqsByCovDocForDoc(coveredReqsByCovDoc, aDoc);

		for (Document coveringDoc : coveredReqsByCovDoc.keySet()) {
			float ratio = ((float) (coveredReqsByCovDoc.get(coveringDoc).size()) / (float) docNbReqs) * 100;
			aDoc.getProject().addCoverageResult(coveringDoc, null, aDoc, ratio);
		}
		affectCoverageRatiosFromCoverPoints(docReqs, coveredReqsByCovDoc, aDoc);
	}

	/**
	 * Completes Covered Requirement by Document Map.
	 * 
	 * @param coveredReqsByCovDoc
	 *            the map that contains covered requirements per document
	 * @param aDoc
	 *            the document that contains covered requirements
	 */
	private void fillCoveredReqsByCovDocForDoc(
			final Map<Document, Set<Requirement>> coveredReqsByCovDoc,
			final Document aDoc) {
		for (Document doc : coveredReqsByCovDoc.keySet()) {
			Set<Requirement> coveredReqs = new HashSet<Requirement>();
			for (Iterator<EObject> i = doc.eAllContents(); i.hasNext();) {
				Object obj = i.next();
				if (obj instanceof HierarchicalElement) {
					for (CoverLink cvLnk : ((HierarchicalElement) obj)
							.getCoverLinks()) {
						if (!cvLnk.isUnresolved()) {
							Requirement req = cvLnk.getTo();
							if (req.getDocument().equals(aDoc)) {
								coveredReqs.add(req);
							}
						}
					}
				}
			}
			coveredReqsByCovDoc.put(doc, coveredReqs);
		}
	}

	/**
	 * @param docNbReqs
	 *            the number of requirements in document.
	 * @param coveredReqsByCovDoc
	 *            the map that contains covered requirements per document
	 * @param aDoc
	 *            the document that is covered
	 */
	private void affectCoverageRatiosFromCoverPoints(
			final Set<Requirement> docReqs,
			final Map<Document, Set<Requirement>> coveredReqsByCovDoc,
			final Document aDoc) {
		int docNbReqs = docReqs.size();
		Set<Requirement> globalCoveredReqs = docReqs;
		int nbCoveredReqs = 0;
		for (Object cvPointIter : aDoc.getCoverPoints()) {
			CoverPoint cvPoint = (CoverPoint) cvPointIter;
			Set<Requirement> coveredReqs = new HashSet<Requirement>();
			for (Object docIter : cvPoint.getConnectedDocuments()) {
				Document doc = (Document) docIter;
				Set<Requirement> reqs = coveredReqsByCovDoc.get(doc);
				if (reqs != null) {
					if (cvPoint.isAndCover() && !(coveredReqs.isEmpty())) {
						/*
						 * requirements of aDoc must be covered in all
						 * coveredDocument from this coverPoint
						 */
						coveredReqs.retainAll(reqs);
					} else {
						/*
						 * if coveredReqs is empty, we initialize it for the
						 * first time here even if we are are in AND covering
						 * mode else when requirements of aDoc must be covered
						 * in at least one coveredDocument from this coverPoint
						 */
						coveredReqs.addAll(reqs);
					}
				}
			}

			float ratio = 0;
			if (docNbReqs != 0) {
				ratio = ((float) (coveredReqs.size()) / (float) docNbReqs) * 100;
			}
			aDoc.getProject().addCoverageResult(null, cvPoint, aDoc, ratio);
			globalCoveredReqs.retainAll(coveredReqs);
			nbCoveredReqs = globalCoveredReqs.size();
		}
		
		// TODO : Ensure this is the proper behavior when there is
		// no requirements (maybe 100% is expected instead ?)
		float ratio = 0.0f; 
		if (docNbReqs != 0) {
			ratio = (((float) nbCoveredReqs) / (float) docNbReqs) * 100;
			aDoc.setGlobalCoverageRatio(ratio);
		}
	}

	/**
	 * Checks for the element in parameter the rules from the RulesSet
	 * extensions.
	 * 
	 * @param elt
	 *            the element that will own some rules
	 */
	private void checkRulesFor(final Element elt) {
		elt.getActivatedRules().clear();
		tte.rules.ExtendedRule[] currentRules = Engine.getRules(currentProject);
		for (int i = 0; i < currentRules.length; i++) {
			tte.rules.ExtendedRule rule = currentRules[i];
			if (rule.basicIsAppliedToElement(elt)
					&& rule.basicIsRaisedForElement(elt, currentProject)) {
				RuleInstance inst = TtmFactory.eINSTANCE.createRuleInstance();
				inst.setBaseRule(rule);
				inst.setElement(elt);
				inst.setDisplayMessage(rule.getMessageForElement(elt));
			}
		}
	}
}
