package tte.rules;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;

import tte.exceptions.AnalysisException;
import ttm.Element;
import ttm.Project;
import ttm.impl.RuleImpl;

/**
 * Class corresponding to the Rule intances
 *  creates from the RulesSet extensions analysis.
 *
 */
public class ExtendedRule extends RuleImpl {

    /**
     * In RulesSet extension structure, the name of internal name attribute.
     */
    private static final String ATT_INTERNAL_NAME = "id";

    /**
     * In RulesSet extension structure, the name of label attribute.
     */
    private static final String ATT_LABEL = "displayName";

    /**
     * In RulesSet extension structure, the name of message attribute.
     */
    private static final String ATT_MESSAGE = "defaultMessage";

    /**
     * In RulesSet extension structure, the name of activate attribute.
     */
    private static final String ATT_ACTIVATE = "activate";

    /**
     * In RulesSet extension structure, the name of application class attribute.
     */
    private static final String ATT_APP_CLASS = "applicationClass";

    /**
     * In RulesSet extension structure, the name of action class attribute.
     */
    private static final String ATT_ACTION_CLASS = "actionClass";

    /**
     * In RulesSet extension structure, the name of level attribute.
     */
    private static final String ATT_LEVEL = "level";

    /**
     * The configuration element.
     */
    private final IConfigurationElement configElement;

    /**
     * The internal name of the rule.
     */
    private final String internalName;

    /**
     * The label of the rule.
     */
    private final String label;

    /**
     * The application class of the rule.
     *  The class in ttm model concerned by this rule.
     */
    private final String applicationClass;

    /**
     * Flag that allows to make a rule silent.
     */
    private final boolean activate;

    /**
     * The level of the severity of the rule.
     */
    private Integer level;

    /**
     * Creates a ne instance of Rule from extension analysis.
     *
     * @param aConfigElement the element that contains the structure
     *  of the rule in the extension declaration.
     */
    public ExtendedRule(final IConfigurationElement aConfigElement) {
        configElement = aConfigElement;
        internalName = getAttribute(configElement, ATT_INTERNAL_NAME, null);
        label = getAttribute(configElement, ATT_LABEL, internalName);
        setName(label);
        applicationClass = getAttribute(
                configElement,
                ATT_APP_CLASS,
                Element.class.getName());
        setMessage(getAttribute(configElement, ATT_MESSAGE, label));
        activate = getAttribute(configElement, ATT_ACTIVATE, "true")
                        .equals("true");
        try {
            level = new Integer(getAttribute(configElement, ATT_LEVEL, "0"));
        } catch (NumberFormatException e) {
            level = 0;
        }
        setLevel(level);
        // Make sure that this class id defined
        // but don't load it
        getAttribute(configElement, ATT_ACTION_CLASS, null);
    }

    /**
     * Gets an attribute which name is in parameter,  from the configElement.
     * @param configElement structure of element from the extension analysis
     * @param name of the requested attribute
     * @param defaultValue if the attribute do not have a value
     * @return the value of the attribute which name is in parameter
     */
    private static String getAttribute(
            final IConfigurationElement configElement,
            final String name,
            final String defaultValue) {
        String value = configElement.getAttribute(name);
        if (value != null) {
            return value;
        }
        if (defaultValue != null) {
            return defaultValue;
        }
        throw new IllegalArgumentException("Missing " + name + " attribute");
    }

    /**
     * Creates an instance of the rule that is corresponding
     *  to the action class defined in extension definition.
     *
     * @return an instance of action class rule
     * @throws AnalysisException when cannot create
     *  instance of action class
     */
    private IRule getRule() throws AnalysisException {
        Object obj;
        try {
            obj = configElement.createExecutableExtension(ATT_ACTION_CLASS);
        } catch (CoreException e) {
            throw new AnalysisException(
                    "Cannot create instance of action class of rule named "
                    + label
                    + ": "
                    + e.getMessage());
        }
        if (obj instanceof IRule) {
            IRule rule = (IRule) obj;
            return rule;
        }
        return null;
    }

    /**
     * Checks if the class of current element is instance
     *  of application class attribute.
     * @param clazz the class of element checked
     * @return if the <b>clazz</b> attribute is instance
     *  of the application class for the rule
     */
    private boolean isClassInstanceOfApplicationClass(final Class<?> clazz) {

        if (clazz.getName().equals(applicationClass)) {
            return true;
        }
        if (clazz.getName().equals("ttm.Element")) {
            return false;
        }
        Class<?>[] interfaces = clazz.getInterfaces();
        for (int i = 0; i < interfaces.length; i++) {
            if (interfaces[i].getName().equals(applicationClass)) {
                return true;
            }
            if (interfaces[i].getName().equals("ttm.Element")) {
                return false;
            }
        }
        return isClassInstanceOfApplicationClass(clazz.getSuperclass());
    }

    /**
     * Checks if the rule is applied to an element.
     * @param elt the element that can own the Rule
     * @return if the rule is applied to an element
     */
    public final boolean basicIsAppliedToElement(final Element elt) {
        if (!activate) {
            return false;
        }
        if (elt == null) {
            return false;
        }
        Class<?> clazz = (Class<?>) elt.getClass();
        return isClassInstanceOfApplicationClass(clazz);
    }

    /**
     * Checks if the Rule is raised for an element.
     *
     * @param elt the element that will own the Rule
     * @param project the project that can be used
     *  to determine if the Rule should be raised
     * @return if the Rule is raised for an element
     */
    public final boolean basicIsRaisedForElement(
            final Element elt, final Project project) {
        try {
            IRule rule = getRule();
            if (rule != null) {
                return rule.isRaisedForElement(elt, project);
            }
            return false;
        } catch (AnalysisException e) {
            System.out.println("Failed to instanciate Rule: "
                    + getAttribute(configElement, ATT_ACTION_CLASS, null));
            return false;
        }
    }

    /**
     * {@inheritDoc}
     * @see ttm.impl.RuleImpl#getMessageForElement(ttm.Element)
     */
    public final String getMessageForElement(final Element elt) {
        try {
            IRule rule = getRule();
            if (rule == null) {
                return super.getMessageForElement(elt);
            } else {
                return rule.getDisplayMessageForElement(elt);
            }
        } catch (AnalysisException e) {
            return super.getMessageForElement(elt);
        }
    }

}
